/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ApplicationOptions.h"

CApplicationOptions::CApplicationOptions(void)
{
    CStdFileOperations::GetAppDirectory(this->ApplicationPath,MAX_PATH);

    // default config file path
    _tcscpy(this->FoldersOptionsConfigFilePath,this->ApplicationPath);
    _tcscat(this->FoldersOptionsConfigFilePath,QUICK_WAY_TO_FOLDERS_FOLDERS_OPTIONS);

    _tcscpy(this->PerUserOptionsConfigFilePath,this->ApplicationPath);
    _tcscat(this->PerUserOptionsConfigFilePath,QUICK_WAY_TO_FOLDERS_PER_USER_OPTIONS);

    // default language file
    this->SetLanguage(QUICK_WAY_TO_FOLDERS_DEFAULT_LANGUAGE);

    // portable by default
    this->bSingleUser=TRUE;

    // launch at startup by default
    this->bRunAtStartup=TRUE;

    this->bUseSkin=FALSE;
    *this->SkinName=0;

    this->LaunchMode = LaunchMode_NOTIFY_ICON;
}

CApplicationOptions::~CApplicationOptions(void)
{
}

void CApplicationOptions::GetLanguage(TCHAR* Language,DWORD LanguageMaxSize)
{
    _tcsncpy(Language,this->Language,LanguageMaxSize);
}

void CApplicationOptions::SetLanguage(TCHAR* Language)
{
    if (Language)
    {
        _tcscpy(this->Language,Language);
        _tcscpy(this->LanguageConfigFilePath,this->ApplicationPath);
        _tcscat(this->LanguageConfigFilePath,QUICK_WAY_TO_FOLDERS_LANGUAGE_DIRECTORY);
        _tcscat(this->LanguageConfigFilePath,Language);
        _tcscat(this->LanguageConfigFilePath,QUICK_WAY_TO_FOLDERS_LANGUAGE_FILES_EXTENSION);
    }
}

BOOL CApplicationOptions::Save(CIniFile* pIniFile)
{
    this->UpdateRunAtStartupStatus(this->bSingleUser);

    pIniFile->Set(CApplicationOptions_KEY_SINGLE_USER,(DWORD)this->bSingleUser);
    pIniFile->Set(CApplicationOptions_KEY_RUN_AT_STARTUP,(DWORD)this->bRunAtStartup);

    CIniFile PerUserIniFile(this->PerUserOptionsConfigFilePath);
    PerUserIniFile.Set(CApplicationOptions_KEY_LANGUAGE,this->Language);
    PerUserIniFile.Set(CApplicationOptions_KEY_USE_SKIN,(DWORD)this->bUseSkin);
    PerUserIniFile.Set(CApplicationOptions_KEY_SKIN_NAME,this->SkinName);
    PerUserIniFile.Set(CApplicationOptions_KEY_LAUNCH_MODE,(DWORD)this->LaunchMode);
    this->LaunchBarOptions.Save(&PerUserIniFile);
    return TRUE;
}

BOOL CApplicationOptions::Load(CIniFile* pIniFile)
{
    // get single user / portable or multi user
    pIniFile->Get(CApplicationOptions_KEY_SINGLE_USER,TRUE,(DWORD*)&this->bSingleUser);

    this->SetUserMode(this->bSingleUser);

    // get startup way
    pIniFile->Get(CApplicationOptions_KEY_RUN_AT_STARTUP,TRUE,(DWORD*)&this->bRunAtStartup);

    CIniFile PerUserIniFile(this->PerUserOptionsConfigFilePath);

    // get skin infos
    PerUserIniFile.Get(CApplicationOptions_KEY_USE_SKIN,FALSE,(DWORD*)&this->bUseSkin);
    PerUserIniFile.Get(CApplicationOptions_KEY_SKIN_NAME,_T(""),this->SkinName,MAX_PATH);

    // load toolbar icons
    this->BrowserToolBarOptions.Load(this->bUseSkin,this->SkinName);

    // get language 
    TCHAR Language[MAX_PATH];
    PerUserIniFile.Get(CApplicationOptions_KEY_LANGUAGE,QUICK_WAY_TO_FOLDERS_DEFAULT_LANGUAGE,Language,MAX_PATH);
    this->SetLanguage(Language);

    // get launch mode
    PerUserIniFile.Get(CApplicationOptions_KEY_LAUNCH_MODE,FALSE,(DWORD*)&this->LaunchMode);

    // get launch bar options
    this->LaunchBarOptions.Load(&PerUserIniFile);

    return TRUE;
}

void CApplicationOptions::Clone(IN OUT CApplicationOptions* pApplicationOptions)
{
    // memcpy(pApplicationOptions,this,sizeof(CApplicationOptions));

    _tcscpy(pApplicationOptions->ApplicationPath,this->ApplicationPath);
    _tcscpy(pApplicationOptions->Language,this->Language);
    _tcscpy(pApplicationOptions->FoldersOptionsConfigFilePath,this->FoldersOptionsConfigFilePath);
    _tcscpy(pApplicationOptions->PerUserOptionsConfigFilePath,this->PerUserOptionsConfigFilePath);
    _tcscpy(pApplicationOptions->LanguageConfigFilePath,this->LanguageConfigFilePath);
    _tcscpy(pApplicationOptions->SkinName,this->SkinName);

    pApplicationOptions->bSingleUser=this->bSingleUser;
    pApplicationOptions->bRunAtStartup=this->bRunAtStartup;
    pApplicationOptions->bUseSkin=this->bUseSkin;

    pApplicationOptions->LaunchMode = this->LaunchMode;
    this->LaunchBarOptions.Clone(&pApplicationOptions->LaunchBarOptions);

    // load toolbar icons
    pApplicationOptions->BrowserToolBarOptions.Load(pApplicationOptions->bUseSkin,pApplicationOptions->SkinName);
}

void CApplicationOptions::SetUserMode(BOOL bSingleUser)
{
    this->bSingleUser = bSingleUser;
    if (this->bSingleUser)
    {
        // load config file from application directory 
        _tcscpy(this->FoldersOptionsConfigFilePath,this->ApplicationPath);
        _tcscat(this->FoldersOptionsConfigFilePath,QUICK_WAY_TO_FOLDERS_FOLDERS_OPTIONS);

        _tcscpy(this->PerUserOptionsConfigFilePath,this->ApplicationPath);
        _tcscat(this->PerUserOptionsConfigFilePath,QUICK_WAY_TO_FOLDERS_PER_USER_OPTIONS);
    }
    else
    {
        // load config file from user's directory
        if (SUCCEEDED(
            ::SHGetFolderPath(NULL,CSIDL_FLAG_CREATE | CSIDL_LOCAL_APPDATA,NULL,0,this->FoldersOptionsConfigFilePath)
            )
            )
        {
            _tcscat(this->FoldersOptionsConfigFilePath,_T("\\"));
            _tcscat(this->FoldersOptionsConfigFilePath,QUICK_WAY_TO_FOLDERS_USER_PATH);

            _tcscpy(this->PerUserOptionsConfigFilePath,this->FoldersOptionsConfigFilePath);

            _tcscat(this->FoldersOptionsConfigFilePath,QUICK_WAY_TO_FOLDERS_FOLDERS_OPTIONS);
            _tcscat(this->PerUserOptionsConfigFilePath,QUICK_WAY_TO_FOLDERS_PER_USER_OPTIONS);
        }
        else
        {
            //::MessageBox(NULL,
            //            _T("Error can't get current user local settings folder\r\n")
            //            _T("Application will act as portable\r\n"),
            //            _T("Error"),
            //            MB_OK | MB_ICONERROR | MB_TOPMOST
            //            );
            this->bSingleUser=TRUE;

            // load config file from application directory 
            _tcscpy(this->FoldersOptionsConfigFilePath,this->ApplicationPath);
            _tcscat(this->FoldersOptionsConfigFilePath,QUICK_WAY_TO_FOLDERS_FOLDERS_OPTIONS);

            _tcscpy(this->PerUserOptionsConfigFilePath,this->ApplicationPath);
            _tcscat(this->PerUserOptionsConfigFilePath,QUICK_WAY_TO_FOLDERS_PER_USER_OPTIONS);
        }
    }
}
void CApplicationOptions::GetUserMode(BOOL* pbSingleUser)
{
    *pbSingleUser=this->bSingleUser;
}

void CApplicationOptions::UpdateRunAtStartupStatus(BOOL bCurrentUser)
{
    
    // remove run at startup options for current user and all users
    this->RemoveFromRunAtStartup(TRUE);
    this->RemoveFromRunAtStartup(FALSE);

    // if user doesn't want to run at startup
    if (!this->bRunAtStartup)
        return;

    // according to this->bSingleUser, add app at startup for current user or all users
    HKEY hRootKey;
    if (bCurrentUser)
        hRootKey=HKEY_CURRENT_USER;
    else
        hRootKey=HKEY_LOCAL_MACHINE;

    HKEY hKey;
    LONG lResult = ERROR_SUCCESS;
    if (::RegOpenKey(hRootKey,CApplicationOptions_WINDOWS_STARTUP_KEY,&hKey)==ERROR_SUCCESS)
    {
        TCHAR szCurrentExePath[MAX_PATH];
        CStdFileOperations::GetAppPath(szCurrentExePath,MAX_PATH);

        lResult = ::RegSetValueEx(hKey,
                                    CApplicationOptions_QUICK_WAY_TO_FOLDERS_STARTUP_KEY_NAME,
                                    NULL,
                                    REG_SZ,
                                    (PBYTE)szCurrentExePath,
                                    (DWORD)((_tcslen(szCurrentExePath)+1)*sizeof(TCHAR))
                                    );

        // close registry key
        ::RegCloseKey(hKey);
    }

    // error writing registry potential rights troubles
    if (lResult != ERROR_SUCCESS)
    {
        // if application can't be launched for all users
        if (!bCurrentUser)
        {
            // try to auto launch it for current user
            this->UpdateRunAtStartupStatus(TRUE);
        }
    }
}
void CApplicationOptions::RemoveFromRunAtStartup(BOOL bCurrentUser)
{
    HKEY hRootKey;
    if (bCurrentUser)
        hRootKey=HKEY_CURRENT_USER;
    else
        hRootKey=HKEY_LOCAL_MACHINE;

    HKEY hKey;
    if (::RegOpenKey(hRootKey,CApplicationOptions_WINDOWS_STARTUP_KEY,&hKey)==ERROR_SUCCESS)
    {
        ::RegDeleteValue(hKey,CApplicationOptions_QUICK_WAY_TO_FOLDERS_STARTUP_KEY_NAME);

        // close registry key
        ::RegCloseKey(hKey);
    }

}