/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#pragma once

#include <windows.h>
#include <vector>
#include "../Tools/Ini/IniFile.h"
#include "../Tools/GUI/TransparentToolbar/TransparentToolbar.h"

#define CBrowserToolbar_SKIN_DIRECTORY  _T("Skins\\")
#define CBrowserToolbar_SKIN_CONFIGURATION_FILENAME _T("SkinConf.ini")

#define CBrowserToolbar_TRANSPARENT_COLOR (RGB(0xFF,0,0xFF))
#define CBrowserToolbar_OPACITY 80
#define CBrowserToolbar_ICON_SIZE 24

#define CBrowserToolbar_SECTION_NAME_PREVIEW       _T("Preview")
#define CBrowserToolbar_KEY_PREVIEW_FILE_NAME      _T("FileName")

#define CBrowserToolbar_SECTION_NAME               _T("BrowserToolbar")
#define CBrowserToolbar_KEY_TRANSPARENTCOLOR       _T("TransparentColor")
#define CBrowserToolbar_KEY_USETRANSPARENTCOLOR    _T("UseTransparentColor")
#define CBrowserToolbar_KEY_BACKGROUNDTYPE         _T("BackgroundType")
#define CBrowserToolbar_KEY_BACKGROUNDCOLOR        _T("BackgroundColor")
#define CBrowserToolbar_KEY_BACKGROUNDCOLOR2       _T("BackgroundColor2")
#define CBrowserToolbar_KEY_BACKGROUNDIMAGE        _T("BackgroundImage")
#define CBrowserToolbar_KEY_OPACITYPERCENT         _T("Opacity")
#define CBrowserToolbar_KEY_MOVE_BUTTON            _T("ShowMoveButton")
#define CBrowserToolbar_KEY_ASSOCIATEDICONPATH     _T("AssociatedIconPath")
#define CBrowserToolbar_KEY_ASSOCIATEDICONINDEX    _T("AssociatedIconIndex")
#define CBrowserToolbar_KEY_POSITION_MODE          _T("Mode") // internal / external

class CBrowserToolbarOptions
{
public:
    typedef enum IconsId
    {
        IconsId_Home=0,
        IconsId_HomeDown,
        IconsId_HomeHot,
        IconsId_Back,
        IconsId_BackDisabled,
        IconsId_BackDown,
        IconsId_BackHot,
        IconsId_Forward,
        IconsId_ForwardDisabled,
        IconsId_ForwardDown,
        IconsId_ForwardHot,
        IconsId_Up,
        IconsId_UpDown,
        IconsId_UpHot,
        IconsId_JumpToExplorer,
        IconsId_JumpToExplorerDown,
        IconsId_JumpToExplorerHot,
        IconsId_Move,
        IconsId_MoveDown,
        IconsId_MoveHot,
        IconsId_Maximize,
        IconsId_MaximizeDown,
        IconsId_MaximizeHot,
        IconsId_Reduce,
        IconsId_ReduceDown,
        IconsId_ReduceHot,
        IconsId_Close,
        IconsId_CloseDown,
        IconsId_CloseHot,

        IconsId_ELEMENTS_COUNT // pseudo id only to get number of icons
    };

    typedef struct tagIconInfos
    {
        TCHAR* AssociatedIconsPrefix;
        int AlternativeResourceId;
        int nIconIndex;
        TCHAR IconPath[MAX_PATH];
        HICON hIcon;
    }ICON_INFOS;

protected:
    HINSTANCE hInstance;
    
    BOOL bUseSkin;
    TCHAR SkinPreview[MAX_PATH];
    TCHAR SkinsPath[MAX_PATH];
    TCHAR CurrentSkinPath[MAX_PATH];
    ICON_INFOS IconInfosArray[IconsId_ELEMENTS_COUNT];

    std::vector<HICON> IconToDestroyList;
    COLORREF TransparentColor;
    BOOL bUseTransparentColor;
    CTransparentToolbar::tagBackgroundType BackGroundType;
    COLORREF BackGroundColor;
    COLORREF BackGroundColor2;
    TCHAR BackGroundImage[MAX_PATH];
    BYTE OpacityPercent;
    BOOL bMoveButtonEnabled;
    CTransparentToolbar::tagBarPositionMode PositionMode;

    HICON LoadIcon(TCHAR* IconRelativeOrAbsolutePath,int IconIndex,int AlternativeIconResourceId);
    HICON LoadIcon(int AlternativeIconResourceId);
    void FreeAllIcons();
    void SetDefaultConfiguration();
public:

    CBrowserToolbarOptions();
    ~CBrowserToolbarOptions(void);
    TCHAR* GetSkinsPath(){return this->SkinsPath;}
    
    BOOL Load(BOOL bUseSkin,TCHAR* SkinName);

    COLORREF GetCurrentSkinTransparentColor(){return this->TransparentColor;}
    BOOL GetCurrentSkinUseTransparentColor(){return this->bUseTransparentColor;}
    CTransparentToolbar::tagBackgroundType GetCurrentSkinBackGroundType(){return this->BackGroundType;}
    COLORREF GetCurrentSkinBackGroundColor(){return this->BackGroundColor;}
    COLORREF GetCurrentSkinBackGroundColor2(){return this->BackGroundColor2;}
    TCHAR* GetCurrentSkinBackGroundImage(){return this->BackGroundImage;}
    BYTE GetCurrentSkinOpacityPercent(){return this->OpacityPercent;}
    BOOL GetCurrentSkinMoveButtonEnableState(){return this->bMoveButtonEnabled;}
    CTransparentToolbar::tagBarPositionMode GetCurrentSkinPositionMode(){return this->PositionMode;}


    HICON GetCurrentSkinIcon(IconsId IconId);
    BOOL GetCurrentSkinPreviewPath(IN OUT TCHAR* Path,IN SIZE_T PathMaxSize);
};

