/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ContainerBase.h"

#include "QuickWayToFolders.h"
#include "..\Tools\ShellExecute\ShellExecuteAndBringToTop.h"

CContainerBase::CContainerBase(HINSTANCE hInstance,CQuickWayToFolders* pQuickWayToFolders)
{
    this->hIcon=NULL;
    this->hIconHot=NULL;
    this->hIconDown=NULL;
    this->pNotifyIcon=NULL;
    this->pLanguage=NULL;
    this->pPopUpMenu=NULL;
    this->hWndOwner = NULL;

    this->hInstance=hInstance;
    this->hDefaultIcon=NULL;
    this->pQuickWayToFolders=pQuickWayToFolders;

    this->LastClickedPoint.x = 0;
    this->LastClickedPoint.y = 0;

    this->ShowUpper = TRUE;
}

CContainerBase::~CContainerBase(void)
{
    if (this->pPopUpMenu)
    {
        delete this->pPopUpMenu;
        this->pPopUpMenu=NULL;
    }

    if (this->pNotifyIcon)
    {
        delete this->pNotifyIcon;
        this->pNotifyIcon=NULL;
    }

    this->DestroyIcons();
}

void CContainerBase::DestroyIcons()
{

    if (this->hIconHot)
    {
        if (this->hIconHot != this->hIcon)
        {
            ::DestroyIcon(this->hIconHot);
            this->hIconHot=NULL;
        }
    }

    if (this->hIconDown)
    {
        if (this->hIconDown != this->hIcon)
        {
            ::DestroyIcon(this->hIconDown);
            this->hIconDown=NULL;
        }
    }

    if (this->hIcon)
    {
        if (this->hIcon != this->hDefaultIcon)
        {
            ::DestroyIcon(this->hIcon);
            this->hIcon=NULL;
        }
    }
}

void CContainerBase::Destroy()
{
    // should never be called directly
#ifdef _DEBUG
    if (::IsDebuggerPresent())
        ::DebugBreak();
#endif
    delete this;
}

BOOL CContainerBase::SetLanguage(CLanguage* pLanguage)
{
    return this->InternalSetLanguage(pLanguage,TRUE);
}

BOOL CContainerBase::InternalSetLanguage(CLanguage* pLanguage,BOOL bAddShowMenuItem)
{
    // update language
    this->pLanguage=pLanguage;

    // recreate menu with new language
    if (this->pPopUpMenu)
        delete this->pPopUpMenu;

    this->pPopUpMenu=new CPopUpMenu();

    // clear menu map
    this->CommonPopUpMenuMap.clear();

    UINT MenuId;
    if (bAddShowMenuItem)
    {
        MenuId=this->pPopUpMenu->Add(pLanguage->Show,IDI_ICON_SHOW,this->hInstance,16,16);
        this->CommonPopUpMenuMap[MenuId]=COMMON_MENU_MAP_DATA_SHOW;

        this->pPopUpMenu->AddSeparator();
    }

    MenuId=this->pPopUpMenu->Add(pLanguage->Options,IDI_ICON_OPTIONS,this->hInstance,16,16);
    this->CommonPopUpMenuMap[MenuId]=COMMON_MENU_MAP_DATA_OPTIONS;

    this->pPopUpMenu->AddSeparator();

    MenuId=this->pPopUpMenu->Add(pLanguage->Help,IDI_ICON_HELP,this->hInstance,16,16);
    this->CommonPopUpMenuMap[MenuId]=COMMON_MENU_MAP_DATA_HELP;

    MenuId=this->pPopUpMenu->Add(pLanguage->About,IDI_ICON_ABOUT,this->hInstance,16,16);
    this->CommonPopUpMenuMap[MenuId]=COMMON_MENU_MAP_DATA_ABOUT;

    this->pPopUpMenu->AddSeparator();

    MenuId=this->pPopUpMenu->Add(pLanguage->CheckForUpdate,IDI_ICON_UPDATE,this->hInstance,16,16);
    this->CommonPopUpMenuMap[MenuId]=COMMON_MENU_MAP_DATA_CHECK_FOR_UPDATE;

    this->pPopUpMenu->AddSeparator();

    MenuId=this->pPopUpMenu->Add(pLanguage->MakeADonation,IDI_ICON_DONATION,this->hInstance,16,16);
    this->CommonPopUpMenuMap[MenuId]=COMMON_MENU_MAP_DATA_MAKEADONATION;

    this->pPopUpMenu->AddSeparator();

    MenuId=this->pPopUpMenu->Add(pLanguage->Exit,IDI_ICON_EXIT,this->hInstance,16,16);
    this->CommonPopUpMenuMap[MenuId]=COMMON_MENU_MAP_DATA_EXIT;

    return TRUE;
}

BOOL CContainerBase::ApplySettings(CSingleFolderOptions* pSingleFolderOptions)
{
    UNREFERENCED_PARAMETER(pSingleFolderOptions);
    // should never be called directly
#ifdef _DEBUG
    if (::IsDebuggerPresent())
        ::DebugBreak();
#endif
    return FALSE;
}

BOOL CContainerBase::OnHotKey()
{
    // update LastClickedPoint in case of menu
    ::GetCursorPos(&this->LastClickedPoint);
    // show or hide
    return this->Show(FALSE);
}

// bShowOnly : TRUE to show only, FALSE to Show or Hide according to internal state
BOOL CContainerBase::Show(BOOL bShowOnly)
{
    UNREFERENCED_PARAMETER(bShowOnly);
    // should never be called directly
#ifdef _DEBUG
    if (::IsDebuggerPresent())
        ::DebugBreak();
#endif
    return FALSE;
}

BOOL CContainerBase::Hide()
{
    // should never be called directly
#ifdef _DEBUG
    if (::IsDebuggerPresent())
        ::DebugBreak();
#endif
    return FALSE;
}

BOOL CContainerBase::ApplySettingsToClassObject(CSingleFolderOptions* pSingleFolderOptions)
{
    this->DestroyIcons();

    int cxIcon;
    int cyIcon;
    switch (this->pQuickWayToFolders->pApplicationOptions->LaunchMode)
    {
    default:
    case CApplicationOptions::LaunchMode_NOTIFY_ICON:
        cxIcon = 16;
        cyIcon = 16;
        break;
    case CApplicationOptions::LaunchMode_LAUNCH_BAR:
        cxIcon = this->pQuickWayToFolders->pApplicationOptions->LaunchBarOptions.IconWidth;
        cyIcon = this->pQuickWayToFolders->pApplicationOptions->LaunchBarOptions.IconHeight;
        break;
    } 

    // extract wanted icon
    if (
        //((int)::ExtractIconEx(pSingleFolderOptions->AssociatedIconPath,
        //                    pSingleFolderOptions->AssociatedIconIndex, 
        //                    NULL, 
        //                    &this->hIcon, 
        //                    1))
        ((int)::PrivateExtractIcons(pSingleFolderOptions->AssociatedIconPath,
                                    pSingleFolderOptions->AssociatedIconIndex,
                                    cxIcon,cyIcon,&this->hIcon,NULL,1,LR_DEFAULTCOLOR))
        <=0
        )
    {
        // on error load default icon
        this->hIcon=this->hDefaultIcon;
    }
    if (!this->hIcon)
        return FALSE;

    // szTip is 64 or 128 byte length according to os, so keep only last path
    TCHAR szTip[MAX_PATH];
    pSingleFolderOptions->GetShortName(szTip);

    if (this->pNotifyIcon)
    {
        delete this->pNotifyIcon;
        this->pNotifyIcon = NULL;
    }
    
    switch (this->pQuickWayToFolders->pApplicationOptions->LaunchMode)
    {
        case CApplicationOptions::LaunchMode_NOTIFY_ICON:
            {
                this->pNotifyIcon=new CNotifyIcon(this->hInstance,this->hIcon,szTip,this);
                // this->pNotifyIcon->Show();

                // at windows startup, icon creation can fail

                // Handling Shell_NotifyIcon failure
                //Shell_NotifyIcon will often fail when called during Windows startup (for instance, if your application is listed in HKLM\Software\Microsoft\Windows\CurrentVersion\Run. This appears to be because the system is busy starting applications. The failure is more common on low-spec computers or computers with some brands of antivirus software installed, which seem to be very intensive at startup.
                //
                //Unfortunately, you cannot rely on the error code returned by GetLastError. When Shell_NotifyIcon returns false, some of the common errors returned by GetLastError are:
                //
                //    ERROR_FILE_NOT_FOUND (2)
                //    ERROR_TIMEOUT (1460)
                //    ERROR_SUCCESS (0)
                //
                //The most appropriate response to any error returned by Shell_NotifyIcon is to sleep for a period of time and retry.
                //
                //An explanation of why the error code may differ has been made by Paul Baker, paraphrased from http://groups.google.com/group/microsoft.public.platformsdk.shell/msg/59235b293cbf5dfa and http://groups.google.com/group/microsoft.public.platformsdk.shell/msg/73973287f15c03fc:
                //
                //Shell_NotifyIcon actually calls SetLastError(0) initially. After that, basically it uses FindWindow to find the tray notification window. If this fails, it will typically return ERROR_FILE_NOT_FOUND. Otherwise it sends a WM_COPYDATA message to the tray notification window, using SendMessageTimeout with a timeout of only 4 seconds. If that message returns zero, then Shell_NotifyIcon will fail with GetLastError returning zero.
                //
                //reply to the above...
                //Applications that want to use the notification APIs that are running before or during explorer startup should listen for the notification message that indicates the taskbar is ready to receive API calls. This is the "TaskbarCreated" message. This also enables your application to re-register if the explorer is re-started.
                //This is described in the section titled "Taskbar Creation Notification on this page:
                //http://msdn.microsoft.com/en-us/library/cc144179(VS.85).aspx
                // let some seconds to create icons
                for (BYTE Cnt=0;Cnt<10;Cnt++)
                {
                    if (this->pNotifyIcon->Show())
                        break;
                    // else
                    ::Sleep(6000);
                }

                this->hWndOwner = this->pNotifyIcon->GetAssocatedNotifyIconHwnd();
            }
            break;
        case CApplicationOptions::LaunchMode_LAUNCH_BAR:
            {
                CTransparentToolbar::BUTTON_DESCRIPTION Button;
                memset(&Button,0,sizeof(Button));
                Button.hIcon = this->hIcon;
                
                // extract wanted icon
                if (
                    //((int)::ExtractIconEx(pSingleFolderOptions->LaunchBarAssociatedIconHotPath,
                    //                    pSingleFolderOptions->LaunchBarAssociatedIconHotIndex, 
                    //                    NULL, 
                    //                    &this->hIconHot, 
                    //                    1))
                    ((int)::PrivateExtractIcons(pSingleFolderOptions->LaunchBarAssociatedIconHotPath,
                                                pSingleFolderOptions->LaunchBarAssociatedIconHotIndex,
                                                cxIcon,cyIcon,&this->hIconHot,NULL,1,LR_DEFAULTCOLOR))
                    <=0
                    )
                {
                    // on error load default icon
                    Button.hIconHot = this->hIcon;
                }
                else
                    Button.hIconHot = this->hIconHot;

                // extract wanted icon
                if (
                    //((int)::ExtractIconEx(pSingleFolderOptions->LaunchBarAssociatedIconDownPath,
                    //                    pSingleFolderOptions->LaunchBarAssociatedIconDownIndex, 
                    //                    NULL, 
                    //                    &this->hIconDown, 
                    //                    1))
                    ((int)::PrivateExtractIcons(pSingleFolderOptions->LaunchBarAssociatedIconDownPath,
                                                pSingleFolderOptions->LaunchBarAssociatedIconDownIndex,
                                                cxIcon,cyIcon,&this->hIconDown,NULL,1,LR_DEFAULTCOLOR))
                    <=0
                    )
                {
                    // on error load default icon
                    Button.hIconDown = this->hIcon;
                }
                else
                    Button.hIconDown = this->hIconDown;

                // no disable icon yet use the default one
                Button.hIconDisabled = this->hIcon;
                Button.Position = pSingleFolderOptions->LaunchBarIconPosition;
                Button.SpacerBeforeIconInPixels = pSingleFolderOptions->LaunchBarFreeSpaceBeforeIcon;
#pragma message ("LineBreakBeforeIcon to be implemented")
                // Button.LineBreakBeforeIcon = pSingleFolderOptions->LaunchBarLineBreakBeforeIcon;
                Button.Enabled = TRUE;
                Button.ButtonId = (DWORD)this->pQuickWayToFolders->pLaunchBar->GetButtonsCount();
                Button.UserData = (PVOID)this;
                TCHAR ShortName[MAX_PATH];
                pSingleFolderOptions->GetShortName(ShortName);
                _tcsncpy(Button.ToolTipText,ShortName,CTransparentToolbar_ToolTipTextLen);
                Button.ToolTipText[CTransparentToolbar_ToolTipTextLen-1]=0;
                
                this->pQuickWayToFolders->pLaunchBar->AddButton(&Button);
                // this->hWndOwner can't be set until launch bar is not created (creation is done in CQuickWayToFolders::CreateAssociatedQuickWayToFoldersItems) 
                // this->hWndOwner = this->pQuickWayToFolders->pLaunchBar->GetControlHandle();
            }
            break;
    }

    return TRUE;
}

BOOL CContainerBase::ReturnHandleToOwnerWindowAfterShow()
{
    // should never be called directly
#ifdef _DEBUG
    if (::IsDebuggerPresent())
        ::DebugBreak();
#endif
    return TRUE;
}


// return TRUE to restore focus to notify icon, FALSE to keep focus
BOOL CContainerBase::OnNotifyIconClick(CNotifyIcon* pSender,LPPOINT pCursorPos)
{
    UNREFERENCED_PARAMETER(pSender);
    this->DoShow(pCursorPos,TRUE);

    return this->ReturnHandleToOwnerWindowAfterShow();
}

// return TRUE to restore focus to notify icon, FALSE to keep focus
BOOL CContainerBase::OnNotifyIconDoubleClick(CNotifyIcon* pSender,LPPOINT pCursorPos)
{
    return this->OnNotifyIconClick(pSender,pCursorPos);
}

// return TRUE to restore focus to notify icon, FALSE to keep focus
BOOL CContainerBase::OnNotifyIconRightClick(CNotifyIcon* pSender,LPPOINT pCursorPos)
{
    UNREFERENCED_PARAMETER(pSender);
    return this->DoPropertyShow(pCursorPos,TRUE);
}

// return TRUE : focus should be returned to owner window, FALSE to should be kept
void CContainerBase::DoShow(LPPOINT pCursorPos,BOOL ShowUpper)
{
    this->LastClickedPoint = *pCursorPos;
    this->ShowUpper = ShowUpper;
    this->Show(TRUE);
}
BOOL CContainerBase::DoPropertyShow(LPPOINT pCursorPos,BOOL ShowUpper)
{
    this->LastClickedPoint = *pCursorPos;
    this->ShowUpper = ShowUpper;

    if (!this->pPopUpMenu)
    {
#ifdef _DEBUG
        if (::IsDebuggerPresent())
            ::DebugBreak();
#endif
        return TRUE;
    }

    UINT ClickedMenuItemID;
    ClickedMenuItemID=this->pPopUpMenu->Show(pCursorPos->x,pCursorPos->y,this->hWndOwner,FALSE,ShowUpper);
    if (!ClickedMenuItemID)
        return FALSE;

    std::map<UINT,tagCommonMenuMapContent>::const_iterator MenuIdIterator;

    MenuIdIterator=this->CommonPopUpMenuMap.find(ClickedMenuItemID);
    if ( MenuIdIterator == this->CommonPopUpMenuMap.end() )
    {
#ifdef _DEBUG
        if (::IsDebuggerPresent())
            ::DebugBreak();
#endif
        return TRUE;
    }
    BOOL bReturnHandleToNotifyIconAfterShow=TRUE;
    switch(MenuIdIterator->second)
    {
    case COMMON_MENU_MAP_DATA_SHOW:
        this->Show(FALSE);
        bReturnHandleToNotifyIconAfterShow=this->ReturnHandleToOwnerWindowAfterShow();
        break;
    case COMMON_MENU_MAP_DATA_MAXIMIZE_REDUCE:
        this->MaximizeReduce();
        bReturnHandleToNotifyIconAfterShow=FALSE;
        break;
    case COMMON_MENU_MAP_DATA_OPTIONS:
        this->ShowOptions();
        bReturnHandleToNotifyIconAfterShow=FALSE;
        break;
    case COMMON_MENU_MAP_DATA_HELP:
        this->ShowHelp();
        bReturnHandleToNotifyIconAfterShow=FALSE;
        break;
    case COMMON_MENU_MAP_DATA_ABOUT:
        this->ShowAbout();
        bReturnHandleToNotifyIconAfterShow=FALSE;
        break;
    case COMMON_MENU_MAP_DATA_MAKEADONATION:
        this->MakeADonation();
        bReturnHandleToNotifyIconAfterShow=FALSE;
        break;
    case COMMON_MENU_MAP_DATA_CHECK_FOR_UPDATE:
        this->CheckForUpdate();
        bReturnHandleToNotifyIconAfterShow=FALSE;
        break;
    case COMMON_MENU_MAP_DATA_EXIT:
        if (this->pQuickWayToFolders)
            this->pQuickWayToFolders->Exit();
        break;
    }

    return bReturnHandleToNotifyIconAfterShow;
}

void CContainerBase::MaximizeReduce()
{

}

BOOL CContainerBase::ReportMessage(tagReportMessageType ReportMessageType,const TCHAR* pszMsg)
{
    BOOL bRet;
    UINT TimeOutInMs=15000;
    if (!this->pNotifyIcon)
        return FALSE;

    switch(ReportMessageType)
    {
    case REPORT_MESSAGE_TYPE_ERROR:
        bRet=this->pNotifyIcon->ShowBallon(pszMsg,_T("Error"),TimeOutInMs,CNotifyIcon::BALLOON_ICON_ERROR);
        break;
    case REPORT_MESSAGE_TYPE_WARNING:
        bRet=this->pNotifyIcon->ShowBallon(pszMsg,_T("Warning"),TimeOutInMs,CNotifyIcon::BALLOON_ICON_WARNING);
        break;
    default:
    case REPORT_MESSAGE_TYPE_INFORMATION:
        bRet=this->pNotifyIcon->ShowBallon(pszMsg,_T("Information"),TimeOutInMs,CNotifyIcon::BALLOON_ICON_INFO);
        break;
    }

    return bRet;
}

void CContainerBase::ShowOptions()
{
    // post message to main window as options can destroy current window
    ::PostMessage(this->pQuickWayToFolders->hWndDialog,CQuickWayToFolders_SHOW_OPTIONS,0,0);
}
void CContainerBase::ShowHelp()
{
    TCHAR sz[MAX_PATH];
    TCHAR szApplicationPath[MAX_PATH];
    CStdFileOperations::GetAppDirectory(szApplicationPath,MAX_PATH);
    _tcscpy(sz,szApplicationPath);
    _tcscat(sz,HELP_FILE);

    // if (((int)ShellExecute(NULL,_T("open"),sz,NULL,NULL,SW_SHOWNORMAL))<33)
    if (((int)CShellExecuteAndBringToTop::ShellExecute(NULL,_T("open"),sz,NULL,NULL,SW_SHOWNORMAL))<33)
    {
        _tcscpy(sz,_T("Error opening help file "));
        _tcscat(sz,HELP_FILE);
        ::MessageBox(NULL,sz,this->pLanguage->Error ,MB_OK|MB_ICONERROR | MB_SYSTEMMODAL);
    }
}
void CContainerBase::ShowAbout()
{
    TCHAR sz[2*MAX_PATH];
    _sntprintf(sz,2*MAX_PATH,
    _T("%s, %s <A ID=\"IdHomePage\">%s</A>\r\n%s <A ID=\"IdUserDoc\">%s</A>\r\n%s : Jacquelin POTIER\r\n               <A ID=\"IdMailto\">jacquelin.potier@free.fr</A>"),
    this->pLanguage->ForMoreInformations,
    this->pLanguage->See,
    this->pLanguage->ProjectHomePage,
    this->pLanguage->Or,
    this->pLanguage->UserManual,
    this->pLanguage->Author);
    CAbout::Show(this->hInstance,NULL,sz);
}
void CContainerBase::MakeADonation()
{
    BOOL bEuroCurrency=FALSE;
    HKEY hKey;
    wchar_t pszCurrency[2];
    DWORD Size=2*sizeof(wchar_t);
    memset(pszCurrency,0,Size);
    TCHAR pszMsg[3*MAX_PATH];

    // check that HKEY_CURRENT_USER\Control Panel\International\sCurrency contains the euro symbol
    // retrieve it in unicode to be quite sure of the money symbol
    if (::RegOpenKeyEx(HKEY_CURRENT_USER,_T("Control Panel\\International"),0,KEY_QUERY_VALUE,&hKey)==ERROR_SUCCESS)
    {
        // use unicode version only to make string compare
        if (::RegQueryValueExW(hKey,L"sCurrency",NULL,NULL,(LPBYTE)pszCurrency,&Size)==ERROR_SUCCESS)
        {
            if (wcscmp(pszCurrency,L"")==0)
                bEuroCurrency=TRUE;
        }
        // close open key
        ::RegCloseKey(hKey);
    }
    // yes, you can do it if u don't like freeware and open sources soft
    // but if you make it, don't blame me for not releasing sources anymore
    _tcscpy(pszMsg,_T("https://www.paypal.com/cgi-bin/webscr?cmd=_xclick&business=jacquelin.potier@free.fr")
        _T("&item_name=Donation%20for%20QuickWayToFolders&return=http://jacquelin.potier.free.fr/QuickWayToFolders/"));
    // in case of euro monetary symbol
    if (bEuroCurrency)
        // add it to link
        _tcscat(pszMsg,_T("&currency_code=EUR"));

    // open donation web page
    // if (((int)ShellExecute(NULL,_T("open"),pszMsg,NULL,NULL,SW_SHOWNORMAL))<33)
    if (((int)CShellExecuteAndBringToTop::ShellExecute(NULL,_T("open"),pszMsg,NULL,NULL,SW_SHOWNORMAL))<33)
        // display error msg in case of failure
        ::MessageBox(NULL,_T("Error Opening default browser. You can make a donation going to ")
        _T("http://jacquelin.potier.free.fr"),
        this->pLanguage->Error,MB_OK|MB_ICONERROR | MB_SYSTEMMODAL);
}
void CContainerBase::CheckForUpdate()
{
    CSoftwareUpdate::CheckForUpdate(QuickWayToFolders_PAD_Url,QuickWayToFoldersVersionSignificantDigits,QuickWayToFoldersDownloadLink);
}