/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#pragma once

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0501 // for xp os
#endif

#ifndef _WIN32_IE
#define _WIN32_IE 0x0500 // should be at least ie 5
#endif

#include <windows.h>
#include <map>

#include "../Tools/GUI/Menu/PopUpMenu.h"
#include "../Tools/GUI/ToolTip/ToolTip.h"
#include "../Tools/GUI/NotifyIcon/NotifyIcon.h"
#include "../Tools/SoftwareUpdate/SoftwareUpdate.h"

#include "Language.h"
#include "about.h"
#include "SingleFolderOptions.h"

#define CContainerBase_KEYDOWN_MSG (WM_USER+0x1000)

// defines for update checking
#define QuickWayToFolders_PAD_Url _T("http://jacquelin.potier.free.fr/QuickWayToFolders/QuickWayToFolders.xml")
#define QuickWayToFoldersVersionSignificantDigits 3
#ifdef _WIN64
#define QuickWayToFoldersDownloadLink _T("http://jacquelin.potier.free.fr/exe/QuickWayToFolders_bin64.zip")
#else
#define QuickWayToFoldersDownloadLink _T("http://jacquelin.potier.free.fr/exe/QuickWayToFolders_bin.zip")
#endif


#define HELP_FILE _T("QuickWayToFolders.chm")

class CQuickWayToFolders;

class CContainerBase:public CNotifyIconEventsBase
{
protected:
    enum tagReportMessageType
    {
        REPORT_MESSAGE_TYPE_ERROR,
        REPORT_MESSAGE_TYPE_WARNING,
        REPORT_MESSAGE_TYPE_INFORMATION
    };

    enum tagCommonMenuMapContent
    {
        COMMON_MENU_MAP_DATA_SHOW,
        COMMON_MENU_MAP_DATA_MAXIMIZE_REDUCE,
        COMMON_MENU_MAP_DATA_OPTIONS,
        COMMON_MENU_MAP_DATA_HELP ,
        COMMON_MENU_MAP_DATA_ABOUT,
        COMMON_MENU_MAP_DATA_MAKEADONATION,
        COMMON_MENU_MAP_DATA_CHECK_FOR_UPDATE,
        COMMON_MENU_MAP_DATA_EXIT
    };

    HINSTANCE hInstance;
    HICON hIcon;
    HICON hIconHot;
    HICON hIconDown;
    CNotifyIcon* pNotifyIcon;
    CPopUpMenu* pPopUpMenu;
    CLanguage* pLanguage;
    POINT LastClickedPoint;
    BOOL ShowUpper;

    std::map<UINT,tagCommonMenuMapContent> CommonPopUpMenuMap;

    BOOL ApplySettingsToClassObject(CSingleFolderOptions* pSingleFolderOptions);

    BOOL ReportMessage(tagReportMessageType ReportMessageType,const TCHAR* pszMsg);

    // return TRUE to restore focus to notify icon, FALSE to keep focus
    virtual BOOL OnNotifyIconClick(CNotifyIcon* pSender,LPPOINT pCursorPos);
    // return TRUE to restore focus to notify icon, FALSE to keep focus
    virtual BOOL OnNotifyIconRightClick(CNotifyIcon* pSender,LPPOINT pCursorPos);
    // return TRUE to restore focus to notify icon, FALSE to keep focus
    virtual BOOL OnNotifyIconDoubleClick(CNotifyIcon* pSender,LPPOINT pCursorPos);

    virtual BOOL ReturnHandleToOwnerWindowAfterShow();

    void ShowOptions();
    void ShowHelp();
    void ShowAbout();
    void MakeADonation();
    void CheckForUpdate();
    BOOL InternalSetLanguage(CLanguage* pLanguage,BOOL bAddShowMenuItem);
    virtual void MaximizeReduce();
    void DestroyIcons();
public:
    CContainerBase(HINSTANCE hInstance,CQuickWayToFolders* pQuickWayToFolders);
    virtual ~CContainerBase(void);

    HICON hDefaultIcon;
    HWND hWndOwner;
    CQuickWayToFolders* pQuickWayToFolders;
    virtual BOOL SetLanguage(CLanguage* pLanguage);
    virtual void Destroy();
    virtual BOOL ApplySettings(CSingleFolderOptions* pSingleFolderOptions);
    virtual BOOL UpdateOptionsBeforeClose()=0;
    virtual BOOL Show(BOOL bShowOnly);
    virtual BOOL Hide();
    virtual BOOL OnHotKey();

    // bShowOnly : TRUE to show only, FALSE to Show or Hide according to internal state
    void DoShow(LPPOINT pCursorPos,BOOL ShowUpper);
    BOOL DoPropertyShow(LPPOINT pCursorPos,BOOL ShowUpper);
};
