/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "DesktopBrowserContainerXP.h"
#include "QuickWayToFolders.h"

#define CDesktopBrowserContainerXP_WATCHED_EVENTS (SHCNE_CREATE | SHCNE_DELETE | SHCNE_DRIVEADD | SHCNE_DRIVEREMOVED | SHCNE_MEDIAINSERTED | SHCNE_MEDIAREMOVED | SHCNE_MKDIR | SHCNE_NETSHARE | SHCNE_NETUNSHARE | SHCNE_RENAMEFOLDER | SHCNE_RENAMEITEM | SHCNE_RMDIR | SHCNE_SERVERDISCONNECT | SHCNE_UPDATEDIR)
#define CDesktopBrowserContainerXP_WM_SHELL_CHANGE_NOTIFY (WM_APP+1345)
CDesktopBrowserContainerXP::CDesktopBrowserContainerXP(HINSTANCE hInstance,CQuickWayToFolders* pQuickWayToFolders):CShellBrowserContainerXP(hInstance,pQuickWayToFolders)
{
    this->pDesktopBrowser=NULL;
}


void CDesktopBrowserContainerXP::OnChangeNotify(long const lEvent, TCHAR* const sz1, TCHAR* const sz2)
{
    UNREFERENCED_PARAMETER(lEvent);
    UNREFERENCED_PARAMETER(sz1);
    UNREFERENCED_PARAMETER(sz2);
#ifdef _DEBUG
    OutputDebugString(_T("CDesktopBrowserContainerXP::Change notification for desktop\r\n"));
#endif

    // if we are not editing or acting on list view (else messages are handled by our DesktopBrowser object)
    HWND hForgroundWindow = ::GetForegroundWindow();
    if (hForgroundWindow == this->hWndContainer)
        return;

    //HWND hFocusWindow = ::GetFocus();
    //if ( (hFocusWindow == ListView_GetEditControl(this->hWndListView) )
    //    && (hFocusWindow == this->hWndListView)
    //    )
    //    return;

    // update desktop content to reflect reality
    MSG Msg;
    Msg.hwnd = this->hWndContainer;
    Msg.message = CContainerBase_KEYDOWN_MSG;
    Msg.lParam = 0;
    Msg.wParam = VK_BROWSER_REFRESH;
    Msg.pt.x = 0;
    Msg.pt.y = 0;
    Msg.time = ::GetTickCount();
    ::SendMessage(this->hWndContainer,CContainerBase_KEYDOWN_MSG,0,(LPARAM)&Msg); 
}

LRESULT CDesktopBrowserContainerXP::OnMessage(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    if (uMsg==CDesktopBrowserContainerXP_WM_SHELL_CHANGE_NOTIFY)
    {
        this->OnChangeMessage(wParam, lParam);
        return 0;
    }
    else
        return CShellBrowserContainerBase::OnMessage_ShellBrowserContainerBase(hWnd, uMsg, wParam, lParam);
}

CDesktopBrowserContainerXP::~CDesktopBrowserContainerXP(void)
{
    if (this->pDesktopBrowser)
    {
        delete this->pDesktopBrowser;
        this->pDesktopBrowser=NULL;
    }
}

void CDesktopBrowserContainerXP::Destroy()
{
    // avoid this->pShellBrowserEx deletion by CShellBrowserContainer::~CShellBrowserContainer()
    this->pShellBrowserEx=NULL;

    delete this;
}

BOOL CDesktopBrowserContainerXP::ApplySettings(CSingleFolderOptions* pSingleFolderOptions)
{
    if (this->pDesktopBrowser)
    {
        delete this->pDesktopBrowser;
        this->pDesktopBrowser=NULL;
        this->pShellBrowserEx=NULL;
    }
    
    this->pSingleFolderOptions=pSingleFolderOptions;

    if (!this->CreateContainer())
        return FALSE;

    FOLDERSETTINGS fs;
    fs.fFlags= FWF_NOWEBVIEW;
    CShellBrowserEx::ShellBrowserViewToFolderSetting(pSingleFolderOptions->FolderViewMode,&fs);

    RECT FolderViewRect;
    this->GetFolderViewRect(&FolderViewRect);
    this->pDesktopBrowser=new CDesktopBrowserXP(this->hWndContainer,&fs,&FolderViewRect);
    if (!this->ApplySettingsToClassObject(pSingleFolderOptions))
        return FALSE;

    if (FAILED(this->pDesktopBrowser->BrowseDesktop()))
        return FALSE;

    // must be done after browsing so can't be in ApplySettingsToClassObject
    if (this->pSingleFolderOptions->bShowAtStartup)
        this->Show(TRUE);

    // must be done after container creation
    ITEMIDLIST* pIdlList;
    if (SHGetSpecialFolderLocation(this->hWndContainer,CSIDL_DESKTOPDIRECTORY,&pIdlList)==S_OK)
    {
        if (pIdlList)
        {
            this->StartWatching(pIdlList,this->hWndContainer , CDesktopBrowserContainerXP_WM_SHELL_CHANGE_NOTIFY, CDesktopBrowserContainerXP_WATCHED_EVENTS, FALSE);
            ::CoTaskMemFree(pIdlList);
        }
    }

    return TRUE;
}

BOOL CDesktopBrowserContainerXP::ApplySettingsToClassObject(CSingleFolderOptions* pSingleFolderOptions)
{
    // set pShellBrowserEx
    this->pShellBrowserEx=(CShellBrowserEx*)this->pDesktopBrowser;

    // apply ShellBrowserEx settings
    return this->CShellBrowserContainerXP::ApplySettingsToClassObject(pSingleFolderOptions);
}

HRESULT CDesktopBrowserContainerXP::GoHome()
{
    if (!this->pDesktopBrowser)
        return S_FALSE;

    return this->pDesktopBrowser->BrowseDesktop();
}

// icon positions is internally handled by CDesktopBrowser
BOOL CDesktopBrowserContainerXP::DoesIconsPositionsNeedToBeSaved(LPITEMIDLIST CurrentPidl,LPITEMIDLIST ToBeBrowsePidl)
{
    UNREFERENCED_PARAMETER(CurrentPidl);
    UNREFERENCED_PARAMETER(ToBeBrowsePidl);
    return FALSE;
}

// icon positions is internally handled by CDesktopBrowser
BOOL CDesktopBrowserContainerXP::DoesIconsPositionsNeedToBeRestored()
{
    return FALSE;
}
// icon positions is internally handled by CDesktopBrowser
BOOL CDesktopBrowserContainerXP::DoesIconsPositionsNeedToBeSaved()
{
    return FALSE;
}