/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "FolderMenuContainer.h"
#include "../Tools/gui/DesktopWorkArea/DesktopWorkArea.h"

#ifndef _countof
#define _countof(array) (sizeof(array)/sizeof(array[0]))
#endif

#define CFolderMenuContainer_WATCHED_EVENTS (SHCNE_CREATE | SHCNE_DELETE | SHCNE_DRIVEADD | SHCNE_DRIVEREMOVED | SHCNE_MEDIAINSERTED | SHCNE_MEDIAREMOVED | SHCNE_MKDIR | SHCNE_NETSHARE | SHCNE_NETUNSHARE | SHCNE_RENAMEFOLDER | SHCNE_RENAMEITEM | SHCNE_RMDIR | SHCNE_SERVERDISCONNECT | SHCNE_UPDATEDIR)
CFolderMenuContainer::CFolderMenuContainer(HINSTANCE hInstance,CQuickWayToFolders* pQuickWayToFolders):CContainerBase(hInstance,pQuickWayToFolders)
{
    this->pFolderMenu=NULL;
    this->pAssociatedSingleFolderOptions = NULL;
    this->bFoldersContentChanged = FALSE;

    // get desktop size
    RECT DesktopRect;
    CDesktopWorkArea::GetWorkArea(&DesktopRect,TRUE);
    this->LastClickedPoint.x = DesktopRect.right;
    this->LastClickedPoint.y = DesktopRect.bottom;
}

LRESULT CALLBACK CFolderMenuContainer::HookWndProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData)
{
    UNREFERENCED_PARAMETER (uIdSubclass);
    switch(uMsg)
    {
    case WM_TIMER:
        {
            CFolderMenuContainer* pObject=(CFolderMenuContainer*)dwRefData;
            if (!pObject)
                return TRUE;

            switch (wParam)
            {
            case CFolderMenuContainer_ITEM_CHANGED_TIMER_ID:
                if (pObject->bFoldersContentChanged)
                {
                    pObject->bFoldersContentChanged = FALSE;
                    pObject->FillMenus();
                }
            	return TRUE;
            }
        }
        break;
    case CFolderMenuContainer_WM_SHELL_CHANGE_NOTIFY:
        {
            CFolderMenuContainer* pObject=(CFolderMenuContainer*)dwRefData;
            if (!pObject)
                return TRUE;
            pObject->OnChangeMessage(wParam, lParam);
            return TRUE;
        }
    }
    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
}

CFolderMenuContainer::~CFolderMenuContainer(void)
{
    this->RemoveEventsHooks();
    this->FreeMemory();
}

void CFolderMenuContainer::FreeMemory()
{
    if (this->pFolderMenu)
    {
        delete this->pFolderMenu;
        this->pFolderMenu=NULL;
    }
}

void CFolderMenuContainer::Destroy()
{
    delete this;
}

BOOL CFolderMenuContainer::ReturnHandleToOwnerWindowAfterShow()
{
    // return TRUE;
    return FALSE;
}

// WARNING : pSingleFolderOptions musn't be deleted before CFolderMenuContainer destruction (used for folder refreshing)
BOOL CFolderMenuContainer::ApplySettings(CSingleFolderOptions* pSingleFolderOptions)
{
    this->FreeMemory();
    this->pFolderMenu=new CFolderMenu();
    return this->ApplySettingsToClassObject(pSingleFolderOptions);
}


// WARNING : pSingleFolderOptions musn't be deleted before CFolderMenuContainer destruction (used for folder refreshing)
BOOL CFolderMenuContainer::ApplySettingsToClassObject(CSingleFolderOptions* pSingleFolderOptions)
{
    this->pAssociatedSingleFolderOptions = pSingleFolderOptions;
    this->CContainerBase::ApplySettingsToClassObject(pSingleFolderOptions);// notify icon is created inside this call

    this->pFolderMenu->SetFiltersExclusion(pSingleFolderOptions->ExclusionFilters);
    this->pFolderMenu->SetFiltersInclusion(pSingleFolderOptions->InclusionFilters);
    this->pFolderMenu->HideFilesExtension(pSingleFolderOptions->bHideFilesExtensionInPopupMenu);
    this->pFolderMenu->HideShortcutsOverlay(pSingleFolderOptions->bHideShortcutsOverlayInPopupMenu);
    this->pFolderMenu->SetFoldersPosition(pSingleFolderOptions->FoldersPositionInPopUpMenu);
    this->pFolderMenu->SetMenusMerge(pSingleFolderOptions->bMergeMenusInPopUpMenu);

    // watch for changes in directories to get menu up to date
    // can be done only after notify icon window creation
    this->AddEventHooks();

    // fill menu with content
    return this->FillMenus();
}
BOOL CFolderMenuContainer::FillMenus()
{
    TCHAR Path[MAX_PATH];
    if (this->pAssociatedSingleFolderOptions->FolderPresentationType==CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU)
    {
        BOOL bRet=TRUE;
        size_t Cnt;
        size_t NbValidItems;
        size_t NbItems=this->pAssociatedSingleFolderOptions->pMultiplePaths->size();
        CFolderMenu::MULTIPLE_PATHS_DATA* PathsData=new CFolderMenu::MULTIPLE_PATHS_DATA[NbItems];
        CFolderMenu::MULTIPLE_PATHS_DATA* pData;

        NbValidItems=0;
        for (Cnt=0;Cnt<NbItems;Cnt++)
        {
            pData=&this->pAssociatedSingleFolderOptions->pMultiplePaths->at(Cnt);

            if (!CSingleFolderOptions::GetPath(pData->Path,Path,_countof(Path)))
                continue;

            if (!CStdFileOperations::DoesDirectoryExists(Path))
            {
                TCHAR Msg[2*MAX_PATH];
                _sntprintf(Msg,_countof(Msg),this->pLanguage->FolderNotFound,Path);
                this->ReportMessage(REPORT_MESSAGE_TYPE_ERROR,Msg);
                continue;
            }
            // copy decoded path
            _tcscpy(PathsData[NbValidItems].Path,Path);
            // copy other informations
            _tcscpy(PathsData[NbValidItems].MergedSpecificFolderName,pData->MergedSpecificFolderName);
            PathsData[NbValidItems].MergeToSpecificFolder = pData->MergeToSpecificFolder;

            NbValidItems++;
        }


        if (!this->pFolderMenu->Fill(PathsData,(DWORD)NbValidItems,
                                    this->pAssociatedSingleFolderOptions->bIncludeSubFolders,
                                    this->pAssociatedSingleFolderOptions->bAllowSubFoldersNavigation,
                                    this->pAssociatedSingleFolderOptions->bAllowSubFoldersLinksNavigation
                                    )
            )
        {
            bRet = FALSE;
        }

        delete[] PathsData;

        return bRet;
    }
    else
    {
        if (!this->pAssociatedSingleFolderOptions->GetPath(Path,_countof(Path)))
            return FALSE;

        if (!CStdFileOperations::DoesDirectoryExists(Path))
        {
            TCHAR Msg[2*MAX_PATH];
            _sntprintf(Msg,_countof(Msg),this->pLanguage->FolderNotFound,Path);
            this->ReportMessage(REPORT_MESSAGE_TYPE_ERROR,Msg);
            return FALSE;
        }

        CFolderMenu::MULTIPLE_PATHS_DATA Data={0};
        // copy decoded path
        _tcscpy(Data.Path,Path);
        // set other informations
        Data.MergeToSpecificFolder = FALSE;
        *Data.MergedSpecificFolderName=0;
        
        
        if (!this->pFolderMenu->Fill(&Data,
                                    this->pAssociatedSingleFolderOptions->bIncludeSubFolders,
                                    this->pAssociatedSingleFolderOptions->bAllowSubFoldersNavigation,
                                    this->pAssociatedSingleFolderOptions->bAllowSubFoldersLinksNavigation
                                    )
           )
        {
            return FALSE;
        }
    }

    return TRUE;
}

// stop watching changes on menu folders
BOOL CFolderMenuContainer::RemoveEventsHooks()
{
    this->StopWatching();
    ::KillTimer(this->hWndOwner,CFolderMenuContainer_ITEM_CHANGED_TIMER_ID);

    // remove subclassing wndproc
    UINT_PTR uIdSubclass=0;
    return RemoveWindowSubclass(this->hWndOwner,CFolderMenuContainer::HookWndProc,uIdSubclass);
}

// start watching changes on menu folders
BOOL CFolderMenuContainer::AddEventHooks()
{
    
    // set timer to delay refreshing in case of multiple item change notifications
    this->bFoldersContentChanged = FALSE;
    ::SetTimer(this->hWndOwner,CFolderMenuContainer_ITEM_CHANGED_TIMER_ID,CFolderMenuContainer_ITEM_CHANGED_DELAY,NULL);

    // subclassing can be done only after notify icon window creation
    UINT_PTR uIdSubclass=0;
    DWORD_PTR dwRefData=(DWORD_PTR)this;
    SetWindowSubclass(this->hWndOwner,CFolderMenuContainer::HookWndProc,uIdSubclass,dwRefData);
    
    TCHAR Path[MAX_PATH];
    if (this->pAssociatedSingleFolderOptions->FolderPresentationType==CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU)
    {
        TCHAR* pszPath;
        size_t Cnt;
        size_t NbValidItems;
        size_t NbItems=this->pAssociatedSingleFolderOptions->pMultiplePaths->size();

        /////////////////////
        // start watching like
        /////////////////////

        SHChangeNotifyEntry* pEntries = new SHChangeNotifyEntry[NbItems];
        NbValidItems = 0;
        // get root folder
        HRESULT hResult= E_FAIL;
        IShellFolder* pDesktopFolder=NULL;
        WCHAR* pwscPath;
        ITEMIDLIST* pidlToWatch=NULL;

        hResult = ::SHGetDesktopFolder(&pDesktopFolder);
        if (SUCCEEDED(hResult)&&(pDesktopFolder))
        {

            for (Cnt=0;Cnt<NbItems;Cnt++)
            {
                pidlToWatch=NULL;

                pszPath=this->pAssociatedSingleFolderOptions->pMultiplePaths->at(Cnt).Path;

                if (!CSingleFolderOptions::GetPath(pszPath,Path,_countof(Path)))
                    continue;

                if (!CStdFileOperations::DoesDirectoryExists(Path))
                    continue;
                
                // convert ansi to unicode if needed 
#if ( defined(UNICODE) || defined(_UNICODE))
                pwscPath=Path;
#else
                size_t NbCharacters = strlen(ItemToWatch)+1;

                pwscPath = (WCHAR*) malloc(NbCharacters*sizeof(WCHAR));
                if (!pwscPath)
                    goto CleanUp;

                // Convert to Unicode.
                if (::MultiByteToWideChar(CP_ACP, 0, Path, (int)NbCharacters,pwscPath, (int)NbCharacters)==0)
                    goto CleanUp;
#endif


                // get object full pidl from path name
                hResult = pDesktopFolder->ParseDisplayName(NULL,
                                                            NULL,
                                                            pwscPath,
                                                            NULL,
                                                            &pidlToWatch,
                                                            NULL);
                if ( FAILED(hResult) || (!pidlToWatch) )
                    goto CleanUp;

                // add item to array
                pEntries[NbValidItems].pidl = pidlToWatch;
                pEntries[NbValidItems].fRecursive = this->pAssociatedSingleFolderOptions->bIncludeSubFolders;
                NbValidItems++;

CleanUp:
#if ( (!defined(UNICODE)) && (!defined(_UNICODE)) )
                if (pwscPath)
                    free(pwscPath);
#endif
                continue; // instruction is needed by cleanup label
            }
            pDesktopFolder->Release();
        }

        this->StopWatching();

        int const nSources = SHCNRF_ShellLevel | SHCNRF_InterruptLevel | SHCNRF_NewDelivery;
        this->RegistrationID = ::SHChangeNotifyRegister(this->hWndOwner, nSources, CFolderMenuContainer_WATCHED_EVENTS, CFolderMenuContainer_WM_SHELL_CHANGE_NOTIFY, (int)NbValidItems, pEntries);

        for (Cnt=0;Cnt<NbValidItems;Cnt++)
        {
            ::ILFree((LPITEMIDLIST)pEntries[Cnt].pidl);
        }
        delete[] pEntries;

        /////////////////////
        // end of start watching like
        /////////////////////
    }
    else
    {
        if (this->pAssociatedSingleFolderOptions->GetPath(Path,_countof(Path)))
        {
            if (CStdFileOperations::DoesDirectoryExists(Path))
            {
                this->StartWatching(Path,this->hWndOwner , CFolderMenuContainer_WM_SHELL_CHANGE_NOTIFY, CFolderMenuContainer_WATCHED_EVENTS, this->pAssociatedSingleFolderOptions->bAllowSubFoldersNavigation);
            }
        }
    }
    return this->RegistrationID!=0;
}

BOOL CFolderMenuContainer::Show(BOOL bShowOnly)
{
    UNREFERENCED_PARAMETER(bShowOnly);
    BOOL bRet;
    bRet = this->pFolderMenu->Show(this->LastClickedPoint.x,this->LastClickedPoint.y,this->hWndOwner,FALSE,this->ShowUpper);

	// 1.2.3 to 1.2.4 change : in previous versions menu was kept in memory for better speed result,
	// but the trouble was the increase of opened gdi handles --> this could cause application crash
	// thanks to M$ limitation of 9999 opened objects
	// so we have to free gdi handle as often as possible; here after menu showing
	this->FillMenus(); // free menus and reload first level menus
	// end of 1.2.3 to 1.2.4 change

	return bRet;
}
BOOL CFolderMenuContainer::Hide()
{
    // menu is hide on any click
    return TRUE;
}

void CFolderMenuContainer::OnChangeNotify(long const lEvent, TCHAR* const sz1, TCHAR* const sz2)
{
    UNREFERENCED_PARAMETER(lEvent);
    UNREFERENCED_PARAMETER(sz1);
    UNREFERENCED_PARAMETER(sz2);
#ifdef _DEBUG
    OutputDebugString(_T("CFolderMenuContainer::Change notification for menu\r\n"));
#endif
    // do a delay to avoid to process many changes notifications if they come in quite the same time
    // this->FillMenus();
    this->bFoldersContentChanged = TRUE;
}