/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "GenericQuickWayToFolderObject.h"

#include "ShellBrowserContainer.h"
#include "ShellBrowserContainerXP.h"
#include "DesktopBrowserContainer.h"
#include "DesktopBrowserContainerXP.h"
#include "FolderMenuContainer.h"
#include "QuickWayToFolders.h"
#include "../Tools/Version/OsVersion.h"

CGenericQuickWayToFolderObject::CGenericQuickWayToFolderObject(void)
{
    this->pContainer=NULL;
}

CGenericQuickWayToFolderObject::~CGenericQuickWayToFolderObject(void)
{
    if (!this->pContainer)
        return;

    // call specialized destructor
    this->pContainer->Destroy();

    this->pContainer=NULL;
}

BOOL CGenericQuickWayToFolderObject::UpdateOptionsBeforeClose()
{
    if (this->pContainer == NULL)
        return FALSE;

    return this->pContainer->UpdateOptionsBeforeClose();
}

BOOL CGenericQuickWayToFolderObject::ApplySettings(CQuickWayToFolders* pQuickWayToFolders,HINSTANCE hInstance,HICON hDefaultIcon,CLanguage* pLanguage)
{
    if (this->FolderPresentationType!=FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU)
    {
        TCHAR AbsolutePath[MAX_PATH];
        if (!this->GetPath(AbsolutePath,_countof(AbsolutePath)))
        {
            TCHAR szMsg[MAX_PATH+128];
            _sntprintf(szMsg,_countof(szMsg),_T("File path %s does not exists !"),this->Path);
            ::MessageBox(NULL,szMsg,_T("Error"),MB_OK | MB_ICONERROR | MB_SYSTEMMODAL);
            return FALSE;
        }
    }

    switch(this->FolderPresentationType)
    {
    case FOLDER_PRESENTATION_TYPE_DESKTOP_BROWSER:
        {
            COsVersion OsVersion;
            if (OsVersion.IsXp())
                this->pContainer=(CContainerBase*) new CDesktopBrowserContainerXP(hInstance,pQuickWayToFolders);
            else
                this->pContainer=(CContainerBase*) new CDesktopBrowserContainer(hInstance,pQuickWayToFolders);
        }
        break;
    case FOLDER_PRESENTATION_TYPE_BROWSER:
        {
            COsVersion OsVersion;
            if (OsVersion.IsXp())
                this->pContainer=(CContainerBase*) new CShellBrowserContainerXP(hInstance,pQuickWayToFolders);
            else
                this->pContainer=(CContainerBase*) new CShellBrowserContainer(hInstance,pQuickWayToFolders);
        }
        
        break;
    case FOLDER_PRESENTATION_TYPE_SINGLE_MENU:
    case FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU:
        this->pContainer=(CContainerBase*) new CFolderMenuContainer(hInstance,pQuickWayToFolders);
        break;
    }

    if (!this->pContainer)
        return FALSE;

    // set default icon handle
    this->pContainer->hDefaultIcon=hDefaultIcon;

    // set language
    this->pContainer->SetLanguage(pLanguage);

    // call specialized apply settings
    return this->pContainer->ApplySettings((CSingleFolderOptions*)this);    
}

void CGenericQuickWayToFolderObject::Clone(IN OUT CGenericQuickWayToFolderObject* pGenericQuickWayToFolderObject)
{
    size_t Cnt;

    ///////////////////////
    // save pointer values
    ///////////////////////
    std::vector<CFolderMenu::MULTIPLE_PATHS_DATA>* pTmpMultiplePaths;
    // save pGenericQuickWayToFolderObject->pMultiplePaths
    pTmpMultiplePaths=pGenericQuickWayToFolderObject->pMultiplePaths;

    std::vector<VECTOR_ICONS_POSITIONS_FOR_HOME_FOLDER>* pTmpIconsPositionsForHomeFolder;
    // save pGenericQuickWayToFolderObject->pIconsPositionsForHomeFolder
    pTmpIconsPositionsForHomeFolder=pGenericQuickWayToFolderObject->pIconsPositionsForHomeFolder;

    ///////////////////////
    // do a dirty object copy
    ///////////////////////
    memcpy(pGenericQuickWayToFolderObject,this,sizeof(CGenericQuickWayToFolderObject));

    ///////////////////////
    // restore pointer values and copy pointers content
    ///////////////////////

    // restore pGenericQuickWayToFolderObject->pMultiplePaths
    pGenericQuickWayToFolderObject->pMultiplePaths=pTmpMultiplePaths;

    // copy vector content
    for (Cnt=0;Cnt<this->pMultiplePaths->size();Cnt++)
    {
        pGenericQuickWayToFolderObject->pMultiplePaths->push_back(this->pMultiplePaths->at(Cnt));
    }

    // restore pGenericQuickWayToFolderObject->pIconsPositionsForHomeFolder;
    pGenericQuickWayToFolderObject->pIconsPositionsForHomeFolder=pTmpIconsPositionsForHomeFolder;

    // copy vector content
    for (Cnt=0;Cnt<this->pIconsPositionsForHomeFolder->size();Cnt++)
    {
        pGenericQuickWayToFolderObject->pIconsPositionsForHomeFolder->push_back(this->pIconsPositionsForHomeFolder->at(Cnt));
    }
}