/*
Copyright (C) 2014 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2014 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: Manage QuickWayToFolder Launch bar
//-----------------------------------------------------------------------------

#include "LaunchBar.h"
#include "ContainerBase.h"
#include "../Tools/GUI/DesktopWorkArea/DesktopWorkArea.h"
#include "../Tools/File/StdFileOperations.h"

CLaunchBar::CLaunchBar(void)
{
    this->pTransparentToolBar = NULL;
    this->pHotWindow = NULL;
    this->Position = CLaunchBarOptions::POSITION_TOP_LEFT;
    this->pLaunchBarContainerBase = NULL;
}


CLaunchBar::~CLaunchBar(void)
{
    this->Destroy();
}


void CLaunchBar::OnTransparentToolbarButtonClick(CTransparentToolbar* pSender,DWORD ButtonId)
{
    // avoid to auto hide bar during menu display (explorer display won't be affected as pContainerBase->DoShow will return immediately)
    this->pHotWindow->StopAutoHide();

    CContainerBase* pContainerBase;
    if (!pSender->GetButtonUserData(ButtonId,(PVOID*)&pContainerBase))
        return;
    // Instead of providing button position, adjust cursor position according to icon and bar position
    POINT CursosrPos;
    // ::GetCursorPos(&CursosrPos);

    RECT ButtonRect;
    pSender->GetButtonRect(ButtonId,&ButtonRect);
    BOOL ShowUpper = FALSE;

    // if vertical launch bar
    if (this->pTransparentToolBar->IsVertical())
    {
        switch(this->Position)
        {
        default:
        case CLaunchBarOptions::POSITION_TOP_LEFT:
        // case CLaunchBarOptions::POSITION_TOP_CENTER:
            CursosrPos.x=ButtonRect.right;
            CursosrPos.y=ButtonRect.top;
            break;

        case CLaunchBarOptions::POSITION_TOP_RIGHT:
            CursosrPos.x=ButtonRect.left;
            CursosrPos.y=ButtonRect.top;
            break;
        case CLaunchBarOptions::POSITION_LEFT_MIDDLE:
            CursosrPos.x=ButtonRect.right;
            CursosrPos.y=ButtonRect.top;
            break;
        case CLaunchBarOptions::POSITION_RIGHT_MIDDLE:
            CursosrPos.x=ButtonRect.left;
            CursosrPos.y=ButtonRect.top;
            break;
        case CLaunchBarOptions::POSITION_BOTTOM_LEFT:
        // case CLaunchBarOptions::POSITION_BOTTOM_CENTER:
            CursosrPos.x=ButtonRect.right;
            CursosrPos.y=ButtonRect.bottom;
            ShowUpper = TRUE;
            break;
        case CLaunchBarOptions::POSITION_BOTTOM_RIGHT:
            CursosrPos.x=ButtonRect.left;
            CursosrPos.y=ButtonRect.bottom;
            ShowUpper = TRUE;
            break;
        }
    }
    else // if horizontal launch bar
    {
        switch(this->Position)
        {
        default:
        case CLaunchBarOptions::POSITION_TOP_LEFT:
        case CLaunchBarOptions::POSITION_TOP_CENTER:
            CursosrPos.x=ButtonRect.left;
            CursosrPos.y=ButtonRect.bottom;
            break;
        
        case CLaunchBarOptions::POSITION_TOP_RIGHT:
            CursosrPos.x=ButtonRect.right;
            CursosrPos.y=ButtonRect.bottom;
            break;
        // case CLaunchBarOptions::POSITION_LEFT_MIDDLE:
        //    break;
        // case CLaunchBarOptions::POSITION_RIGHT_MIDDLE:
        //    break;
        case CLaunchBarOptions::POSITION_BOTTOM_LEFT:
        case CLaunchBarOptions::POSITION_BOTTOM_CENTER:
            CursosrPos.x=ButtonRect.left;
            CursosrPos.y=ButtonRect.top;
            ShowUpper = TRUE;
            break;
        case CLaunchBarOptions::POSITION_BOTTOM_RIGHT:
            CursosrPos.x=ButtonRect.right;
            CursosrPos.y=ButtonRect.top;
            ShowUpper = TRUE;
            break;
        }
    }

    pContainerBase->DoShow(&CursosrPos,ShowUpper);

    // restart auto hide
    this->pHotWindow->StartAutoHide();
}

void CLaunchBar::OnTransparentToolbarBackgroundRightClick(CTransparentToolbar* pSender,POINT* pCursorPos)
{
    UNREFERENCED_PARAMETER(pSender);

    // avoid to auto hide bar during menu display
    this->pHotWindow->StopAutoHide();

    BOOL ShowUpper = FALSE;

    switch(this->Position)
    {
    case CLaunchBarOptions::POSITION_BOTTOM_LEFT:
    case CLaunchBarOptions::POSITION_BOTTOM_CENTER:
    case CLaunchBarOptions::POSITION_BOTTOM_RIGHT:
        ShowUpper = TRUE;
        break;
    }

    // show menu
    this->pLaunchBarContainerBase->DoPropertyShow(pCursorPos,ShowUpper);

    // restart auto hide
    this->pHotWindow->StartAutoHide();
}

void CLaunchBar::OnTransparentToolbarButtonRightClick(CTransparentToolbar* pSender,DWORD ButtonId)
{
    // avoid to auto hide bar during menu display
    this->pHotWindow->StopAutoHide();

    CContainerBase* pContainerBase;
    if (!pSender->GetButtonUserData(ButtonId,(PVOID*)&pContainerBase))
        return;
    POINT CursosrPos;
    ::GetCursorPos(&CursosrPos);

    BOOL ShowUpper = FALSE;

    switch(this->Position)
    {
    case CLaunchBarOptions::POSITION_BOTTOM_LEFT:
    case CLaunchBarOptions::POSITION_BOTTOM_CENTER:
    case CLaunchBarOptions::POSITION_BOTTOM_RIGHT:
        ShowUpper = TRUE;
        break;
    }

    // show menu
    pContainerBase->DoPropertyShow(&CursosrPos,ShowUpper);

    // restart auto hide
    this->pHotWindow->StartAutoHide();
}

void CLaunchBar::OnHotWindowAutoShow(CHotWindow* pSender,OUT BOOL* pUseAutoHideDisabledExtraRect,OUT RECT* pAutoHideDisabledExtraRect)
{
    UNREFERENCED_PARAMETER(pSender);
    UNREFERENCED_PARAMETER(pAutoHideDisabledExtraRect);
    *pUseAutoHideDisabledExtraRect = TRUE;
    ::GetWindowRect(this->GetControlHandle(),pAutoHideDisabledExtraRect);
    this->Show();
}
void CLaunchBar::OnHotWindowAutoHide(CHotWindow* pSender)
{
    UNREFERENCED_PARAMETER(pSender);
    HWND hWnd =::GetForegroundWindow();// to correct bug seen on first options show
    // end menu before hiding (avoid undeleted menu errors)
    ::EndMenu();
    this->Hide();
    ::SetForegroundWindow(hWnd);// to correct bug seen on first options show
}

HWND CLaunchBar::GetControlHandle()
{
    if (this->pTransparentToolBar)
        return this->pTransparentToolBar->GetControlHandle();
    return NULL;
}

BOOL CLaunchBar::Create(CQuickWayToFolders* pQuickWayToFolders,HINSTANCE hInstance,CLaunchBarOptions* pOptions)
{
    this->Destroy();

    this->pLaunchBarContainerBase = new CLaunchBarContainerBase(hInstance,pQuickWayToFolders);

    this->Position = pOptions->Position;

    RECT HotWindowActiveRect;
    RECT UsedScreenRect;
    DWORD Width = pOptions->MinWidth;
    DWORD Height = pOptions->MinHeight;
    LONG x=0;
    LONG y=0;
    // Notice size of launch bar will be provided after layout computation done by CTransparentToolbar constructor
    // so positioning of launch bar must be done after it's creation

    // correct bVertical if needed
    switch(this->Position)
    {
    case CLaunchBarOptions::POSITION_LEFT_MIDDLE:
    case CLaunchBarOptions::POSITION_RIGHT_MIDDLE:
        pOptions->bVertical = TRUE;// vertical only
        break;
    default:
        pOptions->bVertical = FALSE;// horizontal only
        break;

    }

    if (pOptions->bVertical)
    {
        // switch min width and min height
        Height = pOptions->MinWidth;
        Width = pOptions->MinHeight;
    }
    else
    {
        Width = pOptions->MinWidth;
        Height = pOptions->MinHeight;
    }

    // convert vector to array
    SIZE_T ArrayButtonSize = this->ButtonsList.size();
    CTransparentToolbar::BUTTON_DESCRIPTION* ArrayButtons = new CTransparentToolbar::BUTTON_DESCRIPTION[ArrayButtonSize];
    std::vector<CTransparentToolbar::BUTTON_DESCRIPTION>::iterator Iter;
    SIZE_T Index;
    for (Iter = this->ButtonsList.begin(),Index = 0;
        Iter != this->ButtonsList.end();
        Iter++,Index++)
    {
        ArrayButtons[Index] = *Iter;
    }

    // assume to provide an absolute path to CTransparentToolbar
    TCHAR BackgroundImagePathAbsolutePath[MAX_PATH];
    CStdFileOperations::GetAbsolutePath(NULL,pOptions->BackGroundImage,BackgroundImagePathAbsolutePath,_countof(BackgroundImagePathAbsolutePath));

    // create the launch bar
    this->pTransparentToolBar = new CTransparentToolbar(hInstance,NULL,this,FALSE,
                                                        pOptions->bVertical,
                                                        pOptions->BackgroundType,pOptions->BackGroundColor,pOptions->BackGroundGradiantColor1,pOptions->BackGroundGradiantColor2,BackgroundImagePathAbsolutePath,
                                                        pOptions->TransparentColor,pOptions->bUseTransparentColor,pOptions->OpacityPercent,
                                                        CTransparentToolbar::BarPositionMode_POPUP,
                                                        x,y,Width,Height,
                                                        pOptions->IconWidth,pOptions->IconHeight,
                                                        ArrayButtons,ArrayButtonSize,
                                                        NULL,FALSE,FALSE,&pOptions->Margins,TRUE
                                                        );

    delete[] ArrayButtons;

    // if toolbar creation is successful, create the hotWindow area
    if (this->pTransparentToolBar)
    {
        RECT LaunchBarRect;
        ::GetWindowRect(this->pTransparentToolBar->GetControlHandle(),&LaunchBarRect);
        Width = LaunchBarRect.right-LaunchBarRect.left;
        Height = LaunchBarRect.bottom-LaunchBarRect.top;

        CDesktopWorkArea::GetWorkAreaForMonitor(pOptions->ScreenIndex,&UsedScreenRect);

        // manage full screen size (width or height according to launch bar direction
        if (pOptions->bUseFullScreenWidhtOrHeight)
        {
            if (pOptions->bVertical)
                Height = UsedScreenRect.bottom-UsedScreenRect.top;
            else
                Width = UsedScreenRect.right-UsedScreenRect.left;
        }

        // manage position
        switch(this->Position)
        {
        default:
        case CLaunchBarOptions::POSITION_TOP_LEFT:
            x=0;
            y=0;
            break;
        case CLaunchBarOptions::POSITION_TOP_CENTER:
            //pOptions->bVertical = FALSE;// horizontal only
            x=((UsedScreenRect.right-UsedScreenRect.left)-Width) /2;
            y=0;
            break;
        case CLaunchBarOptions::POSITION_TOP_RIGHT:
            x=UsedScreenRect.right- Width;
            y=0;
            break;
        case CLaunchBarOptions::POSITION_LEFT_MIDDLE:
            pOptions->bVertical = TRUE;// vertical only
            x=0;
            y=((UsedScreenRect.bottom-UsedScreenRect.top)-Height) /2;
            break;
        case CLaunchBarOptions::POSITION_RIGHT_MIDDLE:
            pOptions->bVertical = TRUE;// vertical only
            x=UsedScreenRect.right- Width;
            y=((UsedScreenRect.bottom-UsedScreenRect.top)-Height) /2;
            break;
        case CLaunchBarOptions::POSITION_BOTTOM_LEFT:
            x=0;
            y=UsedScreenRect.bottom-Height;
            break;
        case CLaunchBarOptions::POSITION_BOTTOM_CENTER:
            //pOptions->bVertical = FALSE;// horizontal only
            x=((UsedScreenRect.right-UsedScreenRect.left)-Width) /2;
            y=UsedScreenRect.bottom-Height;
            break;
        case CLaunchBarOptions::POSITION_BOTTOM_RIGHT:
            x=UsedScreenRect.right- Width;
            y=UsedScreenRect.bottom-Height;
            break;
        }

        // adjust position
        this->pTransparentToolBar->SetPosition(x,y,Width,Height);

        // create hot area
        this->pHotWindow = new CHotWindow((IHotWindowEvents*)((CHotWindowEventsBase*)this));

        if (pOptions->bVertical)
        {
            // keep height
            HotWindowActiveRect.top = y;
            HotWindowActiveRect.bottom = HotWindowActiveRect.top + Height;
            // limit width of area to specified zone (avoid 64px active area for 64px icons)

            switch(this->Position)
            {
            default:
            case CLaunchBarOptions::POSITION_TOP_LEFT:
            case CLaunchBarOptions::POSITION_LEFT_MIDDLE:
            case CLaunchBarOptions::POSITION_BOTTOM_LEFT:
                HotWindowActiveRect.left = x;
                HotWindowActiveRect.right = HotWindowActiveRect.left+pOptions->ActiveZoneSizeInPixel;
                break;
            case CLaunchBarOptions::POSITION_TOP_RIGHT:
            case CLaunchBarOptions::POSITION_RIGHT_MIDDLE:
            case CLaunchBarOptions::POSITION_BOTTOM_RIGHT:
                HotWindowActiveRect.right = x+Width;
                HotWindowActiveRect.left = HotWindowActiveRect.right-pOptions->ActiveZoneSizeInPixel;
                break;
            // case CLaunchBarOptions::POSITION_BOTTOM_CENTER:
            //    break;
            // case CLaunchBarOptions::POSITION_TOP_CENTER:
            //    break;
            }
        }
        else
        {
            // keep width
            HotWindowActiveRect.left = x;
            HotWindowActiveRect.right = HotWindowActiveRect.left + Width;
            // limit height of area to specified zone (avoid 64px active area for 64px icons)

            switch(this->Position)
            {
            default:
            case CLaunchBarOptions::POSITION_TOP_LEFT:
            case CLaunchBarOptions::POSITION_TOP_CENTER:
            case CLaunchBarOptions::POSITION_TOP_RIGHT:
                HotWindowActiveRect.top = y;
                HotWindowActiveRect.bottom = HotWindowActiveRect.top+pOptions->ActiveZoneSizeInPixel;
                break;
            // case CLaunchBarOptions::POSITION_LEFT_MIDDLE:
            //    break;
            // case CLaunchBarOptions::POSITION_RIGHT_MIDDLE:
            //    break;
            case CLaunchBarOptions::POSITION_BOTTOM_LEFT:
            case CLaunchBarOptions::POSITION_BOTTOM_CENTER:
            case CLaunchBarOptions::POSITION_BOTTOM_RIGHT:
                HotWindowActiveRect.bottom = y+Height;
                HotWindowActiveRect.top = HotWindowActiveRect.bottom-pOptions->ActiveZoneSizeInPixel;
                break;
            }

        }
        this->pHotWindow->SetWindowAndActiveRect(this->GetControlHandle(),&HotWindowActiveRect);
        this->pHotWindow->StartAutoShow();
    }

    return TRUE;
}
void CLaunchBar::Destroy()
{
    if (this->pTransparentToolBar)
    {
        delete this->pTransparentToolBar;
        this->pTransparentToolBar = NULL;
    }
    if (this->pHotWindow)
    {
        delete this->pHotWindow;
        this->pHotWindow = NULL;
    }
    if (this->pLaunchBarContainerBase)
    {
        delete this->pLaunchBarContainerBase;
        this->pLaunchBarContainerBase = NULL;
    }

}

BOOL CLaunchBar::AddButton(CTransparentToolbar::BUTTON_DESCRIPTION* pButton)
{
    this->ButtonsList.push_back(*pButton);
    return TRUE;
}
void CLaunchBar::RemoveAllButtons()
{
    this->ButtonsList.clear();
}
SIZE_T CLaunchBar::GetButtonsCount()
{
    return this->ButtonsList.size();
}

BOOL CLaunchBar::Show()
{
    if (this->pTransparentToolBar==0)
        return FALSE;
    return this->pTransparentToolBar->Show();
}
BOOL CLaunchBar::Hide()
{
    if (this->pTransparentToolBar==0)
        return FALSE;
    return this->pTransparentToolBar->Hide();
}
