/*
Copyright (C) 2014 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2014 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: Manage QuickWayToFolder Launch bar
//-----------------------------------------------------------------------------

#include "LaunchBarOptions.h"


CLaunchBarOptions::CLaunchBarOptions(void)
{
    this->BackgroundType = CTransparentToolbar::BACKGROUND_LINEAR_GRADIENT_MODE_VERTICAL; 
    this->BackGroundColor = CLaunchBar_BACKGROUND_COLOR;
    this->BackGroundGradiantColor1 = CLaunchBar_BACKGROUND_GRADIANT_COLOR1;
    this->BackGroundGradiantColor2 = CLaunchBar_BACKGROUND_GRADIANT_COLOR2;
    *this->BackGroundImage = 0;
    this->bUseTransparentColor = FALSE;
    this->TransparentColor = CLaunchBar_TRANSPARENT_COLOR;
    this->OpacityPercent = CLaunchBar_OPACITY;
    this->IconWidth = CLaunchBar_ICONSIZE;
    this->IconHeight = CLaunchBar_ICONSIZE;
    this->Margins.top=2;
    this->Margins.bottom=2;
    this->Margins.left=2;
    this->Margins.right=2;
    this->MinWidth = 10*this->IconWidth;
    this->MinHeight = this->IconHeight;
    this->bUseFullScreenWidhtOrHeight = TRUE;

    this->ScreenIndex = 0;
    this->Position = POSITION_TOP_CENTER; 

    this->bVertical = FALSE;

    this->ActiveZoneSizeInPixel = CLaunchBar_ACTIVEZONESIZEINPIXEL; // width for vertical, height for horizontal
}


CLaunchBarOptions::~CLaunchBarOptions(void)
{
}

void CLaunchBarOptions::Clone(IN OUT CLaunchBarOptions* pLaunchBarOptions)
{
    pLaunchBarOptions->BackgroundType = this->BackgroundType; 
    pLaunchBarOptions->BackGroundColor = this->BackGroundColor;
    pLaunchBarOptions->BackGroundGradiantColor1 = this->BackGroundGradiantColor1;
    pLaunchBarOptions->BackGroundGradiantColor2 = this->BackGroundGradiantColor2;
    _tcscpy(pLaunchBarOptions->BackGroundImage,this->BackGroundImage);
    pLaunchBarOptions->bUseTransparentColor        =this->bUseTransparentColor       ;  
    pLaunchBarOptions->TransparentColor            =this->TransparentColor           ;
    pLaunchBarOptions->OpacityPercent              =this->OpacityPercent             ;
    pLaunchBarOptions->IconWidth                   =this->IconWidth                  ;
    pLaunchBarOptions->IconHeight                  =this->IconHeight                 ;
    pLaunchBarOptions->Margins                     =this->Margins                    ;
    pLaunchBarOptions->MinWidth                    =this->MinWidth                   ;
    pLaunchBarOptions->MinHeight                   =this->MinHeight                  ;
    pLaunchBarOptions->bUseFullScreenWidhtOrHeight =this->bUseFullScreenWidhtOrHeight;
    pLaunchBarOptions->ScreenIndex                 =this->ScreenIndex                ;
    pLaunchBarOptions->Position                    =this->Position                   ;
    pLaunchBarOptions->bVertical                   =this->bVertical                  ;
    pLaunchBarOptions->ActiveZoneSizeInPixel       =this->ActiveZoneSizeInPixel      ;
}

BOOL CLaunchBarOptions::Save(CIniFile* pIniFile)
{
    // set ini section name
    pIniFile->SetCurrentSectionName(CLaunchBar_SECTION_NAME);

    pIniFile->Set(CLaunchBar_KEY_BACKGROUNDTYPE,(DWORD)this->BackgroundType);
    pIniFile->Set(CLaunchBar_KEY_BACKGROUNDCOLOR,this->BackGroundColor);
    pIniFile->Set(CLaunchBar_KEY_BACKGROUNDCOLOR1,this->BackGroundGradiantColor1);
    pIniFile->Set(CLaunchBar_KEY_BACKGROUNDCOLOR2,this->BackGroundGradiantColor2);
    pIniFile->Set(CLaunchBar_KEY_BACKGROUNDIMAGE,this->BackGroundImage);
    pIniFile->Set(CLaunchBar_KEY_TRANSPARENTCOLOR,this->TransparentColor);
    pIniFile->Set(CLaunchBar_KEY_USETRANSPARENTCOLOR,(DWORD)this->bUseTransparentColor);
    pIniFile->Set(CLaunchBar_KEY_OPACITYPERCENT,(DWORD)this->OpacityPercent);
    pIniFile->Set(CLaunchBar_KEY_ICONSIZE,this->IconWidth);
    pIniFile->Set(CLaunchBar_KEY_MARGINTOP,(DWORD)this->Margins.top);
    pIniFile->Set(CLaunchBar_KEY_MARGINBOTTOM,(DWORD)this->Margins.bottom);
    pIniFile->Set(CLaunchBar_KEY_MARGINLEFT,(DWORD)this->Margins.left);
    pIniFile->Set(CLaunchBar_KEY_MARGINRIGHT,(DWORD)this->Margins.right);
    pIniFile->Set(CLaunchBar_KEY_MINWIDTH,this->MinWidth);
    pIniFile->Set(CLaunchBar_KEY_MINHEIGHT,this->MinHeight);
    pIniFile->Set(CLaunchBar_KEY_VERTICALDIRECTION,(DWORD)this->bVertical);
    pIniFile->Set(CLaunchBar_KEY_USEFULLSCREENWIDTHORHEIGHT,(DWORD)this->bUseFullScreenWidhtOrHeight);
    pIniFile->Set(CLaunchBar_KEY_SCREENINDEX,this->ScreenIndex);
    pIniFile->Set(CLaunchBar_KEY_POSITION,(DWORD)this->Position);
    pIniFile->Set(CLaunchBar_KEY_SIZE,this->ActiveZoneSizeInPixel);
    
    return TRUE;
}
BOOL CLaunchBarOptions::Load(CIniFile* pIniFile)
{
    DWORD dwTmp;

    // set ini section name
    pIniFile->SetCurrentSectionName(CLaunchBar_SECTION_NAME);

    pIniFile->Get(CLaunchBar_KEY_BACKGROUNDTYPE,CTransparentToolbar::BACKGROUND_LINEAR_GRADIENT_MODE_VERTICAL,&dwTmp);
    this->BackgroundType = (CTransparentToolbar::tagBackgroundType)dwTmp;
    pIniFile->Get(CLaunchBar_KEY_BACKGROUNDCOLOR,CLaunchBar_BACKGROUND_COLOR,&this->BackGroundColor);
    pIniFile->Get(CLaunchBar_KEY_BACKGROUNDCOLOR1,CLaunchBar_BACKGROUND_GRADIANT_COLOR1,&this->BackGroundGradiantColor1);
    pIniFile->Get(CLaunchBar_KEY_BACKGROUNDCOLOR2,CLaunchBar_BACKGROUND_GRADIANT_COLOR2,&this->BackGroundGradiantColor2);
    pIniFile->Get(CLaunchBar_KEY_BACKGROUNDIMAGE,_T(""),this->BackGroundImage,MAX_PATH);
    pIniFile->Get(CLaunchBar_KEY_TRANSPARENTCOLOR,CLaunchBar_TRANSPARENT_COLOR,&this->TransparentColor);
    pIniFile->Get(CLaunchBar_KEY_USETRANSPARENTCOLOR,FALSE,(DWORD*)&this->bUseTransparentColor);
    pIniFile->Get(CLaunchBar_KEY_OPACITYPERCENT,CLaunchBar_OPACITY,&dwTmp);
    this->OpacityPercent = (BYTE)dwTmp; 
    pIniFile->Get(CLaunchBar_KEY_ICONSIZE,CLaunchBar_ICONSIZE,&this->IconWidth);
    this->IconHeight = this->IconWidth;
    pIniFile->Get(CLaunchBar_KEY_MARGINTOP,2,(DWORD*)&this->Margins.top);
    pIniFile->Get(CLaunchBar_KEY_MARGINBOTTOM,2,(DWORD*)&this->Margins.bottom);
    pIniFile->Get(CLaunchBar_KEY_MARGINLEFT,2,(DWORD*)&this->Margins.left);
    pIniFile->Get(CLaunchBar_KEY_MARGINRIGHT,2,(DWORD*)&this->Margins.right);
    pIniFile->Get(CLaunchBar_KEY_MINWIDTH,10*this->IconWidth,&this->MinWidth);
    pIniFile->Get(CLaunchBar_KEY_MINHEIGHT,this->IconHeight,&this->MinHeight);
    pIniFile->Get(CLaunchBar_KEY_VERTICALDIRECTION,FALSE,(DWORD*)&this->bVertical);
    pIniFile->Get(CLaunchBar_KEY_USEFULLSCREENWIDTHORHEIGHT,TRUE,(DWORD*)&this->bUseFullScreenWidhtOrHeight);
    pIniFile->Get(CLaunchBar_KEY_SCREENINDEX,0,&this->ScreenIndex);
    pIniFile->Get(CLaunchBar_KEY_POSITION,POSITION_TOP_CENTER,&dwTmp);
    this->Position = (tagPosition)dwTmp;
    pIniFile->Get(CLaunchBar_KEY_SIZE,CLaunchBar_ACTIVEZONESIZEINPIXEL,&this->ActiveZoneSizeInPixel);

    return TRUE;
}