/*
Copyright (C) 2014 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2014 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: Manage QuickWayToFolder Launch bar options
//-----------------------------------------------------------------------------

#pragma once

#include <windows.h>

#include "../Tools/Ini/IniFile.h"
#include "../Tools/GUI/TransparentToolbar/TransparentToolbar.h"

#define CLaunchBar_TRANSPARENT_COLOR (RGB(0xFF,0,0xFF))
#define CLaunchBar_BACKGROUND_COLOR (RGB(0xFF,0xFF,0xFF))
#define CLaunchBar_BACKGROUND_GRADIANT_COLOR1 (RGB(0xFF,0xFF,0xFF))
#define CLaunchBar_BACKGROUND_GRADIANT_COLOR2 (RGB(0,0,0))
#define CLaunchBar_OPACITY 90
#define CLaunchBar_ICONSIZE 24
#define CLaunchBar_ACTIVEZONESIZEINPIXEL 8

#define CLaunchBar_SECTION_NAME               _T("LaunchBar")
#define CLaunchBar_KEY_TRANSPARENTCOLOR       _T("TransparentColor")
#define CLaunchBar_KEY_USETRANSPARENTCOLOR    _T("UseTransparentColor")
#define CLaunchBar_KEY_BACKGROUNDTYPE         _T("BackgroundType")
#define CLaunchBar_KEY_BACKGROUNDCOLOR        _T("BackgroundColor")
#define CLaunchBar_KEY_BACKGROUNDCOLOR1       _T("BackgroundGradiantColor1")
#define CLaunchBar_KEY_BACKGROUNDCOLOR2       _T("BackgroundGradiantColor2")
#define CLaunchBar_KEY_BACKGROUNDIMAGE        _T("BackgroundImage")
#define CLaunchBar_KEY_OPACITYPERCENT         _T("Opacity")
#define CLaunchBar_KEY_ICONSIZE               _T("IconSize")
#define CLaunchBar_KEY_MARGINTOP              _T("MarginTop")
#define CLaunchBar_KEY_MARGINBOTTOM           _T("MarginBottom")
#define CLaunchBar_KEY_MARGINLEFT             _T("MarginLeft")
#define CLaunchBar_KEY_MARGINRIGHT            _T("MarginRight")
#define CLaunchBar_KEY_MINWIDTH               _T("MinWidth")
#define CLaunchBar_KEY_MINHEIGHT              _T("MinHeight")
#define CLaunchBar_KEY_VERTICALDIRECTION      _T("VerticalDirection")
#define CLaunchBar_KEY_USEFULLSCREENWIDTHORHEIGHT _T("UseFullScreenWidth")
#define CLaunchBar_KEY_SCREENINDEX            _T("ScreenIndex")
#define CLaunchBar_KEY_POSITION      _T("Position")
#define CLaunchBar_KEY_SIZE          _T("HotCornerActiveWitdh")

class CLaunchBarOptions
{
public:
    CLaunchBarOptions(void);
    ~CLaunchBarOptions(void);

    enum tagPosition
    {
        POSITION_TOP_LEFT=0,
        POSITION_TOP_CENTER, // horizontal only
        POSITION_TOP_RIGHT,
        POSITION_LEFT_MIDDLE,// vertical only
        POSITION_RIGHT_MIDDLE,// vertical only
        POSITION_BOTTOM_LEFT,
        POSITION_BOTTOM_CENTER,// horizontal only
        POSITION_BOTTOM_RIGHT
    };

    CTransparentToolbar::tagBackgroundType BackgroundType; 
    COLORREF BackGroundColor;
    COLORREF BackGroundGradiantColor1;
    COLORREF BackGroundGradiantColor2;
    TCHAR BackGroundImage[MAX_PATH];
    BOOL bUseTransparentColor;
    COLORREF TransparentColor;
    BYTE OpacityPercent;
    DWORD IconWidth;
    DWORD IconHeight;
    RECT Margins;
    DWORD MinWidth;
    DWORD MinHeight;
    BOOL bUseFullScreenWidhtOrHeight;

    DWORD ScreenIndex;
    tagPosition Position; 
    
    BOOL bVertical;
    DWORD ActiveZoneSizeInPixel; // width for vertical, height for horizontal

    virtual BOOL Save(CIniFile* pIniFile);
    virtual BOOL Load(CIniFile* pIniFile);
    void Clone(IN OUT CLaunchBarOptions* pLaunchBarOptions);
};


