/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manages the options dialog
//-----------------------------------------------------------------------------
#include "OptionsUI.h"
#include "QuickWayToFolders.h"
#include "../Tools/Version/OsVersion.h"
#include "../Tools/GUI/AutoComplete/AutoComplete.h"
#include "../Tools/gui/DesktopWorkArea/DesktopWorkArea.h"
#include <vector>
#include <algorithm> // std::sort

#ifndef _countof
#define _countof(array) (sizeof(array)/sizeof(array[0]))
#endif

#ifndef MAPVK_VK_TO_VSC
#define MAPVK_VK_TO_VSC 0
#endif

COptionsUI::CSIDL_ArrayItem COptionsUI::CSIDL_Array[]=
{
    {CSIDL_ADMINTOOLS                    ,TEXT("CSIDL_ADMINTOOLS")                    },
    {CSIDL_ALTSTARTUP                    ,TEXT("CSIDL_ALTSTARTUP")                    }, 
    {CSIDL_APPDATA                       ,TEXT("CSIDL_APPDATA")                       },
    {CSIDL_BITBUCKET                     ,TEXT("CSIDL_BITBUCKET")                     },
    {CSIDL_CDBURN_AREA                   ,TEXT("CSIDL_CDBURN_AREA")                   },
    {CSIDL_COMMON_ADMINTOOLS             ,TEXT("CSIDL_COMMON_ADMINTOOLS")             },
    {CSIDL_COMMON_ALTSTARTUP             ,TEXT("CSIDL_COMMON_ALTSTARTUP")             }, 
    {CSIDL_COMMON_APPDATA                ,TEXT("CSIDL_COMMON_APPDATA")                },
    {CSIDL_COMMON_DESKTOPDIRECTORY       ,TEXT("CSIDL_COMMON_DESKTOPDIRECTORY")       },
    {CSIDL_COMMON_DOCUMENTS              ,TEXT("CSIDL_COMMON_DOCUMENTS")              }, 
    {CSIDL_COMMON_FAVORITES              ,TEXT("CSIDL_COMMON_FAVORITES")              }, 
    {CSIDL_COMMON_MUSIC                  ,TEXT("CSIDL_COMMON_MUSIC")                  },
    {CSIDL_COMMON_OEM_LINKS              ,TEXT("CSIDL_COMMON_OEM_LINKS")              },
    {CSIDL_COMMON_PICTURES               ,TEXT("CSIDL_COMMON_PICTURES")               },
    {CSIDL_COMMON_PROGRAMS               ,TEXT("CSIDL_COMMON_PROGRAMS")               },
    {CSIDL_COMMON_STARTMENU              ,TEXT("CSIDL_COMMON_STARTMENU")              },
    {CSIDL_COMMON_STARTUP                ,TEXT("CSIDL_COMMON_STARTUP")                },
    {CSIDL_COMMON_TEMPLATES              ,TEXT("CSIDL_COMMON_TEMPLATES")              },
    {CSIDL_COMMON_VIDEO                  ,TEXT("CSIDL_COMMON_VIDEO")                  },
    {CSIDL_COMPUTERSNEARME               ,TEXT("CSIDL_COMPUTERSNEARME")               },
    {CSIDL_CONNECTIONS                   ,TEXT("CSIDL_CONNECTIONS")                   },
    {CSIDL_CONTROLS                      ,TEXT("CSIDL_CONTROLS")                      },
    {CSIDL_COOKIES                       ,TEXT("CSIDL_COOKIES")                       },
    {CSIDL_DESKTOP                       ,TEXT("CSIDL_DESKTOP")                       },
    {CSIDL_DESKTOPDIRECTORY              ,TEXT("CSIDL_DESKTOPDIRECTORY")              }, 
    {CSIDL_DRIVES                        ,TEXT("CSIDL_DRIVES")                        },
    {CSIDL_FAVORITES                     ,TEXT("CSIDL_FAVORITES")                     },
    {CSIDL_FONTS                         ,TEXT("CSIDL_FONTS")                         },
    {CSIDL_HISTORY                       ,TEXT("CSIDL_HISTORY")                       },
    {CSIDL_INTERNET                      ,TEXT("CSIDL_INTERNET")                      },
    {CSIDL_INTERNET_CACHE                ,TEXT("CSIDL_INTERNET_CACHE")                },
    {CSIDL_LOCAL_APPDATA                 ,TEXT("CSIDL_LOCAL_APPDATA")                 },
    {CSIDL_MYDOCUMENTS                   ,TEXT("CSIDL_MYDOCUMENTS")                   },
    {CSIDL_MYMUSIC                       ,TEXT("CSIDL_MYMUSIC")                       },
    {CSIDL_MYPICTURES                    ,TEXT("CSIDL_MYPICTURES")                    },
    {CSIDL_MYVIDEO                       ,TEXT("CSIDL_MYVIDEO")                       },
    {CSIDL_NETHOOD                       ,TEXT("CSIDL_NETHOOD")                       },
    {CSIDL_NETWORK                       ,TEXT("CSIDL_NETWORK")                       },
    // {CSIDL_PERSONAL                      ,TEXT("CSIDL_PERSONAL")                      }, // same as CSIDL_MYDOCUMENTS // #define CSIDL_MYDOCUMENTS CSIDL_PERSONAL //  Personal was just a silly name for My Documents
    {CSIDL_PRINTERS                      ,TEXT("CSIDL_PRINTERS")                      },
    {CSIDL_PRINTHOOD                     ,TEXT("CSIDL_PRINTHOOD")                     },
    {CSIDL_PROFILE                       ,TEXT("CSIDL_PROFILE")                       },
    {CSIDL_PROGRAM_FILES                 ,TEXT("CSIDL_PROGRAM_FILES")                 },
    {CSIDL_PROGRAM_FILESX86              ,TEXT("CSIDL_PROGRAM_FILESX86")              },
    {CSIDL_PROGRAM_FILES_COMMON          ,TEXT("CSIDL_PROGRAM_FILES_COMMON")          },
    {CSIDL_PROGRAM_FILES_COMMONX86       ,TEXT("CSIDL_PROGRAM_FILES_COMMONX86")       },
    {CSIDL_PROGRAMS                      ,TEXT("CSIDL_PROGRAMS")                      },
    {CSIDL_RECENT                        ,TEXT("CSIDL_RECENT")                        },
    {CSIDL_RESOURCES                     ,TEXT("CSIDL_RESOURCES")                     },
    {CSIDL_RESOURCES_LOCALIZED           ,TEXT("CSIDL_RESOURCES_LOCALIZED")           },
    {CSIDL_STARTMENU                     ,TEXT("CSIDL_STARTMENU")                     },
    {CSIDL_STARTUP                       ,TEXT("CSIDL_STARTUP")                       },
    {CSIDL_SYSTEM                        ,TEXT("CSIDL_SYSTEM")                        },
    {CSIDL_SYSTEMX86                     ,TEXT("CSIDL_SYSTEMX86")                     },
    {CSIDL_TEMPLATES                     ,TEXT("CSIDL_TEMPLATES")                     },
    {CSIDL_WINDOWS                       ,TEXT("CSIDL_WINDOWS")                       },
};


COptionsUI::COptionsUI(void)
{
    this->DialogResult=0;
    this->hWndDialog=NULL;
    this->hInstance=NULL;
    this->hParentHandle=NULL;
    this->pTreeview=NULL;
    this->pToolbar=NULL;
    this->pListview=NULL;
    this->pQuickWayToFolders=NULL;
    this->pLanguage=NULL;
	this->hDisplayedBmp=NULL;
    this->hBitmapPreview=NULL;
    this->gdiplusToken=NULL;

    this->bGdiPlusStarted=FALSE;
    *this->AssociatedIconPath=0;
    this->AssociatedIconIndex=0;
    *this->LaunchBarAssociatedIconHotPath=0;
    this->LaunchBarAssociatedIconHotIndex=0;
    *this->LaunchBarAssociatedIconDownPath=0;
    this->LaunchBarAssociatedIconDownIndex=0;

    this->hWndTextColor=NULL;
    this->hWndTextBackgroundColor=NULL;
    this->hWndBackgroundColor=NULL;
    this->hWndBackgroundColor1=NULL;
    this->hWndBackgroundColor2=NULL;

    this->TextColorBrush=NULL;
    this->TextBackgroundColorBrush=NULL;
    this->BackgroundColorBrush=NULL;
    this->BackgroundColor1Brush=NULL;
    this->BackgroundColor2Brush=NULL;

    this->hWndLaunchBarBackgroundColor=NULL;
    this->hWndLaunchBarGradiantBackgroundColor1=NULL;
    this->hWndLaunchBarGradiantBackgroundColor2=NULL;
    this->hWndLaunchBarTransparentColor=NULL;

    this->LaunchBarBackgroundColorBrush=NULL;
    this->LaunchBarGradiantBackgroundColor1Brush=NULL;
    this->LaunchBarGradiantBackgroundColor2Brush=NULL;
    this->LaunchBarTransparentBrush=NULL;

    this->pBitmapDesktop=NULL;
    this->pBitmapBrowser=NULL;
    this->pBitmapMenu=NULL;
    this->pBitmapMultipleMenu=NULL;
    this->pBitmapPreview=NULL;

    this->pCurrentFolderOptions=NULL;
    this->hCurrentFolderOptionsTreeItem=NULL;
    this->bLoadingFolderOptions=FALSE;



    this->TreeViewIconIndexBrowserOptions=NULL;
    this->TreeViewIconIndexCustomize=NULL;
    this->TreeViewIconIndexFilters=NULL;

    this->hItemFolders=NULL;
    this->hItemApplicationOptions=NULL;
    this->hItemLaunchBar=NULL;

    this->HotKeyfsModifiers=0;
    this->HotKeyVirtualKey=0;

    this->hKeyboardHook=0;

    this->pDragAndDrop = NULL;

}

COptionsUI::~COptionsUI(void)
{
    this->DeleteFolderBrushs();
    this->DeleteLaunchBarBrushs();

    for (size_t Cnt=0;Cnt<this->MultipleMenuListviewLoadedIconsToBeDestroyed.size();Cnt++)
        ::DestroyIcon(this->MultipleMenuListviewLoadedIconsToBeDestroyed[Cnt]);
    this->MultipleMenuListviewLoadedIconsToBeDestroyed.clear();


    for (size_t Cnt=0;Cnt<this->LoadedIconsToBeDestroyed.size();Cnt++)
        ::DestroyIcon(this->LoadedIconsToBeDestroyed[Cnt]);
    this->LoadedIconsToBeDestroyed.clear();

}

void COptionsUI::DeleteLaunchBarBrushs()
{
    if (this->LaunchBarBackgroundColorBrush)
    {
        ::DeleteObject(this->LaunchBarBackgroundColorBrush);
    }
    if (this->LaunchBarGradiantBackgroundColor1Brush)
    {
        ::DeleteObject(this->LaunchBarGradiantBackgroundColor1Brush);
    }
    if (this->LaunchBarGradiantBackgroundColor2Brush)
    {
        ::DeleteObject(this->LaunchBarGradiantBackgroundColor2Brush);
    }
    if (this->LaunchBarTransparentBrush)
    {
        ::DeleteObject(this->LaunchBarTransparentBrush);
    }
}
void COptionsUI::DeleteFolderBrushs()
{
    if (this->TextColorBrush)
    {
        ::DeleteObject(this->TextColorBrush);
    }
    if (this->TextBackgroundColorBrush)
    {
        ::DeleteObject(this->TextBackgroundColorBrush);
    }
    if (this->BackgroundColorBrush)
    {
        ::DeleteObject(this->BackgroundColorBrush);
    }
    if (this->BackgroundColor1Brush)
    {
        ::DeleteObject(this->BackgroundColor1Brush);
    }
    if (this->BackgroundColor2Brush)
    {
        ::DeleteObject(this->BackgroundColor2Brush);
    }
}

void COptionsUI::OnFileDropped(HWND hWndControl, TCHAR* DroppedFile, SIZE_T FileDroppedIndex, LPVOID UserParam)
{
    ((COptionsUI*)UserParam)->OnFileDropped(hWndControl,DroppedFile,FileDroppedIndex);
}
void COptionsUI::OnFileDropped(HWND hWndControl, TCHAR* DroppedFile, SIZE_T FileDroppedIndex)
{
    UNREFERENCED_PARAMETER(FileDroppedIndex);
    if ( (hWndControl == ::GetDlgItem(this->hWndDialog,IDC_EDIT_PATH_LOCAL_FOLDER_PATH) )
         || (hWndControl == ::GetDlgItem(this->hWndDialog,IDC_EDIT_CUSTOMIZATION_BACKGROUND_PATH))
       )
    {
        ::SetWindowText(hWndControl,DroppedFile);
    }
}

//-----------------------------------------------------------------------------
// Name: Show
// Object: show the filter dialog box
// Parameters :
//     in  : HINSTANCE hInstance : application instance
//           HWND hWndDialog : main window dialog handle
//     out :
//     return : 
//-----------------------------------------------------------------------------
COptionsUI* pCurrentOptionsUI = NULL;
INT_PTR COptionsUI::Show(HINSTANCE hInstance,HWND hWndDialog,CQuickWayToFolders* pQuickWayToFolders)
{
    if (pCurrentOptionsUI)
    {
        ::ShowWindow(pCurrentOptionsUI->hWndDialog,SW_SHOW);
        ::BringWindowToTop(pCurrentOptionsUI->hWndDialog);
        ::SetForegroundWindow(pCurrentOptionsUI->hWndDialog);
        return IDCANCEL;
    }

    INT_PTR Ret;
    COptionsUI* pOptionsUI=new COptionsUI();
    pCurrentOptionsUI = pOptionsUI;
    pOptionsUI->hInstance=hInstance;
    pOptionsUI->hParentHandle=hWndDialog;
    pOptionsUI->pQuickWayToFolders=pQuickWayToFolders;

    // for each folder hide item (in case of top most browser)
    // and save content
    for (size_t Cnt=0;Cnt<pQuickWayToFolders->FoldersArray.size();Cnt++)
    {
        if (!pQuickWayToFolders->FoldersArray[Cnt]->pContainer)
            continue;
        
        pQuickWayToFolders->FoldersArray[Cnt]->pContainer->UpdateOptionsBeforeClose();
        pQuickWayToFolders->FoldersArray[Cnt]->pContainer->Hide();
    }

    // make a local copy of folder options, language, application options
    // (dirty object copy)
    pOptionsUI->pApplicationOptions=new CApplicationOptions();
    pQuickWayToFolders->pApplicationOptions->Clone(pOptionsUI->pApplicationOptions);
    
    pOptionsUI->pLanguage=new CLanguage();
    pQuickWayToFolders->pLanguage->Clone(pOptionsUI->pLanguage);
    
    CGenericQuickWayToFolderObject* pGenericQuickWayToFolderObject;
    size_t NbItems=pQuickWayToFolders->FoldersArray.size();
    for (size_t Cnt=0;Cnt<NbItems;Cnt++)
    {
        pGenericQuickWayToFolderObject=new CGenericQuickWayToFolderObject();

        if (pQuickWayToFolders->pApplicationOptions->LaunchMode == CApplicationOptions::LaunchMode_NOTIFY_ICON)
        {
            // insert in reverse order as notify icon are inserted from right to left
            pQuickWayToFolders->FoldersArray[NbItems-Cnt-1]->Clone(pGenericQuickWayToFolderObject);
        }
        else
            pQuickWayToFolders->FoldersArray[Cnt]->Clone(pGenericQuickWayToFolderObject);
        
        pOptionsUI->FoldersArray.push_back(pGenericQuickWayToFolderObject);
    }

    Ret=::DialogBoxParam(pOptionsUI->hInstance,(LPCTSTR)IDD_DIALOG_OPTIONS,NULL,(DLGPROC)COptionsUI::WndProc,(LPARAM)pOptionsUI);
    if (Ret==DialogResult_OK)
    {
        // delete old object assuming they are still filled (avoid to delete content before filling them again)
        pOptionsUI->pApplicationOptions->Clone(pQuickWayToFolders->pApplicationOptions);
        delete pOptionsUI->pApplicationOptions;

        pOptionsUI->pLanguage->Clone(pQuickWayToFolders->pLanguage);
        delete pOptionsUI->pLanguage;
        
        pQuickWayToFolders->DestroyAssociatedQuickWayToFoldersItems();

        CGenericQuickWayToFolderObject* pGenericQuickWayToFolderObject;
        NbItems=pOptionsUI->FoldersArray.size();
        for (size_t Cnt=0;Cnt<NbItems;Cnt++)
        {
            if (pQuickWayToFolders->pApplicationOptions->LaunchMode == CApplicationOptions::LaunchMode_NOTIFY_ICON)
            {
                // warning local folder are in reverse order, put them back to original order
                pGenericQuickWayToFolderObject=pOptionsUI->FoldersArray[NbItems-Cnt-1];
            }
            else
                pGenericQuickWayToFolderObject=pOptionsUI->FoldersArray[Cnt];
            
            pGenericQuickWayToFolderObject->pContainer=NULL;// container has been destroyed by pQuickWayToFolders->DestroyAssociatedQuickWayToFoldersItems();
            pQuickWayToFolders->FoldersArray.push_back(pGenericQuickWayToFolderObject);
        }

        pQuickWayToFolders->SaveApplicationOptions();
        pQuickWayToFolders->SaveFoldersOptions();
        // pLanguage must be changed before calling CreateAssociatedQuickWayToFoldersItems
        // if no creation success, reshow options
        if(!pQuickWayToFolders->CreateAssociatedQuickWayToFoldersItems())
            ::PostMessage(pQuickWayToFolders->hWndDialog,CQuickWayToFolders_SHOW_OPTIONS,0,0);
    }
    else
    {
        // delete object created before dialog display
        delete pOptionsUI->pLanguage;
        delete pOptionsUI->pApplicationOptions;
        for (size_t Cnt=0;Cnt<pOptionsUI->FoldersArray.size();Cnt++)
        {
            // assume container won't be destroy (see CGenericQuickWayToFolderObject destructor)
            pOptionsUI->FoldersArray[Cnt]->pContainer=NULL;
            delete pOptionsUI->FoldersArray[Cnt];
        }

    }

    pCurrentOptionsUI = NULL;
    delete pOptionsUI;
    return Ret;
}

void COptionsUI::OnInit()
{
    POINT Pt;
    RECT Rect;
    LONG Width;
    LONG Height;
    LONG x;
    LONG y;
    LONG ExternalRectWidthDelta;
    LONG ExternalRectHeightDelta;
    RECT ClientRect;

    this->pDragAndDrop = new CDragAndDrop(this->hWndDialog,COptionsUI::OnFileDropped,this);
    this->pDragAndDrop->Start();

    CAutoComplete::AddAutoCompleteToEditControl(::GetDlgItem(this->hWndDialog,IDC_EDIT_PATH_LOCAL_FOLDER_PATH));
    CAutoComplete::AddAutoCompleteToEditControl(::GetDlgItem(this->hWndDialog,IDC_EDIT_CUSTOMIZATION_BACKGROUND_PATH));
    ::SendMessage(::GetDlgItem(this->hWndDialog,IDC_SLIDER_LAUNCHBAR_TRANSPARENCY),TBM_SETRANGE,TRUE,MAKELONG(0,100));

    // load current language for dialog
    this->LoadLanguage();

    // compute internal rect / external rect delta
    ::GetClientRect(this->hWndDialog,&ClientRect);
    ::GetWindowRect(this->hWndDialog,&Rect);
    ExternalRectWidthDelta=(Rect.right-Rect.left)-(ClientRect.right-ClientRect.left);
    ExternalRectHeightDelta=(Rect.bottom-Rect.top)-(ClientRect.bottom-ClientRect.top);

    // get max component bounds
    CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FOLDER_TYPE),&Rect);
    Width = Rect.right+COptionsUI_SPACE_BETWEEN_CONTROLS;
    CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_TREE_FOLDERS),&Rect);
    Height = Rect.bottom+COptionsUI_SPACE_BETWEEN_CONTROLS;

    // get desktop size
    CDesktopWorkArea::GetWorkArea(&Rect,TRUE);

    // resize and center window
    x=(Rect.right-Rect.left-Width)/2;
    y=(Rect.bottom-Rect.top-Height)/2;
    ::SetWindowPos(this->hWndDialog,NULL,x,y,Width+ExternalRectWidthDelta,Height+ExternalRectHeightDelta,SWP_NOZORDER);

    // set groups visibility
    CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_APPLICATION_OPTIONS),TRUE);
    CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),FALSE);
    CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),FALSE);
    CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_BROWSE),FALSE);
    CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_CUSTOMIZATION),FALSE);
    CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FILTERS),FALSE);
    CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FOLDER_TYPE),FALSE);
    CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),FALSE);
    CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_MULTIPLE_FOLDERS),FALSE);
    CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),FALSE);// as IDC_STATIC_GROUP_MULTIPLE_FOLDERS contains sub groups, we must do this, as IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER is the first dialog item after sub groups
    CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR),FALSE);

    // set group positions
    CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FOLDER_TYPE),&Rect);
    Pt.x=Rect.left;
    Pt.y=Rect.top;
    CDialogHelper::MoveGroupTo(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_APPLICATION_OPTIONS),&Pt);
    CDialogHelper::MoveGroupTo(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FILTERS),&Pt);
    CDialogHelper::MoveGroupTo(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_CUSTOMIZATION),&Pt);
    CDialogHelper::MoveGroupTo(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_BROWSE),&Pt);
    CDialogHelper::MoveGroupTo(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR),&Pt);

    // startup gdi
    Gdiplus::GdiplusStartupInput gdiplusStartupInput;
    this->gdiplusToken=NULL;
    this->bGdiPlusStarted=(Gdiplus::GdiplusStartup(&this->gdiplusToken, &gdiplusStartupInput, NULL)==Gdiplus::Ok);

    // load folder types thumbs
    if (this->bGdiPlusStarted)
    {
        this->pBitmapDesktop=new CGdiPlusBitmapResource();
        this->pBitmapDesktop->Load(IDR_JPG_DESKTOP, _T("JPG"),this->hInstance);
        this->pBitmapBrowser=new CGdiPlusBitmapResource();
        this->pBitmapBrowser->Load(IDR_JPG_BROWSER, _T("JPG"),this->hInstance);
        this->pBitmapMenu=new CGdiPlusBitmapResource();
        this->pBitmapMenu->Load(IDR_JPG_MENU, _T("JPG"),this->hInstance);
        this->pBitmapMultipleMenu=new CGdiPlusBitmapResource();
        this->pBitmapMultipleMenu->Load(IDR_JPG_MULTIPLE_MENU, _T("JPG"),this->hInstance);
    }

    // create toolbar object
    this->pToolbar=new CToolbar(this->hInstance,this->hWndDialog,TRUE,TRUE,24,24,0);
    this->pToolbar->AddButton(IDC_TOOLBAR_BUTTON_ADD_QUICKWAY_TO_FOLDER,(TCHAR*)this->pLanguage->Add,IDI_ICON_ADD,(TCHAR*)this->pLanguage->Add,24,24);
    this->pToolbar->AddButton(IDC_TOOLBAR_BUTTON_REMOVE_QUICKWAY_TO_FOLDER,(TCHAR*)this->pLanguage->Remove,IDI_ICON_REMOVE,(TCHAR*)this->pLanguage->Remove,24,24);
    this->pToolbar->AddButton(IDC_TOOLBAR_BUTTON_MOVE_QUICKWAY_TO_FOLDER_UP,(TCHAR*)this->pLanguage->MoveUp,IDI_ICON_UP_MOVE,(TCHAR*)this->pLanguage->MoveUp,24,24);
    this->pToolbar->AddButton(IDC_TOOLBAR_BUTTON_MOVE_QUICKWAY_TO_FOLDER_DOWN,(TCHAR*)this->pLanguage->MoveDown,IDI_ICON_DOWN_MOVE,(TCHAR*)this->pLanguage->MoveDown,24,24);
    // assume toolbar as a group style to avoid it to be hidden / shown according to previous group changes
    LONG_PTR Style=::GetWindowLongPtr(this->pToolbar->GetControlHandle(),GWL_STYLE);
    ::SetWindowLongPtr(this->pToolbar->GetControlHandle(),GWL_STYLE,Style | WS_GROUP );
    
    
    // create treeview object associated to treeview component
    this->pTreeview=new CTreeview(::GetDlgItem(this->hWndDialog,IDC_TREE_FOLDERS),FALSE);
    this->pTreeview->EnablePopUpMenu(FALSE);
    this->pTreeview->SetEventsHandler((ITreeviewEvents*)(CTreeviewEventsBase*)this);
    this->FillTreeView();

    // create listview object associated to listview component
    this->pListview=new CListview(::GetDlgItem(this->hWndDialog,IDC_LIST_MULTIPLE_FOLDERS));
    this->pListview->SetStyle(TRUE,FALSE,FALSE,FALSE);
    this->pListview->EnableColumnSorting(FALSE);
    this->pListview->EnablePopUpMenu(FALSE);
    this->pListview->AddColumn(pLanguage->Folders,370,LVCFMT_LEFT);
    this->pListview->AddColumn(pLanguage->Type,110,LVCFMT_LEFT);
    this->pListview->AddColumn(pLanguage->Merged,70,LVCFMT_LEFT);

    // get color buttons handles
    this->hWndTextColor=::GetDlgItem(this->hWndDialog,IDC_BUTTON_CUSTOMIZATION_TEXT_COLOR);
    this->hWndTextBackgroundColor=::GetDlgItem(this->hWndDialog,IDC_BUTTON_CUSTOMIZATION_BACKGROUND_TEXT_COLOR);
    this->hWndBackgroundColor=::GetDlgItem(this->hWndDialog,IDC_BUTTON_CUSTOMIZATION_BACKGROUND_COLOR);
    this->hWndBackgroundColor1=::GetDlgItem(this->hWndDialog,IDC_BUTTON_CUSTOMIZATION_BACKGROUND_COLOR1);
    this->hWndBackgroundColor2=::GetDlgItem(this->hWndDialog,IDC_BUTTON_CUSTOMIZATION_BACKGROUND_COLOR2);

    // set button icons
    CDialogHelper::SetDlgButtonIcon(this->hInstance,this->hWndDialog,IDC_BUTTON_MULTIPLE_FOLDERS_MENU_ADD,IDI_ICON_ADD,16,16);
    CDialogHelper::SetDlgButtonIcon(this->hInstance,this->hWndDialog,IDC_BUTTON_MULTIPLE_FOLDERS_MENU_REMOVE,IDI_ICON_REMOVE,16,16);
    CDialogHelper::SetDlgButtonIcon(this->hInstance,this->hWndDialog,IDC_BUTTON_MULTIPLE_FOLDERS_MENU_UP,IDI_ICON_UP_MOVE,16,16);
    CDialogHelper::SetDlgButtonIcon(this->hInstance,this->hWndDialog,IDC_BUTTON_MULTIPLE_FOLDERS_MENU_DOWN,IDI_ICON_DOWN_MOVE,16,16);
    CDialogHelper::SetDlgButtonIcon(this->hInstance,this->hWndDialog,IDCANCEL,IDI_ICON_CANCEL,16,16);
    CDialogHelper::SetDlgButtonIcon(this->hInstance,this->hWndDialog,IDOK,IDI_ICON_APPLY,16,16);

    // fill Language combo
    this->FillLanguagesCombo();

    // fill combo not depending on languages (other combos are filled in LoadLanguage)
    this->FillCSIDLCombo();
    this->FillLaunchBarIconPosition();
    this->FillLaunchBarMonitorsCombo(IDC_COMBO_LAUNCHBAR_MONITOR_INDEX);
    this->FillLaunchBarMonitorsCombo(IDC_COMBO_HOT_CORNER_MONITOR_INDEX);
    this->FillLaunchBarIconSizesCombo();

    this->LoadApplicationOptions();

    // set keyboard hook for shortcuts
    this->hKeyboardHook  = ::SetWindowsHookEx(WH_KEYBOARD, KeyboardProc, NULL, GetCurrentThreadId());   

    // render layout
    this->OnSize();

    ::BringWindowToTop(this->hWndDialog);
    ::SetForegroundWindow(this->hWndDialog);
}

void COptionsUI::OnTreeviewSelectedItemChanged(CTreeview* pSender,LPNMTREEVIEW pChangeInfos)
{
    UNREFERENCED_PARAMETER(pSender);
    CGenericQuickWayToFolderObject* pOldFolderOptions=this->pCurrentFolderOptions;
    HTREEITEM hOldFolderOptionsTreeItem=this->hCurrentFolderOptionsTreeItem;

    switch(pChangeInfos->itemNew.lParam)
    {
    case TreeViewNodeType_APPLICATION:
        this->pCurrentFolderOptions=NULL;
        this->hCurrentFolderOptionsTreeItem=NULL;
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_APPLICATION_OPTIONS),TRUE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_BROWSE),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_CUSTOMIZATION),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FILTERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FOLDER_TYPE),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_MULTIPLE_FOLDERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),FALSE);// as IDC_STATIC_GROUP_MULTIPLE_FOLDERS contains sub groups, we must do this, as IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER is the first dialog item after sub groups
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR),FALSE);
        break;
    case TreeViewNodeType_FOLDERS:
        this->pCurrentFolderOptions=NULL;
        this->hCurrentFolderOptionsTreeItem=NULL;
        // nothing to show
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_APPLICATION_OPTIONS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_BROWSE),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_CUSTOMIZATION),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FILTERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FOLDER_TYPE),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_MULTIPLE_FOLDERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),FALSE);// as IDC_STATIC_GROUP_MULTIPLE_FOLDERS contains sub groups, we must do this, as IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER is the first dialog item after sub groups
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR),FALSE);
        break;

    case TreeViewNodeType_FOLDER_FILTERS:
        // associated CGenericQuickWayToFolderObject* pSingleFolderOptions is in parent LPARAM
        this->hCurrentFolderOptionsTreeItem=this->pTreeview->GetParent(pChangeInfos->itemNew.hItem);
        this->pTreeview->GetItemUserData(this->hCurrentFolderOptionsTreeItem,(LPVOID*)&this->pCurrentFolderOptions);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_APPLICATION_OPTIONS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_BROWSE),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_CUSTOMIZATION),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FILTERS),TRUE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FOLDER_TYPE),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_MULTIPLE_FOLDERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),FALSE);// as IDC_STATIC_GROUP_MULTIPLE_FOLDERS contains sub groups, we must do this, as IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER is the first dialog item after sub groups
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR),FALSE);
        break;
    case TreeViewNodeType_FOLDER_CUSTOMIZATION:
        // associated CGenericQuickWayToFolderObject* pSingleFolderOptions is in parent LPARAM
        this->hCurrentFolderOptionsTreeItem=this->pTreeview->GetParent(pChangeInfos->itemNew.hItem);
        this->pTreeview->GetItemUserData(this->hCurrentFolderOptionsTreeItem,(LPVOID*)&this->pCurrentFolderOptions);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_APPLICATION_OPTIONS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_BROWSE),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_CUSTOMIZATION),TRUE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FILTERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FOLDER_TYPE),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_MULTIPLE_FOLDERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),FALSE);// as IDC_STATIC_GROUP_MULTIPLE_FOLDERS contains sub groups, we must do this, as IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER is the first dialog item after sub groups
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR),FALSE);
        break;
    case TreeViewNodeType_FOLDER_BROWSING_OPTION:
        this->hCurrentFolderOptionsTreeItem=this->pTreeview->GetParent(pChangeInfos->itemNew.hItem);
        // associated CGenericQuickWayToFolderObject* pSingleFolderOptions is in parent LPARAM
        this->pTreeview->GetItemUserData(this->hCurrentFolderOptionsTreeItem,(LPVOID*)&this->pCurrentFolderOptions);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_APPLICATION_OPTIONS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_BROWSE),TRUE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_CUSTOMIZATION),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FILTERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FOLDER_TYPE),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_MULTIPLE_FOLDERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),FALSE);// as IDC_STATIC_GROUP_MULTIPLE_FOLDERS contains sub groups, we must do this, as IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER is the first dialog item after sub groups
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR),FALSE);
        break;
         
    case TreeViewNodeType_LAUNCHBAR:
        this->hCurrentFolderOptionsTreeItem=this->pTreeview->GetParent(pChangeInfos->itemNew.hItem);
        // associated CGenericQuickWayToFolderObject* pSingleFolderOptions is in parent LPARAM
        this->pTreeview->GetItemUserData(this->hCurrentFolderOptionsTreeItem,(LPVOID*)&this->pCurrentFolderOptions);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_APPLICATION_OPTIONS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_BROWSE),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_CUSTOMIZATION),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FILTERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FOLDER_TYPE),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_MULTIPLE_FOLDERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),FALSE);// as IDC_STATIC_GROUP_MULTIPLE_FOLDERS contains sub groups, we must do this, as IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER is the first dialog item after sub groups
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR),TRUE);
        break;
    default: // pChangeInfos->itemNew.lParam contains CGenericQuickWayToFolderObject* pSingleFolderOptions
        this->hCurrentFolderOptionsTreeItem=pChangeInfos->itemNew.hItem;
        this->pCurrentFolderOptions=(CGenericQuickWayToFolderObject*)pChangeInfos->itemNew.lParam;
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_APPLICATION_OPTIONS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),TRUE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),TRUE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_BROWSE),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_CUSTOMIZATION),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FILTERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FOLDER_TYPE),TRUE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),TRUE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR),FALSE);

        // show IDC_STATIC_GROUP_MULTIPLE_FOLDERS if necessary
        this->UpdatePathAndIconGroupPosition(this->GetGUIFolderPresentationType());

        break;
    }

    // if folder has changed
    if (pOldFolderOptions!=this->pCurrentFolderOptions)
    {
        if (!this->bLoadingFolderOptions)// in some cases LoadFolderOptions affects change treeview state and raise an item selection callback, and folder options are overwritten
        {
            // save previous folder options
            if (pOldFolderOptions)
            {
                this->SaveFolderOptions(pOldFolderOptions);
                if (hOldFolderOptionsTreeItem)
                {
                    this->UpdateFolderTreeNode(hOldFolderOptionsTreeItem,pOldFolderOptions);
                }
            }

            if (this->pCurrentFolderOptions)
            {
                // load options of folder
                this->LoadFolderOptions(this->pCurrentFolderOptions);
            }
        }
    }
}

void COptionsUI::StopKeyBoardHook()
{
    // stop keyboard hook to avoid user interactions
    if (this->hKeyboardHook)
    {
        ::UnhookWindowsHookEx(this->hKeyboardHook);
        this->hKeyboardHook=NULL;
    }
}

void COptionsUI::OnOk()
{
    this->StopKeyBoardHook();

    if (this->pCurrentFolderOptions)
        this->SaveFolderOptions(this->pCurrentFolderOptions);

    this->SaveApplicationOptions();

    this->DialogResult=DialogResult_OK;
    this->Close();
}
void COptionsUI::OnCancel()
{
    this->StopKeyBoardHook();
    this->DialogResult=DialogResult_CANCEL;
    this->Close();
}

void COptionsUI::UpdatePathAndIconGroupPosition(CSingleFolderOptions::tagFolderPresentationType FolderPresentationType)
{
    if (FolderPresentationType == CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_DESKTOP_BROWSER)
        CDialogHelper::EnableGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),FALSE);
    else
        CDialogHelper::EnableGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),TRUE);

    this->EnableSaveIconPositionForHomeFolder(FolderPresentationType);

    if (FolderPresentationType == CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU)
    {
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_MULTIPLE_FOLDERS),TRUE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),TRUE);// as IDC_STATIC_GROUP_MULTIPLE_FOLDERS contains sub groups, we must do this, as IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER is the first dialog item after sub groups
        POINT Pt;
        RECT Rect;

        // put icon group under name group, keep same x pos
        CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_MULTIPLE_FOLDER_DISPLAY_NAME),&Rect);
        Pt.y=Rect.bottom;//+COptionsUI_SPACE_BETWEEN_CONTROLS;   
        CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),&Rect);
        Pt.x=Rect.left;
        CDialogHelper::MoveGroupTo(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),&Pt);

        // put launchbar icon group under name group, keep same x pos
        CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_MULTIPLE_FOLDER_DISPLAY_NAME),&Rect);
        Pt.y=Rect.bottom;//+COptionsUI_SPACE_BETWEEN_CONTROLS;   
        CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),&Rect);
        Pt.x=Rect.left;
        CDialogHelper::MoveGroupTo(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),&Pt);

        // put path group under icon group, at same y pos as Add button , keep same x pos
        CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),&Rect);
        Pt.y=Rect.bottom;  
        CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),&Rect);
        Pt.x=Rect.left;
        CDialogHelper::MoveGroupTo(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),&Pt);
    }
    else
    {
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_MULTIPLE_FOLDERS),FALSE);
        CDialogHelper::ShowGroup(::GetDlgItem(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),FALSE);// as IDC_STATIC_GROUP_MULTIPLE_FOLDERS contains sub groups, we must do this, as IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER is the first dialog item after sub groups
        POINT Pt;
        RECT Rect;
        // put path group just under folder types group
        CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_FOLDER_TYPE),&Rect);
        Pt.y=Rect.bottom;//+COptionsUI_SPACE_BETWEEN_CONTROLS;
        CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),&Rect);
        Pt.x=Rect.left;
        CDialogHelper::MoveGroupTo(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),&Pt);

        // put icon group under path group, keep same x pos
        CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),&Rect);
        Pt.y=Rect.bottom;//+COptionsUI_SPACE_BETWEEN_CONTROLS;  
        CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),&Rect);
        Pt.x=Rect.left;
        CDialogHelper::MoveGroupTo(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON),&Pt);

        // put launchbar icon group under path group, keep same x pos
        CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_PATH),&Rect);
        Pt.y=Rect.bottom;//+COptionsUI_SPACE_BETWEEN_CONTROLS;   
        CDialogHelper::GetClientWindowRect(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),&Rect);
        Pt.x=Rect.left;
        CDialogHelper::MoveGroupTo(this->hWndDialog,::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),&Pt);
    }
}
void COptionsUI::FillTreeView()
{
    int IconIndex;

    this->pTreeview->RemoveAllIcons();
    this->pTreeview->Clear();

    this->hItemApplicationOptions=this->pTreeview->Insert(TVI_ROOT,this->pLanguage->Options,(LPARAM)TreeViewNodeType_APPLICATION);
    IconIndex=this->pTreeview->AddIcon(this->hInstance,IDI_ICON_OPTIONS);
    this->pTreeview->SetItemIconIndex(this->hItemApplicationOptions,IconIndex);

    HTREEITEM hItem=this->pTreeview->Insert(this->hItemApplicationOptions,this->pLanguage->LaunchBar,(LPARAM)TreeViewNodeType_LAUNCHBAR);
    IconIndex=this->pTreeview->AddIcon(this->hInstance,IDI_ICON_LAUNCHBAR);
    this->pTreeview->SetItemIconIndex(hItem,IconIndex);
    this->pTreeview->Expand(this->hItemApplicationOptions);
    this->hItemLaunchBar=hItem;
    
    this->hItemFolders=this->pTreeview->Insert(TVI_ROOT,this->pLanguage->Folders,(LPARAM)TreeViewNodeType_FOLDERS);
    IconIndex=this->pTreeview->AddIcon(this->hInstance,IDI_ICON_FOLDERS);
    this->pTreeview->SetItemIconIndex(this->hItemFolders,IconIndex);

    this->TreeViewIconIndexBrowserOptions=this->pTreeview->AddIcon(this->hInstance,IDI_ICON_BROWSER_OPTIONS);
    this->TreeViewIconIndexCustomize=this->pTreeview->AddIcon(this->hInstance,IDI_ICON_CUSTOMIZE);
    this->TreeViewIconIndexFilters=this->pTreeview->AddIcon(this->hInstance,IDI_ICON_FILTER);

    for (size_t Cnt=0;Cnt<this->FoldersArray.size();Cnt++)
    {
        this->AddFolderToTreeView((CGenericQuickWayToFolderObject*)this->FoldersArray[Cnt],TVI_LAST,FALSE,FALSE);
    }

    this->pTreeview->Expand(this->hItemFolders,TRUE);
}

void COptionsUI::AddFolderToTreeView(CGenericQuickWayToFolderObject* pSingleFolderOptions,HTREEITEM hInsertAfter,BOOL bExpand,BOOL bSetFocus)
{
    HTREEITEM hItemFolder;
    HTREEITEM hItem;
    TCHAR ChildDir[MAX_PATH];

    int IconIndex;
    HICON hIcon;

    // extract wanted icon
    if (((int)::ExtractIconEx(pSingleFolderOptions->AssociatedIconPath,
                                pSingleFolderOptions->AssociatedIconIndex, 
                                NULL, 
                                &hIcon, 
                                1))
                                <=0
                                )
    {
        // on error load default icon
        hIcon=this->pQuickWayToFolders->hDefaultIcon;
    }
    else
    {
        LoadedIconsToBeDestroyed.push_back(hIcon);
    }

    // add associated icon
    IconIndex=this->pTreeview->AddIcon(hIcon);

    // get short name
    pSingleFolderOptions->GetShortName(ChildDir);

    // add node
    hItemFolder=this->pTreeview->Insert(this->hItemFolders,hInsertAfter,ChildDir,(LPARAM)pSingleFolderOptions);

    // associate icon to node
    this->pTreeview->SetItemIconIndex(hItemFolder,IconIndex);

    // fill node content
    hItem=this->pTreeview->Insert(hItemFolder,this->pLanguage->BrowsingOptions,TreeViewNodeType_FOLDER_BROWSING_OPTION);
    this->pTreeview->SetItemIconIndex(hItem,this->TreeViewIconIndexBrowserOptions);
    hItem=this->pTreeview->Insert(hItemFolder,this->pLanguage->FileFilters,TreeViewNodeType_FOLDER_FILTERS);
    this->pTreeview->SetItemIconIndex(hItem,this->TreeViewIconIndexFilters);
    hItem=this->pTreeview->Insert(hItemFolder,this->pLanguage->Customization,TreeViewNodeType_FOLDER_CUSTOMIZATION);
    this->pTreeview->SetItemIconIndex(hItem,this->TreeViewIconIndexCustomize);

    if (bExpand)
        this->pTreeview->Expand(hItemFolder);
    if (bSetFocus)
        this->pTreeview->SetSelectedItem(hItemFolder);
}

void COptionsUI::UpdateFolderTreeNode(HTREEITEM hItem,CGenericQuickWayToFolderObject* pSingleFolderOptions)
{
    TCHAR ChildDir[MAX_PATH];

    // get child dir
    pSingleFolderOptions->GetShortName(ChildDir);
    // if (*ChildDir) let empty field to be multiple languages compatible
        this->pTreeview->SetItemText(hItem,ChildDir);
    // else
    //    this->pTreeview->SetItemText( hItem,TEXT("Error") );
}

void COptionsUI::SetPath(TCHAR* PathOrCSIDL)
{
    if (CSingleFolderOptions::IsCSIDL(PathOrCSIDL))
    {
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_PATH_SPECIAL_FOLDER_LOCATION),TRUE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_PATH_LOCAL_PATH),FALSE);
        // clear the local path
        ::SetDlgItemText(this->hWndDialog,IDC_EDIT_PATH_LOCAL_FOLDER_PATH,_T(""));
        DWORD Csidl;
        if (CSingleFolderOptions::GetCSIDL(PathOrCSIDL,&Csidl))
        {
            // select matching csidl in combo
            HWND hWndCombo = ::GetDlgItem(this->hWndDialog,IDC_COMBO_SPECIAL_FOLDER_LOCATION);
            COMBOBOXEXITEM ComboboxExItem;
            ComboboxExItem.mask = CBEIF_LPARAM;
            DWORD NbCSIDL=ComboBox_GetCount(hWndCombo);
            ComboBox_SetCurSel(hWndCombo,0); // default in case of not found
            for (DWORD Cnt=0;Cnt<NbCSIDL;Cnt++)
            {
                ComboboxExItem.iItem = Cnt;

                if (SendMessage(hWndCombo,CBEM_GETITEM,0,(LPARAM)&ComboboxExItem)==0)
                    continue;

                if ((DWORD)ComboboxExItem.lParam==Csidl)
                {
                    ComboBox_SetCurSel(hWndCombo,Cnt);
                    break;
                }
            }
        }
    }
    else
    {
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_PATH_SPECIAL_FOLDER_LOCATION),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_PATH_LOCAL_PATH),TRUE);
        ::SetDlgItemText(this->hWndDialog,IDC_EDIT_PATH_LOCAL_FOLDER_PATH,PathOrCSIDL);
    }
}

void COptionsUI::GetPath(OUT TCHAR* PathOrCSIDL,IN SIZE_T PathOrCSIDLMaxSize)
{

    if (CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_DESKTOP)))
    {
        _sntprintf(PathOrCSIDL,PathOrCSIDLMaxSize,CSingleFolderOptions_CSIDL_PREFIX _T("0x%X"),CSIDL_DESKTOP);
    }
    else
    {
        if (CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_RADIO_PATH_SPECIAL_FOLDER_LOCATION)))
        {
            HWND hWndCombo = ::GetDlgItem(this->hWndDialog,IDC_COMBO_SPECIAL_FOLDER_LOCATION);
            // get matching csidl in combo
            LONG SelectedItem=ComboBox_GetCurSel(hWndCombo);
            if (SelectedItem == CB_ERR)
                SelectedItem=0;

            COMBOBOXEXITEM ComboboxExItem;
            ComboboxExItem.mask = CBEIF_LPARAM;
            ComboboxExItem.iItem = SelectedItem;
            if (SendMessage(hWndCombo,CBEM_GETITEM,0,(LPARAM)&ComboboxExItem)!=0)
                _sntprintf(PathOrCSIDL,PathOrCSIDLMaxSize,CSingleFolderOptions_CSIDL_PREFIX _T("0x%X"),ComboboxExItem.lParam);
            else
                _sntprintf(PathOrCSIDL,PathOrCSIDLMaxSize,CSingleFolderOptions_CSIDL_PREFIX _T("0x%X"),CSIDL_MYDOCUMENTS);
        }
        else // if (CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_RADIO_PATH_LOCAL_PATH)))
        {
            ::GetDlgItemText(this->hWndDialog,IDC_EDIT_PATH_LOCAL_FOLDER_PATH,PathOrCSIDL,(int)PathOrCSIDLMaxSize);
        }
    }

	PathOrCSIDL[PathOrCSIDLMaxSize-1]=0;
}

void COptionsUI::UpdatePreviewIcon()
{
    // load associated icon
    HICON hIconLarge;
    HICON hIconSmall;
    if ( ((int)::ExtractIconEx(this->AssociatedIconPath,
                                this->AssociatedIconIndex, 
                                &hIconLarge, 
                                &hIconSmall, 
                                1)
         )<=0
       )
    {
        // on error load default icon
        hIconLarge=this->pQuickWayToFolders->hDefaultIcon;
    }
    else
    {
        // add icon handle to the to be destroy vector
        this->LoadedIconsToBeDestroyed.push_back(hIconLarge);
        this->LoadedIconsToBeDestroyed.push_back(hIconSmall);
        
        // add small icon to treeview
        int IconIndex=this->pTreeview->AddIcon(hIconSmall);
        // update treeview item
        this->pTreeview->SetItemIconIndex(this->hCurrentFolderOptionsTreeItem,IconIndex);
    }
    ::SendMessage(::GetDlgItem(this->hWndDialog,IDC_STATIC_ASSOCIATED_ICON),STM_SETICON,(WPARAM)hIconLarge,0);
}

void COptionsUI::EnableBrowserOptions(BOOL bEnable)
{
    // options only for browser
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_REMEMBER_LAST_POSITION),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_SINGLE_CLICK_SELECTION),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_AUTO_HIDE_ON_EXECUTION),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_COMBO_BROWSE_OPTIONS_VIEW_TYPE),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_COMBO_Z_ORDER),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_HIDE_TOOLBAR_FOR_HOME_FOLDER),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_HIDE_TOOLBAR),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_SHOW_AT_STARTUP),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_START_MAXIMIZED),bEnable);
    // removed in 1.3.2 version as hot key is used by some users for menus too
    // ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_EDIT_HOT_KEY),bEnable);
    CDialogHelper::EnableGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_CUSTOMIZATION),bEnable);
    // hot corner : items are not in group to keep show/hide facilities
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_ENABLE_HOT_CORNER),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_COMBO_HOT_CORNER_MONITOR_INDEX),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_COMBO_HOT_CORNER_POSITION),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_EDIT_HOT_CORNER_WIDTH),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_EDIT_HOT_CORNER_HEIGHT),bEnable);

    // options only for menu
    bEnable = !bEnable; // inverse enable state (done once for all following instead of doing it each time)
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_MENU_HIDE_FILE_EXTENSION),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_MENU_HIDE_SHORTCUTS_OVERLAY),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_COMBO_FOLDERS_POSITIONS_IN_POPUPMENU),bEnable);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_ALLOW_SUBFOLDER_LINK_NAVIGATION),bEnable);
}

void COptionsUI::UpdateFolderPresentationType(CSingleFolderOptions::tagFolderPresentationType FolderPresentationType)
{
    BOOL bBrowser=FALSE;

	if (this->hDisplayedBmp)
	{
		::DeleteObject(this->hDisplayedBmp);
		this->hDisplayedBmp=NULL;
	}

    switch(FolderPresentationType)
    {
    case CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_DESKTOP_BROWSER:
        bBrowser=TRUE;
        this->pBitmapDesktop->m_pBitmap->GetHBITMAP(NULL,&this->hDisplayedBmp);
        break;
    case CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_BROWSER:
        bBrowser=TRUE;
        this->pBitmapBrowser->m_pBitmap->GetHBITMAP(NULL,&this->hDisplayedBmp);
        break;
    case CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_SINGLE_MENU:
        this->pBitmapMenu->m_pBitmap->GetHBITMAP(NULL,&this->hDisplayedBmp);
        break;
    case CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU:
        this->pBitmapMultipleMenu->m_pBitmap->GetHBITMAP(NULL,&this->hDisplayedBmp);
        break;
    }

    if (this->hDisplayedBmp)
        ::SendMessage(::GetDlgItem(this->hWndDialog,IDC_STATIC_FOLDER_TYPE_IMAGE),STM_SETIMAGE,(WPARAM)IMAGE_BITMAP,(LPARAM)this->hDisplayedBmp);

    this->EnableBrowserOptions(bBrowser);

    // enable IDC_CHECK_SAVE_ICONS_POSITIONS_FOR_HOME_FOLDER only for browser type (not desktop)
    this->EnableSaveIconPositionForHomeFolder(FolderPresentationType);

    if (this->pTreeview)
    {
        HTREEITEM hItem=this->pTreeview->GetSelectedItem();
        if (hItem)
        {
            LPVOID UserData;
            tagTreeViewNodeType TreeViewNodeType;
            this->pTreeview->GetItemUserData(hItem,&UserData);
            TreeViewNodeType=(tagTreeViewNodeType)(int)UserData;
            switch (TreeViewNodeType)
            {
            case TreeViewNodeType_APPLICATION:
            case TreeViewNodeType_FOLDERS:
            case TreeViewNodeType_FOLDER_FILTERS:
            case TreeViewNodeType_FOLDER_CUSTOMIZATION:
            case TreeViewNodeType_FOLDER_BROWSING_OPTION:
            case TreeViewNodeType_LAUNCHBAR:
                break;
            default: // a folder tree node is selected
                // as UpdatePathAndIconGroupPosition changes multi folder group visibility,
                // call it only if folder tree node is selected
                this->UpdatePathAndIconGroupPosition(FolderPresentationType);
                break;
            }
        }

    }
}


void COptionsUI::EnableSaveIconPositionForHomeFolder(CSingleFolderOptions::tagFolderPresentationType FolderPresentationType)
{
    BOOL bEnable = TRUE;

    CShellBrowserEx::tagShellBrowserView ShellBrowserView = this->ComboViewTypeIndexes[::SendMessage(::GetDlgItem(this->hWndDialog,IDC_COMBO_BROWSE_OPTIONS_VIEW_TYPE),CB_GETCURSEL ,0,0)];
    if ( (ShellBrowserView == CShellBrowser::SHELL_BROWSER_VIEW_LIST)
        || (ShellBrowserView == CShellBrowser::SHELL_BROWSER_VIEW_DETAIL)
        || (ShellBrowserView == CShellBrowser::SHELL_BROWSER_VIEW_THUMBSTRIP)
        )
        bEnable = FALSE;

    if (FolderPresentationType != CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_BROWSER)
        bEnable = FALSE;

    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_SAVE_ICONS_POSITIONS_FOR_HOME_FOLDER),bEnable);
}
void COptionsUI::EnableSaveIconPositionForHomeFolder()
{
    CSingleFolderOptions::tagFolderPresentationType FolderPresentationType = this->GetGUIFolderPresentationType();
    return this->EnableSaveIconPositionForHomeFolder(FolderPresentationType);
}

void COptionsUI::LoadFolderOptions(CGenericQuickWayToFolderObject* pSingleFolderOptions)
{
    this->bLoadingFolderOptions=TRUE;
    SIZE_T Cnt = 0;

    this->pCurrentFolderOptions=pSingleFolderOptions;
    switch(pSingleFolderOptions->FolderPresentationType)
    {
    case CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_DESKTOP_BROWSER:
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_DESKTOP),TRUE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_BROWSER),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_MENU),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_MULTIPLE_FOLDER_MENU),FALSE);
        break;
    case CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_BROWSER:
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_DESKTOP),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_BROWSER),TRUE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_MENU),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_MULTIPLE_FOLDER_MENU),FALSE);
        break;
    case CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_SINGLE_MENU:
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_DESKTOP),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_BROWSER),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_MENU),TRUE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_MULTIPLE_FOLDER_MENU),FALSE);
        break;
    case CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU:
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_DESKTOP),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_BROWSER),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_MENU),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_MULTIPLE_FOLDER_MENU),TRUE);
        break;
    }

    
    for (Cnt=0;Cnt<this->MultipleMenuListviewLoadedIconsToBeDestroyed.size();Cnt++)
        ::DestroyIcon(this->MultipleMenuListviewLoadedIconsToBeDestroyed[Cnt]);
    this->MultipleMenuListviewLoadedIconsToBeDestroyed.clear();

    this->pListview->RemoveAllIcons(CListview::ImageListSmall);
    this->pListview->Clear();
    this->LocalMultiplePath.clear();

    // update display name (used only for multiple menu currently)
    ::SetDlgItemText(this->hWndDialog,IDC_EDIT_MULTIPLE_FOLDERS_MENU_DISPLAY_NAME,pSingleFolderOptions->DisplayName);

    if (pSingleFolderOptions->FolderPresentationType == CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU)
    {
        if (*pSingleFolderOptions->DisplayName == 0)
            //put "Multiple Folder Menu" in path for displayed name in treeview
            _tcscpy(pSingleFolderOptions->DisplayName,this->pLanguage->MultipleFoldersMenu);

        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_PATH_SPECIAL_FOLDER_LOCATION),TRUE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_PATH_LOCAL_PATH),FALSE);

        CDialogHelper::SetButtonCheckState(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER,FALSE);
        ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_EDIT_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),FALSE);

        // clear the local path
        ::SetDlgItemText(this->hWndDialog,IDC_EDIT_PATH_LOCAL_FOLDER_PATH,_T(""));
        for (size_t Cnt=0;Cnt<pSingleFolderOptions->pMultiplePaths->size();Cnt++)
        {
            this->LocalMultiplePath.push_back(pSingleFolderOptions->pMultiplePaths->at(Cnt));
            this->AddFolderToMultipleFolderListview(&this->LocalMultiplePath[Cnt]); // use local memory item
        }
    }
    else
    {
        if (pSingleFolderOptions->FolderPresentationType == CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_DESKTOP_BROWSER)
            _tcscpy(pSingleFolderOptions->DisplayName,this->pLanguage->Desktop);
        this->SetPath(pSingleFolderOptions->Path);
    }

    // copy icons information from options to local object and load icons
    _tcscpy(this->AssociatedIconPath,pSingleFolderOptions->AssociatedIconPath);
    this->AssociatedIconIndex=pSingleFolderOptions->AssociatedIconIndex;
    this->UpdatePreviewIcon();

    HICON hIconLarge;
    _tcscpy(this->LaunchBarAssociatedIconHotPath,pSingleFolderOptions->LaunchBarAssociatedIconHotPath);
    this->LaunchBarAssociatedIconHotIndex=pSingleFolderOptions->LaunchBarAssociatedIconHotIndex;
    hIconLarge = this->ExtractLargeIcon(this->LaunchBarAssociatedIconHotPath,this->LaunchBarAssociatedIconHotIndex);
    ::SendMessage(::GetDlgItem(this->hWndDialog,IDC_STATIC_LAUNCHBAR_HOT_ICON),STM_SETICON,(WPARAM)hIconLarge,0);

    _tcscpy(this->LaunchBarAssociatedIconDownPath,pSingleFolderOptions->LaunchBarAssociatedIconDownPath);
    this->LaunchBarAssociatedIconDownIndex=pSingleFolderOptions->LaunchBarAssociatedIconDownIndex;
    hIconLarge = this->ExtractLargeIcon(this->LaunchBarAssociatedIconDownPath,this->LaunchBarAssociatedIconDownIndex);
    ::SendMessage(::GetDlgItem(this->hWndDialog,IDC_STATIC_LAUNCHBAR_DOWN_ICON),STM_SETICON,(WPARAM)hIconLarge,0);


    // hot corner options (explorer option only : browsing options)
    CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_CHECK_ENABLE_HOT_CORNER),pSingleFolderOptions->bEnableHotCorner);
    ComboBox_SetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_HOT_CORNER_MONITOR_INDEX),pSingleFolderOptions->HotCornerScreenIndex);
    ComboBox_SetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_HOT_CORNER_POSITION),pSingleFolderOptions->HotCornerPosition);
    ::SetDlgItemInt(this->hWndDialog,IDC_EDIT_HOT_CORNER_WIDTH,pSingleFolderOptions->HotCornerWidth,FALSE);
    ::SetDlgItemInt(this->hWndDialog,IDC_EDIT_HOT_CORNER_HEIGHT,pSingleFolderOptions->HotCornerHeight,FALSE);

    // launch bar icon infos
    ComboBox_SetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_ICON_POSITION),pSingleFolderOptions->LaunchBarIconPosition);
    ::SetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_ICON_SPACER_BEFORE_ICON,pSingleFolderOptions->LaunchBarFreeSpaceBeforeIcon,FALSE);
    
    // explorer infos
    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_INCLUDE_SUBFOLDERS),
                                        pSingleFolderOptions->bIncludeSubFolders
                                        );
    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_ALLOW_SUBFOLDER_NAVIGATION),
                                        pSingleFolderOptions->bAllowSubFoldersNavigation
                                        );
    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_ALLOW_SUBFOLDER_LINK_NAVIGATION),
                                        pSingleFolderOptions->bAllowSubFoldersLinksNavigation
                                        );

    ::SetDlgItemText(this->hWndDialog,IDC_EDIT_INCLUSION_FILTERS,pSingleFolderOptions->InclusionFilters);
    ::SetDlgItemText(this->hWndDialog,IDC_EDIT_EXCLUSION_FILTERS,pSingleFolderOptions->ExclusionFilters);

    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_MENU_HIDE_FILE_EXTENSION),
                                        pSingleFolderOptions->bHideFilesExtensionInPopupMenu
                                        );

    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_MENU_HIDE_SHORTCUTS_OVERLAY),
                                        pSingleFolderOptions->bHideShortcutsOverlayInPopupMenu
                                        );

    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_MERGE_MENUS_IN_POPUPMENU),
                                        pSingleFolderOptions->bMergeMenusInPopUpMenu
                                        );

    BOOL bEnableMergeToSepcificFolder;
    bEnableMergeToSepcificFolder = !pSingleFolderOptions->bMergeMenusInPopUpMenu;
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),bEnableMergeToSepcificFolder);
    ::EnableWindow(::GetDlgItem(this->hWndDialog,IDC_EDIT_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),bEnableMergeToSepcificFolder && CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER)));

    ComboBox_SetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_FOLDERS_POSITIONS_IN_POPUPMENU),pSingleFolderOptions->FoldersPositionInPopUpMenu);

    // loop through this->ComboViewTypeIndexes to find item matching view mode
    SIZE_T Index = 0;
    HWND hWndCombo = ::GetDlgItem(this->hWndDialog,IDC_COMBO_BROWSE_OPTIONS_VIEW_TYPE);
    SIZE_T ItemCount = ComboBox_GetCount(hWndCombo);
    for (Cnt = 0; Cnt<ItemCount; Cnt++)
    {
        if (this->ComboViewTypeIndexes[Cnt] == pSingleFolderOptions->FolderViewMode)
        {
            Index = Cnt;
            break;
        }
    }
    ComboBox_SetCurSel(hWndCombo,Index);

    ComboBox_SetCurSel(
                        ::GetDlgItem(this->hWndDialog,IDC_COMBO_Z_ORDER),
                        pSingleFolderOptions->ZOrder
                        );
    
    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_HIDE_TOOLBAR_FOR_HOME_FOLDER),
                                        pSingleFolderOptions->bHideToolBarForHomeFolder
                                        );
    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_HIDE_TOOLBAR),
                                        pSingleFolderOptions->bHideToolBar
                                        );

    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_SHOW_AT_STARTUP),
                                        pSingleFolderOptions->bShowAtStartup
                                        );
    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_START_MAXIMIZED),
                                        pSingleFolderOptions->bStartMaximized
                                        );
    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_SAVE_ICONS_POSITIONS_FOR_HOME_FOLDER),
                                        pSingleFolderOptions->bSaveIconsPositionsForHomeFolder
                                        );

    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_REMEMBER_LAST_POSITION),
                                        pSingleFolderOptions->bRememberLastPosition
                                        );
    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_SINGLE_CLICK_SELECTION),
                                        pSingleFolderOptions->bOneClickActivate
                                        );
    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_AUTO_HIDE_ON_EXECUTION),
                                        pSingleFolderOptions->bAutoHideOnFileExecution
                                        );


    this->HotKeyfsModifiers = pSingleFolderOptions->HotKeyfsModifiers;
    this->HotKeyVirtualKey = pSingleFolderOptions->HotKeyVirtualKey;
    TCHAR HotKey[MAX_PATH];
    this->GetKeysString(this->HotKeyfsModifiers,this->HotKeyVirtualKey,HotKey);
    ::SetDlgItemText(this->hWndDialog,IDC_EDIT_HOT_KEY,HotKey);


    // specific to browser / desktop browser background / text style 
    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_CUSTOMIZATION_TEXT_COLOR),
                                        pSingleFolderOptions->bTextColorSepcified
                                        );
    CDialogHelper::SetButtonCheckState(
                                        ::GetDlgItem(this->hWndDialog,IDC_CHECK_CUSTOMIZATION_TEXT_BACKGROUND_COLOR),
                                        pSingleFolderOptions->bTextBackgroundColorSepcified
                                        );

    this->DeleteFolderBrushs();
    this->TextColorBrush=::CreateSolidBrush(pSingleFolderOptions->TextColor);
    this->TextBackgroundColorBrush=::CreateSolidBrush(pSingleFolderOptions->TextBackgroundColor);
    this->BackgroundColorBrush=::CreateSolidBrush(pSingleFolderOptions->BackgroundColor);
    this->BackgroundColor1Brush=::CreateSolidBrush(pSingleFolderOptions->BackgroundColor1);
    this->BackgroundColor2Brush=::CreateSolidBrush(pSingleFolderOptions->BackgroundColor2);
    // force picture buttons to be redrawn
    ::RedrawWindow(this->hWndTextColor,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME );
    ::RedrawWindow(this->hWndTextBackgroundColor,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME );
    ::RedrawWindow(this->hWndBackgroundColor,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME );
    ::RedrawWindow(this->hWndBackgroundColor1,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME );
    ::RedrawWindow(this->hWndBackgroundColor2,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME );


    switch(pSingleFolderOptions->BackgroundType)
    {
    case CShellBrowserEx::BACKGROUND_TYPE_NONE:
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_NONE),TRUE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_COLOR),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_GRADIANT),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_PICTURE),FALSE);
        break;
    case CShellBrowserEx::BACKGROUND_TYPE_COLOR:
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_NONE),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_COLOR),TRUE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_GRADIANT),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_PICTURE),FALSE);
        break;
    case CShellBrowserEx::BACKGROUND_TYPE_GRADIENT:
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_NONE),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_COLOR),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_GRADIANT),TRUE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_PICTURE),FALSE);
        break;
    case CShellBrowserEx::BACKGROUND_TYPE_PICTURE:
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_NONE),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_COLOR),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_GRADIANT),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_PICTURE),TRUE);
        break;
    }

    ComboBox_SetCurSel(
                        ::GetDlgItem(this->hWndDialog,IDC_COMBO_CUSTOMIZATION_BACKGROUND_GRADIANT_TYPE),
                        pSingleFolderOptions->BackgroundGradiantType //-Gdiplus::LinearGradientModeFirst
                        );
    ComboBox_SetCurSel(
                        ::GetDlgItem(this->hWndDialog,IDC_COMBO_CUSTOMIZATION_BACKGROUND_PICTURE_POSITION),
                        pSingleFolderOptions->BackgroundImagePosition
                        );

    ::SetDlgItemText(this->hWndDialog,IDC_EDIT_CUSTOMIZATION_BACKGROUND_PATH,pSingleFolderOptions->BackgroundImagePath);

    // called only after CB_SETCURSEL on IDC_COMBO_BROWSE_OPTIONS_VIEW_TYPE
    this->UpdateFolderPresentationType(pSingleFolderOptions->FolderPresentationType);

    // disable "hide toolbar for home folder" if "hide toolbar is enabled" (must be called after UpdateFolderPresentationType)
    HWND hWnd = ::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_HIDE_TOOLBAR_FOR_HOME_FOLDER);
    if (::IsWindowEnabled(hWnd))
        ::EnableWindow(hWnd,!pSingleFolderOptions->bHideToolBar);

    this->bLoadingFolderOptions=FALSE;
}

COLORREF COptionsUI::GetBrushColor( HBRUSH hBrush )
{
    LOGBRUSH logBrush={0};
    ::GetObject( hBrush, sizeof(LOGBRUSH), &logBrush );
    return logBrush.lbColor;
}
void COptionsUI::SaveFolderOptions(CGenericQuickWayToFolderObject* pSingleFolderOptions)
{
    pSingleFolderOptions->FolderPresentationType=this->GetGUIFolderPresentationType();

    pSingleFolderOptions->pMultiplePaths->clear();
    if (pSingleFolderOptions->FolderPresentationType == CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU)
    {
        // store changes in pSingleFolderOptions->pMultiplePaths
        for (size_t Cnt=0;Cnt<this->LocalMultiplePath.size();Cnt++)
        {
            pSingleFolderOptions->pMultiplePaths->push_back(this->LocalMultiplePath[Cnt]);
        }
        ::GetDlgItemText(this->hWndDialog,IDC_EDIT_MULTIPLE_FOLDERS_MENU_DISPLAY_NAME,pSingleFolderOptions->DisplayName,_countof(pSingleFolderOptions->DisplayName));
        if (*pSingleFolderOptions->DisplayName == 0)
            _tcscpy(pSingleFolderOptions->DisplayName,this->pLanguage->MultipleFoldersMenu);
    }
    else
    {
        this->GetPath(pSingleFolderOptions->Path,_countof(pSingleFolderOptions->Path));
    }

    // copy icons information from local object to options
    _tcscpy(pSingleFolderOptions->AssociatedIconPath,this->AssociatedIconPath);
    pSingleFolderOptions->AssociatedIconIndex=this->AssociatedIconIndex;

    _tcscpy(pSingleFolderOptions->LaunchBarAssociatedIconHotPath,this->LaunchBarAssociatedIconHotPath);
    pSingleFolderOptions->LaunchBarAssociatedIconHotIndex=this->LaunchBarAssociatedIconHotIndex;

    _tcscpy(pSingleFolderOptions->LaunchBarAssociatedIconDownPath,this->LaunchBarAssociatedIconDownPath);
    pSingleFolderOptions->LaunchBarAssociatedIconDownIndex=this->LaunchBarAssociatedIconDownIndex;

    // hot corner infos
    pSingleFolderOptions->bEnableHotCorner = CDialogHelper::IsButtonChecked(this->hWndDialog,IDC_CHECK_ENABLE_HOT_CORNER);
    pSingleFolderOptions->HotCornerScreenIndex = ComboBox_GetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_HOT_CORNER_MONITOR_INDEX));
    pSingleFolderOptions->HotCornerPosition = (CSingleFolderOptions::tagHotCornerPosition)ComboBox_GetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_HOT_CORNER_POSITION));
    BOOL bSuccess;
    UINT TmpValue;
    TmpValue = ::GetDlgItemInt(this->hWndDialog,IDC_EDIT_HOT_CORNER_WIDTH,&bSuccess,FALSE);
    if (bSuccess)
        pSingleFolderOptions->HotCornerWidth = TmpValue;

    TmpValue = ::GetDlgItemInt(this->hWndDialog,IDC_EDIT_HOT_CORNER_HEIGHT,&bSuccess,FALSE);
    if (bSuccess)
        pSingleFolderOptions->HotCornerHeight = TmpValue;

    // launch bar icon info
    pSingleFolderOptions->LaunchBarIconPosition = (CTransparentToolbar::tagButtonPosition)ComboBox_GetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_ICON_POSITION));
    TmpValue = ::GetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_ICON_SPACER_BEFORE_ICON,&bSuccess,FALSE);
    if (bSuccess)
        pSingleFolderOptions->LaunchBarFreeSpaceBeforeIcon = TmpValue;
    
    // explorer infos
    pSingleFolderOptions->bIncludeSubFolders=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_INCLUDE_SUBFOLDERS));
    pSingleFolderOptions->bAllowSubFoldersNavigation=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_ALLOW_SUBFOLDER_NAVIGATION));
    pSingleFolderOptions->bAllowSubFoldersLinksNavigation=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_ALLOW_SUBFOLDER_LINK_NAVIGATION));

    ::GetDlgItemText(this->hWndDialog,IDC_EDIT_INCLUSION_FILTERS,pSingleFolderOptions->InclusionFilters,CSingleFolderOptions_FILTERS_MAX_SIZE);
    ::GetDlgItemText(this->hWndDialog,IDC_EDIT_EXCLUSION_FILTERS,pSingleFolderOptions->ExclusionFilters,CSingleFolderOptions_FILTERS_MAX_SIZE);

    pSingleFolderOptions->bHideFilesExtensionInPopupMenu=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_MENU_HIDE_FILE_EXTENSION));
    pSingleFolderOptions->bHideShortcutsOverlayInPopupMenu=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_MENU_HIDE_SHORTCUTS_OVERLAY));
    pSingleFolderOptions->bMergeMenusInPopUpMenu=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_MERGE_MENUS_IN_POPUPMENU));
    pSingleFolderOptions->FoldersPositionInPopUpMenu = (CFolderMenu::tagFoldersPosition)ComboBox_GetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_FOLDERS_POSITIONS_IN_POPUPMENU));

    pSingleFolderOptions->FolderViewMode= this->ComboViewTypeIndexes[::SendMessage(::GetDlgItem(this->hWndDialog,IDC_COMBO_BROWSE_OPTIONS_VIEW_TYPE),CB_GETCURSEL ,0,0)];
    if (pSingleFolderOptions->FolderViewMode==CB_ERR)
        pSingleFolderOptions->FolderViewMode=CShellBrowserEx::SHELL_BROWSER_VIEW_ICONS;

    pSingleFolderOptions->ZOrder=(CSingleFolderOptions::tagBrowserZOrder)(::SendMessage(::GetDlgItem(this->hWndDialog,IDC_COMBO_Z_ORDER),CB_GETCURSEL ,0,0));
    if (pSingleFolderOptions->ZOrder==CB_ERR)
        pSingleFolderOptions->ZOrder=CSingleFolderOptions::BROWSER_Z_ORDER_NORMAL;
    pSingleFolderOptions->bHideToolBarForHomeFolder=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_HIDE_TOOLBAR_FOR_HOME_FOLDER));
    pSingleFolderOptions->bHideToolBar=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_HIDE_TOOLBAR));
    pSingleFolderOptions->bShowAtStartup=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_SHOW_AT_STARTUP));
    pSingleFolderOptions->bStartMaximized=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_START_MAXIMIZED));
    if (pSingleFolderOptions->bStartMaximized)
        pSingleFolderOptions->bIsMaximized = TRUE;

    pSingleFolderOptions->bSaveIconsPositionsForHomeFolder=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_SAVE_ICONS_POSITIONS_FOR_HOME_FOLDER));
    
    pSingleFolderOptions->bRememberLastPosition=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_REMEMBER_LAST_POSITION));
    pSingleFolderOptions->bOneClickActivate=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_SINGLE_CLICK_SELECTION));
    pSingleFolderOptions->bAutoHideOnFileExecution=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_AUTO_HIDE_ON_EXECUTION));
    pSingleFolderOptions->bTextColorSepcified=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_CUSTOMIZATION_TEXT_COLOR));
    pSingleFolderOptions->bTextBackgroundColorSepcified=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_CUSTOMIZATION_TEXT_BACKGROUND_COLOR));

    pSingleFolderOptions->TextColor=this->GetBrushColor(this->TextColorBrush);
    pSingleFolderOptions->TextBackgroundColor=this->GetBrushColor(this->TextBackgroundColorBrush);
    pSingleFolderOptions->BackgroundColor=this->GetBrushColor(this->BackgroundColorBrush);
    pSingleFolderOptions->BackgroundColor1=this->GetBrushColor(this->BackgroundColor1Brush);
    pSingleFolderOptions->BackgroundColor2=this->GetBrushColor(this->BackgroundColor2Brush);

    if (CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_NONE)))
    {
        pSingleFolderOptions->BackgroundType=CShellBrowserEx::BACKGROUND_TYPE_NONE;
    }
    else if (CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_COLOR)))
    {
        pSingleFolderOptions->BackgroundType=CShellBrowserEx::BACKGROUND_TYPE_COLOR;
    }
    else if (CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_GRADIANT)))
    {
        pSingleFolderOptions->BackgroundType=CShellBrowserEx::BACKGROUND_TYPE_GRADIENT;
    }
    else if (CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_PICTURE)))
    {
        pSingleFolderOptions->BackgroundType=CShellBrowserEx::BACKGROUND_TYPE_PICTURE;
    }

    pSingleFolderOptions->BackgroundGradiantType=(Gdiplus::LinearGradientMode)::SendMessage(::GetDlgItem(this->hWndDialog,IDC_COMBO_CUSTOMIZATION_BACKGROUND_GRADIANT_TYPE),CB_GETCURSEL ,0,0);
    if (pSingleFolderOptions->BackgroundGradiantType==CB_ERR)
        pSingleFolderOptions->BackgroundGradiantType=Gdiplus::LinearGradientModeHorizontal;
    pSingleFolderOptions->BackgroundImagePosition=(CShellBrowserEx::tagBackgroundImagePosition)::SendMessage(::GetDlgItem(this->hWndDialog,IDC_COMBO_CUSTOMIZATION_BACKGROUND_PICTURE_POSITION),CB_GETCURSEL ,0,0);
    if (pSingleFolderOptions->BackgroundImagePosition==CB_ERR)
        pSingleFolderOptions->BackgroundImagePosition=CShellBrowserEx::BACKGROUND_IMAGE_POSITION_STRETCHED;
        
    ::GetDlgItemText(this->hWndDialog,IDC_EDIT_CUSTOMIZATION_BACKGROUND_PATH,pSingleFolderOptions->BackgroundImagePath,MAX_PATH);

    pSingleFolderOptions->HotKeyfsModifiers = this->HotKeyfsModifiers;
    pSingleFolderOptions->HotKeyVirtualKey = this->HotKeyVirtualKey;
}

void COptionsUI::LoadSkinsList()
{
    TCHAR SearchSubDirectoriesPattern[MAX_PATH];
    TCHAR ConfigFilePath[MAX_PATH];
    TCHAR* SkinsPath = this->pApplicationOptions->BrowserToolBarOptions.GetSkinsPath();
    WIN32_FIND_DATA FindFileData={0};
    _sntprintf(SearchSubDirectoriesPattern,_countof(SearchSubDirectoriesPattern),_T("%s*"),SkinsPath);

    HWND hWnd = ::GetDlgItem(this->hWndDialog,IDC_COMBO_SKIN);
    ComboBox_ResetContent(hWnd);
    ComboBox_AddString(hWnd,this->pLanguage->None);

    // find first file
    HANDLE hFileHandle=::FindFirstFile(SearchSubDirectoriesPattern,&FindFileData);
    if (hFileHandle==INVALID_HANDLE_VALUE)// if no file found
        return;

    do 
    {
        if ((FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0)
        {
            if (     (_tcscmp(FindFileData.cFileName,_T("."))==0)
                || (_tcscmp(FindFileData.cFileName,_T(".."))==0)
                )
                continue;

            _sntprintf(ConfigFilePath,_countof(ConfigFilePath),_T("%s%s\\") CBrowserToolbar_SKIN_CONFIGURATION_FILENAME,SkinsPath,FindFileData.cFileName);
            if (CStdFileOperations::DoesFileExists(ConfigFilePath))
            {
                ComboBox_AddString(hWnd,FindFileData.cFileName);
            }
        }

        // find next file
    } while(::FindNextFile(hFileHandle, &FindFileData));

    ::FindClose(hFileHandle);
}

void COptionsUI::UpdateSkinPreview()
{
    TCHAR SkinPreviewPath[MAX_PATH];
    HWND hWnd = ::GetDlgItem(this->hWndDialog,IDC_STATIC_SKIN_PREVIEW);
    BOOL PreviewAvailable = FALSE;

    if (this->pApplicationOptions->BrowserToolBarOptions.GetCurrentSkinPreviewPath(SkinPreviewPath,_countof(SkinPreviewPath)))
    {
        // assume show preview is visible
        ::ShowWindow(hWnd,SW_NORMAL);

        if (this->bGdiPlusStarted)
        {
            if (this->pBitmapPreview)
                delete this->pBitmapPreview;
            // load image
            this->pBitmapPreview = new CGdiPlusBitmap();
            if (this->pBitmapPreview->Load(SkinPreviewPath))
            {
                // set preview image
                this->pBitmapPreview->m_pBitmap->GetHBITMAP(NULL,&this->hBitmapPreview);
                ::SendMessage(hWnd,STM_SETIMAGE,(WPARAM)IMAGE_BITMAP,(LPARAM)this->hBitmapPreview);
                PreviewAvailable = TRUE;
            }
        }

    }

    if (!PreviewAvailable)
    {
        // hide preview
        ::ShowWindow(hWnd,SW_HIDE);
    }
}

void COptionsUI::LoadApplicationOptions()
{
    // select language in combo
    TCHAR Language[MAX_PATH];
    this->pApplicationOptions->GetLanguage(Language,MAX_PATH);
    HWND hWndCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_APPLICATION_LANGUAGE);
    int ItemIndex=ComboBox_FindStringExact(hWndCombo,-1,Language);
    if (ItemIndex!=CB_ERR)
        ComboBox_SetCurSel(hWndCombo,ItemIndex);
    else
        ComboBox_SetCurSel(hWndCombo,0);

    // run at startup check 
    CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_CHECK_APPLICATION_LAUNCH_AT_WINDOWS_STARTUP),this->pApplicationOptions->bRunAtStartup);

    // single / multi user radio
    BOOL bSingleUser;
    this->pApplicationOptions->GetUserMode(&bSingleUser);
    CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_APPLICATION_CONFIGURATION_TYPE_MULTIPLE_USERS),!bSingleUser);
    CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_APPLICATION_CONFIGURATION_TYPE_SINGLE_USER),bSingleUser);

    this->LoadSkinsList();
    // select current skin in combo if any
    hWndCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_SKIN);
    if (this->pApplicationOptions->bUseSkin)
    {
        ItemIndex = ComboBox_FindStringExact(hWndCombo,0,this->pApplicationOptions->SkinName);
        if (ItemIndex!=CB_ERR)
            ComboBox_SetCurSel(hWndCombo,ItemIndex);
        else
            ComboBox_SetCurSel(hWndCombo,0);

        // update skin preview
        this->UpdateSkinPreview();
    }
    else
        ComboBox_SetCurSel(hWndCombo,0);

    // launch mode
    hWndCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCH_MODE);
    ComboBox_SetCurSel(hWndCombo,this->pApplicationOptions->LaunchMode);

    this->LoadLaunchbarOptions();
}

void COptionsUI::SaveApplicationOptions()
{
    // get language infos
    HWND hWndCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_APPLICATION_LANGUAGE);
    int ItemIndex=ComboBox_GetCurSel(hWndCombo);
    if (ItemIndex!=CB_ERR)
    {
        TCHAR Language[MAX_PATH];
        ComboBox_GetText(hWndCombo,Language,MAX_PATH);
        this->pApplicationOptions->SetLanguage(Language);
    }

    // get startup infos
    this->pApplicationOptions->bRunAtStartup=CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_APPLICATION_LAUNCH_AT_WINDOWS_STARTUP));
    this->pApplicationOptions->SetUserMode(CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_RADIO_APPLICATION_CONFIGURATION_TYPE_SINGLE_USER)));

    // get skin infos
    hWndCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_SKIN);
    ItemIndex=ComboBox_GetCurSel(hWndCombo);
    if (ItemIndex!=CB_ERR)
    {
        if (ItemIndex==0)// no skin
        {
            this->pApplicationOptions->bUseSkin=FALSE;
            *this->pApplicationOptions->SkinName=0;
        }
        else
        {
            this->pApplicationOptions->bUseSkin=TRUE;
            ComboBox_GetText(hWndCombo,this->pApplicationOptions->SkinName,MAX_PATH);
        }
    }

    // launch bar mode
    this->pApplicationOptions->LaunchMode = this->GetSelectedLaunchMode();

    this->SaveLaunchbarOptions();
}

void COptionsUI::FillLaunchBarMonitorsCombo(int IdCombo)
{
    DWORD NbMonitors = CDesktopWorkArea::GetNumberOfDisplayMonitors();
    TCHAR Str[64];
    HWND hWnd = ::GetDlgItem(this->hWndDialog,IdCombo);
    ComboBox_ResetContent(hWnd);

    for (DWORD Cnt=1; Cnt<=NbMonitors; Cnt++)
    {
        _sntprintf(Str,_countof(Str),_T("%u"),Cnt);
        ComboBox_AddString(hWnd,Str);
    }
}
//void COptionsUI::FillLaunchBarDirectionsCombo()
//{
//    HWND hWndCombo = ::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_DIRECTION);
//    // save selection
//    int SelectedItemIndex = ComboBox_GetCurSel(hWndCombo);
//    ComboBox_ResetContent(hWndCombo);
//    ComboBox_AddString(hWndCombo,this->pLanguage->Horizontal);
//    ComboBox_AddString(hWndCombo,this->pLanguage->Vertical);
//    // restore selection
//    ComboBox_SetCurSel(hWndCombo,SelectedItemIndex);
//}
void COptionsUI::FillLaunchBarPositionsCombo(int IdCombo)
{
    HWND hWndCombo = ::GetDlgItem(this->hWndDialog,IdCombo);
    // save selection
    int SelectedItemIndex = ComboBox_GetCurSel(hWndCombo);
    ComboBox_ResetContent(hWndCombo);
    ComboBox_AddString(hWndCombo,this->pLanguage->UpLeft);
    ComboBox_AddString(hWndCombo,this->pLanguage->UpCenter);
    ComboBox_AddString(hWndCombo,this->pLanguage->UpRight);
    ComboBox_AddString(hWndCombo,this->pLanguage->Left);
    ComboBox_AddString(hWndCombo,this->pLanguage->Right);
    ComboBox_AddString(hWndCombo,this->pLanguage->BottomLeft);
    ComboBox_AddString(hWndCombo,this->pLanguage->BottomCenter);
    ComboBox_AddString(hWndCombo,this->pLanguage->BottomRight);
    // restore selection
    ComboBox_SetCurSel(hWndCombo,SelectedItemIndex);
}

void COptionsUI::FillHotCornersPositionsCombo(int IdCombo)
{
    HWND hWndCombo = ::GetDlgItem(this->hWndDialog,IdCombo);
    // save selection
    int SelectedItemIndex = ComboBox_GetCurSel(hWndCombo);
    ComboBox_ResetContent(hWndCombo);
    ComboBox_AddString(hWndCombo,this->pLanguage->UpLeft);
    ComboBox_AddString(hWndCombo,this->pLanguage->UpRight);
    ComboBox_AddString(hWndCombo,this->pLanguage->BottomLeft);
    ComboBox_AddString(hWndCombo,this->pLanguage->BottomRight);
    // restore selection
    ComboBox_SetCurSel(hWndCombo,SelectedItemIndex);
}

void COptionsUI::FillLaunchBarIconSizesCombo()
{
    HWND hWnd = ::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_ICONS_SIZE);
    ComboBox_ResetContent(hWnd);
    ComboBox_AddString(hWnd,_T("16x16"));
    ComboBox_AddString(hWnd,_T("24x24"));
    ComboBox_AddString(hWnd,_T("32x32"));
    ComboBox_AddString(hWnd,_T("48x48"));
    ComboBox_AddString(hWnd,_T("64x64"));
}
void COptionsUI::FillLaunchBarBackgroundGradiantTypesCombo(int DialogId)
{
    HWND hWndCombo=::GetDlgItem(this->hWndDialog,DialogId);
    // save selection
    int SelectedItemIndex = ComboBox_GetCurSel(hWndCombo);
    ComboBox_ResetContent(hWndCombo);
    ComboBox_AddString(hWndCombo,this->pLanguage->Horizontal);
    ComboBox_AddString(hWndCombo,this->pLanguage->Vertical);
    ComboBox_AddString(hWndCombo,this->pLanguage->Diagonal);
    ComboBox_AddString(hWndCombo,this->pLanguage->BackwardDiagonal);
    // restore selection
    ComboBox_SetCurSel(hWndCombo,SelectedItemIndex);
}
void COptionsUI::FillLaunchBarBackgroundPicturePositionsCombo(int DialogId)
{
    HWND hWndCombo = ::GetDlgItem(this->hWndDialog,DialogId);
    // save selection
    int SelectedItemIndex = ComboBox_GetCurSel(hWndCombo);
    ComboBox_ResetContent(hWndCombo);
    ComboBox_AddString(hWndCombo,this->pLanguage->Stretched);
    ComboBox_AddString(hWndCombo,this->pLanguage->Centered);
    ComboBox_AddString(hWndCombo,this->pLanguage->UpLeft);
    ComboBox_AddString(hWndCombo,this->pLanguage->UpCenter);
    ComboBox_AddString(hWndCombo,this->pLanguage->UpRight);
    ComboBox_AddString(hWndCombo,this->pLanguage->Left);
    ComboBox_AddString(hWndCombo,this->pLanguage->Right);
    ComboBox_AddString(hWndCombo,this->pLanguage->BottomLeft);
    ComboBox_AddString(hWndCombo,this->pLanguage->BottomCenter);
    ComboBox_AddString(hWndCombo,this->pLanguage->BottomRight);
    // restore selection
    ComboBox_SetCurSel(hWndCombo,SelectedItemIndex);
}


void COptionsUI::LoadLaunchbarOptions()
{
    // monitor
    ComboBox_SetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_MONITOR_INDEX),
                       this->pApplicationOptions->LaunchBarOptions.ScreenIndex
                      );

    // direction
    //ComboBox_SetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_DIRECTION),
    //                    this->pApplicationOptions->LaunchBarOptions.bVertical ? 1:0
    //                  );

    // position
    ComboBox_SetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_BACKGROUND_POSITION),
                       this->pApplicationOptions->LaunchBarOptions.Position
                      );

    // full width / size
    CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_CHECK_LAUNCHBAR_FULL_WIDTH_OR_HEIGHT),
                                    this->pApplicationOptions->LaunchBarOptions.bUseFullScreenWidhtOrHeight
                                    );

    // icon size
    // for icon width/height, GUI assume that Width=Height
    // this->pApplicationOptions->LaunchBarOptions.IconHeight not used, supposed to be equal to this->pApplicationOptions->LaunchBarOptions.IconWidth
    int Index=0;
    switch (this->pApplicationOptions->LaunchBarOptions.IconWidth)
    {
    case 16:
        Index=0;
    	break;
    case 24:
        Index=1;
        break;
    case 32:
        Index=2;
        break;
    case 48:
        Index=3;
        break;
    case 64:
        Index=4;
        break;
    }
    ComboBox_SetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_ICONS_SIZE),Index);    

    // back ground
    this->DeleteLaunchBarBrushs();

    this->LaunchBarBackgroundColorBrush=::CreateSolidBrush(this->pApplicationOptions->LaunchBarOptions.BackGroundColor);
    this->LaunchBarGradiantBackgroundColor1Brush=::CreateSolidBrush(this->pApplicationOptions->LaunchBarOptions.BackGroundGradiantColor1);
    this->LaunchBarGradiantBackgroundColor2Brush=::CreateSolidBrush(this->pApplicationOptions->LaunchBarOptions.BackGroundGradiantColor2);
    this->LaunchBarTransparentBrush=::CreateSolidBrush(this->pApplicationOptions->LaunchBarOptions.TransparentColor);

    this->hWndLaunchBarBackgroundColor=::GetDlgItem(this->hWndDialog,IDC_BUTTON_LAUNCHBAR_BACKGROUND_COLOR);
    this->hWndLaunchBarGradiantBackgroundColor1=::GetDlgItem(this->hWndDialog,IDC_BUTTON_LAUNCHBAR_BACKGROUND_COLOR1);
    this->hWndLaunchBarGradiantBackgroundColor2=::GetDlgItem(this->hWndDialog,IDC_BUTTON_LAUNCHBAR_BACKGROUND_COLOR2);
    this->hWndLaunchBarTransparentColor=::GetDlgItem(this->hWndDialog,IDC_BUTTON_LAUNCHBAR_TRANSPARENT_COLOR);

    // always do selection (allow to keep information when going back to a previous mode)
    ComboBox_SetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_BACKGROUND_GRADIANT_TYPE),0);
    ComboBox_SetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_BACKGROUND_PICTURE_POSITION),0);
    switch(this->pApplicationOptions->LaunchBarOptions.BackgroundType)
    {
    default:
    case CTransparentToolbar::BACKGROUND_SOLID_BRUSH:
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_COLOR),TRUE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_GRADIANT),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_PICTURE),FALSE);
        break;
    case CTransparentToolbar::BACKGROUND_LINEAR_GRADIENT_MODE_HORIZONTAL:
    case CTransparentToolbar::BACKGROUND_LINEAR_GRADIENT_MODE_VERTICAL:
    case CTransparentToolbar::BACKGROUND_LINEAR_GRADIENT_MODE_FORWARD_DIAGONAL:
    case CTransparentToolbar::BACKGROUND_LINEAR_GRADIENT_MODE_BACKWARD_DIAGONAL:
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_COLOR),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_GRADIANT),TRUE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_PICTURE),FALSE);
        ComboBox_SetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_BACKGROUND_GRADIANT_TYPE),
                            this->pApplicationOptions->LaunchBarOptions.BackgroundType-CTransparentToolbar::BACKGROUND_LINEAR_GRADIENT_MODE_HORIZONTAL
                            );
        break;
    case CTransparentToolbar::BACKGROUND_IMAGE_POSITION_STRETCHED:
    case CTransparentToolbar::BACKGROUND_IMAGE_POSITION_CENTER:
    case CTransparentToolbar::BACKGROUND_IMAGE_POSITION_UP_LEFT:
    case CTransparentToolbar::BACKGROUND_IMAGE_POSITION_UP_CENTER:
    case CTransparentToolbar::BACKGROUND_IMAGE_POSITION_UP_RIGHT:
    case CTransparentToolbar::BACKGROUND_IMAGE_POSITION_LEFT:
    case CTransparentToolbar::BACKGROUND_IMAGE_POSITION_RIGHT:
    case CTransparentToolbar::BACKGROUND_IMAGE_POSITION_BOTTOM_LEFT:
    case CTransparentToolbar::BACKGROUND_IMAGE_POSITION_BOTTOM_CENTER:
    case CTransparentToolbar::BACKGROUND_IMAGE_POSITION_BOTTOM_RIGHT:
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_COLOR),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_GRADIANT),FALSE);
        CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_PICTURE),TRUE);
        ComboBox_SetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_BACKGROUND_PICTURE_POSITION),
                            this->pApplicationOptions->LaunchBarOptions.BackgroundType-CTransparentToolbar::BACKGROUND_IMAGE_POSITION_STRETCHED
                            );
        break;
    }

    // background path
    ::SetDlgItemText(this->hWndDialog,IDC_EDIT_LAUNCHBAR_BACKGROUND_PATH,this->pApplicationOptions->LaunchBarOptions.BackGroundImage);

    // use transparent color
    CDialogHelper::SetButtonCheckState(::GetDlgItem(this->hWndDialog,IDC_CHECK_LAUNCHBAR_USE_TRANSPARENT_COLOR),
                                        this->pApplicationOptions->LaunchBarOptions.bUseTransparentColor
                                        );

    // opacity
    ::SendMessage(::GetDlgItem(this->hWndDialog,IDC_SLIDER_LAUNCHBAR_TRANSPARENCY),TBM_SETPOS,TRUE,this->pApplicationOptions->LaunchBarOptions.OpacityPercent);

    // active area thickness
    ::SetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_THICKNESS,this->pApplicationOptions->LaunchBarOptions.ActiveZoneSizeInPixel,FALSE);

    // min width / height
    ::SetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_MIN_WIDTH,this->pApplicationOptions->LaunchBarOptions.MinWidth,FALSE);
    ::SetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_MIN_HEIGHT,this->pApplicationOptions->LaunchBarOptions.MinHeight,FALSE);

    // margins
    ::SetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_MARGIN_LEFT,this->pApplicationOptions->LaunchBarOptions.Margins.left,TRUE);
    ::SetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_MARGIN_RIGHT,this->pApplicationOptions->LaunchBarOptions.Margins.right,TRUE);
    ::SetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_MARGIN_TOP,this->pApplicationOptions->LaunchBarOptions.Margins.top,TRUE);
    ::SetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_MARGIN_BOTTOM,this->pApplicationOptions->LaunchBarOptions.Margins.bottom,TRUE);
    
}

void COptionsUI::SaveLaunchbarOptions()
{
    // monitor (from 1 based index to 0 based index)
    this->pApplicationOptions->LaunchBarOptions.ScreenIndex = ComboBox_GetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_MONITOR_INDEX));
    // direction
    this->pApplicationOptions->LaunchBarOptions.bVertical = ComboBox_GetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_DIRECTION));

    // position
    this->pApplicationOptions->LaunchBarOptions.Position = (CLaunchBarOptions::tagPosition)ComboBox_GetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_BACKGROUND_POSITION));

    // full width / size
    this->pApplicationOptions->LaunchBarOptions.bUseFullScreenWidhtOrHeight = CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_LAUNCHBAR_FULL_WIDTH_OR_HEIGHT));

    // icon size
    // for icon width/height, GUI assume that Width=Height
    switch (ComboBox_GetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_ICONS_SIZE)))
    {
    case 0:
        this->pApplicationOptions->LaunchBarOptions.IconHeight = 16;
        this->pApplicationOptions->LaunchBarOptions.IconWidth  = 16;
        break;
    case 1:
        this->pApplicationOptions->LaunchBarOptions.IconHeight = 24;
        this->pApplicationOptions->LaunchBarOptions.IconWidth  = 24;
        break;
    case 2:
        this->pApplicationOptions->LaunchBarOptions.IconHeight = 32;
        this->pApplicationOptions->LaunchBarOptions.IconWidth  = 32;
        break;
    case 3:
        this->pApplicationOptions->LaunchBarOptions.IconHeight = 48;
        this->pApplicationOptions->LaunchBarOptions.IconWidth  = 48;
        break;
    case 4:
        this->pApplicationOptions->LaunchBarOptions.IconHeight = 64;
        this->pApplicationOptions->LaunchBarOptions.IconWidth  = 64;
        break;
    }

    // back ground
    if (CDialogHelper::IsButtonChecked(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_COLOR))
        this->pApplicationOptions->LaunchBarOptions.BackgroundType = CTransparentToolbar::BACKGROUND_SOLID_BRUSH;
    else if (CDialogHelper::IsButtonChecked(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_GRADIANT))
    {
        this->pApplicationOptions->LaunchBarOptions.BackgroundType = (CTransparentToolbar::tagBackgroundType) 
                                                                        (CTransparentToolbar::BACKGROUND_LINEAR_GRADIENT_MODE_HORIZONTAL
                                                                         +ComboBox_GetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_BACKGROUND_GRADIANT_TYPE))
                                                                         );
    }
    else if (CDialogHelper::IsButtonChecked(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_PICTURE))
    {
        this->pApplicationOptions->LaunchBarOptions.BackgroundType = (CTransparentToolbar::tagBackgroundType) 
                                                                        (CTransparentToolbar::BACKGROUND_IMAGE_POSITION_STRETCHED
                                                                        +ComboBox_GetCurSel(::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_BACKGROUND_PICTURE_POSITION))
                                                                        );
    }


    this->pApplicationOptions->LaunchBarOptions.BackGroundColor=this->GetBrushColor(this->LaunchBarBackgroundColorBrush);
    this->pApplicationOptions->LaunchBarOptions.BackGroundGradiantColor1=this->GetBrushColor(this->LaunchBarGradiantBackgroundColor1Brush);
    this->pApplicationOptions->LaunchBarOptions.BackGroundGradiantColor2=this->GetBrushColor(this->LaunchBarGradiantBackgroundColor2Brush);
    this->pApplicationOptions->LaunchBarOptions.TransparentColor=this->GetBrushColor(this->LaunchBarTransparentBrush);

    // background path
    ::GetDlgItemText(this->hWndDialog,IDC_EDIT_LAUNCHBAR_BACKGROUND_PATH,this->pApplicationOptions->LaunchBarOptions.BackGroundImage,_countof(this->pApplicationOptions->LaunchBarOptions.BackGroundImage));

    // use transparent color
    this->pApplicationOptions->LaunchBarOptions.bUseTransparentColor = CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_CHECK_LAUNCHBAR_USE_TRANSPARENT_COLOR));

    // opacity
    this->pApplicationOptions->LaunchBarOptions.OpacityPercent = (BYTE)::SendMessage(::GetDlgItem(this->hWndDialog,IDC_SLIDER_LAUNCHBAR_TRANSPARENCY),TBM_GETPOS,0,0);

    BOOL bSuccess;
    UINT TmpValue;
    // active area thickness
    TmpValue = ::GetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_THICKNESS,&bSuccess,FALSE);
    if (bSuccess)
        this->pApplicationOptions->LaunchBarOptions.ActiveZoneSizeInPixel = TmpValue;

    // min width / height
    TmpValue = ::GetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_MIN_WIDTH,&bSuccess,FALSE);
    if (bSuccess)
        this->pApplicationOptions->LaunchBarOptions.MinWidth = TmpValue;
    TmpValue = ::GetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_MIN_HEIGHT,&bSuccess,FALSE);
    if (bSuccess)
        this->pApplicationOptions->LaunchBarOptions.MinHeight = TmpValue;

    // margins
    TmpValue = ::GetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_MARGIN_LEFT,&bSuccess,FALSE);
    if (bSuccess)
        this->pApplicationOptions->LaunchBarOptions.Margins.left = TmpValue;
    TmpValue = ::GetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_MARGIN_RIGHT,&bSuccess,FALSE);
    if (bSuccess)
        this->pApplicationOptions->LaunchBarOptions.Margins.right = TmpValue;
    TmpValue = ::GetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_MARGIN_TOP,&bSuccess,FALSE);
    if (bSuccess)
        this->pApplicationOptions->LaunchBarOptions.Margins.top = TmpValue;
    TmpValue = ::GetDlgItemInt(this->hWndDialog,IDC_EDIT_LAUNCHBAR_MARGIN_BOTTOM,&bSuccess,FALSE);
    if (bSuccess)
        this->pApplicationOptions->LaunchBarOptions.Margins.bottom = TmpValue;
}

CApplicationOptions::tagLaunchMode COptionsUI::GetSelectedLaunchMode()
{
    HWND hWndCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCH_MODE);
    INT32 ItemIndex=ComboBox_GetCurSel(hWndCombo);
    if (ItemIndex==CB_ERR)
        return CApplicationOptions::LaunchMode_NOTIFY_ICON;
    else
        return (CApplicationOptions::tagLaunchMode)ItemIndex;
}

CSingleFolderOptions::tagFolderPresentationType COptionsUI::GetGUIFolderPresentationType()
{
    CSingleFolderOptions::tagFolderPresentationType FolderPresentationType = CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_DESKTOP_BROWSER;
    if (CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_DESKTOP)))
    {
        FolderPresentationType=CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_DESKTOP_BROWSER;
    }
    else if(CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_BROWSER)))
    {
        FolderPresentationType=CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_BROWSER;
    }
    else if(CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_MENU)))
    {
        FolderPresentationType=CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_SINGLE_MENU;
    }
    else if(CDialogHelper::IsButtonChecked(::GetDlgItem(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_MULTIPLE_FOLDER_MENU)))
    {
        FolderPresentationType=CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU;
    }

    return FolderPresentationType;
}

void COptionsUI::RemoveSelectedFoldersFromMultipleFolder()
{
    // listview single selection must be assumed
    int SelectedIndex=this->pListview->GetSelectedIndex();
    if (SelectedIndex<0)
        return;
    this->pListview->RemoveItem(SelectedIndex);
    if (this->LocalMultiplePath.size()!=0)
        this->LocalMultiplePath.erase(this->LocalMultiplePath.begin()+SelectedIndex);
}
void COptionsUI::AddFolderToMultipleFolder()
{
    CFolderMenu::MULTIPLE_PATHS_DATA Data={0};
    this->GetPath(Data.Path,_countof(Data.Path));
    Data.MergeToSpecificFolder=CDialogHelper::IsButtonChecked(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER);
    if (Data.MergeToSpecificFolder)
    {
        ::GetDlgItemText(this->hWndDialog,IDC_EDIT_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER,Data.MergedSpecificFolderName,_countof(Data.MergedSpecificFolderName));
        if (*Data.MergedSpecificFolderName==0)
        {
            this->ReportError(this->pLanguage->EmptyMergedFolderName);
            return;
        }
    }
    this->LocalMultiplePath.push_back(Data);
    this->AddFolderToMultipleFolderListview(&Data);
}

void COptionsUI::MoveFolderInsideMultipleFolderUp()
{
    // listview single selection must be assumed
    int SelectedIndex=this->pListview->GetSelectedIndex();

    // if no item selected
    if (SelectedIndex==-1)
        return;

    // if first item selected
    if (SelectedIndex==0)
        return;

    // swap items
    CFolderMenu::MULTIPLE_PATHS_DATA Data;
    Data=this->LocalMultiplePath[SelectedIndex-1];
    this->LocalMultiplePath[SelectedIndex-1]=this->LocalMultiplePath[SelectedIndex];
    this->LocalMultiplePath[SelectedIndex]=Data;

    LPVOID IconIndex=(LPVOID)-1;
    this->pListview->GetItemUserData(SelectedIndex-1,&IconIndex);
    this->pListview->RemoveItem(SelectedIndex-1);
    
    TCHAR sz[MAX_PATH];
    CSingleFolderOptions::GetPath(Data.Path,sz,_countof(sz));
    TCHAR* psz=sz;
    this->pListview->AddItemAndSubItems(1,&psz,SelectedIndex,FALSE,IconIndex);

    // if path is a system one
    if (CSingleFolderOptions::IsCSIDL(Data.Path))
        this->pListview->SetItemText(SelectedIndex,1,this->pLanguage->SystemSpecialPath);
    else
        this->pListview->SetItemText(SelectedIndex,1,this->pLanguage->LocalOrRelativePath);

    // add specific merged folder name
    if (Data.MergeToSpecificFolder)
        this->pListview->SetItemText(SelectedIndex,2,Data.MergedSpecificFolderName);

    // set icon if possible
    if (IconIndex>=0)
        this->pListview->SetItemIconIndex(SelectedIndex,(int)IconIndex);

    // redraw to restore lines colors
    CDialogHelper::Redraw(this->pListview->GetControlHandle());
}


void COptionsUI::MoveFolderInsideMultipleFolderDown()
{
    // listview single selection must be assumed
    int SelectedIndex=this->pListview->GetSelectedIndex();

    // if no item selected
    if (SelectedIndex==-1)
        return;

    // if last item selected
    if (SelectedIndex==this->pListview->GetItemCount()-1)
        return;

    // swap items
    CFolderMenu::MULTIPLE_PATHS_DATA Data;
    Data=this->LocalMultiplePath[SelectedIndex+1];
    this->LocalMultiplePath[SelectedIndex+1]=this->LocalMultiplePath[SelectedIndex];
    this->LocalMultiplePath[SelectedIndex]=Data;

    LPVOID IconIndex=(LPVOID)-1;
    this->pListview->GetItemUserData(SelectedIndex+1,&IconIndex);
    this->pListview->RemoveItem(SelectedIndex+1);

    TCHAR sz[MAX_PATH];
    CSingleFolderOptions::GetPath(Data.Path,sz,_countof(sz));
    TCHAR* psz=sz;
    this->pListview->AddItemAndSubItems(1,&psz,SelectedIndex,FALSE,IconIndex);

    // if path is a system one
    if (CSingleFolderOptions::IsCSIDL(Data.Path))
        this->pListview->SetItemText(SelectedIndex,1,this->pLanguage->SystemSpecialPath);
    else
        this->pListview->SetItemText(SelectedIndex,1,this->pLanguage->LocalOrRelativePath);

    // add specific merged folder name
    if (Data.MergeToSpecificFolder)
        this->pListview->SetItemText(SelectedIndex,2,Data.MergedSpecificFolderName);

    // set icon if possible
    if (IconIndex>=0)
        this->pListview->SetItemIconIndex(SelectedIndex,(int)IconIndex);

    // redraw to restore lines colors
    CDialogHelper::Redraw(this->pListview->GetControlHandle());
}
void COptionsUI::AddFolderToMultipleFolderListview(CFolderMenu::MULTIPLE_PATHS_DATA* pData)
{
    TCHAR Path[MAX_PATH];
    int ItemIndex;
    int IconIndex=-1;

    // get path
    CSingleFolderOptions::GetPath(pData->Path,Path,_countof(Path));


    // extract path icon
    // extract associated icon
    HICON hIcon=NULL;
    SHFILEINFO shfi;
    memset(&shfi,0,sizeof(shfi));
    if (::SHGetFileInfo(Path,
                        FILE_ATTRIBUTE_NORMAL,
                        &shfi,
                        sizeof(shfi),
                        SHGFI_ICON | SHGFI_SMALLICON
                        )
        )
    {
        // fill icon vars
        hIcon = shfi.hIcon;
    }
    if (hIcon)
    {
        this->MultipleMenuListviewLoadedIconsToBeDestroyed.push_back(hIcon);
    }
    else // avoid to get first inserted icon by default, use default icon instead
    {
        hIcon=this->pQuickWayToFolders->hDefaultIcon;
    }

    
    IconIndex=this->pListview->AddIcon(CListview::ImageListSmall,hIcon);

    // add path content to list view
    ItemIndex=this->pListview->AddItem(Path,(LPVOID)IconIndex,FALSE);

    // if path is a system one
    if (CSingleFolderOptions::IsCSIDL(pData->Path))
        this->pListview->SetItemText(ItemIndex,1,this->pLanguage->SystemSpecialPath);
    else
        this->pListview->SetItemText(ItemIndex,1,this->pLanguage->LocalOrRelativePath);

    // add specific merged folder name
    if (pData->MergeToSpecificFolder)
        this->pListview->SetItemText(ItemIndex,2,pData->MergedSpecificFolderName);

    // set icon if possible
    if (IconIndex>=0)
        this->pListview->SetItemIconIndex(ItemIndex,IconIndex);
}

void COptionsUI::OnAddFolder()
{
    CGenericQuickWayToFolderObject* pGenericQuickWayToFolderObject=new CGenericQuickWayToFolderObject();
    this->FoldersArray.push_back(pGenericQuickWayToFolderObject);
    this->AddFolderToTreeView(pGenericQuickWayToFolderObject,TVI_LAST,TRUE,TRUE);
}
void COptionsUI::OnRemoveFolder()
{
    
    if (!this->pCurrentFolderOptions)
    {
        this->ReportError(this->pLanguage->NoFolderSelected);
        return;
    }

    CGenericQuickWayToFolderObject* pFolderOptions=this->pCurrentFolderOptions;

    for (size_t Cnt=0;Cnt<this->FoldersArray.size();Cnt++)
    {
        if (this->FoldersArray[Cnt]==pFolderOptions)
        {
            // reset pCurrentFolderOptions
            this->pCurrentFolderOptions=NULL;

            // remove item from treeview (can affect this->pCurrentFolderOptions)
            this->pTreeview->Remove(this->hCurrentFolderOptionsTreeItem);

            // remove item from array
            if (this->FoldersArray.size()!=0)
                this->FoldersArray.erase(this->FoldersArray.begin()+Cnt);

            // assume container won't be destroy (see CGenericQuickWayToFolderObject destructor)
            pFolderOptions->pContainer=NULL;

            // delete item
            delete pFolderOptions;

            break;
        }
    }
}
void COptionsUI::OnMoveFolderUp()
{
    if (!this->pCurrentFolderOptions)
    {
        this->ReportError(this->pLanguage->NoFolderSelected);
            return;
    }

    CGenericQuickWayToFolderObject* pFolderOptions=this->pCurrentFolderOptions;

    HTREEITEM hPreviousItem=this->pTreeview->GetPrevious(this->hCurrentFolderOptionsTreeItem);
    if (!hPreviousItem)
        return;

    HTREEITEM hInsertAfter=this->pTreeview->GetPrevious(hPreviousItem);
    if (!hInsertAfter)
        hInsertAfter=TVI_FIRST;

    // swap items inside Folder vector
    for (size_t Cnt=0;Cnt<this->FoldersArray.size();Cnt++)
    {
        if (this->FoldersArray[Cnt]==pFolderOptions)
        {
            if (Cnt==0)
            {
#ifdef _DEBUG
                // should not occur
                if (::IsDebuggerPresent())
                    ::DebugBreak();
#endif // _DEBUG
                return;
            }

            this->FoldersArray[Cnt]=this->FoldersArray[Cnt-1];
            this->FoldersArray[Cnt-1]=pFolderOptions;

            // remove current tree item (can affect this->pCurrentFolderOptions)
            this->pTreeview->Remove(this->hCurrentFolderOptionsTreeItem);

            // add item on treeview
            this->AddFolderToTreeView(pFolderOptions,hInsertAfter,TRUE,TRUE);

            break;
        }
    }

}
void COptionsUI::OnMoveFolderDown()
{
    if (!this->pCurrentFolderOptions)
    {
        this->ReportError(this->pLanguage->NoFolderSelected);
            return;
    }

    CGenericQuickWayToFolderObject* pFolderOptions=this->pCurrentFolderOptions;

    HTREEITEM hNextItem=this->pTreeview->GetNext(this->hCurrentFolderOptionsTreeItem);
    if (!hNextItem)
        return;

    // swap items inside Folder vector
    for (size_t Cnt=0;Cnt<this->FoldersArray.size();Cnt++)
    {
        if (this->FoldersArray[Cnt]==pFolderOptions)
        {
            if (Cnt==this->FoldersArray.size())
            {
#ifdef _DEBUG
                // should not occur
                if (::IsDebuggerPresent())
                    ::DebugBreak();
#endif // _DEBUG
                return;
            }

            this->FoldersArray[Cnt]=this->FoldersArray[Cnt+1];
            this->FoldersArray[Cnt+1]=pFolderOptions;

            // remove current tree item (can affect this->pCurrentFolderOptions)
            this->pTreeview->Remove(this->hCurrentFolderOptionsTreeItem);

            // add item on treeview
            this->AddFolderToTreeView(pFolderOptions,hNextItem,TRUE,TRUE);

            break;
        }
    }

}

void COptionsUI::LoadLanguage()
{
    HWND hWndCombo;

    if (this->hItemLaunchBar)
        this->pTreeview->SetItemText(this->hItemLaunchBar,this->pLanguage->LaunchBar);

    hWndCombo = ::GetDlgItem(this->hWndDialog,IDC_COMBO_SKIN);
    // save selection
    int SelectedItemIndex = ComboBox_GetCurSel(hWndCombo);
    // change string
    ComboBox_DeleteString(hWndCombo,0);
    ComboBox_InsertString(hWndCombo,0,this->pLanguage->None);
    // restore selection
    ComboBox_SetCurSel(hWndCombo,SelectedItemIndex);

    // warning respect enum order + assume combo is not sorted
    this->FillLaunchBarBackgroundGradiantTypesCombo(IDC_COMBO_CUSTOMIZATION_BACKGROUND_GRADIANT_TYPE);

    // warning respect enum order + assume combo is not sorted
    hWndCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_Z_ORDER);
    // save selection
    SelectedItemIndex = ComboBox_GetCurSel(hWndCombo);
    ComboBox_ResetContent(hWndCombo);
    ComboBox_AddString(hWndCombo,this->pLanguage->Normal);
    ComboBox_AddString(hWndCombo,this->pLanguage->TopMost);
    ComboBox_AddString(hWndCombo,this->pLanguage->ZBottom);
    // restore selection
    ComboBox_SetCurSel(hWndCombo,SelectedItemIndex);

    // warning respect enum order + assume combo is not sorted
    this->FillLaunchBarBackgroundPicturePositionsCombo(IDC_COMBO_CUSTOMIZATION_BACKGROUND_PICTURE_POSITION);

    // warning respect enum order + assume combo is not sorted
    hWndCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_BROWSE_OPTIONS_VIEW_TYPE);
    // save selection
    SelectedItemIndex = ComboBox_GetCurSel(hWndCombo);
    ComboBox_ResetContent(hWndCombo);
    COsVersion OsVersion;
    SIZE_T Cnt;
    if ( OsVersion.IsSevenOrNewer() )
    {
        // vista / seven
        ComboBox_AddString(hWndCombo,this->pLanguage->Icons);
        ComboBox_AddString(hWndCombo,this->pLanguage->SmallIcons);
        ComboBox_AddString(hWndCombo,this->pLanguage->List);
        ComboBox_AddString(hWndCombo,this->pLanguage->Detail);
        ComboBox_AddString(hWndCombo,this->pLanguage->Tiles);
        ComboBox_AddString(hWndCombo,this->pLanguage->Medium);
        ComboBox_AddString(hWndCombo,this->pLanguage->Large);
        ComboBox_AddString(hWndCombo,this->pLanguage->ExtraLarge);
        Cnt = 0;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_ICONS;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_SMALL_ICONS;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_LIST;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_DETAIL;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_TILES;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_MEDIUM;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_LARGE;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_EXTRA_LARGE;
    }
    else
    {
        // xp
        ComboBox_AddString(hWndCombo,this->pLanguage->Icons);
        ComboBox_AddString(hWndCombo,this->pLanguage->SmallIcons);
        ComboBox_AddString(hWndCombo,this->pLanguage->List);
        ComboBox_AddString(hWndCombo,this->pLanguage->Detail);
        ComboBox_AddString(hWndCombo,this->pLanguage->Thumbnails);
        ComboBox_AddString(hWndCombo,this->pLanguage->Tiles);
        ComboBox_AddString(hWndCombo,this->pLanguage->ThumbStrip);
        Cnt = 0;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_ICONS;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_SMALL_ICONS;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_LIST;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_DETAIL;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_THUMBNAILS;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_TILES;
        this->ComboViewTypeIndexes[Cnt++] = CShellBrowser::SHELL_BROWSER_VIEW_THUMBSTRIP;
    }
    // restore selection
    ComboBox_SetCurSel(hWndCombo,SelectedItemIndex);

    // update toolbar text
    if (this->pToolbar)
    {
        this->pToolbar->ReplaceText(IDC_TOOLBAR_BUTTON_ADD_QUICKWAY_TO_FOLDER,this->pLanguage->Add);
        this->pToolbar->ReplaceToolTipText(IDC_TOOLBAR_BUTTON_ADD_QUICKWAY_TO_FOLDER,this->pLanguage->Add);

        this->pToolbar->ReplaceText(IDC_TOOLBAR_BUTTON_REMOVE_QUICKWAY_TO_FOLDER,this->pLanguage->Remove);
        this->pToolbar->ReplaceToolTipText(IDC_TOOLBAR_BUTTON_REMOVE_QUICKWAY_TO_FOLDER,this->pLanguage->Remove);

        this->pToolbar->ReplaceText(IDC_TOOLBAR_BUTTON_MOVE_QUICKWAY_TO_FOLDER_UP,this->pLanguage->MoveUp);
        this->pToolbar->ReplaceToolTipText(IDC_TOOLBAR_BUTTON_MOVE_QUICKWAY_TO_FOLDER_UP,this->pLanguage->MoveUp);

        this->pToolbar->ReplaceText(IDC_TOOLBAR_BUTTON_MOVE_QUICKWAY_TO_FOLDER_DOWN,this->pLanguage->MoveDown);
        this->pToolbar->ReplaceToolTipText(IDC_TOOLBAR_BUTTON_MOVE_QUICKWAY_TO_FOLDER_DOWN,this->pLanguage->MoveDown);
    }

    // update treeview nodes
    if (this->pTreeview)
    {
        this->pTreeview->SetItemText(this->hItemApplicationOptions,this->pLanguage->Options);
        this->pTreeview->SetItemText(this->hItemFolders,this->pLanguage->Folders);

        HTREEITEM hItemFolder;
        HTREEITEM hItem;
        LPVOID UserData;
        tagTreeViewNodeType TreeViewNodeType;
        CGenericQuickWayToFolderObject* pTmpFolderOptions;

        // For each folder item
        for (hItemFolder=this->pTreeview->GetChild(this->hItemFolders);hItemFolder!=NULL;hItemFolder=this->pTreeview->GetNext(hItemFolder))
        {

            this->pTreeview->GetItemUserData(hItemFolder,&UserData);
            pTmpFolderOptions=(CGenericQuickWayToFolderObject*)UserData;
            if (pTmpFolderOptions->FolderPresentationType==CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU)
            {
                //put "Multiple Folder Menu" in path for displayed name in treeview
                _tcscpy(pTmpFolderOptions->Path,this->pLanguage->MultipleFoldersMenu);
                this->pTreeview->SetItemText(hItemFolder,this->pLanguage->MultipleFoldersMenu);
            }

            // for each folder item option
            for (hItem=this->pTreeview->GetChild(hItemFolder);hItem!=NULL;hItem=this->pTreeview->GetNext(hItem))
            {
                this->pTreeview->GetItemUserData(hItem,&UserData);
                TreeViewNodeType=(tagTreeViewNodeType)(int)UserData;
                switch(TreeViewNodeType)
                {
                case TreeViewNodeType_FOLDER_FILTERS:
                    this->pTreeview->SetItemText(hItem,this->pLanguage->FileFilters);
                    break;
                case TreeViewNodeType_FOLDER_CUSTOMIZATION:
                    this->pTreeview->SetItemText(hItem,this->pLanguage->Customization);
                    break;
                case TreeViewNodeType_FOLDER_BROWSING_OPTION:
                    this->pTreeview->SetItemText(hItem,this->pLanguage->BrowsingOptions);
                    break;
                }
            }
        }
    }

    // windows caption
    ::SetWindowText(this->hWndDialog,this->pLanguage->Options);

    // update GUI
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_GROUP_FOLDER_TYPE,this->pLanguage->ShortcutType);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_DESKTOP,this->pLanguage->Desktop);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_BROWSER,this->pLanguage->Browser);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_MENU,this->pLanguage->Menu);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_FOLDER_TYPE_MULTIPLE_FOLDER_MENU,this->pLanguage->MultipleFoldersMenu);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_GROUP_MULTIPLE_FOLDERS,this->pLanguage->MultipleFoldersMenu);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER,this->pLanguage->MergeToSpecificFolder);
    ::SetDlgItemText(this->hWndDialog,IDC_BUTTON_MULTIPLE_FOLDERS_MENU_ADD,this->pLanguage->Add);
    ::SetDlgItemText(this->hWndDialog,IDC_BUTTON_MULTIPLE_FOLDERS_MENU_UP,this->pLanguage->MoveUp);
    ::SetDlgItemText(this->hWndDialog,IDC_BUTTON_MULTIPLE_FOLDERS_MENU_DOWN,this->pLanguage->MoveDown);
    ::SetDlgItemText(this->hWndDialog,IDC_BUTTON_MULTIPLE_FOLDERS_MENU_REMOVE,this->pLanguage->Remove);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_GROUP_PATH,this->pLanguage->Path);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_PATH_SPECIAL_FOLDER_LOCATION,this->pLanguage->SystemSpecialPath);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_PATH_LOCAL_PATH,this->pLanguage->LocalOrRelativePath);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_GROUP_ASSOCIATED_ICON,this->pLanguage->AssociatedIcon);
    ::SetDlgItemText(this->hWndDialog,IDOK,this->pLanguage->Ok);
    ::SetDlgItemText(this->hWndDialog,IDCANCEL,this->pLanguage->Cancel);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_GROUP_CUSTOMIZATION,this->pLanguage->Customization);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_CUSTOMIZATION_TEXT_COLOR,this->pLanguage->TextColor);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_CUSTOMIZATION_TEXT_BACKGROUND_COLOR,this->pLanguage->BackgroundColor);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_CUSTOMIZATION_BACKGROUND,this->pLanguage->Background);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_NONE,this->pLanguage->None);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_COLOR,this->pLanguage->Color);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_GRADIANT,this->pLanguage->Gradiant);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_CUSTOMIZATION_BACKGROUND_PICTURE,this->pLanguage->Picture);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_COLOR1,this->pLanguage->Color1);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_COLOR2,this->pLanguage->Color2);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_TYPE,this->pLanguage->Type);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_POSITION,this->pLanguage->Position);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_PATH,this->pLanguage->Path);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_GROUP_FILTERS,this->pLanguage->FileFilters);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_INCLUSION_FILTERS,this->pLanguage->InclusionFilters);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_EXCLUSION_FILTERS,this->pLanguage->ExclusionFilters);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_GROUP_APPLICATION_OPTIONS,this->pLanguage->ApplicationOptions);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_APPLICATION_CONFIGURATION_TYPE,this->pLanguage->ConfigurationType);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_APPLICATION_CONFIGURATION_TYPE_SINGLE_USER,this->pLanguage->SingleUser);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_APPLICATION_CONFIGURATION_TYPE_MULTIPLE_USERS,this->pLanguage->MultipleUsers);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_APPLICATION_LAUNCH_AT_WINDOWS_STARTUP,this->pLanguage->LaunchAtWindowsStartup);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCH_MODE,this->pLanguage->LaunchMode);

    if (this->pListview)
    {
        this->pListview->SetColumnName(0,pLanguage->Folders);
        this->pListview->SetColumnName(1,pLanguage->Type);
        this->pListview->SetColumnName(2,pLanguage->Merged);
    }

    // warning respect enum order + assume combo is not sorted
    hWndCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCH_MODE);
    // save selection
    SelectedItemIndex = ComboBox_GetCurSel(hWndCombo);
    ComboBox_ResetContent(hWndCombo);
    ComboBox_AddString(hWndCombo,this->pLanguage->NotifyIcon);
    ComboBox_AddString(hWndCombo,this->pLanguage->LaunchBar);
    // restore selection
    ComboBox_SetCurSel(hWndCombo,SelectedItemIndex);


    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_APPLICATION_LANGUAGE,this->pLanguage->Language);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_SKIN,this->pLanguage->Skin);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_GROUP_BROWSE,this->pLanguage->BrowsingOptions);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_INCLUDE_SUBFOLDERS,this->pLanguage->IncludeSubfolders);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_ALLOW_SUBFOLDER_NAVIGATION,this->pLanguage->AllowSubfoldersNavigation);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_ALLOW_SUBFOLDER_LINK_NAVIGATION,this->pLanguage->AllowSubfoldersLinksNavigation);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_MENU_HIDE_FILE_EXTENSION,this->pLanguage->HideFilesExtensions);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_MENU_HIDE_SHORTCUTS_OVERLAY,this->pLanguage->HideShortcutsOverlay);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_MERGE_MENUS_IN_POPUPMENU,this->pLanguage->MergeMenus);
    // warning respect enum order + assume combo is not sorted
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_FOLDERS_POSITIONS_IN_POPUPMENU,this->pLanguage->FoldersPositionInPopupMenu);
    hWndCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_FOLDERS_POSITIONS_IN_POPUPMENU);
    // save selection
    SelectedItemIndex = ComboBox_GetCurSel(hWndCombo);
    ComboBox_ResetContent(hWndCombo);
    ComboBox_AddString(hWndCombo,this->pLanguage->Top);
    ComboBox_AddString(hWndCombo,this->pLanguage->Bottom);
    ComboBox_AddString(hWndCombo,this->pLanguage->MixedWithFiles);
    // restore selection
    ComboBox_SetCurSel(hWndCombo,SelectedItemIndex);

    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_REMEMBER_LAST_POSITION,this->pLanguage->RememberLastPosition);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_SINGLE_CLICK_SELECTION,this->pLanguage->SingleClickSelection);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_AUTO_HIDE_ON_EXECUTION,this->pLanguage->AutoHideOnExecution);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_VIEW_TYPE,this->pLanguage->ViewType);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_Z_ORDER,this->pLanguage->ZOrder);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_HIDE_TOOLBAR,this->pLanguage->HideToolBar);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_HIDE_TOOLBAR_FOR_HOME_FOLDER,this->pLanguage->HideToolBarForHomeFolder);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_SAVE_ICONS_POSITIONS_FOR_HOME_FOLDER,this->pLanguage->SaveIconsPositionsForHomeFolder);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_SHOW_AT_STARTUP,this->pLanguage->ShowAtStartup);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_START_MAXIMIZED,this->pLanguage->StartMaximized);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_HOT_KEY,this->pLanguage->ShowHideHotKey);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_MULTIPLE_FOLDER_DISPLAY_NAME,this->pLanguage->Name);

    // hot corner
    // this->FillLaunchBarMonitorsCombo(IDC_COMBO_HOT_CORNER_MONITOR_INDEX); // no text to translate inside
    this->FillHotCornersPositionsCombo(IDC_COMBO_HOT_CORNER_POSITION);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_ENABLE_HOT_CORNER,this->pLanguage->EnableHotCorner);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_HOT_CORNER,this->pLanguage->HotCorner);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_HOT_CORNER_MONITOR,this->pLanguage->Monitor);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_HOT_CORNER_POSITION,this->pLanguage->Position);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_HOT_CORNER_WIDTH,this->pLanguage->Width);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_HOT_CORNER_HEIGHT,this->pLanguage->Height);
  

    /////////////////////////////////////////
    // launchbar
    /////////////////////////////////////////
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR,this->pLanguage->LaunchBar);
    // this->FillLaunchBarMonitorsCombo(IDC_COMBO_LAUNCHBAR_MONITOR_INDEX);  // no text to translate inside
    // this->FillLaunchBarDirectionsCombo(); no more
    this->FillLaunchBarPositionsCombo(IDC_COMBO_LAUNCHBAR_BACKGROUND_POSITION);
    // this->FillLaunchBarIconSizesCombo(); // no text to translate inside
    this->FillLaunchBarBackgroundGradiantTypesCombo(IDC_COMBO_LAUNCHBAR_BACKGROUND_GRADIANT_TYPE);
    this->FillLaunchBarBackgroundPicturePositionsCombo(IDC_COMBO_LAUNCHBAR_BACKGROUND_PICTURE_POSITION);

    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON,this->pLanguage->LaunchBarSpecific);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR,this->pLanguage->LaunchBar);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_HOT_ICON_TEXT,this->pLanguage->OverIcon);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_DOWN_ICON_TEXT,this->pLanguage->PressedIcon);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_ICON_POSITION,this->pLanguage->Position);
    this->FillLaunchBarIconPosition();
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_ICON_SPACER_BEFORE_ICON,this->pLanguage->SpaceBeforeIcon);

    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_MONITOR,this->pLanguage->Monitor);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_DIRECTION ,this->pLanguage->Direction);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_POSITION,this->pLanguage->Position);
    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_LAUNCHBAR_FULL_WIDTH_OR_HEIGHT,this->pLanguage->UseFullWidthHeight); 
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_ICONS_SIZE,this->pLanguage->IconsSize);

    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_BACKGROUND,this->pLanguage->Background);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_COLOR,this->pLanguage->Color);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_GRADIANT,this->pLanguage->Gradiant);
    ::SetDlgItemText(this->hWndDialog,IDC_RADIO_LAUNCHBAR_BACKGROUND_PICTURE,this->pLanguage->Picture);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_COLOR1,this->pLanguage->Color1);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_COLOR2,this->pLanguage->Color2);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_TYPE,this->pLanguage->Type);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_BACKGROUND_IMAGE_POSITION,this->pLanguage->Position);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_PATH,this->pLanguage->Path);

    ::SetDlgItemText(this->hWndDialog,IDC_CHECK_LAUNCHBAR_USE_TRANSPARENT_COLOR,this->pLanguage->UseTransparentColor);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_TRANSPARENCY,this->pLanguage->Opacity);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_THICKNESS,this->pLanguage->ActiveAreaThickness);

    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_MIN_WIDTH,this->pLanguage->MinWidth);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_MIN_HEIGHT,this->pLanguage->MinHeight);
 
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_MARGINS,this->pLanguage->Margins);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_MARGIN_LEFT,this->pLanguage->Left);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_MARGIN_RIGHT,this->pLanguage->Right);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_MARGIN_TOP,this->pLanguage->Top);
    ::SetDlgItemText(this->hWndDialog,IDC_STATIC_LAUNCHBAR_MARGIN_BOTTOM,this->pLanguage->Bottom);
}

// return true if a<b
bool COptionsUI::KnownFoldersListSorting(KNOWN_FOLDERS_LIST_ITEM a,KNOWN_FOLDERS_LIST_ITEM b)
{
    return (_tcscmp(a.Path,b.Path)<0);
}

void COptionsUI::FillCSIDLCombo()
{
    HWND hWndCombo;
    // warning respect array order + assume combo is not sorted
    hWndCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_SPECIAL_FOLDER_LOCATION);
    ComboBox_ResetContent(hWndCombo);

    /*
    CBS_SORT not supported by ComboBoxEx --> use std vector
        LONG_PTR Style = ::GetWindowLongPtr(hWndCombo, GWL_STYLE);
        Style |= CBS_SORT;
        ::SetWindowLongPtr(hWndCombo, GWL_STYLE, Style);
    */


    std::vector<KNOWN_FOLDERS_LIST_ITEM> KnownFolderList;
    std::vector<KNOWN_FOLDERS_LIST_ITEM>::iterator KnownFolderListIterator;
    KNOWN_FOLDERS_LIST_ITEM Item;

    DWORD NbCSIDL=_countof(COptionsUI::CSIDL_Array);
    CSIDL_ArrayItem* pCSIDL_ArrayItem;

    for (DWORD Cnt=0;Cnt<NbCSIDL;Cnt++)
    {
        pCSIDL_ArrayItem = &COptionsUI::CSIDL_Array[Cnt];
        Item.Id = pCSIDL_ArrayItem->Id;
        Item.CSIDL_Name = pCSIDL_ArrayItem->CSIDL_Name;
        if (::SHGetFolderPath(NULL,CSIDL_FLAG_CREATE | Item.Id,NULL,SHGFP_TYPE_CURRENT,Item.Path)// use CSIDL_FLAG_CREATE to assume directory to be displayed
            ==  S_OK)
        {
            KnownFolderList.push_back(Item);
        }
    }
    std::sort(KnownFolderList.begin(), KnownFolderList.end(), COptionsUI::KnownFoldersListSorting );

    COMBOBOXEXITEM ComboboxExItem;
    ComboboxExItem.mask = CBEIF_TEXT | CBEIF_LPARAM;
    ComboboxExItem.iItem = -1; // add at the end of combo
    TCHAR Text[MAX_PATH+256]; // MAX_PATH + MAX_CSIDL_NAME + 4
    ComboboxExItem.pszText= Text;
    for (KnownFolderListIterator = KnownFolderList.begin();KnownFolderListIterator!=KnownFolderList.end();KnownFolderListIterator++)
    {
        _sntprintf(Text,_countof(Text),TEXT("%s    (%s)"),KnownFolderListIterator->Path,KnownFolderListIterator->CSIDL_Name);
        ComboboxExItem.lParam = KnownFolderListIterator->Id;
        SendMessage(hWndCombo,CBEM_INSERTITEM,0,(LPARAM)&ComboboxExItem);
    }
    ComboBox_SetCurSel(hWndCombo,0);
}

void COptionsUI::FillLaunchBarIconPosition()
{
    HWND hWndCombo;
    // warning respect array order + assume combo is not sorted
    hWndCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_LAUNCHBAR_ICON_POSITION);
    // save selection
    int SelectedItemIndex = ComboBox_GetCurSel(hWndCombo);
    ComboBox_ResetContent(hWndCombo);

    ComboBox_AddString(hWndCombo,this->pLanguage->LeftOrTop);
    ComboBox_AddString(hWndCombo,this->pLanguage->CenterOrMiddle);
    ComboBox_AddString(hWndCombo,this->pLanguage->RightOrBottom);

    // restore selection
    ComboBox_SetCurSel(hWndCombo,SelectedItemIndex);
}

void COptionsUI::FillLanguagesCombo()
{
    TCHAR Path[MAX_PATH];
    CStdFileOperations::GetAppDirectory(Path,MAX_PATH);
    _tcscat(Path,COptionsUI_LANGUAGE_SUBDIR);
    _tcscat(Path,_T("*.ini"));
    BOOL bCanceled;
    CFileSearch::Search(Path,FALSE,NULL,COptionsUI::LanguageFileFoundCallBack,this,&bCanceled,FALSE);
}

BOOL COptionsUI::LanguageFileFoundCallBack(TCHAR* Directory,WIN32_FIND_DATA* pWin32FindData,PVOID UserParam)
{
    UNREFERENCED_PARAMETER(Directory);
    COptionsUI* pOptionsUI=(COptionsUI*)UserParam;
    HWND hCombo=::GetDlgItem(pOptionsUI->hWndDialog,IDC_COMBO_APPLICATION_LANGUAGE);

    // add file name without the ini extension to combo
    CStdFileOperations::RemoveFileExt(pWin32FindData->cFileName);
    ComboBox_AddString(hCombo,pWin32FindData->cFileName);
    return TRUE;
}

void COptionsUI::OnLanguageChange()
{
    HWND hCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_APPLICATION_LANGUAGE);
    int ItemIndex=ComboBox_GetCurSel(hCombo);
    if (ItemIndex<0)
        return;

    TCHAR Path[MAX_PATH];
    TCHAR File[MAX_PATH];
    ComboBox_GetText(hCombo,File,MAX_PATH);

    CStdFileOperations::GetAppDirectory(Path,MAX_PATH);
    _tcscat(Path,COptionsUI_LANGUAGE_SUBDIR);
    _tcscat(Path,File);
    _tcscat(Path,_T(".ini"));

    CIniFileEx<CLanguage> FoldersIniFile(Path);
    CLanguage* TmpLanguage=FoldersIniFile.Get(CQuickWayToFolders_LANGUAGE_SECTION_NAME);
    if (TmpLanguage)
    {
        delete this->pLanguage;
        this->pLanguage=TmpLanguage;
        this->LoadLanguage();
    }
}

void COptionsUI::OnSkinChange()
{
    HWND hCombo;
    int ItemIndex;

    BOOL bUseSkin;
    TCHAR SkinName[MAX_PATH];

    hCombo=::GetDlgItem(this->hWndDialog,IDC_COMBO_SKIN);
    ItemIndex=ComboBox_GetCurSel(hCombo);
    if (ItemIndex!=CB_ERR)
    {
        if (ItemIndex==0)// no skin
        {
            bUseSkin=FALSE;
            *SkinName=0;
        }
        else
        {
            bUseSkin=TRUE;
            ComboBox_GetText(hCombo,SkinName,MAX_PATH);
        }
        this->pApplicationOptions->BrowserToolBarOptions.Load(bUseSkin,SkinName);
        this->UpdateSkinPreview();
    }
}

void COptionsUI::OnBrowseViewChange()
{
    this->EnableSaveIconPositionForHomeFolder();
}

void COptionsUI::OnLaunchModeChange()
{
    if (this->GetSelectedLaunchMode()==CApplicationOptions::LaunchMode_LAUNCH_BAR)
    {
        CDialogHelper::EnableGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR),TRUE);
        CDialogHelper::EnableGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),TRUE);
    }
    else
    {
        CDialogHelper::EnableGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR),FALSE);
        CDialogHelper::EnableGroup(::GetDlgItem(this->hWndDialog,IDC_STATIC_GROUP_LAUNCHBAR_ICON),FALSE);
    }
}

void COptionsUI::ChooseIcon()
{
    BOOL bUpdateHotIcon=FALSE;
    BOOL bUpdateDownIcon=FALSE;
    // if hot icon not set or equal to standard icon, mark it to be auto changed
    if ( (*this->pCurrentFolderOptions->LaunchBarAssociatedIconHotPath==0)
         || ( (_tcsicmp(this->LaunchBarAssociatedIconHotPath,this->AssociatedIconPath)==0) && (this->LaunchBarAssociatedIconHotIndex==this->AssociatedIconIndex))
       )
    {
        bUpdateHotIcon = TRUE;
    }
    // if down icon not set or equal to standard icon, mark it to be auto changed
    if ( (*this->pCurrentFolderOptions->LaunchBarAssociatedIconDownPath==0)
        || ( (_tcsicmp(this->LaunchBarAssociatedIconDownPath,this->AssociatedIconPath)==0) && (this->LaunchBarAssociatedIconDownIndex==this->AssociatedIconIndex))
        )
    {
        bUpdateDownIcon = TRUE;
    }

    // get icon
    ::PickIconDlg(this->hWndDialog,this->AssociatedIconPath,MAX_PATH,&this->AssociatedIconIndex);
    // update icon
    this->UpdatePreviewIcon();

    // update hot and down icons if required
    HICON hIconLarge;
    if (bUpdateHotIcon)
    {
        _tcscpy(this->LaunchBarAssociatedIconHotPath,this->AssociatedIconPath);
        this->LaunchBarAssociatedIconHotIndex = this->AssociatedIconIndex;
        hIconLarge = this->ExtractLargeIcon(this->LaunchBarAssociatedIconHotPath,this->LaunchBarAssociatedIconHotIndex);
        ::SendMessage(::GetDlgItem(this->hWndDialog,IDC_STATIC_LAUNCHBAR_HOT_ICON),STM_SETICON,(WPARAM)hIconLarge,0);
    }
    if (bUpdateDownIcon)
    {
        _tcscpy(this->LaunchBarAssociatedIconDownPath,this->AssociatedIconPath);
        this->LaunchBarAssociatedIconDownIndex = this->AssociatedIconIndex;
        hIconLarge = this->ExtractLargeIcon(this->LaunchBarAssociatedIconDownPath,this->LaunchBarAssociatedIconDownIndex);
        ::SendMessage(::GetDlgItem(this->hWndDialog,IDC_STATIC_LAUNCHBAR_DOWN_ICON),STM_SETICON,(WPARAM)hIconLarge,0);
    }
}

HICON COptionsUI::ExtractLargeIcon(TCHAR* IconPath,int IconIndex)
{
        // load associated icon
    HICON hIconLarge;
    if ( ((int)::ExtractIconEx(IconPath,
                               IconIndex, 
                                &hIconLarge, 
                                NULL, 
                                1)
         )<=0
       )
    {
        // on error load default icon
        hIconLarge=this->pQuickWayToFolders->hDefaultIcon;
    }
    else
    {
        // add icon handle to the to be destroy vector
        this->LoadedIconsToBeDestroyed.push_back(hIconLarge);
    }
    return hIconLarge;
}

void COptionsUI::ChooseLaunchBarExtraIcon(int WindowIconId)
{
    TCHAR* IconPath;
    int* pIconIndex;

    switch (WindowIconId)
    {
    default:
    case IDC_STATIC_LAUNCHBAR_HOT_ICON:
        IconPath = this->LaunchBarAssociatedIconHotPath;
        pIconIndex = &this->LaunchBarAssociatedIconHotIndex;
        break;
    case IDC_STATIC_LAUNCHBAR_DOWN_ICON:
        IconPath = this->LaunchBarAssociatedIconDownPath;
        pIconIndex = &this->LaunchBarAssociatedIconDownIndex;
        break;
    }
    ::PickIconDlg(this->hWndDialog,IconPath,MAX_PATH,pIconIndex);
    
    HICON hIconLarge = this->ExtractLargeIcon(IconPath,*pIconIndex);
    ::SendMessage(::GetDlgItem(this->hWndDialog,WindowIconId),STM_SETICON,(WPARAM)hIconLarge,0);
}

void COptionsUI::ChooseColor(HBRUSH* pBrush,HWND hItem)
{
    static COLORREF acrCustClr[16]; // array of custom colors 
    CHOOSECOLOR CC={0};
    CC.lStructSize=sizeof(CHOOSECOLOR);
    CC.lpCustColors = (LPDWORD)acrCustClr;
    CC.hwndOwner=this->hWndDialog;
    CC.Flags=CC_FULLOPEN | CC_ANYCOLOR | CC_RGBINIT;

    CC.rgbResult=this->GetBrushColor(*pBrush);
        
    if (::ChooseColor(&CC))
    {
        // replace brush
        ::DeleteObject(*pBrush);
        *pBrush=::CreateSolidBrush(CC.rgbResult);

        // update item
        // invalidate rect is less resources consuming than RedrawWindow
        RECT Rect;
        CDialogHelper::GetClientWindowRect(this->hWndDialog,hItem,&Rect);
        ::InvalidateRect(this->hWndDialog,&Rect,FALSE);
        // ::RedrawWindow(hItem,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME | RDW_ALLCHILDREN);
    }
}

void COptionsUI::BrowseForFolder()
{
    TCHAR Path[MAX_PATH];
    ::GetWindowText(::GetDlgItem(this->hWndDialog,IDC_EDIT_PATH_LOCAL_FOLDER_PATH),Path,MAX_PATH);
    CBrowseForFolder BrowseForFolder;
    if (BrowseForFolder.BrowseForFolder(this->hWndDialog,Path,NULL,BIF_EDITBOX | BIF_NEWDIALOGSTYLE |BIF_STATUSTEXT))
        ::SetDlgItemText(this->hWndDialog,IDC_EDIT_PATH_LOCAL_FOLDER_PATH,BrowseForFolder.szSelectedPath);
}
void COptionsUI::BrowseForBackGround(int IdResTextResult)
{
    TCHAR szFile[MAX_PATH]=_T("");

    // open dialog
    OPENFILENAME ofn;
    memset(&ofn,0,sizeof (OPENFILENAME));
    ofn.lStructSize=sizeof (OPENFILENAME);
    ofn.hwndOwner=this->hWndDialog;
    ofn.hInstance=this->hInstance;
    ofn.lpstrFilter=_T("All\0*.*\0");
    ofn.nFilterIndex = 1;
    ofn.Flags=OFN_EXPLORER|OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST|OFN_HIDEREADONLY;
    ofn.lpstrDefExt=_T("");
    ofn.lpstrFile=szFile;
    ofn.nMaxFile=MAX_PATH;

    if (!GetOpenFileName(&ofn))
        return;

    // assume image can be loaded by gdiplus

    BOOL bImageFormatSupported=TRUE;
   
    if (this->bGdiPlusStarted)
    {
        WCHAR wcsImagePath[MAX_PATH];

ImageFormatChecking:
        bImageFormatSupported=TRUE;

#if (defined(UNICODE)||defined(_UNICODE))
        wcsncpy(wcsImagePath,szFile,MAX_PATH);
#else
        CAnsiUnicodeConvert::AnsiToUnicode(szFile,wcsImagePath,MAX_PATH);
#endif

        Gdiplus::Image objImage(wcsImagePath,TRUE);
        if (objImage.GetLastStatus()!=Gdiplus::Ok)
        {
            this->ReportError(this->pLanguage->UnsupportedImageFormat);
            bImageFormatSupported=FALSE;

            if (GetOpenFileName(&ofn))
            {
                goto ImageFormatChecking;
            }
        }
    }

    if (bImageFormatSupported)
        ::SetDlgItemText(this->hWndDialog,IdResTextResult,szFile);
}

//-----------------------------------------------------------------------------
// Name: Close
// Object: EndDialog
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void COptionsUI::Close()
{
    if (this->pDragAndDrop)
    {
        this->pDragAndDrop->Stop();
        delete this->pDragAndDrop;
        this->pDragAndDrop = NULL;
    }

    if (this->pTreeview)
    {
        delete this->pTreeview;
        this->pTreeview=NULL;
    }

    if (this->pToolbar)
    {
        delete this->pToolbar;
        this->pToolbar=NULL;
    }

    if (this->pListview)
    {
        delete this->pListview;
        this->pListview=NULL;
    }

    ::EndDialog(this->hWndDialog,this->DialogResult);

    if (this->bGdiPlusStarted)
    {
        if (this->hDisplayedBmp)
        {
            ::DeleteObject(this->hDisplayedBmp);
            this->hDisplayedBmp=NULL;
        }
        if (this->hBitmapPreview)
        {
            ::DeleteObject(this->hBitmapPreview);
            this->hBitmapPreview=NULL;
        }

        if (this->pBitmapDesktop)
        {
            delete this->pBitmapDesktop;
            this->pBitmapDesktop=NULL;
        }

        if (this->pBitmapBrowser)
        {
            delete this->pBitmapBrowser;
            this->pBitmapBrowser=NULL;
        }

        if (this->pBitmapMenu)
        {
            delete this->pBitmapMenu;
            this->pBitmapMenu=NULL;
        }

        if (this->pBitmapMultipleMenu)
        {
            delete this->pBitmapMultipleMenu;
            this->pBitmapMultipleMenu=NULL;
        }
        if (this->pBitmapPreview)
        {
            delete this->pBitmapPreview;
            this->pBitmapPreview=NULL;
        }

        Gdiplus::GdiplusShutdown(this->gdiplusToken);
    }

    return;
}
//-----------------------------------------------------------------------------
// Name: GetAssociatedObject
// Object: Get object associated to window handle
// Parameters :
//     in  : HWND hWndDialog : handle of the window
//     out :
//     return : associated object if found, NULL if not found
//-----------------------------------------------------------------------------
COptionsUI* COptionsUI::GetAssociatedObject(HWND hWndDialog)
{
    return (COptionsUI*)GetWindowLongPtr(hWndDialog,GWLP_USERDATA);
}
//-----------------------------------------------------------------------------
// Name: WndProc
// Object: dialog callback of the dump dialog
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK COptionsUI::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch (uMsg)
    {
    case WM_INITDIALOG:
        {
            COptionsUI* pOptionsUI=(COptionsUI*)lParam;
            pOptionsUI->hWndDialog=hWnd;

            ::SetWindowLongPtr(hWnd,GWLP_USERDATA,(LONG_PTR)pOptionsUI);

            pOptionsUI->OnInit();
            // load dlg icons
            CDialogHelper::SetIcon(hWnd,IDI_ICON_OPTIONS);
        }
        break;
    case WM_CLOSE:
        {
            COptionsUI* pOptionsUI=COptionsUI::GetAssociatedObject(hWnd);
            if (!pOptionsUI)
                break;
            pOptionsUI->Close();
        }
        break;
    case WM_SIZING:
        {
            COptionsUI* pOptionsUI=COptionsUI::GetAssociatedObject(hWnd);
            if (!pOptionsUI)
                break;
            pOptionsUI->OnSizing((RECT*)lParam);
        }
        break;
    case WM_SIZE:
        {
            COptionsUI* pOptionsUI=COptionsUI::GetAssociatedObject(hWnd);
            if (!pOptionsUI)
                break;
            pOptionsUI->OnSize();
        }
        break;
    case WM_COMMAND:
        {
            COptionsUI* pOptionsUI=COptionsUI::GetAssociatedObject(hWnd);
            if (!pOptionsUI)
                break;

            switch (LOWORD(wParam))
            {
            case IDOK:
                pOptionsUI->OnOk();
                break;
            case IDCANCEL:
                pOptionsUI->OnCancel();
                break;
            case IDC_BUTTON_CUSTOMIZATION_TEXT_COLOR:
                pOptionsUI->ChooseColor(&pOptionsUI->TextColorBrush,pOptionsUI->hWndTextColor);
                break;
            case IDC_BUTTON_CUSTOMIZATION_BACKGROUND_TEXT_COLOR:
                pOptionsUI->ChooseColor(&pOptionsUI->TextBackgroundColorBrush,pOptionsUI->hWndTextBackgroundColor);
                break;
            case IDC_BUTTON_CUSTOMIZATION_BACKGROUND_COLOR:
                pOptionsUI->ChooseColor(&pOptionsUI->BackgroundColorBrush,pOptionsUI->hWndBackgroundColor);
                break;
            case IDC_BUTTON_CUSTOMIZATION_BACKGROUND_COLOR1:
                pOptionsUI->ChooseColor(&pOptionsUI->BackgroundColor1Brush,pOptionsUI->hWndBackgroundColor1);
                break;
            case IDC_BUTTON_CUSTOMIZATION_BACKGROUND_COLOR2:
                pOptionsUI->ChooseColor(&pOptionsUI->BackgroundColor2Brush,pOptionsUI->hWndBackgroundColor2);
                break;
            case IDC_BUTTON_CUSTOMIZATION_BROWSE_BACKGROUND:
                pOptionsUI->BrowseForBackGround(IDC_EDIT_CUSTOMIZATION_BACKGROUND_PATH);
                break;

            case IDC_BUTTON_SELECT_LAUNCHBAR_HOT_ICON:
                pOptionsUI->ChooseLaunchBarExtraIcon(IDC_STATIC_LAUNCHBAR_HOT_ICON);
                break;
            case IDC_BUTTON_SELECT_LAUNCHBAR_DOWN_ICON:
                pOptionsUI->ChooseLaunchBarExtraIcon(IDC_STATIC_LAUNCHBAR_DOWN_ICON);
                break;
            case IDC_BUTTON_MULTIPLE_FOLDERS_MENU_UP:
                pOptionsUI->MoveFolderInsideMultipleFolderUp();
                break;
            case IDC_BUTTON_MULTIPLE_FOLDERS_MENU_DOWN:
                pOptionsUI->MoveFolderInsideMultipleFolderDown();
                break;
            case IDC_BUTTON_MULTIPLE_FOLDERS_MENU_REMOVE:
                pOptionsUI->RemoveSelectedFoldersFromMultipleFolder();
                break;
            case IDC_BUTTON_MULTIPLE_FOLDERS_MENU_ADD:
                pOptionsUI->AddFolderToMultipleFolder();
                break;
            case IDC_BUTTON_LAUNCHBAR_BACKGROUND_COLOR:
                pOptionsUI->ChooseColor(&pOptionsUI->LaunchBarBackgroundColorBrush,pOptionsUI->hWndLaunchBarBackgroundColor);
                break;
            case IDC_BUTTON_LAUNCHBAR_BACKGROUND_COLOR1:
                pOptionsUI->ChooseColor(&pOptionsUI->LaunchBarGradiantBackgroundColor1Brush,pOptionsUI->hWndLaunchBarGradiantBackgroundColor1);
                break;
            case IDC_BUTTON_LAUNCHBAR_BACKGROUND_COLOR2:
                pOptionsUI->ChooseColor(&pOptionsUI->LaunchBarGradiantBackgroundColor2Brush,pOptionsUI->hWndLaunchBarGradiantBackgroundColor2);
                break;
            case IDC_BUTTON_LAUNCHBAR_TRANSPARENT_COLOR:
                pOptionsUI->ChooseColor(&pOptionsUI->LaunchBarTransparentBrush,pOptionsUI->hWndLaunchBarTransparentColor);
                break;
            case IDC_BUTTON_LAUNCHBAR_BROWSE_BACKGROUND:
                pOptionsUI->BrowseForBackGround(IDC_EDIT_LAUNCHBAR_BACKGROUND_PATH);
                break;

            case IDC_BUTTON_SELECT_ASSOCIATED_ICON:
                pOptionsUI->ChooseIcon();
                break;
            case IDC_BUTTON_PATH_BROWSE_LOCAL_PATH:
                pOptionsUI->BrowseForFolder();
                break;
            case IDC_RADIO_FOLDER_TYPE_DESKTOP:
                pOptionsUI->UpdateFolderPresentationType(CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_DESKTOP_BROWSER);
                break;
            case IDC_RADIO_FOLDER_TYPE_BROWSER:
                pOptionsUI->UpdateFolderPresentationType(CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_BROWSER);
                break;
            case IDC_RADIO_FOLDER_TYPE_MENU:
                pOptionsUI->UpdateFolderPresentationType(CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_SINGLE_MENU);
                break;
            case IDC_RADIO_FOLDER_TYPE_MULTIPLE_FOLDER_MENU:
                pOptionsUI->UpdateFolderPresentationType(CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU);
                break;

            case IDC_TOOLBAR_BUTTON_ADD_QUICKWAY_TO_FOLDER:
                pOptionsUI->OnAddFolder();
                break;
            case IDC_TOOLBAR_BUTTON_REMOVE_QUICKWAY_TO_FOLDER:
                pOptionsUI->OnRemoveFolder();
                break;
            case IDC_TOOLBAR_BUTTON_MOVE_QUICKWAY_TO_FOLDER_UP:
                pOptionsUI->OnMoveFolderUp();
                break;
            case IDC_TOOLBAR_BUTTON_MOVE_QUICKWAY_TO_FOLDER_DOWN:
                pOptionsUI->OnMoveFolderDown();
                break;

            case IDC_CHECK_BROWSE_OPTIONS_HIDE_TOOLBAR:
                {
                    BOOL bEnableHideForHomeFolder;
                    bEnableHideForHomeFolder = !CDialogHelper::IsButtonChecked(::GetDlgItem(pOptionsUI->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_HIDE_TOOLBAR));
                    ::EnableWindow(::GetDlgItem(pOptionsUI->hWndDialog,IDC_CHECK_BROWSE_OPTIONS_HIDE_TOOLBAR_FOR_HOME_FOLDER),bEnableHideForHomeFolder);
                }
                break;

            case IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER:
                {
                    BOOL bEnableMergedFolderName;
                    bEnableMergedFolderName = CDialogHelper::IsButtonChecked(::GetDlgItem(pOptionsUI->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER));
                    ::EnableWindow(::GetDlgItem(pOptionsUI->hWndDialog,IDC_EDIT_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),bEnableMergedFolderName);
                }
                break;
            case IDC_CHECK_MERGE_MENUS_IN_POPUPMENU:
                {
                    BOOL bEnableMergeToSepcificFolder;
                    bEnableMergeToSepcificFolder = !CDialogHelper::IsButtonChecked(::GetDlgItem(pOptionsUI->hWndDialog,IDC_CHECK_MERGE_MENUS_IN_POPUPMENU));
                    ::EnableWindow(::GetDlgItem(pOptionsUI->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),bEnableMergeToSepcificFolder);
                    ::EnableWindow(::GetDlgItem(pOptionsUI->hWndDialog,IDC_EDIT_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER),bEnableMergeToSepcificFolder && CDialogHelper::IsButtonChecked(::GetDlgItem(pOptionsUI->hWndDialog,IDC_CHECK_FOLDER_MENU_MERGE_TO_SPECIFIC_FOLDER)));
                }
                break;

            case IDC_COMBO_APPLICATION_LANGUAGE:
                {
                    switch(HIWORD(wParam))
                    {
                    case CBN_SELCHANGE:
                        pOptionsUI->OnLanguageChange();
                    	break;
                    default:
                        break;
                    }
                }
                break;
            case IDC_COMBO_SKIN:
                {
                    switch(HIWORD(wParam))
                    {
                    case CBN_SELCHANGE:
                        pOptionsUI->OnSkinChange();
                        break;
                    default:
                        break;
                    }
                }
                break;
            case IDC_COMBO_BROWSE_OPTIONS_VIEW_TYPE:
                {
                    switch(HIWORD(wParam))
                    {
                    case CBN_SELCHANGE:
                        pOptionsUI->OnBrowseViewChange();
                        break;
                    default:
                        break;
                    }
                }
                break;
            case IDC_COMBO_LAUNCH_MODE:
                {
                    switch(HIWORD(wParam))
                    {
                    case CBN_SELCHANGE:
                        pOptionsUI->OnLaunchModeChange();
                        break;
                    default:
                        break;
                    }
                }
                break;
            }
        }
        break;
    case WM_NOTIFY:
        {
            COptionsUI* pOptionsUI=COptionsUI::GetAssociatedObject(hWnd);
            if (!pOptionsUI)
                break;

            if (pOptionsUI->pToolbar)
            {
                if (pOptionsUI->pToolbar->OnNotify(wParam, lParam))
                    break;
            }

            if (pOptionsUI->pTreeview)
            {
                if (pOptionsUI->pTreeview->OnNotify(wParam, lParam))
                    break;
            }

            if (pOptionsUI->pListview)
            {
                if (pOptionsUI->pListview->OnNotify(wParam, lParam))
                    break;
            }
        }
        break;
    case WM_DRAWITEM: 
        {
            COptionsUI* pOptionsUI=COptionsUI::GetAssociatedObject(hWnd);
            if (!pOptionsUI)
                break;

            LPDRAWITEMSTRUCT lpdis = (LPDRAWITEMSTRUCT) lParam;

            if (lpdis->hwndItem==pOptionsUI->hWndTextColor)
            {
                ::FillRect(lpdis->hDC,&lpdis->rcItem,pOptionsUI->TextColorBrush);
                return TRUE;
            }
            else if (lpdis->hwndItem==pOptionsUI->hWndTextBackgroundColor)
            {
                ::FillRect(lpdis->hDC,&lpdis->rcItem,pOptionsUI->TextBackgroundColorBrush);
                return TRUE;
            }
            else if (lpdis->hwndItem==pOptionsUI->hWndBackgroundColor)
            {
                ::FillRect(lpdis->hDC,&lpdis->rcItem,pOptionsUI->BackgroundColorBrush);
                return TRUE;
            }
            else if (lpdis->hwndItem==pOptionsUI->hWndBackgroundColor1)
            {
                ::FillRect(lpdis->hDC,&lpdis->rcItem,pOptionsUI->BackgroundColor1Brush);
                return TRUE;
            }
            else if (lpdis->hwndItem==pOptionsUI->hWndBackgroundColor2)
            {
                ::FillRect(lpdis->hDC,&lpdis->rcItem,pOptionsUI->BackgroundColor2Brush);
                return TRUE;
            }
            else if (lpdis->hwndItem==pOptionsUI->hWndLaunchBarBackgroundColor)
            {
                ::FillRect(lpdis->hDC,&lpdis->rcItem,pOptionsUI->LaunchBarBackgroundColorBrush);
                return TRUE;
            }
            else if (lpdis->hwndItem==pOptionsUI->hWndLaunchBarGradiantBackgroundColor1)
            {
                ::FillRect(lpdis->hDC,&lpdis->rcItem,pOptionsUI->LaunchBarGradiantBackgroundColor1Brush);
                return TRUE;
            }
            else if (lpdis->hwndItem==pOptionsUI->hWndLaunchBarGradiantBackgroundColor2)
            {
                ::FillRect(lpdis->hDC,&lpdis->rcItem,pOptionsUI->LaunchBarGradiantBackgroundColor2Brush);
                return TRUE;
            }
            else if (lpdis->hwndItem==pOptionsUI->hWndLaunchBarTransparentColor)
            {
                ::FillRect(lpdis->hDC,&lpdis->rcItem,pOptionsUI->LaunchBarTransparentBrush);
                return TRUE;
            }

        }
        break;
    case WM_DROPFILES:
        {
            COptionsUI* pOptionsUI=COptionsUI::GetAssociatedObject(hWnd);
            if (!pOptionsUI)
                break;
            pOptionsUI->pDragAndDrop->OnWM_DROPFILES(wParam,lParam);
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: ReportError
// Object: function to report errors
// Parameters :
//     in  : TCHAR* Msg : error message
//     out :
//     return : 
//-----------------------------------------------------------------------------
void COptionsUI::ReportError(TCHAR* Msg)
{
    // sometimes MB_TOPMOST make MessageBox fail if no dialog is associated to thread
    if (!MessageBox(this->hWndDialog,Msg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST))
        MessageBox(this->hWndDialog,Msg,_T("Error"),MB_OK|MB_ICONERROR);
}

//-----------------------------------------------------------------------------
// Name: OnSizing
// Object: check dialog box size
// Parameters :
//     in out  : RECT* pRect : pointer to dialog rect
//     return : 
//-----------------------------------------------------------------------------
void COptionsUI::OnSizing(RECT* pRect)
{
    // check min width and min height
    if ((pRect->right-pRect->left)<COptionsUI_DIALOG_MIN_WIDTH)
        pRect->right=pRect->left+COptionsUI_DIALOG_MIN_WIDTH;
    if ((pRect->bottom-pRect->top)<COptionsUI_DIALOG_MIN_HEIGHT)
        pRect->bottom=pRect->top+COptionsUI_DIALOG_MIN_HEIGHT;
}

//-----------------------------------------------------------------------------
// Name: OnSize
// Object: Resize controls
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void COptionsUI::OnSize()
{
// TODO
    //RECT RectWindow;
    //RECT RectGroup;
    //RECT Rect;
    //HWND hWnd;
    //POINT Point;
    //DWORD SpaceBetweenControls;

    //// get window Rect
    //GetClientRect(this->hWndDialog,&RectWindow);

    /////////////////
    //// search Group
    /////////////////
    //hWnd=GetDlgItem(this->hWndDialog,IDC_GROUP_FIND);
    //CDialogHelper::GetClientWindowRect(this->hWndDialog,hWnd,&RectGroup);
    //SpaceBetweenControls=RectGroup.left;
    //Point.x=RectGroup.left;
    //Point.y=RectWindow.bottom-(RectGroup.bottom-RectGroup.top)-SpaceBetweenControls;
    //CDialogHelper::MoveGroupTo(this->hWndDialog,hWnd,&Point);
    //// update RectGroup
    //CDialogHelper::GetClientWindowRect(this->hWndDialog,hWnd,&RectGroup);

    ///////////////
    // treeview 
    ///////////////
    //CDialogHelper::GetClientWindowRect(this->hWndDialog,this->pTreeview->GetControlHandle(),&Rect);
    //SetWindowPos(this->pTreeview->GetControlHandle(),HWND_NOTOPMOST,
    //    0,0,
    //    (RectWindow.right-RectWindow.left)-2*SpaceBetweenControls,
    //    (RectGroup.top-RectWindow.top)-Rect.top-SpaceBetweenControls,
    //    SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE);

    // redraw dialog
    CDialogHelper::Redraw(this->hWndDialog);

    if (this->pToolbar)    
        this->pToolbar->Autosize();
}



LRESULT CALLBACK COptionsUI::KeyboardProc (int code, WPARAM wParam, LPARAM lParam) 
{
    BOOL bMessageUsed=FALSE;
    LRESULT Res;

    if(code == HC_ACTION) 
    {
        HWND hWnd = ::GetActiveWindow();
        if (::GetFocus() == ::GetDlgItem(hWnd,IDC_EDIT_HOT_KEY))
        {
            BOOL bKeyUp=(BOOL)(lParam>>31);
            BOOL bKeyDown=!bKeyUp;
            COptionsUI* pOptionsUI=COptionsUI::GetAssociatedObject(hWnd);
            if (pOptionsUI && bKeyDown)
            {
                bMessageUsed=TRUE;

                UINT fsModifiers;
                UINT VirtualKey;

                #define KEY_STATE_DOWN 0x8000
                #define VK_ALT VK_MENU

                fsModifiers = 0;
                // we just check for control actions
                if (::GetKeyState(VK_CONTROL) & KEY_STATE_DOWN)
                    fsModifiers |= MOD_CONTROL;
                if (::GetKeyState(VK_SHIFT) & KEY_STATE_DOWN)
                    fsModifiers |= MOD_SHIFT;
                if (::GetKeyState(VK_ALT) & KEY_STATE_DOWN)
                    fsModifiers |= MOD_ALT;
                if (::GetKeyState(VK_LWIN) & KEY_STATE_DOWN)
                    fsModifiers |= MOD_WIN;
                if (::GetKeyState(VK_RWIN) & KEY_STATE_DOWN)
                    fsModifiers |= MOD_WIN;

                VirtualKey = (UINT)wParam;

                pOptionsUI->HotKeyfsModifiers=fsModifiers;
                pOptionsUI->HotKeyVirtualKey=VirtualKey;

                TCHAR String[MAX_PATH];
                pOptionsUI->GetKeysString(fsModifiers,VirtualKey,String);
#ifdef _DEBUG
                _tcscat(String,_T("\r\n"));
                ::OutputDebugString(String);
#endif
                ::SetDlgItemText(hWnd,IDC_EDIT_HOT_KEY,String);
            }            
        }
    }

    // call next hook
    Res=::CallNextHookEx(NULL, code, wParam, lParam);

    // if message has been taken into account return a non null value 
    // (avoid beep as for unsupported key)
    if (bMessageUsed)
        Res=1;

    return Res;
}


BOOL COptionsUI::GetKeysString(UINT fsModifiers,UINT VirtualKey,OUT TCHAR* pString)
{
    TCHAR String[MAX_PATH];
    *String = 0;
    *pString = 0;

    // a modifier AND virtual key must be defined
    if (   (fsModifiers == 0) 
         || (VirtualKey == 0)
         )
    {
        return FALSE;
    }

    // if virtual key is the modifier
    if ( (VirtualKey==VK_CONTROL)
         || (VirtualKey==VK_SHIFT)
         || (VirtualKey==VK_ALT)
         || (VirtualKey==VK_LWIN)
         || (VirtualKey==VK_RWIN)
         )
    {
        return FALSE;
    }

    if (fsModifiers & MOD_ALT)
        _tcscat(String,_T("ALT + "));
    if (fsModifiers & MOD_CONTROL)
        _tcscat(String,_T("CTRL + "));
    if (fsModifiers & MOD_SHIFT)
        _tcscat(String,_T("SHIFT + "));
    if (fsModifiers & MOD_WIN)
        _tcscat(String,_T("WIN + "));

    UINT ScanCode = MapVirtualKey(VirtualKey, MAPVK_VK_TO_VSC);

    // because MapVirtualKey strips the extended bit for some keys
    switch (VirtualKey)
    {
        case VK_LEFT: case VK_UP: case VK_RIGHT: case VK_DOWN: // arrow keys
        case VK_PRIOR: case VK_NEXT: // page up and page down
        case VK_END: case VK_HOME:
        case VK_INSERT: case VK_DELETE:
        case VK_DIVIDE: // numpad slash
        case VK_NUMLOCK:
        {
            ScanCode |= 0x100; // set extended bit
            break;
        }
    }

    if (GetKeyNameText( ScanCode << 16, &String[_tcslen(String)], (int)(_countof(String)-_tcslen(String)) ) != 0)
    {
        _tcscpy(pString,String);
        return TRUE;
    }
    else
    {
        *pString = 0;
        return FALSE;
    }
}
