/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manages the options dialog
//-----------------------------------------------------------------------------


#pragma once

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0501 // for xp os
#endif

#ifndef _WIN32_IE
#define _WIN32_IE 0x0501 // for xp os
#endif

#include <windows.h>
#include <WindowsX.h>
#include <vector>

#include "resource.h"

#include "../Tools/GUI/Dialog/DialogHelper.h"
#include "../tools/GUI/Treeview/Treeview.h"
#include "../tools/GUI/ToolBar/Toolbar.h"
#include "../Tools/GUI/ListView/Listview.h"
#include "../Tools/GUI/BrowseForFolder/BrowseForFolder.h"
#include "../Tools/File/FileSearch.h"
#include "../Tools/GUI/DragAndDrop/DragAndDrop.h"
//#include "../tools/String/StringConverter.h"

#include "CGdiPlusBitmap.h"
#include "SingleFolderOptions.h"
#include "ApplicationOptions.h"
#include "Language.h"

#include "GenericQuickWayToFolderObject.h"


#define COptionsUI_DIALOG_MIN_WIDTH 400
#define COptionsUI_DIALOG_MIN_HEIGHT 400
#define COptionsUI_SPACE_BETWEEN_CONTROLS 5
#define COptionsUI_LANGUAGE_SUBDIR _T("Languages\\")

class CQuickWayToFolders;
class COptionsUI:public CTreeviewEventsBase
{
protected:
    typedef struct 
    {
        DWORD Id;
        const TCHAR* CSIDL_Name;
    }CSIDL_ArrayItem;
    static CSIDL_ArrayItem CSIDL_Array[];
    
    enum DialogResult
    {
        DialogResult_OK = 0xABCDEF, // don't let 1 because 1 can be returned after a call to PostQuitMessage
        DialogResult_CANCEL
    };

    typedef struct tagKnownFoldersListItem
    {
        TCHAR Path[MAX_PATH];
        DWORD Id;
        const TCHAR* CSIDL_Name;
    }KNOWN_FOLDERS_LIST_ITEM;
    static bool KnownFoldersListSorting(KNOWN_FOLDERS_LIST_ITEM a,KNOWN_FOLDERS_LIST_ITEM b);

    HWND hWndDialog;
    INT_PTR DialogResult;
    HINSTANCE hInstance;
    HANDLE hParentHandle;
    CTreeview* pTreeview;
    HTREEITEM hItemFolders;
    HTREEITEM hItemApplicationOptions;
    HTREEITEM hItemLaunchBar;
    CToolbar* pToolbar;
    CListview* pListview;
    CQuickWayToFolders* pQuickWayToFolders;
    std::vector<CFolderMenu::MULTIPLE_PATHS_DATA> LocalMultiplePath;
    CLanguage* pLanguage;
    std::vector<CGenericQuickWayToFolderObject*> FoldersArray;
    CApplicationOptions* pApplicationOptions;
    std::vector<HICON> MultipleMenuListviewLoadedIconsToBeDestroyed;
    std::vector<HICON> LoadedIconsToBeDestroyed;
    int TreeViewIconIndexBrowserOptions;
    int TreeViewIconIndexCustomize;
    int TreeViewIconIndexFilters;
    CDragAndDrop* pDragAndDrop;

    BOOL bGdiPlusStarted;
    ULONG_PTR gdiplusToken;
    BOOL bLoadingFolderOptions;
    
    TCHAR AssociatedIconPath[MAX_PATH];
    int AssociatedIconIndex;
    TCHAR LaunchBarAssociatedIconHotPath[MAX_PATH];
    int LaunchBarAssociatedIconHotIndex;
    TCHAR LaunchBarAssociatedIconDownPath[MAX_PATH];
    int LaunchBarAssociatedIconDownIndex;

    CGdiPlusBitmapResource* pBitmapDesktop;
    CGdiPlusBitmapResource* pBitmapBrowser;
    CGdiPlusBitmapResource* pBitmapMenu;
    CGdiPlusBitmapResource* pBitmapMultipleMenu;
    CGdiPlusBitmap* pBitmapPreview;
	HBITMAP hDisplayedBmp;
    HBITMAP hBitmapPreview;

    HBRUSH TextColorBrush;
    HBRUSH TextBackgroundColorBrush;
    HBRUSH BackgroundColorBrush;
    HBRUSH BackgroundColor1Brush;
    HBRUSH BackgroundColor2Brush;

    HWND hWndTextColor;
    HWND hWndTextBackgroundColor;
    HWND hWndBackgroundColor;
    HWND hWndBackgroundColor1;
    HWND hWndBackgroundColor2;

    HWND hWndLaunchBarBackgroundColor;
    HWND hWndLaunchBarGradiantBackgroundColor1;
    HWND hWndLaunchBarGradiantBackgroundColor2;
    HWND hWndLaunchBarTransparentColor;

    HBRUSH LaunchBarBackgroundColorBrush;
    HBRUSH LaunchBarGradiantBackgroundColor1Brush;
    HBRUSH LaunchBarGradiantBackgroundColor2Brush;
    HBRUSH LaunchBarTransparentBrush;


    UINT HotKeyfsModifiers;
    UINT HotKeyVirtualKey;
    HHOOK hKeyboardHook;

    enum tagTreeViewNodeType
    {
        TreeViewNodeType_APPLICATION,
        TreeViewNodeType_FOLDERS,
        TreeViewNodeType_FOLDER_FILTERS,
        TreeViewNodeType_FOLDER_CUSTOMIZATION,
        TreeViewNodeType_FOLDER_BROWSING_OPTION,
        TreeViewNodeType_LAUNCHBAR
        // CSingleFolderOptions* pointer for Folder
    };

    CShellBrowser::tagShellBrowserView ComboViewTypeIndexes[CShellBrowser::SHELL_BROWSER_VIEW_COUNT];

    CGenericQuickWayToFolderObject* pCurrentFolderOptions;
    HTREEITEM hCurrentFolderOptionsTreeItem;

    void UpdatePreviewIcon();
    void ChooseColor(HBRUSH* pBrush,HWND hItem);
    void ChooseIcon();
    void ChooseLaunchBarExtraIcon(int WindowIconId);
    HICON ExtractLargeIcon(TCHAR* IconPath,int IconIndex);
    void DeleteFolderBrushs();
    void DeleteLaunchBarBrushs();
    void FillCSIDLCombo();
    void FillTreeView();
    void FillLanguagesCombo();
    void FillLaunchBarIconPosition();
    void FillLaunchBarMonitorsCombo(int IdCombo);
    // void FillLaunchBarDirectionsCombo();
    void FillLaunchBarPositionsCombo(int IdCombo);
    void FillHotCornersPositionsCombo(int IdCombo);
    void FillLaunchBarIconSizesCombo();
    void FillLaunchBarBackgroundGradiantTypesCombo(int DialogId);
    void FillLaunchBarBackgroundPicturePositionsCombo(int DialogId);
    void LoadFolderOptions(CGenericQuickWayToFolderObject* pSingleFolderOptions);
    void SaveFolderOptions(CGenericQuickWayToFolderObject* pSingleFolderOptions);
    void LoadApplicationOptions();
    void SaveApplicationOptions();
    void LoadLaunchbarOptions();
    void SaveLaunchbarOptions();
    void LoadLanguage();
    void EnableBrowserOptions(BOOL bEnable);
    void EnableSaveIconPositionForHomeFolder();
    void EnableSaveIconPositionForHomeFolder(CSingleFolderOptions::tagFolderPresentationType FolderPresentationType);
    void SetPath(TCHAR* PathOrCSIDL);
    void GetPath(OUT TCHAR* PathOrCSIDL,IN SIZE_T PathOrCSIDLMaxSize);
    void AddFolderToMultipleFolderListview(CFolderMenu::MULTIPLE_PATHS_DATA* pData);
    void RemoveSelectedFoldersFromMultipleFolder();
    void AddFolderToMultipleFolder();
    void MoveFolderInsideMultipleFolderUp();
    void MoveFolderInsideMultipleFolderDown();
    void BrowseForFolder();
    void BrowseForBackGround(int IdResTextResult);
    void UpdateFolderPresentationType(CSingleFolderOptions::tagFolderPresentationType FolderPresentationType);
    void UpdatePathAndIconGroupPosition(CSingleFolderOptions::tagFolderPresentationType FolderPresentationType);
    CSingleFolderOptions::tagFolderPresentationType GetGUIFolderPresentationType();
    void AddFolderToTreeView(CGenericQuickWayToFolderObject* pSingleFolderOptions,HTREEITEM hInsertAfter,BOOL bExpand,BOOL bSetFocus);
    void UpdateFolderTreeNode(HTREEITEM hItem,CGenericQuickWayToFolderObject* pSingleFolderOptions);
    COLORREF GetBrushColor( HBRUSH hBrush );
    void LoadSkinsList();
    void UpdateSkinPreview();
    CApplicationOptions::tagLaunchMode GetSelectedLaunchMode();

    void StopKeyBoardHook();
    BOOL GetKeysString(UINT fsModifiers,UINT VirtualKey,OUT TCHAR* pString);
    void ReportError(TCHAR* Msg);
    void Close();

    void OnAddFolder();
    void OnRemoveFolder();
    void OnMoveFolderUp();
    void OnMoveFolderDown();
    void OnLanguageChange();
    void OnSkinChange();
    void OnBrowseViewChange();
    void OnLaunchModeChange();

    void OnOk();
    void OnCancel();
    void OnInit();
    void OnSize();
    void OnSizing(RECT* pRect);

    virtual void OnTreeviewSelectedItemChanged(CTreeview* pSender,LPNMTREEVIEW pChangeInfos);

    static BOOL LanguageFileFoundCallBack(TCHAR* Directory,WIN32_FIND_DATA* pWin32FindData,PVOID UserParam);
    static COptionsUI* GetAssociatedObject(HWND hWndDialog);
    static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK KeyboardProc (int code, WPARAM wParam, LPARAM lParam);
    static void OnFileDropped(HWND hWndControl, TCHAR* DroppedFile, SIZE_T FileDroppedIndex, LPVOID UserParam);
    void OnFileDropped(HWND hWndControl, TCHAR* DroppedFile, SIZE_T FileDroppedIndex);
public:
    COptionsUI(void);
    ~COptionsUI(void);
    static INT_PTR Show(HINSTANCE hInstance,HWND hWndDialog,CQuickWayToFolders* pQuickWayToFolders);
};