/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "QuickWayToFolders.h"
#include "OptionsUI.h"

#ifndef _countof
#define _countof(array) (sizeof(array)/sizeof(array[0]))
#endif

CQuickWayToFolders::CQuickWayToFolders(void)
{
    this->hWndDialog=NULL;
    this->hInstance=NULL;
    this->hDefaultIcon=NULL;
    this->pApplicationOptions=NULL;
    this->pLanguage=NULL;
    this->bFirstLaunch=FALSE;
    this->EndSessionProcessed = FALSE;
    this->pLaunchBar = NULL;

    TCHAR ApplicationPath[MAX_PATH];
    CStdFileOperations::GetAppDirectory(ApplicationPath,MAX_PATH);
    _tcscpy(this->ApplicationConfigFilePath,ApplicationPath);
    _tcscat(this->ApplicationConfigFilePath,QUICK_WAY_TO_FOLDERS_APPLICATION_CONFIGURATION_FILE);

    
    this->hModuleUser32 = ::LoadLibrary(_T("User32.dll"));
    this->pShutdownBlockReasonCreate = (pfShutdownBlockReasonCreate)::GetProcAddress(this->hModuleUser32,"ShutdownBlockReasonCreate");
    this->pShutdownBlockReasonDestroy = (pfShutdownBlockReasonDestroy)::GetProcAddress(this->hModuleUser32,"ShutdownBlockReasonDestroy");
}

CQuickWayToFolders::~CQuickWayToFolders(void)
{
    if (this->pLanguage)
    {
        delete this->pLanguage;
        this->pLanguage=NULL;
    }

    if (this->pLaunchBar)
    {
        delete this->pLaunchBar;
        this->pLaunchBar = NULL;
    }

    if (this->pApplicationOptions)
    {
        delete this->pApplicationOptions;
        this->pApplicationOptions=NULL;
    }
	if (this->hModuleUser32)
		::FreeLibrary(this->hModuleUser32);
}
BOOL CQuickWayToFolders::LoadApplicationOptions()
{
    if (this->pApplicationOptions)
    {
        delete this->pApplicationOptions;
        this->pApplicationOptions=NULL;
    }

    ///////////////////////////////////////////////////////
    // Load application options (portable, language file)
    ///////////////////////////////////////////////////////
    if (CStdFileOperations::DoesFileExists(this->ApplicationConfigFilePath))
    {
        CIniFileEx<CApplicationOptions> ApplicationIniFile(this->ApplicationConfigFilePath);
        this->pApplicationOptions=ApplicationIniFile.Get(CQuickWayToFolders_QUICK_WAY_TO_FOLDERS_SECTION_NAME);
    }
    if (!this->pApplicationOptions) // ini file not found, or invalid ini file
    {
        this->pApplicationOptions=new CApplicationOptions();

        /////////////////////////////////////
        // do special first launch operation
        /////////////////////////////////////

        // show option on dialog creation
        this->bFirstLaunch=TRUE;
    }
    return TRUE;
}

BOOL CQuickWayToFolders::QueryNotWritableConfigurationFileRetry(TCHAR* FilePath)
{
    TCHAR Msg[MAX_PATH+128];
    _sntprintf(Msg,_countof(Msg),TEXT("File %s is not writable\r\nTo save your configuration this file must be writable"),FilePath);
    Msg[_countof(Msg)-1]=0;
    int iRet = ::MessageBox(this->hWndDialog,Msg,TEXT("Configuration file not writable"),MB_ICONQUESTION | MB_RETRYCANCEL);
    return (iRet == IDRETRY);
}

BOOL CQuickWayToFolders::SaveApplicationOptions()
{
    if (!this->pApplicationOptions)
        return FALSE;

    // check for application write only in single user mode. 
    // In multiple user mode, application configuration can be read only with folder options stored in user path with read / write access
    BOOL bSingleUser;
    this->pApplicationOptions->GetUserMode(&bSingleUser);
    if (bSingleUser)
    {
        while (CStdFileOperations::IsWritable(this->ApplicationConfigFilePath) == FALSE)
        {
            if (this->QueryNotWritableConfigurationFileRetry(this->ApplicationConfigFilePath) == FALSE)
                return FALSE;
        }
    }

    // save to ini file
    CIniFileEx<CApplicationOptions> ApplicationIniFile(this->ApplicationConfigFilePath);
    return ApplicationIniFile.Set(CQuickWayToFolders_QUICK_WAY_TO_FOLDERS_SECTION_NAME,this->pApplicationOptions);
}

BOOL CQuickWayToFolders::LoadLanguageFile()
{
    if (this->pLanguage)
    {
        delete this->pLanguage;
        this->pLanguage=NULL;
    }

    // load from ini file
    CIniFileEx<CLanguage> FoldersIniFile(this->pApplicationOptions->LanguageConfigFilePath);
    this->pLanguage=FoldersIniFile.Get(CQuickWayToFolders_LANGUAGE_SECTION_NAME);
    if (!this->pLanguage)
        this->pLanguage=new CLanguage();

    return (this->pLanguage!=NULL);
}

BOOL CQuickWayToFolders::LoadFoldersOptions()
{
    if (!this->pApplicationOptions)
        return FALSE;

    // destroy objects
    this->DestroyAssociatedQuickWayToFoldersItems();

    // load from ini file
    CIniFileEx<CGenericQuickWayToFolderObject> FoldersIniFile(this->pApplicationOptions->FoldersOptionsConfigFilePath);
    FoldersIniFile.Get(CQuickWayToFolders_FOLDERS_SECTION_NAME,&this->FoldersArray);

    // if no folder, assume to put at least a quick way to desktop
    if (this->FoldersArray.size()==0)
    {
        CGenericQuickWayToFolderObject* pGenericQuickWayToFolderObject=new CGenericQuickWayToFolderObject();
        pGenericQuickWayToFolderObject->FolderPresentationType=CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_DESKTOP_BROWSER;
        _sntprintf(pGenericQuickWayToFolderObject->Path,_countof(pGenericQuickWayToFolderObject->Path),CSingleFolderOptions_CSIDL_PREFIX _T("0x%X"),CSIDL_DESKTOP);
        this->FoldersArray.push_back(pGenericQuickWayToFolderObject);
    }

    return TRUE;
}

BOOL CQuickWayToFolders::SaveFoldersOptions()
{
    if (!this->pApplicationOptions)
        return FALSE;


    while (CStdFileOperations::IsWritable(this->pApplicationOptions->FoldersOptionsConfigFilePath) == FALSE)
    {
        if (this->QueryNotWritableConfigurationFileRetry(this->pApplicationOptions->FoldersOptionsConfigFilePath) == FALSE)
            return FALSE;
    }

    // make a backup
    TCHAR BackUpIniFile[MAX_PATH];
    _tcscpy(BackUpIniFile,this->pApplicationOptions->FoldersOptionsConfigFilePath);
    CStdFileOperations::ChangeFileExt(BackUpIniFile,_T("bak"));

    // replace old file (avoid file increase due to array savings)
    ::MoveFileEx(this->pApplicationOptions->FoldersOptionsConfigFilePath,BackUpIniFile,MOVEFILE_COPY_ALLOWED | MOVEFILE_REPLACE_EXISTING | MOVEFILE_WRITE_THROUGH);

    // save to ini file
    CIniFileEx<CGenericQuickWayToFolderObject> FoldersIniFile(this->pApplicationOptions->FoldersOptionsConfigFilePath);
    BOOL bRet = FoldersIniFile.Set(CQuickWayToFolders_FOLDERS_SECTION_NAME,&this->FoldersArray);
    if (!bRet)
    {
#ifdef _DEBUG
        DWORD dwLastError = ::GetLastError();
        dwLastError;
        ::DebugBreak();
#endif
        // restore file (avoid leak of option file)
        ::MoveFileEx(BackUpIniFile,this->pApplicationOptions->FoldersOptionsConfigFilePath,MOVEFILE_COPY_ALLOWED | MOVEFILE_REPLACE_EXISTING | MOVEFILE_WRITE_THROUGH);
    }
    return bRet;
}

BOOL CQuickWayToFolders::UpdateOptionsBeforeClose()
{
    CGenericQuickWayToFolderObject* pGenericQuickWayToFolderObject;
    // for each folder destroy object
    for (size_t Cnt=0;Cnt<this->FoldersArray.size();Cnt++)
    {
        pGenericQuickWayToFolderObject=this->FoldersArray[Cnt];
        // on construction error, pGenericQuickWayToFolderObject->pContainer can be null
        if (pGenericQuickWayToFolderObject->pContainer)
            pGenericQuickWayToFolderObject->pContainer->UpdateOptionsBeforeClose();
    }
    return TRUE;
}

BOOL CQuickWayToFolders::AddHotKeys()
{
    BOOL bRetValue = TRUE;
    BOOL bCurrentRetValue;
    // for each folder
    for (size_t Cnt=0;Cnt<this->FoldersArray.size();Cnt++)
    {
        if (this->FoldersArray[Cnt]->HotKeyfsModifiers && this->FoldersArray[Cnt]->HotKeyVirtualKey)
        {

            bCurrentRetValue = ::RegisterHotKey(this->hWndDialog,
                                                (int)Cnt,
                                                this->FoldersArray[Cnt]->HotKeyfsModifiers,
                                                this->FoldersArray[Cnt]->HotKeyVirtualKey
                                                );
            if (!bCurrentRetValue)
            {
                TCHAR Msg[2*MAX_PATH];
                TCHAR ShortName[MAX_PATH];

                this->FoldersArray[Cnt]->GetShortName(ShortName);
                
                _sntprintf(Msg,_countof(Msg),
                            _T("Invalid HotKey for %s (Item %u)\r\n")
                            _T("Hotkey may already used by another application."),
                            ShortName,
                            Cnt+1
                            );
                ::MessageBox(this->hWndDialog,Msg,_T("QuickWayToFolders"),MB_OK | MB_ICONWARNING);
            }
            bRetValue = bRetValue && bCurrentRetValue;
        }
    }
    return bRetValue;
}

BOOL CQuickWayToFolders::RemoveHotKeys()
{
    BOOL bRetValue = TRUE;
    BOOL bCurrentRetValue;
    // for each folder
    for (size_t Cnt=0;Cnt<this->FoldersArray.size();Cnt++)
    {
        if (this->FoldersArray[Cnt]->HotKeyfsModifiers && this->FoldersArray[Cnt]->HotKeyVirtualKey)
        {
            bCurrentRetValue = ::UnregisterHotKey(this->hWndDialog,(int)Cnt);
            bRetValue = bRetValue && bCurrentRetValue;
        }
    }

    return bRetValue;
}

BOOL CQuickWayToFolders::CreateAssociatedQuickWayToFoldersItems()
{
    BOOL bAtLeastOneOk = FALSE;
    BOOL bLastOk;

    if (this->pLaunchBar)
    {
        delete this->pLaunchBar;
        this->pLaunchBar = NULL;
    }
    if (this->pApplicationOptions->LaunchMode == CApplicationOptions::LaunchMode_LAUNCH_BAR)
    {
        this->pLaunchBar = new CLaunchBar();
    }

    // for each folder create object
    for (size_t Cnt=0;Cnt<this->FoldersArray.size();Cnt++)
    {
        bLastOk = this->FoldersArray[Cnt]->ApplySettings(this,this->hInstance,this->GetDefaultIconHandle(),this->pLanguage);
        bAtLeastOneOk = bAtLeastOneOk || bLastOk; 
    }

    if (this->pApplicationOptions->LaunchMode == CApplicationOptions::LaunchMode_LAUNCH_BAR)
    {
        this->pLaunchBar->Create(this,this->hInstance,&this->pApplicationOptions->LaunchBarOptions);
        for (size_t Cnt=0;Cnt<this->FoldersArray.size();Cnt++)
        {
            if (this->FoldersArray[Cnt]->pContainer)
                this->FoldersArray[Cnt]->pContainer->hWndOwner = this->pLaunchBar->GetControlHandle();
        }
        this->pLaunchBar->pLaunchBarContainerBase->SetLanguage(this->pLanguage);
        this->pLaunchBar->pLaunchBarContainerBase->hWndOwner = this->pLaunchBar->GetControlHandle();
    }

    this->AddHotKeys();

    return bAtLeastOneOk;
}

BOOL CQuickWayToFolders::DestroyAssociatedQuickWayToFoldersItems()
{
    this->RemoveHotKeys();

    CGenericQuickWayToFolderObject* pGenericQuickWayToFolderObject;
    // for each folder destroy object
    for (size_t Cnt=0;Cnt<this->FoldersArray.size();Cnt++)
    {
        pGenericQuickWayToFolderObject=this->FoldersArray[Cnt];
        delete pGenericQuickWayToFolderObject;
    }
    this->FoldersArray.clear();

    if (this->pLaunchBar)
    {
        delete this->pLaunchBar;
        this->pLaunchBar = NULL;
    }

    return TRUE;
}

HICON CQuickWayToFolders::GetDefaultIconHandle()
{
    if (this->hDefaultIcon)
        return this->hDefaultIcon;

    this->hDefaultIcon=(HICON)::LoadImage(this->hInstance, MAKEINTRESOURCE(CQuickWayToFolders_DefaultIconID),IMAGE_ICON,0,0,LR_SHARED);
    return this->hDefaultIcon;
}

BOOL CQuickWayToFolders::Start(HINSTANCE hInstance)
{
    // to enable XP visual style
    InitCommonControls();

    // required
    ::OleInitialize(0);

    this->hInstance=hInstance;

    // load applications options
    if (!this->LoadApplicationOptions())
        return FALSE;

    // load language file
    this->LoadLanguageFile();

    // load folders options
    if (!this->LoadFoldersOptions())
        return FALSE;

/////////////////
// debug
//CGenericQuickWayToFolderObject* pf1=new CGenericQuickWayToFolderObject();
//_tcscpy(pf1->Path,_T("c:\\"));
//this->FoldersArray.push_back(pf1);

//CGenericQuickWayToFolderObject* pf2=new CGenericQuickWayToFolderObject();
//pf2->bRememberLastPosition=FALSE;
//pf2->BackgroundColor=RGB(0xff,0,0);
//pf2->TextBackgroundColor=RGB(0,0xff,0);
//pf2->TextColor=RGB(0,0,0xff);
////pf2->BackgroundType=CShellBrowserEx::BACKGROUND_TYPE_COLOR;
//pf2->FolderPresentationType=CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_DESKTOP_BROWSER;//FOLDER_PRESENTATION_TYPE_BROWSER;
//_stprintf(pf2->Path,CSingleFolderOptions_CSIDL_PREFIX _T("0x%X"),CSIDL_DESKTOP);
//// _tcscpy(pf2->Path,_T("c:\\"));
//this->FoldersArray.push_back(pf2);
//
//
//
//CGenericQuickWayToFolderObject* pf3=new CGenericQuickWayToFolderObject();
//pf3->bRememberLastPosition=FALSE;
//pf3->BackgroundColor=RGB(0xff,0,0);
//pf3->TextBackgroundColor=RGB(0,0xff,0);
//pf3->TextColor=RGB(0,0,0xff);
//pf3->BackgroundType=CShellBrowserEx::BACKGROUND_TYPE_COLOR;
//pf3->FolderPresentationType=CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_BROWSER;
//_tcscpy(pf3->Path,_T("c:\\"));
//this->FoldersArray.push_back(pf3);
///////////////////

    // wait for application end
    // create window waiting for WM_CLOSE / WM_QUIT / WM_QUERYENDSESSION
    this->WaitApplicationEnd();

    ::OleUninitialize();

    return TRUE;
}
BOOL CQuickWayToFolders::Exit()
{
    if (!this->hWndDialog)
        return FALSE;

    // use post message to allow current message to be fully processed before closing
    ::PostMessage(this->hWndDialog,WM_CLOSE,0,0);

    return TRUE;
}

BOOL CQuickWayToFolders::WaitApplicationEnd()
{
    WNDCLASSEX wc;
    HWND hwnd;
    MSG Msg;

    // if CQuickWayToFolders_CLASS_NAME is not already registered
    if (!GetClassInfoEx(this->hInstance, CQuickWayToFolders_CLASS_NAME , &wc))
    {
        // register it

        wc.cbSize        = sizeof(WNDCLASSEX);
        wc.style         = 0;
        wc.lpfnWndProc   = CQuickWayToFolders::WndProc;
        wc.cbClsExtra    = 0;
        wc.cbWndExtra    = 0;
        wc.hInstance     = this->hInstance;
        wc.hIcon         = NULL;
        wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
        wc.hbrBackground = (HBRUSH)COLOR_WINDOWFRAME;
        wc.lpszMenuName  = NULL;
        wc.lpszClassName = CQuickWayToFolders_CLASS_NAME;
        wc.hIconSm       = NULL;
        if(!RegisterClassEx(&wc))
        {
            ::MessageBox(NULL, _T("Window Registration Failed"), _T("Error"),MB_OK|MB_ICONERROR | MB_SYSTEMMODAL);
            return FALSE;
        }
    }

    // create the dialog window
    hwnd = ::CreateWindowEx(0,
                            CQuickWayToFolders_CLASS_NAME,
                            _T(""),
                            WS_DLGFRAME,
                            0,
                            0, 
                            0, 
                            0,
                            HWND_DESKTOP,
                            NULL,
                            this->hInstance,
                            this);

    if(hwnd == NULL)
    {
        ::MessageBox(NULL, _T("Window Creation Failed"), _T("Error"),MB_OK|MB_ICONERROR | MB_SYSTEMMODAL);
        return FALSE;
    }

    BOOL bRet;
    HWND hActiveWindow;
    while( (bRet = ::GetMessage( &Msg, NULL, 0, 0 )) != 0)
    { 
        if (bRet == -1)
        {
            // handle the error and possibly exit
        }
        else
        {
            if (Msg.message==WM_HOTKEY)
            {
                if (Msg.wParam <this->FoldersArray.size())
                {
                    if (this->FoldersArray[Msg.wParam]->pContainer)
                        this->FoldersArray[Msg.wParam]->pContainer->OnHotKey();
                }
                continue;
            }
            else if (Msg.message==WM_KEYDOWN)
            {
                hActiveWindow=::GetActiveWindow();
                if (::SendMessage(hActiveWindow,CContainerBase_KEYDOWN_MSG,0,(LPARAM)&Msg))
                {
                    // if message has been successfully proceeded, do not translate it
                    continue;
                }
            }
            ::TranslateMessage(&Msg); 
            ::DispatchMessage(&Msg); 
        }
    }
    // return (DWORD)Msg.wParam;
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: GetAssociatedObject
// Object: Get object associated to window handle
// Parameters :
//     in  : HWND hWndDialog : handle of the window
//     out :
//     return : associated object if found, NULL if not found
//-----------------------------------------------------------------------------
CQuickWayToFolders* CQuickWayToFolders::GetAssociatedObject(HWND hWndDialog)
{
    return (CQuickWayToFolders*)GetWindowLongPtr(hWndDialog,GWLP_USERDATA);
}

void CQuickWayToFolders::OnInit()
{
    BOOL bAtLeastOneOk;
    // create objects
    // must be done in message loop because RegisterHotKey / UnregisterHotKey must be done in same thread
    // so this call must be done in same thread as COptionsUI::Show
    bAtLeastOneOk = this->CreateAssociatedQuickWayToFoldersItems();

    if (this->bFirstLaunch || (!bAtLeastOneOk) )
    {
        ::PostMessage(this->hWndDialog,CQuickWayToFolders_SHOW_OPTIONS,0,0);
    }
}
void CQuickWayToFolders::OnClose()
{
    // save folder options to store window position, size and maximized state
    // do it before returning true to WM_QUERYENDSESSION
    // this->SaveFoldersOptions();

    // destroy objects if any MUST BE DONE BEFORE CALLING ::OleUninitialize();
    // must be done in message loop because RegisterHotKey / UnregisterHotKey must be done in same thread
    // so this call must be done in same thread as COptionsUI::Show
    this->DestroyAssociatedQuickWayToFoldersItems();

    ::DestroyWindow(this->hWndDialog);
    this->hWndDialog=NULL;
    ::PostQuitMessage(0);
}

LRESULT CALLBACK CQuickWayToFolders::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
    case WM_CREATE:
        {
            CREATESTRUCT* pCreateStruct;
            CQuickWayToFolders* pDialogObject; 
            pCreateStruct=(CREATESTRUCT*)lParam;
            pDialogObject=(CQuickWayToFolders*)pCreateStruct->lpCreateParams;
            SetWindowLongPtr(hWnd,GWLP_USERDATA,(LONG_PTR)pDialogObject);
            pDialogObject->hWndDialog=hWnd;
            pDialogObject->OnInit();
        }
        break;
    case CQuickWayToFolders_SHOW_OPTIONS:
        {
            CQuickWayToFolders* pDialogObject; 
            pDialogObject=CQuickWayToFolders::GetAssociatedObject(hWnd);
            if (!pDialogObject)
                break;

            COptionsUI::Show(pDialogObject->hInstance,pDialogObject->hWndDialog,pDialogObject);
        }
        break;
    case WM_QUERYENDSESSION:
    case WM_ENDSESSION:
        {
            CQuickWayToFolders* pDialogObject; 
            pDialogObject=CQuickWayToFolders::GetAssociatedObject(hWnd);
            if (!pDialogObject)
                break;

            if (!pDialogObject->EndSessionProcessed)
            {
                if (pDialogObject->pShutdownBlockReasonCreate)
                    pDialogObject->pShutdownBlockReasonCreate(pDialogObject->hWndDialog,L"Saving Configuration");
                pDialogObject->EndSessionProcessed = TRUE;

                // query windows to update options before saving them
                pDialogObject->UpdateOptionsBeforeClose();
                // save folder options to store window position, size and maximized state
                pDialogObject->SaveFoldersOptions();

                if (pDialogObject->pShutdownBlockReasonDestroy)
                    pDialogObject->pShutdownBlockReasonDestroy(pDialogObject->hWndDialog);

                pDialogObject->OnClose();
            }

            if (uMsg == WM_QUERYENDSESSION)
            {
                // WM_QUERYENDSESSION : If an application can terminate conveniently, it should return TRUE; otherwise, it should return FALSE
                return TRUE;
            }
        }
    case WM_QUIT:
    case WM_CLOSE:
        {
            CQuickWayToFolders* pDialogObject; 
            pDialogObject=CQuickWayToFolders::GetAssociatedObject(hWnd);
            if (!pDialogObject)
                break;

            if (!pDialogObject->EndSessionProcessed)
            {
                // query windows to update options before saving them
                pDialogObject->UpdateOptionsBeforeClose();
                // save folder options to store window position, size and maximized state
                pDialogObject->SaveFoldersOptions();

                pDialogObject->OnClose();
            }
        }
        break;
    default:
        return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
    }
    return 0;
}