/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#pragma once

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0501 // for xp os
#endif

#ifndef _WIN32_IE
#define _WIN32_IE 0x0500 // should be at least ie 5
#endif

#include <Windows.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)

#include <vector>
#include "resource.h"

#include "../Tools/File/StdFileOperations.h"
#include "../Tools/Ini/IniFileEx.h"
#include "../Tools/Ini/IniFileEx.cpp" // IniFileEx is a template class

#include "SingleFolderOptions.h"
#include "ApplicationOptions.h"
#include "Language.h"
#include "GenericQuickWayToFolderObject.h"
#include "LaunchBar.h"

#define CQuickWayToFolders_CLASS_NAME                        _T("CQuickWayToFoldersClassName")
#define CQuickWayToFolders_QUICK_WAY_TO_FOLDERS_SECTION_NAME _T("QuickWayToFolders")
#define CQuickWayToFolders_FOLDERS_SECTION_NAME              _T("Folders")
#define CQuickWayToFolders_LANGUAGE_SECTION_NAME             _T("Language")
#define CQuickWayToFolders_DefaultIconID                     IDI_ICON_DEFAULT_FOLDER_ICON
#define CQuickWayToFolders_SHOW_OPTIONS                      (WM_APP+2)
#define CQuickWayToFolders_MAX_PATH                          2048


class CQuickWayToFolders
{
    friend class COptionsUI;
    friend class CContainerBase;
    friend class CShellBrowserContainerBase;
    friend class CDesktopBrowserContainerXP;
    friend class CDesktopBrowserContainer;
protected:
    TCHAR ApplicationConfigFilePath[MAX_PATH];
    HINSTANCE hInstance;
    HWND hWndDialog;
    std::vector<CGenericQuickWayToFolderObject*> FoldersArray;
    CApplicationOptions* pApplicationOptions;
    CLanguage* pLanguage;
    HICON hDefaultIcon;
    BOOL bFirstLaunch;
    BOOL EndSessionProcessed;
	HMODULE hModuleUser32;
    CLaunchBar* pLaunchBar;

    BOOL LoadApplicationOptions();
    BOOL SaveApplicationOptions();
    BOOL LoadFoldersOptions();
    BOOL SaveFoldersOptions();
    BOOL UpdateOptionsBeforeClose();
    BOOL LoadLanguageFile();
    BOOL AddHotKeys();
    BOOL RemoveHotKeys();
    BOOL DestroyAssociatedQuickWayToFoldersItems();
    BOOL CreateAssociatedQuickWayToFoldersItems();
    BOOL QueryNotWritableConfigurationFileRetry(TCHAR* FilePath);

    BOOL WaitApplicationEnd();
    static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static CQuickWayToFolders* GetAssociatedObject(HWND hWndDialog);
    HICON GetDefaultIconHandle();
    void OnInit();
    void OnClose();

    typedef BOOL (WINAPI *pfShutdownBlockReasonCreate)(HWND hWnd,LPCWSTR pwszReason);
    pfShutdownBlockReasonCreate pShutdownBlockReasonCreate;
    typedef BOOL (WINAPI *pfShutdownBlockReasonDestroy)(HWND hWnd);
    pfShutdownBlockReasonDestroy pShutdownBlockReasonDestroy;

public:
    CQuickWayToFolders(void);
    ~CQuickWayToFolders(void);

    BOOL Start(HINSTANCE hInstance);
    BOOL Exit();
};
