/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ShellBrowserContainer.h"

CShellBrowserContainer::CShellBrowserContainer(HINSTANCE hInstance,CQuickWayToFolders* pQuickWayToFolders):CShellBrowserContainerBase(hInstance,pQuickWayToFolders)
{
    this->bRefreshing = FALSE;
}

CShellBrowserContainer::~CShellBrowserContainer(void)
{

}

void CShellBrowserContainer::Destroy()
{
    delete this;
}


void CShellBrowserContainer::RestoreAllItemsPositions()
{
    // remove auto-arrange style which can be added some times
    // this is required for Desktop view and folder with option "Save icons position for home folder"
    LONG_PTR Style=GetWindowLongPtr(this->hWndListView,GWL_STYLE);
    if (Style & LVS_AUTOARRANGE)
    {
        // SHELL_VIEW_AUTO_ARRANGE_SEVEN required for seven (else browser has auto arrange style)
        // LVS_AUTOARRANGE flag remove too early and set against next ???
        ::SendMessage(this->hWndShellView,WM_COMMAND,SHELL_VIEW_AUTO_ARRANGE_SEVEN,0);

        Style&=~LVS_AUTOARRANGE;
        SetWindowLongPtr(this->hWndListView,GWL_STYLE,Style);
    }

    // already managed by CShellBrowser::ShellBrowserViewToFolderSetting with pfs->fFlags |= FWF_NOHEADERINALLVIEWS
    //if ( (Style & LVS_REPORT) == 0 )
    //{
    //    Style |= LVS_NOCOLUMNHEADER;
    //    SetWindowLongPtr(this->hWndListView,GWL_STYLE,Style);
    //}
    

    this->CShellBrowserContainerBase::RestoreAllItemsPositions();
}

BOOL CShellBrowserContainer::Refresh()
{
    // assume previous refresh has ended
    if (this->bRefreshing)
        return FALSE;

    this->bRefreshing = TRUE;

    if (this->DoesIconsPositionsNeedToBeSaved())
    {
        this->SaveIconsPositions();
        this->pShellBrowserEx->RecreateCurrentView(); // required on seven (else refresh doesn't send the LVN_GETDISPINFO notification, and we can't reposition icons)
    }
    else
    {
        // refresh shell control
        this->pShellBrowserEx->Refresh();
    }

    this->bRefreshing = FALSE;
    return TRUE;
}

void CShellBrowserContainer::InstallSubProc()
{
    if (this->bSubProcInstalled)
        return;

    UINT_PTR uIdSubclass=(UINT_PTR)this;
    DWORD_PTR dwRefData=(DWORD_PTR)this;

    ::SetWindowSubclass(this->hWndShellView,CShellBrowserContainer::SubClassProcShellView,uIdSubclass,dwRefData);
}

void CShellBrowserContainer::RemoveSubProc()
{

    if (!this->bSubProcInstalled)
        return;

    // remove subclassing (do it first to avoid next messages to be trashed)
    UINT_PTR uIdSubclass=(UINT_PTR)this;
    ::RemoveWindowSubclass(this->hWndShellView,CShellBrowserContainer::SubClassProcShellView,uIdSubclass);

#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
    this->StopRestoreAllItemsPositions();
#endif
}

//-----------------------------------------------------------------------------
// Name: SubClassProcShellView
// Object: shellview subclassing proc
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CShellBrowserContainer::SubClassProcShellView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData)
{
    UNREFERENCED_PARAMETER(uIdSubclass);
    CShellBrowserContainer* pShellBrowserContainer=(CShellBrowserContainer*)dwRefData;

    switch(uMsg)
    {
#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
    case WM_TIMER:
        if (wParam == CShellBrowserContainer_RESTORE_HOME_ICON_POSITION_NEEDED)
        {
            pShellBrowserContainer->RestoreAllItemsPositionsInternal(pShellBrowserContainer->RestoreAllItemsNbRetry,pShellBrowserContainer->RestoreAllItemsLastNbItems);
            // WM_TIMER : An application should return zero if it processes this message
            return 0;
        }
        break;
#else
    case CShellBrowserContainer_RESTORE_HOME_ICON_POSITION_NEEDED:
        pShellBrowserContainer->RestoreAllItemsPositionsInternal((SIZE_T)wParam,(int)lParam);
        return 0; // do not call DefSubclassProc
        // break;
#endif
    }
    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
}
