/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ShellBrowserContainerBase.h"
#include "QuickWayToFolders.h"
#include "../Tools/gui/DesktopWorkArea/DesktopWorkArea.h"
#include "..\Tools\ShellExecute\ShellExecuteAndBringToTop.h"

#ifndef GET_X_LPARAM
#define GET_X_LPARAM(lp)                        ((int)(short)LOWORD(lp))
#endif
#ifndef GET_Y_LPARAM
#define GET_Y_LPARAM(lp)                        ((int)(short)HIWORD(lp))
#endif

// #define TRACE_POSITION

#ifdef TRACE_POSITION
#pragma message("WARNING TRACE_POSITION SET !!!!! $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$")
#endif

CShellBrowserContainerBase::CShellBrowserContainerBase(HINSTANCE hInstance,CQuickWayToFolders* pQuickWayToFolders):CContainerBase(hInstance,pQuickWayToFolders)
{
    this->InsideViewMoveCounted=0;
    this->pSingleFolderOptions=NULL;
    this->pShellBrowserEx=NULL;
    this->bRememberLastPosition=FALSE;
    this->hWndContainer=NULL;
    this->pTransparentToolbar=NULL;
    this->bInsideMaximizeReduce=FALSE;
    this->bOperationAfterResolutionChangedMustBeDoneBeforeShow = TRUE;

    this->pMalloc=NULL;
    this->pidlDesktop=NULL;
    this->bIsHomeFolder = FALSE;
    *this->CurrentPath = 0;
    this->bRestoringView = FALSE;
    this->DisabledWM_MOVE_Counted = 0;
    this->bSubProcInstalled = FALSE;
    this->hWndListView = 0;
    this->hWndShellView = 0;
    this->bRestoringItemsPositions = TRUE;
    this->bItemsHasBeenSorted = FALSE;
    this->pHotWindow = NULL;

#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
    this->RestoreAllItemsNbRetry = CShellBrowserContainer_DEFAULT_NB_SET_POS_RETRY;
    this->RestoreAllItemsLastNbItems = 0;
#endif

    this->GetDesktopPidl();
    this->UpdateMaximizedRects();
}

CShellBrowserContainerBase::~CShellBrowserContainerBase(void)
{
    if (this->pShellBrowserEx)
    {
        delete this->pShellBrowserEx;
        this->pShellBrowserEx=NULL;
    }

    // free desktop pidl
    if (this->pidlDesktop)
        pMalloc->Free(this->pidlDesktop);

    if (this->pMalloc)
        pMalloc->Release();

    this->DestroyContainer();
}

BOOL CShellBrowserContainerBase::ReturnHandleToOwnerWindowAfterShow()
{
    return FALSE;
}

BOOL CShellBrowserContainerBase::SetLanguage(CLanguage* pLanguage)
{
    if (!this->CContainerBase::InternalSetLanguage(pLanguage,TRUE))
        return FALSE;

    int MenuId;
    UINT MenuIndex;

    /////////////////////////////////
    // replace "show" by "show / hide" to menu
    /////////////////////////////////

    // Iterate over the map.
    // Using a const_iterator since we are not going to change the values.
    std::map<UINT,tagCommonMenuMapContent>::const_iterator it;
    MenuId =-1;
    for( it = this->CommonPopUpMenuMap.begin(); it != this->CommonPopUpMenuMap.end(); ++it)
    {
        if (it->second == COMMON_MENU_MAP_DATA_SHOW)
        {
            MenuId = it->first;
            break;
        }
    }
    if (MenuId == -1)
        return FALSE;
    
    TCHAR MenuText[MAX_PATH];
    _sntprintf(MenuText,MAX_PATH,_T("%s / %s"),this->pLanguage->Show,this->pLanguage->Hide);
    this->pPopUpMenu->SetText(MenuId,MenuText);
    
    /////////////////////////////////
    // add maximize reduce to menu after Show /Hide
    /////////////////////////////////

    // get menu "show / hide" menu index
    MenuIndex = this->pPopUpMenu->GetIndex(MenuId);

    _sntprintf(MenuText,MAX_PATH,_T("%s / %s"),this->pLanguage->Maximize,this->pLanguage->Reduce);
    MenuId=this->pPopUpMenu->Add(MenuText,IDI_ICON_REDUCE_SMALL,this->hInstance,MenuIndex+1);
    this->CommonPopUpMenuMap[MenuId]=COMMON_MENU_MAP_DATA_MAXIMIZE_REDUCE;

    return TRUE;
}

BOOL CShellBrowserContainerBase::DoesIconsPositionsNeedToBeSaved(LPITEMIDLIST CurrentPidl,LPITEMIDLIST ToBeBrowsePidl)
{
    // if no folder change
    if (::ILIsEqual(CurrentPidl,ToBeBrowsePidl))
        return FALSE;

    return this->DoesIconsPositionsNeedToBeSaved();
}

BOOL CShellBrowserContainerBase::UpdateOptionsBeforeClose()
{
    if (this->DoesIconsPositionsNeedToBeSaved())
        this->SaveIconsPositions();
    return TRUE;
}

BOOL CShellBrowserContainerBase::DoesIconsPositionsNeedToBeSaved()
{
    if (!this->pSingleFolderOptions)
        return FALSE;

    // option to save items positions must set
    if (!this->pSingleFolderOptions->bSaveIconsPositionsForHomeFolder)
        return FALSE;

    // detail and list views can't be reordered
    if ( (this->pSingleFolderOptions->FolderViewMode == CShellBrowser::SHELL_BROWSER_VIEW_DETAIL)
        || (this->pSingleFolderOptions->FolderViewMode == CShellBrowser::SHELL_BROWSER_VIEW_LIST)
        || (this->pSingleFolderOptions->FolderViewMode == CShellBrowser::SHELL_BROWSER_VIEW_THUMBSTRIP)
        )
        return FALSE;

    // we should be in home folder
    if (!this->bIsHomeFolder)
        return FALSE;

    if (this->ShellBrowserView != this->pSingleFolderOptions->FolderViewMode)
        return FALSE;

    if (this->bItemsHasBeenSorted)
        return FALSE;

    return TRUE;
}

BOOL CShellBrowserContainerBase::DoesIconsPositionsNeedToBeRestored()
{
    if (!this->pSingleFolderOptions)
        return FALSE;

    // option to save items positions must set
    if (!this->pSingleFolderOptions->bSaveIconsPositionsForHomeFolder)
        return FALSE;

    // detail and list views can't be reordered
    if ( (this->pSingleFolderOptions->FolderViewMode == CShellBrowser::SHELL_BROWSER_VIEW_DETAIL)
        || (this->pSingleFolderOptions->FolderViewMode == CShellBrowser::SHELL_BROWSER_VIEW_LIST)
        || (this->pSingleFolderOptions->FolderViewMode == CShellBrowser::SHELL_BROWSER_VIEW_THUMBSTRIP)
        )
        return FALSE;

    // we should be in home folder
    if (!this->bIsHomeFolder)
        return FALSE;

    return TRUE;
}

BOOL CShellBrowserContainerBase::SaveIconsPositions()
{
    if (!this->pSingleFolderOptions)
        return FALSE;

    // store icon positions in vector
    LRESULT NbItemCount = ::SendMessage(this->hWndListView,LVM_GETITEMCOUNT,0,0);
    LONG_PTR Cnt;
    CSingleFolderOptions::VECTOR_ICONS_POSITIONS_FOR_HOME_FOLDER VectorItem;
    
    LVITEM Item;
    memset(&Item,0,sizeof(LVITEM));
    Item.mask = LVIF_TEXT;
    Item.cchTextMax = _countof(VectorItem.Name);
    Item.iSubItem = 0;
    Item.pszText = VectorItem.Name;

    // clear vector content
    this->pSingleFolderOptions->pIconsPositionsForHomeFolder->clear();

    // for each listview items
    for (Cnt = 0; Cnt < NbItemCount ; Cnt++)
    {
        // get position
        ::SendMessage(hWndListView,LVM_GETITEMPOSITION,Cnt,(LPARAM)&VectorItem.Position);

        // get name
        Item.iItem = (int)Cnt;
        ::SendMessage(hWndListView,LVM_GETITEMTEXT,Cnt,(LPARAM)&Item);

        // add item to vector
        this->pSingleFolderOptions->pIconsPositionsForHomeFolder->push_back(VectorItem);
    }
    
    return TRUE;
}

BOOL CShellBrowserContainerBase::StoreBrowserView(CShellBrowser::tagShellBrowserView NewView)
{
    if (!this->bRestoringView)
        this->ShellBrowserView = NewView;
    return TRUE;
}

BOOL CShellBrowserContainerBase::IsHomeFolder()
{
    if (!this->pSingleFolderOptions)
        return TRUE;
    if (*this->CurrentPath == 0) // no navigation has been done, first show of folder --> home folder
        return TRUE;

    TCHAR szPath[MAX_PATH];
    this->pSingleFolderOptions->GetPath(szPath,_countof(szPath));
    return (_tcsicmp(szPath,this->CurrentPath) == 0);
}

BOOL CShellBrowserContainerBase::ShouldTransparentToolbarBeShown()
{
    if (!this->pSingleFolderOptions)
        return TRUE;
    if (this->pSingleFolderOptions->bHideToolBar)
        return FALSE;
    return !( (this->IsHomeFolder()) 
              && (this->pSingleFolderOptions->bHideToolBarForHomeFolder)
            );
}

void CShellBrowserContainerBase::UpdateMaximizedRects()
{
    RECT WindowRect;
    // get rect without shadow on win 10 (DO NOT use ::GetWindowRect(this->hWndContainer,&ContainerRect); which will contain shadow)
    this->DialogWindowRect.GetWindowRectWithoutShadow(this->hWndContainer,&WindowRect);
    RECT MonitorRect;
    DWORD MonitorIndex;
    this->GetMonitorForRect(&WindowRect,&MonitorIndex,&MonitorRect);

    this->MaximizedRect.right=MonitorRect.right;
    this->MaximizedRect.left=MonitorRect.left;
    this->MaximizedRect.bottom=MonitorRect.bottom;
    this->MaximizedRect.top=MonitorRect.top;

    if (this->pTransparentToolbar 
        && (this->pTransparentToolbar->GetBarPositionMode() == CTransparentToolbar::BarPositionMode_POPUP)
       )
    {
        if ( this->ShouldTransparentToolbarBeShown() )
        {
            DWORD ToolbarHeight = this->pTransparentToolbar? this->pTransparentToolbar->GetHeight():CBrowserToolbar_ICON_SIZE;
            this->MaximizedRect.top=MonitorRect.top+ToolbarHeight;
        }
    }

#ifdef TRACE_POSITION
    {
        TCHAR Msg[512];
        _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::UpdateMaximizedRects: MonitorRect Rect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
            MonitorRect.left,MonitorRect.top,MonitorRect.right,MonitorRect.bottom);
        ::OutputDebugString(Msg);
    }
#endif
}


BOOL CShellBrowserContainerBase::CreateContainer()
{
    // destroy previous container if any
    this->DestroyContainer();

    WNDCLASSEX wc;
    RECT WindowRect;
    RECT AdjustedRect;
    DWORD WindowStyle;
    DWORD WindowStyleEx;

    WindowStyle= WS_POPUP | WS_THICKFRAME; // MSDN : Note that you cannot specify the WS_OVERLAPPED style.
    WindowStyleEx=/*WS_EX_TOPMOST |*/ WS_EX_TOOLWINDOW;

    // if CShellBrowserContainer_CLASS_NAME is not already registered
    if (!::GetClassInfoEx(this->hInstance, CShellBrowserContainer_CLASS_NAME, &wc))
    {
        // register it

        wc.cbSize        = sizeof(WNDCLASSEX);
        wc.style         = 0;
        wc.lpfnWndProc   = CShellBrowserContainerBase::WndProc;
        wc.cbClsExtra    = 0;
        wc.cbWndExtra    = 0;
        wc.hInstance     = this->hInstance;
        wc.hIcon         = this->hIcon;
        wc.hCursor       = ::LoadCursor(NULL, IDC_ARROW);
        wc.hbrBackground = (HBRUSH)COLOR_WINDOWFRAME;
        wc.lpszMenuName  = NULL;
        wc.lpszClassName = CShellBrowserContainer_CLASS_NAME;
        wc.hIconSm       = this->hIcon;
        if(!::RegisterClassEx(&wc))
        {
            // ::MessageBox(NULL, _T("Window Registration Failed"), _T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
            return FALSE;
        }
    }

    memcpy(&WindowRect,&this->pSingleFolderOptions->BrowserRect,sizeof(RECT));
    ::OffsetRect(&WindowRect,this->pSingleFolderOptions->WindowPosition.x,this->pSingleFolderOptions->WindowPosition.y);

    // assume window is fully visible (assume to be visible even after a resolution change)
    RECT ShadowRect;
    memset(&ShadowRect,0,sizeof(RECT));
    this->AssumeWindowVisibility(&WindowRect,&ShadowRect);

    memcpy(&AdjustedRect,&WindowRect,sizeof(RECT));
    // since v1.3.12 AdjustWindowRectEx is no more required as stored rect is the container window outer rect
    // ::AdjustWindowRectEx(&AdjustedRect,WindowStyle,FALSE,WindowStyleEx);

    // create the dialog window
    this->hWndContainer = ::CreateWindowEx(WindowStyleEx,
                                            CShellBrowserContainer_CLASS_NAME,
                                            _T(""),
                                            WindowStyle,
                                            AdjustedRect.left,
                                            AdjustedRect.top, 
                                            AdjustedRect.right-AdjustedRect.left, 
                                            AdjustedRect.bottom-AdjustedRect.top,
                                            HWND_DESKTOP,
                                            NULL,
                                            this->hInstance,
                                            this);

    if(this->hWndContainer == NULL)
    {
        // ::MessageBox(NULL, _T("Window Creation Failed"), _T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    // DwmGetWindowAttribute / GetWindowRectWithoutShadow bug workaround (troubles until window is shown)
    ::ShowWindow(this->hWndContainer,SW_SHOWNORMAL);
    ::ShowWindow(this->hWndContainer,SW_HIDE);
    // end of bug workaround

    switch (this->pSingleFolderOptions->ZOrder)
    {
    case CSingleFolderOptions::BROWSER_Z_ORDER_BOTTOM:
        {
            // ::SetWindowPos(this->hWndContainer,HWND_BOTTOM,0,0,0,0,SWP_NOSIZE | SWP_NOMOVE);
            HWND hWnd = ::FindWindow(L"Progman", L"Program Manager");
            // HWND ProgmanHwnd = ::FindWindow(_T("ProgMan"), NULL);
            HWND hShellView = ::FindWindowEx(hWnd,NULL,_T("SHELLDLL_DefView"),NULL);

            // bug solved QuickWayToFolder 1.2.8
            // in case of use of direct X wallpaper _T("Progman") doesn't contain _T("SHELLDLL_DefView")
            if (hShellView==0)
            {
                // use of DirectX --> ::FindWindow(_T("WorkerW"), NULL); // trouble : multiple WorkerW
                HWND hWndDesktop = ::GetDesktopWindow();
                HWND hWndChild = 0;
                for(;;)
                {
                    hWndChild = ::FindWindowEx( hWndDesktop, hWndChild,  _T("WorkerW"),  NULL );
                    if (!hWndChild)
                        return FALSE;
                    hShellView = ::FindWindowEx(hWndChild,NULL,_T("SHELLDLL_DefView"),NULL);

                    if (hShellView)
                        break;
                }
            }

            // find associated list view control
            HWND hListView = ::FindWindowEx(hShellView,NULL,WC_LISTVIEW,NULL);
            if (!::IsWindowVisible(hListView))
            {
                hListView = ::FindWindowEx(hShellView,NULL,CShellBrowser_ThumbnailVwExtWnd32,NULL);
                hListView = ::FindWindowEx(hListView,NULL,WC_LISTVIEW,NULL);
            }

            // use hListView instead of ProgmanHwnd as parent window to avoid to be taken as the real desktop
            ::SetParent(this->hWndContainer, hListView);

        }

    	break;
    case CSingleFolderOptions::BROWSER_Z_ORDER_TOP_MOST:
        ::SetWindowPos(this->hWndContainer,HWND_TOPMOST,0,0,0,0,SWP_NOSIZE | SWP_NOMOVE);
        break;
    }
    
    /////////////////////////////////////////////////////
    // ToolBar
    /////////////////////////////////////////////////////
    CBrowserToolbarOptions* pToolbarOptions = &(this->pQuickWayToFolders->pApplicationOptions->BrowserToolBarOptions);
    int NbToolBarItems = CShellBrowserContainer_TRANSPARENT_TOOLBAR_BUTTONS_NUMBER_IN_NO_BROWSING_MODE;
    int ToolBarIndex=0;
    
    memset(&this->TransparentToolbarMoveButton,0,sizeof(this->TransparentToolbarMoveButton));
    this->TransparentToolbarMoveButton.ButtonId=TransparentToolbarButtonsId_MOVE;
    this->TransparentToolbarMoveButton.Enabled= TRUE;
    this->TransparentToolbarMoveButton.Position=CTransparentToolbar::ButtonPosition_CENTER_OR_MIDDLE;
    this->TransparentToolbarMoveButton.hIcon=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_Move);
    this->TransparentToolbarMoveButton.hIconHot=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_MoveHot);
    this->TransparentToolbarMoveButton.hIconDown=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_MoveDown);
    this->TransparentToolbarMoveButton.hIconDisabled=this->TransparentToolbarMoveButton.hIcon;// no disable state
    _tcscpy(this->TransparentToolbarMoveButton.ToolTipText,this->pLanguage->Move);

    memset(&this->TransparentToolbarArrayButtons,0,sizeof(this->TransparentToolbarArrayButtons));
    if (this->pSingleFolderOptions->bAllowSubFoldersNavigation)
    {
        NbToolBarItems = CShellBrowserContainer_TRANSPARENT_TOOLBAR_BUTTONS_NUMBER;

        this->TransparentToolbarArrayButtons[ToolBarIndex].ButtonId=TransparentToolbarButtonsId_HOME;
        this->TransparentToolbarArrayButtons[ToolBarIndex].Enabled=TRUE;
        this->TransparentToolbarArrayButtons[ToolBarIndex].Position=CTransparentToolbar::ButtonPosition_LEFT_OR_TOP;
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIcon=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_Home);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconHot=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_HomeHot);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDown=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_HomeDown);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDisabled=this->TransparentToolbarArrayButtons[ToolBarIndex].hIcon;// no disable state
        _tcscpy(this->TransparentToolbarArrayButtons[ToolBarIndex].ToolTipText,this->pLanguage->GoToHomeFolder);
        ToolBarIndex++;

        this->TransparentToolbarArrayButtons[ToolBarIndex].ButtonId=TransparentToolbarButtonsId_PREV;
        this->TransparentToolbarArrayButtons[ToolBarIndex].Enabled=TRUE;
        this->TransparentToolbarArrayButtons[ToolBarIndex].Position=CTransparentToolbar::ButtonPosition_LEFT_OR_TOP;
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIcon=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_Back);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconHot=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_BackHot);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDown=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_BackDown);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDisabled=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_BackDisabled);
        _tcscpy(this->TransparentToolbarArrayButtons[ToolBarIndex].ToolTipText,this->pLanguage->GoBack);
        ToolBarIndex++;

        this->TransparentToolbarArrayButtons[ToolBarIndex].ButtonId=TransparentToolbarButtonsId_NEXT;
        this->TransparentToolbarArrayButtons[ToolBarIndex].Enabled=TRUE;
        this->TransparentToolbarArrayButtons[ToolBarIndex].Position=CTransparentToolbar::ButtonPosition_LEFT_OR_TOP;
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIcon=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_Forward);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconHot=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_ForwardHot);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDown=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_ForwardDown);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDisabled=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_ForwardDisabled);
        _tcscpy(this->TransparentToolbarArrayButtons[ToolBarIndex].ToolTipText,this->pLanguage->GoForward);
        ToolBarIndex++;

        this->TransparentToolbarArrayButtons[ToolBarIndex].ButtonId=TransparentToolbarButtonsId_UP;
        this->TransparentToolbarArrayButtons[ToolBarIndex].Enabled=TRUE;
        this->TransparentToolbarArrayButtons[ToolBarIndex].Position=CTransparentToolbar::ButtonPosition_LEFT_OR_TOP;
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIcon=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_Up);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconHot=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_UpHot);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDown=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_UpDown);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDisabled=this->TransparentToolbarArrayButtons[ToolBarIndex].hIcon;// no disable state
        _tcscpy(this->TransparentToolbarArrayButtons[ToolBarIndex].ToolTipText,this->pLanguage->GoUp);
        ToolBarIndex++;

        this->TransparentToolbarArrayButtons[ToolBarIndex].ButtonId=TransparentToolbarButtonsId_JUMP_TO_EXPLORER;
        this->TransparentToolbarArrayButtons[ToolBarIndex].Enabled=TRUE;
        this->TransparentToolbarArrayButtons[ToolBarIndex].Position=CTransparentToolbar::ButtonPosition_LEFT_OR_TOP;
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIcon=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_JumpToExplorer);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconHot=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_JumpToExplorerHot);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDown=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_JumpToExplorerDown);
        this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDisabled=this->TransparentToolbarArrayButtons[ToolBarIndex].hIcon;// no disable state
        _tcscpy(this->TransparentToolbarArrayButtons[ToolBarIndex].ToolTipText,this->pLanguage->OpenFolderWithWindowsExplorer);
        ToolBarIndex++;

    }

    this->TransparentToolbarArrayButtons[ToolBarIndex].ButtonId=TransparentToolbarButtonsId_MAXIMIZE;
    this->TransparentToolbarArrayButtons[ToolBarIndex].Enabled=TRUE;
    this->TransparentToolbarArrayButtons[ToolBarIndex].Position=CTransparentToolbar::ButtonPosition_RIGHT_OR_BOTTOM;
    this->TransparentToolbarArrayButtons[ToolBarIndex].hIcon=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_Maximize);
    this->TransparentToolbarArrayButtons[ToolBarIndex].hIconHot=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_MaximizeHot);
    this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDown=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_MaximizeDown);
    this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDisabled=this->TransparentToolbarArrayButtons[ToolBarIndex].hIcon;// no disable state
    _sntprintf(this->TransparentToolbarArrayButtons[ToolBarIndex].ToolTipText,CTransparentToolbar_ToolTipTextLen,_T("%s / %s"),this->pLanguage->Maximize,this->pLanguage->Reduce);
    this->TransparentToolbarArrayButtons[ToolBarIndex].ToolTipText[CTransparentToolbar_ToolTipTextLen-1]=0;
    ToolBarIndex++;

    this->TransparentToolbarArrayButtons[ToolBarIndex].ButtonId=TransparentToolbarButtonsId_CLOSE;
    this->TransparentToolbarArrayButtons[ToolBarIndex].Enabled=TRUE;
    this->TransparentToolbarArrayButtons[ToolBarIndex].Position=CTransparentToolbar::ButtonPosition_RIGHT_OR_BOTTOM;
    this->TransparentToolbarArrayButtons[ToolBarIndex].hIcon=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_Close);
    this->TransparentToolbarArrayButtons[ToolBarIndex].hIconHot=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_CloseHot);
    this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDown=pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_CloseDown);
    this->TransparentToolbarArrayButtons[ToolBarIndex].hIconDisabled=this->TransparentToolbarArrayButtons[ToolBarIndex].hIcon;// no disable state
    _tcscpy(this->TransparentToolbarArrayButtons[ToolBarIndex].ToolTipText,this->pLanguage->Close);
    ToolBarIndex++;
    RECT Margins = {0,0,0,0};

    CTransparentToolbar::tagBarPositionMode PositionMode;
    PositionMode = pToolbarOptions->GetCurrentSkinPositionMode();
    LONG xPos;
    LONG yPos;
    LONG Width;
    if (PositionMode == CTransparentToolbar::BarPositionMode_POPUP)
    {
        // get rect without shadow on win 10 (DO NOT use ::GetWindowRect(this->hWndContainer,&ContainerRect); which will contain shadow)
        this->DialogWindowRect.GetWindowRectWithoutShadow(this->hWndContainer,&WindowRect);

        xPos = WindowRect.left;
        yPos = WindowRect.top-CBrowserToolbar_ICON_SIZE;
        Width = WindowRect.right-WindowRect.left; // external rect width
    }
    else
    {
        xPos = 0;
        yPos = 0;
        RECT TmpRect;
        ::GetClientRect(this->hWndContainer,&TmpRect);
        Width = TmpRect.right-TmpRect.left; // internal rect width
    }

    this->DisabledWM_MOVE_Counted++;
    this->pTransparentToolbar=new CTransparentToolbar(this->hInstance,
                                                      this->hWndContainer,
                                                      this,
                                                      FALSE,
                                                      FALSE,
                                                      pToolbarOptions->GetCurrentSkinBackGroundType(),
                                                      pToolbarOptions->GetCurrentSkinBackGroundColor(),
                                                      pToolbarOptions->GetCurrentSkinBackGroundColor(),
                                                      pToolbarOptions->GetCurrentSkinBackGroundColor2(),
                                                      pToolbarOptions->GetCurrentSkinBackGroundImage(),
                                                      pToolbarOptions->GetCurrentSkinTransparentColor(),
                                                      pToolbarOptions->GetCurrentSkinUseTransparentColor(),
                                                      pToolbarOptions->GetCurrentSkinOpacityPercent(),
                                                      pToolbarOptions->GetCurrentSkinPositionMode(),
                                                      xPos, // x
                                                      yPos, // y
                                                      Width, // width
                                                      CBrowserToolbar_ICON_SIZE, // height
                                                      CBrowserToolbar_ICON_SIZE,
                                                      CBrowserToolbar_ICON_SIZE,
                                                      this->TransparentToolbarArrayButtons,
                                                      NbToolBarItems,
                                                      &this->TransparentToolbarMoveButton,
                                                      pToolbarOptions->GetCurrentSkinMoveButtonEnableState(),
                                                      TRUE,
                                                      &Margins,FALSE
                                                      );
    this->DisabledWM_MOVE_Counted--;

    this->pTransparentToolbar->EnableButton(TransparentToolbarButtonsId_PREV,FALSE);
    this->pTransparentToolbar->EnableButton(TransparentToolbarButtonsId_NEXT,FALSE);

    // update maximized rect after toolbar creation (maximized rect depends of toolbar mode popup/child)
    this->UpdateMaximizedRects();

    /////////////////////////////////////////////////////
    // Hot Corner (auto showing)
    /////////////////////////////////////////////////////
    if (this->pSingleFolderOptions->bEnableHotCorner)
    {
        RECT MonitorArea;
        RECT ActiveRect;
        // use full monitor rect instead of work area rect for hot corners
        CDesktopWorkArea::GetWorkAreaForMonitor(this->pSingleFolderOptions->HotCornerScreenIndex,&MonitorArea,TRUE);
        switch(this->pSingleFolderOptions->HotCornerPosition)
        {
        case CSingleFolderOptions::HOT_CORNER_POSITION_TOP_LEFT:
            ActiveRect.left = MonitorArea.left;
            ActiveRect.right = ActiveRect.left+this->pSingleFolderOptions->HotCornerWidth;
            ActiveRect.top = MonitorArea.top;
            ActiveRect.bottom = ActiveRect.top+this->pSingleFolderOptions->HotCornerHeight;
            break;
        case CSingleFolderOptions::HOT_CORNER_POSITION_TOP_RIGHT:
            ActiveRect.right = MonitorArea.right;
            ActiveRect.left = ActiveRect.right-this->pSingleFolderOptions->HotCornerWidth;
            ActiveRect.top = MonitorArea.top;
            ActiveRect.bottom = ActiveRect.top+this->pSingleFolderOptions->HotCornerHeight;
            break;
        case CSingleFolderOptions::HOT_CORNER_POSITION_BOTTOM_LEFT:
            ActiveRect.left = MonitorArea.left;
            ActiveRect.right = ActiveRect.left+this->pSingleFolderOptions->HotCornerWidth;
            ActiveRect.bottom = MonitorArea.bottom;
            ActiveRect.top = ActiveRect.bottom-this->pSingleFolderOptions->HotCornerHeight;
            break;
        case CSingleFolderOptions::HOT_CORNER_POSITION_BOTTOM_RIGHT:
            ActiveRect.right = MonitorArea.right;
            ActiveRect.left = ActiveRect.right-this->pSingleFolderOptions->HotCornerWidth;
            ActiveRect.bottom = MonitorArea.bottom;
            ActiveRect.top = ActiveRect.bottom-this->pSingleFolderOptions->HotCornerHeight;
            break;
        }

        this->pHotWindow = new CHotWindow((IHotWindowEvents*)((CHotWindowEventsBase*)this));
        this->pHotWindow->SetWindowAndActiveRect(this->hWndContainer,&ActiveRect);
        this->pHotWindow->StartAutoShow();
    }

    return TRUE;
}

BOOL CShellBrowserContainerBase::DestroyContainer()
{
    if (!this->hWndContainer)
        return TRUE;

    if (this->pHotWindow)
    {
        delete this->pHotWindow;
        this->pHotWindow = NULL;
    }

    if (this->pTransparentToolbar)
    {
        delete this->pTransparentToolbar;
        this->pTransparentToolbar=NULL;
    }

    ::DestroyWindow(this->hWndContainer);
    this->hWndContainer=NULL;

    return TRUE;
}

void CShellBrowserContainerBase::OnMove(WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(wParam);
    UNREFERENCED_PARAMETER(lParam);

    if (this->bInsideMaximizeReduce)
        return;
    if (this->InsideViewMoveCounted>0)
        return;
    if (!this->pTransparentToolbar)
        return;

    CTransparentToolbar::tagBarPositionMode BarPositionMode;
    BarPositionMode = this->pTransparentToolbar->GetBarPositionMode();
    if (BarPositionMode == CTransparentToolbar::BarPositionMode_CHILD)
        return;

    this->InsideViewMoveCounted++;

    // not usable due to window decoration --> use this->DialogWindowRect.GetWindowRectWithoutShadow instead
    //LONG NewPositionX=GET_X_LPARAM(lParam);   // horizontal position
    //LONG NewPositionY=GET_Y_LPARAM(lParam);   // vertical position

    RECT Rect;
    // get rect without shadow on win 10 (DO NOT use ::GetWindowRect(this->hWndContainer,&ContainerRect); which will contain shadow)
    this->DialogWindowRect.GetWindowRectWithoutShadow(this->hWndContainer,&Rect);
    this->pTransparentToolbar->SetPosition(Rect.left,Rect.top-this->pTransparentToolbar->GetHeight(),Rect.right-Rect.left,this->pTransparentToolbar->GetHeight());
#ifdef TRACE_POSITION
    {
        TCHAR Msg[512];
        _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::OnMove SetPosition ContainerRect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
            Rect.left,Rect.top,Rect.right,Rect.bottom);
        ::OutputDebugString(Msg);

        ::GetWindowRect(this->pTransparentToolbar->GetControlHandle(),&Rect);
        _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::OnMove (After SetPosition): Real ContainerRect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
            Rect.left,Rect.top,Rect.right,Rect.bottom);
        ::OutputDebugString(Msg);
    }
#endif

    this->InsideViewMoveCounted--;
}

void CShellBrowserContainerBase::OnTransparentToolbarMove(CTransparentToolbar* pSender,LONG DeltaX,LONG DeltaY)
{
    CTransparentToolbar::tagBarPositionMode BarPositionMode;
    BarPositionMode = pSender->GetBarPositionMode();
    if (BarPositionMode == CTransparentToolbar::BarPositionMode_CHILD)
    {
            RECT ContainerRectWithShadow;
            ::GetWindowRect(this->hWndContainer,&ContainerRectWithShadow);
            // LONG Width = ContainerRectWithShadow.right - ContainerRectWithShadow.left;
            // LONG Height = ContainerRectWithShadow.bottom - ContainerRectWithShadow.top;
            ::SetWindowPos(this->hWndContainer,0,
                            ContainerRectWithShadow.left+DeltaX,
                            ContainerRectWithShadow.top+DeltaY,
                            0, // Width,
                            0, // Height,
                            SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOSIZE
                            );
    }
    else
    {
        if (this->bInsideMaximizeReduce)
            return;

        if (this->DisabledWM_MOVE_Counted<=0)
        {
            if (this->InsideViewMoveCounted>0)
                return;
            this->InsideViewMoveCounted++;

            RECT TransparentToolBarRect;
            ::GetWindowRect(pSender->GetControlHandle(),&TransparentToolBarRect); // transparent toolbar has no shadow
            RECT ContainerRectWithShadow;
            RECT ContainerRectWithoutShadow;
            ::GetWindowRect(this->hWndContainer,&ContainerRectWithShadow);
            this->DialogWindowRect.GetWindowRectWithoutShadow(this->hWndContainer,&ContainerRectWithoutShadow);
            LONG ShadowLeftWidth = ContainerRectWithoutShadow.left-ContainerRectWithShadow.left;
            LONG ShadowRightWidth = ContainerRectWithShadow.right-ContainerRectWithoutShadow.right;
            ::SetWindowPos(this->hWndContainer,0,
                            TransparentToolBarRect.left-ShadowLeftWidth,
                            TransparentToolBarRect.top+this->pTransparentToolbar->GetHeight(),
                            TransparentToolBarRect.right-TransparentToolBarRect.left+ShadowLeftWidth+ShadowRightWidth,
                            ContainerRectWithShadow.bottom-ContainerRectWithShadow.top,
                            SWP_NOACTIVATE | SWP_NOZORDER // | SWP_NOSIZE better to ensure size (avoid bad rendering in case of previous error)
                            );

#ifdef TRACE_POSITION
            {
                TCHAR Msg[512];

                _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::OnTransparentToolbarMove TransparentToolBarRect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                    TransparentToolBarRect.left,
                    TransparentToolBarRect.top,
                    TransparentToolBarRect.right,
                    TransparentToolBarRect.bottom
                    );
                ::OutputDebugString(Msg);

                _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::OnTransparentToolbarMove SetPosition ContainerRect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                    TransparentToolBarRect.left-ShadowLeftWidth,
                    TransparentToolBarRect.top+this->pTransparentToolbar->GetHeight(),
                    TransparentToolBarRect.right+ShadowRightWidth,
                    TransparentToolBarRect.bottom
                    );
                ::OutputDebugString(Msg);

                ::GetWindowRect(this->hWndContainer,&ContainerRectWithShadow);
                _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::OnTransparentToolbarMove (After SetPosition): Real ContainerRect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                    ContainerRectWithShadow.left,ContainerRectWithShadow.top,ContainerRectWithShadow.right,ContainerRectWithShadow.bottom);
                ::OutputDebugString(Msg);
            }
#endif

            this->InsideViewMoveCounted--;
        }
    }
}

void CShellBrowserContainerBase::OnTransparentToolbarMoveEnd(CTransparentToolbar* pSender)
{
    UNREFERENCED_PARAMETER(pSender);

    // store new window position in pSingleFolderOptions
    RECT Rect;
    // ::GetWindowRect(pSender->GetControlHandle(),&Rect);
    ::GetWindowRect(this->hWndContainer,&Rect);
    this->pSingleFolderOptions->WindowPosition.x=Rect.left;
    this->pSingleFolderOptions->WindowPosition.y=Rect.top;

    // restore focus to listview
    ::SetFocus(this->hWndListView);
    ::RedrawWindow(this->hWndListView,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME | RDW_ALLCHILDREN);
}

// OnClose : just use return 0; to avoid window closing (DefWindowProc calls DestroyWindow)
LRESULT CShellBrowserContainerBase::OnTransparentToolbarClose(CTransparentToolbar* pSender,WPARAM wParam,LPARAM lParam)
{
    UNREFERENCED_PARAMETER(pSender);
    UNREFERENCED_PARAMETER(wParam);
    UNREFERENCED_PARAMETER(lParam);
    this->OnClose();
    return 0;
}

// OnTransparentToolbarKeyDown: An application should return zero if it processes this message
LRESULT CShellBrowserContainerBase::OnTransparentToolbarKeyDown(CTransparentToolbar* pSender,WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(pSender);
    return this->OnKeyDown_ShellBrowserContainerBase(wParam,lParam);
}

LRESULT CShellBrowserContainerBase::OnTransparentToolbarMouseMediaButtonDown(CTransparentToolbar* pSender,WPARAM wParam, LPARAM lParam)
{
    return this->OnMessage(pSender->GetControlHandle(),WM_XBUTTONDOWN,wParam,lParam);
}

void CShellBrowserContainerBase::OnTransparentToolbarBackgroundLeftDoubleClick(CTransparentToolbar* pSender)
{
    UNREFERENCED_PARAMETER(pSender);

    // assume call will be made in the same thread
    // do the same as maximize reduce
    ::PostMessage(this->hWndContainer,CShellBrowserContainer_TOOLBAR_BUTTON_CLICK,0,TransparentToolbarButtonsId_MAXIMIZE);
}

void CShellBrowserContainerBase::OnTransparentToolbarButtonClick(CTransparentToolbar* pSender,DWORD ButtonId)
{
    UNREFERENCED_PARAMETER(pSender);
    // assume call will be made in the same thread
    ::PostMessage(this->hWndContainer,CShellBrowserContainer_TOOLBAR_BUTTON_CLICK,0,ButtonId);
}

void CShellBrowserContainerBase::OnTransparentToolbarButtonClick(DWORD ButtonId)
{
    switch(ButtonId)
    {
    case TransparentToolbarButtonsId_HOME:
        this->GoHome();
    	break;
    case TransparentToolbarButtonsId_PREV:
        if (FAILED(this->pShellBrowserEx->BrowseBack()))
            this->UpdateNavigateBackForwardState();// update back/forward buttons state. In case of failure (in case of success UpdateNavigateBackForwardState is called by BrowserAfterBrowseCallBack)
    	break;
    case TransparentToolbarButtonsId_NEXT:
        if (FAILED(this->pShellBrowserEx->BrowseForward()))
            this->UpdateNavigateBackForwardState();// update back/forward buttons state. In case of failure (in case of success UpdateNavigateBackForwardState is called by BrowserAfterBrowseCallBack)
        break;
    case TransparentToolbarButtonsId_UP:
        if (FAILED(this->pShellBrowserEx->BrowseParent()))
            this->UpdateNavigateBackForwardState();// update back/forward buttons state. In case of failure (in case of success UpdateNavigateBackForwardState is called by BrowserAfterBrowseCallBack)
        break;
    case TransparentToolbarButtonsId_MAXIMIZE:
        this->MaximizeReduce();
        break;
    case TransparentToolbarButtonsId_CLOSE:
        // close quick browser
        ::PostMessage(this->hWndContainer,WM_CLOSE,0,0);
        break;
    case TransparentToolbarButtonsId_JUMP_TO_EXPLORER:
        {
            TCHAR CurrentFolderPath[MAX_PATH];
            if (this->pShellBrowserEx->GetCurrentFolderPath(CurrentFolderPath))
                // ::ShellExecute(NULL, _T("explore"), CurrentFolderPath, NULL, NULL, SW_SHOWNORMAL);
                CShellExecuteAndBringToTop::ShellExecute(NULL,_T("explore"),CurrentFolderPath,NULL,NULL,SW_SHOWNORMAL);

            // close quick browser
            ::PostMessage(this->hWndContainer,WM_CLOSE,0,0);
        }

    default:
        break;
    }

    // restore focus to listview
    ::SetFocus(this->hWndListView);
}

void CShellBrowserContainerBase::MaximizeReduce()
{
    if (this->pSingleFolderOptions->bIsMaximized)
    {
        this->Reduce();
    }
    else
    {
        this->Maximize();
    }
}

void CShellBrowserContainerBase::Maximize()
{
    LONG_PTR WindowStyle;
    BOOL bSameRectSize;
    LONG lWidth;
    LONG hHeight;
    RECT ContainerRect;
    CTransparentToolbar::tagBarPositionMode PositionMode;
    CBrowserToolbarOptions* pToolbarOptions = &this->pQuickWayToFolders->pApplicationOptions->BrowserToolBarOptions;
    PositionMode = pToolbarOptions->GetCurrentSkinPositionMode();

    ::GetWindowRect(this->hWndContainer,&ContainerRect);

    if (!this->pSingleFolderOptions->bIsMaximized)
    {
        this->pSingleFolderOptions->WindowPosition.x=ContainerRect.left;
        this->pSingleFolderOptions->WindowPosition.y=ContainerRect.top;

        memcpy(&this->pSingleFolderOptions->BrowserRect,&ContainerRect,sizeof(RECT));
        ::OffsetRect(&this->pSingleFolderOptions->BrowserRect,
                    -this->pSingleFolderOptions->BrowserRect.left,
                    -this->pSingleFolderOptions->BrowserRect.top);
    }

    this->bInsideMaximizeReduce=TRUE;

    // avoid sizing
    WindowStyle = ::GetWindowLongPtr(this->hWndContainer,GWL_STYLE);
    ::SetWindowLongPtr(this->hWndContainer,GWL_STYLE,WindowStyle & (~WS_THICKFRAME) );

    // update working space
    this->UpdateMaximizedRects();

    this->ShowListviewBorder(FALSE);

    bSameRectSize = ::EqualRect(&this->MaximizedRect,&ContainerRect);
    lWidth = this->MaximizedRect.right-this->MaximizedRect.left;
    hHeight = this->MaximizedRect.bottom-this->MaximizedRect.top;

    // big trick : assume WM_SIZE message will be sent and shell browser layout rendering
    if (bSameRectSize)
        hHeight++;

    // if window was reduced , maximize it
    ::SetWindowPos(this->hWndContainer,0,
					this->MaximizedRect.left,
					this->MaximizedRect.top,
					lWidth,
					hHeight, 
					SWP_NOACTIVATE | SWP_NOZORDER );

#ifdef TRACE_POSITION
    {
        TCHAR MsgTrace[1024];
        _sntprintf(MsgTrace,_countof(MsgTrace),
                    TEXT("CShellBrowserContainerBase::Maximize SetWindowPos (x=%d, y=%d, w=%d, h=%d) \r\n"),
                    this->MaximizedRect.left,this->MaximizedRect.top,lWidth,hHeight
                    );
        ::OutputDebugString(MsgTrace);
    }
#endif

    if (PositionMode == CTransparentToolbar::BarPositionMode_POPUP)
    {
        // sometimes WM_SIZE message seems to be missed on some OS and transparent toolbar get incorrect size
        // --> ensure to resize transparent toolbar
        this->pTransparentToolbar->SetPosition(this->MaximizedRect.left,this->MaximizedRect.top-this->pTransparentToolbar->GetHeight(),this->MaximizedRect.right-this->MaximizedRect.left,this->pTransparentToolbar->GetHeight());
#ifdef TRACE_POSITION
        {
            TCHAR Msg[512];
            _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::Maximize SetPosition TransparentToolbar left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                this->MaximizedRect.left,this->MaximizedRect.top-this->pTransparentToolbar->GetHeight(),this->MaximizedRect.right,this->MaximizedRect.top+this->pTransparentToolbar->GetHeight());
            ::OutputDebugString(Msg);

            ::GetWindowRect(this->pTransparentToolbar->GetControlHandle(),&Rect);
            _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::Maximize (After SetPosition): Real TransparentToolbar left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                Rect.left,Rect.top,Rect.right,Rect.bottom);
            ::OutputDebugString(Msg);
        }
#endif
    }
    else
    {
        RECT ContainerClientRect;
        ::GetClientRect(this->hWndContainer,&ContainerClientRect);
        this->pTransparentToolbar->SetWidth(ContainerClientRect.right-ContainerClientRect.left);
    }

    this->bInsideMaximizeReduce=FALSE;

    // hide move button
    this->pTransparentToolbar->ShowMove(FALSE);

    // change maximize buttons to reduce buttons
    this->pTransparentToolbar->ChangeIcons(
                                            TransparentToolbarButtonsId_MAXIMIZE,
                                            pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_Reduce),
                                            pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_ReduceHot),
                                            pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_ReduceDown),
                                            pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_Reduce)
                                            );

    // switch maximized state
    this->pSingleFolderOptions->bIsMaximized=TRUE;
}

void CShellBrowserContainerBase::Reduce()
{
    RECT Rect;
    BOOL bSameRectSize;
    LONG lWidth;
    LONG hHeight;
    RECT ContainerRect;

    // on win10 get rect with shadow
    ::GetWindowRect(this->hWndContainer,&ContainerRect);

    LONG_PTR WindowStyle;
    LONG_PTR WindowStyleEx;
    // allow sizing again (change style before calling AdjustWindowRectEx)
    WindowStyle = ::GetWindowLongPtr(this->hWndContainer,GWL_STYLE);
    WindowStyle |=WS_THICKFRAME;
    ::SetWindowLongPtr(this->hWndContainer,GWL_STYLE,WindowStyle);

    WindowStyle &= ~WS_OVERLAPPED; // MSDN : Note that you cannot specify the WS_OVERLAPPED style for AdjustWindowRectEx
    WindowStyleEx= ::GetWindowLongPtr(this->hWndContainer,GWL_EXSTYLE);

    // if window was maximized, reduce it
    memcpy(&Rect,&this->pSingleFolderOptions->BrowserRect,sizeof(RECT));
    // AdjustWindowRectEx is no more required since outer rect is used (v1.3.12)
    // ::AdjustWindowRectEx(&Rect,(DWORD)WindowStyle,FALSE,(DWORD)WindowStyleEx); // get rect with shadow on win 10

    this->bInsideMaximizeReduce=TRUE;

    this->ShowListviewBorder(TRUE);

    bSameRectSize = ::EqualRect(&Rect,&ContainerRect);
    lWidth = Rect.right-Rect.left;
    hHeight = Rect.bottom-Rect.top;

    // big trick : assume WM_SIZE message will be sent and shell browser layout rendering
    if (bSameRectSize)
        hHeight--;

#ifdef TRACE_POSITION
    {
        TCHAR Msg[512];
        _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::Reduce (Before SetPosition)\r\n"));
        ::OutputDebugString(Msg);
    }
#endif

    // reduce window
    ::SetWindowPos(this->hWndContainer,0,
                    this->pSingleFolderOptions->WindowPosition.x,
                    this->pSingleFolderOptions->WindowPosition.y,
                    lWidth,
                    hHeight,
                    SWP_NOACTIVATE | SWP_NOZORDER );


    CTransparentToolbar::tagBarPositionMode PositionMode;
    CBrowserToolbarOptions* pToolbarOptions = &this->pQuickWayToFolders->pApplicationOptions->BrowserToolBarOptions;
    PositionMode = pToolbarOptions->GetCurrentSkinPositionMode();
    if (PositionMode == CTransparentToolbar::BarPositionMode_POPUP)
    {
        // get rect without shadow on win 10 (DO NOT use ::GetWindowRect(this->hWndContainer,&ContainerRect); which will contain shadow)
        this->DialogWindowRect.GetWindowRectWithoutShadow(this->hWndContainer,&ContainerRect);

        // sometimes WM_SIZE message seems to be missed on some OS and transparent toolbar get incorrect size
        // --> ensure to resize transparent toolbar
        this->pTransparentToolbar->SetPosition(ContainerRect.left,
                                               ContainerRect.top-this->pTransparentToolbar->GetHeight(),
                                               ContainerRect.right-ContainerRect.left,
                                               this->pTransparentToolbar->GetHeight()
                                               );
#ifdef TRACE_POSITION
        {
            TCHAR Msg[512];
            _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::Reduce SetPosition TransparentToolbar left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                ContainerRect.left,
                ContainerRect.top-this->pTransparentToolbar->GetHeight(),
                ContainerRect.right,
                ContainerRect.top
                );
            ::OutputDebugString(Msg);

            ::GetWindowRect(this->pTransparentToolbar->GetControlHandle(),&Rect);
            _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::Reduce (After SetPosition): Real TransparentToolbar left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                Rect.left,Rect.top,Rect.right,Rect.bottom);
            ::OutputDebugString(Msg);

            _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::Reduce (After SetPosition): Real ContainerRect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                ContainerRect.left,ContainerRect.top,ContainerRect.right,ContainerRect.bottom);
            ::OutputDebugString(Msg);
        }
#endif
    }
    else
    {
        RECT ContainerClientRect;
        ::GetClientRect(this->hWndContainer,&ContainerClientRect);
        this->pTransparentToolbar->SetWidth(ContainerClientRect.right-ContainerClientRect.left);
    }


    // show move button
    this->pTransparentToolbar->ShowMove(TRUE);

    // change reduce buttons to maximize buttons
    this->pTransparentToolbar->ChangeIcons(
                                            TransparentToolbarButtonsId_MAXIMIZE,
                                            pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_Maximize),
                                            pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_MaximizeHot),
                                            pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_MaximizeDown),
                                            pToolbarOptions->GetCurrentSkinIcon(CBrowserToolbarOptions::IconsId_Maximize)
                                            );

    // switch minimized state
    this->pSingleFolderOptions->bIsMaximized=FALSE;

    this->bInsideMaximizeReduce=FALSE;

    if (PositionMode == CTransparentToolbar::BarPositionMode_POPUP)
    {
        // ensure toolbar position
        ::PostMessage(this->hWndContainer,CShellBrowserContainer_ENSURE_TOOLBAR_POSITION,0,0);
    }
}

void CShellBrowserContainerBase::GetFolderViewRect(OUT RECT* pFolderViewRect)
{
    ::GetClientRect(this->hWndContainer,pFolderViewRect);
    CTransparentToolbar::tagBarPositionMode BarPositionMode;
    CBrowserToolbarOptions* pToolbarOptions = &this->pQuickWayToFolders->pApplicationOptions->BrowserToolBarOptions;
    BarPositionMode = pToolbarOptions->GetCurrentSkinPositionMode();
    if (BarPositionMode == CTransparentToolbar::BarPositionMode_CHILD)
    {
        if (this->ShouldTransparentToolbarBeShown())
        {
            // adjust rect size to remove height of toolbar
            pFolderViewRect->top = CBrowserToolbar_ICON_SIZE;
        }
    }
}

BOOL CShellBrowserContainerBase::ApplySettings(CSingleFolderOptions* pSingleFolderOptions)
{
    if (this->pShellBrowserEx)
    {
        delete this->pShellBrowserEx;
        this->pShellBrowserEx=NULL;
    }

    this->pSingleFolderOptions=pSingleFolderOptions;
    if (!this->CreateContainer())
        return FALSE;

    FOLDERSETTINGS fs;
    fs.fFlags= FWF_NOWEBVIEW;
    CShellBrowserEx::ShellBrowserViewToFolderSetting(pSingleFolderOptions->FolderViewMode,&fs);

    RECT FolderViewRect;
    this->GetFolderViewRect(&FolderViewRect);
    this->pShellBrowserEx=new CShellBrowserEx(this->hWndContainer,&fs,pSingleFolderOptions->FolderViewMode,&FolderViewRect);
    // trash arrange messages for Desktop
    this->pShellBrowserEx->bTrashArrangeMessages = (pSingleFolderOptions->FolderPresentationType == CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_DESKTOP_BROWSER);
    if(!this->ApplySettingsToClassObject(this->pSingleFolderOptions))
        return FALSE;

    TCHAR szPath[MAX_PATH];
    if (!this->pSingleFolderOptions->GetPath(szPath,_countof(szPath)))
        return FALSE;
    
    if (FAILED(this->pShellBrowserEx->BrowseObject(szPath)))
        return FALSE;

    // must be done after browsing so can't be in ApplySettingsToClassObject
    if (this->pSingleFolderOptions->bShowAtStartup)
        this->Show(TRUE);

    return TRUE;
}

void CShellBrowserContainerBase::BrowserDefaultCommandCallBack(PVOID UserParam)
{
    CShellBrowserContainerBase* pShellBrowserContainer = (CShellBrowserContainerBase*)UserParam;
    if (pShellBrowserContainer->pSingleFolderOptions->bAutoHideOnFileExecution)
        ::SendMessage(pShellBrowserContainer->hWndContainer,WM_CLOSE,0,0);
}

void CShellBrowserContainerBase::UpdateNavigateBackForwardState()
{
    if (this->pShellBrowserEx->GetBackwardDepth()>0)
    {
        this->pTransparentToolbar->EnableButton(TransparentToolbarButtonsId_PREV,TRUE);
    }
    else
    {
        this->pTransparentToolbar->EnableButton(TransparentToolbarButtonsId_PREV,FALSE);
    }

    if (this->pShellBrowserEx->GetForwardDepth()>0)
    {
        this->pTransparentToolbar->EnableButton(TransparentToolbarButtonsId_NEXT,TRUE);
    }
    else
    {
        this->pTransparentToolbar->EnableButton(TransparentToolbarButtonsId_NEXT,FALSE);
    }
}

void CShellBrowserContainerBase::ShowListviewBorder(BOOL bShow)
{
    // hide listview border
    HWND hWnd = this->hWndListView;
    LONG_PTR Style = ::GetWindowLongPtr(hWnd,GWL_EXSTYLE);
    if (bShow)
        Style |= WS_EX_CLIENTEDGE;
    else
        Style &= (~WS_EX_CLIENTEDGE);
    ::SetWindowLongPtr(hWnd,GWL_EXSTYLE,Style);
}

void CShellBrowserContainerBase::BrowserViewChangedCallBack(HWND hWndListView,CShellBrowser::tagShellBrowserView NewView,LPVOID UserParam)
{
    UNREFERENCED_PARAMETER(hWndListView);
    CShellBrowserContainerBase* pShellBrowserContainer = (CShellBrowserContainerBase*)UserParam;

    if (pShellBrowserContainer->pSingleFolderOptions->bSaveIconsPositionsForHomeFolder)
    {
        if (
            pShellBrowserContainer->DoesIconsPositionsNeedToBeRestored()
            && (NewView == pShellBrowserContainer->pSingleFolderOptions->FolderViewMode)
            )
        {
            pShellBrowserContainer->InstallSubProc();
            pShellBrowserContainer->RestoreAllItemsPositions();
        }
        else
        {
            // has view has changed we should no more set icons position
            pShellBrowserContainer->RemoveSubProc();
        }
    }


    // store new view
    pShellBrowserContainer->StoreBrowserView(NewView); 
}

void CShellBrowserContainerBase::BrowserBeforeBrowseCallBack(LPITEMIDLIST CurrentPidl,LPITEMIDLIST ToBeBrowsePidl,PVOID UserParam)
{
    CShellBrowserContainerBase* pShellBrowserContainer = (CShellBrowserContainerBase*)UserParam;
    if (pShellBrowserContainer->DoesIconsPositionsNeedToBeSaved(CurrentPidl,ToBeBrowsePidl))
    {
        pShellBrowserContainer->SaveIconsPositions();
        pShellBrowserContainer->RemoveSubProc();
    }
}

void CShellBrowserContainerBase::BrowserAfterBrowseCallBack(LPITEMIDLIST FolderPidl,PVOID UserParam)
{
    CShellBrowserContainerBase* pShellBrowserContainer = (CShellBrowserContainerBase*)UserParam;
    pShellBrowserContainer->hWndShellView = pShellBrowserContainer->pShellBrowserEx->GetControlHandle();
    pShellBrowserContainer->hWndListView = pShellBrowserContainer->pShellBrowserEx->GetListViewHandle();
    pShellBrowserContainer->bItemsHasBeenSorted = FALSE;

    pShellBrowserContainer->UpdateNavigateBackForwardState();

    if (pShellBrowserContainer->pSingleFolderOptions->bIsMaximized)
    {
        pShellBrowserContainer->Maximize(); // used to force ShowListviewBorder(FALSE); and redraw
    }

    BOOL bWasHomeFolder = pShellBrowserContainer->bIsHomeFolder;
    ::SHGetPathFromIDList(FolderPidl,pShellBrowserContainer->CurrentPath);
    pShellBrowserContainer->bIsHomeFolder = pShellBrowserContainer->IsHomeFolder();
    if (pShellBrowserContainer->pSingleFolderOptions->bHideToolBarForHomeFolder && (!pShellBrowserContainer->pSingleFolderOptions->bHideToolBar) )
    {
        // update visible state only if required
        if (bWasHomeFolder != pShellBrowserContainer->bIsHomeFolder)
        {
            // hide before potential sizing
            if (pShellBrowserContainer->bIsHomeFolder)
            {
                pShellBrowserContainer->pTransparentToolbar->Hide();
                if (pShellBrowserContainer->pTransparentToolbar->GetBarPositionMode()==CTransparentToolbar::BarPositionMode_CHILD)
                {
                    RECT ContainerInternalRect;
                    ::GetClientRect(pShellBrowserContainer->hWndContainer,&ContainerInternalRect);
                    ::SetWindowPos(pShellBrowserContainer->pShellBrowserEx->GetControlHandle(),0,
                                   ContainerInternalRect.left,ContainerInternalRect.top,
                                   ContainerInternalRect.right-ContainerInternalRect.left,
                                   ContainerInternalRect.bottom-ContainerInternalRect.top,
                                   SWP_NOZORDER | SWP_NOACTIVATE);
                }
            }

            if (pShellBrowserContainer->pSingleFolderOptions->bIsMaximized)
            {
                pShellBrowserContainer->UpdateMaximizedRects();

                pShellBrowserContainer->bInsideMaximizeReduce=TRUE;

                ::SetWindowPos(pShellBrowserContainer->hWndContainer,0,
                                pShellBrowserContainer->MaximizedRect.left,
                                pShellBrowserContainer->MaximizedRect.top,
                                pShellBrowserContainer->MaximizedRect.right-pShellBrowserContainer->MaximizedRect.left,
                                pShellBrowserContainer->MaximizedRect.bottom-pShellBrowserContainer->MaximizedRect.top, 
                                SWP_NOACTIVATE | SWP_NOZORDER );

#ifdef TRACE_POSITION
                {
                    TCHAR Msg[512];
                    _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::BrowserAfterBrowseCallBack SetPosition ContainerRect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                        pShellBrowserContainer->MaximizedRect.left,pShellBrowserContainer->MaximizedRect.top,pShellBrowserContainer->MaximizedRect.right,pShellBrowserContainer->MaximizedRect.bottom);
                    ::OutputDebugString(Msg);

                    RECT Rect;
                    ::GetWindowRect(pShellBrowserContainer->hWndContainer,&Rect);
                    _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::BrowserAfterBrowseCallBack (After SetPosition): Real ContainerRect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                        Rect.left,Rect.top,Rect.right,Rect.bottom);
                    ::OutputDebugString(Msg);
                }
#endif


                pShellBrowserContainer->bInsideMaximizeReduce=FALSE;

            }

            // show after potential sizing
            if (!pShellBrowserContainer->bIsHomeFolder)
            {
                if (pShellBrowserContainer->pTransparentToolbar->GetBarPositionMode()==CTransparentToolbar::BarPositionMode_CHILD)
                {
                    RECT ContainerInternalRect;
                    ::GetClientRect(pShellBrowserContainer->hWndContainer,&ContainerInternalRect);
                    ::SetWindowPos(pShellBrowserContainer->pShellBrowserEx->GetControlHandle(),0,
                                    ContainerInternalRect.left,pShellBrowserContainer->pTransparentToolbar->GetHeight(),
                                    ContainerInternalRect.right-ContainerInternalRect.left,
                                    ContainerInternalRect.bottom-ContainerInternalRect.top-pShellBrowserContainer->pTransparentToolbar->GetHeight(),
                                    SWP_NOZORDER | SWP_NOACTIVATE);
                }

                pShellBrowserContainer->pTransparentToolbar->Show();
            }
        }
    }

    // after bIsHomeFolder is set and listview is resized
    if (pShellBrowserContainer->DoesIconsPositionsNeedToBeRestored())
    {
        // assume view is the same as queried one
        pShellBrowserContainer->InstallSubProc();
        // if folder presentation type is not the same as the folder option one
        if (pShellBrowserContainer->ShellBrowserView != pShellBrowserContainer->pSingleFolderOptions->FolderViewMode)
        {
            // restore view
            pShellBrowserContainer->bRestoringView = TRUE;
            pShellBrowserContainer->pShellBrowserEx->SetView(pShellBrowserContainer->pSingleFolderOptions->FolderViewMode);
            pShellBrowserContainer->bRestoringView = FALSE;
        }

        pShellBrowserContainer->RestoreAllItemsPositions();
    }
    else
    {
        if (bWasHomeFolder)
        {
            // restore previous view
            if (pShellBrowserContainer->ShellBrowserView != pShellBrowserContainer->pSingleFolderOptions->FolderViewMode)
            {
                pShellBrowserContainer->pShellBrowserEx->SetView(pShellBrowserContainer->ShellBrowserView);
            }
        }
#pragma message("trouble with icon / small icon view (header space on top)")
//        LONG_PTR Style=GetWindowLongPtr(pShellBrowserContainer->hWndListView,GWL_STYLE);
//        if ( (Style & LVS_REPORT) == 0 )
//        {
//            Style |= LVS_NOCOLUMNHEADER;
//            SetWindowLongPtr(pShellBrowserContainer->hWndListView,GWL_STYLE,Style);
//        }
//#if _WIN32_WINNT < 0x0600
//#define LVS_EX_AUTOAUTOARRANGE  0x01000000  // Icons automatically arrange if no icon positions have been set
//#define LVS_EX_HEADERINALLVIEWS 0x02000000  // Display column header in all view modes
//#endif
//        DWORD ExStyle = ListView_GetExtendedListViewStyle(pShellBrowserContainer->hWndListView);
//        ExStyle &= ~(LVS_EX_FULLROWSELECT | LVS_EX_AUTOAUTOARRANGE | LVS_EX_HEADERINALLVIEWS); // disabling LVS_EX_FULLROWSELECT is required even in small icon view due to specific painting
//        ListView_SetExtendedListViewStyle(pShellBrowserContainer->hWndListView, ExStyle);
    }
}

void CShellBrowserContainerBase::StopRestoreAllItemsPositions()
{
#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
    ::KillTimer(this->hWndShellView,CShellBrowserContainer_RESTORE_HOME_ICON_POSITION_NEEDED);
#endif
}


void CShellBrowserContainerBase::RestoreAllItemsPositions()
{

#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
    this->RestoreAllItemsNbRetry = CShellBrowserContainer_DEFAULT_NB_SET_POS_RETRY;
    this->RestoreAllItemsLastNbItems = 0;
    ::SetTimer(this->hWndShellView,CShellBrowserContainer_RESTORE_HOME_ICON_POSITION_NEEDED,CShellBrowserContainer_RESTORE_ITEMS_POSITION_TIMER_IN_MS,NULL);
#else
    ::PostMessage(this->hWndShellView, CShellBrowserContainer_RESTORE_HOME_ICON_POSITION_NEEDED, CShellBrowserContainer_DEFAULT_NB_SET_POS_RETRY,0);
#endif
}

BOOL CShellBrowserContainerBase::RestoreAllItemsPositionsInternal(SIZE_T NbRetry,int LastTryNbIcons)
{
    if (this->pSingleFolderOptions == NULL)
        return FALSE;

    this->bRestoringItemsPositions = TRUE;

    // if nothing to restore
    if (this->pSingleFolderOptions->pIconsPositionsForHomeFolder->size() == 0)
    {
        this->bRestoringItemsPositions = FALSE;
#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
        this->StopRestoreAllItemsPositions();
#endif
        return FALSE;
    }

    int NbListViewItems = ListView_GetItemCount(this->hWndListView);
    if ( NbListViewItems <= 0) // listview not ready yet
    {
        // delay positioning
        if (NbRetry>0)
        {
#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
            this->RestoreAllItemsNbRetry--;
            this->RestoreAllItemsLastNbItems = 0;
#else
            ::PostMessage(this->hWndShellView, CShellBrowserContainer_RESTORE_HOME_ICON_POSITION_NEEDED, NbRetry-1,0);
#endif
        }
        return FALSE;
    }
    else
    {
        // try to ensure control has finished to add items (aka LastTryNbIcons == NbListViewItems)
        if ( (NbRetry>0) && (LastTryNbIcons<NbListViewItems) )
        {
#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
            this->RestoreAllItemsNbRetry--;
            this->RestoreAllItemsLastNbItems = NbListViewItems;
#else
            ::PostMessage(this->hWndShellView, CShellBrowserContainer_RESTORE_HOME_ICON_POSITION_NEEDED, NbRetry-1,NbListViewItems);
#endif
            
            return FALSE;
        }
    }

    //////////////////////////////////////////////////////////////////////
    // From this point, we consider that content of listview is stable
    //////////////////////////////////////////////////////////////////////

#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
    this->StopRestoreAllItemsPositions();
#endif

    LVITEM Item;
    TCHAR ItemText[CSingleFolderOptions_MAX_NAME_SIZE];
    *ItemText=0;
    memset(&Item,0,sizeof(LVITEM));
    Item.mask = LVIF_TEXT;
    Item.cchTextMax = _countof(ItemText);
    Item.iSubItem = 0;
    Item.pszText = ItemText;

    for (int ItemIndex = 0; ItemIndex < NbListViewItems ; ItemIndex++)
    {
        *ItemText=0;
        Item.iItem = (int)ItemIndex;
        ::SendMessage(hWndListView,LVM_GETITEMTEXT,ItemIndex,(LPARAM)&Item);

        if (*ItemText == 0)
            continue;

        // for each Item in vector
        for (size_t CntVector=0;CntVector<this->pSingleFolderOptions->pIconsPositionsForHomeFolder->size();CntVector++)
        {

            // if item has the same name (= item found)
            if (_tcsicmp(this->pSingleFolderOptions->pIconsPositionsForHomeFolder->at(CntVector).Name,ItemText)==0)
            {
                POINT CurrentPoint;
                POINT pt = this->pSingleFolderOptions->pIconsPositionsForHomeFolder->at(CntVector).Position;
                // get current position
                ListView_GetItemPosition(this->hWndListView,ItemIndex,&CurrentPoint);
                // check position (avoid to set item if already at good place)
                if ( memcmp(&CurrentPoint , &pt, sizeof(POINT)) !=0 )
                {
                    // set item position MUST use PostMessage (item not redrawn else)
                    // ListView_SetItemPosition(this->hWndListView,ItemIndex,pt.x,pt.y);
                    //::PostMessage(this->hWndListView,LVM_SETITEMPOSITION,ItemIndex,(LPARAM) MAKELPARAM ( (int) pt.x, (int) pt.y) );
                    ::SendMessage(this->hWndListView,LVM_SETITEMPOSITION32,ItemIndex,(LPARAM)&pt);
                }
                break;
            }
        }
    }

    this->bRestoringItemsPositions = FALSE;
    this->bItemsHasBeenSorted = FALSE;
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: GetDesktopPidl
// Object: get pidl associated to user desktop folder
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
BOOL CShellBrowserContainerBase::GetDesktopPidl()
{
    BOOL bRet=FALSE;
    if (this->pidlDesktop)
        return TRUE;

    HRESULT hResult = E_FAIL;

    if (!this->pMalloc)
    {
        // get pointer to malloc interface
        hResult = ::SHGetMalloc(&this->pMalloc);
        if (FAILED(hResult)||(!this->pMalloc))
            goto CleanUp;
    }

    // get folder full idl list
    hResult = ::SHGetFolderLocation(this->hWndContainer,CSIDL_DESKTOP,NULL,NULL,&this->pidlDesktop);
    if ( FAILED(hResult) || (!this->pidlDesktop) )
        goto CleanUp;

    bRet=TRUE;

CleanUp:
    return bRet;
}

BOOL CShellBrowserContainerBase::ApplySettingsToClassObject(CSingleFolderOptions* pSingleFolderOptions)
{
    // update current path from options
    pSingleFolderOptions->GetPath(this->CurrentPath,MAX_PATH);
    this->CContainerBase::ApplySettingsToClassObject(pSingleFolderOptions);

    this->pShellBrowserEx->SetDefaultCommandCallBack(CShellBrowserContainerBase::BrowserDefaultCommandCallBack,this);
    this->pShellBrowserEx->SetBeforeBrowseCallBack(CShellBrowserContainerBase::BrowserBeforeBrowseCallBack,this);
    this->pShellBrowserEx->SetAfterBrowseCallBack(CShellBrowserContainerBase::BrowserAfterBrowseCallBack,this);
    this->ShellBrowserView=this->pSingleFolderOptions->FolderViewMode;
    this->pShellBrowserEx->SetViewChangedCallBack(CShellBrowserContainerBase::BrowserViewChangedCallBack,this);

    this->bRememberLastPosition=pSingleFolderOptions->bRememberLastPosition;

    this->pShellBrowserEx->SetFiltersExclusion(pSingleFolderOptions->ExclusionFilters);
    this->pShellBrowserEx->SetFiltersInclusion(pSingleFolderOptions->InclusionFilters);
    this->pShellBrowserEx->bBrowseInsideView=pSingleFolderOptions->bAllowSubFoldersNavigation;
    this->pShellBrowserEx->bShowSubFolders=pSingleFolderOptions->bIncludeSubFolders;
    this->pShellBrowserEx->bOneClickActivate=pSingleFolderOptions->bOneClickActivate;

    if (pSingleFolderOptions->bTextColorSepcified)
        this->pShellBrowserEx->SetTextColor(pSingleFolderOptions->TextColor);
    if (pSingleFolderOptions->bTextBackgroundColorSepcified)
        this->pShellBrowserEx->SetTextBackgroundColor(pSingleFolderOptions->TextBackgroundColor);

    this->pShellBrowserEx->SetBackgroundType(pSingleFolderOptions->BackgroundType);
    switch(pSingleFolderOptions->BackgroundType)
    {
    case CShellBrowserEx::BACKGROUND_TYPE_COLOR:
        this->pShellBrowserEx->SetBackgroundColor(pSingleFolderOptions->BackgroundColor);
        break;
    case CShellBrowserEx::BACKGROUND_TYPE_GRADIENT:
        this->pShellBrowserEx->SetBackgroundGradient(pSingleFolderOptions->BackgroundColor1,
                                                    pSingleFolderOptions->BackgroundColor2,
                                                    pSingleFolderOptions->BackgroundGradiantType
                                                    );
        break;
    case CShellBrowserEx::BACKGROUND_TYPE_PICTURE:
        {
            TCHAR FullPath[MAX_PATH];
            // if BackgroundImagePath contains full path
            if (CStdFileOperations::IsFullPath(pSingleFolderOptions->BackgroundImagePath))
            {
                _tcscpy(FullPath,pSingleFolderOptions->BackgroundImagePath);
            }
            // else image is supposed to be in the application path
            else
            {
                // get application path
                CStdFileOperations::GetAppDirectory(FullPath,MAX_PATH);
                // add image name
                _tcscat(FullPath,pSingleFolderOptions->BackgroundImagePath);
            }

            BOOL bRet;
            bRet = this->pShellBrowserEx->SetBackgroundPicture(FullPath,
                                                               pSingleFolderOptions->BackgroundImagePosition
                                                               );
#ifdef _DEBUG
            if (!bRet)
                this->ReportMessage(REPORT_MESSAGE_TYPE_ERROR,_T("Error setting background picture"));
#endif // _DEBUG
        }
        break;
    }

    if (this->pSingleFolderOptions->bIsMaximized)
    {
        // invert flag as it will be restored by MaximizeReduce()
        this->pSingleFolderOptions->bIsMaximized=!this->pSingleFolderOptions->bIsMaximized;

        // do maximizing
        this->MaximizeReduce();
    }

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: GetAssociatedObject
// Object: Get object associated to window handle
// Parameters :
//     in  : HWND hWndDialog : handle of the window
//     out :
//     return : associated object if found, NULL if not found
//-----------------------------------------------------------------------------
CShellBrowserContainerBase* CShellBrowserContainerBase::GetAssociatedObject(HWND hWndDialog)
{
    return (CShellBrowserContainerBase*)GetWindowLongPtr(hWndDialog,GWLP_USERDATA);
}

LRESULT CALLBACK CShellBrowserContainerBase::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    if (uMsg == WM_CREATE)
    {
        CREATESTRUCT* pCreateStruct;
        CShellBrowserContainerBase* pDialogObject; 
        pCreateStruct=(CREATESTRUCT*)lParam;
        pDialogObject=(CShellBrowserContainerBase*)pCreateStruct->lpCreateParams;
        SetWindowLongPtr(hWnd,GWLP_USERDATA,(LONG_PTR)pDialogObject);
        pDialogObject->hWndContainer = hWnd;
        return pDialogObject->OnMessage(hWnd, uMsg, wParam, lParam);
    }
    else
    {
        CShellBrowserContainerBase* pDialogObject; 
        pDialogObject=CShellBrowserContainerBase::GetAssociatedObject(hWnd);
        if (!pDialogObject)
            return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
        return pDialogObject->OnMessage(hWnd, uMsg, wParam, lParam);
    }
}

// An application should return zero if it processes this message
LRESULT CShellBrowserContainerBase::OnKeyDown_ShellBrowserContainerBase(WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);
    switch(wParam)
    {
    case VK_BACK:
        if (!this->pShellBrowserEx->IsEditing())
        {
            this->pShellBrowserEx->BrowseParent();
            return 0;
        }
        return 1;
    case VK_BROWSER_BACK:
    case VK_XBUTTON1: // mouse backward
    case VK_MEDIA_PREV_TRACK:
        this->pShellBrowserEx->BrowseBack();
        return 0;
    case VK_BROWSER_FORWARD:
    case VK_XBUTTON2: // mouse forward
    case VK_MEDIA_NEXT_TRACK:
        this->pShellBrowserEx->BrowseForward();
        return 0;
    case VK_F5:
    case VK_BROWSER_REFRESH:
        this->Refresh();
        return 0;
    default:
        return 1;
    }
}

LRESULT CShellBrowserContainerBase::OnMessage(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    return this->OnMessage_ShellBrowserContainerBase(hWnd, uMsg, wParam, lParam);
}

#ifndef WM_GETISHELLBROWSER
    #define WM_GETISHELLBROWSER (WM_USER+7)
#endif
LRESULT CShellBrowserContainerBase::OnMessage_ShellBrowserContainerBase(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
    case WM_CREATE:
        this->DisabledWM_MOVE_Counted++;
        this->OnCreate();
        this->DisabledWM_MOVE_Counted--;
        break;
    case WM_SIZE:
        this->DisabledWM_MOVE_Counted++;
        this->OnSize();
        this->DisabledWM_MOVE_Counted--;
        break;
    case WM_SIZING:
        this->DisabledWM_MOVE_Counted++;
        this->OnSizing((RECT*)lParam);
        this->DisabledWM_MOVE_Counted--;
        break;
    case CContainerBase_KEYDOWN_MSG:
        {
            PMSG pMsg=(PMSG)lParam;
            if (this->OnKeyDown_ShellBrowserContainerBase(pMsg->wParam,pMsg->lParam) == 0)
                return TRUE;
            else
                return this->pShellBrowserEx->TranslateAccelerator(pMsg);

            break;
        }
    case WM_XBUTTONDOWN: // handle mouse media buttons
        {
            WORD fwKeys = GET_KEYSTATE_WPARAM (wParam); 
            // WORD fwButton = GET_XBUTTON_WPARAM (wParam);
            // int xPos = GET_X_LPARAM(lParam); 
            // int yPos = GET_Y_LPARAM(lParam);
            switch (fwKeys)
            {
                case MK_XBUTTON1: // backward
                    if (FAILED(this->pShellBrowserEx->BrowseBack()))
                        this->UpdateNavigateBackForwardState();// update back/forward buttons state. In case of failure (in case of success UpdateNavigateBackForwardState is called by BrowserAfterBrowseCallBack)
                    return TRUE;

                case MK_XBUTTON2: // forward
                    if (FAILED(this->pShellBrowserEx->BrowseForward()))
                        this->UpdateNavigateBackForwardState();// update back/forward buttons state. In case of failure (in case of success UpdateNavigateBackForwardState is called by BrowserAfterBrowseCallBack)
                    return TRUE;
            }
            break;
        }
    case CShellBrowserContainer_TOOLBAR_BUTTON_CLICK:
        this->OnTransparentToolbarButtonClick((DWORD)lParam);
        break;
    case CShellBrowserContainer_ENSURE_TOOLBAR_POSITION:
        this->EnsureToolBarPosition();
        break;
    case WM_WINDOWPOSCHANGED:
        // WM_WINDOWPOSCHANGED internally PostMessage WM_SIZE and WM_MOVE messages
        // so it must be trashed when WM_MOVE is disabled 
        // (else as messages are posted, our DisabledWM_MOVE_Counted protection is bypassed;
        // this hadn't occurred if WM_WINDOWPOSCHANGED used SendMessage)
        if (this->DisabledWM_MOVE_Counted<=0)
            return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
        break;
    case WM_MOVE:
        if (this->DisabledWM_MOVE_Counted<=0)
            this->OnMove(wParam,lParam);
        break;
    case WM_QUIT:
    case WM_CLOSE:
        this->OnClose();
        break;
    // DO NOT CALL ::PostQuitMessage(0); because we use a single message loop for application and shell browsers
    // case WM_DESTROY:
    //    ::PostQuitMessage(0);
    //    break;

    //case WM_PAINT:
    //    {
    //        CShellBrowserContainerBase* pDialogObject; 
    //        pDialogObject=CShellBrowserContainerBase::GetAssociatedObject(hWnd);
    //        if (!pDialogObject)
    //            break;

    //        ::SendMessage(pDialogObject->pShellBrowserEx->GetControlHandle(),uMsg,wParam,lParam);
    //    }
    //    break;
    case WM_DISPLAYCHANGE:
        this->OnResolutionChanged();
        break;
    case WM_GETISHELLBROWSER:
        // Q157247
        // https://web.archive.org/web/20051225035102/http://support.microsoft.com:80/kb/q157247/
        // we must return IShellBrowser* there
#ifdef _DEBUG
        ::OutputDebugString(TEXT("CShellBrowserContainerBase::OnMessage_ShellBrowserContainerBase WM_GETISHELLBROWSER\r\n"));
#endif
        return (LRESULT)(IShellBrowser*)(CShellBrowser*)this->pShellBrowserEx;
    default:
        return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
    }
    return 0;
}

void CShellBrowserContainerBase::OnCreate()
{

}

void CShellBrowserContainerBase::OnHotWindowAutoShow(IN CHotWindow* pSender,OUT BOOL* pUseAutoHideDisabledExtraRect,OUT RECT* pAutoHideDisabledExtraRect)
{
    UNREFERENCED_PARAMETER(pSender);
    this->Show(TRUE);
    *pUseAutoHideDisabledExtraRect = FALSE;
    UNREFERENCED_PARAMETER(pAutoHideDisabledExtraRect);
}
void CShellBrowserContainerBase::OnHotWindowAutoHide(CHotWindow* pSender)
{
    UNREFERENCED_PARAMETER(pSender);
    // nothing to do
}


void CShellBrowserContainerBase::OnClose()
{
    this->Hide();
}

HRESULT CShellBrowserContainerBase::GoHome()
{
    if (!this->pShellBrowserEx)
        return S_FALSE;

    TCHAR szPath[MAX_PATH];
    pSingleFolderOptions->GetPath(szPath,_countof(szPath));
    return this->pShellBrowserEx->BrowseObject(szPath);
}

// bShowOnly : TRUE to show only, FALSE to Show or Hide according to internal state
BOOL CShellBrowserContainerBase::Show(BOOL bShowOnly)
{
    if (!this->pShellBrowserEx)
        return FALSE;
    if (!this->pTransparentToolbar)
        return FALSE;

    if (!bShowOnly)
    {
        if (::IsWindowVisible(this->pShellBrowserEx->GetControlHandle()))
        {
            this->OnClose();
            return TRUE;
        }
    }

    if (this->bOperationAfterResolutionChangedMustBeDoneBeforeShow)
        this->DoOperationAfterResolutionChanged();

    if ( this->ShouldTransparentToolbarBeShown() )
    {
        // show toolbar
        this->pTransparentToolbar->Show();
    }

    if ( (!this->bRememberLastPosition)
         
         //and window has been closed (else window can have loose focus and go to background; and in this case we just want to show again)
         && (!::IsWindowVisible(this->hWndContainer))
       )
    {
        this->GoHome();
    }


    ::ShowWindow(this->hWndContainer,TRUE);
    ::BringWindowToTop(this->hWndContainer);
    ::SetForegroundWindow(this->hWndContainer);
    ::SetFocus(this->hWndListView);
    return TRUE;
}

BOOL CShellBrowserContainerBase::Hide()
{
    if (this->pTransparentToolbar->GetBarPositionMode()==CTransparentToolbar::BarPositionMode_POPUP)
    {
        // hide toolbar
        this->pTransparentToolbar->Hide();
    }

    // hide container
    ::ShowWindow(this->hWndContainer,FALSE);

    return TRUE;
}

void CShellBrowserContainerBase::OnSize()
{
    if (!this->pShellBrowserEx)
        return;
    if (!this->pTransparentToolbar)
        return;

    RECT Rect;
    CTransparentToolbar::tagBarPositionMode BarPositionMode;
    CBrowserToolbarOptions* pToolbarOptions = &this->pQuickWayToFolders->pApplicationOptions->BrowserToolBarOptions;
    BarPositionMode = pToolbarOptions->GetCurrentSkinPositionMode();

    // update toolbar pos
    if (!this->bInsideMaximizeReduce)
    {
        if (BarPositionMode == CTransparentToolbar::BarPositionMode_POPUP)
        {

            // get rect without shadow on win 10 (DO NOT use ::GetWindowRect(this->hWndContainer,&ContainerRect); which will contain shadow)
            this->DialogWindowRect.GetWindowRectWithoutShadow(this->hWndContainer,&Rect);
            this->pTransparentToolbar->SetPosition(Rect.left,Rect.top-this->pTransparentToolbar->GetHeight(),Rect.right-Rect.left,this->pTransparentToolbar->GetHeight());

#ifdef TRACE_POSITION
            {
                TCHAR Msg[512];
                _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::OnSize: ContainerRect Rect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                            Rect.left,Rect.top,Rect.right,Rect.bottom);
                ::OutputDebugString(Msg);

                _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::OnSize SetPosition TransparentToolbar left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                    Rect.left,
                    Rect.top-this->pTransparentToolbar->GetHeight(),
                    Rect.right,
                    Rect.top+this->pTransparentToolbar->GetHeight()
                    );
                ::OutputDebugString(Msg);

                ::GetWindowRect(this->pTransparentToolbar->GetControlHandle(),&Rect);
                _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::OnSize (After SetPosition): Real TransparentToolbar left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                    Rect.left,Rect.top,Rect.right,Rect.bottom);
                ::OutputDebugString(Msg);
            }
#endif
        }
        else
        {
            RECT ContainerClientRect;
            ::GetClientRect(this->hWndContainer,&ContainerClientRect);
            this->pTransparentToolbar->SetWidth(ContainerClientRect.right-ContainerClientRect.left);
        }
    }

    // get container internal rect
    ::GetClientRect(this->hWndContainer,&Rect);

    // store current browser rect if not maximized one
    if (!this->bInsideMaximizeReduce)
    {
        if (!::EqualRect(&this->MaximizedRect,&Rect))
        {
            RECT RectWindow;
            // get rect with shadow 
            ::GetWindowRect(this->hWndContainer,&RectWindow);
            this->pSingleFolderOptions->WindowPosition.x=RectWindow.left;
            this->pSingleFolderOptions->WindowPosition.y=RectWindow.top;

            // ::GetClientRect(this->hWndContainer,&this->pSingleFolderOptions->BrowserRect);
            memcpy(&this->pSingleFolderOptions->BrowserRect,&RectWindow,sizeof(RECT));
            ::OffsetRect(&this->pSingleFolderOptions->BrowserRect,
                        -this->pSingleFolderOptions->BrowserRect.left,
                        -this->pSingleFolderOptions->BrowserRect.top);
        }

    }

    if (BarPositionMode == CTransparentToolbar::BarPositionMode_CHILD)
    {
        Rect.top = this->pTransparentToolbar->GetHeight();
    }

    // update browser window
#pragma message(__FILE__ " To be solved : too slow.... may delayed post message to avoid to recompute layout on sizing")
    this->pShellBrowserEx->UpdateRect(&Rect);

//#ifdef _DEBUG
//    {
//        TCHAR MsgTrace[1024];
//        _sntprintf(MsgTrace,_countof(MsgTrace),
//            TEXT("CShellBrowserContainerBase::OnSize UpdateRect (left=%d, top=%d, right=%d, bottom=%d) \r\n"),
//            Rect.left,Rect.top,Rect.right,Rect.bottom
//            );
//        ::OutputDebugString(MsgTrace);
//    }
//#endif

    if (BarPositionMode == CTransparentToolbar::BarPositionMode_POPUP)
    {
        ::PostMessage(this->hWndContainer,CShellBrowserContainer_ENSURE_TOOLBAR_POSITION,0,0);
    }
}

void CShellBrowserContainerBase::EnsureToolBarPosition()
{
    RECT Rect;
    // get rect without shadow on win 10 (DO NOT use ::GetWindowRect(this->hWndContainer,&ContainerRect); which will contain shadow)
    this->DialogWindowRect.GetWindowRectWithoutShadow(this->hWndContainer,&Rect);
    this->pTransparentToolbar->SetPosition(Rect.left,Rect.top-this->pTransparentToolbar->GetHeight(),Rect.right-Rect.left,this->pTransparentToolbar->GetHeight());

#ifdef TRACE_POSITION
    {
        TCHAR Msg[512];
        _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::EnsureToolBarPosition: ContainerRect Rect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
            Rect.left,Rect.top,Rect.right,Rect.bottom);
        ::OutputDebugString(Msg);

        _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::EnsureToolBarPosition: Toolbar SetPosition Rect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
            Rect.left,Rect.top-this->pTransparentToolbar->GetHeight(),Rect.right,Rect.top);
        ::OutputDebugString(Msg);

        ::GetWindowRect(this->pTransparentToolbar->GetControlHandle(),&Rect);
        _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::EnsureToolBarPosition: Toolbar GetWindowRect after SetPosition Rect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
            Rect.left,Rect.top,Rect.right,Rect.bottom);
        ::OutputDebugString(Msg);
    }
#endif
}

void CShellBrowserContainerBase::OnSizing(RECT* pRect)
{
    if (!this->pShellBrowserEx)
        return;

    if (!this->pTransparentToolbar)
        return;

    if (this->bInsideMaximizeReduce)
        return;

    // size limit
    if (pRect->right-pRect->left<this->pTransparentToolbar->GetMinWidth())
        pRect->right=pRect->left+this->pTransparentToolbar->GetMinWidth();

    CTransparentToolbar::tagBarPositionMode BarPositionMode;
    CBrowserToolbarOptions* pToolbarOptions = &this->pQuickWayToFolders->pApplicationOptions->BrowserToolBarOptions;
    BarPositionMode = pToolbarOptions->GetCurrentSkinPositionMode();
    if (BarPositionMode == CTransparentToolbar::BarPositionMode_POPUP)
    {
        // update toolbar pos (better rendering than only on onsize event)
        RECT NoShadowRect;
        // get rect without shadow on win 10 (DO NOT use ::GetWindowRect(this->hWndContainer,&ContainerRect); which will contain shadow)
        this->DialogWindowRect.GetWindowRectWithoutShadow(this->hWndContainer,&NoShadowRect); // get rect without shadow on win 10

        RECT TransparentToolBarRect = NoShadowRect;
        TransparentToolBarRect.top -= this->pTransparentToolbar->GetHeight();
        TransparentToolBarRect.bottom = TransparentToolBarRect.top+this->pTransparentToolbar->GetHeight();
        this->pTransparentToolbar->SetPosition(TransparentToolBarRect.left,TransparentToolBarRect.top,
                                                TransparentToolBarRect.right-TransparentToolBarRect.left,this->pTransparentToolbar->GetHeight());
#ifdef TRACE_POSITION
        {
            TCHAR Msg[512];

            _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::OnSizing : ContainerBase pRect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
               pRect->left,pRect->top,pRect->right,pRect->bottom);
            ::OutputDebugString(Msg);

            _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::OnSizing : Container NoShadowRect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                NoShadowRect.left,NoShadowRect.top,NoShadowRect.right,NoShadowRect.bottom);
            ::OutputDebugString(Msg);


            _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::OnSizing SetPosition TransparentToolbar left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                TransparentToolBarRect.left,
                TransparentToolBarRect.top,
                TransparentToolBarRect.right,
                TransparentToolBarRect.bottom
                );
            ::OutputDebugString(Msg);

            RECT TmpRect;
            ::GetWindowRect(this->pTransparentToolbar->GetControlHandle(),&TmpRect);
            _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::OnSizing (After SetPosition): Real TransparentToolbar left=%d ,top=%d, right=%d, bottom=%d\r\n"),
                TmpRect.left,TmpRect.top,TmpRect.right,TmpRect.bottom);
            ::OutputDebugString(Msg);
        }
#endif
    }
    else
    {
        RECT ContainerClientRect;
        ::GetClientRect(this->hWndContainer,&ContainerClientRect);
        this->pTransparentToolbar->SetWidth(ContainerClientRect.right-ContainerClientRect.left);
    }
}

void CShellBrowserContainerBase::OnResolutionChanged()
{
    if (::IsWindowVisible(this->hWndContainer))
        this->DoOperationAfterResolutionChanged();
    else
        this->bOperationAfterResolutionChangedMustBeDoneBeforeShow = TRUE;
}

void CShellBrowserContainerBase::DoOperationAfterResolutionChanged()
{
    // update the desktop and maximized rects
    this->UpdateMaximizedRects();

    // assume window is fully visible (assume to be visible even after a resolution change)
    RECT WindowRectWithShadow;
    RECT WindowWithoutShadowRect;
    RECT ShadowBorderRect;
    // get adjusted rect (dialog without shadow border)
    this->DialogWindowRect.GetWindowRectWithoutShadow(this->hWndContainer,&WindowWithoutShadowRect);
    this->DialogWindowRect.GetWindowRectWithShadow(this->hWndContainer,&WindowRectWithShadow);
    ShadowBorderRect.left = WindowWithoutShadowRect.left - WindowRectWithShadow.left;
    ShadowBorderRect.top  = WindowWithoutShadowRect.top - WindowRectWithShadow.top;
    ShadowBorderRect.bottom = WindowRectWithShadow.bottom - WindowWithoutShadowRect.bottom;
    ShadowBorderRect.right  = WindowRectWithShadow.right - WindowWithoutShadowRect.right;
    this->AssumeWindowVisibility(&WindowRectWithShadow,&ShadowBorderRect);

    this->DisabledWM_MOVE_Counted++;

    // use adjusted window rect
    // ::MoveWindow(this->hWndContainer,WindowRectWithShadow.left,WindowRectWithShadow.top,WindowRectWithShadow.right-WindowRectWithShadow.left,WindowRectWithShadow.bottom-WindowRectWithShadow.top,TRUE);
    ::SetWindowPos(this->hWndContainer,NULL,WindowRectWithShadow.left,WindowRectWithShadow.top,WindowRectWithShadow.right-WindowRectWithShadow.left,WindowRectWithShadow.bottom-WindowRectWithShadow.top,SWP_NOZORDER);

#ifdef TRACE_POSITION
    {
        TCHAR Msg[512];
        _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::DoOperationAfterResolutionChanged SetPosition ContainerRect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
            WindowRectWithShadow.left,WindowRectWithShadow.top,WindowRectWithShadow.right,WindowRectWithShadow.bottom);
        ::OutputDebugString(Msg);

        RECT Rect;
        ::GetWindowRect(this->hWndContainer,&Rect);
        _sntprintf(Msg,_countof(Msg),TEXT("CShellBrowserContainerBase::DoOperationAfterResolutionChanged (After SetPosition): Real ContainerRect left=%d ,top=%d, right=%d, bottom=%d\r\n"),
            Rect.left,Rect.top,Rect.right,Rect.bottom);
        ::OutputDebugString(Msg);
    }
#endif


    CTransparentToolbar::tagBarPositionMode BarPositionMode;
    CBrowserToolbarOptions* pToolbarOptions = &this->pQuickWayToFolders->pApplicationOptions->BrowserToolBarOptions;
    BarPositionMode = pToolbarOptions->GetCurrentSkinPositionMode();
    if (BarPositionMode == CTransparentToolbar::BarPositionMode_POPUP)
    {
        // get rect without shadow on win 10 (DO NOT use ::GetWindowRect(this->hWndContainer,&ContainerRect); which will contain shadow)
        this->DialogWindowRect.GetWindowRectWithoutShadow(this->hWndContainer,&WindowRectWithShadow);

        this->pTransparentToolbar->SetPosition(WindowRectWithShadow.left,WindowRectWithShadow.top-this->pTransparentToolbar->GetHeight(),WindowRectWithShadow.right-WindowRectWithShadow.left,this->pTransparentToolbar->GetHeight());
    }
    else
    {
        RECT ContainerClientRect;
        ::GetClientRect(this->hWndContainer,&ContainerClientRect);
        this->pTransparentToolbar->SetWidth(ContainerClientRect.right-ContainerClientRect.left);
    }

    this->DisabledWM_MOVE_Counted--;

    // desktop icon position may have change with resolution change, 
    // in case of multiple screen and icon other different monitors.
    // This update also CDesktopBrowser::bUsePrimaryDisplayMonitorOnly
    // which is used for percent usage and by CDesktopBrowser::DoesSmallViewBestFit()
    this->Refresh();

    this->bOperationAfterResolutionChangedMustBeDoneBeforeShow = FALSE;
}

void CShellBrowserContainerBase::GetMonitorForRect(IN RECT* pWindowRect,OUT DWORD* pMonitorIndex,OUT RECT* pMonitorRect)
{
    POINT Pt;

    // use middle point of the window
    Pt.x = pWindowRect->left + (pWindowRect->right - pWindowRect->left)/2;
    Pt.y = pWindowRect->top  + (pWindowRect->bottom - pWindowRect->top)/2;

    if (!CDesktopWorkArea::GetWorkAreaForMonitorFromPoint(&Pt,pMonitorIndex,pMonitorRect,FALSE))
    {
        // find monitor owning top left point of the window
        Pt.x = pWindowRect->left;
        Pt.y = pWindowRect->top;
        if (!CDesktopWorkArea::GetWorkAreaForMonitorFromPoint(&Pt,pMonitorIndex,pMonitorRect,FALSE))
        {
            // give another try find monitor owing bottom right point of the window
            Pt.x = pWindowRect->right;
            Pt.y = pWindowRect->bottom;
            if (!CDesktopWorkArea::GetWorkAreaForMonitorFromPoint(&Pt,pMonitorIndex,pMonitorRect,FALSE))
            {
                // give another try find monitor owing bottom left point of the window
                Pt.x = pWindowRect->left;
                Pt.y = pWindowRect->bottom;
                if (!CDesktopWorkArea::GetWorkAreaForMonitorFromPoint(&Pt,pMonitorIndex,pMonitorRect,FALSE))
                {
                    // give another try find monitor owing top right point of the window
                    Pt.x = pWindowRect->right;
                    Pt.y = pWindowRect->top;
                    if (!CDesktopWorkArea::GetWorkAreaForMonitorFromPoint(&Pt,pMonitorIndex,pMonitorRect,FALSE))
                    {
                        // no monitor found, use the main monitor
                        CDesktopWorkArea::GetWorkArea(pMonitorRect,TRUE);
                        if (pMonitorIndex)
                            *pMonitorIndex = 0;
                    }
                }
            }
        }
    }
}

void CShellBrowserContainerBase::AssumeWindowVisibility(IN OUT RECT* pWindowRect,IN RECT* pShadowBorderRect)
{
    // assume window is fully visible (assume to be visible even after a resolution change)
    RECT IntersectRect;
    RECT RealRect;
    BOOL RectFullyVisible = TRUE;
    memcpy(&RealRect,pWindowRect,sizeof(RECT));

    RECT MonitorRect;
    DWORD MonitorIndex;
    this->GetMonitorForRect(pWindowRect,&MonitorIndex,&MonitorRect);

    if (!::IntersectRect(&IntersectRect,&RealRect,&MonitorRect))
        RectFullyVisible = FALSE;
    if (RectFullyVisible)
    {
        // if rect is fully visible
        RectFullyVisible = ( memcmp(&RealRect,&IntersectRect,sizeof(RECT)) == 0);
    }
    if (!RectFullyVisible)
    {
        // 1) adjust size
        if ( (RealRect.right-RealRect.left) > (MonitorRect.right-MonitorRect.left) )
        {
            RealRect.left = MonitorRect.left;
            RealRect.right = (MonitorRect.right-MonitorRect.left);
        }
        if ( (RealRect.bottom-RealRect.top) > (MonitorRect.bottom-MonitorRect.top) )
        {
            RealRect.left = MonitorRect.top;
            RealRect.right = (MonitorRect.bottom-MonitorRect.top);
        }

        // 2) adjust position
        LONG Width = RealRect.right-RealRect.left;
        LONG Height = RealRect.bottom-RealRect.top;

        if ( (this->pSingleFolderOptions->WindowPosition.x + pShadowBorderRect->left) < MonitorRect.left )
            this->pSingleFolderOptions->WindowPosition.x=MonitorRect.left-pShadowBorderRect->left;

        if ( (this->pSingleFolderOptions->WindowPosition.x + Width - pShadowBorderRect->right) > MonitorRect.right )
            this->pSingleFolderOptions->WindowPosition.x=MonitorRect.right - Width + pShadowBorderRect->right;

        if ( (this->pSingleFolderOptions->WindowPosition.y + pShadowBorderRect->top ) < MonitorRect.top )
            this->pSingleFolderOptions->WindowPosition.y=MonitorRect.top - pShadowBorderRect->top;
            
        if ( (this->pSingleFolderOptions->WindowPosition.y + Height - pShadowBorderRect->bottom) > MonitorRect.bottom )
            this->pSingleFolderOptions->WindowPosition.y=MonitorRect.bottom - Height + pShadowBorderRect->bottom;

        RealRect.left = this->pSingleFolderOptions->WindowPosition.x;
        RealRect.right = RealRect.left+Width;
        RealRect.top = this->pSingleFolderOptions->WindowPosition.y;
        RealRect.bottom = RealRect.top+Height;
    }
    memcpy(pWindowRect,&RealRect,sizeof(RECT));

    // from screen to client rect
    // ::OffsetRect(&RealRect,-this->pSingleFolderOptions->WindowPosition.x,-this->pSingleFolderOptions->WindowPosition.y);

    // save window rect changes to options
    memcpy(&this->pSingleFolderOptions->BrowserRect,&RealRect,sizeof(RECT));
}
