/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#pragma once

#include "resource.h"
#include "ContainerBase.h"
#include "BrowserToolbarOptions.h"
#include "../Tools/gui/ShellBrowser/ShellBrowserEx.h"
#include "../Tools/gui/TransparentToolbar/TransparentToolbar.h"
#include "../Tools/gui/HotWindow/HotWindow.h"
#include "../Tools/gui/Dialog/DialogWindowRect.h"

#define CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER

#define CShellBrowserContainer_CLASS_NAME _T("CShellBrowserContainerClassName")
#define CShellBrowserContainer_TOOLBAR_BUTTON_CLICK     (WM_APP+1)
#define CShellBrowserContainer_ENSURE_TOOLBAR_POSITION  (WM_APP+10)
#define CShellBrowserContainer_RESTORE_HOME_ICON_POSITION_NEEDED  (WM_APP+11)

#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
    #define CShellBrowserContainer_RESTORE_ITEMS_POSITION_TIMER_IN_MS     10
    #define CShellBrowserContainer_DEFAULT_NB_SET_POS_RETRY               100 // dev should ensure (restore time * nb try) is no too high
#else
#define CShellBrowserContainer_DEFAULT_NB_SET_POS_RETRY 10000 // seen on win10 up to 700 repost

#endif

#define CShellBrowserContainer_TRANSPARENT_TOOLBAR_BUTTONS_NUMBER 7
#define CShellBrowserContainer_TRANSPARENT_TOOLBAR_BUTTONS_NUMBER_IN_NO_BROWSING_MODE 2

#ifndef _countof
    #define _countof(array) (sizeof(array)/sizeof(array[0]))
#endif
class CShellBrowserContainerBase : public CContainerBase, public CTransparentToolbarEventsBase, public CHotWindowEventsBase
{
protected:
    BOOL InsideViewMoveCounted;
    HWND hWndContainer;
    RECT MaximizedRect;// required for maximizing
    CDialogWindowRect DialogWindowRect;
    CSingleFolderOptions* pSingleFolderOptions;
    CTransparentToolbar* pTransparentToolbar;
    BOOL bRememberLastPosition;
    BOOL bRestoringView;
    LONG DisabledWM_MOVE_Counted;
    ITEMIDLIST* pidlDesktop;
    IMalloc* pMalloc;
    BOOL bIsHomeFolder;
    TCHAR CurrentPath[MAX_PATH];
    CShellBrowser::tagShellBrowserView ShellBrowserView;
    BOOL bSubProcInstalled;
    BOOL bRestoringItemsPositions;
    BOOL bItemsHasBeenSorted;
    HWND hWndListView;
    HWND hWndShellView;
    CBrowserToolbarOptions* ToolbarOptions;
    CHotWindow* pHotWindow;

#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
    SIZE_T RestoreAllItemsNbRetry;
    int RestoreAllItemsLastNbItems;
#endif

    enum TransparentToolbarButtonsId
    {
        TransparentToolbarButtonsId_HOME,
        TransparentToolbarButtonsId_PREV,
        TransparentToolbarButtonsId_NEXT,
        TransparentToolbarButtonsId_UP,
        TransparentToolbarButtonsId_JUMP_TO_EXPLORER,
        TransparentToolbarButtonsId_MOVE,
        TransparentToolbarButtonsId_MAXIMIZE,
        TransparentToolbarButtonsId_CLOSE
    };
    CTransparentToolbar::BUTTON_DESCRIPTION TransparentToolbarMoveButton;
    CTransparentToolbar::BUTTON_DESCRIPTION TransparentToolbarArrayButtons[CShellBrowserContainer_TRANSPARENT_TOOLBAR_BUTTONS_NUMBER];

    BOOL ApplySettingsToClassObject(CSingleFolderOptions* pSingleFolderOptions);
    BOOL CreateContainer();
    BOOL DestroyContainer();
    void UpdateNavigateBackForwardState();
    void UpdateMaximizedRects();

    BOOL RestoreAllItemsPositionsInternal(SIZE_T NbRetry,int LastTryNbIcons);

    virtual BOOL ReturnHandleToOwnerWindowAfterShow();

    void OnCreate();
    void OnClose();
    void OnSize();
    void OnSizing(RECT* pRect);
    void OnMove(WPARAM wParam, LPARAM lParam);
    void OnResolutionChanged();
    void AssumeWindowVisibility(IN OUT RECT* pWindowRect,IN RECT* pShadowBorderRect);
    virtual void MaximizeReduce();
    void Maximize();
    void Reduce();
    static CShellBrowserContainerBase* GetAssociatedObject(HWND hWndDialog);
    static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    virtual void OnTransparentToolbarMove(CTransparentToolbar* pSender,LONG DeltaX,LONG DeltaY);
    virtual void OnTransparentToolbarMoveEnd(CTransparentToolbar* pSender);
    virtual void OnTransparentToolbarButtonClick(CTransparentToolbar* pSender,DWORD ButtonId);
    virtual void OnTransparentToolbarBackgroundLeftDoubleClick(CTransparentToolbar* pSender);
    // OnTransparentToolbarKeyDown: An application should return zero if it processes this message
    virtual LRESULT OnTransparentToolbarKeyDown(CTransparentToolbar* pSender,WPARAM wParam, LPARAM lParam);
    virtual LRESULT OnTransparentToolbarMouseMediaButtonDown(CTransparentToolbar* pSender,WPARAM wParam, LPARAM lParam);
    // OnClose : just use return 0; to avoid window closing (DefWindowProc calls DestroyWindow)
    virtual LRESULT OnTransparentToolbarClose(CTransparentToolbar* pSender,WPARAM wParam,LPARAM lParam);

    virtual void OnHotWindowAutoShow(IN CHotWindow* pSender,OUT BOOL* pUseAutoHideDisabledExtraRect,OUT RECT* pAutoHideDisabledExtraRect);
    virtual void OnHotWindowAutoHide(CHotWindow* pSender);


    void OnTransparentToolbarButtonClick(DWORD ButtonId);
    static void BrowserDefaultCommandCallBack(PVOID UserParam);
    static void BrowserAfterBrowseCallBack(LPITEMIDLIST FolderPidl,PVOID UserParam);
    static void BrowserBeforeBrowseCallBack(LPITEMIDLIST CurrentPidl,LPITEMIDLIST ToBeBrowsePidl,PVOID UserParam);
    static void BrowserViewChangedCallBack(HWND hWndListView,CShellBrowser::tagShellBrowserView NewView,LPVOID UserParam);
    BOOL StoreBrowserView(CShellBrowser::tagShellBrowserView NewView);

    BOOL GetDesktopPidl();
    BOOL ShouldTransparentToolbarBeShown();
    BOOL IsHomeFolder();
    virtual LRESULT OnMessage(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT OnKeyDown_ShellBrowserContainerBase(WPARAM wParam, LPARAM lParam);
    LRESULT OnMessage_ShellBrowserContainerBase(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    virtual BOOL DoesIconsPositionsNeedToBeSaved();
    virtual BOOL DoesIconsPositionsNeedToBeSaved(LPITEMIDLIST CurrentPidl,LPITEMIDLIST ToBeBrowsePidl);
    virtual BOOL DoesIconsPositionsNeedToBeRestored();
    BOOL SaveIconsPositions();
    void ShowListviewBorder(BOOL bShow);
    virtual BOOL Refresh()=0;
    virtual void InstallSubProc()=0;
    virtual void RemoveSubProc()=0;
    virtual void RestoreAllItemsPositions();
    virtual void StopRestoreAllItemsPositions();
    virtual BOOL UpdateOptionsBeforeClose();

    BOOL bOperationAfterResolutionChangedMustBeDoneBeforeShow;
    void DoOperationAfterResolutionChanged();
    
    void EnsureToolBarPosition();

    void GetFolderViewRect(OUT RECT* pFolderViewRect);
    void GetMonitorForRect(IN RECT* pWindowRect,OUT DWORD* pMonitorIndex,OUT RECT* pMonitorRect);
public:
    CShellBrowserContainerBase(HINSTANCE hInstance,CQuickWayToFolders* pQuickWayToFolders);
    virtual ~CShellBrowserContainerBase(void);

    CShellBrowserEx* pShellBrowserEx;
    virtual void Destroy()=0; // overrides CContainerBase::Destroy
    virtual BOOL ApplySettings(CSingleFolderOptions* pSingleFolderOptions);// overrides CContainerBase::ApplySettings
    // bShowOnly : TRUE to show only, FALSE to Show or Hide according to internal state
    virtual BOOL Show(BOOL bShowOnly);// overrides CContainerBase::Show
    virtual BOOL Hide();// overrides CContainerBase::Hide
    BOOL bInsideMaximizeReduce;
    virtual HRESULT GoHome();
    virtual BOOL SetLanguage(CLanguage* pLanguage);
};
