/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ShellBrowserContainerXP.h"

CShellBrowserContainerXP::CShellBrowserContainerXP(HINSTANCE hInstance,CQuickWayToFolders* pQuickWayToFolders):CShellBrowserContainerBase(hInstance,pQuickWayToFolders)
{
    this->bRefreshing = FALSE;
    this->bFirstShow = TRUE;
}

CShellBrowserContainerXP::~CShellBrowserContainerXP(void)
{

}

void CShellBrowserContainerXP::Destroy()
{
    delete this;
}

BOOL CShellBrowserContainerXP::Refresh()
{
    // assume previous refresh has ended
    if (this->bRefreshing)
        return FALSE;

    this->bRefreshing = TRUE;

    if (this->DoesIconsPositionsNeedToBeSaved())
        this->SaveIconsPositions();

    // refresh shell control
    this->pShellBrowserEx->Refresh();

    // 1.2.16 bug solved : case LVN_GETDISPINFOA: && case LVN_GETDISPINFOW: are not enough to reset flag (at least on XP before SP1)
    this->bRefreshing = FALSE;
    return TRUE;
}

void CShellBrowserContainerXP::InstallSubProc()
{
    if (this->bSubProcInstalled)
        return;

    UINT_PTR uIdSubclass=(UINT_PTR)this;
    DWORD_PTR dwRefData=(DWORD_PTR)this;

    ::SetWindowSubclass(this->hWndListView,CShellBrowserContainerXP::SubClassProcListView,uIdSubclass,dwRefData);
    ::SetWindowSubclass(this->hWndShellView,CShellBrowserContainerXP::SubClassProcShellView,uIdSubclass,dwRefData);
}

void CShellBrowserContainerXP::RemoveSubProc()
{
    if (!this->bSubProcInstalled)
        return;

    // remove subclassing (do it first to avoid next messages to be trashed)
    UINT_PTR uIdSubclass=(UINT_PTR)this;
    ::RemoveWindowSubclass(this->hWndListView,CShellBrowserContainerXP::SubClassProcListView,uIdSubclass);
    ::RemoveWindowSubclass(this->hWndShellView,CShellBrowserContainerXP::SubClassProcShellView,uIdSubclass);
#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
    this->StopRestoreAllItemsPositions();
#endif
}

//-----------------------------------------------------------------------------
// Name: SubClassProcListView
// Object: listview subclassing proc 
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CShellBrowserContainerXP::SubClassProcListView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData)
{
    UNREFERENCED_PARAMETER(uIdSubclass);
    CShellBrowserContainerXP* pShellBrowserContainer=(CShellBrowserContainerXP*)dwRefData;
    // in case of IShellView::Refresh, messages order is
    //LVM_DELETEALLITEMS
    //LVM_INSERTITEM (icons only with LPSTR_TEXTCALLBACK flag)
    //LVM_SORTITEMS/LVM_SORTITEMSEX
    //LVN_GETDISPINFO (apply text only after sorting)

#ifdef TRACE_RestoreAllItemsPositionsInternal
    TCHAR Msg[512];
    _sntprintf(Msg,_countof(Msg),TEXT("SubClassProcListView uMsg=%u wParam=%d, lParam=%d\r\n"),uMsg,wParam,lParam);
    OutputDebugString(Msg);
#endif

    switch (uMsg)
    {
    case LVM_ARRANGE:
#ifdef _DEBUG
        OutputDebugString(_T("CShellBrowserContainerXP LVM_ARRANGE\r\n"));
#endif
        if (pShellBrowserContainer->bRestoringItemsPositions)
            return TRUE;
        if (pShellBrowserContainer->DoesIconsPositionsNeedToBeRestored())
            pShellBrowserContainer->RestoreAllItemsPositions();
        return TRUE;

    case LVM_SORTITEMS:
    case LVM_SORTITEMSEX:
        {
#ifdef _DEBUG
            OutputDebugString(_T("CShellBrowserContainerXP LVM_SORTITEMS/LVM_SORTITEMSEX\r\n"));
#endif
            if (pShellBrowserContainer->bRestoringItemsPositions)
                return TRUE;

            if (pShellBrowserContainer->bRefreshing)
                return TRUE;

            pShellBrowserContainer->bItemsHasBeenSorted = TRUE;
            // disable item sorting
            //if (pShellBrowserContainer->DoesIconsPositionsNeedToBeRestored())
            //{
            //    // if all item text are filled
            //    pShellBrowserContainer->RestoreIconsPositions();
            //}
            //return TRUE;
        }
    }
    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
}

//-----------------------------------------------------------------------------
// Name: SubClassProcShellView
// Object: shellview subclassing proc
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CShellBrowserContainerXP::SubClassProcShellView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData)
{
    UNREFERENCED_PARAMETER(uIdSubclass);
    CShellBrowserContainerXP* pShellBrowserContainer=(CShellBrowserContainerXP*)dwRefData;

    switch(uMsg)
    {
#ifdef CShellBrowserContainer_RESTORE_ITEMS_POSITION_USE_TIMER
    case WM_TIMER:
        if (wParam == CShellBrowserContainer_RESTORE_HOME_ICON_POSITION_NEEDED)
        {
            pShellBrowserContainer->RestoreAllItemsPositionsInternal(pShellBrowserContainer->RestoreAllItemsNbRetry,pShellBrowserContainer->RestoreAllItemsLastNbItems);
            // WM_TIMER : An application should return zero if it processes this message
            return 0;
        }
        break;
#else
    case CShellBrowserContainer_RESTORE_HOME_ICON_POSITION_NEEDED:
        pShellBrowserContainer->RestoreAllItemsPositionsInternal(wParam,(int)lParam);
        return 0; // do not call DefSubclassProc
        // break;

#endif
    }
    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
}

BOOL CShellBrowserContainerXP::Show(BOOL bShowOnly)
{
    BOOL bRet = this->CShellBrowserContainerBase::Show(bShowOnly);

    // workaround to solve first show icon bad repositioning on first show
    if (this->bFirstShow)
    {
        this->bFirstShow = FALSE;

        if (this->DoesIconsPositionsNeedToBeRestored())
            this->RestoreAllItemsPositions();
    }

    return bRet;
}
