/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "SingleFolderOptions.h"
#include "../Tools/gui/DesktopWorkArea/DesktopWorkArea.h"

#ifndef _countof
#define _countof(array) (sizeof(array)/sizeof(array[0]))
#endif

CSingleFolderOptions::CSingleFolderOptions(void)
{
    this->FolderPresentationType=FOLDER_PRESENTATION_TYPE_SINGLE_MENU;
    *this->DisplayName = 0;
    _tcscpy(this->Path,_T("c:\\"));
    *this->AssociatedIconPath=0;
    this->AssociatedIconIndex=0;
    *this->LaunchBarAssociatedIconHotPath=0;
    this->LaunchBarAssociatedIconHotIndex=0;
    *this->LaunchBarAssociatedIconDownPath=0;
    this->LaunchBarAssociatedIconDownIndex=0;
    this->LaunchBarIconPosition=CTransparentToolbar::ButtonPosition_CENTER_OR_MIDDLE;

    this->bIncludeSubFolders=TRUE;
    this->bAllowSubFoldersNavigation=TRUE;
    this->bAllowSubFoldersLinksNavigation=TRUE;
    *this->InclusionFilters=0;
    *this->ExclusionFilters=0;
    this->bHideFilesExtensionInPopupMenu=TRUE;
    this->bHideShortcutsOverlayInPopupMenu = TRUE;
    this->FoldersPositionInPopUpMenu=CFolderMenu::FoldersPosition_ON_TOP;
    this->bMergeMenusInPopUpMenu=TRUE;

    // specific to browser / desktop browser
    this->FolderViewMode= CShellBrowserEx::SHELL_BROWSER_VIEW_LIST;
    this->bRememberLastPosition=TRUE;
    this->bOneClickActivate=TRUE;
    this->bAutoHideOnFileExecution=TRUE;
    this->bIsMaximized=FALSE;
    this->ZOrder = BROWSER_Z_ORDER_NORMAL;
    this->bHideToolBar = FALSE;
    this->bHideToolBarForHomeFolder = FALSE;
    this->bShowAtStartup = FALSE;
    this->bStartMaximized = FALSE;
    // specific to browser
    this->bSaveIconsPositionsForHomeFolder = TRUE;
    this->pIconsPositionsForHomeFolder = new std::vector<VECTOR_ICONS_POSITIONS_FOR_HOME_FOLDER>;

    // get desktop size
    RECT Rect;
    CDesktopWorkArea::GetWorkArea(&Rect,TRUE);
    // use desktop size / 3 as default size for first launch
    this->BrowserRect.left=0;
    this->BrowserRect.right=(Rect.right-Rect.left)/3;
    this->BrowserRect.top=0;
    this->BrowserRect.bottom=(Rect.bottom-Rect.top)/3;

    // center window position for first launch
    this->WindowPosition.x=(Rect.right-Rect.left)/3;
    this->WindowPosition.y=(Rect.bottom-Rect.top)/3;

    // specific to browser / desktop browser background / text style 
    this->bTextColorSepcified=FALSE;
    this->TextColor=RGB(0,0,0);
    this->bTextBackgroundColorSepcified=FALSE;
    this->TextBackgroundColor=RGB(0xff,0xff,0xff);
    this->BackgroundType=CShellBrowserEx::BACKGROUND_TYPE_NONE;
    this->BackgroundColor=RGB(0,0,0);
    this->BackgroundColor1=RGB(0,0xff,0);
    this->BackgroundColor2=RGB(0,0,0xff);
    this->BackgroundGradiantType=Gdiplus::LinearGradientModeHorizontal;
    _tcscpy(this->BackgroundImagePath,_T(""));
    this->BackgroundImagePosition=CShellBrowserEx::BACKGROUND_IMAGE_POSITION_STRETCHED;

    this->HotKeyfsModifiers=0;
    this->HotKeyVirtualKey=0;

    this->pMultiplePaths=new std::vector<CFolderMenu::MULTIPLE_PATHS_DATA>();

    // hot corner
    this->bEnableHotCorner = FALSE;
    this->HotCornerScreenIndex = 0;
    this->HotCornerPosition = HOT_CORNER_POSITION_TOP_LEFT;
    this->HotCornerWidth = 8;
    this->HotCornerHeight = 8;

    this->LaunchBarFreeSpaceBeforeIcon = 0;
}

CSingleFolderOptions::~CSingleFolderOptions(void)
{
    if (this->pIconsPositionsForHomeFolder)
        delete this->pIconsPositionsForHomeFolder;

    if (this->pMultiplePaths)
        delete this->pMultiplePaths;
}

// ShortName must be at least MAX_PATH len
BOOL CSingleFolderOptions::GetShortName(IN OUT TCHAR* ShortName)
{
    *ShortName = 0;
    // TCHAR ShortName[MAX_PATH];
    // in case of multiple folders, just put a standard tooltip (app name)
    switch (this->FolderPresentationType)
    {
    case FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU:
        _tcsncpy(ShortName,this->DisplayName,MAX_PATH);
        ShortName[MAX_PATH-1]=0;
        return TRUE;
    default:
        // prefer to use translated folders name so do not use GetChildDirName
        // return CSingleFolderOptions::GetChildDirName(this->Path,ShortName);
        {
            // get path
            TCHAR szPath[MAX_PATH];
            if (!CSingleFolderOptions::GetPath(this->Path,szPath,_countof(szPath)))
                return FALSE;
            SHFILEINFO Shfi;
            if (::SHGetFileInfo(szPath,0,&Shfi,sizeof(Shfi),SHGFI_DISPLAYNAME))
            {
                _tcsncpy(ShortName,Shfi.szDisplayName,MAX_PATH);
                ShortName[MAX_PATH-1]=0;
                return TRUE;
            }
            else
                return CSingleFolderOptions::GetChildDirName(this->Path,ShortName);
        }
    }
}

BOOL CSingleFolderOptions::Save(CIniFile* pIniFile)
{
    pIniFile->Set(CSingleFolderOptions_KEY_FOLDERPRESENTATIONTYPE,(DWORD)this->FolderPresentationType);
    if (this->FolderPresentationType==FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU)
    {
        pIniFile->Set(CSingleFolderOptions_KEY_DISPLAY_NAME,this->DisplayName);

        TCHAR KeyName[MAX_PATH];
        size_t NbDirectories=this->pMultiplePaths->size();
        CFolderMenu::MULTIPLE_PATHS_DATA* pData;
        pIniFile->Set(CSingleFolderOptions_KEY_MULTIPLE_DIRECTORIES_NUMBER,(DWORD)NbDirectories);
        for(size_t Cnt=0; Cnt<NbDirectories; Cnt++)
        {
            pData = &this->pMultiplePaths->at(Cnt);
            _sntprintf(KeyName,_countof(KeyName),CSingleFolderOptions_KEY_MULTIPLE_DIRECTORIES_DIR_PREFIX _T("%u"),Cnt);
            pIniFile->Set(KeyName,pData->Path);

            _sntprintf(KeyName,_countof(KeyName),CSingleFolderOptions_KEY_MULTIPLE_DIRECTORIES_MERGE_TO_SPECIFIC_FOLDER_PREFIX _T("%u"),Cnt);
            pIniFile->Set(KeyName,(DWORD)pData->MergeToSpecificFolder);

            _sntprintf(KeyName,_countof(KeyName),CSingleFolderOptions_KEY_MULTIPLE_DIRECTORIES_MERGED_FOLDER_NAME_PREFIX _T("%u"),Cnt);
            pIniFile->Set(KeyName,pData->MergedSpecificFolderName);
        }
    }
    pIniFile->Set(CSingleFolderOptions_KEY_PATH,this->Path);
    pIniFile->Set(CSingleFolderOptions_KEY_ASSOCIATEDICONPATH,this->AssociatedIconPath);
    pIniFile->Set(CSingleFolderOptions_KEY_ASSOCIATEDICONINDEX,this->AssociatedIconIndex);
    pIniFile->Set(CSingleFolderOptions_KEY_ASSOCIATEDHOTICONPATH,this->LaunchBarAssociatedIconHotPath);
    pIniFile->Set(CSingleFolderOptions_KEY_ASSOCIATEDHOTICONINDEX,this->LaunchBarAssociatedIconHotIndex);
    pIniFile->Set(CSingleFolderOptions_KEY_ASSOCIATEDDOWNICONPATH,this->LaunchBarAssociatedIconDownPath);
    pIniFile->Set(CSingleFolderOptions_KEY_ASSOCIATEDDOWNICONINDEX,this->LaunchBarAssociatedIconDownIndex);
    pIniFile->Set(CSingleFolderOptions_KEY_ASSOCIATEDICONPOSITION,(DWORD)this->LaunchBarIconPosition);
    pIniFile->Set(CSingleFolderOptions_KEY_INCLUDESUBFOLDERS,(DWORD)this->bIncludeSubFolders);
    pIniFile->Set(CSingleFolderOptions_KEY_ALLOWSUBFOLDERSNAVIGATION,(DWORD)this->bAllowSubFoldersNavigation);
    pIniFile->Set(CSingleFolderOptions_KEY_ALLOWSUBFOLDERSLINKSNAVIGATION,(DWORD)this->bAllowSubFoldersLinksNavigation);
    pIniFile->Set(CSingleFolderOptions_KEY_INCLUSIONFILTERS,this->InclusionFilters);
    pIniFile->Set(CSingleFolderOptions_KEY_EXCLUSIONFILTERS,this->ExclusionFilters);
    pIniFile->Set(CSingleFolderOptions_KEY_HIDE_FILES_EXTENSION_IN_POPUPMENU,(DWORD)this->bHideFilesExtensionInPopupMenu);
    pIniFile->Set(CSingleFolderOptions_KEY_HIDE_SHORTCUTS_OVERLAY_IN_POPUPMENU,(DWORD)this->bHideShortcutsOverlayInPopupMenu);
    pIniFile->Set(CSingleFolderOptions_KEY_FOLDER_POSITION_IN_POPUPMENU,(DWORD)this->FoldersPositionInPopUpMenu);
    pIniFile->Set(CSingleFolderOptions_KEY_MERGE_MENUS_IN_POPUPMENU,(DWORD)this->bMergeMenusInPopUpMenu);
    pIniFile->Set(CSingleFolderOptions_KEY_FOLDERVIEWMODE,(DWORD)this->FolderViewMode);
    pIniFile->Set(CSingleFolderOptions_KEY_REMEMBERLASTPOSITION,(DWORD)this->bRememberLastPosition);
    pIniFile->Set(CSingleFolderOptions_KEY_ONECLICKACTIVATE,(DWORD)this->bOneClickActivate);
    pIniFile->Set(CSingleFolderOptions_KEY_AUTOHIDEONFILEEXECUTION,(DWORD)this->bAutoHideOnFileExecution);
    pIniFile->Set(CSingleFolderOptions_KEY_ISMAXIMIZED,(DWORD)this->bIsMaximized);
    pIniFile->Set(CSingleFolderOptions_KEY_BROWSERWIDTH,(DWORD)(this->BrowserRect.right-this->BrowserRect.left));
    pIniFile->Set(CSingleFolderOptions_KEY_BROWSERHEIGHT,(DWORD)(this->BrowserRect.bottom-this->BrowserRect.top));
    pIniFile->Set(CSingleFolderOptions_KEY_WINDOWPOSITIONX,(DWORD)(this->WindowPosition.x));
    pIniFile->Set(CSingleFolderOptions_KEY_WINDOWPOSITIONY,(DWORD)(this->WindowPosition.y));
    pIniFile->Set(CSingleFolderOptions_KEY_Z_ORDER,(DWORD)this->ZOrder);
    pIniFile->Set(CSingleFolderOptions_KEY_HIDE_TOOLBAR_FOR_HOME_FOLDER,(DWORD)this->bHideToolBarForHomeFolder);
    pIniFile->Set(CSingleFolderOptions_KEY_HIDE_TOOLBAR,(DWORD)this->bHideToolBar);
    pIniFile->Set(CSingleFolderOptions_KEY_SHOW_AT_STARTUP,(DWORD)this->bShowAtStartup);
    pIniFile->Set(CSingleFolderOptions_KEY_START_MAXIMIZED,(DWORD)this->bStartMaximized);
    pIniFile->Set(CSingleFolderOptions_KEY_SAVE_ICONS_POSITIONS_FOR_HOME_FOLDER,(DWORD)this->bSaveIconsPositionsForHomeFolder);
    if (this->bSaveIconsPositionsForHomeFolder)
    {
        INT Cnt;
        INT NbIcons;
        TCHAR KeyName[MAX_PATH];
        CSingleFolderOptions::VECTOR_ICONS_POSITIONS_FOR_HOME_FOLDER VectorItem;

        NbIcons = (INT)this->pIconsPositionsForHomeFolder->size();
        pIniFile->Set(CSingleFolderOptions_KEY_NB_ICONS_IN_HOME_FOLDER,(DWORD)NbIcons);
        for (Cnt = 0; Cnt<NbIcons; Cnt++)
        {
            VectorItem = this->pIconsPositionsForHomeFolder->at(Cnt);
            _sntprintf(KeyName,_countof(KeyName),_T("%s_%u"),CSingleFolderOptions_KEY_HOME_FOLDER_ITEM_NAME,(DWORD)Cnt);
            pIniFile->Set(KeyName,VectorItem.Name);
            _sntprintf(KeyName,_countof(KeyName),_T("%s_%u"),CSingleFolderOptions_KEY_HOME_FOLDER_ITEM_X_POSITION,(DWORD)Cnt);
            pIniFile->Set(KeyName,(DWORD)VectorItem.Position.x);
            _sntprintf(KeyName,_countof(KeyName),_T("%s_%u"),CSingleFolderOptions_KEY_HOME_FOLDER_ITEM_Y_POSITION,(DWORD)Cnt);
            pIniFile->Set(KeyName,(DWORD)VectorItem.Position.y);
        }
    }

    pIniFile->Set(CSingleFolderOptions_KEY_TEXTCOLORSEPCIFIED,(DWORD)this->bTextColorSepcified);
    pIniFile->Set(CSingleFolderOptions_KEY_TEXTCOLOR,this->TextColor);
    pIniFile->Set(CSingleFolderOptions_KEY_TEXTBACKGROUNDCOLORSEPCIFIED,(DWORD)this->bTextBackgroundColorSepcified);
    pIniFile->Set(CSingleFolderOptions_KEY_TEXTBACKGROUNDCOLOR,this->TextBackgroundColor);
    pIniFile->Set(CSingleFolderOptions_KEY_BACKGROUNDTYPE,(DWORD)this->BackgroundType);
    pIniFile->Set(CSingleFolderOptions_KEY_BACKGROUNDCOLOR,this->BackgroundColor);
    pIniFile->Set(CSingleFolderOptions_KEY_BACKGROUNDCOLOR1,this->BackgroundColor1);
    pIniFile->Set(CSingleFolderOptions_KEY_BACKGROUNDCOLOR2,this->BackgroundColor2);
    pIniFile->Set(CSingleFolderOptions_KEY_BACKGROUNDGRADIANTTYPE,(DWORD)this->BackgroundGradiantType);
    pIniFile->Set(CSingleFolderOptions_KEY_BACKGROUNDIMAGEPATH,this->BackgroundImagePath);
    pIniFile->Set(CSingleFolderOptions_KEY_BACKGROUNDIMAGEPOSITION,(DWORD)this->BackgroundImagePosition);
    pIniFile->Set(CSingleFolderOptions_HOTKEY_MODIFIER,this->HotKeyfsModifiers);
    pIniFile->Set(CSingleFolderOptions_HOTKEY_VIRTUALKEY,this->HotKeyVirtualKey);
    pIniFile->Set(CSingleFolderOptions_ENABLEHOTCORNER,(DWORD)this->bEnableHotCorner);
    pIniFile->Set(CSingleFolderOptions_HOTCORNERSCREENINDEX,this->HotCornerScreenIndex);
    pIniFile->Set(CSingleFolderOptions_HOTCORNERPOSITION,(DWORD)this->HotCornerPosition);
    pIniFile->Set(CSingleFolderOptions_HOTCORNERWIDTH,this->HotCornerWidth);
    pIniFile->Set(CSingleFolderOptions_HOTCORNERHEIGHT,this->HotCornerHeight);
    pIniFile->Set(CSingleFolderOptions_LAUNCHBARSPACERBEFOREICONINPIXELS,this->LaunchBarFreeSpaceBeforeIcon);

#pragma message ("LineBreakBeforeIcon to be implemented")

    return TRUE;
}
BOOL CSingleFolderOptions::Load(CIniFile* pIniFile)
{
    pIniFile->Get(CSingleFolderOptions_KEY_FOLDERPRESENTATIONTYPE,FOLDER_PRESENTATION_TYPE_SINGLE_MENU,(DWORD*)&this->FolderPresentationType);
    if (this->FolderPresentationType==CSingleFolderOptions::FOLDER_PRESENTATION_TYPE_DESKTOP_BROWSER)
        _sntprintf(this->Path,MAX_PATH,CSingleFolderOptions_CSIDL_PREFIX _T("0x%X"),CSIDL_DESKTOP);
    else
        pIniFile->Get(CSingleFolderOptions_KEY_PATH,_T("c:\\"),this->Path,MAX_PATH);

    if (this->FolderPresentationType==FOLDER_PRESENTATION_TYPE_MULTIPLE_MENU)
    {
        pIniFile->Get(CSingleFolderOptions_KEY_DISPLAY_NAME,_T(""),this->DisplayName,_countof(DisplayName));
        TCHAR KeyName[MAX_PATH];
        CFolderMenu::MULTIPLE_PATHS_DATA Data={0};
        DWORD NbDirectories=0;
        pIniFile->Get(CSingleFolderOptions_KEY_MULTIPLE_DIRECTORIES_NUMBER,0,&NbDirectories);
        for(DWORD Cnt=0; Cnt<NbDirectories; Cnt++)
        {
            _sntprintf(KeyName,_countof(KeyName),CSingleFolderOptions_KEY_MULTIPLE_DIRECTORIES_DIR_PREFIX _T("%u"),Cnt);
            pIniFile->Get(KeyName,_T("c:\\"),Data.Path,MAX_PATH);

            _sntprintf(KeyName,_countof(KeyName),CSingleFolderOptions_KEY_MULTIPLE_DIRECTORIES_MERGE_TO_SPECIFIC_FOLDER_PREFIX _T("%u"),Cnt);
            pIniFile->Get(KeyName,FALSE,(DWORD*)&Data.MergeToSpecificFolder);

            _sntprintf(KeyName,_countof(KeyName),CSingleFolderOptions_KEY_MULTIPLE_DIRECTORIES_MERGED_FOLDER_NAME_PREFIX _T("%u"),Cnt);
            pIniFile->Get(KeyName,_T("MergedFolder"),Data.MergedSpecificFolderName,CFolderMenu_MergedSpecificFolderName_SIZE);

            this->pMultiplePaths->push_back(Data);
        }
    }

    pIniFile->Get(CSingleFolderOptions_KEY_ASSOCIATEDICONPATH,_T(""),this->AssociatedIconPath,MAX_PATH);
    pIniFile->Get(CSingleFolderOptions_KEY_ASSOCIATEDICONINDEX,0,(DWORD*)&this->AssociatedIconIndex);
    pIniFile->Get(CSingleFolderOptions_KEY_ASSOCIATEDHOTICONPATH,this->AssociatedIconPath,this->LaunchBarAssociatedIconHotPath,MAX_PATH);
    pIniFile->Get(CSingleFolderOptions_KEY_ASSOCIATEDHOTICONINDEX,this->AssociatedIconIndex,(DWORD*)&this->LaunchBarAssociatedIconHotIndex);
    pIniFile->Get(CSingleFolderOptions_KEY_ASSOCIATEDDOWNICONPATH,this->AssociatedIconPath,this->LaunchBarAssociatedIconDownPath,MAX_PATH);
    pIniFile->Get(CSingleFolderOptions_KEY_ASSOCIATEDDOWNICONINDEX,this->AssociatedIconIndex,(DWORD*)&this->LaunchBarAssociatedIconDownIndex);
    pIniFile->Get(CSingleFolderOptions_KEY_ASSOCIATEDICONPOSITION,0,(DWORD*)&this->LaunchBarIconPosition);
    pIniFile->Get(CSingleFolderOptions_KEY_INCLUDESUBFOLDERS,TRUE,(DWORD*)&this->bIncludeSubFolders);
    pIniFile->Get(CSingleFolderOptions_KEY_ALLOWSUBFOLDERSNAVIGATION,TRUE,(DWORD*)&this->bAllowSubFoldersNavigation);
    pIniFile->Get(CSingleFolderOptions_KEY_ALLOWSUBFOLDERSLINKSNAVIGATION,TRUE,(DWORD*)&this->bAllowSubFoldersLinksNavigation);
    pIniFile->Get(CSingleFolderOptions_KEY_INCLUSIONFILTERS,_T(""),this->InclusionFilters,CSingleFolderOptions_FILTERS_MAX_SIZE);
    pIniFile->Get(CSingleFolderOptions_KEY_EXCLUSIONFILTERS,_T(""),this->ExclusionFilters,CSingleFolderOptions_FILTERS_MAX_SIZE);
    pIniFile->Get(CSingleFolderOptions_KEY_HIDE_FILES_EXTENSION_IN_POPUPMENU,TRUE,(DWORD*)&this->bHideFilesExtensionInPopupMenu);
    pIniFile->Get(CSingleFolderOptions_KEY_HIDE_SHORTCUTS_OVERLAY_IN_POPUPMENU,TRUE,(DWORD*)&this->bHideShortcutsOverlayInPopupMenu);
    pIniFile->Get(CSingleFolderOptions_KEY_FOLDER_POSITION_IN_POPUPMENU,CFolderMenu::FoldersPosition_ON_TOP,(DWORD*)&this->FoldersPositionInPopUpMenu);
    pIniFile->Get(CSingleFolderOptions_KEY_MERGE_MENUS_IN_POPUPMENU,TRUE,(DWORD*)&this->bMergeMenusInPopUpMenu);
    pIniFile->Get(CSingleFolderOptions_KEY_FOLDERVIEWMODE,CShellBrowserEx::SHELL_BROWSER_VIEW_LIST,(DWORD*)&this->FolderViewMode);
    pIniFile->Get(CSingleFolderOptions_KEY_REMEMBERLASTPOSITION,TRUE,(DWORD*)&this->bRememberLastPosition);
    pIniFile->Get(CSingleFolderOptions_KEY_ONECLICKACTIVATE,TRUE,(DWORD*)&this->bOneClickActivate);
    pIniFile->Get(CSingleFolderOptions_KEY_AUTOHIDEONFILEEXECUTION,TRUE,(DWORD*)&this->bAutoHideOnFileExecution);
    pIniFile->Get(CSingleFolderOptions_KEY_ISMAXIMIZED,FALSE,(DWORD*)&this->bIsMaximized);

    // get desktop size
    RECT Rect;
    CDesktopWorkArea::GetWorkArea(&Rect,TRUE);
    // use desktop size / 3 as default size for first launch
    this->BrowserRect.left=0;
    this->BrowserRect.right=(Rect.right-Rect.left)/3;
    this->BrowserRect.top=0;
    this->BrowserRect.bottom=(Rect.bottom-Rect.top)/3;
    // center window position for first launch
    this->WindowPosition.x=(Rect.right-Rect.left)/3;
    this->WindowPosition.y=(Rect.bottom-Rect.top)/3;

    pIniFile->Get(CSingleFolderOptions_KEY_BROWSERWIDTH,this->BrowserRect.right,(DWORD*)&this->BrowserRect.right);
    pIniFile->Get(CSingleFolderOptions_KEY_BROWSERHEIGHT,this->BrowserRect.bottom,(DWORD*)&this->BrowserRect.bottom);
    pIniFile->Get(CSingleFolderOptions_KEY_WINDOWPOSITIONX,this->WindowPosition.x,(DWORD*)&this->WindowPosition.x);
    pIniFile->Get(CSingleFolderOptions_KEY_WINDOWPOSITIONY,this->WindowPosition.y,(DWORD*)&this->WindowPosition.y);
    pIniFile->Get(CSingleFolderOptions_KEY_Z_ORDER,BROWSER_Z_ORDER_NORMAL,(DWORD*)&this->ZOrder);
    pIniFile->Get(CSingleFolderOptions_KEY_HIDE_TOOLBAR_FOR_HOME_FOLDER,FALSE,(DWORD*)&this->bHideToolBarForHomeFolder);
    pIniFile->Get(CSingleFolderOptions_KEY_HIDE_TOOLBAR,FALSE,(DWORD*)&this->bHideToolBar);
    pIniFile->Get(CSingleFolderOptions_KEY_SHOW_AT_STARTUP,FALSE,(DWORD*)&this->bShowAtStartup);
    pIniFile->Get(CSingleFolderOptions_KEY_START_MAXIMIZED,FALSE,(DWORD*)&this->bStartMaximized);
    if (this->bStartMaximized)
        this->bIsMaximized = TRUE;
    pIniFile->Get(CSingleFolderOptions_KEY_SAVE_ICONS_POSITIONS_FOR_HOME_FOLDER,TRUE,(DWORD*)&this->bSaveIconsPositionsForHomeFolder);
    if (this->bSaveIconsPositionsForHomeFolder)
    {
        DWORD Cnt;
        DWORD NbIcons;
        TCHAR KeyName[MAX_PATH];
        CSingleFolderOptions::VECTOR_ICONS_POSITIONS_FOR_HOME_FOLDER VectorItem;

        pIniFile->Get(CSingleFolderOptions_KEY_NB_ICONS_IN_HOME_FOLDER,0,&NbIcons);
        for (Cnt = 0; Cnt<NbIcons; Cnt++)
        {
            _sntprintf(KeyName,_countof(KeyName),_T("%s_%u"),CSingleFolderOptions_KEY_HOME_FOLDER_ITEM_NAME,Cnt);
            pIniFile->Get(KeyName,_T(""),VectorItem.Name,MAX_PATH);
            _sntprintf(KeyName,_countof(KeyName),_T("%s_%u"),CSingleFolderOptions_KEY_HOME_FOLDER_ITEM_X_POSITION,Cnt);
            pIniFile->Get(KeyName,0,(DWORD*)&VectorItem.Position.x);
            _sntprintf(KeyName,_countof(KeyName),_T("%s_%u"),CSingleFolderOptions_KEY_HOME_FOLDER_ITEM_Y_POSITION,Cnt);
            pIniFile->Get(KeyName,0,(DWORD*)&VectorItem.Position.y);
            this->pIconsPositionsForHomeFolder->push_back(VectorItem);
        }
    }

    pIniFile->Get(CSingleFolderOptions_KEY_TEXTCOLORSEPCIFIED,FALSE,(DWORD*)&this->bTextColorSepcified);
    pIniFile->Get(CSingleFolderOptions_KEY_TEXTCOLOR,RGB(0,0,0),(DWORD*)&this->TextColor);
    pIniFile->Get(CSingleFolderOptions_KEY_TEXTBACKGROUNDCOLORSEPCIFIED,FALSE,(DWORD*)&this->bTextBackgroundColorSepcified);
    pIniFile->Get(CSingleFolderOptions_KEY_TEXTBACKGROUNDCOLOR,RGB(0xff,0xff,0xff),(DWORD*)&this->TextBackgroundColor);
    pIniFile->Get(CSingleFolderOptions_KEY_BACKGROUNDTYPE,CShellBrowserEx::BACKGROUND_TYPE_NONE,(DWORD*)&this->BackgroundType);
    pIniFile->Get(CSingleFolderOptions_KEY_BACKGROUNDCOLOR,RGB(0,0,0),(DWORD*)&this->BackgroundColor);
    pIniFile->Get(CSingleFolderOptions_KEY_BACKGROUNDCOLOR1,RGB(0,0xff,0),(DWORD*)&this->BackgroundColor1);
    pIniFile->Get(CSingleFolderOptions_KEY_BACKGROUNDCOLOR2,RGB(0,0,0xff),(DWORD*)&this->BackgroundColor2);
    pIniFile->Get(CSingleFolderOptions_KEY_BACKGROUNDGRADIANTTYPE,Gdiplus::LinearGradientModeHorizontal,(DWORD*)&this->BackgroundGradiantType);
    pIniFile->Get(CSingleFolderOptions_KEY_BACKGROUNDIMAGEPATH,_T(""),this->BackgroundImagePath,MAX_PATH);
    pIniFile->Get(CSingleFolderOptions_KEY_BACKGROUNDIMAGEPOSITION,CShellBrowserEx::BACKGROUND_IMAGE_POSITION_STRETCHED,(DWORD*)&this->BackgroundImagePosition);
    pIniFile->Get(CSingleFolderOptions_HOTKEY_MODIFIER,0,&this->HotKeyfsModifiers);
    pIniFile->Get(CSingleFolderOptions_HOTKEY_VIRTUALKEY,0,&this->HotKeyVirtualKey);
    pIniFile->Get(CSingleFolderOptions_ENABLEHOTCORNER,FALSE,(DWORD*)&this->bEnableHotCorner);
    pIniFile->Get(CSingleFolderOptions_HOTCORNERSCREENINDEX,0,&this->HotCornerScreenIndex);
    pIniFile->Get(CSingleFolderOptions_HOTCORNERPOSITION,HOT_CORNER_POSITION_TOP_LEFT,(DWORD*)&this->HotCornerPosition);
    pIniFile->Get(CSingleFolderOptions_HOTCORNERWIDTH,8,&this->HotCornerWidth);
    pIniFile->Get(CSingleFolderOptions_HOTCORNERHEIGHT,8,&this->HotCornerHeight);
    pIniFile->Get(CSingleFolderOptions_LAUNCHBARSPACERBEFOREICONINPIXELS,0,&this->LaunchBarFreeSpaceBeforeIcon);
#pragma message ("LineBreakBeforeIcon to be implemented")
    return TRUE;
}

BOOL CSingleFolderOptions::GetChildDirName(TCHAR* CSIDLorPath,OUT TCHAR* ChildDir)
{
    *ChildDir=0;

    // get path
    TCHAR szPath[MAX_PATH];
    if (!CSingleFolderOptions::GetPath(CSIDLorPath,szPath,_countof(szPath)))
        return FALSE;

    // if szPath ends with '\\', remove back slash
    size_t szPathLasCharPos=_tcslen(szPath)-1;
    if (szPath[szPathLasCharPos]=='\\')
        szPath[szPathLasCharPos]='\0';

    // use GetFileName to get last child directory name
    _tcscpy(ChildDir,CStdFileOperations::GetFileName(szPath));

    return TRUE;
}

BOOL CSingleFolderOptions::IsCSIDL()
{
    return CSingleFolderOptions::IsCSIDL(this->Path);
}
BOOL CSingleFolderOptions::IsCSIDL(TCHAR* CSIDLorPath)
{
    return (_tcsnicmp(CSingleFolderOptions_CSIDL_PREFIX,CSIDLorPath,_tcslen(CSingleFolderOptions_CSIDL_PREFIX))==0);
}

BOOL CSingleFolderOptions::GetCSIDL(TCHAR* CSIDLorPath,OUT DWORD* pCSIDL)
{
    *pCSIDL = 0;
    if (!CSingleFolderOptions::IsCSIDL(CSIDLorPath))
        return FALSE;

    return (_stscanf(CSIDLorPath,CSingleFolderOptions_CSIDL_PREFIX _T("0x%X"),pCSIDL) == 1);
}

BOOL CSingleFolderOptions::GetPath(OUT TCHAR* Path,IN SIZE_T PathMaxSize)
{
    return CSingleFolderOptions::GetPath(this->Path,Path,PathMaxSize);
}

BOOL CSingleFolderOptions::GetPath(TCHAR* CSIDLorPath,OUT TCHAR* Path,IN SIZE_T PathMaxSize)
{
    *Path = 0; // in case path doesn't exists
    if (!CSingleFolderOptions::IsCSIDL(CSIDLorPath))
    {
        if(!CStdFileOperations::GetAbsolutePath(CSIDLorPath,Path,(DWORD)PathMaxSize))
            return FALSE;
        return CStdFileOperations::DoesDirectoryExists(Path);
    }
    int CSIDL_Value=0;
    if (_stscanf(CSIDLorPath,CSingleFolderOptions_CSIDL_PREFIX _T("0x%X"),&CSIDL_Value)!=1)
        return FALSE;

    return SUCCEEDED(::SHGetFolderPath(NULL,CSIDL_FLAG_CREATE | CSIDL_Value,NULL,SHGFP_TYPE_CURRENT,Path));
}