/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class to quickly check file names according to a filter list
//-----------------------------------------------------------------------------
#include "FilesFilters.h"

//-----------------------------------------------------------------------------
// Name: CFilesFilters
// Object: constructor
// Parameters :
//     in  : tagFilterType FiltersType : FILTER_TYPE_EXCLUSION or FILTER_TYPE_INCLUSION
//           TCHAR* Filters : wild char pattern splitted by ';' (like like "*.exe;*.jpg;b?gin*end;*.mp3")
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
CFilesFilters::CFilesFilters(tagFilterType FiltersType,TCHAR* Filters)
{
    this->Filters=NULL;
    this->FiltersType=FiltersType;

    // if null string
    if (!Filters)
        return;

    // if empty string
    if (!*Filters)// if (Filters[0]=='\0') 
        return;

    // do a local copy of filters to use _tcstok
    this->Filters=_tcsdup(Filters);
    if (!this->Filters)
        return;

    /////////////////////////////
    // translate filters into a list
    /////////////////////////////
    TCHAR* Filter = _tcstok( this->Filters, CFilesFilters_SPLITTER );
    if (Filter)
        this->FiltersList.push_back(Filter);
    while( Filter )
    {
        Filter = _tcstok( NULL, CFilesFilters_SPLITTER );
        if (Filter)
            this->FiltersList.push_back(Filter);
    }


}

CFilesFilters::~CFilesFilters(void)
{
    if (this->Filters)
        free(this->Filters);
}

//-----------------------------------------------------------------------------
// Name: Check
// Object: check file name according to filter
// Parameters :
//     in  : TCHAR* FileName : file name to check according to filter
//     out :
//     return : TRUE if file name respect filters, FALSE else
//-----------------------------------------------------------------------------
BOOL CFilesFilters::Check(TCHAR* FileName)
{
    BOOL bChecked;
    size_t NbItems=this->FiltersList.size();
    if (NbItems==0)
        return TRUE;

    // for each filter of the list
    for (size_t Cnt=0;Cnt<NbItems;Cnt++)
    {
        // do a wild char case insensitive comparison
        bChecked=CWildCharCompare::WildICmp(this->FiltersList[Cnt],FileName);

        // return according to result and filter type
        if (bChecked)
        {
            switch(this->FiltersType)
            {
                // for exclusion, item must match all exclusion filters
                // so as soon as item match, we must return FALSE
                case FILTER_TYPE_EXCLUSION:
                    return FALSE;

                // for inclusion filter, item must match only one filter,
                // so as soon as item match, we must return TRUE
                case FILTER_TYPE_INCLUSION:
                    return TRUE;
            }
        }
    }

    switch(this->FiltersType)
    {
        // for exclusion, that means item match all exclusion filters;
        // so we must return TRUE
        case FILTER_TYPE_EXCLUSION:
            return TRUE;

        // for inclusion filter, item must match at least one filter,
        // and if we are here that means it's not the case; so we must return FALSE
        case FILTER_TYPE_INCLUSION:
            return FALSE;

        default:
            return TRUE;
    }
}