/*
Copyright (C) 2014-2020 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2014 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: get windows desktop work area for single or multiple monitors 
//         as ::SystemParametersInfo(SPI_GETWORKAREA,0,&Rect,0); returns only main monitor rect
//-----------------------------------------------------------------------------

#pragma once

#include <windows.h>

class CDesktopWorkArea
{
private:
    static BOOL CALLBACK GetWorkAreaCallback(HMONITOR hMonitor,HDC hdcMonitor,LPRECT lprcMonitor,LPARAM lpData);
    static BOOL CALLBACK GetNumberOfDisplayMonitorCallback(HMONITOR hMonitor,HDC hdcMonitor,LPRECT lprcMonitor,LPARAM lpData);
    static BOOL CALLBACK GetWorkAreaForMonitorCallback(HMONITOR hMonitor,HDC hdcMonitor,LPRECT lprcMonitor,LPARAM lpData);
    
    typedef struct tagGetWorkAreaCallbackParam
    {
        RECT PrimaryDisplayWorkAreaRect;
        RECT AllDisplaysWorkAreaRect;
        BOOL PrimaryDisplayInfoFilled;
        BOOL GetMonitorRectInsteadOfWorkArea;
    }GET_WORK_AREA_CALLBACK_PARAM;

    enum tagGetWorkAreaForMonitorWay {GetWorkAreaForMonitorWay_FROM_INDEX=0,GetWorkAreaForMonitorWay_FROM_POINT};
    typedef struct tagGetWorkAreaForMonitorCallbackParam
    {
        DWORD WantedMonitorIndex;
        DWORD CurrentMonitorIndex;
        RECT DisplayWorkAreaRect;
        BOOL DisplayInfoFilled;
        BOOL GetMonitorRectInsteadOfWorkArea;
        tagGetWorkAreaForMonitorWay Way;
        POINT Point;
    }GET_WORK_AREA_FOR_MONITOR_CALLBACK_PARAM;

public:
    // get desktop work area. Warning left and top can be negative !
    static BOOL GetWorkArea(OUT RECT* pWorkAreaRect,BOOL bPrimaryDisplayMonitorOnly,BOOL bGetFullMonitorRectInsteadOfWorkArea=FALSE);
    static BOOL GetWorkAreaEx(OUT RECT* pPrimaryDisplayWorkAreaRect,OUT RECT* pAllDisplaysWorkAreaRect,BOOL bGetFullMonitorRectInsteadOfWorkArea=FALSE);
    static DWORD GetNumberOfDisplayMonitors();
    static BOOL GetWorkAreaForMonitor(IN DWORD ZeroBasedMonitorIndex,OUT RECT* pWorkAreaRect,BOOL bGetFullMonitorRectInsteadOfWorkArea=FALSE);
    static BOOL GetWorkAreaForMonitorFromPoint(IN POINT* pPoint,OUT DWORD* pMonitorIndex,OUT RECT* pWorkAreaRect,BOOL bGetFullMonitorRectInsteadOfWorkArea=FALSE);
};
