/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "DialogWindowRect.h"

#include <Dwmapi.h>

CDialogWindowRect::CDialogWindowRect()
{
    this->pfDwmIsCompositionEnabled = NULL;
    this->pfDwmGetWindowAttribute = NULL;
    this->dwmapiDllHandle = ::LoadLibrary(L"dwmapi.dll");  
    if (this->dwmapiDllHandle)
    {
        this->pfDwmIsCompositionEnabled = (pfnDwmIsCompositionEnabled) ::GetProcAddress(this->dwmapiDllHandle, "DwmIsCompositionEnabled");
        this->pfDwmGetWindowAttribute = (pfnDwmGetWindowAttribute) ::GetProcAddress(this->dwmapiDllHandle, "DwmGetWindowAttribute") ;
    }

    // this->pGetDpiForWindow = (pfnGetDpiForWindow) ::GetProcAddress(::GetModuleHandle(TEXT("user32.dll")), "GetDpiForWindow") ;
}
CDialogWindowRect::~CDialogWindowRect()
{
    if (this->dwmapiDllHandle)
        ::FreeLibrary(this->dwmapiDllHandle); 
}

BOOL CDialogWindowRect::GetWindowRectWithShadow(IN HWND hWndDialog,OUT RECT* pDialogRect)
{
    return ::GetWindowRect(hWndDialog, pDialogRect);
}

#pragma message("To be checked on new win10 versions Warning: DwmGetWindowAttribute BUG !!! hWndDialog must have been shown to work properly (at least on win10 technical preview. So the calls ::ShowWindow(hWnd,SW_SHOWNORMAL);::ShowWindow(hWnd,SW_HIDE); are required for hidden dialog at least once for the wanted dialog")
#include <math.h>
BOOL CDialogWindowRect::GetWindowRectWithoutShadow(IN HWND hWndDialog,OUT RECT* pDialogRect)
{
    static RECT ShadowRect;
    static BOOL ShadowRectFilled = FALSE;
    if (!::GetWindowRect(hWndDialog, pDialogRect)) // default to old way of getting the window rectangle
        return FALSE; // invalid window handle, no need to go further

    if (this->pfDwmIsCompositionEnabled)
    {
        BOOL bIsEnabled = FALSE;
        HRESULT hResult;
        hResult = this->pfDwmIsCompositionEnabled(&bIsEnabled);
        if ( SUCCEEDED(hResult) && (bIsEnabled == TRUE) )
        {
            // when ms should better have continued to use GetSystemMetrics... 
            // instead of asking for TPM2.0, start using good coders (and stop releasing untested functions)
            if(this->pfDwmGetWindowAttribute)
            {
                // call the bugged ms API as less as possible, using cached shadow rect
                if (ShadowRectFilled) // doing caching introduce a limitation : function should be called only for dialog having shadow
                {
                    pDialogRect->left += ShadowRect.left;
                    pDialogRect->right -= ShadowRect.right;
                    pDialogRect->top += ShadowRect.top;
                    pDialogRect->bottom -= ShadowRect.bottom;
                }
                else
                {
                    RECT ScaledRectWithoutShadow;
                    // warning DWMWA_EXTENDED_FRAME_BOUNDS return scaled rect !!!
                    this->pfDwmGetWindowAttribute( hWndDialog, DWMWA_EXTENDED_FRAME_BOUNDS, &ScaledRectWithoutShadow, sizeof(RECT) ) ;

                    // if no-shadow right border is greater than with shadow border => it's incoherent so scale != 1.0 (aka there is a scale factor)
                    // Notice : pfDwmGetWindowAttribute is so bugged (at least in win 10 21H2) that you can get correct bottom value but wrong right value
                    //          so to be sure that scaling is applied you need to check both
                    if ( (ScaledRectWithoutShadow.right>pDialogRect->right) && (ScaledRectWithoutShadow.bottom>pDialogRect->bottom) )
                    {
                        // get scale factor from hand as the entire ms api scaling / dpi sucks (at least in 2023 with win7, win10, win11)

                        // In theory we should have
                        // ScalingFactor = ScaledRectWithoutShadow.right / NotScaledDialogRectWithoutShadow.right
                        // but as we don't have NotScaledDialogRectWithoutShadow, we use NotScaledDialogRectWithShadow as an approximation
                        // so as NotScaledDialogRectWithShadow is larger than NotScaledDialogRectWithoutShadow, the real scale factor is a little larger than the computed one,
                        // that why we use ceiling to increase factor to the upper 0.1 value
                        // Notice: we use rect right instead of rect width to avoid to double the shadow width scaling factor error

                        // compute approximate factor
                        double ScalingFactor = (double)ScaledRectWithoutShadow.right/(double)pDialogRect->right; // notice this is a little wrong as there is the shadow delta

                        // increase approximate factor to the upper 0.1 value
                        ScalingFactor = ceil(ScalingFactor*10.0)/10.0; 

                        DWORD PercentScalingFactor = (DWORD)(ScalingFactor*100.0); // we get an implicit rounding
                        // on win 10 basic scaling factor can be 125, 150 and 175
                        // so adjust the values below 180 to these predefined values (avoid scaling approximation errors)
                        if (PercentScalingFactor<180)
                        {
                            if (PercentScalingFactor>165)
                                PercentScalingFactor = 175;
                            else if (PercentScalingFactor>135)
                                PercentScalingFactor = 150;
                            else 
                                PercentScalingFactor = 125;

                            ScalingFactor = ((double)PercentScalingFactor)/100.0;
                        }

                        // remove scaling from WithoutShadow rect
                        RECT NotScaledRectWithoutShadow;
                        NotScaledRectWithoutShadow.left = (LONG)(ScaledRectWithoutShadow.left / ScalingFactor);
                        NotScaledRectWithoutShadow.right = (LONG)(ScaledRectWithoutShadow.right / ScalingFactor);
                        NotScaledRectWithoutShadow.top = (LONG)(ScaledRectWithoutShadow.top / ScalingFactor);
                        NotScaledRectWithoutShadow.bottom = (LONG)(ScaledRectWithoutShadow.bottom / ScalingFactor);

                        ShadowRect.left = NotScaledRectWithoutShadow.left-pDialogRect->left;
                        ShadowRect.top = NotScaledRectWithoutShadow.top-pDialogRect->top;
                        ShadowRect.right = pDialogRect->right-NotScaledRectWithoutShadow.right;
                        ShadowRect.bottom = pDialogRect->bottom-NotScaledRectWithoutShadow.bottom;
                    }
                    else
                    {
                        if (ScaledRectWithoutShadow.right>pDialogRect->right)
                        {
                            // try to correct the sucking API (bug at least in win 10 21H2)
                            // suppose that there is the same shadow width on left and right side
                            ScaledRectWithoutShadow.right = pDialogRect->right - (ScaledRectWithoutShadow.left-pDialogRect->left);
                        }
                        else if (ScaledRectWithoutShadow.bottom>pDialogRect->bottom)
                        {
                            ScaledRectWithoutShadow.bottom = pDialogRect->bottom;
                        }

                        ShadowRect.left = ScaledRectWithoutShadow.left-pDialogRect->left;
                        ShadowRect.top = ScaledRectWithoutShadow.top-pDialogRect->top;
                        ShadowRect.right = pDialogRect->right-ScaledRectWithoutShadow.right;
                        ShadowRect.bottom = pDialogRect->bottom-ScaledRectWithoutShadow.bottom;

                    }

                    if (   ((ShadowRect.left<0) || (ShadowRect.left > 20))
                        || ((ShadowRect.right<0) || (ShadowRect.right > 20))
                        || ((ShadowRect.top<0) || (ShadowRect.top > 20))
                        || ((ShadowRect.bottom<0) || (ShadowRect.bottom > 20))
                        )
                    {
                        // use win 10 default shadow value
                        ShadowRect.top = 0;
                        ShadowRect.left = 6;
                        ShadowRect.right = 6;
                        ShadowRect.bottom = 6;
                    }

                    ShadowRectFilled = TRUE; // next time we will use cached value

                    pDialogRect->left += ShadowRect.left;
                    pDialogRect->right -= ShadowRect.right;
                    pDialogRect->top += ShadowRect.top;
                    pDialogRect->bottom -= ShadowRect.bottom;

                

                    // WM_DPICHANGED
                    // SetProcessDpiAwarenessContext(DPI_AWARENESS_CONTEXT_UNAWARE)
                    //if (this->pGetDpiForWindow)
                    //{
                    //    // surprisingly ??? GetDpiForWindow sucks and return 96 even for scaling of 125
                    //    UINT WindowDpi = this->pGetDpiForWindow(hWndDialog);
                    //    if (WindowDpi != USER_DEFAULT_SCREEN_DPI)
                    //    {
                    //        double Ratio = ((double)USER_DEFAULT_SCREEN_DPI)/WindowDpi;
                    //        pDialogRect->left *= Ratio;
                    //        pDialogRect->right *= Ratio;
                    //        pDialogRect->top *= Ratio;
                    //        pDialogRect->bottom *= Ratio;
                    //    }
                    //}
                }
            }
        }
    }

    return TRUE;
}
