/*
Copyright (C) 2004-2016 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2016 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manage a folder menu (menu containing folder items)
//-----------------------------------------------------------------------------

#include "FolderMenu.h"
#include "../../ShellExecute/ShellExecuteAndBringToTopEx.h"
// #define CFolderMenu_USE_SHELL_FILE_NAMES // can cause troubles for start menu (sytem+user) as m$ is too lazy (at least on win7) to translate system start menu
                                            // --> shell folder names of user is translated bu not those of system --> 2 differents directories for same menu entry
#include <algorithm> // std::sort

//-----------------------------------------------------------------------------
// Name: CFolderMenu
// Object: Constructor.
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
CFolderMenu::CFolderMenu(void)
{
    this->bMergedMenu = TRUE;
    this->FoldersPosition = FoldersPosition_ON_TOP;
    this->bShowSubFolders=TRUE;
    this->bExpandSubFolders=FALSE;
    this->bExpandSubFoldersLinks=FALSE;
    this->pPopUpMenu=NULL;
    this->hWndDialog=NULL;
    this->pFilesFiltersInclusion=NULL;
    this->pFilesFiltersExclusion=NULL;
    this->bHideFilesExtension = TRUE;
    this->bHideShortcutsOverlay = TRUE;
}

//-----------------------------------------------------------------------------
// Name: ~CFolderMenu
// Object: Destructor.
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
CFolderMenu::~CFolderMenu(void)
{
    this->FreeMenus();

    if (this->pFilesFiltersInclusion)
        delete this->pFilesFiltersInclusion;

    if (this->pFilesFiltersExclusion)
        delete this->pFilesFiltersExclusion;
}

//-----------------------------------------------------------------------------
// Name: FreeMenus
// Object: free memory associated to menus
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CFolderMenu::FreeMenus()
{
    // if memory is already free
    if (!this->pPopUpMenu)
        return;

    std::vector<CFolderPopUpMenuInfos*>::iterator Array_pMenusIterator;
    CFolderPopUpMenuInfos* pMenu;
    std::vector<CFolderPopUpMenuItemInfos*>::iterator Array_pFolderPopUpMenuItemInfosIterator;
    CFolderPopUpMenuItemInfos* pMenuItemInfos;
    // search through menu array
    for (Array_pMenusIterator = this->AllMenusAndSubMenusList.begin();
        Array_pMenusIterator!=this->AllMenusAndSubMenusList.end(); 
        Array_pMenusIterator++
        )
    {
        pMenu = *Array_pMenusIterator;
        // search through menu item array
        for (Array_pFolderPopUpMenuItemInfosIterator=pMenu->FolderPopUpMenuItemInfosList.begin();
            Array_pFolderPopUpMenuItemInfosIterator!=pMenu->FolderPopUpMenuItemInfosList.end(); 
            Array_pFolderPopUpMenuItemInfosIterator++
            )
        {
            pMenuItemInfos = *Array_pFolderPopUpMenuItemInfosIterator;
            // destroy CFolderPopUpMenuItemInfos item
            delete pMenuItemInfos;
        }

        // destroy CPopUpMenu associated to CFolderPopUpMenuInfos
        delete pMenu->pPopUpMenu;

        // destroy CFolderPopUpMenuInfos
        delete pMenu;
    }
    // clear AllMenusAndSubMenusList
    this->AllMenusAndSubMenusList.clear();

    // this->pPopUpMenu has already been deleted as it belongs to this->AllMenusAndSubMenusList
    this->pPopUpMenu=NULL;
}

//-----------------------------------------------------------------------------
// Name: Show
// Object: Display folder menu
// Parameters :
//     in  : int x : x pos of menu
//           int y : y pos of menu
//           HWND hOwner : owning dialog HWND
//     out :
//     return : TRUE on success, FALSE on error or if user cancel action
//-----------------------------------------------------------------------------
BOOL CFolderMenu::Show(int x,int y, HWND hOwner)
{
    if (!this->pPopUpMenu)
        return FALSE;

    this->hWndDialog=hOwner;

    // display pop up menu
    UINT ItemId=this->pPopUpMenu->Show(x,y,hOwner);

    // DoDefaultAction
    return this->DoDefaultAction(ItemId);
}

//-----------------------------------------------------------------------------
// Name: Show
// Object: Display folder menu
// Parameters :
//     in  : int x : x pos of menu
//           int y : y pos of menu
//           HWND hOwner : owning dialog HWND
//           BOOL PositionRelativeToOwner : if TRUE x and y are relative to hOwner
//     out :
//     return : TRUE on success, FALSE on error or if user cancel action
//-----------------------------------------------------------------------------
BOOL CFolderMenu::Show(int x,int y, HWND hOwner,BOOL PositionRelativeToOwner)
{
    if (!this->pPopUpMenu)
        return FALSE;

    this->hWndDialog=hOwner;

    // display pop up menu
    UINT ItemId=this->pPopUpMenu->Show(x,y,hOwner,PositionRelativeToOwner);

    // DoDefaultAction
    return this->DoDefaultAction(ItemId);
}

//-----------------------------------------------------------------------------
// Name: Show
// Object: Display folder menu
// Parameters :
//     in  : int x : x pos of menu
//           int y : y pos of menu
//           HWND hOwner : owning dialog HWND
//           BOOL PositionRelativeToOwner : if TRUE x and y are relative to hOwner
//           BOOL ShowUpper : if TRUE, menu is shown upper x,y point
//     out :
//     return : TRUE on success, FALSE on error or if user cancel action
//-----------------------------------------------------------------------------
BOOL CFolderMenu::Show(int x,int y, HWND hOwner,BOOL PositionRelativeToOwner,BOOL ShowUpper)
{
    if (!this->pPopUpMenu)
        return FALSE;

    this->hWndDialog=hOwner;

    // display pop up menu
    UINT ItemId=this->pPopUpMenu->Show(x,y,hOwner,PositionRelativeToOwner,ShowUpper);

    // DoDefaultAction
    return this->DoDefaultAction(ItemId);
}

//-----------------------------------------------------------------------------
// Name: Fill
// Object: Fill menu with specified path content
// Parameters :
//     in  : TCHAR* Path : path to be parsed
//           BOOL bShowSubFolders : TRUE to show sub folders
//           BOOL bExpandSubFolders : TRUE to expand sub folders
//     out :
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CFolderMenu::Fill(MULTIPLE_PATHS_DATA* PathData,BOOL bShowSubFolders,BOOL bExpandSubFolders,BOOL bExpandSubFoldersLinks)
{
    return this->Fill(PathData,1,bShowSubFolders,bExpandSubFolders,bExpandSubFoldersLinks);
}

//-----------------------------------------------------------------------------
// Name: Fill
// Object: Fill menu with specified paths content
// Parameters :
//     in  : TCHAR** Paths : array of paths to be parsed
//           DWORD NbPaths : number of paths
//           BOOL bShowSubFolders : TRUE to show sub folders
//           BOOL bExpandSubFolders : TRUE to expand sub folders
//     out :
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CFolderMenu::Fill(MULTIPLE_PATHS_DATA* PathsData,DWORD NbPaths,BOOL bShowSubFolders,BOOL bExpandSubFolders,BOOL bExpandSubFoldersLinks)
{
    // store options
    this->bExpandSubFolders=bExpandSubFolders;
    this->bExpandSubFoldersLinks=bExpandSubFoldersLinks;
    this->bShowSubFolders=bShowSubFolders;
    

    // delete previous menus if any
    this->FreeMenus();

    // create new pop up menu
    this->pPopUpMenu=new CPopUpMenu();
    if (!this->pPopUpMenu)
        return FALSE;

    // set menu events handler callback
    this->pPopUpMenu->SetEventsHandler((IPopUpMenuEvents*)(CPopUpMenuEventsBase*)this);

    // add menu to AllMenusAndSubMenusList
    CFolderPopUpMenuInfos* pMenuInfos=new CFolderPopUpMenuInfos(this->pPopUpMenu);
    this->AllMenusAndSubMenusList.push_back(pMenuInfos);

    BOOL bRet=FALSE;
    switch (NbPaths)
    {
    case 0:
        bRet = TRUE;
        break;
    case 1:
        // fill menu
        bRet = this->FillMenuInfos(pMenuInfos,PathsData[0].Path,FALSE);
        break;
    default:
        {
            CFolderPopUpMenuItemInfos* pFolderPopUpMenuItemInfos;

            // set menu as filled
            pMenuInfos->Filled=TRUE;

            bRet = TRUE;

            if (this->bMergedMenu)
            {
                for (DWORD Cnt=0;Cnt<NbPaths;Cnt++)
                {
                    this->FillMenuInfos(pMenuInfos,PathsData[Cnt].Path,TRUE);
                }
                // as menu are merged, menu content is shown, so sort menu content
                this->DoPostProcessingAfterItemsRetrieval(pMenuInfos,TRUE);
            }
            else
            {
                std::vector<CFolderPopUpMenuItemInfos*> MergedMenuListToFill;
                std::vector<CFolderPopUpMenuItemInfos*>::iterator MergedMenuIterator;
                BOOL bAlreadyAdded;
                for (DWORD Cnt=0;Cnt<NbPaths;Cnt++)
                {
                    if (PathsData[Cnt].MergeToSpecificFolder)
                    {
                        if (this->AddFileToMenu(pMenuInfos,PathsData[Cnt].Path,PathsData[Cnt].MergedSpecificFolderName,TRUE,TRUE,TRUE,TRUE,&pFolderPopUpMenuItemInfos))
                        {
                            // force first level preloading :
                            // this force dll containing first level icon to be loaded in memory
                            // to avoid possible latency at first display
                            this->FillMenuInfos(pFolderPopUpMenuItemInfos->pChildFolderPopUpMenuInfos,PathsData[Cnt].Path,TRUE);


                            // if not already in MergedMenuListToFill, add it
                            bAlreadyAdded = FALSE;
                            for (MergedMenuIterator= MergedMenuListToFill.begin();MergedMenuIterator!=MergedMenuListToFill.end();MergedMenuIterator++)
                            {
                                if (*MergedMenuIterator == pFolderPopUpMenuItemInfos)
                                {
                                    bAlreadyAdded=TRUE;
                                    break;
                                }
                            }
                            if (!bAlreadyAdded)
                                MergedMenuListToFill.push_back(pFolderPopUpMenuItemInfos);
                        }
                        else
                        {
                            bRet=FALSE;
                        }
                    }
                    else
                    {
                        if (this->AddFileToMenu(pMenuInfos,PathsData[Cnt].Path,CStdFileOperations::GetFileName(PathsData[Cnt].Path),TRUE,TRUE,FALSE,FALSE,&pFolderPopUpMenuItemInfos))
                        {
                            // force first level preloading :
                            // this force dll containing first level icon to be loaded in memory
                            // to avoid possible latency at first display
                            this->FillMenuInfos(pFolderPopUpMenuItemInfos->pChildFolderPopUpMenuInfos,PathsData[Cnt].Path,FALSE);
                        }
                        else
                        {
                            bRet=FALSE;
                        }
                    }
                }

                // fill root menu (level 0 menu)
                // keep separated folders order
                this->DoPostProcessingAfterItemsRetrieval(pMenuInfos,FALSE);

                // fill all merged menu (level 1 menus)
                for (MergedMenuIterator= MergedMenuListToFill.begin();MergedMenuIterator!=MergedMenuListToFill.end();MergedMenuIterator++)
                {
                    this->DoPostProcessingAfterItemsRetrieval((*MergedMenuIterator)->pChildFolderPopUpMenuInfos,TRUE);
                }
            }
        }
        break;
    }

    return bRet;
}

BOOL CFolderMenu::FillMenuInfos(CFolderPopUpMenuInfos* pMenuInfos,std::vector<CFolderPopUpMenuItemInfos::tagPath>* pPathList)
{
    BOOL bRetValue=FALSE;
    BOOL TmpRetValue;
    std::vector<CFolderPopUpMenuItemInfos::tagPath>::iterator Iter;
    for (Iter = pPathList->begin();Iter!=pPathList->end();Iter++)
    {
        // use temp var to assume function call even c++ optimization
        TmpRetValue = this->FillMenuInfos(pMenuInfos,Iter->Path,TRUE);
        bRetValue = bRetValue || TmpRetValue;
    }
    this->DoPostProcessingAfterItemsRetrieval(pMenuInfos,TRUE);

    return bRetValue;
}

//-----------------------------------------------------------------------------
// Name: Fill
// Object: fill menu associated to pMenuInfos with folder content
// Parameters :
//     in  : CFolderPopUpMenuInfos* pMenuInfos : menu informations
//           TCHAR* Path : path to be parsed. Content of path will be added to menu associated to pMenuInfos
//     out :
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CFolderMenu::FillMenuInfos(CFolderPopUpMenuInfos* pMenuInfos,TCHAR* Path,BOOL FromMultipleSingleLevelPath)
{
    BOOL bRet=TRUE;
    TCHAR FileName[MAX_PATH];
    TCHAR LocalPath[MAX_PATH+1];
    TCHAR Search[MAX_PATH+2];
    WIN32_FIND_DATA FindFileData={0};

    if (!pMenuInfos->pPopUpMenu)
        return FALSE;

#ifdef _DEBUG
    TCHAR Msg[512];
    _sntprintf(Msg,_countof(Msg),TEXT("Filling menu for folder %s\r\n"),Path);
    Msg[_countof(Msg)-1]=0;
    ::OutputDebugString(Msg);
#endif

    // assume path ends with '\'
    _tcsncpy(LocalPath,Path,_countof(LocalPath));
    LocalPath[_countof(LocalPath)-2]=0;
    if (LocalPath[_tcslen(LocalPath)-1]!='\\')
        _tcscat(LocalPath,_T("\\"));

    // add * after path to create the searched string
    _tcscpy(Search,LocalPath);
    _tcscat(Search,_T("*"));

    // set menu as filled
    pMenuInfos->Filled=TRUE;

    // find first file
    HANDLE hFileHandle = ::FindFirstFile(Search,&FindFileData);
    if (hFileHandle==INVALID_HANDLE_VALUE)// if no matching file found
    {
        bRet=FALSE;
        goto End;
    }

    do 
    {
        // make full path from file name
        _tcscpy(FileName,FindFileData.cFileName);
        _sntprintf(FindFileData.cFileName,MAX_PATH,_T("%s%s"),LocalPath,FileName);
        FindFileData.cFileName[MAX_PATH-1]=0;

        // if file is a directory
        BOOL bFolder=FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;
        // add file to menu
        this->AddFileToMenu(pMenuInfos,FindFileData.cFileName,bFolder,FromMultipleSingleLevelPath);

      // find next file
    } while(::FindNextFile(hFileHandle, &FindFileData));

    ::FindClose(hFileHandle);

End:
    if (!FromMultipleSingleLevelPath)
        this->DoPostProcessingAfterItemsRetrieval(pMenuInfos,TRUE);

    return bRet;
}
// return true if a<b
bool CFolderMenu::CompareMenuItemsFoldersOnBottom( CFolderPopUpMenuItemInfos* a, CFolderPopUpMenuItemInfos* b)
{
    if (a->IsFolder() && !b->IsFolder())
        return false;

    if (!a->IsFolder() && b->IsFolder())
        return true;

    return CFolderMenu::CompareMenuItemsFoldersNoSpecificPosition(a,b);
}
// return true if a<b
bool CFolderMenu::CompareMenuItemsFoldersOnTop( CFolderPopUpMenuItemInfos* a, CFolderPopUpMenuItemInfos* b)
{
    if (a->IsFolder() && !b->IsFolder())
        return true;

    if (!a->IsFolder() && b->IsFolder())
        return false;

    return CFolderMenu::CompareMenuItemsFoldersNoSpecificPosition(a,b);
}

// return true if a<b
bool CFolderMenu::CompareMenuItemsFoldersNoSpecificPosition(CFolderPopUpMenuItemInfos* a,CFolderPopUpMenuItemInfos* b)
{
    // Do menu sorting: folders first, next files, by pseudo alphabetical order [if number at first place do number sorting]

    // if number at first place do number sorting
    BOOL aBeginsWithNumber = a->IsDisplayNameBeginingWithNumber();
    BOOL bBeginsWithNumber = b->IsDisplayNameBeginingWithNumber();
    UINT32 aValue;
    UINT32 bValue;

    if (aBeginsWithNumber && !bBeginsWithNumber)
        return true;

    if (!aBeginsWithNumber && bBeginsWithNumber)
        return false;

    if (aBeginsWithNumber /* && bBeginsWithNumber not required */)
    {
        _stscanf(a->DisplayName,_T("%u"),&aValue);
        _stscanf(b->DisplayName,_T("%u"),&bValue);

        if (aValue<bValue)
            return true;
        else if (aValue>bValue)
            return false;
        // else
            // let default string comparison do the job
    }

    // basic string compare
    if (_tcsicmp(a->DisplayName,b->DisplayName)<0)
        return true;
    else
        return false;
}

BOOL CFolderMenu::DoPostProcessingAfterItemsRetrieval(CFolderPopUpMenuInfos* pMenuInfos,BOOL bDoMenuSorting)
{

    // should not occur anymore for sub menus since empty folders are auto hidden
    // can occur only for main menu (paths provided by user)
    if (pMenuInfos->FolderPopUpMenuItemInfosList.size() == 0)
    {
        UINT ItemId=pMenuInfos->pPopUpMenu->Add(_T("Empty"));
        if (ItemId)
            pMenuInfos->pPopUpMenu->SetEnabledState(ItemId,FALSE);

        // menu is empty, no further processing is requested
        return TRUE;
    }

    if (bDoMenuSorting)
    {
        // Why stl sucks so much and doesn't provide additional user pointer ????
        typedef bool (*pfCompare)(CFolderPopUpMenuItemInfos* a,CFolderPopUpMenuItemInfos* b);
        pfCompare pCompare;
        switch(this->FoldersPosition)
        {
            default:
            case FoldersPosition_NO_SPECIFIC_POSITION:
                pCompare = CFolderMenu::CompareMenuItemsFoldersNoSpecificPosition;
                break;
            case FoldersPosition_ON_TOP:
                pCompare = CFolderMenu::CompareMenuItemsFoldersOnTop;
                break;
            case FoldersPosition_ON_BOTTOM:
                pCompare = CFolderMenu::CompareMenuItemsFoldersOnBottom;
                break;
        }

        // sort items contained in pMenuInfos->FolderPopUpMenuItemInfosList
        std::sort(pMenuInfos->FolderPopUpMenuItemInfosList.begin(), pMenuInfos->FolderPopUpMenuItemInfosList.end(), pCompare );
    }

    ///////////////////////////////////////////////////////////////////
    // add content of pMenuInfos->FolderPopUpMenuItemInfosList to menu
    ///////////////////////////////////////////////////////////////////
    std::vector<CFolderPopUpMenuItemInfos*>::iterator pMenuItemInfosIterator;
    CFolderPopUpMenuItemInfos* pMenuItemInfos;

    // search through menu item array
    for (pMenuItemInfosIterator=pMenuInfos->FolderPopUpMenuItemInfosList.begin();
        pMenuItemInfosIterator!=pMenuInfos->FolderPopUpMenuItemInfosList.end(); 
        pMenuItemInfosIterator++
        )
    {
        pMenuItemInfos = *pMenuItemInfosIterator;

        // if item is a folder and folder needs to be expanded
        if (pMenuItemInfos->IsFolder() && pMenuItemInfos->pChildFolderPopUpMenuInfos)
        {
            // add submenu to current menu
            pMenuItemInfos->Id = pMenuInfos->pPopUpMenu->AddSubMenu(pMenuItemInfos->DisplayName,pMenuItemInfos->pChildFolderPopUpMenuInfos->pPopUpMenu,pMenuItemInfos->hIcon);
        }
        else // item or not expanded folder
        {
            if (this->bHideFilesExtension && ( !pMenuItemInfos->IsFolder() ) )
            {
                // remove file extension
                CStdFileOperations::RemoveFileExt(pMenuItemInfos->DisplayName);
            }

            // add item to menu
            pMenuItemInfos->Id = pMenuInfos->pPopUpMenu->Add(pMenuItemInfos->DisplayName,pMenuItemInfos->hIcon);
        }
    }
    return TRUE;
}

BOOL CFolderMenu::IsFolderEmpty(TCHAR* Path)
{
    BOOL bIsEmpty=TRUE;
    TCHAR FileName[MAX_PATH];
    TCHAR LocalPath[MAX_PATH];
    TCHAR Search[2*MAX_PATH];
    TCHAR LinkTarget[MAX_PATH];
    WIN32_FIND_DATA FindFileData={0};
    BOOL bRespectInclusionFilters;
    BOOL bRespectExclusionFilters;
    BOOL bFolder;
    TCHAR* pFileName;
    TCHAR* FolderPath = NULL;
    SIZE_T LocalPathSize;

    if (Path == NULL)
        return TRUE;

    if (*Path == 0)
        return TRUE;

    if (_tcslen(Path)>=(MAX_PATH-2))
        return TRUE;

    // assume path ends with '\'
    _tcscpy(LocalPath,Path);
    if (LocalPath[_tcslen(LocalPath)-1]!='\\')
        _tcscat(LocalPath,_T("\\"));

    LocalPathSize=_tcslen(LocalPath);

    // add * after path to create the searched string
    _tcscpy(Search,LocalPath);
    _tcscat(Search,_T("*"));

    // put LocalPath to lower
    _tcslwr(LocalPath);

    // find first file
    HANDLE hFileHandle=::FindFirstFile(Search,&FindFileData);
    if (hFileHandle==INVALID_HANDLE_VALUE)// if no matching file found
        return TRUE;

    do 
    {
        _tcscpy(FileName,FindFileData.cFileName);

        // new path > MAX_PATH : an error will occur --> concider it as empty
        if ((LocalPathSize+_tcslen(FileName))>(MAX_PATH-1))
            continue;

        // make full path from file name
        _sntprintf(FindFileData.cFileName,MAX_PATH,_T("%s%s"),LocalPath,FileName);
        FindFileData.cFileName[MAX_PATH-1]=0;

        // if file is a directory
        bFolder=FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;

        if (bFolder)
        {
            FolderPath = FindFileData.cFileName;
        }
        else
        {
            if (this->bExpandSubFoldersLinks)
            {
                // if file is link
                if ( CStdFileOperations::DoesExtensionMatch(FindFileData.cFileName,_T("lnk")) )
                {
                    if ( this->GetLinkTarget(FindFileData.cFileName,LinkTarget) )
                    {
                        if ( CStdFileOperations::IsDirectory(LinkTarget) )
                        {
                            // if link point to an accessible folder
                            if ( CStdFileOperations::DoesDirectoryExists(LinkTarget) )
                            {
                                FolderPath = LinkTarget;
                                bFolder = TRUE;
                            }
                        }
                        else if (CStdFileOperations::DoesExtensionMatch(LinkTarget,_T("library-ms")))
                        {
                            if (!this->bShowSubFolders)
                                continue;

                            if (this->IsLibraryEmpty(LinkTarget))
                                continue;
                            else
                            {
                                // sub folder is not empty --> current folder is not empty
                                bIsEmpty = FALSE;
                                break;
                            }
                        }
                    }
                }
            }
        }

        // if item is a folder
        if (bFolder)
        {
            if (!this->bShowSubFolders)
                continue;

            // get file name
            // find last '\' 
            pFileName=_tcsrchr(FolderPath,'\\');
            // if found
            if (FileName)
            {
                pFileName++;

                // assume folder is not current or upper directory
                if ( (_tcscmp(pFileName,_T("."))==0)
                    || (_tcscmp(pFileName,_T(".."))==0))
                    continue;
            }

            _tcslwr(FolderPath);
            // if new folder path is included LocalPath
            // that means we are going to infinite loop
            if (_tcsstr(LocalPath,FolderPath))
            {
                // sub folder is not empty --> current folder is not empty
                bIsEmpty = FALSE;
                break;
            }

            if (this->IsFolderEmpty(FolderPath))
            {
                // if folder is empty check next file/dir (we can't conclude yet)
                continue;
            }
            else
            {
                // sub folder is not empty --> current folder is not empty
                bIsEmpty = FALSE;
                break;
            }
        }
        else // item is a file
        {
            bRespectInclusionFilters=TRUE;
            bRespectExclusionFilters=TRUE;
            // do filters checking
            if (this->pFilesFiltersInclusion)
            {
                if (!this->pFilesFiltersInclusion->Check(FileName))
                    bRespectInclusionFilters = FALSE;
            }

            if (this->pFilesFiltersExclusion)
            {
                if (!this->pFilesFiltersExclusion->Check(FileName))
                    bRespectExclusionFilters = FALSE;
            }

            if (bRespectInclusionFilters && bRespectExclusionFilters)
            {
                bIsEmpty = FALSE;
                break;
            }
        }

        // find next file
    } while(::FindNextFile(hFileHandle, &FindFileData));

    ::FindClose(hFileHandle);

    return bIsEmpty;
}

// GetLinkTarget
// IN TCHAR* LnkPath : .lnk file path
// OUT TCHAR* IconPath : in out path pointed by link, must be at least MAX_PATH char buffer must be allocated by caller
// IN OUT int* pIconIndex : icon index
BOOL CFolderMenu::GetLinkIcon(IN TCHAR* LnkPath, IN OUT TCHAR* IconPath,IN OUT int* pIconIndex)
{
    BOOL bRet = FALSE;
    HRESULT hResult;
    IShellLink *pShellLink = NULL;
    IPersistFile *pPersistFile = NULL;
    WCHAR* pwLinkName;
    *IconPath=0;
    *pIconIndex=0;

    // create IShellLink object
    hResult = ::CoCreateInstance(  CLSID_ShellLink,
        NULL,
        CLSCTX_INPROC_SERVER,
        IID_IShellLink,
        (LPVOID*)&pShellLink);
    if ( (hResult!=S_OK) || (pShellLink==NULL) )
        goto CleanUp;

    // get IPersistFile interface
    hResult = pShellLink->QueryInterface(IID_IPersistFile,(LPVOID*)&pPersistFile);
    if ( (hResult!=S_OK) || (pPersistFile==NULL) )
        goto CleanUp;

#if (defined(UNICODE)||defined(_UNICODE))
    pwLinkName=LnkPath;
#else
    CAnsiUnicodeConvert::AnsiToUnicode(LnkPath,&pwLinkName);
#endif

    // load shortcut infos
    hResult = pPersistFile->Load(pwLinkName,STGM_READ);
#if ( (!defined(UNICODE)) && (!defined(_UNICODE)))
    free(pwLinkName);
#endif
    if ( hResult!=S_OK )
    {
#ifdef _DEBUG // avoid noisy messages as current function is going to be called on each lnk present in menu folder
        this->ShowComError(hResult);
#endif
        goto CleanUp;
    }

    // don't work for new links...
    //hResult = pShellLink->GetPath(  TargetPath,
    //                                MAX_PATH,
    //                                &ShellLinkWin32FindData,
    //                                SLGP_UNCPRIORITY 
    //                              );
    // going to the new m$ way
    DWORD Flags;
    Flags = SLR_ANY_MATCH
            | SLR_INVOKE_MSI 
            | SLR_NO_UI 
            | SLR_UPDATE;
    hResult = pShellLink->Resolve(NULL,Flags);
// try to continue even in case of Resolve failure
//    if ( hResult!=S_OK )
//    {
//#ifdef _DEBUG // avoid noisy messages as current function is going to be called on each lnk present in menu folder
//        this->ShowComError(hResult);
//#endif
//        goto CleanUp;
//    }

    WCHAR* pIconLocation;
#if (defined(UNICODE)||defined(_UNICODE))
    pIconLocation = IconPath;
#else
    WCHAR wIconLocation[MAX_PATH];
    pIconLocation = wIconLocation;
#endif

    hResult = pShellLink->GetIconLocation(pIconLocation,MAX_PATH,pIconIndex);

#if ( (!defined(UNICODE)) && (!defined(_UNICODE)) )
    CAnsiUnicodeConvert::UnicodeToAnsi(wIconLocation,IconPath,MAX_PATH);
#endif

    if ( (hResult!=S_OK) || (*IconPath==0) ) // as always m$ sucks and is happy to return S_OK in case of failure (shortcuts on computers or control panel)
        goto CleanUp;

    bRet = TRUE;
CleanUp:

    if (pShellLink)
        pShellLink->Release();

    if (pPersistFile)
        pPersistFile->Release();

    return bRet;
}

// GetLinkTarget
// IN TCHAR* LnkPath : .lnk file path
// OUT TCHAR* TargetPath : in out path pointed by link, must be at least MAX_PATH char buffer must be allocated by caller
BOOL CFolderMenu::GetLinkTarget(IN TCHAR* LnkPath, IN OUT TCHAR* TargetPath)
{
    BOOL bRet = FALSE;
    HRESULT hResult;
    IShellLink *pShellLink = NULL;
    IPersistFile *pPersistFile = NULL;
    WCHAR* pwLinkName;
    PIDLIST_ABSOLUTE pidl = NULL;
    *TargetPath=0;

    WIN32_FIND_DATA ShellLinkWin32FindData;

    // create IShellLink object
    hResult = ::CoCreateInstance(  CLSID_ShellLink,
                                    NULL,
                                    CLSCTX_INPROC_SERVER,
                                    IID_IShellLink,
                                    (LPVOID*)&pShellLink);
    if ( (hResult!=S_OK) || (pShellLink==NULL) )
        goto CleanUp;

    // get IPersistFile interface
    hResult = pShellLink->QueryInterface(IID_IPersistFile,(LPVOID*)&pPersistFile);
    if ( (hResult!=S_OK) || (pPersistFile==NULL) )
        goto CleanUp;

#if (defined(UNICODE)||defined(_UNICODE))
    pwLinkName=LnkPath;
#else
    CAnsiUnicodeConvert::AnsiToUnicode(LnkPath,&pwLinkName);
#endif

    // load shortcut infos
    hResult = pPersistFile->Load(pwLinkName,STGM_READ);
#if ( (!defined(UNICODE)) && (!defined(_UNICODE)))
    free(pwLinkName);
#endif
    if ( hResult!=S_OK )
    {
#ifdef _DEBUG // avoid noisy messages as current function is going to be called on each lnk present in menu folder
        this->ShowComError(hResult);
#endif
        goto CleanUp;
    }

    // don't work for new links...
    //hResult = pShellLink->GetPath(  TargetPath,
    //                                MAX_PATH,
    //                                &ShellLinkWin32FindData,
    //                                SLGP_UNCPRIORITY 
    //                              );
    // going to the new m$ way
    DWORD Flags;
    Flags = SLR_ANY_MATCH
            | SLR_INVOKE_MSI 
            | SLR_NO_UI 
            | SLR_UPDATE;
    hResult = pShellLink->Resolve(NULL,Flags);
// try to continue even in case of Resolve failure
//    if ( hResult!=S_OK )
//    {
//#ifdef _DEBUG // avoid noisy messages as current function is going to be called on each lnk present in menu folder
//        this->ShowComError(hResult);
//#endif
//        goto CleanUp;
//    }
    hResult = pShellLink->GetIDList(&pidl);
    if ( ( hResult!=S_OK ) || (pidl==NULL) )
    {
#ifdef _DEBUG // avoid noisy messages as current function is going to be called on each lnk present in menu folder
        this->ShowComError(hResult);
#endif
        goto CleanUp;
    }

    // get shortcut target path
    if (!::SHGetPathFromIDList(pidl,TargetPath))//  The address of a buffer to receive the file system path. This buffer must be at least MAX_PATH characters in size.
    {
        // ::SHGetPathFromIDList fails for shortcuts on library,
        // but in this case pShellLink->GetPath works (m$ will always sucks)
        hResult = pShellLink->GetPath(  TargetPath,
                                        MAX_PATH,
                                        &ShellLinkWin32FindData,
                                        SLGP_UNCPRIORITY 
                                      );
        // paths to specific folders (control panel, computer, ...) can't be resolved with the 2 previous call
        if ( hResult != S_OK )
            goto CleanUp;
    }

    bRet = TRUE;
CleanUp:
    if (pidl)
        ::CoTaskMemFree(pidl);

    if (pShellLink)
        pShellLink->Release();

    if (pPersistFile)
        pPersistFile->Release();

    return bRet;
}

BOOL CFolderMenu::IsLibraryEmpty(IN TCHAR* LibraryPath)
{
    BOOL bRet = TRUE; // by default folder is empty
    HRESULT hResult;
    PIDLIST_ABSOLUTE pidl = NULL;
    IShellItem *pShellItem = NULL;
    IShellFolder *pShellFolder = NULL;
    IEnumIDList *pEnumIDList = NULL;

    WCHAR* pwscPath;
    IShellFolder* pDesktopFolder=NULL;
    IMalloc* pMalloc=NULL;
    ITEMIDLIST* pFullpidl=NULL;

    // convert ansi to unicode if needed
#if ( defined(UNICODE) || defined(_UNICODE))
    pwscPath=LibraryPath;
#else
    CAnsiUnicodeConvert::AnsiToUnicode(LibraryPath,&pwscPath);
#endif

    // get pointer to malloc interface
    hResult = ::SHGetMalloc(&pMalloc);
    if ((hResult!=S_OK)||(!pMalloc))
        goto CleanUp;

    // get root folder
    hResult = ::SHGetDesktopFolder(&pDesktopFolder);
    if ((hResult!=S_OK)||(!pDesktopFolder))
        goto CleanUp;

    // get object full pidl from path name
    hResult = pDesktopFolder->ParseDisplayName(NULL,
                                                NULL,
                                                pwscPath,
                                                NULL,
                                                &pFullpidl,
                                                NULL);
    if ( (hResult!=S_OK) || (!pFullpidl) )
        goto CleanUp;
   
    // create shell item from pidl
    hResult = ::SHCreateShellItem(NULL, NULL, pFullpidl, &pShellItem);
    if ( ( hResult!=S_OK ) || (pShellItem==NULL) )
    {
#ifdef _DEBUG // avoid noisy messages as current function is going to be called on each lnk present in menu folder
        this->ShowComError(hResult);
#endif
        goto CleanUp;
    }

    // from https://msdn.microsoft.com/en-us/magazine/dd861346.aspx
    hResult = pShellItem->BindToHandler(NULL, BHID_SFObject , IID_PPV_ARGS(&pShellFolder));
    if ( ( hResult!=S_OK ) || (pShellFolder==NULL) )
    {
#ifdef _DEBUG // avoid noisy messages as current function is going to be called on each lnk present in menu folder
        this->ShowComError(hResult);
#endif
        goto CleanUp;
    }

    hResult = pShellFolder->EnumObjects(NULL, SHCONTF_FOLDERS | SHCONTF_NONFOLDERS | SHCONTF_INCLUDEHIDDEN, &pEnumIDList);
    if ( ( hResult!=S_OK ) || (pEnumIDList==NULL) )
    {
#ifdef _DEBUG // avoid noisy messages as current function is going to be called on each lnk present in menu folder
        this->ShowComError(hResult);
#endif
        goto CleanUp;
    }
    //use penumIDList to enumerate the content of the folder

    // Get the pidl for the first item in the folder.
    LPITEMIDLIST pSubFolderIdl;
    for(;;)
    {
        pSubFolderIdl = NULL;
        hResult = pEnumIDList->Next(1, &pSubFolderIdl, 0);
        ::CoTaskMemFree(pSubFolderIdl);
        if (hResult != S_OK)
            break;
        
        // we have found an item inside folder --> folder is not empty
        bRet = FALSE;
        break;
    }

CleanUp:
    if (pEnumIDList)
        pEnumIDList->Release();

    if (pShellFolder)
        pShellFolder->Release();

    if (pShellItem)
        pShellItem->Release();

    if (pidl)
        ::CoTaskMemFree(pidl);

#if ( (!defined(UNICODE)) && (!defined(_UNICODE)) )
    if (pwscPath)
        free(pwscPath);
#endif

    if (pFullpidl)
        pMalloc->Free(pFullpidl);

    return bRet;
}

BOOL CFolderMenu::AddLibraryToMenu(IN CFolderPopUpMenuInfos* pMenuInfos,IN TCHAR* LibraryPath)
{
    BOOL bRet = FALSE;
    HRESULT hResult;
    PIDLIST_ABSOLUTE pidl = NULL;
    IShellItem *pShellItem = NULL;
    IShellFolder *pShellFolder = NULL;
    IEnumIDList *pEnumIDList = NULL;

    WCHAR* pwscPath;
    IShellFolder* pDesktopFolder=NULL;
    IMalloc* pMalloc=NULL;
    ITEMIDLIST* pFullpidl=NULL;

    // convert ansi to unicode if needed
#if ( defined(UNICODE) || defined(_UNICODE))
    pwscPath=LibraryPath;
#else
    CAnsiUnicodeConvert::AnsiToUnicode(LibraryPath,&pwscPath);
#endif

    // get pointer to malloc interface
    hResult = ::SHGetMalloc(&pMalloc);
    if ((hResult!=S_OK)||(!pMalloc))
        goto CleanUp;

    // get root folder
    hResult = ::SHGetDesktopFolder(&pDesktopFolder);
    if ((hResult!=S_OK)||(!pDesktopFolder))
        goto CleanUp;

    // get object full pidl from path name
    hResult = pDesktopFolder->ParseDisplayName(NULL,
                                                NULL,
                                                pwscPath,
                                                NULL,
                                                &pFullpidl,
                                                NULL);
    if ( (hResult!=S_OK) || (!pFullpidl) )
        goto CleanUp;

    // create shell item from pidl
    hResult = ::SHCreateShellItem(NULL, NULL, pFullpidl, &pShellItem);
    if ( ( hResult!=S_OK ) || (pShellItem==NULL) )
    {
#ifdef _DEBUG // avoid noisy messages as current function is going to be called on each lnk present in menu folder
        this->ShowComError(hResult);
#endif
        goto CleanUp;
    }

    // from https://msdn.microsoft.com/en-us/magazine/dd861346.aspx
    hResult = pShellItem->BindToHandler(NULL, BHID_SFObject , IID_PPV_ARGS(&pShellFolder));
    if ( ( hResult!=S_OK ) || (pShellFolder==NULL) )
    {
#ifdef _DEBUG // avoid noisy messages as current function is going to be called on each lnk present in menu folder
        this->ShowComError(hResult);
#endif
        goto CleanUp;
    }

    hResult = pShellFolder->EnumObjects(NULL, SHCONTF_FOLDERS | SHCONTF_NONFOLDERS | SHCONTF_INCLUDEHIDDEN, &pEnumIDList);
    if ( ( hResult!=S_OK ) || (pEnumIDList==NULL) )
    {
#ifdef _DEBUG // avoid noisy messages as current function is going to be called on each lnk present in menu folder
        this->ShowComError(hResult);
#endif
        goto CleanUp;
    }
    //use penumIDList to enumerate the content of the folder

    // Get the pidl for the first item in the folder.
    LPITEMIDLIST pSubItemIdl;
    STRRET StrRet;
    StrRet.uType = STRRET_WSTR;
    TCHAR SubItemPath[MAX_PATH];
    BOOL bFolder;
    BOOL bLinkToCurrentLibrary;
    for(;;)
    {
        pSubItemIdl = NULL;
        hResult = pEnumIDList->Next(1, &pSubItemIdl, 0);
        // If result is something other than
        // S_OK then some error occurred.
        if (hResult != S_OK)
        {
            if (pSubItemIdl)
                ::CoTaskMemFree(pSubItemIdl);
            break;
        }
        
        hResult = pShellFolder->GetDisplayNameOf(pSubItemIdl,SHGDN_FORPARSING,&StrRet);
        // WIN32_FIND_DATA SubFolderData;
        // hResult = ::SHGetDataFromIDList(pShellFolder,pSubItemIdl,SHGDFIL_FINDDATA,&SubFolderData,sizeof(WIN32_FIND_DATA));
        if (hResult != S_OK)
        {
#ifdef _DEBUG // avoid noisy messages as current function is going to be called on each lnk present in menu folder
            this->ShowComError(hResult);
#endif
            ::CoTaskMemFree(pSubItemIdl);
            continue;
        }

        // convert StrRet.pOleStr to tchar and release it
        CAnsiUnicodeConvert::UnicodeToTchar(StrRet.pOleStr,SubItemPath,MAX_PATH);
        ::CoTaskMemFree(StrRet.pOleStr);

        bLinkToCurrentLibrary = FALSE;
        if (this->bExpandSubFoldersLinks)
        {
            // if file is link
            if ( CStdFileOperations::DoesExtensionMatch(SubItemPath,_T("lnk")) )
            {
                TCHAR LinkTarget[MAX_PATH];
                if ( this->GetLinkTarget(SubItemPath,LinkTarget) )
                {
                    if (CStdFileOperations::DoesExtensionMatch(LinkTarget,_T("library-ms")))
                    {
                        if (_tcsicmp(LibraryPath,LinkTarget)==0)
                            bLinkToCurrentLibrary = TRUE;
                    }
                }
            }
        }
        if (!bLinkToCurrentLibrary) // avoid infinite loop
        {
            bFolder = CStdFileOperations::IsDirectory(SubItemPath);
            this->AddFileToMenu(pMenuInfos,SubItemPath,bFolder,FALSE);
        }

        ::CoTaskMemFree(pSubItemIdl);
    }


    bRet = TRUE;
CleanUp:
    if (pEnumIDList)
        pEnumIDList->Release();

    if (pShellFolder)
        pShellFolder->Release();

    if (pShellItem)
        pShellItem->Release();

    if (pidl)
        ::CoTaskMemFree(pidl);

#if ( (!defined(UNICODE)) && (!defined(_UNICODE)) )
    if (pwscPath)
        free(pwscPath);
#endif

    if (pFullpidl)
        pMalloc->Free(pFullpidl);

    return bRet;
}



//-----------------------------------------------------------------------------
// Name: AddFileToMenu
// Object: add file to menu associated to pMenuInfos
// Parameters :
//     in  : CFolderPopUpMenuInfos* pMenuInfos : menu informations
//           TCHAR* Path : path of file to be added
//           BOOL bFolder : TRUE if Path is a folder
//     out :
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CFolderMenu::AddFileToMenu(CFolderPopUpMenuInfos* pMenuInfos,TCHAR* Path,BOOL bFolder,BOOL FromMultipleSingleLevelPath)
{
    TCHAR* DisplayName = NULL;
    TCHAR* LocalDisplayName = NULL;
    BOOL bRet = FALSE;

    BOOL bLibrary = FALSE;
    BOOL bDirectory = CStdFileOperations::IsDirectory(Path);
    
    if ( bDirectory == FALSE)
        bLibrary = CStdFileOperations::DoesExtensionMatch(Path,_T("library-ms"));

    if (this->bShowSubFolders && this->bExpandSubFolders )
    {
        if (bLibrary)
        {
            // do not add empty folders
            if (this->IsLibraryEmpty(Path))
                goto CleanUp;

            CFolderPopUpMenuItemInfos* pLibraryMenuItemInfos=NULL;
            // store display name before affecting Path
            // as we are going to change Path, DisplayName can't be a pointer into original Path
            // DisplayName = CStdFileOperations::GetFileName(Path); bad
            LocalDisplayName = _tcsdup(CStdFileOperations::GetFileName(Path)); 
            DisplayName = LocalDisplayName;
            // remove ".library-ms" extension from display name
            CStdFileOperations::RemoveFileExt(DisplayName);

            bRet = this->AddFileToMenu(pMenuInfos, Path,DisplayName,TRUE,this->bExpandSubFolders,FromMultipleSingleLevelPath,FALSE,&pLibraryMenuItemInfos);
            free(LocalDisplayName);
            LocalDisplayName = NULL;

            if ( !bRet)
                return FALSE;  
                
            return this->AddLibraryToMenu(pLibraryMenuItemInfos->pChildFolderPopUpMenuInfos,Path);
        }
        // if file is link to folder
        else if (this->bExpandSubFoldersLinks)
        {
            // if file is link
            if ( ( bDirectory == FALSE) && CStdFileOperations::DoesExtensionMatch(Path,_T("lnk")) )
            {
                TCHAR LinkTarget[MAX_PATH];
                if ( this->GetLinkTarget(Path,LinkTarget) )
                {
                    if ( CStdFileOperations::IsDirectory(LinkTarget) )
                    {
                        // if link point to an accessible folder
                        if ( CStdFileOperations::DoesDirectoryExists(LinkTarget) )
                        {
                            // store display name before affecting Path
                            // as we are going to change Path, DisplayName can't be a pointer into original Path
                            // DisplayName = CStdFileOperations::GetFileName(Path); bad
                            LocalDisplayName = _tcsdup(CStdFileOperations::GetFileName(Path)); 
                            DisplayName = LocalDisplayName;
                            // remove ".lnk" extension from display name
                            CStdFileOperations::RemoveFileExt(DisplayName);

                            // change Path for AddFileToMenu call
                            _tcscpy(Path,LinkTarget);

                            bFolder = TRUE;
                        }
                    }
                    else if (CStdFileOperations::DoesExtensionMatch(LinkTarget,_T("library-ms")))
                    {
                        // do not add empty folders
                        if (this->IsLibraryEmpty(LinkTarget))
                            goto CleanUp;

                        CFolderPopUpMenuItemInfos* pLibraryMenuItemInfos=NULL;
                        // store display name before affecting Path
                        // as we are going to change Path, DisplayName can't be a pointer into original Path
                        // DisplayName = CStdFileOperations::GetFileName(Path); bad
                        LocalDisplayName = _tcsdup(CStdFileOperations::GetFileName(Path)); 
                        DisplayName = LocalDisplayName;
                        // remove ".library-ms" extension from display name
                        CStdFileOperations::RemoveFileExt(DisplayName);

                        bRet = this->AddFileToMenu(pMenuInfos, Path,DisplayName,TRUE,this->bExpandSubFolders,FromMultipleSingleLevelPath,FALSE,&pLibraryMenuItemInfos);
                        free(LocalDisplayName);
                        LocalDisplayName = NULL;

                        if (!bRet)
                            return FALSE; 

                        return this->AddLibraryToMenu(pLibraryMenuItemInfos->pChildFolderPopUpMenuInfos,LinkTarget);
                    }
                }
            }
        }
    }

    if (!this->bShowSubFolders)
    {
        if (bFolder || bLibrary)
            goto CleanUp;
    }


    // if item is a folder
    if (bFolder)
    {
        // get file name
        TCHAR* FileName;
        // find last '\'
        FileName=_tcsrchr(Path,'\\');
        // if found
        if (FileName)
        {
            FileName++;

            // assume folder is not current or upper directory
            if ( (_tcscmp(FileName,_T("."))==0)
                || (_tcscmp(FileName,_T(".."))==0))
                goto CleanUp;
        }

        // do not add empty folders
        if (this->IsFolderEmpty(Path))
            goto CleanUp;
    }
    
    if (DisplayName == NULL)
    {
#ifdef CFolderMenu_USE_SHELL_FILE_NAMES
        SHFILEINFO TmpShfi;
        if (::SHGetFileInfo(Path,0,&TmpShfi,sizeof(TmpShfi),SHGFI_DISPLAYNAME))
            DisplayName = TmpShfi.szDisplayName;
        else
#endif
        DisplayName = CStdFileOperations::GetFileName(Path);
    }

    bRet = this->AddFileToMenu(pMenuInfos, Path,DisplayName,bFolder,bFolder && this->bExpandSubFolders,FromMultipleSingleLevelPath,FALSE,NULL);

CleanUp:
    if (LocalDisplayName)
        free(LocalDisplayName);

    return bRet;
}

//-----------------------------------------------------------------------------
// Name: AddFileToMenu
// Object: add file to menu associated to pMenuInfos
// Parameters :
//          CFolderPopUpMenuInfos* pMenuInfos : menu informations
//          TCHAR* FullPath : file / path to be added to be added
//          TCHAR* DisplayName : name used in the menu
//          BOOL bFolder : TRUE if FullPath is a folder
//          BOOL FolderNeedingToBeExpanded : TRUE if FullPath is a folder and must be expanded
//          BOOL FromMultipleSingleLevelPath : TRUE if different folders content can be merged into the same display name
//          BOOL bUseDisplayNameInsteadOfPathForMerging : if TRUE, use DisplayName instead of FullPath for directory merging
//          OUT CFolderPopUpMenuItemInfos** ppFolderPopUpMenuItemInfos : added or existing matching menu item (can be NULL)
//     return : TRUE on success (menu item added, or existing menu item found and completed)
//-----------------------------------------------------------------------------
BOOL CFolderMenu::AddFileToMenu(CFolderPopUpMenuInfos* pMenuInfos,TCHAR* FullPath,TCHAR* DisplayName,
                                BOOL bFolder,BOOL FolderNeedingToBeExpanded,BOOL FromMultipleSingleLevelPath,
                                BOOL bUseDisplayNameInsteadOfPathForMerging,
                                OUT CFolderPopUpMenuItemInfos** ppFolderPopUpMenuItemInfos)
{
    CFolderPopUpMenuInfos* pSubMenuInfos=NULL;
    BOOL bRetValue=FALSE;
    TCHAR LinkTargetPath[MAX_PATH];
    TCHAR* FileIconPath;
    if (ppFolderPopUpMenuItemInfos)
        *ppFolderPopUpMenuItemInfos = NULL;

    // extract associated icon
    HICON hIcon=NULL;

    FileIconPath = FullPath;

    if (this->bHideShortcutsOverlay)
    {
        // for shortcuts, use targeted file icon instead of shortcut icon (avoid shortcut overlay on menu)
        if (CStdFileOperations::IsDirectory(FullPath) == FALSE)
        {
            if ( CStdFileOperations::DoesExtensionMatch(FullPath,_T("lnk") ) )
            {
                TCHAR IconPath[MAX_PATH];
                int IconIndex;
                if (this->GetLinkIcon(FullPath,IconPath,&IconIndex))
                    ::ExtractIconEx(IconPath,IconIndex,NULL,&hIcon,1);
                if (hIcon==NULL)
                {
                    if (this->GetLinkTarget(FullPath,LinkTargetPath))
                        FileIconPath = LinkTargetPath;
                }
            }
        }
    }

    if (hIcon == NULL)
    {
        SHFILEINFO shfi;
#ifdef CFolderMenu_USE_SHELL_FILE_NAMES
        SHFILEINFO TmpShfi;
#endif
        memset(&shfi,0,sizeof(shfi));
        if (::SHGetFileInfo(FileIconPath,
                            FILE_ATTRIBUTE_NORMAL,
                            &shfi,
                            sizeof(shfi),
                            SHGFI_ICON | SHGFI_SMALLICON
                            )
            )
        {
            hIcon = shfi.hIcon;
        }
    }
    
    BOOL bFileNameAlreadyUsed;

    // can't be used as popup menu contains only display name (aka name without extension)
    // bFileNameAlreadyUsed = pMenuInfos->pPopUpMenu->IsNameUsed(DisplayName);

    // find matching pFolderPopUpMenuItemInfos
    CFolderPopUpMenuItemInfos* pFolderPopUpMenuItemInfos;

    pFolderPopUpMenuItemInfos = NULL;
    if (FromMultipleSingleLevelPath)
    {
        std::vector<CFolderPopUpMenuItemInfos*>::iterator pMenuItemInfosIterator;
        CFolderPopUpMenuItemInfos* pMenuItemInfos;

        // search through menu item array
        for (pMenuItemInfosIterator=pMenuInfos->FolderPopUpMenuItemInfosList.begin();
            pMenuItemInfosIterator!=pMenuInfos->FolderPopUpMenuItemInfosList.end(); 
            pMenuItemInfosIterator++
            )
        {
            pMenuItemInfos = *pMenuItemInfosIterator;

            if (bUseDisplayNameInsteadOfPathForMerging)
            {
                if (_tcscmp(CStdFileOperations::GetFileName(pMenuItemInfos->DisplayName),DisplayName)==0)
                {
                    pFolderPopUpMenuItemInfos = pMenuItemInfos;
                    break;
                }
            }
            else
            {
                // as name are always the same only use the first item of the list
#ifdef CFolderMenu_USE_SHELL_FILE_NAMES
                if (!::SHGetFileInfo(pMenuItemInfos->GetPath(),0,&TmpShfi,sizeof(TmpShfi),SHGFI_DISPLAYNAME))
                {
                    _tcsncpy(TmpShfi.szDisplayName,pMenuItemInfos->GetPath(),MAX_PATH);
                    TmpShfi.szDisplayName[MAX_PATH-1]=0;
                }
                if (_tcscmp(CStdFileOperations::GetFileName(TmpShfi.szDisplayName),DisplayName)==0)
#else
                if (_tcscmp(CStdFileOperations::GetFileName(pMenuItemInfos->GetPath()),DisplayName)==0)
#endif
                {
                    pFolderPopUpMenuItemInfos = pMenuItemInfos;
                    break;
                }

            }
        }
    }

    bFileNameAlreadyUsed = (pFolderPopUpMenuItemInfos != NULL);

    // if folder and must be expanded
    if (FolderNeedingToBeExpanded)
    {
        if (bFileNameAlreadyUsed)
        {
            // add path to the found pFolderPopUpMenuItemInfos path list
            pFolderPopUpMenuItemInfos->AddPath(FullPath);

            // as item has already been added, destroy new retrieved handle (icon handle as already been loaded and stored)
            if (hIcon)
                ::DestroyIcon(hIcon);

            if (ppFolderPopUpMenuItemInfos)
                *ppFolderPopUpMenuItemInfos = pFolderPopUpMenuItemInfos;

            return TRUE;
        }
        else
        {
            // create submenu
            CPopUpMenu* pSubMenu = new CPopUpMenu(pMenuInfos->pPopUpMenu);

            // NOTICE DATA IS ADDED TO MENU IN DoPostProcessingAfterItemsRetrieval
            // thanks to pMenuInfos->FolderPopUpMenuItemInfosList content

            // add submenu to AllMenusAndSubMenusList
            pSubMenuInfos=new CFolderPopUpMenuInfos(pSubMenu);
            this->AllMenusAndSubMenusList.push_back(pSubMenuInfos);

            // add menu select and right click callback
            pSubMenu->SetEventsHandler((IPopUpMenuEvents*)(CPopUpMenuEventsBase*)this);
        }
    }
    else
    {
        // item is not an expanded directory (filename or not expanded directory)
        // if item already exists in menu, discard it
        // Notice : order has importance and only first item will be kept
        if (bFileNameAlreadyUsed)
        {
            // as item has already been added, destroy new retrieved handle (icon handle as already been loaded and stored)
            if (hIcon)
                ::DestroyIcon(hIcon);

            if (ppFolderPopUpMenuItemInfos)
                *ppFolderPopUpMenuItemInfos = pFolderPopUpMenuItemInfos;

            return TRUE;
        }

        if (!bFolder)
        {
            // do filters checking
            if (this->pFilesFiltersInclusion)
            {
                if (!this->pFilesFiltersInclusion->Check(DisplayName))
                    goto CleanUp;
            }

            if (this->pFilesFiltersExclusion)
            {
                if (!this->pFilesFiltersExclusion->Check(DisplayName))
                    goto CleanUp;
            }
        }

        // NOTICE DATA IS ADDED TO MENU IN DoPostProcessingAfterItemsRetrieval, 
        // thanks to pMenuInfos->FolderPopUpMenuItemInfosList content

    }
    // add item to parent menu FolderPopUpMenuItemInfosList
    pFolderPopUpMenuItemInfos = new CFolderPopUpMenuItemInfos(FullPath,DisplayName,hIcon,bFolder,pSubMenuInfos);
    pMenuInfos->FolderPopUpMenuItemInfosList.push_back(pFolderPopUpMenuItemInfos);
 
    if (ppFolderPopUpMenuItemInfos)
        *ppFolderPopUpMenuItemInfos = pFolderPopUpMenuItemInfos;

    bRetValue = TRUE;
CleanUp:
    if (!bRetValue)
    {
        if (hIcon)
            ::DestroyIcon(hIcon);
    }
    return bRetValue;
}

//-----------------------------------------------------------------------------
// Name: GetPopUpMenuItemInfos
// Object: get CFolderPopUpMenuItemInfos* object from hMenu and MenuIndex
// Parameters :
//     in  : HMENU hMenu : menu handle
//           UINT MenuIndex : menu index
//     out :
//     return : a CFolderPopUpMenuItemInfos* pointer object on success, NULL on error
//-----------------------------------------------------------------------------
CFolderMenu::CFolderPopUpMenuItemInfos* CFolderMenu::GetPopUpMenuItemInfos(HMENU hMenu,UINT MenuIndex)
{

    // search through menu array
    std::vector<CFolderPopUpMenuInfos*>::iterator Array_pMenusIterator;
    CFolderPopUpMenuInfos* pMenu;
    std::vector<CFolderPopUpMenuItemInfos*>::iterator Array_pFolderPopUpMenuItemInfosIterator;
    CFolderPopUpMenuItemInfos* pMenuItemInfos;
    // search through menu array
    for (Array_pMenusIterator = this->AllMenusAndSubMenusList.begin();
        Array_pMenusIterator!=this->AllMenusAndSubMenusList.end(); 
        Array_pMenusIterator++
        )
    {
        pMenu = *Array_pMenusIterator;

        // assume hmenu match
        if (hMenu == pMenu->pPopUpMenu->GetControlHandle())
        {

            // search through menu item array
            for (Array_pFolderPopUpMenuItemInfosIterator=pMenu->FolderPopUpMenuItemInfosList.begin();
                Array_pFolderPopUpMenuItemInfosIterator!=pMenu->FolderPopUpMenuItemInfosList.end(); 
                Array_pFolderPopUpMenuItemInfosIterator++
                )
            {
                pMenuItemInfos = *Array_pFolderPopUpMenuItemInfosIterator;

                // assume MenuIndex match
                if( (UINT) pMenu->pPopUpMenu->GetID(MenuIndex) == pMenuItemInfos->Id )
                {
                    return pMenuItemInfos;
                }
            }
        }
    }
    return NULL;
}

//-----------------------------------------------------------------------------
// Name: GetPopUpMenuItemInfos
// Object: get CFolderPopUpMenuItemInfos* object from ItemId
// Parameters :
//     in  : ItemId : menu item id
//     out :
//     return : a CFolderPopUpMenuItemInfos* pointer object on success, NULL on error
//-----------------------------------------------------------------------------
CFolderMenu::CFolderPopUpMenuItemInfos* CFolderMenu::GetPopUpMenuItemInfos(UINT ItemId)
{
    std::vector<CFolderPopUpMenuInfos*>::iterator Array_pMenusIterator;
    CFolderPopUpMenuInfos* pMenu;
    std::vector<CFolderPopUpMenuItemInfos*>::iterator Array_pFolderPopUpMenuItemInfosIterator;
    CFolderPopUpMenuItemInfos* pMenuItemInfos;
    // search through menu array
    for (Array_pMenusIterator = this->AllMenusAndSubMenusList.begin();
        Array_pMenusIterator!=this->AllMenusAndSubMenusList.end(); 
        Array_pMenusIterator++
        )
    {
        pMenu = *Array_pMenusIterator;
        // search through menu item array
        for (Array_pFolderPopUpMenuItemInfosIterator=pMenu->FolderPopUpMenuItemInfosList.begin();
            Array_pFolderPopUpMenuItemInfosIterator!=pMenu->FolderPopUpMenuItemInfosList.end(); 
            Array_pFolderPopUpMenuItemInfosIterator++
            )
        {
            pMenuItemInfos = *Array_pFolderPopUpMenuItemInfosIterator;
            if(ItemId == pMenuItemInfos->Id)
            {
                return pMenuItemInfos;
            }
        }
    }
    return NULL;
}

//-----------------------------------------------------------------------------
// Name: DoDefaultAction
// Object: Do a default action for clicked item
// Parameters :
//     in  : ItemId : menu item id
//     out :
//     return : TRUE if default action as been successfully done, FALSE on error
//-----------------------------------------------------------------------------
BOOL CFolderMenu::DoDefaultAction(UINT ItemId)
{
    // if no item selected
    if (ItemId==0)
        return FALSE;

    CFolderPopUpMenuItemInfos* pFolderPopUpMenuItemInfos;
    // get informations associated to item
    pFolderPopUpMenuItemInfos=this->GetPopUpMenuItemInfos(ItemId);
    if (!pFolderPopUpMenuItemInfos)
        return FALSE;

    return CShellExecuteAndBringToTopEx::ShellExecute(pFolderPopUpMenuItemInfos->GetPath(),NULL);
}

#include <comdef.h>
#ifdef _NATIVE_WCHAR_T_DEFINED
    #ifdef _DEBUG
        #pragma comment(lib, "comsuppwd.lib")
    #else
        #pragma comment(lib, "comsuppw.lib")
    #endif
#else
    #ifdef _DEBUG
        #pragma comment(lib, "comsuppd.lib")
    #else
        #pragma comment(lib, "comsupp.lib")
    #endif
#endif
void CFolderMenu::ShowComError(HRESULT hr)
{
    ::_com_error err(hr);
    LPCTSTR errMsg = err.ErrorMessage();
    ::MessageBox(NULL,errMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
}

//-----------------------------------------------------------------------------
// Name: MenuSelectCallback
// Object: callback called when a menu item is selected
// Parameters :
//     in  : WPARAM wParam, LPARAM lParam : WM_MENUSELECT parameters
//           PVOID UserParam : CFolderMenu* object pointer
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CFolderMenu::OnPopUpMenuMenuSelect(CPopUpMenu* pSender,WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(pSender);

    if ( (!this->bExpandSubFolders) || (!this->bShowSubFolders) )
        return;

    // WM_MENUSELECT
    WORD Index=LOWORD(wParam);
    WORD Flag=HIWORD(wParam);
    HMENU hMenu=(HMENU)lParam;

    // if selected item is a submenu
    if (Flag & MF_POPUP)
    {
        // get informations associated to item
        CFolderPopUpMenuItemInfos* pFolderPopUpMenuItemInfos;
        pFolderPopUpMenuItemInfos=this->GetPopUpMenuItemInfos(hMenu,Index);

        if (pFolderPopUpMenuItemInfos)
        {
            // if item has a child menu (should be the case)
            if (pFolderPopUpMenuItemInfos->pChildFolderPopUpMenuInfos)
            {
                // if child menu is not filled
                if (!pFolderPopUpMenuItemInfos->pChildFolderPopUpMenuInfos->Filled)
                {
                    // change cursor
                    HCURSOR hCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

                    // fill child menu
                    // this->FillMenuInfos(pFolderPopUpMenuItemInfos->pChildFolderPopUpMenuInfos,pFolderPopUpMenuItemInfos->Path);
                    // if multiple path, fill content for all paths
                    this->FillMenuInfos(pFolderPopUpMenuItemInfos->pChildFolderPopUpMenuInfos,&pFolderPopUpMenuItemInfos->PathList);
                    
                    // restore original cursor
                    ::SetCursor(hCursor);
                }
            }
        }
    }
}

//-----------------------------------------------------------------------------
// Name: MouseRightButtonUpCallback
// Object: callback called when a menu item right clicked
// Parameters :
//     in  : WPARAM wParam, LPARAM lParam : WM_MENURBUTTONUP parameters
//           PVOID UserParam : CFolderMenu* object pointer
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CFolderMenu::OnPopUpMenuMouseRightButtonUp(CPopUpMenu* pSender,WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(pSender);
    // WM_MENURBUTTONUP
    // WORD Index=LOWORD(wParam);
    UINT Index = (UINT)(wParam);
    HMENU hMenu=(HMENU)lParam;


    CFolderPopUpMenuItemInfos* pFolderPopUpMenuItemInfos;
    // get informations associated to selected item
    pFolderPopUpMenuItemInfos=this->GetPopUpMenuItemInfos(hMenu,Index);

    if (!pFolderPopUpMenuItemInfos)
        return;

    // create a shell menu
    CShellMenu ShellMenu(this->hWndDialog,pFolderPopUpMenuItemInfos->GetPath());
    if (ShellMenu.IsCreationSuccessful())
    {
        // get mouse position
        POINT Point;
        ::GetCursorPos(&Point);
#ifdef _DEBUG
        ::OutputDebugString(_T("ShellMenu.Show\r\n"));
#endif
        // display shell menu
        if (ShellMenu.Show(Point.x,Point.y))
        {
            // if user has done an action on shell menu,
            // close current menu
            ::EndMenu();
        }
    }
}

// NULL to remove inclusion filters
BOOL CFolderMenu::SetFiltersInclusion(TCHAR* pszFiltersInclusion)
{
    // delete previous filter if any
    if (this->pFilesFiltersInclusion)
    {
        delete this->pFilesFiltersInclusion;
        this->pFilesFiltersInclusion=NULL;
    }

    this->pFilesFiltersInclusion=new CFilesFilters(CFilesFilters::FILTER_TYPE_INCLUSION,pszFiltersInclusion);

    return TRUE;
}

// NULL to remove exclusion filters
BOOL CFolderMenu::SetFiltersExclusion(TCHAR* pszFiltersExclusion)
{
    // delete previous filter if any
    if (this->pFilesFiltersExclusion)
    {
        delete this->pFilesFiltersExclusion;
        this->pFilesFiltersExclusion=NULL;
    }

    this->pFilesFiltersExclusion=new CFilesFilters(CFilesFilters::FILTER_TYPE_EXCLUSION,pszFiltersExclusion);

    return TRUE;
}

BOOL CFolderMenu::HideFilesExtension(BOOL bHideFilesExtension)
{
    this->bHideFilesExtension = bHideFilesExtension;
    return TRUE;
}

BOOL CFolderMenu::HideShortcutsOverlay(BOOL bHideShortcutsOverlay)
{
    this->bHideShortcutsOverlay = bHideShortcutsOverlay;
    return TRUE;
}

BOOL CFolderMenu::SetFoldersPosition(tagFoldersPosition FoldersPosition)
{
    this->FoldersPosition = FoldersPosition;
    return TRUE;
}
BOOL CFolderMenu::SetMenusMerge(BOOL bMergeMenus)
{
    this->bMergedMenu = bMergeMenus;
    return TRUE;
}
