/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manage a folder menu (menu containing folder items)
//-----------------------------------------------------------------------------

/*
example :

CFolderMenu FolderMenu;
FolderMenu.Fill(_T("c:\\"),TRUE,TRUE);
FolderMenu.Show(100,100,hWndDialog);

*/

#pragma once

#include "../Menu/PopUpMenu.h"
#include "../ShellMenu/ShellMenu.h"
#include "../../File/FilesFilters.h"
#include "../../File/StdFileOperations.h"
#include "../../String/AnsiUnicodeConvert.h"

#include <shellapi.h>
#pragma comment(lib,"shell32.lib")
#include <vector>

class CFolderMenu:public CPopUpMenuEventsBase
{
public:
    CFolderMenu(void);
    ~CFolderMenu(void);
    typedef enum tagFoldersPosition{FoldersPosition_ON_TOP = 0,FoldersPosition_ON_BOTTOM,FoldersPosition_NO_SPECIFIC_POSITION};

    #define CFolderMenu_MergedSpecificFolderName_SIZE 250  // tries to round up struct for unicode BOOL + (MAX_PATH+250)*2=1024
    typedef struct tagMultiplePathsData 
    {
        BOOL MergeToSpecificFolder;
        TCHAR MergedSpecificFolderName[CFolderMenu_MergedSpecificFolderName_SIZE];
        TCHAR Path[MAX_PATH];
    }MULTIPLE_PATHS_DATA,*PMULTIPLE_PATHS_DATA;

    BOOL Fill(MULTIPLE_PATHS_DATA* PathData,BOOL bShowSubFolders,BOOL bExpandSubFolders,BOOL bExpandSubFoldersLinks);
    BOOL Fill(MULTIPLE_PATHS_DATA* PathsData,DWORD NbPaths,BOOL bShowSubFolders,BOOL bExpandSubFolders,BOOL bExpandSubFoldersLinks);

    BOOL Show(int x,int y, HWND hOwner);
    BOOL Show(int x,int y, HWND hOwner,BOOL PositionRelativeToOwner);
    BOOL Show(int x,int y, HWND hOwner,BOOL PositionRelativeToOwner,BOOL ShowUpper);

    BOOL SetFiltersInclusion(TCHAR* pszFiltersInclusion);
    BOOL SetFiltersExclusion(TCHAR* pszFiltersExclusion);
    BOOL HideFilesExtension(BOOL bHideFilesExtension);
    BOOL HideShortcutsOverlay(BOOL bHideShortcutsOverlay);
    BOOL SetFoldersPosition(tagFoldersPosition FoldersPosition);
    BOOL SetMenusMerge(BOOL bMergeMenus);

private:

    class CFolderPopUpMenuInfos;

    class CFolderPopUpMenuItemInfos
    {
    public:
        typedef struct tagPath
        {
            TCHAR Path[MAX_PATH];
        };
        std::vector<tagPath> PathList;
        TCHAR DisplayName[MAX_PATH];
        UINT Id;
        BOOL bFolder;
        HICON hIcon;
        CFolderPopUpMenuInfos* pChildFolderPopUpMenuInfos;// handle to child submenu (if any)

        void AddPath(TCHAR* Path)
        {
            tagPath TmpFixedPath;
            *TmpFixedPath.Path=0;

            if (Path)
            {
                _tcsncpy(TmpFixedPath.Path,Path,MAX_PATH);
                TmpFixedPath.Path[MAX_PATH-1]=0;
            }

            PathList.push_back(TmpFixedPath);
        }
        CFolderPopUpMenuItemInfos(TCHAR* Path,TCHAR* DisplayName,HICON hIcon,BOOL bFolder,CFolderPopUpMenuInfos* pChildFolderPopUpMenuInfos)
        {
            this->AddPath(Path);
            _tcsncpy(this->DisplayName,DisplayName,MAX_PATH);
            this->DisplayName[MAX_PATH-1]=0;
            this->bFolder = bFolder;
            this->Id=0;
            this->hIcon=hIcon;
            this->pChildFolderPopUpMenuInfos=pChildFolderPopUpMenuInfos;
        }
        ~CFolderPopUpMenuItemInfos()
        {
            if (this->hIcon)
                ::DestroyIcon(this->hIcon);
        };
        BOOL IsFolder(){return this->bFolder;}
        BOOL IsDisplayNameBeginingWithNumber()
        {
            TCHAR c = *this->DisplayName;
            return ( (c>='0') && (c<='9') );
        }
        TCHAR* GetPath()
        {
            if (this->PathList.size()==0)
                return NULL;
            return this->PathList.begin()->Path;
        }
    };

    class CFolderPopUpMenuInfos
    {
    public:
        CFolderPopUpMenuInfos(CPopUpMenu* pPopUpMenu)
        {
            this->pPopUpMenu=pPopUpMenu;
            this->Filled=FALSE;
        }
        ~CFolderPopUpMenuInfos(){};

        std::vector<CFolderPopUpMenuItemInfos*> FolderPopUpMenuItemInfosList; // list all 1 level items of popup menu
        CPopUpMenu* pPopUpMenu;
        BOOL Filled;
    };
    static bool CompareMenuItemsFoldersNoSpecificPosition( CFolderPopUpMenuItemInfos* a, CFolderPopUpMenuItemInfos* b);
    static bool CompareMenuItemsFoldersOnBottom( CFolderPopUpMenuItemInfos* a, CFolderPopUpMenuItemInfos* b);
    static bool CompareMenuItemsFoldersOnTop( CFolderPopUpMenuItemInfos* a, CFolderPopUpMenuItemInfos* b);
    CFilesFilters* pFilesFiltersInclusion;
    CFilesFilters* pFilesFiltersExclusion;

    std::vector<CFolderPopUpMenuInfos*> AllMenusAndSubMenusList;// list of all menus and sub menus
    CPopUpMenu* pPopUpMenu;
    BOOL bShowSubFolders;
    BOOL bExpandSubFolders;
    BOOL bExpandSubFoldersLinks;
    BOOL bHideFilesExtension;
    BOOL bHideShortcutsOverlay;
    BOOL bMergedMenu;
    tagFoldersPosition FoldersPosition;
    HWND hWndDialog;
    BOOL FillMenuInfos(CFolderPopUpMenuInfos* pMenuInfos,TCHAR* Path,BOOL FromMultipleSingleLevelPath);
    BOOL FillMenuInfos(CFolderPopUpMenuInfos* pMenuInfos,std::vector<CFolderPopUpMenuItemInfos::tagPath>* pPathList);
    BOOL DoPostProcessingAfterItemsRetrieval(CFolderPopUpMenuInfos* pMenuInfos,BOOL bDoMenuSorting);
    BOOL IsFolderEmpty(TCHAR* Path);
    BOOL AddFileToMenu(CFolderPopUpMenuInfos* pMenuInfos,TCHAR* Path,BOOL bFolder,BOOL FromMultipleSingleLevelPath);
    BOOL AddFileToMenu(CFolderPopUpMenuInfos* pMenuInfos,TCHAR* FullPath,TCHAR* DisplayName,
                        BOOL bFolder,BOOL FolderNeedingToBeExpanded,BOOL FromMultipleSingleLevelPath,
                        BOOL bUseDisplayNameInsteadOfPathForMerging,
                        OUT CFolderPopUpMenuItemInfos** ppFolderPopUpMenuItemInfos);
    BOOL AddLibraryToMenu(IN CFolderPopUpMenuInfos* pMenuInfos,IN TCHAR* LibraryPath);
    BOOL DoDefaultAction(UINT ItemId);
    BOOL LaunchLink(TCHAR* Path);
    BOOL GetLinkTarget(IN TCHAR* LnkPath, IN OUT TCHAR* TargetPath);
    BOOL GetLinkIcon(IN TCHAR* LnkPath, IN OUT TCHAR* IconPath,IN OUT int* pIconIndex);
    BOOL IsLibraryEmpty(IN TCHAR* LibraryPath);
    
    CFolderPopUpMenuItemInfos* GetPopUpMenuItemInfos(HMENU hMenu,UINT MenuIndex);
    CFolderPopUpMenuItemInfos* GetPopUpMenuItemInfos(UINT ItemId);
    void FreeMenus();
    void ShowComError(HRESULT hr);

    virtual void OnPopUpMenuMouseRightButtonUp(CPopUpMenu* pSender,WPARAM wParam, LPARAM lParam);
    virtual void OnPopUpMenuMenuSelect(CPopUpMenu* pSender,WPARAM wParam, LPARAM lParam);
};
