/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper for hot window (mouse trap for auto show / auto hide window)
//-----------------------------------------------------------------------------

//Example of use
//        HWND hWnd = YouWindowhWnd;
//
//        class CHotWindowEvents:public CHotWindowEventsBase
//        {
//        public:
//            virtual void OnHotWindowAutoShow(IN CHotWindow* pSender,OUT BOOL* pUseAutoHideDisabledExtraRect,OUT RECT* pAutoHideDisabledExtraRect)
//            {
//                UNREFERENCED_PARAMETER(pSender);
//                ::ShowWindow(hWnd,SW_NORMAL);
//                *pUseAutoHideDisabledExtraRect = TRUE;
//                ::GetWindowRect(hWnd,pAutoHideDisabledExtraRect);
//            }
//            virtual void OnHotWindowAutoHide(CHotWindow* pSender)
//            {
//                UNREFERENCED_PARAMETER(pSender);
//                ::ShowWindow(hWnd,SW_HIDE);
//            }
//        };
//        RECT ActiveRect = {0,0,100,50 };
//        CHotWindowEvents EventsMgr;
//        CHotWindow* pHotWindow = new CHotWindow(&EventsMgr);
//        pHotWindow->SetWindowAndActiveRect(hWnd,&ActiveRect);
//        pHotWindow->StartAutoShow();



#pragma once

#include <windows.h>

class CHotWindow;

class IHotWindowEvents
{
public:
    // OUT BOOL* pUseAutoHideDisabledExtraRect : if TRUE, in addition to ActiveRect, if mouse is in ExtraRect, auto hide is disabled (allow to have window using more space than auto show rect)
    // OUT RECT* pAutoHideDisabledExtraRect : additionnal ActiveRect where auto hide is disabled (allow to have window using more space than auto show rect)
    virtual void OnHotWindowAutoShow(IN CHotWindow* pSender,OUT BOOL* pUseAutoHideDisabledExtraRect,OUT RECT* pAutoHideDisabledExtraRect)=0;
    virtual void OnHotWindowAutoHide(CHotWindow* pSender)=0;
};

class CHotWindowEventsBase: public IHotWindowEvents
{
public:
    virtual void OnHotWindowAutoShow(CHotWindow* pSender,OUT BOOL* pUseAutoHideDisabledExtraRect,OUT RECT* pAutoHideDisabledExtraRect)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(pAutoHideDisabledExtraRect);
        *pUseAutoHideDisabledExtraRect = FALSE;
    }
    virtual void OnHotWindowAutoHide(CHotWindow* pSender)
    {
        UNREFERENCED_PARAMETER(pSender);
    }
};

class CHotWindow
{
protected:
    HWND hWnd;
    HWND RegisteredhWnd;
    RECT ActiveRect;
    POINT OldCursorPos;
    IHotWindowEvents* pEventsHandler;
    BOOL bAutoShowStarted;
    BOOL bAutoHideStarted;
    UINT AutoShowTime;
    UINT AutoHideTime;
    BOOL bUseAutoHideDisabledExtraRect;
    RECT AutoHideDisabledExtraRect;

    static void CALLBACK TimerProcAutoShow(HWND hWnd,UINT uMsg,UINT_PTR idEvent,DWORD dwTime);
    static void CALLBACK TimerProcAutoHide(HWND hWnd,UINT uMsg,UINT_PTR idEvent,DWORD dwTime);
    static BOOL CALLBACK EnumWindowsProc(HWND hwnd,LPARAM lParam);
    typedef struct tagEnumWindowParam
    {
        POINT CursorPos;
        BOOL  bAWindowIsInFullScreenMode;
    };
    static BOOL DoesScreenContainingMouseHasAFullScreenWindow(POINT* pCursorPos);
public:
    CHotWindow(IHotWindowEvents* pEventsHandler);
    virtual ~CHotWindow();

    BOOL SetWindowAndActiveRect(HWND hWnd,RECT* pActiveRect);
    // Notice as auto show and auto hide are exclusive, they share the same timer
    BOOL StartAutoShow();
    BOOL StartAutoHide();
    BOOL StopAutoShow();
    BOOL StopAutoHide();

    void SetAutoShowTime(UINT Time);
    void SetAutoHideTime(UINT Time);
};
