/*
Copyright (C) 2004-2013 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2013 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manage a desktop like browser
//-----------------------------------------------------------------------------

#include "DesktopBrowser.h"
#include "../DesktopWorkArea/DesktopWorkArea.h"

#if _WIN32_WINNT < 0x0600
#define LVS_EX_JUSTIFYCOLUMNS   0x00200000  // Icons are lined up in columns that use up the whole view area.
#define LVS_EX_TRANSPARENTBKGND 0x00400000  // Background is painted by the parent via WM_PRINTCLIENT
#define LVS_EX_TRANSPARENTSHADOWTEXT 0x00800000  // Enable shadow text on transparent backgrounds only (useful with bitmaps)
#define LVS_EX_AUTOAUTOARRANGE  0x01000000  // Icons automatically arrange if no icon positions have been set
#define LVS_EX_HEADERINALLVIEWS 0x02000000  // Display column header in all view modes
#define LVS_EX_AUTOCHECKSELECT  0x08000000
#define LVS_EX_AUTOSIZECOLUMNS  0x10000000
#define LVS_EX_COLUMNSNAPPOINTS 0x40000000
#define LVS_EX_COLUMNOVERFLOW   0x80000000
#define FWF_NOGROUPING          0x00100000
#define FWF_NOFILTERS           0x00400000
#endif

#define CDesktopBrowser_DEFAULT_NB_SET_POS_RETRY 10000 // seen on win10 up to 700 repost

//-----------------------------------------------------------------------------
// Name: CDesktopBrowser
// Object: Constructor.
// Parameters :
//     in  : HWND hWndParent : parent window handle
//     out :
//     return : 
//-----------------------------------------------------------------------------
CDesktopBrowser::CDesktopBrowser(HWND hWndParent):CShellBrowserEx(hWndParent)
{
    this->CommonConstructor();
    this->FolderSettings.ViewMode=FVM_SMALLICON/*FVM_LIST*/;// better style to avoid scrolling bar
    this->FolderSettings.fFlags=FWF_NOWEBVIEW | FWF_ABBREVIATEDNAMES | FWF_NOGROUPING| FWF_NOFILTERS/*| FWF_HIDEFILENAMES/*| FWF_NOCOLUMNHEADER*/;
}

//-----------------------------------------------------------------------------
// Name: CDesktopBrowser
// Object: Constructor.
// Parameters :
//     in  : HWND hWndParent : parent window handle
//           FOLDERSETTINGS* pfs : folder settings
//     out :
//     return : 
//-----------------------------------------------------------------------------
CDesktopBrowser::CDesktopBrowser(HWND hWndParent,FOLDERSETTINGS* pfs):CShellBrowserEx(hWndParent,pfs)
{
    this->CommonConstructor();
}

//-----------------------------------------------------------------------------
// Name: CDesktopBrowser
// Object: Constructor.
// Parameters :
//     in  : HWND hWndParent : parent window handle
//           FOLDERSETTINGS* pfs : folder settings
//           RECT* pRect : Rect use for shell browser
//     out :
//     return : 
//-----------------------------------------------------------------------------
CDesktopBrowser::CDesktopBrowser(HWND hWndParent,FOLDERSETTINGS* pfs,RECT* pRect):CShellBrowserEx(hWndParent,pfs,pRect)
{
    this->CommonConstructor();
}

//-----------------------------------------------------------------------------
// Name: CommonConstructor
// Object: common Constructor.
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowser::CommonConstructor()
{
    ::InitializeCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
    ::InitializeCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    CShellBrowser::FolderSettingToShellBrowserView(&this->FolderSettings,&this->PreviousView);
    this->pMalloc=NULL;
    this->pidlDesktop=NULL;
    this->GetDesktopPidl();
    
    this->bSmallIconsView=TRUE;
    this->bCurrentViewIsDesktopView=TRUE;
    this->bShowDrives=FALSE;
    this->ToolTipInitialTime=MAKELONG(-1,0);
    this->bDesktopLikeView=TRUE;
    this->bEmptyText=FALSE;
    this->bSortItemNeeded=FALSE;
    this->bSettingViewInternally=FALSE;
    this->InsideLVN_GETDISPINFO = 0;
    this->InsideSetItemPosition = 0;

    this->hDesktopListView = NULL;

    this->LoadHiddenIcons();
}

//-----------------------------------------------------------------------------
// Name: ~CDesktopBrowser
// Object: Destructor.
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
CDesktopBrowser::~CDesktopBrowser(void)
{
    // free hidden pidl
    for (size_t Cnt=0;Cnt<this->HiddenItems.size();Cnt++)
    {
        this->pMalloc->Free(this->HiddenItems[Cnt]);
    }

    // free desktop pidl
    if (this->pidlDesktop)
        pMalloc->Free(this->pidlDesktop);

    if (this->pMalloc)
        pMalloc->Release();
    
    ::DeleteCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
    ::DeleteCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
}

//-----------------------------------------------------------------------------
// Name: LoadHiddenIcons
// Object: load hidden icon from registry
// Parameters :
//     in  : 
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowser::LoadHiddenIcons()
{
    ///////////////////////////////////
    // get hidden desktop items from registry
    ///////////////////////////////////
    this->HiddenItems.clear();

    // hidden desktop icons
    // "HKCU\Software\Microsoft\Windows\CurrentVersion\Explorer\HideDesktopIcons\ClassicStartMenu" classical start menu
    // "HKCU\Software\Microsoft\Windows\CurrentVersion\Explorer\HideDesktopIcons\NewStartPanel" XP start menu (warning it has nothing to see with Vista Aero style as mention on some web sites)
    // ; 0 = Show  1 = Hide
    //;-----  {20D04FE0-3AEA-1069-A2D8-08002B30309D} My Computer
    //;-----  {450D8FBA-AD25-11D0-98A8-0800361B1103} My Documents
    //;-----  {208D2C60-3AEA-1069-A2D7-08002B30309D} My Network Places
    //;-----  {871C5380-42A0-1069-A2EA-08002B30309D} Internet Explorer
    //;-----  {645FF040-5081-101B-9F08-00AA002F954E} Recycle Bin 
    // 
    //;-----  {00020D75-0000-0000-C000-000000000046} Outlook
    // ...

    // --> the only way is to enum all key, and next use IShellFolder::ParseDisplayName (,,L"::{GUID}",...)
    //     to get the hidden pidl
    TCHAR* SubKeyName;
    TCHAR KeyName[CDesktopBrowser_MAX_PATH];

    // get registry key according to user interface
    // if Classic Start Menu
    if (CStartMenu::IsClassicStartMenu())
        SubKeyName=CDesktopBrowser_CLASSIC_START_MENU_KEY;
    else // XP Start Menu
        SubKeyName=CDesktopBrowser_NEW_START_PANEL_KEY;

    // make full key name
    _tcscpy(KeyName,CDesktopBrowser_HIDE_DESKTOP_ICONS_KEY);
    _tcscat(KeyName,SubKeyName);

    // parse HKEY_LOCAL_MACHINE first, to next overwrite settings with HKEY_CURRENT_USER one's
    this->LoadHiddenIcons(HKEY_LOCAL_MACHINE,KeyName);
    this->LoadHiddenIcons(HKEY_CURRENT_USER,KeyName);

    this->RemoveControlPanelForSeven();
}

void CDesktopBrowser::RemoveControlPanelForSeven()
{
    IShellFolder* pDesktopFolder=NULL;
    HRESULT hResult;

    // get root folder
    hResult= ::SHGetDesktopFolder(&pDesktopFolder);
    if (SUCCEEDED(hResult)&&(pDesktopFolder))
    {
        ITEMIDLIST* pFullpidl=NULL;

        // get object full pidl from path name
        hResult = pDesktopFolder->ParseDisplayName(NULL,
                                                    NULL,
                                                    _T("::{26EE0668-A00A-44D7-9371-BEB064C98683}"),// MSDN : can be "::{GUID}"
                                                    NULL,
                                                    &pFullpidl,
                                                    NULL);
        if ( SUCCEEDED(hResult) && (pFullpidl) )
        {
            this->HiddenItems.push_back(pFullpidl);
        }

        hResult = pDesktopFolder->ParseDisplayName(NULL,
                                                    NULL,
                                                    _T("::{5399E694-6CE5-4D6C-8FCE-1D8870FDCBA0}"),// MSDN : can be "::{GUID}"
                                                    NULL,
                                                    &pFullpidl,
                                                    NULL);
        if ( SUCCEEDED(hResult) && (pFullpidl) )
        {
            this->HiddenItems.push_back(pFullpidl);
        }
    }
}
//-----------------------------------------------------------------------------
// Name: LoadHiddenIcons
// Object: load hidden icon from registry according to hRootKey and KeyName
// Parameters :
//     in  : HKEY hRootKey : HKLM or HKCU
//     out : TCHAR* KeyName : CDesktopBrowser_CLASSIC_START_MENU_KEY or CDesktopBrowser_NEW_START_PANEL_KEY
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowser::LoadHiddenIcons(HKEY hRootKey,TCHAR* KeyName)
{
    HKEY hKey;

    // open registry key
    if (::RegOpenKey(hRootKey,KeyName,&hKey)==ERROR_SUCCESS)
    {
        IShellFolder* pDesktopFolder=NULL;
        WCHAR wKey[CDesktopBrowser_MAX_PATH+2];
        DWORD KeyLen;
        DWORD KeyType;
        DWORD Data;
        DWORD DataSize;
        HRESULT hResult;

        // prepare buffer by prefixing "::"
        wcscpy(wKey,L"::");// MSDN IShellFolder::ParseDisplayName must be "::{GUID}"

        // get root folder
        hResult= ::SHGetDesktopFolder(&pDesktopFolder);
        if (SUCCEEDED(hResult)&&(pDesktopFolder))
        {
            LONG Result=ERROR_SUCCESS;
            ITEMIDLIST* pFullpidl=NULL;

            // for each registry key <-> each possible hidden guid
            for (DWORD Cnt=0;Result!=ERROR_NO_MORE_ITEMS;Cnt++)
            {
                DataSize=sizeof(DWORD);
                KeyLen=CDesktopBrowser_MAX_PATH;
                Result=::RegEnumValueW(hKey,Cnt,&wKey[2],&KeyLen,0,&KeyType,(LPBYTE)&Data,&DataSize);
                if (Result!=ERROR_SUCCESS)
                    continue;

                if (KeyType!=REG_DWORD) // can be default key
                    continue;

                // 0 = Show ; 1 = Hide
                if (hRootKey == HKEY_LOCAL_MACHINE)
                {
                    // only store hidden item
                    if (Data==0)
                        continue;
                }

                // get object full pidl from path name
                hResult = pDesktopFolder->ParseDisplayName(NULL,
                                                            NULL,
                                                            wKey,// MSDN : can be "::{GUID}"
                                                            NULL,
                                                            &pFullpidl,
                                                            NULL);
                if ( SUCCEEDED(hResult) && (pFullpidl) )
                {
                    if (hRootKey == HKEY_CURRENT_USER)
                    {
                        BOOL bFound = FALSE;
                        size_t FoundItemIndex = 0;
                        // check if pidl belongs to hidden items
                        for (size_t Cnt2=0;Cnt2<this->HiddenItems.size();Cnt2++)
                        {
                            if (::ILIsEqual(this->HiddenItems[Cnt2],pFullpidl))
                            {
                                bFound = TRUE;
                                FoundItemIndex = Cnt2;
                                break;
                            }
                        }
                        if (bFound)
                        {
                            if (Data==0)// item must be shown --> remove it from list
                            {
                                if (this->HiddenItems.size()!=0)
                                    this->HiddenItems.erase(this->HiddenItems.begin() + FoundItemIndex);
                            }
                            // else item is already in list and must be hidden --> nothing to do
                        }
                        else
                        {
                            if (Data==0)// item must be shown --> don't add it to list
                                continue;
                            // else : item must be hidden --> add it to list

                            // add pidl to hidden items
                            this->HiddenItems.push_back(pFullpidl);
                        }
                    }
                    else // (hRootKey == HKEY_LOCAL_MACHINE)
                    {
                        // add pidl to hidden items
                        this->HiddenItems.push_back(pFullpidl);
                    }
                }
            }
        }

        // close registry key
        ::RegCloseKey(hKey);
    }
}

//-----------------------------------------------------------------------------
// Name: SetViewInternal
// Object: change browser view without storing new view in PreviousView
//          NOTICE : SHELL_BROWSER_VIEW_SMALL_ICONS seems to have no effect
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CDesktopBrowser::SetView(tagShellBrowserView ShellBrowserView)
{
    HRESULT hResult;
    this->bSettingViewInternally=TRUE;
    hResult=this->CShellBrowserEx::SetView(ShellBrowserView);
    this->bSettingViewInternally=FALSE;

    return hResult;
}


//-----------------------------------------------------------------------------
// Name: BrowseDesktop
// Object: browse user desktop
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CDesktopBrowser::BrowseDesktop()
{
    return this->BrowseObject(CSIDL_DESKTOP);
}

//-----------------------------------------------------------------------------
// Name: GetDesktopPidl
// Object: get pidl associated to user desktop folder
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
BOOL CDesktopBrowser::GetDesktopPidl()
{
    BOOL bRet=FALSE;
    if (this->pidlDesktop)
        return TRUE;

    HRESULT hResult = E_FAIL;

    if (!this->pMalloc)
    {
        // get pointer to malloc interface
        hResult = ::SHGetMalloc(&this->pMalloc);
        if (FAILED(hResult)||(!this->pMalloc))
            goto CleanUp;
    }

    // get folder full idl list
    hResult = ::SHGetFolderLocation(this->hWndParent,CSIDL_DESKTOP,NULL,NULL,&this->pidlDesktop);
    if ( FAILED(hResult) || (!this->pidlDesktop) )
        goto CleanUp;

    bRet=TRUE;

CleanUp:
    return bRet;
}

//-----------------------------------------------------------------------------
// Name: IsDesktopFolder
// Object: test if pidl correspond to user desktop folder
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
BOOL CDesktopBrowser::IsDesktopFolder(LPCITEMIDLIST pidl)
{
    if ( (!pidl) || (!this->pidlDesktop) )
        return FALSE;

    return ::ILIsEqual(pidl,this->pidlDesktop);
}

//-----------------------------------------------------------------------------
// Name: DoesSmallViewBestFit
// Object: check if small icons is the best view 
// Parameters :
//     in  : 
//     out :
//     return : TRUE if small icons is the best view, FALSE if big icons are the best view
//-----------------------------------------------------------------------------
BOOL CDesktopBrowser::DoesSmallViewBestFit()
{
    RECT DesktopRect;
    CDesktopWorkArea::GetWorkArea(&DesktopRect,this->bUsePrimaryDisplayMonitorOnly);
    // if browser rect < 80% of desktop rect
    if 
    (
        (((this->BrowserRect.bottom-this->BrowserRect.top)*100)/(DesktopRect.bottom-DesktopRect.top)<80)
        && (((this->BrowserRect.right-this->BrowserRect.left)*100)/(DesktopRect.right-DesktopRect.left)<80)
    )
    {
        return TRUE;
    }

    return FALSE;
}

//-----------------------------------------------------------------------------
// Name: UpdateRect
// Object: update browser rect
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CDesktopBrowser::UpdateRect(RECT* pRect)
{
    // if no rect changes
    if (::EqualRect(&this->BrowserRect,pRect))
        return S_OK;

    BOOL bIsNewRectLarger;

    bIsNewRectLarger = (pRect->right-pRect->left>=this->BrowserRect.right-this->BrowserRect.left)
                        && (pRect->bottom-pRect->top>=this->BrowserRect.bottom-this->BrowserRect.top);

    // if new rect is larger, to avoid scrollbars, resize control before setting items position
    if (bIsNewRectLarger)
    {
        // update item rect
        this->CShellBrowserEx::InternalUpdateRect(pRect,TRUE);
    }
    else
    {
        // store new rect size for rect based function computation
        memcpy(&this->BrowserRect,pRect,sizeof(RECT));
    }

    // if not in desktop like view
    if (!this->bDesktopLikeView)
    {
        if (!bIsNewRectLarger)
        {
            this->CShellBrowserEx::InternalUpdateRect(pRect,TRUE);
        }
        return S_OK;
    }

    HRESULT hResult;
    BOOL bOldSmallIconsView=this->bSmallIconsView;
    this->bSmallIconsView=this->DoesSmallViewBestFit();

    // if best fit has changed
    if (bOldSmallIconsView!=this->bSmallIconsView)
    {
        if (this->bSmallIconsView)
        {
            this->SetView(SHELL_BROWSER_VIEW_SMALL_ICONS);

            // restore state of properties affected by InternalAfterViewChangedCallBack (RemoveSubProc)
            this->bDesktopLikeView=TRUE;
            this->bSmallIconsView=TRUE;

            // begin subclassing again
            this->InstallSubProc();
        }
        else
        {
            this->SetView(SHELL_BROWSER_VIEW_MEDIUM);

            // restore state of properties affected by InternalAfterViewChangedCallBack (RemoveSubProc)
            this->bDesktopLikeView=TRUE;

            // begin subclassing again
            this->InstallSubProc();
        }
    }

    // update item positions
    if (this->SetItemsPosLikeDesktop(FALSE,CDesktopBrowser_DEFAULT_NB_SET_POS_RETRY))// FALSE : earn time as UpdateRect is generally called on a WM_SIZE msg
        hResult = S_OK;
    else
        hResult = E_FAIL;

    // if new rect is smaller, to avoid scrollbars, resize control after setting items position
    if (!bIsNewRectLarger)
    {
        this->CShellBrowserEx::InternalUpdateRect(pRect,TRUE);
    }

    return hResult;
}

//-----------------------------------------------------------------------------
// Name: Refresh
// Object: refresh current view
// Parameters :
//     in  : 
//     out : 
//     return : S_OK on success
//-----------------------------------------------------------------------------
HRESULT CDesktopBrowser::Refresh()
{
    // if not desktop view return CShellBrowserEx refresh value
    if (!this->bDesktopLikeView)
        return this->CShellBrowserEx::Refresh();

    // update real desktop item position in case of changes
    this->GetDesktopItemsPos();
    HRESULT Result = this->RecreateCurrentView();// required on seven
// items are not present in the listview at this stage
//    if (this->bCurrentViewIsDesktopView)
//        this->SetItemsPosLikeDesktop(FALSE);
    return Result;
}

//-----------------------------------------------------------------------------
// Name: IncludeObject
// Object: ICommDlgBrowser::IncludeObject specific implementation
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
STDMETHODIMP CDesktopBrowser::IncludeObject(THIS_ struct IShellView *ppshv,
                                            LPCITEMIDLIST pidl)
{
    UNREFERENCED_PARAMETER(ppshv);
    return this->InternalIncludeObject(pidl);
}

//-----------------------------------------------------------------------------
// Name: InternalIncludeObject
// Object: ICommDlgBrowser::IncludeObject specific implementation
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CDesktopBrowser::InternalIncludeObject(LPCITEMIDLIST pidl)
{
    if (this->bCurrentViewIsDesktopView)
    {
        // check if pidl belongs to hidden items
        for (size_t Cnt=0;Cnt<this->HiddenItems.size();Cnt++)
        {
            if (::ILIsEqual(this->HiddenItems[Cnt],pidl))
            {
                // object must be hidden
                return S_FALSE;
            }
        }
    }
    return this->CShellBrowserEx::InternalIncludeObject(pidl);
}

//-----------------------------------------------------------------------------
// Name: InternalAfterListViewCreationCallBackVirtual
// Object: method called after listview creation
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowser::InternalAfterListViewCreationCallBackVirtual()
{
    this->InternalAfterListViewCreationCallBack();
}

//-----------------------------------------------------------------------------
// Name: InternalAfterListViewCreationCallBack
// Object: method called after listview creation
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowser::InternalAfterListViewCreationCallBack()
{
    this->bEmptyText=FALSE;
    this->bSortItemNeeded=FALSE;

    // if desktop like view
    if (this->bDesktopLikeView)
    {
        // disable theming that surround icon selection with large rect
        ::SetWindowTheme(this->hWndListView,NULL,L"");

        this->LoadHiddenIcons();

        this->bSmallIconsView=this->DoesSmallViewBestFit();

        if (this->bSmallIconsView)
        {
            this->SetView(SHELL_BROWSER_VIEW_SMALL_ICONS);

            // restore state of properties affected by InternalAfterViewChangedCallBack (RemoveSubProc)
            this->bDesktopLikeView=TRUE;
            this->bSmallIconsView=TRUE;

            // call parent object (CShellBrowserEx) InternalAfterListViewCreationCallBack method
            // after having changed listview representation and before installing subproc
            this->CShellBrowserEx::InternalAfterListViewCreationCallBack();

            // begin subclassing again
            this->InstallSubProc();
        }
        else
        {
            this->SetView(SHELL_BROWSER_VIEW_ICONS);

            // restore state of properties affected by InternalAfterViewChangedCallBack (RemoveSubProc)
            this->bDesktopLikeView=TRUE;

            // call parent object (CShellBrowserEx) InternalAfterListViewCreationCallBack method
            // after having changed listview representation and before installing subproc
            this->CShellBrowserEx::InternalAfterListViewCreationCallBack();

            // begin subclassing again
            this->InstallSubProc();
        }

        this->SetItemsPosLikeDesktop(FALSE,CDesktopBrowser_DEFAULT_NB_SET_POS_RETRY);
    }
    else
    {
        // restore previous view
        this->SetView(this->PreviousView);

        // call parent object (CShellBrowserEx) InternalAfterListViewCreationCallBack method
        // after having changed listview representation
        this->CShellBrowserEx::InternalAfterListViewCreationCallBack();
    }
}

//-----------------------------------------------------------------------------
// Name: InstallSubProc
// Object: install subclassing and do all the required job
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowser::InstallSubProc()
{
    UINT_PTR uIdSubclass=0;
    DWORD_PTR dwRefData=(DWORD_PTR)this;

    this->bEmptyText=FALSE;
    this->bSortItemNeeded=FALSE;

    // if current displayed folder is the desktop one
    if (this->bCurrentViewIsDesktopView)
    {
        if (this->bSmallIconsView)
        {
            // set icon spacing (allow tooltip to appear only when mouse is hover icon)
            ListView_SetIconSpacing(this->hWndListView,4,4);

            this->bEmptyText=TRUE;

            // change tooltip delay time (as text is removed, tooltip must appear as soon as mouse is hover an item)
            HWND hToopTip = ListView_GetToolTips(this->hWndListView);
            this->ToolTipInitialTime=(LPARAM)::SendMessage(hToopTip,TTM_GETDELAYTIME,TTDT_INITIAL,0);
            ::SendMessage(hToopTip,TTM_SETDELAYTIME,TTDT_INITIAL,MAKELONG(0,0));

            // change tooltip margins
            RECT MarginRect={0,0,0,0};
            ::SendMessage(hToopTip,TTM_SETMARGIN,0,(LPARAM)&MarginRect);

            DWORD ExStyle = ListView_GetExtendedListViewStyle(this->hWndListView);
            ExStyle &= ~(LVS_EX_FULLROWSELECT | LVS_EX_AUTOAUTOARRANGE | LVS_EX_HEADERINALLVIEWS); // disabling LVS_EX_FULLROWSELECT is required even in small icon view due to specific painting
            ListView_SetExtendedListViewStyle(this->hWndListView, ExStyle);

            // subclass shellview, listview and tooltip
            // NOTICE : do this at least (avoid to process/block our own messages)
            ::SetWindowSubclass(hToopTip,CDesktopBrowser::SubClassProcListViewToolTip,uIdSubclass,dwRefData);
        }
        else
        {
            DWORD ExStyle = ListView_GetExtendedListViewStyle(this->hWndListView);
            ExStyle &= ~(LVS_EX_FULLROWSELECT | LVS_EX_AUTOAUTOARRANGE | LVS_EX_HEADERINALLVIEWS); // disabling LVS_EX_FULLROWSELECT is required even in small icon view due to specific painting
            ListView_SetExtendedListViewStyle(this->hWndListView, ExStyle);
        }

	    // remove auto-arrange style which can be added some times
	    LONG_PTR Style=GetWindowLongPtr(this->hWndListView,GWL_STYLE);
        if (Style&LVS_AUTOARRANGE)
        {
            // SHELL_VIEW_AUTO_ARRANGE_SEVEN required for seven (else browser has auto arrange style)
            ::SendMessage(this->hWndShellView,WM_COMMAND,SHELL_VIEW_AUTO_ARRANGE_SEVEN,0);

	        Style&=~LVS_AUTOARRANGE;
	        SetWindowLongPtr(this->hWndListView,GWL_STYLE,Style);
        }

        ::SetWindowSubclass(this->hWndShellView,CDesktopBrowser::SubClassProcShellView,uIdSubclass,dwRefData);
        ::SetWindowSubclass(this->hWndListView,CDesktopBrowser::SubClassProcListView,uIdSubclass,dwRefData);
    }
}

//-----------------------------------------------------------------------------
// Name: RemoveSubProc
// Object: remove subclassing and restore all to default
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowser::RemoveSubProc()
{
    // if current view is the desktop one and view was the small icon one
    if (this->bCurrentViewIsDesktopView && this->bDesktopLikeView)
    {
        this->bDesktopLikeView=FALSE;
        this->bSmallIconsView=FALSE;

        // remove subclassing (do it first to avoid next messages to be trashed)
        UINT_PTR uIdSubclass=0;

        ::RemoveWindowSubclass(this->hWndListView,CDesktopBrowser::SubClassProcListView,uIdSubclass);

        ::RemoveWindowSubclass(this->hWndShellView,CDesktopBrowser::SubClassProcShellView,uIdSubclass);

        HWND hToopTip = ListView_GetToolTips(this->hWndListView);
        ::RemoveWindowSubclass(hToopTip,CDesktopBrowser::SubClassProcListViewToolTip,uIdSubclass);

        // restore tooltip time
        ::SendMessage(hToopTip,TTM_SETDELAYTIME,TTDT_INITIAL,this->ToolTipInitialTime);

        // reset icon spacing to default
        ListView_SetIconSpacing(this->hWndListView,-1,-1);

        this->RestoreItemsText();

        // clear vector
        ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
        this->LocalViewDesktopItemsInfos.clear();
        ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    }
}

//-----------------------------------------------------------------------------
// Name: RestoreItemsText
// Object: if text has been removed, restore original item text
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowser::RestoreItemsText()
{
    if (!this->bEmptyText)
        return;

    LOCAL_VIEW_DESKTOP_ITEM_INFOS Infos={0};
    // restore items text (must be done after removing subclasses)
    size_t NbItems=ListView_GetItemCount(this->hWndListView);
    for (size_t Cnt=0;Cnt<NbItems;Cnt++)
    {
        if (this->GetItemInfos(Cnt,&Infos))
            ListView_SetItemText(this->hWndListView,Cnt,0,Infos.Name);
    }

    this->bEmptyText=FALSE;
}

//-----------------------------------------------------------------------------
// Name: SetItemsPosLikeDesktop
// Object: set items like desktop items positions
// Parameters :
//     in  : BOOL bGetRealDesktopItemsPosition : TRUE to refresh real desktop items position
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CDesktopBrowser::SetItemsPosLikeDesktop(BOOL bGetRealDesktopItemsPosition,SIZE_T NbRetry)
{
    if (bGetRealDesktopItemsPosition 
        || (this->RealDesktopItemsInfos.size()==0)// assume GetDesktopItemsPos has been called once
        )
        // get desktop items positions
        this->GetDesktopItemsPos(); // fill this->RealDesktopItemsInfos

    POINT pt={0,0};
    int NbItems;

    ::EnterCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
    int InfosSize=(int)this->RealDesktopItemsInfos.size();
    ::LeaveCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
    if (InfosSize==0)
        return FALSE;
    BOOL bFound;
    TCHAR* pszName;
    REAL_DESKTOP_ITEM_INFOS* pRealDesktopItem;

    ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    NbItems=(int)this->LocalViewDesktopItemsInfos.size();
    ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    if (NbItems == 0)
    {
        int nItems = ListView_GetItemCount(this->hWndListView);
        if (nItems<=0)
        {
            if (NbRetry>0)
            {
                // can occur under win10 due to items caching and lack of LVN_GETDISPINFO sending
                // can occur if function is called too early (before icon addition in the list view)
                ::PostMessage(this->hWndShellView,CDesktopBrowser_RESTORE_DESKTOP_ICON_POSITION_NEEDED,NbRetry,0);
            }

            return FALSE;
        }

        ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
        NbItems = nItems;
        LOCAL_VIEW_DESKTOP_ITEM_INFOS LocalItemInfo;
        // we don't care of position, we are going to set it
        LocalItemInfo.Position.x = 0;
        LocalItemInfo.Position.y = 0;
        for (int Cnt = 0; Cnt<NbItems ; Cnt++)
        {
            *LocalItemInfo.Name = 0;
            ListView_GetItemText(this->hWndListView,Cnt,0,LocalItemInfo.Name,CDesktopBrowser_MAX_PATH);
            this->LocalViewDesktopItemsInfos.push_back(LocalItemInfo);
        }

        ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    }

    // get our target listview size
    RECT ListViewRect;
    // use this->BrowserRect instead of querying listview rect
    // It's interesting in case of listview size reducing, as UpdateRect position items
    // before sizing listview
    // ::GetClientRect(this->hWndListView,&ListViewRect);
    memcpy(&ListViewRect,&this->BrowserRect,sizeof(RECT));
    // to make sure there won't be scroll bars, reduce real available space by scroll bar size
    ListViewRect.right-=::GetSystemMetrics(SM_CXVSCROLL);
    ListViewRect.bottom-=::GetSystemMetrics(SM_CYHSCROLL);

    ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    NbItems=(int)this->LocalViewDesktopItemsInfos.size();

    RECT PrimaryDesktopWorkArea;
    RECT AllDisplayWorkArea;
    RECT DesktopRect;
    CDesktopWorkArea::GetWorkAreaEx(&PrimaryDesktopWorkArea,&AllDisplayWorkArea);
    // put primary rect into relative coordinate of top left display
    ::OffsetRect(&PrimaryDesktopWorkArea,-AllDisplayWorkArea.left,-AllDisplayWorkArea.top);
    ::OffsetRect(&AllDisplayWorkArea,-AllDisplayWorkArea.left,-AllDisplayWorkArea.top);
    if (this->bUsePrimaryDisplayMonitorOnly)
        DesktopRect = PrimaryDesktopWorkArea;
    else
        DesktopRect = AllDisplayWorkArea;

    LONG BorderSecurity;// security space to avoid scroll bars
    LONG IconSize;
    for (int Cnt=0;Cnt<NbItems;Cnt++)
    {
        // get current item name 
        pszName = this->LocalViewDesktopItemsInfos[Cnt].Name;

        // search name in RealDesktopItemsInfos
        ::EnterCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
        InfosSize=(int)this->RealDesktopItemsInfos.size();
        bFound=FALSE;
        for (int CntKnown=0;CntKnown<InfosSize;CntKnown++)
        {
            pRealDesktopItem = &this->RealDesktopItemsInfos[CntKnown];
            if (_tcscmp(pRealDesktopItem->Name,pszName)==0)
            {
                memcpy(&pt,&pRealDesktopItem->Position,sizeof(POINT));
                bFound=TRUE;
                break;
            }
        }
        ::LeaveCriticalSection(&this->RealDesktopItemsInfosCriticalSection);

        // if not found
        if (!bFound)
        {
#ifdef _DEBUG
            if (::IsDebuggerPresent())
                ::DebugBreak();
#endif
            continue;
        }

        // if item position retrieval was successful
        if (   (pt.x!=CDesktopBrowser_ITEM_POSITION_ERROR) 
            && (pt.y!=CDesktopBrowser_ITEM_POSITION_ERROR)
           )
        {
            if (this->bSmallIconsView) 
            {
                IconSize=16;
                BorderSecurity=5;
            }
            else // large icons
            {
                IconSize=32;
                BorderSecurity=10;
            }

            // we need to apply a ratio : target listview size / real desktop size
            // make ratio
            // warning DesktopRect.left and DesktopRect.top are not 0 on multiple screen configuration
            pt.x=((pt.x-DesktopRect.left)*(ListViewRect.right-ListViewRect.left))/(DesktopRect.right-DesktopRect.left);
            pt.y=((pt.y-DesktopRect.top)*(ListViewRect.bottom-ListViewRect.top))/(DesktopRect.bottom-DesktopRect.top);

            // assume item can be drawn in listview without scroll bars
            if (pt.x<BorderSecurity)
                pt.x=BorderSecurity;
            if (pt.x>ListViewRect.right-IconSize-BorderSecurity)
                pt.x=ListViewRect.right-IconSize-BorderSecurity;

            if (pt.y<BorderSecurity)
                pt.y=BorderSecurity;
            if (pt.y>ListViewRect.bottom-IconSize-BorderSecurity)
                pt.y=ListViewRect.bottom-IconSize-BorderSecurity;

            // set item position
            //ListView_SetItemPosition(this->hWndListView,
            //                        Cnt,
            //                        pt.x,
            //                        pt.y
            //                        );
            ::PostMessage(this->hWndListView,LVM_SETITEMPOSITION,Cnt,(LPARAM) MAKELPARAM ( (int) pt.x, (int) pt.y) );
        }
    }
    ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);

    // use post message as we do PostMessage with LVM_SETITEMPOSITION, so ensure all the LVM_SETITEMPOSITION messages have been taken into account
    // ::RedrawWindow( this->hWndListView,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME | RDW_ALLCHILDREN);
    ::PostMessage(this->hWndListView,WM_SETREDRAW,TRUE,0);
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SetItemsPosLikeDesktop
// Object: set items like desktop items positions
// Parameters :
//     in  : BOOL bGetRealDesktopItemsPosition : TRUE to refresh real desktop items position
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CDesktopBrowser::SetItemsPosLikeDesktop(SIZE_T ItemIndex,TCHAR* ItemOriginalText,BOOL bGetRealDesktopItemsPosition)
{
    if (bGetRealDesktopItemsPosition 
        || (this->RealDesktopItemsInfos.size()==0)// assume GetDesktopItemsPos has been called once
        )
        // get desktop items positions
        this->GetDesktopItemsPos(); // fill this->RealDesktopItemsInfos

    POINT pt={CDesktopBrowser_ITEM_POSITION_ERROR,CDesktopBrowser_ITEM_POSITION_ERROR};

    size_t CntKnown;
    BOOL bFound;
    size_t InfosSize;

    // search name in RealDesktopItemsInfos
    bFound=FALSE;
    #define RealDesktopItemsInfos_Search_MAX_TRY 4
    for(SIZE_T Cnt=0;Cnt<RealDesktopItemsInfos_Search_MAX_TRY;Cnt++)
    {
        ::EnterCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
        InfosSize=this->RealDesktopItemsInfos.size();
        if (InfosSize==0)
        {
            ::LeaveCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
            return FALSE;
        }

        for (CntKnown=0;CntKnown<InfosSize;CntKnown++)
        {
            if (_tcscmp(this->RealDesktopItemsInfos[CntKnown].Name,ItemOriginalText)==0)
            {
                memcpy(&pt,&this->RealDesktopItemsInfos[CntKnown].Position,sizeof(POINT));
                bFound=TRUE;
                break;
            }
        }
        ::LeaveCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
        if (bFound)
            break;

        //else
        this->GetDesktopItemsPos(); // update desktop infos
    }

    // get our target listview size
    RECT ListViewRect;
    // use this->BrowserRect instead of querying listview rect
    // It's interesting in case of listview size reducing, as UpdateRect position items
    // before sizing listview
    // ::GetClientRect(this->hWndListView,&ListViewRect);
    memcpy(&ListViewRect,&this->BrowserRect,sizeof(RECT));
    // to make sure there won't be scroll bars, reduce real available space by scroll bar size
    ListViewRect.right-=::GetSystemMetrics(SM_CXVSCROLL);
    ListViewRect.bottom-=::GetSystemMetrics(SM_CYHSCROLL);


    LONG BorderSecurity;// security space to avoid scroll bars
    LONG IconSize;
    if (this->bSmallIconsView) 
    {
        IconSize=16;
        BorderSecurity=5;
    }
    else // large icons
    {
        IconSize=32;
        BorderSecurity=10;
    }

    // assume to be up to date
    RECT PrimaryDesktopWorkArea;
    RECT AllDisplayWorkArea;
    RECT DesktopRect;
    CDesktopWorkArea::GetWorkAreaEx(&PrimaryDesktopWorkArea,&AllDisplayWorkArea);
    // put primary rect into relative coordinate of top left display
    ::OffsetRect(&PrimaryDesktopWorkArea,-AllDisplayWorkArea.left,-AllDisplayWorkArea.top);
    ::OffsetRect(&AllDisplayWorkArea,-AllDisplayWorkArea.left,-AllDisplayWorkArea.top);
    if (this->bUsePrimaryDisplayMonitorOnly)
        DesktopRect = PrimaryDesktopWorkArea;
    else
        DesktopRect = AllDisplayWorkArea;


    // if item position retrieval was successful
    if (   (pt.x!=CDesktopBrowser_ITEM_POSITION_ERROR) 
        && (pt.y!=CDesktopBrowser_ITEM_POSITION_ERROR)
        )
    {
        // we need to apply a ratio : target listview size / workarea desktop size
        pt.x=((pt.x-DesktopRect.left)*(ListViewRect.right-ListViewRect.left))/(DesktopRect.right-DesktopRect.left);
        pt.y=((pt.y-DesktopRect.top)*(ListViewRect.bottom-ListViewRect.top))/(DesktopRect.bottom-DesktopRect.top);

        // assume item can be drawn in listview without scroll bars
        if (pt.x<BorderSecurity)
            pt.x=BorderSecurity;
        if (pt.x>ListViewRect.right-IconSize-BorderSecurity)
            pt.x=ListViewRect.right-IconSize-BorderSecurity;

        if (pt.y<BorderSecurity)
            pt.y=BorderSecurity;
        if (pt.y>ListViewRect.bottom-IconSize-BorderSecurity)
            pt.y=ListViewRect.bottom-IconSize-BorderSecurity;
    
        POINT CurrentPoint;
        ListView_GetItemPosition(this->hWndListView,
                                ItemIndex,
                                &CurrentPoint
                                );

        if ( memcmp(&CurrentPoint , &pt, sizeof(POINT)) !=0 )
        {
            ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
            InfosSize=this->LocalViewDesktopItemsInfos.size();
            for (CntKnown=0;CntKnown<InfosSize;CntKnown++)
            {
                if (_tcscmp(this->LocalViewDesktopItemsInfos[CntKnown].Name,ItemOriginalText)==0)
                {
                    memcpy(&this->LocalViewDesktopItemsInfos[CntKnown].Position,&pt,sizeof(POINT));
                    break;
                }
            }
            ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);

            // use post message else icon position can be not taken into account
            // set item position
            //ListView_SetItemPosition(this->hWndListView,
            //                        ItemIndex,
            //                        pt.x,
            //                        pt.y
            //                        );
            ::PostMessage(this->hWndListView,LVM_SETITEMPOSITION,ItemIndex,(LPARAM) MAKELPARAM ( (int) pt.x, (int) pt.y) );
        }
    }
    return TRUE;
}

HWND CDesktopBrowser::GetDesktopListView()
{
    if (this->hDesktopListView)
        return this->hDesktopListView;

    HWND hListView = NULL;

    // find well known window containing desktop
    HWND hWnd = ::FindWindow(_T("Progman"), _T("Program Manager"));//::FindWindow(_T("ProgMan"), NULL);

    // find shell view
    HWND hShellView = ::FindWindowEx(hWnd,NULL,_T("SHELLDLL_DefView"),NULL);

    // bug solved QuickWayToFolder 1.2.8
    // in case of use of direct X wallpaper _T("Progman") doesn't contain _T("SHELLDLL_DefView")
    if (hShellView==0)
    {
        // use of DirectX --> ::FindWindow(_T("WorkerW"), NULL); // trouble : multiple WorkerW
        HWND hWndDesktop = ::GetDesktopWindow();
        HWND hWndChild = 0;
        for(;;)
        {
            hWndChild = ::FindWindowEx( hWndDesktop, hWndChild,  _T("WorkerW"),  NULL );
            if (!hWndChild)
                goto CleanUp;
            hShellView = ::FindWindowEx(hWndChild,NULL,_T("SHELLDLL_DefView"),NULL);

            if (hShellView)
                break;
        }
    }

    // find associated list view control
    hListView = ::FindWindowEx(hShellView,NULL,WC_LISTVIEW,NULL);
    if (!::IsWindowVisible(hListView))
    {
        hListView = ::FindWindowEx(hShellView,NULL,CShellBrowser_ThumbnailVwExtWnd32,NULL);
        hListView = ::FindWindowEx(hListView,NULL,WC_LISTVIEW,NULL);
    }

CleanUp:
    this->hDesktopListView = hListView;

    return this->hDesktopListView;
}

//-----------------------------------------------------------------------------
// Name: GetDesktopItemsPos
// Object: get real desktop items positions
//		/!\ Warning works only on correct identical OS architecture version LVITEM is platform dependent (32 and 64 bit structs are different)
// Parameters :
//     in  :
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CDesktopBrowser::GetDesktopItemsPos()
{
    BOOL bRet=FALSE;
    HANDLE hProcess=NULL;
    REAL_DESKTOP_ITEM_INFOS* lpData = NULL;
    this->bUsePrimaryDisplayMonitorOnly=TRUE;

    ::EnterCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
    this->RealDesktopItemsInfos.clear();

    HWND hListView = this->GetDesktopListView();

    DWORD dwProcessId;
    DWORD dwAccess;

    // get process associated to desktop
    ::GetWindowThreadProcessId(hListView, &dwProcessId);

    // open process with allocation, read and write memory access
    dwAccess = PROCESS_VM_OPERATION|PROCESS_VM_READ|PROCESS_VM_WRITE;
    hProcess = ::OpenProcess(dwAccess, FALSE, dwProcessId);
    if(!hProcess)
        goto CleanUp;

    // allocate required structs in remote process memory
    lpData =(REAL_DESKTOP_ITEM_INFOS*)::VirtualAllocEx(hProcess, NULL, sizeof(REAL_DESKTOP_ITEM_INFOS)+sizeof(LVITEM), MEM_COMMIT, PAGE_READWRITE);
    if(!lpData)
        goto CleanUp;

    // get pointer to 2nd struct
    LVITEM* lpItem=(LVITEM*) ( ((PBYTE)lpData) + sizeof(REAL_DESKTOP_ITEM_INFOS) );

    REAL_DESKTOP_ITEM_INFOS ItemInfos;

    // prepare an LVITEM struct
    LVITEM lvi;
    memset(&lvi, 0, sizeof(LVITEM));
    lvi.mask = LVIF_TEXT;
    lvi.pszText	= (LPTSTR) lpData->Name;
    lvi.cchTextMax = CDesktopBrowser_MAX_PATH;

    // get number of item on desktop
    int ItemCount =(int) ::SendMessage(hListView, LVM_GETITEMCOUNT, 0, 0);

    RECT PrimaryDesktopWorkArea;
    RECT AllDisplayWorkArea;
    CDesktopWorkArea::GetWorkAreaEx(&PrimaryDesktopWorkArea,&AllDisplayWorkArea);
    // put primary rect into relative coordinate of top left display
    ::OffsetRect(&PrimaryDesktopWorkArea,-AllDisplayWorkArea.left,-AllDisplayWorkArea.top);
    ::OffsetRect(&AllDisplayWorkArea,-AllDisplayWorkArea.left,-AllDisplayWorkArea.top);

    // for each item
    for(int Cnt = 0; Cnt < ItemCount; Cnt++)
    {
        // get item pos
        ItemInfos.Position.x=CDesktopBrowser_ITEM_POSITION_ERROR;
        ItemInfos.Position.y=CDesktopBrowser_ITEM_POSITION_ERROR;
        if (::SendMessage(hListView, LVM_GETITEMPOSITION, Cnt, (LPARAM) &lpData->Position))
        {
            // translate result from remote process memory to current process memory
            ::ReadProcessMemory(hProcess, &lpData->Position, &ItemInfos.Position, sizeof(POINT), NULL);

            if (!::PtInRect(&PrimaryDesktopWorkArea,ItemInfos.Position))
                this->bUsePrimaryDisplayMonitorOnly=FALSE;
        }

        // get item text
        lvi.iItem = Cnt;
        *ItemInfos.Name=0;
        // update remote LVITEM struct
        ::WriteProcessMemory(hProcess, lpItem, &lvi, sizeof(LVITEM), NULL);
        if (::SendMessage(hListView, LVM_GETITEMTEXT, Cnt, (LPARAM) lpItem) > 0)
        {
            // translate result from remote process memory to current process memory
            ::ReadProcessMemory(hProcess, &lpData->Name, &ItemInfos.Name, CDesktopBrowser_MAX_PATH*sizeof(TCHAR), NULL);
        }

        // store item text and position into RealDesktopItemsInfos vector
        this->RealDesktopItemsInfos.push_back(ItemInfos);
    }

    bRet=TRUE;

CleanUp:
    ::LeaveCriticalSection(&this->RealDesktopItemsInfosCriticalSection);

    // free memory 

    if (lpData)
        ::VirtualFreeEx(hProcess, lpData, 0, MEM_RELEASE);
    if (hProcess)
        ::CloseHandle(hProcess);

    return bRet;
}

//-----------------------------------------------------------------------------
// Name: InternalBeforeBrowseCallBackVirtual
// Object: method called before browsing a new folder
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowser::InternalBeforeBrowseCallBackVirtual(LPCITEMIDLIST pidlToBeBrowsed)
{
    this->InternalBeforeBrowseCallBack(pidlToBeBrowsed);
}

//-----------------------------------------------------------------------------
// Name: InternalBeforeBrowseCallBack
// Object: method called before browsing a new folder
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowser::InternalBeforeBrowseCallBack(LPCITEMIDLIST pidlToBeBrowsed)
{
    this->CShellBrowserEx::InternalBeforeBrowseCallBack(pidlToBeBrowsed);

    if (this->bDesktopLikeView && this->bCurrentViewIsDesktopView)
    {
        this->RestoreItemsText(); // must be called else sorting troubles occurs when browsing back to desktop
        // to be cleaner, we fully remove subclassing and restore items text
        this->RemoveSubProc();
        
    }

    // check if next folder will be the desktop one
    if (this->IsDesktopFolder(pidlToBeBrowsed))
    {
        this->bDesktopLikeView=TRUE;
        this->bCurrentViewIsDesktopView=TRUE;
        this->bSmallIconsView=this->DoesSmallViewBestFit();
        // as view can be changed do not use FWF_DESKTOP or FWF_NOSCROLL style
        // this->FolderSettings.fFlags|=FWF_NOWEBVIEW | FWF_DESKTOP | FWF_TRANSPARENT;
        this->bShowDrives = FALSE; // Desktop can contains drives but don't show them (seen on win 10)
    }
    else
    {
        this->bDesktopLikeView=FALSE;
        this->bCurrentViewIsDesktopView=FALSE;
        // as view can be changed do not use FWF_DESKTOP or FWF_NOSCROLL style
        // this->FolderSettings.fFlags&=~(FWF_NOWEBVIEW | FWF_DESKTOP | FWF_TRANSPARENT);
        this->bShowDrives = TRUE;// Desktop can contains drives but don't show them (seen on win 10)
    }
}

//-----------------------------------------------------------------------------
// Name: InternalAfterViewChangedCallBackVirtual
// Object: method called after folder view changed
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowser::InternalAfterViewChangedCallBackVirtual(tagShellBrowserView NewShellBrowserView)
{
    this->InternalAfterViewChangedCallBack(NewShellBrowserView);
}

//-----------------------------------------------------------------------------
// Name: InternalAfterViewChangedCallBack
// Object: method called after folder view changed
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowser::InternalAfterViewChangedCallBack(tagShellBrowserView NewShellBrowserView)
{
    // call parent object (CShellBrowserEx) method
    this->CShellBrowserEx::InternalAfterViewChangedCallBack(NewShellBrowserView);
}

void CDesktopBrowser::InternalBeforeViewChangedCallBackVirtual(tagShellBrowserView NewShellBrowserView)
{
    this->InternalBeforeViewChangedCallBack(NewShellBrowserView);
}
void CDesktopBrowser::InternalBeforeViewChangedCallBack(tagShellBrowserView NewShellBrowserView)
{
    UNREFERENCED_PARAMETER(NewShellBrowserView);
    if (!this->bSettingViewInternally)
    {
        this->PreviousView=NewShellBrowserView;

        if (this->bDesktopLikeView && this->bCurrentViewIsDesktopView)
        {
            this->RestoreItemsText(); // must be called else sorting troubles occurs when browsing back to desktop
            // to be cleaner, we fully remove subclassing and restore items text
            this->RemoveSubProc();
            this->bDesktopLikeView=FALSE;
        }

    }


    // call parent object (CShellBrowserEx) method
    this->CShellBrowserEx::InternalBeforeViewChangedCallBack(NewShellBrowserView);
}


//-----------------------------------------------------------------------------
// Name: SubClassProcListViewToolTip
// Object: listview tooltip subclassing proc : trash undesirable messages
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CDesktopBrowser::SubClassProcListViewToolTip(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData)
{
    UNREFERENCED_PARAMETER(uIdSubclass);
    UNREFERENCED_PARAMETER(dwRefData);
    // CDesktopBrowser* pDesktopBrowser=(CDesktopBrowser*)dwRefData;

    switch (uMsg)
    {
    case TTM_SETMARGIN:
        // shell view set tooltip margins to 4,4 --> avoid this for small icon view
        return TRUE;
    }
    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
}

void CDesktopBrowser::AddItemA(int ItemIndex,CHAR* Text)
{
    TCHAR* tText = (TCHAR*)LPSTR_TEXTCALLBACKA;

#if ( defined(UNICODE) || defined(_UNICODE))
    WCHAR wText[CDesktopBrowser_MAX_PATH];
#else
    
#endif

    if (Text!=LPSTR_TEXTCALLBACKA)
    {
        if (!::IsBadReadPtr (Text,sizeof(WCHAR)))
        {
#if ( defined(UNICODE) || defined(_UNICODE))
            CAnsiUnicodeConvert::AnsiToUnicode(Text,wText,CDesktopBrowser_MAX_PATH);
            tText = wText;
#else
            tText = Text;
#endif
        }
    }
    this->AddItemInfos((SIZE_T)ItemIndex,tText);
}

void CDesktopBrowser::AddItemW(int ItemIndex,WCHAR* Text)
{
    TCHAR* tText = (TCHAR*)LPSTR_TEXTCALLBACKW;

#if ( defined(UNICODE) || defined(_UNICODE))

#else
    CHAR cText[CDesktopBrowser_MAX_PATH];
#endif

    if (Text!=LPSTR_TEXTCALLBACKW)
    {
        if (!::IsBadReadPtr (Text,sizeof(WCHAR)))
        {
#if ( defined(UNICODE) || defined(_UNICODE))
            tText = Text;
#else
            CAnsiUnicodeConvert::UnicodeToAnsi(Text,cText,CDesktopBrowser_MAX_PATH);
            tText = cText;
#endif
        }
    }
    this->AddItemInfos((SIZE_T)ItemIndex,tText);

}

void CDesktopBrowser::AddItemInfos(SIZE_T Index,TCHAR* Text)
{
    LOCAL_VIEW_DESKTOP_ITEM_INFOS ItemInfos ={0};

    // if item has already been added
    if (this->GetItemInfos(Index,&ItemInfos))
        return;

    // item not found : add it
    if ( (Text!=(TCHAR*)LPSTR_TEXTCALLBACKW) || (Text!=(TCHAR*)LPSTR_TEXTCALLBACKA) )
    {
        if (!::IsBadReadPtr (Text,sizeof(TCHAR)))
        {
            _tcsncpy(ItemInfos.Name,Text,CDesktopBrowser_MAX_PATH-1);
            ItemInfos.Name[CDesktopBrowser_MAX_PATH-1]=0;
        }
    }

    if (Text == NULL)
        return;

    if (*Text == 0)
        return;

    ListView_GetItemPosition(this->hWndListView,Index,&ItemInfos.Position);// position is used as item id there's no other solution (we can't play with lparam)
    ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    this->LocalViewDesktopItemsInfos.push_back(ItemInfos);
    ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
    TCHAR sz[CDesktopBrowser_MAX_PATH];
    if ( (Text==(TCHAR*)LPSTR_TEXTCALLBACKW) || (Text==(TCHAR*)LPSTR_TEXTCALLBACKA) )
        _sntprintf(sz,_countof(sz),_T("Item inserted with Text callback\r\n"));
    else
        _sntprintf(sz,_countof(sz),L"Item inserted %s\r\n",Text);
    OutputDebugString(sz);
#endif
#endif

}

BOOL CDesktopBrowser::UpdateRealDesktopItemsInfosName(TCHAR* OldName,TCHAR* NewName)
{
    ::EnterCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
    size_t InfosSize=this->RealDesktopItemsInfos.size();
    if (InfosSize==0)
    {
        ::LeaveCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
        return FALSE;
    }
    size_t CntKnown;
    REAL_DESKTOP_ITEM_INFOS* pRealDesktopItem;
    for (CntKnown=0;CntKnown<InfosSize;CntKnown++)
    {
        pRealDesktopItem = &this->RealDesktopItemsInfos[CntKnown];
        if (_tcscmp(pRealDesktopItem->Name,OldName)==0)
        {
            _tcsncpy(pRealDesktopItem->Name,NewName,CDesktopBrowser_MAX_PATH);
            pRealDesktopItem->Name[CDesktopBrowser_MAX_PATH-1]=0;
            ::LeaveCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
            return TRUE;
        }
    }
    ::LeaveCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
    return FALSE;
}

void CDesktopBrowser::ReplaceItemA(int ItemIndex,CHAR* Text)
{
    LOCAL_VIEW_DESKTOP_ITEM_INFOS Infos={0};

    // if item has no infos
    if (!this->GetItemInfos(ItemIndex,&Infos))
    {
        return this->AddItemA(ItemIndex,Text);
    }
    
    if (Text!=LPSTR_TEXTCALLBACKA)
    {
        if (!::IsBadReadPtr (Text,sizeof(CHAR)))
        {
            if (*Text!='\0')
            {
                TCHAR NameBeforeRenaming[CDesktopBrowser_MAX_PATH];
#if ( defined(UNICODE) || defined(_UNICODE))
                WCHAR Tmp[CDesktopBrowser_MAX_PATH];
                CAnsiUnicodeConvert::AnsiToUnicode(Text,Tmp,CDesktopBrowser_MAX_PATH);
                if (wcscmp(Infos.Name,Tmp)!=0)
                {
                    _tcscpy(NameBeforeRenaming,Infos.Name);
                    wcsncpy(Infos.Name,Tmp,CDesktopBrowser_MAX_PATH);
                    this->UpdateRealDesktopItemsInfosName(NameBeforeRenaming,Infos.Name);
                }

#else
                if (strcmp(Infos.Name,Text)!=0)
                {
                    _tcscpy(NameBeforeRenaming,Infos.Name);
                    strncpy(Infos.Name,Text,CDesktopBrowser_MAX_PATH);
                    this->UpdateRealDesktopItemsInfosName(NameBeforeRenaming,Infos.Name);
                }
#endif
                Infos.Name[CDesktopBrowser_MAX_PATH-1]=0;

#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                CHAR Sz[CDesktopBrowser_MAX_PATH];
                sprintf(Sz,"Item %d replaced with %s\r\n",ItemIndex,Text);
                OutputDebugStringA(Sz);
#endif
#endif
            }
        }
    }
}

void CDesktopBrowser::ReplaceItemW(int ItemIndex,WCHAR* Text)
{
    LOCAL_VIEW_DESKTOP_ITEM_INFOS Infos={0};
    // if item has no infos
    if (!this->GetItemInfos(ItemIndex,&Infos))
    {
        return this->AddItemW(ItemIndex,Text);
    }

    if (Text==LPSTR_TEXTCALLBACKW)
        return;

    if (!::IsBadReadPtr (Text,sizeof(WCHAR)))
    {
        if (*Text!='\0')
        {
            TCHAR NameBeforeRenaming[CDesktopBrowser_MAX_PATH];
#if ( defined(UNICODE) || defined(_UNICODE))
            if (wcscmp(Infos.Name,Text)!=0)
            {
                _tcscpy(NameBeforeRenaming,Infos.Name);
                wcsncpy(Infos.Name,Text,CDesktopBrowser_MAX_PATH);
                this->UpdateRealDesktopItemsInfosName(NameBeforeRenaming,Infos.Name);
            }
#else
            CHAR Tmp[CDesktopBrowser_MAX_PATH];
            CAnsiUnicodeConvert::UnicodeToAnsi(Text,Tmp,CDesktopBrowser_MAX_PATH);
            if (strcmp(Infos.Name,Tmp)!=0)
            {
                _tcscpy(NameBeforeRenaming,Infos.Name);
                strncpy(Infos.Name,Tmp,CDesktopBrowser_MAX_PATH);
                this->UpdateRealDesktopItemsInfosName(NameBeforeRenaming,Infos.Name);
            }
#endif
            Infos.Name[CDesktopBrowser_MAX_PATH-1]=0;

#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
            WCHAR Sz[CDesktopBrowser_MAX_PATH];
            wsprintf(Sz,L"Item %d replaced with %s\r\n",ItemIndex,Text);
            OutputDebugStringW(Sz);
#endif
#endif
        }
    }
}
BOOL CDesktopBrowser::GetItemInfos(IN TCHAR* Name,IN OUT LOCAL_VIEW_DESKTOP_ITEM_INFOS* pInfos)
{
    ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    std::vector <LOCAL_VIEW_DESKTOP_ITEM_INFOS>::iterator pItemInfos;
    for (pItemInfos = this->LocalViewDesktopItemsInfos.begin(); pItemInfos!=this->LocalViewDesktopItemsInfos.end(); pItemInfos++)
    {
        if ( _tcsicmp(Name,pItemInfos->Name)==0 )
        {
            memcpy(pInfos,&(*pItemInfos),sizeof(CDesktopBrowser::LOCAL_VIEW_DESKTOP_ITEM_INFOS));// &(*pItemInfos): iterator shit to access pointer to data
            ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
            return TRUE;
        }
    }
    ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    return FALSE;
}
BOOL CDesktopBrowser::GetItemInfos(IN SIZE_T ItemIndex,IN OUT LOCAL_VIEW_DESKTOP_ITEM_INFOS* pInfos)
{
    TCHAR ItemName[CDesktopBrowser_MAX_PATH];
    *ItemName=0;

    this->InsideLVN_GETDISPINFO++;
    ListView_GetItemText(this->hWndListView,ItemIndex,0,ItemName,CDesktopBrowser_MAX_PATH);
    this->InsideLVN_GETDISPINFO--;

    // if found by name,
    if (this->GetItemInfos(ItemName,pInfos))
        return TRUE;
    // else

    POINT pt={-1,-1};
    ListView_GetItemPosition(this->hWndListView,ItemIndex,&pt);
    // try with position in case of renaming
    std::vector <LOCAL_VIEW_DESKTOP_ITEM_INFOS>::iterator pItemInfos;
    ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    for (pItemInfos = this->LocalViewDesktopItemsInfos.begin(); pItemInfos!=this->LocalViewDesktopItemsInfos.end(); pItemInfos++)
    {
        if ( memcmp(&pt,&pItemInfos->Position,sizeof(POINT))==0 )
        {
            memcpy(pInfos,&(*pItemInfos),sizeof(CDesktopBrowser::LOCAL_VIEW_DESKTOP_ITEM_INFOS));// &(*pItemInfos): iterator shit to access pointer to data
            ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
            return TRUE;
        }
    }
    ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    return FALSE;
}

LRESULT CALLBACK CDesktopBrowser::SubClassProcListView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData)
{
    CDesktopBrowser* pDesktopBrowser = (CDesktopBrowser*)dwRefData;
    UNREFERENCED_PARAMETER(uIdSubclass);
    switch (uMsg)
    {
    case WM_MOUSEWHEEL:
        if (pDesktopBrowser && pDesktopBrowser->bCurrentViewIsDesktopView)
        {
            BOOL bCtrlKeyDown;
            bCtrlKeyDown=GetKeyState(VK_CONTROL) & 0x8000;

            if (bCtrlKeyDown)
            {
                // discard message
                return 0;
            }
        }
        break;
//#ifdef _DEBUG
//        default:
//            if (uMsg>=LVM_FIRST)
//            {
//                TCHAR sz[512];
//                _stprintf(sz,_T("CDesktopBrowser::SubClassProcListView LVM_FIRST+%u\r\n"),uMsg-LVM_FIRST);
//                OutputDebugString(sz);
//            }
//
//            if (uMsg>=WM_USER)
//            {
//                TCHAR sz[512];
//                _stprintf(sz,_T("CDesktopBrowser::SubClassProcListView WM_USER+%u\r\n"),uMsg-WM_USER);
//                OutputDebugString(sz);
//            }
//            else
//            {
//                TCHAR sz[512];
//                _stprintf(sz,_T("CDesktopBrowser::SubClassProcListView Message=0x%X wParam=0x%X lParam=0x%X\r\n"),uMsg,wParam,lParam);
//                OutputDebugString(sz);
//            }
//            break;
//#endif
    }
    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
}

//-----------------------------------------------------------------------------
// Name: SubClassProcShellView
// Object: shellview subclassing proc : 
//         - change info tip values (display item name instead of informations)
//         - spy LVN_BEGINLABELEDIT and LVN_ENDLABELEDIT to keep our info tips up to date
//		/!\ Warning works only on correct identical OS architecture version LVITEM (and associated structs) are platform dependent (32 and 64 bit structs are different)
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CDesktopBrowser::SubClassProcShellView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData)
{
    UNREFERENCED_PARAMETER(uIdSubclass);
    CDesktopBrowser* pDesktopBrowser=(CDesktopBrowser*)dwRefData;

    switch(uMsg)
    {
    case CDesktopBrowser_RESTORE_DESKTOP_ICON_POSITION_NEEDED:
        pDesktopBrowser->SetItemsPosLikeDesktop(FALSE,(SIZE_T)wParam-1);
        break;
    case WM_NOTIFY:
        {
            NMHDR* pnmh=((NMHDR*)lParam);

            // listview messages
            if (pnmh->hwndFrom==pDesktopBrowser->hWndListView)
            {
                LRESULT lResult;
                switch (pnmh->code) 
                {
                // if no information returned, only default tooltip is used
                // Sent by a large icon view list-view control that has the LVS_EX_INFOTIP extended style.
                // This notification is sent when the list-view control is requesting additional text information to be displayed in a ToolTip
                case LVN_GETINFOTIPA    :
                    {
                        NMLVGETINFOTIPA* pInfoTip=(LPNMLVGETINFOTIPA)lParam;
                        size_t Min=__min(CDesktopBrowser_MAX_PATH,pInfoTip->cchTextMax);
                        LOCAL_VIEW_DESKTOP_ITEM_INFOS Infos={0};
                        
                        if (!pDesktopBrowser->GetItemInfos(pInfoTip->iItem,&Infos))
                        {
#ifdef _DEBUG
                            if (::IsDebuggerPresent())
                                ::DebugBreak();
#endif
                            break;
                        }

                        if (pInfoTip->cchTextMax>0)
                        {
#ifdef _DEBUG
                            if (Infos.Name[0]==0)
                            {
                                pDesktopBrowser->DisplayLocalViewDesktopItemsInfos();
                                if (::IsDebuggerPresent())
                                    ::DebugBreak();
                            }
#endif

#if ( defined(UNICODE) || defined(_UNICODE))
                            CAnsiUnicodeConvert::UnicodeToAnsi(Infos.Name,pInfoTip->pszText,Min);
#else
                            strncpy(pInfoTip->pszText,Infos.Name,Min);
#endif
                            pInfoTip->pszText[pInfoTip->cchTextMax-1]=0;
                        }
                    }
                    return TRUE;
                case LVN_GETINFOTIPW    :
                    {
                        NMLVGETINFOTIPW* pInfoTip=(LPNMLVGETINFOTIPW)lParam;
                        int Min=__min(CDesktopBrowser_MAX_PATH,pInfoTip->cchTextMax);
                        LOCAL_VIEW_DESKTOP_ITEM_INFOS Infos={0};

                        if (!pDesktopBrowser->GetItemInfos(pInfoTip->iItem,&Infos))
                        {
#ifdef _DEBUG
                            if (::IsDebuggerPresent())
                                ::DebugBreak();
#endif
                            break;
                        }

                        if (pInfoTip->cchTextMax>0)
                        {
#ifdef _DEBUG
                            if (Infos.Name[0]==0)
                            {
                                pDesktopBrowser->DisplayLocalViewDesktopItemsInfos();
                                if (::IsDebuggerPresent())
                                    ::DebugBreak();
                            }
#endif


#if ( defined(UNICODE) || defined(_UNICODE))
                            wcsncpy(pInfoTip->pszText,Infos.Name,Min);
#else
                            CAnsiUnicodeConvert::AnsiToUnicode(Infos.Name,pInfoTip->pszText,Min);
#endif
                            pInfoTip->pszText[pInfoTip->cchTextMax-1]=0;
                        }
                    }
                    return TRUE;

                // To allow the user to edit the label, return FALSE.
                // To prevent the user from editing the label, return TRUE.
                case LVN_BEGINLABELEDITA:
                case LVN_BEGINLABELEDITW:
                    {
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                        OutputDebugString(_T("LVN_BEGINLABELEDIT\r\n"));
#endif
#endif
                        // hWndEdit already has good informations (thanks to ???)

//                        NMLVDISPINFO* pDispInfo=(NMLVDISPINFO*)lParam;
//                        LOCAL_VIEW_DESKTOP_ITEM_INFOS* pItemInfos=&pDesktopBrowser->LocalViewDesktopItemsInfos[pDispInfo->item.iItem];
//                        if (pDesktopBrowser->LocalViewDesktopItemsInfos.size()<=(size_t)pDispInfo->item.iItem)
//                        {
//#ifdef _DEBUG
//                            pDesktopBrowser->DisplayLocalViewDesktopItemsInfos();
//                            if (::IsDebuggerPresent())
//                                ::DebugBreak();
//#endif
//                            break;
//                        }
//                        lResult = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
//                        HWND hWndEdit=ListView_GetEditControl(pnmh->hwndFrom);
//                        if (pItemInfos->ItemText[0]==0)
//                        {
//                            // update ItemInfos.ItemText if it is empty
//                            ::GetWindowText(hWndEdit,pItemInfos->ItemText,CDesktopBrowser_MAX_PATH);
//                        }
//                        ::SetWindowText(hWndEdit,pDesktopBrowser->LocalViewDesktopItemsInfos[pDispInfo->item.iItem].ItemText);
//                        return lResult;
                    }
                    break;

                // If the pszText member of the LVITEM structure is non-NULL, return TRUE to set the item's label to the edited text. Return FALSE to reject the edited text and revert to the original label. 
                // If the pszText member of the LVITEM structure is NULL, the return value is ignored.
                case LVN_ENDLABELEDITA  :
                    {
                        // If the user cancels editing, the pszText member of the LVITEM structure is NULL; otherwise, pszText is the address of the edited text
                        NMLVDISPINFOA* pDispInfo=(NMLVDISPINFOA*)lParam;

                        lResult = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                        CHAR Sz[CDesktopBrowser_MAX_PATH];
                        sprintf(Sz,"LVN_ENDLABELEDITA return value %d, text %p\r\n",lResult,pDispInfo->item.pszText);
                        OutputDebugStringA(Sz);
#endif
#endif
                        if (
                            // (lResult==TRUE) && // only if name changes have been accepted (MSDN seems to be wrong : change is accepted even if lResult=FALSE)
                            (pDispInfo->item.pszText!=NULL)
                            )
                        {
                            pDesktopBrowser->ReplaceItemA(pDispInfo->item.iItem,pDispInfo->item.pszText);
                        }
                        return lResult;
                    }
                    break;
                case LVN_ENDLABELEDITW  :
                    {
                        NMLVDISPINFOW* pDispInfo=(NMLVDISPINFOW*)lParam;
                        lResult = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                        WCHAR Sz[CDesktopBrowser_MAX_PATH];
                        swprintf(Sz,_countof(Sz),L"LVN_ENDLABELEDITW return value %d, text %p\r\n",lResult,pDispInfo->item.pszText);
                        OutputDebugStringW(Sz);
#endif
#endif
                        if (
                            // (lResult==TRUE) && // only if name changes have been accepted (MSDN seems to be wrong : change is accepted even if lResult=FALSE)
                            (pDispInfo->item.pszText!=NULL)
                            )
                        {
                            // If the user cancels editing, the pszText member of the LVITEM structure is NULL; otherwise, pszText is the address of the edited text
                            pDesktopBrowser->ReplaceItemW(pDispInfo->item.iItem,pDispInfo->item.pszText);
                        }
                        return lResult;
                    }
                    break;

                case LVN_GETDISPINFOA:
                    {
                        NMLVDISPINFOA* pDispInfos=(NMLVDISPINFOA*)lParam;
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                        OutputDebugString(_T("LVN_GETDISPINFOA\r\n"));
                        {
                            TCHAR tc[256];
                            _sntprintf(tc,_countof(tc),_T("mask : 0x%x\r\n"),pDispInfos->item.mask);
                            OutputDebugString(tc);
                        }
#endif
#endif
                        
                        if (pDispInfos->item.mask & LVIF_TEXT)
                        {
                            // let original wnd proc do it's job
                            lResult = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);

                            if (pDesktopBrowser->InsideLVN_GETDISPINFO == 0)
                            {
                                pDesktopBrowser->InsideLVN_GETDISPINFO++;

                                // store text
                                pDesktopBrowser->ReplaceItemA(pDispInfos->item.iItem,pDispInfos->item.pszText);

                                // avoid infinite loop
                                if (pDesktopBrowser->InsideSetItemPosition == 0)
                                {
#if (defined(UNICODE)||defined(_UNICODE))
                                    WCHAR TextW [CDesktopBrowser_MAX_PATH];
                                    CAnsiUnicodeConvert::AnsiToUnicode(pDispInfos->item.pszText,TextW,CDesktopBrowser_MAX_PATH);
                                    pDesktopBrowser->SetItemsPosLikeDesktop(pDispInfos->item.iItem,TextW,FALSE);// setitempos calls LVN_GETDISPINFO                                
#else
                                    pDesktopBrowser->SetItemsPosLikeDesktop(pDispInfos->item.iItem,pDispInfos->item.pszText,FALSE);// setitempos calls LVN_GETDISPINFO
#endif
                                }

                                if (pDesktopBrowser->bEmptyText)
                                    // return an empty text
                                    *pDispInfos->item.pszText=0;

                                pDesktopBrowser->InsideLVN_GETDISPINFO--;
                            }
                            
                            return lResult;
                        }
                    }

                    break;
                case LVN_GETDISPINFOW:
                    {
                        NMLVDISPINFOW* pDispInfos=(NMLVDISPINFOW*)lParam;

#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                        OutputDebugString(_T("LVN_GETDISPINFOW\r\n"));
                        {
                            TCHAR tc[1024];
                            _sntprintf(tc,_countof(tc),_T("mask : 0x%x"),pDispInfos->item.mask);

#ifndef LVIF_COLFMT
#define LVIF_COLFMT             0x00010000 // The piColFmt member is valid in addition to puColumns
#endif
                            if (pDispInfos->item.mask & LVIF_TEXT)
                                _tcscat(tc,_T(" LVIF_TEXT "));
                            if (pDispInfos->item.mask & LVIF_IMAGE)
                                _tcscat(tc,_T(" LVIF_IMAGE "));
                            if (pDispInfos->item.mask & LVIF_PARAM)
                                _tcscat(tc,_T(" LVIF_PARAM "));
                            if (pDispInfos->item.mask & LVIF_STATE)
                                _tcscat(tc,_T(" LVIF_STATE "));
                            if (pDispInfos->item.mask & LVIF_INDENT)
                                _tcscat(tc,_T(" LVIF_INDENT "));
                            if (pDispInfos->item.mask & LVIF_NORECOMPUTE)
                                _tcscat(tc,_T(" LVIF_NORECOMPUTE "));
                            if (pDispInfos->item.mask & LVIF_GROUPID)
                                _tcscat(tc,_T(" LVIF_GROUPID "));
                            if (pDispInfos->item.mask & LVIF_COLUMNS)
                                _tcscat(tc,_T(" LVIF_COLUMNS "));
                            if (pDispInfos->item.mask & LVIF_COLFMT)
                                _tcscat(tc,_T(" LVIF_COLFMT "));
                            if (pDispInfos->item.mask & LVIF_DI_SETITEM)
                                _tcscat(tc,_T(" LVIF_DI_SETITEM "));

                            _tcscat(tc,_T("\r\n"));
                            OutputDebugString(tc);
                        }
#endif
#endif
                        
                        if (pDispInfos->item.mask & LVIF_TEXT)
                        {
                            // let original wnd proc do it's job
                            lResult = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);

                            if (pDesktopBrowser->InsideLVN_GETDISPINFO == 0)
                            {
                                pDesktopBrowser->InsideLVN_GETDISPINFO++;

                                // store text
                                pDesktopBrowser->ReplaceItemW(pDispInfos->item.iItem,pDispInfos->item.pszText);

                                // avoid infinite loop
                                if (pDesktopBrowser->InsideSetItemPosition == 0)
                                {
#if (defined(UNICODE)||defined(_UNICODE))
                                    pDesktopBrowser->SetItemsPosLikeDesktop(pDispInfos->item.iItem,pDispInfos->item.pszText,FALSE);// setitempos calls LVN_GETDISPINFO
#else
                                    CHAR TextA [CDesktopBrowser_MAX_PATH];
                                    CAnsiUnicodeConvert::UnicodeToAnsi(pDispInfos->item.pszText,TextA,CDesktopBrowser_MAX_PATH);
                                    pDesktopBrowser->SetItemsPosLikeDesktop(pDispInfos->item.iItem,TextA,FALSE);// setitempos calls LVN_GETDISPINFO
#endif
                                }

                                if (pDesktopBrowser->bEmptyText)
                                    // return an empty text
                                    *pDispInfos->item.pszText=0;

                                pDesktopBrowser->InsideLVN_GETDISPINFO--;
                            }


                            return lResult;
                        }

                        // on seven always check item position, as item takes more time to be displayed in listview

                    }
                    break;
//#ifdef _DEBUG
//                default:
//                    if (LVN_FIRST >= pnmh->code) // LVN_XX is defined as LVN_FIRST - Value
//                    {
//                        TCHAR sz[CDesktopBrowser_MAX_PATH];
//                        _stprintf(sz,_T("CDesktopBrowser::SubClassProcShellView LVN_FIRST-%u\r\n"),LVN_FIRST - pnmh->code);
//                        OutputDebugString(sz);
//                    }
//                    
//                    break;
//#endif
                }
            }
        }
        break;
    case LVM_SETITEMPOSITION:
        {
            LRESULT lResult;
            pDesktopBrowser->InsideSetItemPosition++;
            lResult  = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
            pDesktopBrowser->InsideSetItemPosition--;
            return lResult;
        }
        break;
    case WM_COMMAND:
        if (CShellBrowser::IsSortingMessage(wParam) 
            )
        {
            // discard arrange message to avoid dead locks

            //LRESULT lResult;
            //pDesktopBrowser->InsideSetItemPosition++;
            //lResult  = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
            //pDesktopBrowser->InsideSetItemPosition--;
            //return lResult;
            return 0;
        }

        break;
    case WM_MOUSEWHEEL: // warning mouse wheel message can be send directly to list view
        {
            BOOL bCtrlKeyDown;
            bCtrlKeyDown=GetKeyState(VK_CONTROL) & 0x8000;

#ifdef _DEBUG
            {
                TCHAR MsgTrace[1024];
                _sntprintf(MsgTrace,_countof(MsgTrace),
                    TEXT("CDesktopBrowser::SubClassProcShellView WM_MOUSEWHEEL bCtrlKeyDown=%u \r\n"),bCtrlKeyDown
                    );
                ::OutputDebugString(MsgTrace);
            }
#endif

            if (bCtrlKeyDown)
            {
                // discard message
                return 0;
#if 0
                // on ctrl + mouse wheel, send VK_F5 to reset layout 
                --> same effect as no message processing
                LRESULT ret = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
                ::PostMessage(hWnd,WM_KEYDOWN,VK_F5,0);
                return ret;
#endif

#if 0
                not working

                // on ctrl + mouse wheel, we get icon increase / decrease and potential scroll bars
                LRESULT ret = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);

                SCROLLINFO ScrollInfo;
                memset(&ScrollInfo,0,sizeof(ScrollInfo));
                ScrollInfo.cbSize = sizeof(ScrollInfo);
                ScrollInfo.fMask = SIF_RANGE | SIF_PAGE | SIF_POS ;
                ScrollInfo.nMin = 1;
                ScrollInfo.nPos = 1;
                ScrollInfo.nMax = 1;
                ScrollInfo.nPage = 1;

                ::SetScrollInfo(hWnd, SB_HORZ, &ScrollInfo, FALSE);
                ::SetScrollInfo(hWnd, SB_VERT, &ScrollInfo, FALSE);

                ::ShowScrollBar(hWnd,SB_HORZ,FALSE);
                ::ShowScrollBar(hWnd,SB_VERT,FALSE);

                return ret;
#endif
            }
        }
        break;
//#ifdef _DEBUG
//    default:
//        {
//            if (uMsg>=WM_USER)
//            {
//                TCHAR sz[CDesktopBrowser_MAX_PATH];
//                _stprintf(sz,_T("CDesktopBrowser::SubClassProcShellView WM_USER+%u\r\n"),uMsg-WM_USER);
//                OutputDebugString(sz);
//            }
//            else
//            {
//                TCHAR sz[CDesktopBrowser_MAX_PATH];
//                _stprintf(sz,_T("CDesktopBrowser::SubClassProcShellView Message=0x%X wParam=0x%X lParam=0x%X\r\n"),uMsg,wParam,lParam);
//                OutputDebugString(sz);
//            }
//        }
//        break;
//#endif
    }
    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
}

#ifdef _DEBUG
// #define DisplayLocalViewDesktopItemsInfos_TRACE 1
#define DisplayLocalViewDesktopItemsInfos_TRACE 0
void CDesktopBrowser::DisplayLocalViewDesktopItemsInfos()
{
#if (DisplayLocalViewDesktopItemsInfos_TRACE==1)
    TCHAR sz[CSingleFolderOptions_MAX_NAME_SIZE];
    ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    for (size_t Cnt=0;Cnt<this->LocalViewDesktopItemsInfos.size();Cnt++)
    {
        _sntprintf(sz,_countof(sz),_T("%s\r\n"),this->LocalViewDesktopItemsInfos[Cnt].Name);
        OutputDebugString(sz);
    }
    ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
#endif
}
#endif