/*
Copyright (C) 2004-2013 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2013 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manage a desktop like browser
//-----------------------------------------------------------------------------

#pragma once

#include "ShellBrowserEx.h"
#include "../StartMenu/StartMenu.h"
#include <vector>


#pragma comment(lib,"Advapi32.lib")

#define CDesktopBrowser_HIDE_DESKTOP_ICONS_KEY _T("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\HideDesktopIcons\\")
#define CDesktopBrowser_NEW_START_PANEL_KEY _T("NewStartPanel")
#define CDesktopBrowser_CLASSIC_START_MENU_KEY _T("ClassicStartMenu")
#define CDesktopBrowser_ITEM_POSITION_ERROR 0xffffffff
#define CDesktopBrowser_FULL_REFRESH_NEEDED (WM_APP+0x2FF1)  // WM_APP through WM_APP+0x3FFF Messages available for use by applications. 
                                                        // message sent to parent window, when a call to CDesktopBrowser::Refresh() must be done
#define CDesktopBrowser_RESTORE_DESKTOP_ICON_POSITION_NEEDED (WM_APP+0x2FF2)
#define CDesktopBrowser_LISTVIEW_APPEND (0x7FFFFFFF)
#define CDesktopBrowser_MAX_PATH 2048

class CDesktopBrowser:public CShellBrowserEx
{
public:
    CDesktopBrowser(HWND hWndParent);
    CDesktopBrowser(HWND hWndParent,FOLDERSETTINGS* pfs);
    CDesktopBrowser(HWND hWndParent,FOLDERSETTINGS* pfs,RECT* pRect);
    ~CDesktopBrowser(void);
    HRESULT BrowseDesktop();
    BOOL bUsePrimaryDisplayMonitorOnly;
    // ICommDlgBrowser::IncludeObject specific implementation
    STDMETHODIMP IncludeObject(THIS_ struct IShellView *ppshv,LPCITEMIDLIST pidl);
    virtual HRESULT UpdateRect(RECT* pRect);
    virtual HRESULT Refresh();
protected:

    CRITICAL_SECTION RealDesktopItemsInfosCriticalSection;
    CRITICAL_SECTION LocalViewDesktopItemsInfosCriticalSection;

    typedef struct tagRealDesktopItemInfos
    {
        POINT Position;
        TCHAR Name[CDesktopBrowser_MAX_PATH];
    }REAL_DESKTOP_ITEM_INFOS;

    typedef struct tagLocalViewDesktopItemsInfos
    {
        POINT Position;
        TCHAR Name[CDesktopBrowser_MAX_PATH];
    }LOCAL_VIEW_DESKTOP_ITEM_INFOS;

    std::vector <LOCAL_VIEW_DESKTOP_ITEM_INFOS> LocalViewDesktopItemsInfos;
    std::vector <ITEMIDLIST*> HiddenItems;
    std::vector <REAL_DESKTOP_ITEM_INFOS> RealDesktopItemsInfos;

    ITEMIDLIST* pidlDesktop;
    IMalloc* pMalloc;
    SIZE_T InsideLVN_GETDISPINFO; // if >0 do not call SetItemsPosLikeDesktop during GetDispInfo notification management
    SIZE_T InsideSetItemPosition;

    HWND hDesktopListView;

    BOOL GetItemInfos(IN TCHAR* Name,IN OUT LOCAL_VIEW_DESKTOP_ITEM_INFOS* pInfos);
    BOOL GetItemInfos(IN SIZE_T ItemIndex,IN OUT LOCAL_VIEW_DESKTOP_ITEM_INFOS* pInfos);
    BOOL UpdateRealDesktopItemsInfosName(TCHAR* OldName,TCHAR* NewName);

    virtual void InternalBeforeBrowseCallBackVirtual(LPCITEMIDLIST pidlToBeBrowsed);
    void InternalBeforeBrowseCallBack(LPCITEMIDLIST pidlToBeBrowsed);

    virtual void InternalBeforeViewChangedCallBackVirtual(tagShellBrowserView NewShellBrowserView);
    void InternalBeforeViewChangedCallBack(tagShellBrowserView NewShellBrowserView);
    virtual void InternalAfterListViewCreationCallBackVirtual();
    void InternalAfterListViewCreationCallBack();

    virtual void InternalAfterViewChangedCallBackVirtual(tagShellBrowserView NewShellBrowserView);
    void InternalAfterViewChangedCallBack(tagShellBrowserView NewShellBrowserView);

    HRESULT InternalIncludeObject(LPCITEMIDLIST pidl);

    static LRESULT CALLBACK SubClassProcShellView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData);
    static LRESULT CALLBACK CDesktopBrowser::SubClassProcListView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData);
    static LRESULT CALLBACK SubClassProcListViewToolTip(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData);

    void AddItemInfos(SIZE_T Index,TCHAR* Text);
    void LoadHiddenIcons();
    void LoadHiddenIcons(HKEY hRootKey,TCHAR* KeyName);
    void RemoveControlPanelForSeven();
    void CommonConstructor();
    void InstallSubProc();
    void RemoveSubProc();
    void RestoreItemsText();

#ifdef _DEBUG
    void DisplayLocalViewDesktopItemsInfos();
#endif

    void AddItemA(int ItemIndex,CHAR* Text);
    void AddItemW(int ItemIndex,WCHAR* Text);
    void ReplaceItemA(int ItemIndex,CHAR* Text);
    void ReplaceItemW(int ItemIndex,WCHAR* Text);

    HRESULT InternalUpdateRect(RECT* pRect);
    BOOL IsDesktopFolder(LPCITEMIDLIST pidl);
    BOOL DoesSmallViewBestFit();
    HWND GetDesktopListView();
    BOOL GetDesktopPidl();
    BOOL GetDesktopItemsPos();
    BOOL SetItemsPosLikeDesktop(SIZE_T ItemIndex,TCHAR* ItemOriginalText,BOOL bGetRealDesktopItemsPosition);
    BOOL SetItemsPosLikeDesktop(BOOL bGetRealDesktopItemsPosition,SIZE_T NbRetry);

    virtual HRESULT SetView(tagShellBrowserView ShellBrowserView);
    BOOL bCurrentViewIsDesktopView;// TRUE if current folder is the desktop one
    BOOL bSmallIconsView;// TRUE if small icons best match browser size
    BOOL bDesktopLikeView;// TRUE if items must be positioned like on desktop
    tagShellBrowserView PreviousView;
    BOOL bSettingViewInternally;
    BOOL bEmptyText;
    BOOL bSortItemNeeded;
    LPARAM ToolTipInitialTime;
};