/*
Copyright (C) 2004-2013 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2013 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manage a desktop like browser
//-----------------------------------------------------------------------------

#include "DesktopBrowserXP.h"
#include "../DesktopWorkArea/DesktopWorkArea.h"

//-----------------------------------------------------------------------------
// Name: CDesktopBrowserXP
// Object: Constructor.
// Parameters :
//     in  : HWND hWndParent : parent window handle
//     out :
//     return : 
//-----------------------------------------------------------------------------
CDesktopBrowserXP::CDesktopBrowserXP(HWND hWndParent):CShellBrowserEx(hWndParent)
{
    this->CommonConstructor();
    this->FolderSettings.ViewMode=FVM_LIST;// better style to avoid scrolling bar
    this->FolderSettings.fFlags=FWF_NOWEBVIEW;
}

//-----------------------------------------------------------------------------
// Name: CDesktopBrowserXP
// Object: Constructor.
// Parameters :
//     in  : HWND hWndParent : parent window handle
//           FOLDERSETTINGS* pfs : folder settings
//     out :
//     return : 
//-----------------------------------------------------------------------------
CDesktopBrowserXP::CDesktopBrowserXP(HWND hWndParent,FOLDERSETTINGS* pfs):CShellBrowserEx(hWndParent,pfs)
{
    this->CommonConstructor();
}

//-----------------------------------------------------------------------------
// Name: CDesktopBrowserXP
// Object: Constructor.
// Parameters :
//     in  : HWND hWndParent : parent window handle
//           FOLDERSETTINGS* pfs : folder settings
//           RECT* pRect : Rect use for shell browser
//     out :
//     return : 
//-----------------------------------------------------------------------------
CDesktopBrowserXP::CDesktopBrowserXP(HWND hWndParent,FOLDERSETTINGS* pfs,RECT* pRect):CShellBrowserEx(hWndParent,pfs,pRect)
{
    this->CommonConstructor();
}

//-----------------------------------------------------------------------------
// Name: CommonConstructor
// Object: common Constructor.
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowserXP::CommonConstructor()
{
    ::InitializeCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
    ::InitializeCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    
    CShellBrowser::FolderSettingToShellBrowserView(&this->FolderSettings,&this->PreviousView);
    this->pMalloc=NULL;
    this->pidlDesktop=NULL;
    this->GetDesktopPidl();
    
    // get real desktop size
    this->bSmallIconsView=TRUE;
    this->bCurrentViewIsDesktopView=TRUE;
    this->ToolTipInitialTime=MAKELONG(-1,0);
    this->bDesktopLikeView=TRUE;
    this->bEmptyText=FALSE;
    this->bIsSettingItemPosition=FALSE;
    this->bSortItemNeeded=FALSE;
    this->bSettingViewInternally=FALSE;

    this->hDesktopListView = NULL;

    this->LoadHiddenIcons();
}

//-----------------------------------------------------------------------------
// Name: ~CDesktopBrowserXP
// Object: Destructor.
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
CDesktopBrowserXP::~CDesktopBrowserXP(void)
{
    // free hidden pidl
    for (size_t Cnt=0;Cnt<this->HiddenItems.size();Cnt++)
    {
        this->pMalloc->Free(this->HiddenItems[Cnt]);
    }

    // free desktop pidl
    if (this->pidlDesktop)
        pMalloc->Free(this->pidlDesktop);

    if (this->pMalloc)
        pMalloc->Release();

    ::DeleteCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
    ::DeleteCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
}

//-----------------------------------------------------------------------------
// Name: LoadHiddenIcons
// Object: load hidden icon from registry
// Parameters :
//     in  : 
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowserXP::LoadHiddenIcons()
{
    ///////////////////////////////////
    // get hidden desktop items from registry
    ///////////////////////////////////
    this->HiddenItems.clear();

    // hidden desktop icons
    // "HKCU\Software\Microsoft\Windows\CurrentVersion\Explorer\HideDesktopIcons\ClassicStartMenu" classical start menu
    // "HKCU\Software\Microsoft\Windows\CurrentVersion\Explorer\HideDesktopIcons\NewStartPanel" XP start menu (warning it has nothing to see with Vista Aero style as mention on some web sites)
    // ; 0 = Show  1 = Hide
    //;-----  {20D04FE0-3AEA-1069-A2D8-08002B30309D} My Computer
    //;-----  {450D8FBA-AD25-11D0-98A8-0800361B1103} My Documents
    //;-----  {208D2C60-3AEA-1069-A2D7-08002B30309D} My Network Places
    //;-----  {871C5380-42A0-1069-A2EA-08002B30309D} Internet Explorer
    //;-----  {645FF040-5081-101B-9F08-00AA002F954E} Recycle Bin 
    // 
    //;-----  {00020D75-0000-0000-C000-000000000046} Outlook
    // ...

    // --> the only way is to enum all key, and next use IShellFolder::ParseDisplayName (,,L"::{GUID}",...)
    //     to get the hidden pidl
    TCHAR* SubKeyName;
    TCHAR KeyName[MAX_PATH];

    // get registry key according to user interface
    // if Classic Start Menu
    if (CStartMenu::IsClassicStartMenu())
        SubKeyName=CDesktopBrowserXP_CLASSIC_START_MENU_KEY;
    else // XP Start Menu
        SubKeyName=CDesktopBrowserXP_NEW_START_PANEL_KEY;

    // make full key name
    _tcscpy(KeyName,CDesktopBrowserXP_HIDE_DESKTOP_ICONS_KEY);
    _tcscat(KeyName,SubKeyName);

    // parse HKEY_LOCAL_MACHINE first, to next overwrite settings with HKEY_CURRENT_USER one's
    this->LoadHiddenIcons(HKEY_LOCAL_MACHINE,KeyName);
    this->LoadHiddenIcons(HKEY_CURRENT_USER,KeyName);
}
//-----------------------------------------------------------------------------
// Name: LoadHiddenIcons
// Object: load hidden icon from registry according to hRootKey and KeyName
// Parameters :
//     in  : HKEY hRootKey : HKLM or HKCU
//     out : TCHAR* KeyName : CDesktopBrowserXP_CLASSIC_START_MENU_KEY or CDesktopBrowserXP_NEW_START_PANEL_KEY
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowserXP::LoadHiddenIcons(HKEY hRootKey,TCHAR* KeyName)
{
    HKEY hKey;

    // open registry key
    if (::RegOpenKey(hRootKey,KeyName,&hKey)==ERROR_SUCCESS)
    {
        IShellFolder* pDesktopFolder=NULL;
        WCHAR wKey[MAX_PATH+2];
        DWORD KeyLen;
        DWORD KeyType;
        DWORD Data;
        DWORD DataSize;
        HRESULT hResult;

        // prepare buffer by prefixing "::"
        wcscpy(wKey,L"::");// MSDN IShellFolder::ParseDisplayName must be "::{GUID}"

        // get root folder
        hResult= ::SHGetDesktopFolder(&pDesktopFolder);
        if (SUCCEEDED(hResult)&&(pDesktopFolder))
        {
            LONG Result=ERROR_SUCCESS;
            ITEMIDLIST* pFullpidl=NULL;

            // for each registry key <-> each possible hidden guid
            for (DWORD Cnt=0;Result!=ERROR_NO_MORE_ITEMS;Cnt++)
            {
                DataSize=sizeof(DWORD);
                KeyLen=MAX_PATH;
                Result=::RegEnumValueW(hKey,Cnt,&wKey[2],&KeyLen,0,&KeyType,(LPBYTE)&Data,&DataSize);
                if (Result!=ERROR_SUCCESS)
                    continue;

                if (KeyType!=REG_DWORD) // can be default key
                    continue;

                // 0 = Show ; 1 = Hide
                if (hRootKey == HKEY_LOCAL_MACHINE)
                {
                    // only store hidden item
                    if (Data==0)
                        continue;
                }

                // get object full pidl from path name
                hResult = pDesktopFolder->ParseDisplayName(NULL,
                    NULL,
                    wKey,// MSDN : can be "::{GUID}"
                    NULL,
                    &pFullpidl,
                    NULL);
                if ( SUCCEEDED(hResult) && (pFullpidl) )
                {
                    if (hRootKey == HKEY_CURRENT_USER)
                    {
                        BOOL bFound = FALSE;
                        size_t FoundItemIndex = 0;
                        // check if pidl belongs to hidden items
                        for (size_t Cnt2=0;Cnt2<this->HiddenItems.size();Cnt2++)
                        {
                            if (::ILIsEqual(this->HiddenItems[Cnt2],pFullpidl))
                            {
                                bFound = TRUE;
                                FoundItemIndex = Cnt2;
                                break;
                            }
                        }
                        if (bFound)
                        {
                            if (Data==0)// item must be shown --> remove it from list
                            {
                                if (this->HiddenItems.size()!=0)
                                    this->HiddenItems.erase(this->HiddenItems.begin() + FoundItemIndex);
                            }
                            // else item is already in list and must be hidden --> nothing to do
                        }
                        else
                        {
                            if (Data==0)// item must be shown --> don't add it to list
                                continue;
                            // else : item must be hidden --> add it to list

                            // add pidl to hidden items
                            this->HiddenItems.push_back(pFullpidl);
                        }
                    }
                    else // (hRootKey == HKEY_LOCAL_MACHINE)
                    {
                        // add pidl to hidden items
                        this->HiddenItems.push_back(pFullpidl);
                    }
                }
            }
        }

        // close registry key
        ::RegCloseKey(hKey);
    }
}

//-----------------------------------------------------------------------------
// Name: SetViewInternal
// Object: change browser view without storing new view in PreviousView
//          NOTICE : SHELL_BROWSER_VIEW_SMALL_ICONS seems to have no effect
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CDesktopBrowserXP::SetViewInternal(tagShellBrowserView ShellBrowserView)
{
    HRESULT hResult;
    this->bSettingViewInternally=TRUE;
    hResult=this->SetView(ShellBrowserView);
    this->bSettingViewInternally=FALSE;

    return hResult;
}


//-----------------------------------------------------------------------------
// Name: BrowseDesktop
// Object: browse user desktop
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CDesktopBrowserXP::BrowseDesktop()
{
    return this->BrowseObject(CSIDL_DESKTOP);
}

//-----------------------------------------------------------------------------
// Name: GetDesktopPidl
// Object: get pidl associated to user desktop folder
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
BOOL CDesktopBrowserXP::GetDesktopPidl()
{
    BOOL bRet=FALSE;
    if (this->pidlDesktop)
        return TRUE;

    HRESULT hResult = E_FAIL;

    if (!this->pMalloc)
    {
        // get pointer to malloc interface
        hResult = ::SHGetMalloc(&this->pMalloc);
        if (FAILED(hResult)||(!this->pMalloc))
            goto CleanUp;
    }

    // get folder full idl list
    hResult = ::SHGetFolderLocation(this->hWndParent,CSIDL_DESKTOP,NULL,NULL,&this->pidlDesktop);
    if ( FAILED(hResult) || (!this->pidlDesktop) )
        goto CleanUp;

    bRet=TRUE;

CleanUp:
    return bRet;
}

//-----------------------------------------------------------------------------
// Name: IsDesktopFolder
// Object: test if pidl correspond to user desktop folder
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
BOOL CDesktopBrowserXP::IsDesktopFolder(LPCITEMIDLIST pidl)
{
    if ( (!pidl) || (!this->pidlDesktop) )
        return FALSE;

    return ::ILIsEqual(pidl,this->pidlDesktop);
}

//-----------------------------------------------------------------------------
// Name: IsSettingItemPosition
// Object: to know if we currently are setting position of items
// Parameters :
//     in  : 
//     out :
//     return : TRUE if we currently are setting position of items
//-----------------------------------------------------------------------------
BOOL CDesktopBrowserXP::IsSettingItemPosition()
{
    return this->bIsSettingItemPosition;
}

//-----------------------------------------------------------------------------
// Name: DoesSmallViewBestFit
// Object: check if small icons is the best view 
// Parameters :
//     in  : 
//     out :
//     return : TRUE if small icons is the best view, FALSE if big icons are the best view
//-----------------------------------------------------------------------------
BOOL CDesktopBrowserXP::DoesSmallViewBestFit()
{
    RECT DesktopRect;
    CDesktopWorkArea::GetWorkArea(&DesktopRect,this->bUsePrimaryDisplayMonitorOnly);
    // if browser rect < 80% of desktop rect
    if 
    (
        (((this->BrowserRect.bottom-this->BrowserRect.top)*100)/(DesktopRect.bottom-DesktopRect.top)<80)
        && (((this->BrowserRect.right-this->BrowserRect.left)*100)/(DesktopRect.right-DesktopRect.left)<80)
    )
    {
        return TRUE;
    }

    return FALSE;
}

//-----------------------------------------------------------------------------
// Name: UpdateRect
// Object: update browser rect
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CDesktopBrowserXP::UpdateRect(RECT* pRect)
{
    // if no rect changes
    if (::EqualRect(&this->BrowserRect,pRect))
        return S_OK;

    BOOL bIsNewRectLarger;

    bIsNewRectLarger = (pRect->right-pRect->left>=this->BrowserRect.right-this->BrowserRect.left)
                        && (pRect->bottom-pRect->top>=this->BrowserRect.bottom-this->BrowserRect.top);

    // if new rect is larger, to avoid scrollbars, resize control before setting items position
    if (bIsNewRectLarger)
    {
        // update item rect
        this->CShellBrowserEx::InternalUpdateRect(pRect,TRUE);
    }
    else
    {
        // store new rect size for rect based function computation
        memcpy(&this->BrowserRect,pRect,sizeof(RECT));
    }

    // if not in desktop like view
    if (!this->bDesktopLikeView)
    {
        if (!bIsNewRectLarger)
        {
            this->CShellBrowserEx::InternalUpdateRect(pRect,TRUE);
        }
        return S_OK;
    }

    HRESULT hResult;
    BOOL bOldSmallIconsView=this->bSmallIconsView;
    this->bSmallIconsView=this->DoesSmallViewBestFit();

    // if best fit has changed
    if (bOldSmallIconsView!=this->bSmallIconsView)
    {
        if (this->bSmallIconsView)
        {
            this->SetViewInternal(SHELL_BROWSER_VIEW_SMALL_ICONS);
            
            // restore state of properties affected by InternalAfterViewChangedCallBack (RemoveSubProc)
            this->bDesktopLikeView=TRUE;
            this->bSmallIconsView=TRUE;

            // begin subclassing again
            this->InstallSubProc();
        }
        else
        {
            this->SetViewInternal(SHELL_BROWSER_VIEW_ICONS);
            // restore state of properties affected by InternalAfterViewChangedCallBack (RemoveSubProc)
            this->bDesktopLikeView=TRUE;

            // begin subclassing again
            this->InstallSubProc();
        }
    }

    // update item positions
    if (this->SetItemsPosLikeDesktop(FALSE))// FALSE : earn time as UpdateRect is generally called on a WM_SIZE msg
        hResult = S_OK;
    else
        hResult = E_FAIL;

    // if new rect is smaller, to avoid scrollbars, resize control after setting items position
    if (!bIsNewRectLarger)
    {
        this->CShellBrowserEx::InternalUpdateRect(pRect,TRUE);
    }

    return hResult;
}

//-----------------------------------------------------------------------------
// Name: Refresh
// Object: refresh current view
// Parameters :
//     in  : 
//     out : 
//     return : S_OK on success
//-----------------------------------------------------------------------------
HRESULT CDesktopBrowserXP::Refresh()
{
    // if not desktop view return CShellBrowserEx refresh value
    if (!this->bDesktopLikeView)
        return this->CShellBrowserEx::Refresh();

    else // only to create a block and create local variables in this case
    {
        // At least until XP SP3 hiding a desktop icon from registry 
        // don't send a shell notification, so LVM_DELETEITEM is not send
        // The consequence for us can be a desynchronization between our text list and the real items text list
        // So the simple way is to fully recreate the view if registry has changed

        BOOL bHiddenIconsHaveChanged = FALSE;
        // store old hidden items
        std::vector <ITEMIDLIST*> OldHiddenItems;
        LPITEMIDLIST pItemIdList;
        for (size_t Cnt=0;Cnt<this->HiddenItems.size();Cnt++)
        {
            pItemIdList = ::ILClone(this->HiddenItems[Cnt]);
            OldHiddenItems.push_back(pItemIdList);
        }

        // get hidden icon from registry
        this->LoadHiddenIcons();

        // check if hidden items in registry have changed
        if (OldHiddenItems.size() != this->HiddenItems.size())
        {
            bHiddenIconsHaveChanged = TRUE;
        }
        for (size_t Cnt=0;Cnt<OldHiddenItems.size();Cnt++)
        {
            if (!bHiddenIconsHaveChanged)
            {
                if (!::ILIsEqual(this->HiddenItems[Cnt],OldHiddenItems[Cnt]))
                {
                    bHiddenIconsHaveChanged = TRUE;
                }
            }
            // free allocated memory
            ::ILFree(OldHiddenItems[Cnt]);
        }

        this->GetDesktopItemsPos();
        // if hidden icons have changed, it's better for us to recreate the view
        if (bHiddenIconsHaveChanged)
            return this->RecreateCurrentView();
        else // else do a simple refresh (less time consuming)
            return this->CShellBrowserEx::Refresh();
    }

}

//-----------------------------------------------------------------------------
// Name: IncludeObject
// Object: ICommDlgBrowser::IncludeObject specific implementation
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
STDMETHODIMP CDesktopBrowserXP::IncludeObject(THIS_ struct IShellView *ppshv,
                                            LPCITEMIDLIST pidl)
{
    UNREFERENCED_PARAMETER(ppshv);
    return this->InternalIncludeObject(pidl);
}

//-----------------------------------------------------------------------------
// Name: InternalIncludeObject
// Object: ICommDlgBrowser::IncludeObject specific implementation
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CDesktopBrowserXP::InternalIncludeObject(LPCITEMIDLIST pidl)
{
    if (this->bCurrentViewIsDesktopView)
    {
        // check if pidl belongs to hidden items
        for (size_t Cnt=0;Cnt<this->HiddenItems.size();Cnt++)
        {
            if (::ILIsEqual(this->HiddenItems[Cnt],pidl))
            {
                // object must be hidden
                return S_FALSE;
            }
        }
    }
    return this->CShellBrowserEx::InternalIncludeObject(pidl);
}

//-----------------------------------------------------------------------------
// Name: InternalAfterListViewCreationCallBackVirtual
// Object: method called after listview creation
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowserXP::InternalAfterListViewCreationCallBackVirtual()
{
    this->InternalAfterListViewCreationCallBack();
}

//-----------------------------------------------------------------------------
// Name: InternalAfterListViewCreationCallBack
// Object: method called after listview creation
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowserXP::InternalAfterListViewCreationCallBack()
{
    this->bEmptyText=FALSE;
    this->bSortItemNeeded=FALSE;
    this->bIsSettingItemPosition=FALSE;

    // if desktop like view
    if (this->bDesktopLikeView)
    {
        this->LoadHiddenIcons();

        this->bSmallIconsView=this->DoesSmallViewBestFit();

        if (this->bSmallIconsView)
        {
            this->SetViewInternal(SHELL_BROWSER_VIEW_SMALL_ICONS);

            // restore state of properties affected by InternalAfterViewChangedCallBack (RemoveSubProc)
            this->bDesktopLikeView=TRUE;
            this->bSmallIconsView=TRUE;

            // call parent object (CShellBrowserEx) InternalAfterListViewCreationCallBack method
            // after having changed listview representation and before installing subproc
            this->CShellBrowserEx::InternalAfterListViewCreationCallBack();

            // begin subclassing again
            this->InstallSubProc();
        }
        else
        {
            this->SetViewInternal(SHELL_BROWSER_VIEW_ICONS);

            // restore state of properties affected by InternalAfterViewChangedCallBack (RemoveSubProc)
            this->bDesktopLikeView=TRUE;

            // call parent object (CShellBrowserEx) InternalAfterListViewCreationCallBack method
            // after having changed listview representation and before installing subproc
            this->CShellBrowserEx::InternalAfterListViewCreationCallBack();

            // begin subclassing again
            this->InstallSubProc();
        }
    }
    else
    {
        // restore previous view
        this->SetViewInternal(this->PreviousView);

        // call parent object (CShellBrowserEx) InternalAfterListViewCreationCallBack method
        // after having changed listview representation
        this->CShellBrowserEx::InternalAfterListViewCreationCallBack();
    }
}

//-----------------------------------------------------------------------------
// Name: InstallSubProc
// Object: install subclassing and do all the required job
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowserXP::InstallSubProc()
{
    UINT_PTR uIdSubclass=0;
    DWORD_PTR dwRefData=(DWORD_PTR)this;

    this->bEmptyText=FALSE;
    this->bSortItemNeeded=FALSE;
    this->bIsSettingItemPosition=FALSE;

    // if current displayed folder is the desktop one
    if (this->bCurrentViewIsDesktopView)
    {
        // first set view (else next changes will be trashed)
        if (this->bSmallIconsView)
        {
            // set view
            ListView_SetView(this->hWndListView,LVS_SMALLICON);
        }
        else
        {
            ListView_SetView(this->hWndListView,LV_VIEW_ICON);
        }

        if (this->bSmallIconsView)
        {
            // set icon spacing (allow tooltip to appear only when mouse is hover icon)
            ListView_SetIconSpacing(this->hWndListView,4,4);

            // must be called after ListView_SetIconSpacing
            this->SetItemsPosLikeDesktop(TRUE);

            ///////////////
            //remove text : text won't be displayed in listview but in tooltips
            ///////////////
            ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
            this->LocalViewDesktopItemsInfos.clear();
            LOCAL_VIEW_DESKTOP_ITEM_INFOS Item;
            int NbItems=ListView_GetItemCount(this->hWndListView);
            for (int Cnt=0;Cnt<NbItems;Cnt++)
            {
                // get text and store it to vector
                ListView_GetItemText(this->hWndListView,Cnt,0,Item.ItemText,MAX_PATH);
                this->LocalViewDesktopItemsInfos.push_back(Item);

                // remove text (must be done before using SubClassProcListView subclassing)
                ListView_SetItemText(this->hWndListView,Cnt,0,_T(""));
            }
            ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);

            this->bEmptyText=TRUE;

            // change tooltip delay time (as text is removed, tooltip must appear as soon as mouse is hover an item)
            HWND hToopTip = ListView_GetToolTips(this->hWndListView);
            this->ToolTipInitialTime=(LPARAM)::SendMessage(hToopTip,TTM_GETDELAYTIME,TTDT_INITIAL,0);
            ::SendMessage(hToopTip,TTM_SETDELAYTIME,TTDT_INITIAL,MAKELONG(0,0));

            // change tooltip margins
            RECT MarginRect={0,0,0,0};
            ::SendMessage(hToopTip,TTM_SETMARGIN,0,(LPARAM)&MarginRect);

            // subclass shellview, listview and tooltip
            // NOTICE : do this at least (avoid to process/block our own messages)
            ::SetWindowSubclass(this->hWndListView,CDesktopBrowserXP::SubClassProcListView,uIdSubclass,dwRefData);
            ::SetWindowSubclass(this->hWndShellView,CDesktopBrowserXP::SubClassProcShellView,uIdSubclass,dwRefData);
            ::SetWindowSubclass(hToopTip,CDesktopBrowserXP::SubClassProcListViewToolTip,uIdSubclass,dwRefData);
        }
        else
        {
            this->SetItemsPosLikeDesktop(TRUE);

            // subclass listview only to restore icons position
            ::SetWindowSubclass(this->hWndListView,CDesktopBrowserXP::SubClassProcListView,uIdSubclass,dwRefData);
        }
    }
}

//-----------------------------------------------------------------------------
// Name: RemoveSubProc
// Object: remove subclassing and restore all to default
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowserXP::RemoveSubProc()
{
    // if current view is the desktop one and view was the small icon one
    if (this->bCurrentViewIsDesktopView && this->bDesktopLikeView)
    {
        this->bDesktopLikeView=FALSE;
        this->bSmallIconsView=FALSE;

        // remove subclassing (do it first to avoid next messages to be trashed)
        UINT_PTR uIdSubclass=0;
        ::RemoveWindowSubclass(this->hWndListView,CDesktopBrowserXP::SubClassProcListView,uIdSubclass);
        ::RemoveWindowSubclass(this->hWndShellView,CDesktopBrowserXP::SubClassProcShellView,uIdSubclass);

        HWND hToopTip = ListView_GetToolTips(this->hWndListView);
        ::RemoveWindowSubclass(hToopTip,CDesktopBrowserXP::SubClassProcListViewToolTip,uIdSubclass);

        // restore tooltip time
        ::SendMessage(hToopTip,TTM_SETDELAYTIME,TTDT_INITIAL,this->ToolTipInitialTime);

        // reset icon spacing to default
        ListView_SetIconSpacing(this->hWndListView,-1,-1);

        this->RestoreItemsText();

        // clear vector
        ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
        this->LocalViewDesktopItemsInfos.clear();
        ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    }
}

//-----------------------------------------------------------------------------
// Name: RestoreItemsText
// Object: if text has been removed, restore original item text
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowserXP::RestoreItemsText()
{
    if (!this->bEmptyText)
        return;

    // restore items text (must be done after removing subclasses)
    ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    int NbItems=(int)this->LocalViewDesktopItemsInfos.size();
    for (int Cnt=0;Cnt<NbItems;Cnt++)
    {
        ListView_SetItemText(this->hWndListView,Cnt,0,this->LocalViewDesktopItemsInfos[Cnt].ItemText);
    }
    ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);

    this->bEmptyText=FALSE;
}

//-----------------------------------------------------------------------------
// Name: SetItemsPosLikeDesktop
// Object: set items like desktop items positions
// Parameters :
//     in  : BOOL bGetRealDesktopItemsPosition : TRUE to refresh real desktop items position
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CDesktopBrowserXP::SetItemsPosLikeDesktop(BOOL bGetRealDesktopItemsPosition)
{
    if (bGetRealDesktopItemsPosition 
        || (this->RealDesktopItemsInfos.size()==0)// assume GetDesktopItemsPos has been called once
        )
        // get desktop items positions
        this->GetDesktopItemsPos(); // fill this->RealDesktopItemsInfos

    POINT pt={0,0};
    int NbItems;

    ::EnterCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
    int InfosSize=(int)this->RealDesktopItemsInfos.size();
    if (InfosSize==0)
    {
        ::LeaveCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
        return FALSE;
    }
    BOOL bFound;
    TCHAR Name[CDesktopBrowser_MAX_PATH];

    ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    if (this->bEmptyText)
        NbItems=(int)this->LocalViewDesktopItemsInfos.size();
    else
        NbItems=ListView_GetItemCount(this->hWndListView);

    // get our target listview size
    RECT ListViewRect;
    // use this->BrowserRect instead of querying listview rect
    // It's interesting in case of listview size reducing, as UpdateRect position items
    // before sizing listview
    // ::GetClientRect(this->hWndListView,&ListViewRect);
    memcpy(&ListViewRect,&this->BrowserRect,sizeof(RECT));
    // to make sure there won't be scroll bars, reduce real available space by scroll bar size
    ListViewRect.right-=::GetSystemMetrics(SM_CXVSCROLL);
    ListViewRect.bottom-=::GetSystemMetrics(SM_CYHSCROLL);

    RECT PrimaryDesktopWorkArea;
    RECT AllDisplayWorkArea;
    RECT DesktopRect;
    CDesktopWorkArea::GetWorkAreaEx(&PrimaryDesktopWorkArea,&AllDisplayWorkArea);
    // put primary rect into relative coordinate of top left display
    ::OffsetRect(&PrimaryDesktopWorkArea,-AllDisplayWorkArea.left,-AllDisplayWorkArea.top);
    ::OffsetRect(&AllDisplayWorkArea,-AllDisplayWorkArea.left,-AllDisplayWorkArea.top);
    if (this->bUsePrimaryDisplayMonitorOnly)
        DesktopRect = PrimaryDesktopWorkArea;
    else
        DesktopRect = AllDisplayWorkArea;

    LONG BorderSecurity;// security space to avoid scroll bars
    LONG IconSize;
    for (int Cnt=0;Cnt<NbItems;Cnt++)
    {
        // get current item name 
        *Name=0;

        if (this->bEmptyText)
            _tcscpy(Name,this->LocalViewDesktopItemsInfos[Cnt].ItemText);
        else
            ListView_GetItemText(this->hWndListView,Cnt,0,Name,MAX_PATH);

        // search name in RealDesktopItemsInfos
        bFound=FALSE;
        for (int CntKnown=0;CntKnown<InfosSize;CntKnown++)
        {
            if (_tcscmp(this->RealDesktopItemsInfos[CntKnown].Name,Name)==0)
            {
                memcpy(&pt,&this->RealDesktopItemsInfos[CntKnown].Position,sizeof(POINT));
                bFound=TRUE;
                break;
            }
        }

        // if not found
        if (!bFound)
        {
#ifdef _DEBUG
            if (::IsDebuggerPresent())
                ::DebugBreak();
#endif
            continue;
        }


        // if item position retrieval was successful
        if (   (pt.x!=CDesktopBrowserXP_ITEM_POSITION_ERROR)
            && (pt.y!=CDesktopBrowserXP_ITEM_POSITION_ERROR)
            )
        {
            if (this->bSmallIconsView) 
            {
                IconSize=16;
                BorderSecurity=5;
            }
            else // large icons
            {
                IconSize=32;
                BorderSecurity=10;
            }

            // we need to apply a ratio : target listview size / real desktop size
            // make ratio
            // warning DesktopRect.left and DesktopRect.top are not 0 on multiple screen configuration
            pt.x=((pt.x-DesktopRect.left)*(ListViewRect.right-ListViewRect.left))/(DesktopRect.right-DesktopRect.left);
            pt.y=((pt.y-DesktopRect.top)*(ListViewRect.bottom-ListViewRect.top))/(DesktopRect.bottom-DesktopRect.top);

            // assume item can be drawn in listview without scroll bars
            if (pt.x<BorderSecurity)
                pt.x=BorderSecurity;
            if (pt.x>ListViewRect.right-IconSize-BorderSecurity)
                pt.x=ListViewRect.right-IconSize-BorderSecurity;

            if (pt.y<BorderSecurity)
                pt.y=BorderSecurity;
            if (pt.y>ListViewRect.bottom-IconSize-BorderSecurity)
                pt.y=ListViewRect.bottom-IconSize-BorderSecurity;

            this->bIsSettingItemPosition=TRUE;

            // set item position
            ListView_SetItemPosition(this->hWndListView,
                                    Cnt,
                                    pt.x,
                                    pt.y
                                    );

            this->bIsSettingItemPosition=FALSE;
        }
    }
    ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    ::LeaveCriticalSection(&this->RealDesktopItemsInfosCriticalSection);

    ::RedrawWindow( this->hWndListView,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME | RDW_ALLCHILDREN);
    return TRUE;
}

HWND CDesktopBrowserXP::GetDesktopListView()
{
    if (this->hDesktopListView)
        return this->hDesktopListView;

    HWND hListView = NULL;

    // find well known window containing desktop
    HWND hWnd = ::FindWindow(_T("Progman"), _T("Program Manager"));//::FindWindow(_T("ProgMan"), NULL);

    // find shell view
    HWND hShellView = ::FindWindowEx(hWnd,NULL,_T("SHELLDLL_DefView"),NULL);

    // bug solved QuickWayToFolder 1.2.8
    // in case of use of direct X wallpaper _T("Progman") doesn't contain _T("SHELLDLL_DefView")
    if (hShellView==0)
    {
        // use of DirectX --> ::FindWindow(_T("WorkerW"), NULL); // trouble : multiple WorkerW
        HWND hWndDesktop = ::GetDesktopWindow();
        HWND hWndChild = 0;
        for(;;)
        {
            hWndChild = ::FindWindowEx( hWndDesktop, hWndChild,  _T("WorkerW"),  NULL );
            if (!hWndChild)
                goto CleanUp;
            hShellView = ::FindWindowEx(hWndChild,NULL,_T("SHELLDLL_DefView"),NULL);

            if (hShellView)
                break;
        }
    }

    // find associated list view control
    hListView = ::FindWindowEx(hShellView,NULL,WC_LISTVIEW,NULL);
    if (!::IsWindowVisible(hListView))
    {
        hListView = ::FindWindowEx(hShellView,NULL,CShellBrowser_ThumbnailVwExtWnd32,NULL);
        hListView = ::FindWindowEx(hListView,NULL,WC_LISTVIEW,NULL);
    }

CleanUp:
    this->hDesktopListView = hListView;

    return this->hDesktopListView;
}

//-----------------------------------------------------------------------------
// Name: GetDesktopItemsPos
//		/!\ Warning works only on correct identical OS architecture version LVITEM is platform dependent (32 and 64 bit structs are different)
// Object: get real desktop items positions
// Parameters :
//     in  :
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CDesktopBrowserXP::GetDesktopItemsPos()
{
    BOOL bRet=FALSE;
    HANDLE hProcess=NULL;
    REAL_DESKTOP_ITEM_INFOS* lpData = NULL;
    this->bUsePrimaryDisplayMonitorOnly=TRUE;

    ::EnterCriticalSection(&this->RealDesktopItemsInfosCriticalSection);
    this->RealDesktopItemsInfos.clear();

    HWND hListView = this->GetDesktopListView();

    DWORD dwProcessId;
    DWORD dwAccess;

    // get process associated to desktop
    ::GetWindowThreadProcessId(hListView, &dwProcessId);

    // open process with allocation, read and write memory access
    dwAccess = PROCESS_VM_OPERATION|PROCESS_VM_READ|PROCESS_VM_WRITE;
    hProcess = ::OpenProcess(dwAccess, FALSE, dwProcessId);
    if(!hProcess)
        goto CleanUp;

    // allocate required structs in remote process memory
    lpData =(REAL_DESKTOP_ITEM_INFOS*)::VirtualAllocEx(hProcess, NULL, sizeof(REAL_DESKTOP_ITEM_INFOS)+sizeof(LVITEM), MEM_COMMIT, PAGE_READWRITE);
    if(!lpData)
        goto CleanUp;

    // get pointer to 2nd struct
    LVITEM* lpItem=(LVITEM*) ( ((PBYTE)lpData) + sizeof(REAL_DESKTOP_ITEM_INFOS) );

    REAL_DESKTOP_ITEM_INFOS ItemInfos;

    // prepare an LVITEM struct
    LVITEM lvi;
    memset(&lvi, 0, sizeof(LVITEM));
    lvi.mask = LVIF_TEXT;
    lvi.pszText	= (LPTSTR) lpData->Name;
    lvi.cchTextMax = MAX_PATH;

    // get number of item on desktop
    int ItemCount =(int) ::SendMessage(hListView, LVM_GETITEMCOUNT, 0, 0);

    RECT PrimaryDesktopWorkArea;
    RECT AllDisplayWorkArea;
    CDesktopWorkArea::GetWorkAreaEx(&PrimaryDesktopWorkArea,&AllDisplayWorkArea);
    // put primary rect into relative coordinate of top left display
    ::OffsetRect(&PrimaryDesktopWorkArea,-AllDisplayWorkArea.left,-AllDisplayWorkArea.top);
    ::OffsetRect(&AllDisplayWorkArea,-AllDisplayWorkArea.left,-AllDisplayWorkArea.top);

    // for each item
    for(int Cnt = 0; Cnt < ItemCount; Cnt++)
    {
        // get item pos
        ItemInfos.Position.x=CDesktopBrowserXP_ITEM_POSITION_ERROR;
        ItemInfos.Position.y=CDesktopBrowserXP_ITEM_POSITION_ERROR;
        if (::SendMessage(hListView, LVM_GETITEMPOSITION, Cnt, (LPARAM) &lpData->Position))
        {
            // translate result from remote process memory to current process memory
            ::ReadProcessMemory(hProcess, &lpData->Position, &ItemInfos.Position, sizeof(POINT), NULL);

            if (!::PtInRect(&PrimaryDesktopWorkArea,ItemInfos.Position))
                this->bUsePrimaryDisplayMonitorOnly=FALSE;
        }

        // get item text
        lvi.iItem = Cnt;
        *ItemInfos.Name=0;
        // update remote LVITEM struct
        ::WriteProcessMemory(hProcess, lpItem, &lvi, sizeof(LVITEM), NULL);
        if (::SendMessage(hListView, LVM_GETITEMTEXT, Cnt, (LPARAM) lpItem) > 0)
        {
            // translate result from remote process memory to current process memory
            ::ReadProcessMemory(hProcess, &lpData->Name, &ItemInfos.Name, MAX_PATH*sizeof(TCHAR), NULL);
        }

        // store item text and position into RealDesktopItemsInfos vector
        this->RealDesktopItemsInfos.push_back(ItemInfos);
    }

    bRet=TRUE;

CleanUp:
    ::LeaveCriticalSection(&this->RealDesktopItemsInfosCriticalSection);

    // free memory 

    if (lpData)
        ::VirtualFreeEx(hProcess, lpData, 0, MEM_RELEASE);
    if (hProcess)
        ::CloseHandle(hProcess);

    return bRet;
}

//-----------------------------------------------------------------------------
// Name: InternalBeforeBrowseCallBackVirtual
// Object: method called before browsing a new folder
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowserXP::InternalBeforeBrowseCallBackVirtual(LPCITEMIDLIST pidlToBeBrowsed)
{
    this->InternalBeforeBrowseCallBack(pidlToBeBrowsed);
}

//-----------------------------------------------------------------------------
// Name: InternalBeforeBrowseCallBack
// Object: method called before browsing a new folder
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowserXP::InternalBeforeBrowseCallBack(LPCITEMIDLIST pidlToBeBrowsed)
{
    this->CShellBrowserEx::InternalBeforeBrowseCallBack(pidlToBeBrowsed);

    if (this->bDesktopLikeView && this->bCurrentViewIsDesktopView)
    {
        // this->RestoreItemsText(); // must be called else sorting troubles occurs when browsing back to desktop
        // to be cleaner, we fully remove subclassing and restore items text
        this->RemoveSubProc();
        
    }

    // check if next folder will be the desktop one
    if (this->IsDesktopFolder(pidlToBeBrowsed))
    {
        this->bDesktopLikeView=TRUE;
        this->bCurrentViewIsDesktopView=TRUE;
        this->bSmallIconsView=this->DoesSmallViewBestFit();
        // as view can be changed do not use FWF_DESKTOP or FWF_NOSCROLL style
        // this->FolderSettings.fFlags|=FWF_NOWEBVIEW | FWF_DESKTOP | FWF_TRANSPARENT;
    }
    else
    {
        this->bDesktopLikeView=FALSE;
        this->bCurrentViewIsDesktopView=FALSE;
        // as view can be changed do not use FWF_DESKTOP or FWF_NOSCROLL style
        // this->FolderSettings.fFlags&=~(FWF_NOWEBVIEW | FWF_DESKTOP | FWF_TRANSPARENT);
    }
}

//-----------------------------------------------------------------------------
// Name: InternalAfterViewChangedCallBackVirtual
// Object: method called after folder view changed
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowserXP::InternalAfterViewChangedCallBackVirtual(tagShellBrowserView NewShellBrowserView)
{
    this->InternalAfterViewChangedCallBack(NewShellBrowserView);
}

//-----------------------------------------------------------------------------
// Name: InternalAfterViewChangedCallBack
// Object: method called after folder view changed
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDesktopBrowserXP::InternalAfterViewChangedCallBack(tagShellBrowserView NewShellBrowserView)
{
    // call parent object (CShellBrowserEx) method
    this->CShellBrowserEx::InternalAfterViewChangedCallBack(NewShellBrowserView);
    
    if (!this->bSettingViewInternally)
    {
        this->PreviousView=NewShellBrowserView;
    }

    this->RemoveSubProc();
}


//-----------------------------------------------------------------------------
// Name: SubClassProcListViewToolTip
// Object: listview tooltip subclassing proc : trash undesirable messages
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CDesktopBrowserXP::SubClassProcListViewToolTip(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData)
{
    UNREFERENCED_PARAMETER(uIdSubclass);
    UNREFERENCED_PARAMETER(dwRefData);
    // CDesktopBrowserXP* pDesktopBrowser=(CDesktopBrowserXP*)dwRefData;

    switch (uMsg)
    {
    case TTM_SETMARGIN:
        // shell view set tooltip margins to 4,4 --> avoid this for small icon view
        return TRUE;
    }
    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
}

void CDesktopBrowserXP::AddItemA(int ItemIndex,CHAR* Text)
{
    LOCAL_VIEW_DESKTOP_ITEM_INFOS Item;
    *Item.ItemText=0;
    if (Text!=LPSTR_TEXTCALLBACKA)
    {
        if (!::IsBadReadPtr (Text,sizeof(CHAR)))
        {
#if ( defined(UNICODE) || defined(_UNICODE))
            CAnsiUnicodeConvert::AnsiToUnicode(Text,Item.ItemText,MAX_PATH);
#else
            strncpy(Item.ItemText,Text,MAX_PATH);
#endif
            Item.ItemText[MAX_PATH-1]=0;
        }
    }
    if (ItemIndex>=0)
    {
        ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
        if ( (ItemIndex==CDesktopBrowserXP_LISTVIEW_APPEND) || (this->LocalViewDesktopItemsInfos.size()==0) )
            this->LocalViewDesktopItemsInfos.push_back(Item);
        else
            this->LocalViewDesktopItemsInfos.insert(this->LocalViewDesktopItemsInfos.begin()+ItemIndex,Item);
        ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
        CHAR Sz[MAX_PATH];
        if (Text==LPSTR_TEXTCALLBACKA)
            sprintf(Sz,"Item inserted with Text callback\r\n");
        else
            sprintf(Sz,"Item inserted %s\r\n",Text);
        OutputDebugStringA(Sz);
#endif
#endif
    }
}

void CDesktopBrowserXP::AddItemW(int ItemIndex,WCHAR* Text)
{
    LOCAL_VIEW_DESKTOP_ITEM_INFOS Item;
    *Item.ItemText=0;
    if (Text!=LPSTR_TEXTCALLBACKW)
    {
        if (!::IsBadReadPtr (Text,sizeof(WCHAR)))
        {
#if ( defined(UNICODE) || defined(_UNICODE))
            wcsncpy(Item.ItemText,Text,MAX_PATH);
#else
            CAnsiUnicodeConvert::UnicodeToAnsi(Text,Item.ItemText,MAX_PATH);
#endif
            Item.ItemText[MAX_PATH-1]=0;
        }
    }

    if (ItemIndex>=0)
    {
        ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
        if ( (ItemIndex==CDesktopBrowserXP_LISTVIEW_APPEND) || (this->LocalViewDesktopItemsInfos.size()==0) )
            this->LocalViewDesktopItemsInfos.push_back(Item);
        else
            this->LocalViewDesktopItemsInfos.insert(this->LocalViewDesktopItemsInfos.begin()+ItemIndex,Item);
        ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
        WCHAR Wsz[MAX_PATH];
        if (Text==LPSTR_TEXTCALLBACKW)
            swprintf(Wsz,_countof(Wsz),L"Item inserted with Text callback\r\n");
        else
            swprintf(Wsz,_countof(Wsz),L"Item inserted %s\r\n",Text);
        OutputDebugStringW(Wsz);
#endif
#endif
    }
}
void CDesktopBrowserXP::ReplaceItemA(int ItemIndex,CHAR* Text)
{
    ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    if (this->LocalViewDesktopItemsInfos.size()<=(size_t)ItemIndex)
    {
#ifdef _DEBUG
        if (::IsDebuggerPresent())
            ::DebugBreak();
#endif
        ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
        return;
    }

    if (Text!=LPSTR_TEXTCALLBACKA)
    {
        if (!::IsBadReadPtr (Text,sizeof(CHAR)))
        {
            if (*Text!='\0')
            {
#if ( defined(UNICODE) || defined(_UNICODE))
                CAnsiUnicodeConvert::AnsiToUnicode(Text,this->LocalViewDesktopItemsInfos[ItemIndex].ItemText,MAX_PATH);
#else
                strncpy(this->LocalViewDesktopItemsInfos[ItemIndex].ItemText,Text,MAX_PATH);
#endif
                this->LocalViewDesktopItemsInfos[ItemIndex].ItemText[MAX_PATH-1]=0;

#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                CHAR Sz[MAX_PATH];
                sprintf(Sz,"Item %d replaced with %s\r\n",ItemIndex,Text);
                OutputDebugStringA(Sz);
#endif
#endif
            }
        }
    }
    ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
}
void CDesktopBrowserXP::ReplaceItemW(int ItemIndex,WCHAR* Text)
{
    ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    if (this->LocalViewDesktopItemsInfos.size()<=(size_t)ItemIndex)
    {
#ifdef _DEBUG
        if (::IsDebuggerPresent())
            ::DebugBreak();
#endif
        ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
        return;
    }

    if (Text!=LPSTR_TEXTCALLBACKW)
    {
        if (!::IsBadReadPtr (Text,sizeof(WCHAR)))
        {
            if (*Text!='\0')
            {
#if ( defined(UNICODE) || defined(_UNICODE))
                wcsncpy(this->LocalViewDesktopItemsInfos[ItemIndex].ItemText,Text,MAX_PATH);
#else
                CAnsiUnicodeConvert::UnicodeToAnsi(Text,this->LocalViewDesktopItemsInfos[ItemIndex].ItemText,MAX_PATH);
#endif
                this->LocalViewDesktopItemsInfos[ItemIndex].ItemText[MAX_PATH-1]=0;

#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                WCHAR Wsz[MAX_PATH];
                swprintf(Wsz,_countof(Wsz),L"Item %d replaced with %s\r\n",ItemIndex,Text);
                OutputDebugStringW(Wsz);
#endif
#endif
            }
        }
    }
    ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
}

// in case of IShellView::Refresh, messages order is
//LVM_DELETEALLITEMS
//LVM_INSERTITEM (icons only with LPSTR_TEXTCALLBACK flag)
//LVM_SORTITEMS/LVM_SORTITEMSEX
//LVN_GETDISPINFO (apply text only after sorting)
// and as SetItemsPosLikeDesktop needs item text, we should call it only at the end of refreshing
void CDesktopBrowserXP::RestoreItemPositionAtEndOfRefresh()
{
    // if no sort item message sent during refresh
    if (!this->bSortItemNeeded)
        return;

    // if all item text are filled
    ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    for (size_t Cnt=0;Cnt<this->LocalViewDesktopItemsInfos.size();Cnt++)
    {
        // if item text is empty
        if (this->LocalViewDesktopItemsInfos[Cnt].ItemText[0]=='\0')
        {
            ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
            return;
        }
    }
    ::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);

    this->bSortItemNeeded=FALSE;
    // don't car about wParam and lParam until we catch LVM_SORTITEMS messages
    // we MUST use PostMessage
    ::PostMessage(this->hWndListView,LVM_SORTITEMS,0,(LPARAM)CDesktopBrowserXP::CompareFunc);// provide a valid compare func in the case of subclassing is removed and message is not treated
}

int CALLBACK CDesktopBrowserXP::CompareFunc(LPARAM lParam1, LPARAM lParam2,LPARAM lParamSort)
{
    // should never be called
    UNREFERENCED_PARAMETER(lParam1);
    UNREFERENCED_PARAMETER(lParam2);
    UNREFERENCED_PARAMETER(lParamSort);
    return 0;
}

//-----------------------------------------------------------------------------
// Name: SubClassProcListView
// Object: listview subclassing proc : 
//          for small icon view, as we removed text and use tooltip to display items name,
//          we need to spy item addition, removal and editing to keep up to date informations
//		/!\ Warning works only on correct identical OS architecture version LVITEM (and associated structs) are platform dependent (32 and 64 bit structs are different)
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CDesktopBrowserXP::SubClassProcListView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData)
{
    UNREFERENCED_PARAMETER(uIdSubclass);
    CDesktopBrowserXP* pDesktopBrowser=(CDesktopBrowserXP*)dwRefData;

    switch (uMsg)
    {
    case LVM_ARRANGE:
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
        OutputDebugString(_T("LVM_ARRANGE\r\n"));
#endif
#endif
        pDesktopBrowser->SetItemsPosLikeDesktop(TRUE);
        return TRUE;

    case LVM_SORTITEMS:
    case LVM_SORTITEMSEX:
        {
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
            OutputDebugString(_T("LVM_SORTITEMS/LVM_SORTITEMSEX\r\n"));
#endif
#endif
            // in case of IShellView::Refresh, messages order is
            //LVM_DELETEALLITEMS
            //LVM_INSERTITEM (icons only with LPSTR_TEXTCALLBACK flag)
            //LVM_SORTITEMS/LVM_SORTITEMSEX
            //LVN_GETDISPINFO (apply text only after sorting)
            // and as SetItemsPosLikeDesktop needs item text, we should call it only at the end of refreshing

            if (!pDesktopBrowser->bSmallIconsView)
            {
                pDesktopBrowser->SetItemsPosLikeDesktop(TRUE);
                return TRUE;
            }

            BOOL bAllItemsFilled=TRUE;
            // if all item text are filled
            ::EnterCriticalSection(&pDesktopBrowser->LocalViewDesktopItemsInfosCriticalSection);
            for (size_t Cnt=0;Cnt<pDesktopBrowser->LocalViewDesktopItemsInfos.size();Cnt++)
            {
                // if item text is empty
                if (pDesktopBrowser->LocalViewDesktopItemsInfos[Cnt].ItemText[0]=='\0')
                {
                    bAllItemsFilled=FALSE;
                    break;
                }
            }
            ::LeaveCriticalSection(&pDesktopBrowser->LocalViewDesktopItemsInfosCriticalSection);
            if (bAllItemsFilled)
                pDesktopBrowser->SetItemsPosLikeDesktop(TRUE);
            else
                pDesktopBrowser->bSortItemNeeded=TRUE;

            return TRUE;
        }
    case LVM_UPDATE:
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
        OutputDebugString(_T("LVM_UPDATE\r\n"));
#endif
#endif
        return TRUE;

    case LVM_REDRAWITEMS:
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
        OutputDebugString(_T("LVM_REDRAWITEMS\r\n"));
#endif
#endif
        return TRUE;

    case LVM_SETITEMPOSITION:
    case LVM_SETITEMPOSITION32:
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
        OutputDebugString(_T("LVM_SETITEMPOSITION / LVM_SETITEMPOSITION32\r\n"));
#endif
#endif
        // allow item position changes only for our internal calls
        if (!pDesktopBrowser->IsSettingItemPosition())
            return TRUE;
        break;

    case LVM_DELETEITEM:
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
        OutputDebugString(_T("LVM_DELETEITEM\r\n"));
#endif
#endif
        // in case of large icons, no change is done on text --> do nothing
        if (!pDesktopBrowser->bSmallIconsView)
            break;

        if (wParam>=0)
        {
            ::EnterCriticalSection(&pDesktopBrowser->LocalViewDesktopItemsInfosCriticalSection);
            if (pDesktopBrowser->LocalViewDesktopItemsInfos.size()!=0)
                pDesktopBrowser->LocalViewDesktopItemsInfos.erase(pDesktopBrowser->LocalViewDesktopItemsInfos.begin()+wParam);
            ::LeaveCriticalSection(&pDesktopBrowser->LocalViewDesktopItemsInfosCriticalSection);
        }

        break;
    case LVM_DELETEALLITEMS:
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
        OutputDebugString(_T("LVM_DELETEALLITEMS\r\n"));
#endif
#endif
        // in case of large icons, no change is done on text --> do nothing
        if (!pDesktopBrowser->bSmallIconsView)
            break;
        ::EnterCriticalSection(&pDesktopBrowser->LocalViewDesktopItemsInfosCriticalSection);
        pDesktopBrowser->LocalViewDesktopItemsInfos.clear();
        ::LeaveCriticalSection(&pDesktopBrowser->LocalViewDesktopItemsInfosCriticalSection);
        break;

    case LVM_INSERTITEMA:
        {
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
            OutputDebugString(_T("LVM_INSERTITEMA\r\n"));
#endif
#endif

            // in case of large icons, no change is done on text --> do nothing
            if (!pDesktopBrowser->bSmallIconsView)
                break;

            LPLVITEMA pItem = (LPLVITEMA)lParam;
            // if no text information
            if ( (pItem->mask & LVIF_TEXT) == NULL)
                pItem->pszText=NULL;

            if (pItem->pszText!=LPSTR_TEXTCALLBACKA)// if (pItem->pszText==LPSTR_TEXTCALLBACKA), let LVN_GETDISPINFOA do the job for us
                pItem->mask&=~LVIF_TEXT;

            pDesktopBrowser->AddItemA(pItem->iItem,pItem->pszText);
            pDesktopBrowser->bSortItemNeeded=TRUE;
        }
        break;

    case LVM_INSERTITEMW:
        {
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
            OutputDebugString(_T("LVM_INSERTITEMW\r\n"));
#endif
#endif
            // in case of large icons, no change is done on text --> do nothing
            if (!pDesktopBrowser->bSmallIconsView)
                break;

            LPLVITEMW pItem = (LPLVITEMW)lParam;
            // if no text information
            if ( (pItem->mask & LVIF_TEXT) == NULL)
                pItem->pszText=NULL;

            if (pItem->pszText!=LPSTR_TEXTCALLBACKW)// if (pItem->pszText==LPSTR_TEXTCALLBACKW), let LVN_GETDISPINFOW do the job for us
                pItem->mask&=~LVIF_TEXT;
            
            pDesktopBrowser->AddItemW(pItem->iItem,pItem->pszText);
            pDesktopBrowser->bSortItemNeeded=TRUE;
        }
        break;
    
    case LVM_SETITEMA:
        {
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
            OutputDebugString(_T("LVM_SETITEMA\r\n"));
#endif
#endif

            // in case of large icons, no change is done on text --> do nothing
            if (!pDesktopBrowser->bSmallIconsView)
                break;

            LPLVITEMA pItem = (LPLVITEMA)lParam;
            // if no text change
            if ( (pItem->mask & LVIF_TEXT) == NULL)
            {
                // break
                break;
            }

            // else 
            if (pItem->pszText!=LPSTR_TEXTCALLBACKA)// if (pItem->pszText==LPSTR_TEXTCALLBACKA), let LVN_GETDISPINFOA do the job for us
            {
                pItem->mask&=~LVIF_TEXT;
                pDesktopBrowser->ReplaceItemA((int)wParam,pItem->pszText);
            }
            break;
        }
    case LVM_SETITEMTEXTA:
        {
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
            OutputDebugString(_T("LVM_SETITEMTEXTA\r\n"));
#endif
#endif
            // in case of large icons, no change is done on text --> do nothing
            if (!pDesktopBrowser->bSmallIconsView)
                break;

            LPLVITEMA pItem = (LPLVITEMA)lParam;
            pDesktopBrowser->ReplaceItemA((int)wParam,pItem->pszText);
        }
        // LVM_SETITEMTEXT mustn't insert text so return without calling DefSubclassProc
        return TRUE;

    case LVM_SETITEMW:
        {
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
            OutputDebugString(_T("LVM_SETITEMW\r\n"));
#endif
#endif

            // in case of large icons, no change is done on text --> do nothing
            if (!pDesktopBrowser->bSmallIconsView)
                break;

            LPLVITEMW pItem = (LPLVITEMW)lParam;
            // if no text change
            if ( (pItem->mask & LVIF_TEXT) == NULL)
            {
                // break
                break;
            }
            // else 
            if (pItem->pszText!=LPSTR_TEXTCALLBACKW)// if (pItem->pszText==LPSTR_TEXTCALLBACKW), let LVN_GETDISPINFOW do the job for us
            {
                pItem->mask&=~LVIF_TEXT;
                pDesktopBrowser->ReplaceItemW((int)wParam,pItem->pszText);
            }
            break;
        }
    case LVM_SETITEMTEXTW:
        {
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
            OutputDebugString(_T("LVM_SETITEMTEXTW\r\n"));
#endif
#endif
            // in case of large icons, no change is done on text --> do nothing
            if (!pDesktopBrowser->bSmallIconsView)
                break;

            LPLVITEMW pItem = (LPLVITEMW)lParam;
            pDesktopBrowser->ReplaceItemW((int)wParam,pItem->pszText);

        }
        // LVM_SETITEMTEXT musn't insert text so return without calling DefSubclassProc
        return TRUE;
    }
    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
}

//-----------------------------------------------------------------------------
// Name: SubClassProcShellView
// Object: shellview subclassing proc : 
//         - change info tip values (display item name instead of informations)
//         - spy LVN_BEGINLABELEDIT and LVN_ENDLABELEDIT to keep our info tips up to date
//		/!\ Warning works only on correct identical OS architecture version LVITEM (and associated structs) are platform dependent (32 and 64 bit structs are different)
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CDesktopBrowserXP::SubClassProcShellView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData)
{
    UNREFERENCED_PARAMETER(uIdSubclass);
    CDesktopBrowserXP* pDesktopBrowser=(CDesktopBrowserXP*)dwRefData;

    switch(uMsg)
    {
    case WM_NOTIFY:
        {
            NMHDR* pnmh=((NMHDR*)lParam);

            // listview messages
            if (pnmh->hwndFrom==pDesktopBrowser->hWndListView)
            {
                LRESULT lResult;
                switch (pnmh->code) 
                {
                // if no information returned, only default tooltip is used
                // Sent by a large icon view list-view control that has the LVS_EX_INFOTIP extended style.
                // This notification is sent when the list-view control is requesting additional text information to be displayed in a ToolTip
                case LVN_GETINFOTIPA    :
                    {
                        NMLVGETINFOTIPA* pInfoTip=(LPNMLVGETINFOTIPA)lParam;
                        size_t Min=__min(MAX_PATH,pInfoTip->cchTextMax);
                        ::EnterCriticalSection(&pDesktopBrowser->LocalViewDesktopItemsInfosCriticalSection);
                        if (pDesktopBrowser->LocalViewDesktopItemsInfos.size()<=(size_t)pInfoTip->iItem)
                        {
#ifdef _DEBUG
                            if (::IsDebuggerPresent())
                                ::DebugBreak();
#endif
                            ::LeaveCriticalSection(&pDesktopBrowser->LocalViewDesktopItemsInfosCriticalSection);
                            break;
                        }

                        if (pInfoTip->cchTextMax>0)
                        {
#ifdef _DEBUG
                            if (pDesktopBrowser->LocalViewDesktopItemsInfos[pInfoTip->iItem].ItemText[0]==0)
                            {
                                pDesktopBrowser->DisplayLocalViewDesktopItemsInfos();
                                if (::IsDebuggerPresent())
                                    ::DebugBreak();
                            }
#endif

#if ( defined(UNICODE) || defined(_UNICODE))
                            CAnsiUnicodeConvert::UnicodeToAnsi(pDesktopBrowser->LocalViewDesktopItemsInfos[pInfoTip->iItem].ItemText,pInfoTip->pszText,Min);
#else
                            strncpy(pInfoTip->pszText,pDesktopBrowser->LocalViewDesktopItemsInfos[pInfoTip->iItem].ItemText,Min);
#endif
                            pInfoTip->pszText[pInfoTip->cchTextMax-1]=0;
                        }
                        ::LeaveCriticalSection(&pDesktopBrowser->LocalViewDesktopItemsInfosCriticalSection);
                    }
                    return TRUE;
                case LVN_GETINFOTIPW    :
                    {
                        NMLVGETINFOTIPW* pInfoTip=(LPNMLVGETINFOTIPW)lParam;
                        ::EnterCriticalSection(&pDesktopBrowser->LocalViewDesktopItemsInfosCriticalSection);
                        if (pDesktopBrowser->LocalViewDesktopItemsInfos.size()<=(size_t)pInfoTip->iItem)
                        {
#ifdef _DEBUG
                            if (::IsDebuggerPresent())
                                ::DebugBreak();
#endif
                            ::LeaveCriticalSection(&pDesktopBrowser->LocalViewDesktopItemsInfosCriticalSection);
                            break;
                        }

                        int Min=__min(MAX_PATH,pInfoTip->cchTextMax);

                        if (pInfoTip->cchTextMax>0)
                        {
#ifdef _DEBUG
                            if (pDesktopBrowser->LocalViewDesktopItemsInfos[pInfoTip->iItem].ItemText[0]==0)
                            {
                                pDesktopBrowser->DisplayLocalViewDesktopItemsInfos();
                                if (::IsDebuggerPresent())
                                    ::DebugBreak();
                            }
#endif


#if ( defined(UNICODE) || defined(_UNICODE))
                            wcsncpy(pInfoTip->pszText,pDesktopBrowser->LocalViewDesktopItemsInfos[pInfoTip->iItem].ItemText,Min);
#else
                            CAnsiUnicodeConvert::AnsiToUnicode(pDesktopBrowser->LocalViewDesktopItemsInfos[pInfoTip->iItem].ItemText,pInfoTip->pszText,Min);
#endif
                            pInfoTip->pszText[pInfoTip->cchTextMax-1]=0;
                        }
                        ::LeaveCriticalSection(&pDesktopBrowser->LocalViewDesktopItemsInfosCriticalSection);
                    }
                    return TRUE;

                // To allow the user to edit the label, return FALSE.
                // To prevent the user from editing the label, return TRUE.
                case LVN_BEGINLABELEDITA:
                case LVN_BEGINLABELEDITW:
                    {
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                        OutputDebugString(_T("LVN_BEGINLABELEDIT\r\n"));
#endif
#endif
                        // hWndEdit already has good informations (thanks to ???)

//                        NMLVDISPINFO* pDispInfo=(NMLVDISPINFO*)lParam;
//                        LOCAL_VIEW_DESKTOP_ITEM_INFOS* pItemInfos=&pDesktopBrowser->LocalViewDesktopItemsInfos[pDispInfo->item.iItem];
//                        if (pDesktopBrowser->LocalViewDesktopItemsInfos.size()<=(size_t)pDispInfo->item.iItem)
//                        {
//#ifdef _DEBUG
//                            pDesktopBrowser->DisplayLocalViewDesktopItemsInfos();
//                            if (::IsDebuggerPresent())
//                                ::DebugBreak();
//#endif
//                            break;
//                        }
//                        lResult = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
//                        HWND hWndEdit=ListView_GetEditControl(pnmh->hwndFrom);
//                        if (pItemInfos->ItemText[0]==0)
//                        {
//                            // update ItemInfos.ItemText if it is empty
//                            ::GetWindowText(hWndEdit,pItemInfos->ItemText,MAX_PATH);
//                        }
//                        ::SetWindowText(hWndEdit,pDesktopBrowser->LocalViewDesktopItemsInfos[pDispInfo->item.iItem].ItemText);
//                        return lResult;
                    }
                    break;

                // If the pszText member of the LVITEM structure is non-NULL, return TRUE to set the item's label to the edited text. Return FALSE to reject the edited text and revert to the original label. 
                // If the pszText member of the LVITEM structure is NULL, the return value is ignored.
                case LVN_ENDLABELEDITA  :
                    {
                        // If the user cancels editing, the pszText member of the LVITEM structure is NULL; otherwise, pszText is the address of the edited text
                        NMLVDISPINFOA* pDispInfo=(NMLVDISPINFOA*)lParam;

                        lResult = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                        CHAR Sz[MAX_PATH];
                        sprintf(Sz,"LVN_ENDLABELEDITA return value %d, text %p\r\n",lResult,pDispInfo->item.pszText);
                        OutputDebugStringA(Sz);
#endif
#endif
                        if (
                            // (lResult==TRUE) && // only if name changes have been accepted (MSDN seems to be wrong : change is accepted even if lResult=FALSE)
                            (pDispInfo->item.pszText!=NULL)
                            )
                        {
                            pDesktopBrowser->ReplaceItemA(pDispInfo->item.iItem,pDispInfo->item.pszText);
                        }
                        return lResult;
                    }
                    break;
                case LVN_ENDLABELEDITW  :
                    {
                        NMLVDISPINFOW* pDispInfo=(NMLVDISPINFOW*)lParam;
                        lResult = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                        WCHAR Sz[MAX_PATH];
                        swprintf(Sz,_countof(Sz),L"LVN_ENDLABELEDITW return value %d, text %p\r\n",lResult,pDispInfo->item.pszText);
                        OutputDebugStringW(Sz);
#endif
#endif
                        if (
                            // (lResult==TRUE) && // only if name changes have been accepted (MSDN seems to be wrong : change is accepted even if lResult=FALSE)
                            (pDispInfo->item.pszText!=NULL)
                            )
                        {
                            // If the user cancels editing, the pszText member of the LVITEM structure is NULL; otherwise, pszText is the address of the edited text
                            pDesktopBrowser->ReplaceItemW(pDispInfo->item.iItem,pDispInfo->item.pszText);
                        }
                        return lResult;
                    }
                    break;

                case LVN_GETDISPINFOA:
                    {
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                        OutputDebugString(_T("LVN_GETDISPINFOA\r\n"));
#endif
#endif
                        NMLVDISPINFOA* pDispInfos=(NMLVDISPINFOA*)lParam;
                        if (pDispInfos->item.mask & LVIF_TEXT)
                        {
                            // let original wnd proc do it's job
                            lResult = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
                            // store text
                            pDesktopBrowser->ReplaceItemA(pDispInfos->item.iItem,pDispInfos->item.pszText);

                            // restore item position at the end of refresh
                            pDesktopBrowser->RestoreItemPositionAtEndOfRefresh();

                            // return an empty text
                            pDispInfos->item.pszText="";
                            
                            return lResult;
                        }
                    }

                    break;
                case LVN_GETDISPINFOW:
                    {
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
                        OutputDebugString(_T("LVN_GETDISPINFOW\r\n"));
#endif
#endif
                        NMLVDISPINFOW* pDispInfos=(NMLVDISPINFOW*)lParam;
                        if (pDispInfos->item.mask & LVIF_TEXT)
                        {
                            // let original wnd proc do it's job
                            lResult = ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
                            // store text
                            pDesktopBrowser->ReplaceItemW(pDispInfos->item.iItem,pDispInfos->item.pszText);

                            // restore item position at the end of refresh
                            pDesktopBrowser->RestoreItemPositionAtEndOfRefresh();

                            // return an empty text
                            pDispInfos->item.pszText=L"";

                            return lResult;
                        }
                    }
                    break;
                }
            }
        }
        break;
#ifdef _DEBUG
#if (OUTPUT_DEBUG_STRING_INFOS == 1)
    default:
        {
            if (uMsg>WM_USER)
            {
                TCHAR sz[MAX_PATH];
                _sntprintf(sz,_countof(sz),_T("CDesktopBrowserXP::SubClassProcShellView WM_USER+0x%x\r\n"),uMsg-WM_USER);
                OutputDebugString(sz);
            }
        }
        break;
#endif
#endif
    }
    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
}

#ifdef _DEBUG
// #define DisplayLocalViewDesktopItemsInfos_TRACE 1
#define DisplayLocalViewDesktopItemsInfos_TRACE 0
void CDesktopBrowserXP::DisplayLocalViewDesktopItemsInfos()
{
#if (DisplayLocalViewDesktopItemsInfos_TRACE==1)
    TCHAR sz[MAX_PATH];
    // don't use critical section caller must already be in critical section--> avoid deadlock or chances with LeaveCriticalSection/EnterCriticalSection sequences
    // ::EnterCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
    for (size_t Cnt=0;Cnt<this->LocalViewDesktopItemsInfos.size();Cnt++)
    {
        _sntprintf(sz,_countof(sz),_T("%s\r\n"),this->LocalViewDesktopItemsInfos[Cnt].ItemText);
        OutputDebugString(sz);
    }
    //::LeaveCriticalSection(&this->LocalViewDesktopItemsInfosCriticalSection);
#endif
}
#endif