/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manage a shell browser with extended features
//-----------------------------------------------------------------------------

#include "shellbrowser.h"
#include <crtdbg.h>// for _ASSERT

#include "../../Version/OsVersion.h"

CShellBrowser::CShellBrowser(HWND hWndParent)
{
    FOLDERSETTINGS fs = { FVM_ICON, 0 };
    tagShellBrowserView ShellBrowserView;
    CShellBrowser::FolderSettingToShellBrowserView(&fs,&ShellBrowserView);
    this->CommonConstructor(hWndParent,&fs,ShellBrowserView);
}
CShellBrowser::CShellBrowser(HWND hWndParent,FOLDERSETTINGS* pfs)
{
    tagShellBrowserView ShellBrowserView;
    CShellBrowser::FolderSettingToShellBrowserView(pfs,&ShellBrowserView);
    this->CommonConstructor(hWndParent,pfs,ShellBrowserView);
}

CShellBrowser::CShellBrowser(HWND hWndParent,FOLDERSETTINGS* pfs,RECT* pBrowserRect)
{
    tagShellBrowserView ShellBrowserView;
    CShellBrowser::FolderSettingToShellBrowserView(pfs,&ShellBrowserView);
    this->CommonConstructor(hWndParent,pfs,ShellBrowserView);
    memcpy(&this->BrowserRect,pBrowserRect,sizeof(RECT));
}
CShellBrowser::CShellBrowser(HWND hWndParent,FOLDERSETTINGS* pfs,tagShellBrowserView ShellBrowserView)
{
    this->CommonConstructor(hWndParent,pfs,ShellBrowserView);
}
CShellBrowser::CShellBrowser(HWND hWndParent,FOLDERSETTINGS* pfs,tagShellBrowserView ShellBrowserView,RECT* pBrowserRect)
{
    this->CommonConstructor(hWndParent,pfs,ShellBrowserView);
    memcpy(&this->BrowserRect,pBrowserRect,sizeof(RECT));
}

//-----------------------------------------------------------------------------
// Name: CommonConstructor
// Object: CommonConstructor.
// Parameters :
//     in  : HWND hWndParent : parent window handle
//           FOLDERSETTINGS* pfs : folder settings
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CShellBrowser::CommonConstructor(HWND hWndParent,FOLDERSETTINGS* pfs,tagShellBrowserView ShellBrowserView)
{
    // XP
    // messages send to Shell_DefView window throw WPARAM of WM_COMMAND 
    //enum tagShellBrowserView for XP
    //{
    //    SHELL_BROWSER_VIEW_ICONS=0x7029,
    //    SHELL_BROWSER_VIEW_FIRST=SHELL_BROWSER_VIEW_ICONS,
    //    SHELL_BROWSER_VIEW_SMALL_ICONS=0x702A,
    //    SHELL_BROWSER_VIEW_LIST=0x702B,
    //    SHELL_BROWSER_VIEW_DETAIL=0x702C,
    //    SHELL_BROWSER_VIEW_THUMBNAILS=0x702D,
    //    SHELL_BROWSER_VIEW_TILES=0x702E,
    //    SHELL_BROWSER_VIEW_THUMBSTRIP=0x702F
    //};

    // seven
    // messages send to Shell_DefView window throw WPARAM of WM_COMMAND 
    //704b details
    //704d extra large
    //704c tiles
    //704e medium icons
    //704f large
    //7050 small
    //7051 list
    //7071 autoarrange enable/disable
    //7074 align to grid enable / disable
    //
    //7600 group by size ascending
    //
    //7603 group by name descending 
    //7604 group by date descending 
    //7605 group by type descending 
    //7606 group by size descending 
    //
    //7b04 sort by name
    //enum tagShellBrowserView // for seven
    //{
    //    SHELL_BROWSER_VIEW_ICONS=0x7029,
    //    SHELL_BROWSER_VIEW_FIRST=SHELL_BROWSER_VIEW_ICONS,
    //    SHELL_BROWSER_VIEW_SMALL_ICONS=0x7050,
    //    SHELL_BROWSER_VIEW_LIST=0x7051,
    //    SHELL_BROWSER_VIEW_DETAIL=0x704B,
    //    SHELL_BROWSER_VIEW_EXTRA_LARGE = 0x704D, // TODO
    //    SHELL_BROWSER_VIEW_MEDIUM = 0x704E, // TODO
    //    SHELL_BROWSER_VIEW_LARGE = 0x704F, // TODO
    //    SHELL_BROWSER_VIEW_TILES=0x704C,

    //    SHELL_BROWSER_VIEW_THUMBNAILS=???, // to check on 7
    //    SHELL_BROWSER_VIEW_THUMBSTRIP=??? // to check on 7
    //};

    COsVersion OSVersion;
    if (OSVersion.IsVistaOrNewer())
    {
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_ICONS].Value=0x7029;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_FIRST].Value=this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_ICONS].Value;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_SMALL_ICONS].Value=0x7050;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_LIST].Value=0x7051;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_DETAIL].Value=0x704B;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_EXTRA_LARGE ].Value= 0x704D;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_MEDIUM ].Value= 0x704E;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_LARGE ].Value= 0x704F;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_TILES].Value=0x704C;

        // undefined view type for multiple os code compatibility
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_THUMBNAILS].Value=(SIZE_T)-1;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_THUMBSTRIP].Value=(SIZE_T)-1;
    }
    else // xp
    {
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_ICONS].Value=0x7029;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_FIRST].Value=this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_ICONS].Value;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_SMALL_ICONS].Value=0x702A;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_LIST].Value=0x702B;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_DETAIL].Value=0x702C;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_THUMBNAILS].Value=0x702D;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_TILES].Value=0x702E;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_THUMBSTRIP].Value=0x702F;

        // undefined view type for multiple os code compatibility
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_EXTRA_LARGE ].Value= (SIZE_T)-1;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_MEDIUM ].Value= (SIZE_T)-1;
        this->ShellBrowserViewInfo[SHELL_BROWSER_VIEW_LARGE ].Value= (SIZE_T)-1;
    }

    this->hWndParent = hWndParent;
    memcpy(&this->FolderSettings,pfs,sizeof(FOLDERSETTINGS));
    this->ShellBrowserView = ShellBrowserView;

    this->bRecreatingCurrentView = FALSE;
    this->RefCount=1; // as we don't use class factory, this must be set to 1
    this->pShellFolder = NULL;
    this->FolderPidl = NULL;
    this->pShellView = NULL;
    this->hWndShellView = NULL;
	this->hWndListView = NULL;
    this->hItemsMem=NULL;
    this->bBrowseInsideView=TRUE;
    this->CF_IDLIST = ::RegisterClipboardFormat(CFSTR_SHELLIDLIST);
    this->HistoryIndex=-1;// set to -1 so first index will be 0

    this->pAfterBrowseCallBack=NULL;
    this->AfterBrowseCallBackUserParam=NULL;
    this->pDefaultCommandCallBack=NULL;
    this->DefaultCommandCallBackUserParam=NULL;
    this->pBeforeBrowseCallBack=NULL;
    this->BeforeBrowseCallBackUserParam=NULL;

    ::GetClientRect(this->hWndParent,&this->BrowserRect);

    this->BrowseObjectNestedCount = 0;
}

CShellBrowser::~CShellBrowser(void)
{
    if (this->pShellView)
    {
        this->pShellView->UIActivate(SVUIA_DEACTIVATE);
        IShellView* pOldShellView = this->pShellView;
        this->pShellView=NULL;// assume current view is always NULL or valid

        pOldShellView->DestroyViewWindow();// decrease this->RefCount by 1
        pOldShellView->Release();// decrease this->RefCount by 1
    }
    // from now this->RefCount should be 1, else there is memory leaks
    _ASSERT(this->RefCount==1);

    if (this->pShellFolder)
    {
        this->pShellFolder->Release();
        this->pShellFolder = NULL;
    }

    // free current pidl allocated memory
    if (this->FolderPidl)
    {
        ::ILFree(this->FolderPidl);
        this->FolderPidl = NULL;
    }

    // free history
    for (UINT Cnt=0;Cnt<this->History.size();Cnt++)
    {
        ::ILFree(this->History.at(Cnt));
    }
    this->History.clear();

}

//-----------------------------------------------------------------------------
// Name: GetControlHandle
// Object: get shell browser control handle
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HWND CShellBrowser::GetControlHandle()
{
    return this->hWndShellView;
}

//-----------------------------------------------------------------------------
// Name: GetControlHandle
// Object: get handle of listview associated to shell browser control
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HWND CShellBrowser::GetListViewHandle()
{
    if (!this->hWndShellView)
        return NULL;

    // Find the list view control.
    HWND hWndListView = NULL;

    HWND hWndThumbnail = ::FindWindowEx(this->hWndShellView,NULL,CShellBrowser_ThumbnailVwExtWnd32,NULL);
    if (hWndThumbnail)
        hWndListView = ::FindWindowEx(hWndThumbnail,NULL,WC_LISTVIEW,NULL);

    if (!hWndListView)
        hWndListView = ::FindWindowEx(this->hWndShellView,NULL,WC_LISTVIEW,NULL);

    return hWndListView;
}

//-----------------------------------------------------------------------------
// Name: GetCurrentFolderPath
// Object: Get current path
// Parameters :
//     in  : 
//     out : TCHAR* pszPath : Address of a buffer to receive the file system path. 
//                            THIS BUFFER MUST BE AT LEAST MAX_PATH CHARACTERS IN SIZE
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowser::GetCurrentFolderPath(OUT TCHAR* pszPath)
{
    *pszPath = 0;
    return ::SHGetPathFromIDList(this->FolderPidl, pszPath);
}

//-----------------------------------------------------------------------------
// Name: UpdateRect
// Object: update browser rect (virtual)
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CShellBrowser::UpdateRect(RECT* pRect)
{
    return this->InternalUpdateRect(pRect,FALSE);
}

//-----------------------------------------------------------------------------
// Name: InternalUpdateRect
// Object: update browser rect
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CShellBrowser::InternalUpdateRect(RECT* pRect,BOOL bForceRedraw)
{
    if (!bForceRedraw)
    {
        if (::EqualRect(&this->BrowserRect,pRect))
            return S_OK;
    }

    memcpy(&this->BrowserRect,pRect,sizeof(RECT));

    // update control size
    ::SetWindowPos(this->hWndShellView,
                    0,
                    pRect->left,
                    pRect->top,
                    pRect->right-pRect->left,
                    pRect->bottom-pRect->top,
                    SWP_NOACTIVATE|SWP_NOZORDER);

    // redraw is not required as SWP_NOREDRAW is not specified for SetWindowPos
    // ::RedrawWindow( this->hWndParent,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME | RDW_ALLCHILDREN);

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: SetView
// Object: change browser view
//          NOTICE : SHELL_BROWSER_VIEW_SMALL_ICONS seems to have no effect
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CShellBrowser::SetView(tagShellBrowserView ShellBrowserView)
{
    tagShellBrowserView LocalShellBrowserView = ShellBrowserView;
    if (LocalShellBrowserView>=SHELL_BROWSER_VIEW_COUNT)
        LocalShellBrowserView = SHELL_BROWSER_VIEW_ICONS;
    
    return (HRESULT)::SendMessage(this->hWndShellView,WM_COMMAND,this->ShellBrowserViewInfo[LocalShellBrowserView].Value,NULL);
}

//-----------------------------------------------------------------------------
// Name: IsFolder
// Object: check if provided pidl represent a folder
// Parameters :
//     in  : LPCITEMIDLIST pidl : relative pidl
//           IShellFolder *psfParent : parent shell folder
//     out :
//     return : TRUE if provided pidl represent a folder
//-----------------------------------------------------------------------------
BOOL CShellBrowser::IsFolder(LPCITEMIDLIST pidl,IShellFolder *psfParent)
{
    if ( (!psfParent) || (!pidl) )
    {
        _ASSERT(0);
        return FALSE;
    }

    SFGAOF Attribs = SFGAO_FOLDER | SFGAO_FILESYSTEM | SFGAO_FILESYSANCESTOR | SFGAO_BROWSABLE;
    HRESULT hResult = psfParent->GetAttributesOf(1,&pidl,&Attribs);

    return ((S_OK == hResult &&
        (Attribs & SFGAO_FOLDER) != 0 &&
        ((Attribs & SFGAO_FILESYSTEM) != 0 ||
        (Attribs & SFGAO_FILESYSANCESTOR) != 0)) && !(Attribs & SFGAO_BROWSABLE));
}

//-----------------------------------------------------------------------------
// Name: IsLink
// Object: check if provided pidl represent a link
//          WARNING : if pOutPidl is a valid pointer, caller is responsible to call ::ILFree(pOutPidl);
// Parameters :
//     in  : LPCITEMIDLIST pidl : pidl
//     out : LPITEMIDLIST* pOutPidl : link target pidl
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowser::IsLink(LPCITEMIDLIST pidl,LPITEMIDLIST* pOutPidl)
{
    HRESULT hResult;
    BOOL bRetValue;

    // check parameter
    if ((this->pShellFolder == NULL) || (pidl ==NULL) || (pOutPidl ==NULL) )
    {
        _ASSERT(0);
        return FALSE;
    }

    *pOutPidl=NULL;

    // default return value
    bRetValue=FALSE;

    // Is pidl a link?
    DWORD dwAttribs = SFGAO_LINK;
    this->pShellFolder->GetAttributesOf(1,&pidl,&dwAttribs);

    if (!(dwAttribs & SFGAO_LINK))
        return FALSE;

    // Get the link object.
    IShellLink* pLink=NULL;
    hResult = this->pShellFolder->GetUIObjectOf(this->hWndParent,1,&pidl,IID_IShellLink,0,(void**)&pLink);
    if (FAILED(hResult) || (!pLink))
        return FALSE;

    // Get link target's pidl.
    LPITEMIDLIST LinkTargetPidl = NULL;

    // get target pidl
    hResult = pLink->GetIDList(&LinkTargetPidl);
    if (SUCCEEDED(hResult) && (LinkTargetPidl))
    {
        bRetValue=TRUE;

        if (::IsBadReadPtr(pOutPidl,sizeof(LPITEMIDLIST)))         
        {
            ::ILFree(LinkTargetPidl);
        }
        else
        {
            *pOutPidl=LinkTargetPidl;
        }
    }
    pLink->Release();

    return bRetValue;
}

//-----------------------------------------------------------------------------
// Name: IsFolderLink
// Object: check if provided pidl represent a folder link
//          WARNING : if pOutPidl is a valid pointer, caller is responsible to call ::ILFree(pOutPidl);
// Parameters :
//     in  : LPCITEMIDLIST pidl : pidl
//     out : LPITEMIDLIST* pOutPidl : link target pidl
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowser::IsFolderLink(LPCITEMIDLIST pidl,LPITEMIDLIST* pOutPidl)
{
    HRESULT hResult;
    BOOL bRetValue;

    // check parameter
    if ((this->pShellFolder == NULL) || (pidl ==NULL) || (pOutPidl ==NULL) )
    {
        _ASSERT(0);
        return FALSE;
    }

    *pOutPidl=NULL;

    // default return value
    bRetValue=FALSE;

    // Is pidl a link?
    DWORD dwAttribs = SFGAO_LINK;
    this->pShellFolder->GetAttributesOf(1,&pidl,&dwAttribs);

    if (!(dwAttribs & SFGAO_LINK))
        return FALSE;

    // Get the link object.
    IShellLink* pLink=NULL;
    hResult = this->pShellFolder->GetUIObjectOf(this->hWndParent,1,&pidl,IID_IShellLink,0,(void**)&pLink);
    if (FAILED(hResult) || (!pLink))
        return FALSE;

    // Get link target's pidl.
    LPITEMIDLIST LinkTargetPidl = NULL;
    LPCITEMIDLIST pidlChild = NULL;

    // get target pidl
    hResult = pLink->GetIDList(&LinkTargetPidl);
    if (SUCCEEDED(hResult) && (LinkTargetPidl))
    {
        IShellFolder* pLinkShellFolder=NULL;
        // bind to parent shell folder
        hResult = ::SHBindToParent(LinkTargetPidl,IID_IShellFolder,(void**)&pLinkShellFolder,&pidlChild);
        if (SUCCEEDED(hResult) && pLinkShellFolder && pidlChild)
        {
            // check if target is a folder
            bRetValue = this->IsFolder(pidlChild,pLinkShellFolder);
        }
        if (pLinkShellFolder)
            pLinkShellFolder->Release();

        if (bRetValue)
        {
            if (::IsBadReadPtr(pOutPidl,sizeof(LPITEMIDLIST)))         
            {
                ::ILFree(LinkTargetPidl);
            }
            else
            {
                *pOutPidl=LinkTargetPidl;
            }
        }
        else
        {
            ::ILFree(LinkTargetPidl);
        }
    }
    pLink->Release();

    return bRetValue;
}

HRESULT STDMETHODCALLTYPE CShellBrowser::QueryInterface( REFIID riid, void **ppInterface )
{
    if (ppInterface == NULL)
        return E_POINTER;

    *ppInterface = NULL;

    if(IsEqualIID(riid,IID_IUnknown))
    {
        *ppInterface = (IUnknown*)(IShellBrowser*)(this);
    }
    else if(IsEqualIID(riid,IID_IOleWindow))
    {
        *ppInterface = (IOleWindow*)(this);
    }
    else if(IsEqualIID(riid,IID_IShellBrowser))
    {
        *ppInterface = (IShellBrowser*)(this);
    }
    else if(IsEqualIID(riid,IID_ICommDlgBrowser))
    {
        *ppInterface = (ICommDlgBrowser*)(this);
    }
    else
    {
        *ppInterface=NULL;
        return E_NOINTERFACE;
    }

    ((IUnknown *)(*ppInterface))->AddRef();
    return S_OK;
}

ULONG STDMETHODCALLTYPE CShellBrowser::AddRef()
{
    InterlockedIncrement(&this->RefCount);
    return this->RefCount;
}

ULONG STDMETHODCALLTYPE CShellBrowser::Release()
{
    ULONG localRefCount;
    localRefCount=InterlockedDecrement(&this->RefCount);

    // object not created with class factory --> must be released by "delete pShellBrowser;"
    //if (this->RefCount==0)
    //    delete this;
    return localRefCount;
}

STDMETHODIMP CShellBrowser::ContextSensitiveHelp(BOOL fEnterMode)
{
    UNREFERENCED_PARAMETER(fEnterMode);
    return E_NOTIMPL;
}

STDMETHODIMP CShellBrowser::GetWindow(HWND *lphwnd)
{
    *lphwnd = this->hWndParent;
    return S_OK;
}

STDMETHODIMP CShellBrowser::IncludeObject(THIS_ struct IShellView *ppshv,
                                               LPCITEMIDLIST pidl)
{
    UNREFERENCED_PARAMETER(ppshv);
    UNREFERENCED_PARAMETER(pidl);
    // TODO: Implement filtering support.
    //if (isXXX(pidl))
    //    return S_OK;// Show
    //return S_FALSE; // Hide

    return S_OK;
}

// The browser should return S_OK if it has processed the action or S_FALSE to let the view perform the default action
STDMETHODIMP CShellBrowser::OnDefaultCommand(THIS_ struct IShellView *ppshv)
{
    UNREFERENCED_PARAMETER(ppshv);
    HRESULT hResult;
    LPITEMIDLIST pidlTo = NULL;

    if (!this->bBrowseInsideView)
    {
        // call callback
        this->pDefaultCommandCallBack(this->DefaultCommandCallBackUserParam);

        return S_FALSE;
    }

    CIDA *pData = this->BeginGetItems(TRUE);
    if (!pData)
    {
        hResult = S_FALSE;
        goto CleanUp;
    }

    // get first selected item
    LPCITEMIDLIST pidl = this->GetPIDLItem(pData,0);
    if (!pidl)
    {
        hResult = S_FALSE;
        goto CleanUp;
    }

    // Check if we're dealing with a folder or a folder link.
    if (this->IsFolder(pidl,this->pShellFolder))
    {
        this->BrowseObject(pidl,SBSP_SAMEBROWSER | SBSP_RELATIVE);
        hResult = S_OK;
    }
    else if (this->IsFolderLink(pidl,&pidlTo))
    {
        if (pidlTo)
        {
            this->BrowseObject(pidlTo,SBSP_SAMEBROWSER | SBSP_ABSOLUTE);
            ::ILFree(pidlTo);
            hResult = S_OK;
        }
        else
            hResult = S_FALSE;
    }
    else
        hResult = S_FALSE;

    if (hResult == S_FALSE)
    {
        // call callback
        this->pDefaultCommandCallBack(this->DefaultCommandCallBackUserParam);
    }

CleanUp:
    this->EndGetItems();
    return hResult;
}

// Returns S_OK if successful, or an error value otherwise
STDMETHODIMP CShellBrowser::OnStateChange(THIS_ struct IShellView *ppshv,ULONG uChange)
{
    UNREFERENCED_PARAMETER(ppshv);
    UNREFERENCED_PARAMETER(uChange);
    //CDBOSC_SETFOCUS
    //The focus has been set to the view.
    //CDBOSC_KILLFOCUS
    //The view has lost the focus. 
    //CDBOSC_SELCHANGE
    //The selection has changed.
    //CDBOSC_RENAME
    //An item has been renamed.
    //CDBOSC_STATECHANGE
    //An item has been checked or unchecked.
#ifdef _DEBUG
    switch(uChange)
    {
    case CDBOSC_SETFOCUS:
        OutputDebugString(_T("CDBOSC_SETFOCUS\r\n"));
        break;
    case CDBOSC_KILLFOCUS:
        OutputDebugString(_T("CDBOSC_KILLFOCUS\r\n"));
        break;
    case CDBOSC_SELCHANGE:
        OutputDebugString(_T("CDBOSC_SELCHANGE\r\n"));
        break;
    case CDBOSC_RENAME:
        OutputDebugString(_T("CDBOSC_RENAME\r\n"));
        break;
    case CDBOSC_STATECHANGE:
        OutputDebugString(_T("CDBOSC_STATECHANGE\r\n"));
        break;
    default:
        OutputDebugString(_T("UNKNOWN CDBOSC\r\n"));
        break;
    }
#endif
    return E_NOTIMPL;
}

HRESULT CShellBrowser::BrowseObject(int SpecialFolderCSIDL)
{
    HRESULT hResult = E_FAIL;
    ITEMIDLIST* pFullpidl=NULL;
    IMalloc* pMalloc=NULL;

    // get pointer to malloc interface
    hResult = ::SHGetMalloc(&pMalloc);
    if (FAILED(hResult)||(!pMalloc))
        goto CleanUp;

    // get folder full idl list
    hResult = ::SHGetFolderLocation(this->hWndParent,SpecialFolderCSIDL,NULL,NULL,&pFullpidl);
    if ( FAILED(hResult) || (!pFullpidl) )
        goto CleanUp;

    // do browse
    hResult = this->BrowseObject(pFullpidl,SBSP_SAMEBROWSER | SBSP_ABSOLUTE);

CleanUp:
    if (pFullpidl)
        pMalloc->Free(pFullpidl);

    if (pMalloc)
        pMalloc->Release();

    return hResult;
}

HRESULT CShellBrowser::BrowseObject(TCHAR* Path)
{
    HRESULT hResult= E_FAIL;
    WCHAR* pwscPath;
    IShellFolder* pDesktopFolder=NULL;
    IMalloc* pMalloc=NULL;
    ITEMIDLIST* pFullpidl=NULL;

    // check parameter
    if (::IsBadReadPtr(Path,sizeof(TCHAR)))
        return FALSE;

    // convert ansi to unicode if needed
#if ( defined(UNICODE) || defined(_UNICODE))
    pwscPath=Path;
#else
    size_t NbCharacters = strlen(Path)+1;

    pwscPath = (WCHAR*) malloc(NbCharacters*sizeof(WCHAR));
    if (!pwscPath)
        return FALSE;

    // Convert to Unicode.
    if (::MultiByteToWideChar(CP_ACP, 0, Path, (int)NbCharacters,pwscPath, (int)NbCharacters)==0)
    {
        free(pwscPath);
        return FALSE;
    }
#endif

    // get pointer to malloc interface
    hResult = ::SHGetMalloc(&pMalloc);
    if (FAILED(hResult)||(!pMalloc))
        goto CleanUp;

    // get root folder
    hResult = ::SHGetDesktopFolder(&pDesktopFolder);
    if (FAILED(hResult)||(!pDesktopFolder))
        goto CleanUp;

    // get object full pidl from path name
    hResult = pDesktopFolder->ParseDisplayName(NULL,
                                                NULL,
                                                pwscPath,
                                                NULL,
                                                &pFullpidl,
                                                NULL);
    if ( FAILED(hResult) || (!pFullpidl) )
        goto CleanUp;

    // browse
    hResult = this->BrowseObject(pFullpidl,SBSP_SAMEBROWSER | SBSP_ABSOLUTE);

CleanUp:

#if ( (!defined(UNICODE)) && (!defined(_UNICODE)) )
    if (pwscPath)
        free(pwscPath);
#endif

    if (pFullpidl)
        pMalloc->Free(pFullpidl);

    return hResult;
}

//-----------------------------------------------------------------------------
// Name: RenameSelected
// Object: rename first selected item
// Parameters :
//     in  : 
//     out : 
//     return : S_OK on success
//-----------------------------------------------------------------------------
HRESULT CShellBrowser::RenameSelected()
{
    HRESULT hResult=E_FAIL;
    if (!this->pShellView)
        return E_FAIL;

    CIDA* pData = this->BeginGetItems(TRUE);
    if (!pData)
        goto CleanUp;

    // get first selected item
    LPCITEMIDLIST pidl = this->GetPIDLItem(pData,0); // (pData->cidl -1) for last item
    if (!pidl)
        goto CleanUp;

    IShellView2* pShellView2=NULL;
    hResult = this->pShellView->QueryInterface(IID_IShellView2,(void**)&pShellView2);
    if (FAILED(hResult) || (!pShellView2))
        goto CleanUp;

    hResult = pShellView2->HandleRename(pidl);

    pShellView2->Release();

CleanUp:
    this->EndGetItems();
    return hResult;
}

//-----------------------------------------------------------------------------
// Name: Refresh
// Object: refresh current view
// Parameters :
//     in  : 
//     out : 
//     return : S_OK on success
//-----------------------------------------------------------------------------
HRESULT CShellBrowser::Refresh()
{
    // assume there's an existing shellview
    if (this->pShellView == NULL)
    {
        _ASSERT(0);
        return E_FAIL;
    }

    return this->pShellView->Refresh();
}

HRESULT CShellBrowser::BrowseBack()
{
    return this->BrowseObject(NULL,SBSP_SAMEBROWSER | SBSP_NAVIGATEBACK);
}

size_t CShellBrowser::GetBackwardDepth()
{
    if (this->HistoryIndex<0)
        return 0;
    return this->HistoryIndex;
}
size_t CShellBrowser::GetForwardDepth()
{
    if (this->HistoryIndex<0)
        return 0;
    return (this->History.size()-1)-this->HistoryIndex;
}

HRESULT CShellBrowser::BrowseForward()
{
    return this->BrowseObject(NULL,SBSP_SAMEBROWSER | SBSP_NAVIGATEFORWARD);
}
HRESULT CShellBrowser::BrowseParent()
{
    return this->BrowseObject(NULL,SBSP_SAMEBROWSER | SBSP_PARENT);
}

void CShellBrowser::InternalBeforeBrowseCallBackVirtual(LPCITEMIDLIST pidlToBeBrowsed)
{
    this->InternalBeforeBrowseCallBack(pidlToBeBrowsed);
}

void CShellBrowser::InternalBeforeBrowseCallBack(LPCITEMIDLIST pidlToBeBrowsed)
{
    UNREFERENCED_PARAMETER(pidlToBeBrowsed);
}
HRESULT CShellBrowser::RecreateCurrentView()
{
    HRESULT hResult;
    if (this->FolderPidl==NULL)
    {
        _ASSERT(0);
        return E_FAIL;
    }
    this->bRecreatingCurrentView = TRUE;
    hResult = this->BrowseObject(this->FolderPidl,SBSP_ABSOLUTE);
    this->bRecreatingCurrentView = FALSE;
    return hResult;
}
STDMETHODIMP CShellBrowser::BrowseObject(LPCITEMIDLIST pidl,UINT wFlags)
{
    HRESULT hResult;
    IShellFolder* pNewShellFolder=NULL;
    IShellFolder* pOldShellFolder=NULL;
    IShellView* pNewShellView=NULL;
    IShellFolder* pDesktopShellFolder=NULL;
    LPITEMIDLIST pNewFolderPidl=NULL;
    LPITEMIDLIST pOldFolderPidl=NULL;
    BOOL bNoFolderChange=FALSE;
    HWND OldFocus = ::GetFocus();
    BOOL bSetFocus=FALSE;

#ifdef _DEBUG
    TCHAR MsgTrace[512];
    _sntprintf(MsgTrace,_countof(MsgTrace),TEXT("CShellBrowser::BrowseObject start for object %p\r\n"),this);
    ::OutputDebugString(MsgTrace);
#endif

    // change notification can occur during the this->pShellView->CreateViewWindow() call
    // and if there is no lock, our objects can be deleted
    //          CDesktopBrowserContainer::Change notification for desktop
    //          CShellBrowser::BrowseObject start for object 0000000003158E50
    //          ShellBrowser.cpp : Before CreateViewObject 
    //          ShellBrowser.cpp : After CreateViewObject 
    //          ShellBrowser.cpp : Before CreateViewWindow ---> call not ended before nested CShellBrowser::BrowseObject call
    //          CDesktopBrowserContainer::Change notification for desktop
    //          CShellBrowser::BrowseObject start for object 0000000003158E50
    //          ShellBrowser.cpp : Before CreateViewObject 
    //          ShellBrowser.cpp : After CreateViewObject 
    //          ShellBrowser.cpp : Before CreateViewWindow 
    //          ShellBrowser.cpp : After CreateViewWindow 
    //          CShellBrowser::BrowseObject end for object 0000000003158E50. Result=0x00000000
    //          CDesktopBrowserContainer::Change notification for desktop end
    //          (2684.d70): Access violation - code c0000005 (first chance) <--- CreateViewWindow crash as our objects are now deleted
    //          First chance exceptions are reported before any exception handling.
    ::InterlockedIncrement(&this->BrowseObjectNestedCount);
    BOOL bNestedCall = (this->BrowseObjectNestedCount>1);

#ifndef _DEBUG
    // in case of _DEBUG we print location before skipping
    // in release mode we can go out directly
    if (bNestedCall)
    {
        hResult = E_FAIL;
        goto CleanUp;
    }
#endif

	if (this->hWndListView)
	{
		if (OldFocus==this->hWndListView)
			bSetFocus=TRUE;
	}

    /*
    SBSP_ABSOLUTE
        An absolute pointer to an item identifier list (PIDL) (relative from the desktop).
    SBSP_RELATIVE
        A relative PIDL (relative from the current folder).
    SBSP_PARENT
        Browse the parent folder (ignores the PIDL).
    SBSP_NAVIGATEBACK
        Navigate back (ignores the PIDL).
    SBSP_NAVIGATEFORWARD
        Navigate forward(ignores the PIDL).
    */
    if(wFlags & SBSP_RELATIVE)
    {
        if (this->FolderPidl==NULL)
        {
            _ASSERT(0);
            hResult = E_FAIL;
            goto CleanUp;
        }
        if (pidl == NULL)
        {
            _ASSERT(0);
            hResult = E_FAIL;
            goto CleanUp;
        }
        pNewFolderPidl=::ILCombine(this->FolderPidl,pidl);
    }
    else if(wFlags & SBSP_PARENT)
    {
        if (this->FolderPidl==NULL)
        {
            _ASSERT(0);
            hResult = E_FAIL;
            goto CleanUp;
        }
        pNewFolderPidl=::ILClone(this->FolderPidl);
        if (pNewFolderPidl)
            ::ILRemoveLastID(pNewFolderPidl);
    }
    else if(wFlags & SBSP_NAVIGATEBACK)
    {
        if (this->HistoryIndex>0)
        {
            this->HistoryIndex--;
            if (this->History[this->HistoryIndex]==NULL)
            {
                _ASSERT(0);
                hResult = E_FAIL;
                goto CleanUp;
            }
            pNewFolderPidl=::ILClone(this->History[this->HistoryIndex]);
        }
        else
        {
            hResult = E_FAIL;
            goto CleanUp;
        }
    }
    else if(wFlags & SBSP_NAVIGATEFORWARD)
    {
        if (this->HistoryIndex<(int)(this->History.size()-1))
        {
            this->HistoryIndex++;
            if (this->History[this->HistoryIndex]==NULL)
            {
                _ASSERT(0);
                hResult = E_FAIL;
                goto CleanUp;
            }
            pNewFolderPidl=::ILClone(this->History[this->HistoryIndex]);
        }
        else
        {
            hResult = E_FAIL;
            goto CleanUp;
        }
    }
    else // if (wFlags & SBSP_ABSOLUTE) // stupid m$ stuff : SBSP_ABSOLUTE = 0
    {
        if (pidl == NULL)
        {
            _ASSERT(0);
            hResult = E_FAIL;
            goto CleanUp;
        }
        pNewFolderPidl=::ILClone(pidl);
    }
    
    if (pNewFolderPidl==NULL)
    {
        _ASSERT(0);
        hResult = E_FAIL;
        goto CleanUp;
    }

    // check if folder has changed
    if (this->FolderPidl)
    {
        bNoFolderChange=::ILIsEqual(this->FolderPidl,pNewFolderPidl);
        if (bNoFolderChange && (!this->bRecreatingCurrentView) )
        {
#ifdef _DEBUG
            ::OutputDebugString(TEXT("CShellBrowser::BrowseObject: Same pidl view recreate avoided\r\n"));
#endif
            ::ILFree(pNewFolderPidl);
            // this->Refresh(); // calling refresh is slow, and as Shell def view use SHChangeNotifyRegister to be up to date, there's no need to call it
            hResult = S_OK;
            goto CleanUp;
        }
    }

#ifdef _DEBUG
    {
        TCHAR szPath[MAX_PATH];
        *szPath = 0;
        ::SHGetPathFromIDList(pNewFolderPidl, szPath);
        _sntprintf(MsgTrace,_countof(MsgTrace),TEXT("CShellBrowser::BrowseObject new path %s\r\n"),szPath);
        ::OutputDebugString(MsgTrace);

        if (bNestedCall)
        {
            ::OutputDebugString(TEXT("CShellBrowser::BrowseObject Re entered\r\n"));
            hResult = E_FAIL;
            goto CleanUp;
        }
    }
#endif

    if (this->pBeforeBrowseCallBack)
        this->pBeforeBrowseCallBack(this->FolderPidl,pNewFolderPidl,this->BeforeBrowseCallBackUserParam);

    pOldFolderPidl = this->FolderPidl;
    // store pidl
    this->FolderPidl=pNewFolderPidl;
    // free previous pidl allocated memory
    if (pOldFolderPidl)
        ::ILFree(pOldFolderPidl);

    // call callback only now (when we get full pidl)
    this->InternalBeforeBrowseCallBackVirtual(pNewFolderPidl);

    if ( !(wFlags & SBSP_NAVIGATEFORWARD)
        && !(wFlags & SBSP_NAVIGATEBACK)
        && !(bNoFolderChange)
        )
    {
        // if a part of history must be trashed
        if (this->HistoryIndex!=(int)this->History.size())
        {
            // free associated items
            for (UINT Cnt=(this->HistoryIndex+1);Cnt<this->History.size();Cnt++)
            {
                ::ILFree(this->History[Cnt]);
            }
            this->History.resize(this->HistoryIndex+1);
        }

        this->HistoryIndex++;
        this->History.push_back(::ILClone(pNewFolderPidl));
    }

    if(wFlags & SBSP_RELATIVE)
    {
        // already tested previously
        //if (this->pShellFolder == NULL)
        //{
        //    _ASSERT(0);
        //    return E_FAIL;
        //}

        //if (pidl == NULL)
        //{
        //    _ASSERT(0);
        //    return E_POINTER;
        //}

        hResult = this->pShellFolder->BindToObject(pidl,NULL,IID_IShellFolder,(void **)&pNewShellFolder);
        if ( FAILED(hResult) || (!pNewShellFolder) )
        {
            _ASSERT(0);
            hResult = E_FAIL;
            goto CleanUp;
        }
    }
    else
    {
        hResult = ::SHGetDesktopFolder(&pDesktopShellFolder);
        if ( FAILED(hResult) || (!pDesktopShellFolder) )
        {
            _ASSERT(0);
            hResult = E_FAIL;
            goto CleanUp;
        }

        hResult = pDesktopShellFolder->BindToObject(this->FolderPidl,NULL,IID_IShellFolder, (void**)&pNewShellFolder);
        if ( FAILED(hResult) || (!pNewShellFolder) ) // can occur if queried pidl is the desktop one
            pNewShellFolder=pDesktopShellFolder;
        else
            pDesktopShellFolder->Release();
    }

    if (!pNewShellFolder)
    {
        hResult = E_FAIL;
        goto CleanUp;
    }

    pOldShellFolder = this->pShellFolder;
    this->pShellFolder=pNewShellFolder;

    // store current view infos
    if (this->pShellView)
        this->pShellView->GetCurrentInfo(&this->FolderSettings);

    // https://web.archive.org/web/20051225035102/http://support.microsoft.com:80/kb/q157247/
    // PRB: IShellFolder::CreateViewObject() causes access violation
    // Article ID	:	157247
    // Last Review	:	July 11, 2005
    // Revision	:	2.1
    // This article was previously published under Q157247

    // Using the file system implementation of IShellFolder::CreateViewObject() 
    // results in the shell sending a yet-to-be documented message, the WM_GETISHELLBROWSER message (currently WM_USER+7) 
    // to the hWndOwner. The shell assumes the HWND passed in as the owner responds properly to the WM_GETISHELLBROWSER message. 
    // If it doesn't, there is an access violation in Shell32.DLL

    // RESOLUTION
    // The shell expects that the hWndOwner will supply the IShellBrowser interface to it in response to the WM_GETISHELLBROWSER message.
    // 
    // For example, if your window is a dialog, the following snippet of code would respond correctly to the message:
    // 
    //    ...
    //        #ifndef WM_GETISHELLBROWSER
    //            #define WM_GETISHELLBROWSER (WM_USER+7)
    //        #endif
    //    ...
    //        IShellBrowser *m_psb;
    //    ...
    //        BOOL DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
    //        {
    //            switch(uMsg)
    //            {
    //    ...
    //            case WM_GETISHELLBROWSER:
    //                SetWindowLong(hDlg, DWL_MSGRESULT, (LONG)m_psb);
    //                return TRUE;
#ifdef _DEBUG
    ::OutputDebugString(TEXT("ShellBrowser.cpp : Before CreateViewObject \r\n"));
#endif
    hResult = this->pShellFolder->CreateViewObject(this->hWndParent,IID_IShellView,(void **)&pNewShellView);
#ifdef _DEBUG
    ::OutputDebugString(TEXT("ShellBrowser.cpp : After CreateViewObject \r\n"));
#endif
    if ( FAILED(hResult) || (!pNewShellView) )
    {
        _ASSERT(0);
        hResult = E_FAIL;
        goto CleanUp;
    }


    IShellView* pOldShellView = this->pShellView;
    this->pShellView=pNewShellView;// assume current view is always NULL or valid

#ifdef _DEBUG
    ::OutputDebugString(TEXT("ShellBrowser.cpp : Before CreateViewWindow \r\n"));
#endif
    // Even destroying pOldShellView before call CreateViewWindow doesn't repost LVN_GETDISPINFOW (at least on win10) in case of refresh
    hResult = this->pShellView->CreateViewWindow(pOldShellView,&this->FolderSettings,(IShellBrowser*)this,&this->BrowserRect,&this->hWndShellView);
    // hResult = this->pShellView->CreateViewWindow(NULL,&this->FolderSettings,(IShellBrowser*)this,&this->BrowserRect,&this->hWndShellView);
#ifdef _DEBUG
    ::OutputDebugString(TEXT("ShellBrowser.cpp : After CreateViewWindow \r\n"));
#endif

    if (pOldShellView)
    {
        pOldShellView->UIActivate(SVUIA_DEACTIVATE);
        pOldShellView->DestroyViewWindow();
        pOldShellView->Release();
    }

    if (pOldShellFolder)
        pOldShellFolder->Release();

    if ( FAILED(hResult) || (!this->hWndShellView) )
    {
        hResult = E_FAIL;
        goto CleanUp;
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // seven is happy to make us shit by changing specified view for icon : 
    // it seems it tries to adjust icon size (small,medium,large or extra large) to window available space 
    // --> we have to check view just after setting it :(
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    
    //// get applied view mode // not applicable for extended view (like large, extra large)
    //FOLDERSETTINGS fs;
    //this->pShellView->GetCurrentInfo(&fs);
    //// if mode view differ
    //if (fs.ViewMode!=this->FolderSettings.ViewMode)
    //{
    //    tagShellBrowserView shview;
    //    // if translation from folder setting to shell browser view mode is ok
    //    if (CShellBrowser::FolderSettingToShellBrowserView(&this->FolderSettings,&shview))
    //        // apply OUR view mode
    //        this->SetView(shview);
    //}
    this->SetView(this->ShellBrowserView);

    /////////////////////////
    // GUI
    /////////////////////////

    // Only set the focus to the new list view if the old shell list view instance
    // had the focus.
    this->pShellView->UIActivate(bSetFocus ? SVUIA_ACTIVATE_FOCUS : SVUIA_ACTIVATE_NOFOCUS);

    // show listview window
    this->hWndListView = this->GetListViewHandle();
    ::ShowWindow(this->hWndShellView,SW_NORMAL);

    this->InternalAfterListViewCreationCallBackVirtual();

    if (this->pAfterBrowseCallBack)
        this->pAfterBrowseCallBack(this->FolderPidl,this->AfterBrowseCallBackUserParam);

    hResult = S_OK;

CleanUp:

    ::InterlockedDecrement(&this->BrowseObjectNestedCount);

#ifdef _DEBUG
    _sntprintf(MsgTrace,_countof(MsgTrace),TEXT("CShellBrowser::BrowseObject end for object %p. Result=0x%08x\r\n"),this,(DWORD)hResult),
    ::OutputDebugString(MsgTrace);
#endif

    return hResult;
}


void CShellBrowser::InternalAfterListViewCreationCallBackVirtual()
{
    this->InternalAfterListViewCreationCallBack();
}

void CShellBrowser::InternalAfterListViewCreationCallBack()
{

}

STDMETHODIMP CShellBrowser::EnableModelessSB(BOOL fEnable)
{
    UNREFERENCED_PARAMETER(fEnable);
    return E_NOTIMPL;
}

STDMETHODIMP CShellBrowser::GetControlWindow(UINT id,HWND *lphwnd)
{
    UNREFERENCED_PARAMETER(id);
    if (lphwnd == NULL)
        return E_POINTER;

    //switch(id)
    //{
    //case FCW_STATUS:
    //    *lphwnd = this->hWndStatusBar;
    //    return S_OK;
    //case FCW_TOOLBAR:
    //    *lphwnd = this->hWndToolBar;
    //    return S_OK;
    // // FCW_TREE, FCW_PROGRESS, FCW_INTERNETBAR
    //}

    return E_NOTIMPL;
}

STDMETHODIMP CShellBrowser::GetViewStateStream(DWORD grfMode,LPSTREAM *ppStrm)
{
    UNREFERENCED_PARAMETER(grfMode);
    if(ppStrm == NULL)
        return E_POINTER;
    return E_NOTIMPL;
}

STDMETHODIMP CShellBrowser::InsertMenusSB(HMENU hmenuShared,
                                          LPOLEMENUGROUPWIDTHS lpMenuWidths)
{
#ifdef _DEBUG
    OutputDebugString(_T("InsertMenusSB\r\n"));
#endif
    UNREFERENCED_PARAMETER(hmenuShared);
    UNREFERENCED_PARAMETER(lpMenuWidths);
    return E_NOTIMPL;
}

STDMETHODIMP CShellBrowser::OnViewWindowActive(struct IShellView *ppshv)
{
#ifdef _DEBUG
    OutputDebugString(_T("OnViewWindowActive\r\n"));
#endif
    if(ppshv == NULL)
        return E_POINTER;

    return E_NOTIMPL;
}

STDMETHODIMP CShellBrowser::QueryActiveShellView(struct IShellView **ppshv)
{
#ifdef _DEBUG
    OutputDebugString(_T("QueryActiveShellView\r\n"));
#endif
    if(ppshv == NULL)
        return E_POINTER;
    if (!this->pShellView)
        return E_FAIL;
    this->pShellView->AddRef();
    *ppshv = this->pShellView;
    return S_OK;
}

STDMETHODIMP CShellBrowser::RemoveMenusSB(HMENU hmenuShared)
{
#ifdef _DEBUG
    OutputDebugString(_T("RemoveMenusSB\r\n"));
#endif
    UNREFERENCED_PARAMETER(hmenuShared);
    return E_NOTIMPL;
}

STDMETHODIMP CShellBrowser::SendControlMsg(UINT id,UINT uMsg,WPARAM wParam,
                                                LPARAM lParam,LRESULT *pret)
{
    UNREFERENCED_PARAMETER(id);
    UNREFERENCED_PARAMETER(uMsg);
    UNREFERENCED_PARAMETER(wParam);
    UNREFERENCED_PARAMETER(lParam);

    if(pret == NULL)
        return E_POINTER;

    //switch(id)
    //{
    //case FCW_STATUS:
    //    *pret = ::SendMessage(this->hWndStatusBar,uMsg,wParam,lParam);
    //    return S_OK;
    //case FCW_TOOLBAR:
    //    *lphwnd = *pret = ::SendMessage(this->hWndToolBar,uMsg,wParam,lParam);
    //    return S_OK;
    //}

    return E_NOTIMPL;
}

STDMETHODIMP CShellBrowser::SetMenuSB(HMENU hmenuShared,HOLEMENU holemenuReserved,HWND hwndActiveObject)
{
#ifdef _DEBUG
    OutputDebugString(_T("SetMenuSB\r\n"));
#endif
    UNREFERENCED_PARAMETER(hmenuShared);
    UNREFERENCED_PARAMETER(holemenuReserved);
    UNREFERENCED_PARAMETER(hwndActiveObject);
    return E_NOTIMPL;
}

STDMETHODIMP CShellBrowser::SetStatusTextSB(LPCOLESTR lpszStatusText)
{
#ifdef _DEBUG
    OutputDebugString(_T("SetStatusTextSB\r\n"));
#endif
    UNREFERENCED_PARAMETER(lpszStatusText);
    return E_NOTIMPL;
}

STDMETHODIMP CShellBrowser::SetToolbarItems(LPTBBUTTON lpButtons,UINT nButtons,UINT uFlags)
{
    UNREFERENCED_PARAMETER(lpButtons);
    UNREFERENCED_PARAMETER(nButtons);
    UNREFERENCED_PARAMETER(uFlags);
    return E_NOTIMPL;
}

STDMETHODIMP CShellBrowser::TranslateAcceleratorSB(LPMSG lpmsg,WORD wID)
{
    UNREFERENCED_PARAMETER(lpmsg);
    UNREFERENCED_PARAMETER(wID);
#ifdef _DEBUG
    OutputDebugString(_T("TranslateAcceleratorSB : \r\n"));
#endif

    return S_OK;
}

BOOL CShellBrowser::TranslateAccelerator(PMSG pMsg)
{
    if (!this->pShellView)
        return FALSE;
    return (this->pShellView->TranslateAccelerator(pMsg)==S_OK);
}

BOOL CShellBrowser::SetDefaultCommandCallBack(tagDefaultCommandCallBack pDefaultCommandCallBack,PVOID UserParam)
{
    if (::IsBadCodePtr((FARPROC)pDefaultCommandCallBack))
    {
        this->pDefaultCommandCallBack = NULL;
        return FALSE;
    }

    this->pDefaultCommandCallBack=pDefaultCommandCallBack;
    this->DefaultCommandCallBackUserParam=UserParam;

    return TRUE;
}
BOOL CShellBrowser::SetAfterBrowseCallBack(tagAfterBrowseCallBack pAfterBrowseCallBack,PVOID UserParam)
{
    if (::IsBadCodePtr((FARPROC)pAfterBrowseCallBack))
    {
        this->pAfterBrowseCallBack = NULL;
        return FALSE;
    }

    this->pAfterBrowseCallBack=pAfterBrowseCallBack;
    this->AfterBrowseCallBackUserParam=UserParam;

    return TRUE;
}

BOOL CShellBrowser::SetBeforeBrowseCallBack(tagBeforeBrowseCallBack pBeforeBrowseCallBack,PVOID UserParam)
{
    if (::IsBadCodePtr((FARPROC)pBeforeBrowseCallBack))
    {
        this->pBeforeBrowseCallBack = NULL;
        return FALSE;
    }

    this->pBeforeBrowseCallBack=pBeforeBrowseCallBack;
    this->BeforeBrowseCallBackUserParam=UserParam;

    return TRUE;
}

BOOL CShellBrowser::FolderSettingToShellBrowserView(FOLDERSETTINGS* pfs,OUT tagShellBrowserView* pShellBrowserView)
{
    BOOL bRet=TRUE;

    if (pShellBrowserView == NULL)
        return FALSE;
    *pShellBrowserView = SHELL_BROWSER_VIEW_LIST;

    if (pfs == NULL)
        return FALSE;

    switch (pfs->ViewMode)
    {
    default:
        *pShellBrowserView=SHELL_BROWSER_VIEW_LIST;
        bRet=FALSE;
        break;
    case FVM_ICON:
        *pShellBrowserView=SHELL_BROWSER_VIEW_ICONS;
        break;
    case FVM_SMALLICON:
        *pShellBrowserView=SHELL_BROWSER_VIEW_SMALL_ICONS;
        break;
    case FVM_LIST:
        *pShellBrowserView=SHELL_BROWSER_VIEW_LIST;
        break;
    case FVM_DETAILS:
        *pShellBrowserView=SHELL_BROWSER_VIEW_DETAIL;
        break;
    case FVM_THUMBNAIL:
        *pShellBrowserView=SHELL_BROWSER_VIEW_THUMBNAILS;
        break;
    case FVM_THUMBSTRIP:
        *pShellBrowserView=SHELL_BROWSER_VIEW_THUMBSTRIP;
        break;
    }
    return bRet;
}

BOOL CShellBrowser::ShellBrowserViewToFolderSetting(tagShellBrowserView ShellBrowserView,OUT FOLDERSETTINGS* pfs)
{
    BOOL bRet=TRUE;
    if (pfs == NULL)
        return FALSE;

    pfs->fFlags= FWF_NOWEBVIEW | FWF_NOHEADERINALLVIEWS | FWF_NOGROUPING; // FWF_NOCOLUMNHEADER
    // FWF_NOHEADERINALLVIEWS : Only show the column header in details view mode
    // FWF_NOCOLUMNHEADER : Do not display a column header in the view in any view mode

    switch (ShellBrowserView)
    {
    default:
        pfs->ViewMode=FVM_LIST;
        bRet=FALSE;
        break;
    case CShellBrowser::SHELL_BROWSER_VIEW_MEDIUM:
    case CShellBrowser::SHELL_BROWSER_VIEW_ICONS:
    case CShellBrowser::SHELL_BROWSER_VIEW_LARGE:
    case CShellBrowser::SHELL_BROWSER_VIEW_EXTRA_LARGE:
        pfs->ViewMode=FVM_ICON;
        break;
    case CShellBrowser::SHELL_BROWSER_VIEW_SMALL_ICONS:
        pfs->ViewMode=FVM_SMALLICON;
        break;
    case CShellBrowser::SHELL_BROWSER_VIEW_LIST:
        pfs->ViewMode=FVM_LIST;
        break;
    case CShellBrowser::SHELL_BROWSER_VIEW_DETAIL:
        pfs->ViewMode=FVM_DETAILS;
        // pfs->fFlags &= ~FWF_NOCOLUMNHEADER; // only case where we want columns headers
        break;
    case CShellBrowser::SHELL_BROWSER_VIEW_THUMBNAILS:
        pfs->ViewMode=FVM_THUMBNAIL;
        break;
    case CShellBrowser::SHELL_BROWSER_VIEW_THUMBSTRIP:
        pfs->ViewMode=FVM_THUMBSTRIP;
        break;
    }
    return bRet;
}
//////////////////////////////////////////////////////////////////////////////////////


// look at MSDN "Shell Clipboard Formats"
CIDA* CShellBrowser::BeginGetItems(BOOL bOnlySelected)
{
    if (!this->pShellView)
    {
        _ASSERT(0);
        return NULL;
    }

    if (this->hItemsMem != NULL)
        this->EndGetItems();

    IDataObject* pSelection=NULL;
    UINT uItem=bOnlySelected ? SVGIO_SELECTION : SVGIO_ALLVIEW;

    HRESULT hResult = this->pShellView->GetItemObject(uItem,IID_IDataObject,(void **)&pSelection);
    if (FAILED(hResult) || (!pSelection))
        return NULL;

    // CFSTR_SHELLIDLIST
    FORMATETC fetc={0};
    fetc.cfFormat = (CLIPFORMAT)this->CF_IDLIST;
    fetc.dwAspect = DVASPECT_CONTENT;
    fetc.ptd = NULL;
    fetc.lindex = -1;
    fetc.tymed = TYMED_HGLOBAL;

    hResult = pSelection->QueryGetData(&fetc);
    if (FAILED(hResult))
        return NULL;

    STGMEDIUM stm={0};
    hResult = pSelection->GetData(&fetc,&stm);
    if (FAILED(hResult) || (!stm.hGlobal))
        return NULL;

    this->hItemsMem = stm.hGlobal;
    CIDA* pData = (CIDA*)::GlobalLock(stm.hGlobal);
    if (pData == NULL)
        return NULL;

    if (pData->cidl < 1)
        return NULL;

    return pData;
}
// look at MSDN "Shell Clipboard Formats"
void CShellBrowser::EndGetItems()
{
    if (this->hItemsMem)
    {
        ::GlobalUnlock(this->hItemsMem);
        this->hItemsMem = NULL;
    }
}

//https://docs.microsoft.com/en-us/windows/desktop/shell/clipboard
//#define GetPIDLFolder(pida) (LPCITEMIDLIST)(((LPBYTE)pida)+(pida)->aoffset[0])
//#define GetPIDLItem(pida, i) (LPCITEMIDLIST)(((LPBYTE)pida)+(pida)->aoffset[i+1])

// look at MSDN "Shell Clipboard Formats"
LPCITEMIDLIST CShellBrowser::GetPIDLFolder(CIDA* pida)
{
    if (!pida)
        return NULL;
    return (LPCITEMIDLIST)(((LPBYTE)pida)+(pida)->aoffset[0]);
}

// look at MSDN "Shell Clipboard Formats"
LPCITEMIDLIST CShellBrowser::GetPIDLItem(CIDA* pida,UINT ItemIndex)
{
    if (!pida)
        return NULL;
    return (LPCITEMIDLIST)(((LPBYTE)pida)+(pida)->aoffset[ItemIndex+1]);
}