/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manage a shell browser
//-----------------------------------------------------------------------------
#pragma once

#ifndef _WIN32_WINNT
    #define _WIN32_WINNT 0x0501 // for xp os
#endif

#include <windows.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)

#include <objbase.h>
#include <shlobj.h>
#include <ShObjIdl.h>
#pragma comment(lib,"ole32")

#include <vector>

// shell def view messages
#define WM_DSV_FSNOTIFY                 (WM_USER+0xA0)
#define WM_DSV_DESTROYSTATIC            (WM_USER+0xA1)
#define WM_DSV_BACKGROUNDENUMDONE       (WM_USER+0xA2)
#define WM_DSV_UPDATEICON               (WM_USER+0xA3)
#define WM_DSV_SHOWDRAGIMAGE            (WM_USER+0xA4)
#define WM_DSV_MENUTERM                 (WM_USER+0xA5)
#define WM_DSV_DISABLEACTIVEDESKTOP     (WM_USER+0xA6)
#define WM_DSV_REARRANGELISTVIEW        (WM_USER+0xA7)
#define WM_DSV_SENDSELECTIONCHANGED     (WM_USER+0xA8)
#define WM_DSV_UPDATECOLDATA            (WM_USER+0xA9)
#define WM_DSV_DESKTOPCONTEXTMENU       (WM_USER+0xAA)
#define WM_DSV_UPDATEOVERLAY            (WM_USER+0xAB)
#define WM_DSV_DESKHTML_CHANGES         (WM_USER+0xAC)
#define WM_DSV_FILELISTENUMDONE         (WM_USER+0xAD)
//AE
//AF

// seven
// messages send to Shell_DefView window throw WPARAM of WM_COMMAND 
//WM_COMMAND wParam: 0x00007B00  lParam:0x00000000 ascending
//WM_COMMAND wParam: 0x00007B01  lParam:0x00000000 descending (available for detailed view)
//
// user specific: depends of the "sort by more" order 0x00007B03 + 1 based item order
// to switch between ascending and descending order, just send the same message again
//WM_COMMAND wParam: 0x00007B04  lParam:0x00000000 name
//WM_COMMAND wParam: 0x00007B05  lParam:0x00000000 date modified
//WM_COMMAND wParam: 0x00007B06  lParam:0x00000000 type
//WM_COMMAND wParam: 0x00007B07  lParam:0x00000000 size
//// end of user specific
//
//WM_COMMAND wParam: 0x00007033  lParam:0x00000000 sort by more / group by more
//
//
//WM_COMMAND wParam: 0x00007600  lParam:0x00000000 group by ascending
//WM_COMMAND wParam: 0x00007601  lParam:0x00000000 group by descending
//WM_COMMAND wParam: 0x00007602  lParam:0x00000000 group by none
//
// user specific: depends of the "group by more" order 0x00007602 + 1 based item order
// to switch between ascending and descending order, just send the same message again
//WM_COMMAND wParam: 0x00007603  lParam:0x00000000 group by name
//WM_COMMAND wParam: 0x00007604  lParam:0x00000000 group by date modified
//WM_COMMAND wParam: 0x00007605  lParam:0x00000000 group by type
//WM_COMMAND wParam: 0x00007606  lParam:0x00000000 group by size
//// end of user specific
//
//
//WM_COMMAND wParam: 0x00007053  lParam:0x00000000 expand all groups
//WM_COMMAND wParam: 0x00007054  lParam:0x00000000 collapse all groups

//7071 autoarrange enable/disable
//7074 align to grid enable / disable

#define SHELL_VIEW_AUTO_ARRANGE_SEVEN           0x7071
#define SHELL_VIEW_ALIGN_TO_GRID_SEVEN          0x7074

#define SHELL_VIEW_GROUP_BY_ASCENDING_SEVEN     0x7600 // sort ascending
#define SHELL_VIEW_GROUP_BY_DESCENDING_SEVEN    0x7601 // sort descending
#define SHELL_VIEW_GROUP_BY_NONE_SEVEN          0x7602 // sort none
#define SHELL_VIEW_GROUP_BY_NAME_SEVEN          0x7603 // group by name 
#define SHELL_VIEW_GROUP_BY_DATE_SEVEN          0x7604 // group by date 
#define SHELL_VIEW_GROUP_BY_TYPE_SEVEN          0x7605 // group by type  
#define SHELL_VIEW_GROUP_BY_SIZE_D_SEVEN        0x7606 // group by size  
#define SHELL_VIEW_SORT_BY_NAME_SEVEN           0x7b04 // sort by name
#define SHELL_VIEW_SORT_BY_DATE_SEVEN           0x7b05 // sort by date
#define SHELL_VIEW_SORT_BY_TYPE_SEVEN           0x7b06 // sort by type
#define SHELL_VIEW_SORT_BY_SIZE_SEVEN           0x7b07 // sort by size

#define SHELL_VIEW_SORT_BY_GROUP_SEVEN          0x7033 // sort by more / group by more
#define SHELL_VIEW_EXPAND_GROUPS_SEVEN          0x7053 // expand all groups
#define SHELL_VIEW_COLLAPSE_GROUPS_SEVEN        0x7054 // collapse all groups

#define CShellBrowser_ThumbnailVwExtWnd32 _T("ThumbnailVwExtWnd32")
class CShellBrowser : public ICommDlgBrowser, public IShellBrowser
{
public:
    enum tagShellBrowserView
    {
        SHELL_BROWSER_VIEW_ICONS = 0,
        SHELL_BROWSER_VIEW_FIRST,
        SHELL_BROWSER_VIEW_SMALL_ICONS,
        SHELL_BROWSER_VIEW_LIST,
        SHELL_BROWSER_VIEW_DETAIL,
        SHELL_BROWSER_VIEW_MEDIUM,
        SHELL_BROWSER_VIEW_LARGE,
        SHELL_BROWSER_VIEW_EXTRA_LARGE,
        SHELL_BROWSER_VIEW_TILES,
        SHELL_BROWSER_VIEW_THUMBNAILS,
        SHELL_BROWSER_VIEW_THUMBSTRIP,

        SHELL_BROWSER_VIEW_COUNT
    };
    typedef struct tagShellBrowserViewInfo
    {
        SIZE_T Value;
    };

    tagShellBrowserViewInfo ShellBrowserViewInfo[SHELL_BROWSER_VIEW_COUNT];
    BOOL bBrowseInsideView;

    typedef void (*tagDefaultCommandCallBack)(PVOID UserParam);
    typedef void (*tagAfterBrowseCallBack)(LPITEMIDLIST FolderPidl,PVOID UserParam);
    typedef void (*tagBeforeBrowseCallBack)(LPITEMIDLIST CurrentPidl,LPITEMIDLIST ToBeBrowsePidl,PVOID UserParam);
    tagDefaultCommandCallBack pDefaultCommandCallBack;
    PVOID DefaultCommandCallBackUserParam;
    tagAfterBrowseCallBack pAfterBrowseCallBack;
    PVOID AfterBrowseCallBackUserParam;
    tagBeforeBrowseCallBack pBeforeBrowseCallBack;
    PVOID BeforeBrowseCallBackUserParam;

    CShellBrowser(HWND hWndParent);
    CShellBrowser(HWND hWndParent,FOLDERSETTINGS* pfs);
    CShellBrowser(HWND hWndParent,FOLDERSETTINGS* pfs,RECT* pBrowserRect);
    CShellBrowser(HWND hWndParent,FOLDERSETTINGS* pfs,tagShellBrowserView ShellBrowserView);
    CShellBrowser(HWND hWndParent,FOLDERSETTINGS* pfs,tagShellBrowserView ShellBrowserView,RECT* pBrowserRect);
    ~CShellBrowser(void);

    SIZE_T GetShellBrowserViewValue(tagShellBrowserView ShellBrowserView)
    {
        return ShellBrowserViewInfo[ShellBrowserView].Value;
    }

    // IUnknown members.
    STDMETHOD(QueryInterface)(REFIID iid,void **ppvObject);
    STDMETHOD_(ULONG,AddRef)();
    STDMETHOD_(ULONG,Release)();

    // IOleWindow members.
    STDMETHOD(ContextSensitiveHelp)(BOOL fEnterMode);
    STDMETHOD(GetWindow)(HWND *lphwnd);

    // ICommDlgBrowser members.
    STDMETHOD(IncludeObject)(THIS_ struct IShellView *ppshv,LPCITEMIDLIST pidl);
    STDMETHOD(OnDefaultCommand)(THIS_ struct IShellView *ppshv);
    STDMETHOD(OnStateChange)(THIS_ struct IShellView *ppshv,ULONG uChange);

    // IShellBrowser members (same as IOleInPlaceFrame).
    STDMETHOD(BrowseObject)(LPCITEMIDLIST pidl,UINT wFlags);
    STDMETHOD(EnableModelessSB)(BOOL fEnable);
    STDMETHOD(GetControlWindow)(UINT id,HWND *lphwnd);
    STDMETHOD(GetViewStateStream)(DWORD grfMode,LPSTREAM *ppStrm);
    STDMETHOD(InsertMenusSB)(HMENU hmenuShared,LPOLEMENUGROUPWIDTHS lpMenuWidths);
    STDMETHOD(OnViewWindowActive)(struct IShellView *ppshv);
    STDMETHOD(QueryActiveShellView)(struct IShellView **ppshv);
    STDMETHOD(RemoveMenusSB)(HMENU hmenuShared);
    STDMETHOD(SendControlMsg)(UINT id,UINT uMsg,WPARAM wParam,LPARAM lParam,LRESULT *pret);
    STDMETHOD(SetMenuSB)(HMENU hmenuShared,HOLEMENU holemenuReserved,HWND hwndActiveObject);
    STDMETHOD(SetStatusTextSB)(LPCOLESTR lpszStatusText);
    STDMETHOD(SetToolbarItems)(LPTBBUTTON lpButtons,UINT nButtons,UINT uFlags);
    STDMETHOD(TranslateAcceleratorSB)(LPMSG lpmsg,WORD wID);

    BOOL TranslateAccelerator(PMSG pMsg);

    HWND GetListViewHandle();
    HWND GetControlHandle();
    BOOL GetCurrentFolderPath(OUT TCHAR* pszPath);
    HRESULT BrowseObject(int SpecialFolderCSIDL);
    HRESULT BrowseObject(TCHAR* Path);
    HRESULT BrowseBack();
    size_t GetBackwardDepth();
    HRESULT BrowseForward();
    size_t GetForwardDepth();
    HRESULT BrowseParent();
    virtual HRESULT Refresh();
    HRESULT RecreateCurrentView();
    HRESULT RenameSelected();
    virtual HRESULT SetView(tagShellBrowserView ShellBrowserView);
    virtual HRESULT UpdateRect(RECT* pRect);

    BOOL SetDefaultCommandCallBack(tagDefaultCommandCallBack pDefaultCommandCallBack,PVOID UserParam);
    BOOL SetAfterBrowseCallBack(tagAfterBrowseCallBack pAfterBrowseCallBack,PVOID UserParam);
    BOOL SetBeforeBrowseCallBack(tagBeforeBrowseCallBack pBeforeBrowseCallBack,PVOID UserParam);

    static BOOL FolderSettingToShellBrowserView(FOLDERSETTINGS* pfs,OUT tagShellBrowserView* pShellBrowserView);
    static BOOL ShellBrowserViewToFolderSetting(tagShellBrowserView ShellBrowserView,OUT FOLDERSETTINGS* pfs);

    static BOOL IsSortingMessage(WPARAM wParam)
    {
        switch (wParam)
        {
        // case SHELL_VIEW_AUTO_ARRANGE_SEVEN        : // allow auto arrange
        case SHELL_VIEW_ALIGN_TO_GRID_SEVEN       :
        case SHELL_VIEW_GROUP_BY_ASCENDING_SEVEN  :
        case SHELL_VIEW_GROUP_BY_DESCENDING_SEVEN :
        case SHELL_VIEW_GROUP_BY_NONE_SEVEN       :
        case SHELL_VIEW_GROUP_BY_NAME_SEVEN       :
        case SHELL_VIEW_GROUP_BY_DATE_SEVEN       :
        case SHELL_VIEW_GROUP_BY_TYPE_SEVEN       :
        case SHELL_VIEW_GROUP_BY_SIZE_D_SEVEN     :
        case SHELL_VIEW_SORT_BY_NAME_SEVEN        :
        case SHELL_VIEW_SORT_BY_DATE_SEVEN        :
        case SHELL_VIEW_SORT_BY_TYPE_SEVEN        :
        case SHELL_VIEW_SORT_BY_SIZE_SEVEN        :
        case SHELL_VIEW_SORT_BY_GROUP_SEVEN       :
        case SHELL_VIEW_EXPAND_GROUPS_SEVEN       :
        case SHELL_VIEW_COLLAPSE_GROUPS_SEVEN     :
            return TRUE;
        default:
            return FALSE;
        }
    }
protected:
    LONG RefCount;

    HWND hWndParent;
    HWND hWndShellView;
    HWND hWndListView;
    FOLDERSETTINGS FolderSettings;
    tagShellBrowserView ShellBrowserView;
    HGLOBAL hItemsMem;
    UINT CF_IDLIST;
    LPITEMIDLIST FolderPidl;
    RECT BrowserRect;

    IShellFolder *pShellFolder;
    IShellView   *pShellView;
    std::vector<LPITEMIDLIST> History;
    INT HistoryIndex;
    BOOL bRecreatingCurrentView;
    LONG volatile BrowseObjectNestedCount;

    void CommonConstructor(HWND hWndParent,FOLDERSETTINGS* pfs,tagShellBrowserView ShellBrowserView);
    CIDA* BeginGetItems(BOOL bOnlySelected);
    void EndGetItems();
    LPCITEMIDLIST GetPIDLFolder(CIDA* pida);
    LPCITEMIDLIST GetPIDLItem(CIDA* pida,UINT ItemIndex);
    BOOL IsFolder(LPCITEMIDLIST pidl,IShellFolder *psfParent);
    BOOL IsFolderLink(LPCITEMIDLIST pidl,LPITEMIDLIST *pOutPidl);
    BOOL IsLink(LPCITEMIDLIST pidl,LPITEMIDLIST* pOutPidl);

    virtual void InternalAfterListViewCreationCallBackVirtual();
    void InternalAfterListViewCreationCallBack();
    virtual void InternalBeforeBrowseCallBackVirtual(LPCITEMIDLIST pidlToBeBrowsed);
    void InternalBeforeBrowseCallBack(LPCITEMIDLIST pidlToBeBrowsed);
    HRESULT InternalUpdateRect(RECT* pRect,BOOL bForceRedraw);
};
