/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manage a shell browser with extended features
//-----------------------------------------------------------------------------
#include "ShellBrowserEx.h"
#include "../../ShellExecute/ShellExecuteAndBringToTop.h"

//-----------------------------------------------------------------------------
// Name: CShellBrowserEx
// Object: Constructor.
// Parameters :
//     in  : HWND hWndParent : parent window handle
//     out :
//     return : 
//-----------------------------------------------------------------------------
CShellBrowserEx::CShellBrowserEx(HWND hWndParent):CShellBrowser(hWndParent)
{
    this->CommonConstructor();
}

//-----------------------------------------------------------------------------
// Name: CShellBrowserEx
// Object: Constructor.
// Parameters :
//     in  : HWND hWndParent : parent window handle
//     out :
//     return : 
//-----------------------------------------------------------------------------
CShellBrowserEx::CShellBrowserEx(HWND hWndParent,FOLDERSETTINGS* pfs):CShellBrowser(hWndParent,pfs)
{
    this->CommonConstructor();
}

//-----------------------------------------------------------------------------
// Name: CShellBrowserEx
// Object: Constructor.
// Parameters :
//     in  : HWND hWndParent : parent window handle
//     out :
//     return : 
//-----------------------------------------------------------------------------
CShellBrowserEx::CShellBrowserEx(HWND hWndParent,FOLDERSETTINGS* pfs,RECT* pBrowserRect):CShellBrowser(hWndParent,pfs,pBrowserRect)
{
    this->CommonConstructor();
}
CShellBrowserEx::CShellBrowserEx(HWND hWndParent,FOLDERSETTINGS* pfs,tagShellBrowserView ShellBrowserView):CShellBrowser(hWndParent,pfs,ShellBrowserView)
{
    this->CommonConstructor();
}
CShellBrowserEx::CShellBrowserEx(HWND hWndParent,FOLDERSETTINGS* pfs,tagShellBrowserView ShellBrowserView,RECT* pBrowserRect):CShellBrowser(hWndParent,pfs,ShellBrowserView,pBrowserRect)
{
    this->CommonConstructor();
}

//-----------------------------------------------------------------------------
// Name: CommonConstructor
// Object: common Constructor.
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CShellBrowserEx::CommonConstructor()
{
    // initiate GDI
    Gdiplus::GdiplusStartupInput gdiplusStartupInput;
    this->gdiplusToken=NULL;
    bGdiPlusSuccessfullyStarted=(Gdiplus::GdiplusStartup(&this->gdiplusToken, &gdiplusStartupInput, NULL)==Gdiplus::Ok);

    this->BackgroundType=BACKGROUND_TYPE_NONE;
    this->TextColorType=COLOR_TYPE_NONE;
    this->TextBackgroundColorType=COLOR_TYPE_NONE;
    this->TextColor=RGB(0,0,0);
    this->TextBackgroundColor=RGB(0xff,0xff,0xff);
    this->BackgroundColor=RGB(0xff,0xff,0xff);
    this->pBackgroundBitmap=NULL;
    this->BackgroundImagePosition=BACKGROUND_IMAGE_POSITION_STRETCHED;

    this->bTrashArrangeMessages = FALSE;
    this->bOneClickActivate=FALSE;
    this->bShowSubFolders=TRUE;
    this->bShowDrives=TRUE;

    this->ListViewCreationCallBack=NULL;
    this->ListViewCreationCallBackUserParam=NULL;
    this->ViewChangedCallBackUserParam=NULL;
    this->ViewChangedCallBack=NULL;

    this->hWndListView=NULL;
    this->pFilesFiltersInclusion=NULL;
    this->pFilesFiltersExclusion=NULL;

    this->bIsEditing=FALSE;

    this->hHandCursor=::LoadCursor(NULL,IDC_HAND);

    this->BackgroundGradiantColor1=RGB(0,0xff,0);
    this->BackgroundGradiantColor2=RGB(0,0,0xff);
    this->BackgroundGradiantType=Gdiplus::LinearGradientModeHorizontal;

    *this->BackgroundImagePath = 0;
    this->bSettingBackGroundImageInternally=FALSE;
    this->bApplyingTextStyle = FALSE;

    //enum InterpolationMode
    //{
    //    InterpolationModeInvalid          = QualityModeInvalid,
    //    InterpolationModeDefault          = QualityModeDefault,
    //    InterpolationModeLowQuality       = QualityModeLow,
    //    InterpolationModeHighQuality      = QualityModeHigh,
    //    InterpolationModeBilinear,
    //    InterpolationModeBicubic,
    //    InterpolationModeNearestNeighbor,
    //    InterpolationModeHighQualityBilinear,
    //    InterpolationModeHighQualityBicubic
    //};
    this->BackgroundPictureInterpolationMode=Gdiplus::InterpolationModeHighQualityBicubic;
}

//-----------------------------------------------------------------------------
// Name: ~CShellBrowserEx
// Object: Destructor.
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
CShellBrowserEx::~CShellBrowserEx(void)
{
    if (this->pBackgroundBitmap)
        delete this->pBackgroundBitmap;

    if (this->pFilesFiltersInclusion)
        delete this->pFilesFiltersInclusion;

    if (this->pFilesFiltersExclusion)
        delete this->pFilesFiltersExclusion;

    // at last
    // assume GdiplusShutdown is not in the same function that stack allocated objects
    Gdiplus::GdiplusShutdown(this->gdiplusToken);
}

//-----------------------------------------------------------------------------
// Name: HideListViewHeaders
// Object: assume header will be hidden
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
#if _WIN32_WINNT < 0x0600
#define LVS_EX_JUSTIFYCOLUMNS   0x00200000  // Icons are lined up in columns that use up the whole view area.
#define LVS_EX_TRANSPARENTBKGND 0x00400000  // Background is painted by the parent via WM_PRINTCLIENT
#define LVS_EX_TRANSPARENTSHADOWTEXT 0x00800000  // Enable shadow text on transparent backgrounds only (useful with bitmaps)
#define LVS_EX_AUTOAUTOARRANGE  0x01000000  // Icons automatically arrange if no icon positions have been set
#define LVS_EX_HEADERINALLVIEWS 0x02000000  // Display column header in all view modes
#define LVS_EX_AUTOCHECKSELECT  0x08000000
#define LVS_EX_AUTOSIZECOLUMNS  0x10000000
#define LVS_EX_COLUMNSNAPPOINTS 0x40000000
#define LVS_EX_COLUMNOVERFLOW   0x80000000
#endif
void CShellBrowserEx::AdjustListViewStyle()
{
    DWORD ExStyle = ListView_GetExtendedListViewStyle(this->hWndListView);
    ExStyle = ExStyle 
                & (~LVS_EX_HEADERINALLVIEWS) // disable columns view in all mode
                & (~LVS_EX_FULLROWSELECT) // avoid rect superposition when item position is specified
                // & (~LVS_EX_JUSTIFYCOLUMNS)
                // & (~LVS_EX_AUTOSIZECOLUMNS)
                // & (~LVS_EX_COLUMNOVERFLOW) 
            ;
    ListView_SetExtendedListViewStyle(this->hWndListView,ExStyle);
}

//-----------------------------------------------------------------------------
// Name: IsEditing
// Object: check if an item is being edited
// Parameters :
//     in  : 
//     out : 
//     return : TRUE if an item is being edited
//-----------------------------------------------------------------------------
BOOL CShellBrowserEx::IsEditing()
{
    return this->bIsEditing;
}


//-----------------------------------------------------------------------------
// Name: SetFiltersInclusion
// Object: check if an item is being edited
// Parameters :
//     in  : TCHAR* pszFiltersInclusion : inclusion filters (NULL to remove inclusion filters)
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowserEx::SetFiltersInclusion(TCHAR* pszFiltersInclusion)
{
    // delete previous filter if any
    if (this->pFilesFiltersInclusion)
    {
        delete this->pFilesFiltersInclusion;
        this->pFilesFiltersInclusion=NULL;
    }

    this->pFilesFiltersInclusion=new CFilesFilters(CFilesFilters::FILTER_TYPE_INCLUSION,pszFiltersInclusion);

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SetFiltersExclusion
// Object: check if an item is being edited
// Parameters :
//     in  : TCHAR* pszFiltersInclusion : inclusion filters (NULL to remove inclusion filters)
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowserEx::SetFiltersExclusion(TCHAR* pszFiltersExclusion)
{
    // delete previous filter if any
    if (this->pFilesFiltersExclusion)
    {
        delete this->pFilesFiltersExclusion;
        this->pFilesFiltersExclusion=NULL;
    }

    this->pFilesFiltersExclusion=new CFilesFilters(CFilesFilters::FILTER_TYPE_EXCLUSION,pszFiltersExclusion);

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SetBackgroundType
// Object: set background type
// Parameters :
//     in  : tagBackgroundType BackgroundType : new background type to be used
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowserEx::SetBackgroundType(tagBackgroundType BackgroundType)
{
    this->BackgroundType=BackgroundType;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SetTextBackgroundColor
// Object: set text background color
// Parameters :
//     in  : COLORREF TextBackgroundColor
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowserEx::SetTextBackgroundColor(COLORREF TextBackgroundColor)
{
    this->TextBackgroundColor=TextBackgroundColor;
    this->TextBackgroundColorType=COLOR_TYPE_SPECIFIED;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SetTextColor
// Object: set text color
// Parameters :
//     in  : COLORREF TextColor
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowserEx::SetTextColor(COLORREF TextColor)
{
    this->TextColor=TextColor;
    this->TextColorType=COLOR_TYPE_SPECIFIED;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SetBackgroundColor
// Object: set background color
// Parameters :
//     in  : COLORREF BackgroundColor
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowserEx::SetBackgroundColor(COLORREF BackgroundColor)
{
    this->BackgroundColor=BackgroundColor;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SetBackgroundGradient
// Object: set background gradient
// Parameters :
//     in  : COLORREF BackgroundGradiantColor1
//           COLORREF BackgroundGradiantColor2
//           Gdiplus::LinearGradientMode BackgroundGradiantType
//                      typedef enum {
//                          LinearGradientModeHorizontal = 0,
//                          LinearGradientModeVertical = 1,
//                          LinearGradientModeForwardDiagonal = 2,
//                          LinearGradientModeBackwardDiagonal = 3
//                      } Gdiplus::LinearGradientMode;
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowserEx::SetBackgroundGradient(COLORREF BackgroundGradiantColor1,COLORREF BackgroundGradiantColor2,Gdiplus::LinearGradientMode BackgroundGradiantType)
{
    // store parameters in case of resize
    this->BackgroundGradiantColor1=BackgroundGradiantColor1;
    this->BackgroundGradiantColor2=BackgroundGradiantColor2;
    this->BackgroundGradiantType=BackgroundGradiantType;

    /////////////////////////
    // create destination bitmap
    ///////////////////////// 
    LONG ControlWith=this->BrowserRect.right-this->BrowserRect.left;
    LONG ControlHeight=this->BrowserRect.bottom-this->BrowserRect.top;
    if (this->pBackgroundBitmap)
    {
        Gdiplus::Bitmap* pBmp=this->pBackgroundBitmap;
        this->pBackgroundBitmap=NULL;
        delete pBmp;
    }
    this->pBackgroundBitmap=new Gdiplus::Bitmap(ControlWith,ControlHeight,PixelFormat32bppARGB);

    /////////////////////////
    // create graphic associated to destination bitmap
    ///////////////////////// 
    Gdiplus::Graphics objGraphic(this->pBackgroundBitmap);
    if (objGraphic.GetLastStatus()!=Gdiplus::Ok)
        return FALSE;

    Gdiplus::Rect GdiplusBrowserRect(0,0,ControlWith,ControlHeight);
    Gdiplus::Color Color1;
    Gdiplus::Color Color2;
    Color1.SetFromCOLORREF(BackgroundGradiantColor1 | 0xff000000); // "| 0xff000000" --> convert RGB to RGBA
    Color2.SetFromCOLORREF(BackgroundGradiantColor2 | 0xff000000); // "| 0xff000000" --> convert RGB to RGBA
    // create gradient
    Gdiplus::LinearGradientBrush objLinearGradientBrush(GdiplusBrowserRect,
                                                        Color1,
                                                        Color2,
                                                        BackgroundGradiantType
                                                        );


    // draw gradient to destination bitmap
    return (objGraphic.FillRectangle(&objLinearGradientBrush,GdiplusBrowserRect)==Gdiplus::Ok);
}

//-----------------------------------------------------------------------------
// Name: SetBackgroundPicture
// Object: set background picture
// Parameters :
//     in  : TCHAR* ImagePath
//           tagBackgroundImagePosition BackgroundImagePosition
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowserEx::SetBackgroundPicture(TCHAR* ImagePath,tagBackgroundImagePosition BackgroundImagePosition)
{
    // store parameters in case of resize
    _tcsncpy(this->BackgroundImagePath,ImagePath,MAX_PATH);
    this->BackgroundImagePosition=BackgroundImagePosition;

    WCHAR wcsImagePath[MAX_PATH];
#if (defined(UNICODE)||defined(_UNICODE))
    wcsncpy(wcsImagePath,ImagePath,MAX_PATH);
#else
    CAnsiUnicodeConvert::AnsiToUnicode(ImagePath,wcsImagePath,MAX_PATH);
#endif

    Gdiplus::Image objImage(wcsImagePath,TRUE);
    if (objImage.GetLastStatus()!=Gdiplus::Ok)
        return FALSE;

    LONG ImageWidth=objImage.GetWidth();
    LONG ImageHeight=objImage.GetHeight();


    /////////////////////////
    // create destination bitmap
    ///////////////////////// 
    LONG ControlWith=this->BrowserRect.right-this->BrowserRect.left;
    LONG ControlHeight=this->BrowserRect.bottom-this->BrowserRect.top;
    if (this->pBackgroundBitmap)
    {
        Gdiplus::Bitmap* pBmp=this->pBackgroundBitmap;
        this->pBackgroundBitmap=NULL;
        delete pBmp;
    }
    this->pBackgroundBitmap=new Gdiplus::Bitmap(ControlWith,ControlHeight,objImage.GetPixelFormat());

    /////////////////////////
    // create graphic associated to destination bitmap
    ///////////////////////// 
    Gdiplus::Graphics objGraphic(this->pBackgroundBitmap);
    if (objGraphic.GetLastStatus()!=Gdiplus::Ok)
        return FALSE;

    Gdiplus::Status RetValue;

    LONG xLeft=0;
    LONG xCenter=(ControlWith-ImageWidth)/2;
    LONG xRight=ControlWith-ImageWidth;
    LONG yUp=0;
    LONG yCenter=(ControlHeight-ImageHeight)/2;
    LONG yDown=ControlHeight-ImageHeight;

    // draw resized image in destination bitmap
    switch(BackgroundImagePosition)
    {
    case BACKGROUND_IMAGE_POSITION_UP_LEFT:
        RetValue=objGraphic.DrawImage(&objImage,xLeft,yUp,ImageWidth,ImageHeight);
        break;
    case BACKGROUND_IMAGE_POSITION_UP_CENTER:
        RetValue=objGraphic.DrawImage(&objImage,xCenter,yUp,ImageWidth,ImageHeight);
        break;
    case BACKGROUND_IMAGE_POSITION_UP_RIGHT:
        RetValue=objGraphic.DrawImage(&objImage,xRight,yUp,ImageWidth,ImageHeight);
        break;

    case BACKGROUND_IMAGE_POSITION_LEFT:
        RetValue=objGraphic.DrawImage(&objImage,xLeft,yCenter,ImageWidth,ImageHeight);
        break;
    case BACKGROUND_IMAGE_POSITION_CENTER:
        RetValue=objGraphic.DrawImage(&objImage,xCenter,yCenter,ImageWidth,ImageHeight);
        break;
    case BACKGROUND_IMAGE_POSITION_RIGHT:
        RetValue=objGraphic.DrawImage(&objImage,xRight,yCenter,ImageWidth,ImageHeight);
        break;

    case BACKGROUND_IMAGE_POSITION_BOTTOM_LEFT:
        RetValue=objGraphic.DrawImage(&objImage,xLeft,yDown,ImageWidth,ImageHeight);
        break;
    case BACKGROUND_IMAGE_POSITION_BOTTOM_CENTER:
        RetValue=objGraphic.DrawImage(&objImage,xCenter,yDown,ImageWidth,ImageHeight);
        break;
    case BACKGROUND_IMAGE_POSITION_BOTTOM_RIGHT:
        RetValue=objGraphic.DrawImage(&objImage,xRight,yDown,ImageWidth,ImageHeight);
        break;

    case BACKGROUND_IMAGE_POSITION_STRETCHED:
    default:
        // set InterpolationMode
        objGraphic.SetInterpolationMode(this->BackgroundPictureInterpolationMode);

        // draw resized image
        RetValue=objGraphic.DrawImage(&objImage,xLeft,yUp,ControlWith,ControlHeight);
        break;
    }

    return (RetValue==Gdiplus::Ok);
}

//-----------------------------------------------------------------------------
// Name: UpdateRect
// Object: update browser rect (virtual)
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CShellBrowserEx::UpdateRect(RECT* pRect)
{
    return this->InternalUpdateRect(pRect,FALSE);
}

//-----------------------------------------------------------------------------
// Name: InternalUpdateRect
// Object: update browser rect
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
HRESULT CShellBrowserEx::InternalUpdateRect(RECT* pRect,BOOL bForceRedraw)
{
    if (!bForceRedraw)
    {
        if (::EqualRect(&this->BrowserRect,pRect))
            return S_OK;
    }

    CShellBrowser::InternalUpdateRect(pRect,bForceRedraw);

    switch (this->BackgroundType)
    {
    case BACKGROUND_TYPE_PICTURE:
        this->SetBackgroundPicture(this->BackgroundImagePath,this->BackgroundImagePosition);
        this->ApplyBackground();
        break;
    case BACKGROUND_TYPE_GRADIENT:
        this->SetBackgroundGradient(this->BackgroundGradiantColor1,this->BackgroundGradiantColor2,this->BackgroundGradiantType);
        this->ApplyBackground();
        break;
    }
    return S_OK;
}

STDMETHODIMP CShellBrowserEx::IncludeObject(THIS_ struct IShellView *ppshv,
                                          LPCITEMIDLIST pidl)
{
    UNREFERENCED_PARAMETER(ppshv);
    return this->InternalIncludeObject(pidl);
}

BOOL CShellBrowserEx::IsDrive(LPCITEMIDLIST pidl)
{
    TCHAR szPath[MAX_PATH];
    *szPath=0;
    // get path from pidl
    ::SHGetPathFromIDList(pidl, szPath);
    return (_tcslen(szPath) == 3);
}

HRESULT CShellBrowserEx::InternalIncludeObject(LPCITEMIDLIST pidl)
{
    /////////////////////////////////
    // add folders and shortcuts to folders according to this->bShowSubFolders
    /////////////////////////////////

    BOOL bIsFolderOrFolderLink=FALSE;
    // check if folder
    if (this->IsFolder(pidl,this->pShellFolder))
    {
        bIsFolderOrFolderLink=TRUE;
        if (!this->bShowDrives)
        {
            if (IsDrive(pidl))
                return S_FALSE;
        }
    }
    else
    {
        // check if folder link
        LPITEMIDLIST pidlTo = NULL;
        if (this->IsFolderLink(pidl,&pidlTo))
        {
            ::ILFree(pidlTo);
            bIsFolderOrFolderLink=TRUE;
        }
    }

    // if folder or folder link
    if (bIsFolderOrFolderLink)
    {
        // if folder must be shown
        if (this->bShowSubFolders)
            return S_OK;
        else
            return S_FALSE;
    }

    /////////////////////////////////
    // for files
    /////////////////////////////////

    TCHAR szPath[MAX_PATH];
    TCHAR* FileName;
    // get path from pidl
    ::SHGetPathFromIDList(pidl, szPath);
    FileName=CStdFileOperations::GetFileName(szPath);
    if (!FileName)
        return S_FALSE;

    // check inclusions filters
    if (this->pFilesFiltersInclusion)
    {
        if (!this->pFilesFiltersInclusion->Check(FileName))
            return S_FALSE;
    }

    // check exclusions filters
    if (this->pFilesFiltersExclusion)
    {
        if (!this->pFilesFiltersExclusion->Check(FileName))
            return S_FALSE;
    }

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: SetListViewCreationCallBack
// Object: allow to define a listview creation callback
// Parameters :
//     in  : pfListViewCreationCallBack CallBack : call back
//           PVOID UserParam : user param
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowserEx::SetListViewCreationCallBack(pfListViewCreationCallBack CallBack,PVOID UserParam)
{
    if (::IsBadCodePtr((FARPROC)CallBack))
        return FALSE;

    this->ListViewCreationCallBack=CallBack;
    this->ListViewCreationCallBackUserParam=UserParam;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SetViewChangedCallBack
// Object: allow to define a view changed callback
// Parameters :
//     in  : pfViewChangedCallBack CallBack : call back
//           PVOID UserParam : user param
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CShellBrowserEx::SetViewChangedCallBack(pfViewChangedCallBack CallBack,PVOID UserParam)
{
    if (::IsBadCodePtr((FARPROC)CallBack))
        return FALSE;

    this->ViewChangedCallBack=CallBack;
    this->ViewChangedCallBackUserParam=UserParam;

    return TRUE;
}

void CShellBrowserEx::InternalAfterListViewCreationCallBackVirtual()
{
    this->InternalAfterListViewCreationCallBack();
}

void CShellBrowserEx::InternalBeforeBrowseCallBackVirtual(LPCITEMIDLIST pidlToBeBrowsed)
{
    this->InternalBeforeBrowseCallBack(pidlToBeBrowsed);
}

void CShellBrowserEx::InternalBeforeBrowseCallBack(LPCITEMIDLIST pidlToBeBrowsed)
{
    this->CShellBrowser::InternalBeforeBrowseCallBack(pidlToBeBrowsed);
}

void CShellBrowserEx::InternalAfterViewChangedCallBackVirtual(tagShellBrowserView NewShellBrowserView)
{
    this->InternalAfterViewChangedCallBack(NewShellBrowserView);
}
void CShellBrowserEx::InternalAfterViewChangedCallBack(tagShellBrowserView NewShellBrowserView)
{
    // update CShellBrowser state to avoid to go back to initial mode
    this->ShellBrowserView = NewShellBrowserView;
    CShellBrowser::ShellBrowserViewToFolderSetting(NewShellBrowserView,&this->FolderSettings);
    this->AdjustListViewStyle();
}

void CShellBrowserEx::InternalBeforeViewChangedCallBackVirtual(tagShellBrowserView NewShellBrowserView)
{
    this->InternalBeforeViewChangedCallBack(NewShellBrowserView);
}
void CShellBrowserEx::InternalBeforeViewChangedCallBack(tagShellBrowserView NewShellBrowserView)
{
    UNREFERENCED_PARAMETER(NewShellBrowserView);
}

void CShellBrowserEx::InternalAfterListViewCreationCallBack()
{
    ::SetWindowTheme(this->hWndListView,NULL,L"");// removing theme allow to get old listview component which allow more changes
    this->CShellBrowser::InternalAfterListViewCreationCallBack();
    this->AdjustListViewStyle();
    this->ApplyTextStyle();
    this->ApplyBackground();

    // //  LVS_EX_ONECLICKACTIVATE generate a too slow navigation 
    // // --> hook WM_SETCURSOR and NM_CLICK instead
    //if (this->bOneClickActivate)
    //{
    //    DWORD Style=ListView_GetExtendedListViewStyle(this->hWndListView);
    //    Style|=LVS_EX_ONECLICKACTIVATE;
    //    Style&=~LVS_EX_UNDERLINECOLD;// remove underline
    //    Style&=~LVS_EX_UNDERLINEHOT;// remove underline
    //    ListView_SetExtendedListViewStyle(this->hWndListView,Style);
    //    ListView_SetHoverTime(this->hWndListView,0);
    //}

    this->SetListviewHook();// must be called after ApplyBackground (see CShellBrowserEx::SubClassProcListView)
    if (this->ListViewCreationCallBack)
        this->ListViewCreationCallBack(this->hWndListView,this->ListViewCreationCallBackUserParam);
}

//-----------------------------------------------------------------------------
// Name: SetListviewHook
// Object: set hooks
// Parameters :
//     in  : 
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CShellBrowserEx::SetListviewHook()
{
    UINT_PTR uIdSubclass=0;
    DWORD_PTR dwRefData=(DWORD_PTR)this;
    ::SetWindowSubclass(this->hWndListView,CShellBrowserEx::SubClassProcListView,uIdSubclass,dwRefData);
    ::SetWindowSubclass(this->hWndShellView,CShellBrowserEx::SubClassProcShellView,uIdSubclass,dwRefData);
}
HRESULT CShellBrowserEx::Refresh()
{
    HRESULT hResult = this->CShellBrowser::Refresh();
    this->ApplyTextStyle();
    return hResult;
}

BOOL CShellBrowserEx::IsViewChangeCommandMessage(WPARAM wParam,OUT SIZE_T* pViewIndex)
{
    SIZE_T ViewIndex;
    *pViewIndex = 0;

    for (ViewIndex = 0;ViewIndex<SHELL_BROWSER_VIEW_COUNT;ViewIndex++)
    {
        if (wParam == this->ShellBrowserViewInfo[ViewIndex].Value)
        {
            *pViewIndex = ViewIndex;
            return TRUE;
        }
    }

    return FALSE;
}

//-----------------------------------------------------------------------------
// Name: SubClassProcShellView
// Object: shell browser control subclassing
// Parameters :
//     in  : 
//     out : 
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CShellBrowserEx::SubClassProcShellView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData)
{
    UNREFERENCED_PARAMETER(uIdSubclass);
    switch (uMsg)
    {
    case WM_COMMAND:
        {
            CShellBrowserEx* pShellBrowserEx=(CShellBrowserEx*)dwRefData;
            BOOL bViewChangeMessage;
            SIZE_T ViewIndex;

            // if message is a view changed message
            bViewChangeMessage = pShellBrowserEx->IsViewChangeCommandMessage(wParam,&ViewIndex);

            if (bViewChangeMessage)
            {
                
                pShellBrowserEx->InternalBeforeViewChangedCallBackVirtual((tagShellBrowserView)ViewIndex);

                // do default actions
                LRESULT lResult=::DefSubclassProc(hWnd,uMsg,wParam,lParam);

#ifdef _DEBUG
                switch(ViewIndex)
                {
                case SHELL_BROWSER_VIEW_ICONS:
                    OutputDebugString(_T("View switched to icons\r\n"));
            	    break;
                case SHELL_BROWSER_VIEW_SMALL_ICONS:
                    OutputDebugString(_T("View switched to small icons\r\n"));
            	    break;
                case SHELL_BROWSER_VIEW_LIST:
                    OutputDebugString(_T("View switched to list\r\n"));
                    break;
                case SHELL_BROWSER_VIEW_DETAIL:
                    OutputDebugString(_T("View switched to detail\r\n"));
                    break;
                case SHELL_BROWSER_VIEW_THUMBNAILS:
                    OutputDebugString(_T("View switched to thumbs\r\n"));
                    break;
                case SHELL_BROWSER_VIEW_TILES:
                    OutputDebugString(_T("View switched to tiles\r\n"));
                    break;
                case SHELL_BROWSER_VIEW_EXTRA_LARGE:
                    OutputDebugString(_T("View switched to extra large\r\n"));
                    break;
                case SHELL_BROWSER_VIEW_MEDIUM:
                    OutputDebugString(_T("View switched to medium\r\n"));
                    break;
                case SHELL_BROWSER_VIEW_LARGE:
                    OutputDebugString(_T("View switched to large\r\n"));
                    break;
                default:
                    OutputDebugString(_T("View switched to ???\r\n"));
                    break;
                }
#endif

                pShellBrowserEx->InternalAfterViewChangedCallBackVirtual((tagShellBrowserView)ViewIndex);
                // if a view changed callback has been defined
                if (pShellBrowserEx->ViewChangedCallBack)
                    // call callback
                    pShellBrowserEx->ViewChangedCallBack(pShellBrowserEx->hWndListView,(tagShellBrowserView)ViewIndex,pShellBrowserEx->ViewChangedCallBackUserParam);

                // return default actions result
                return lResult;
            }

            else
            {
#ifdef _DEBUG
                TCHAR Msg[512];
                _sntprintf (Msg, _countof(Msg), _T("ShellBrowser WM_COMMAND wParam: 0x%p  lParam:0x%p\r\n"),wParam,lParam);
                OutputDebugString(Msg);
#endif
                if (pShellBrowserEx->bTrashArrangeMessages && CShellBrowser::IsSortingMessage(wParam))
                {
                    // discard arrange message to avoid dead locks

                    // trash message
                    return 0;
                }
            }
        }
        break;
    case WM_SETCURSOR:
        {
            CShellBrowserEx* pShellBrowserEx=(CShellBrowserEx*)dwRefData;

            if (pShellBrowserEx->bOneClickActivate)
            {
                LVHITTESTINFO lvhittest={0};
                // get cursor pos
                ::GetCursorPos(&lvhittest.pt);
                // get cursor pos in client coordinates
                ::ScreenToClient(pShellBrowserEx->hWndListView,&lvhittest.pt);
                // do hit test
                ::SendMessage(pShellBrowserEx->hWndListView,LVM_HITTEST,0,(LPARAM)&lvhittest);

                // if mouse is on icon, label or state icon
                if (
                        (   
                            (lvhittest.flags==LVHT_ONITEMICON)
                            || (lvhittest.flags==LVHT_ONITEMLABEL)
                            || (lvhittest.flags==LVHT_ONITEMSTATEICON)
                        )
                        &&
                        (lvhittest.iItem>=0)// LVHT_ONITEMSTATEICON sucks sometimes (lvhittest.iItem=-1)
                    )

                {
                    // set hand cursor
                    ::SetCursor(pShellBrowserEx->hHandCursor);
                    // stop WM_SETCURSOR processing
                    ::SetWindowLong(pShellBrowserEx->hWndParent,DWLP_MSGRESULT,TRUE);
                    return TRUE;
                }
            }
        }
        break;

    case WM_NOTIFY:
        {
            NMHDR* pnmh=((NMHDR*)lParam);

            switch (pnmh->code)
            {
                // uncomment the following to use background color instead of text background color in report mode
                //case NM_CUSTOMDRAW:
                //{
                //    CShellBrowserEx* pShellBrowserEx=(CShellBrowserEx*)dwRefData;

                //    // listview messages
                //    if (pnmh->hwndFrom==pShellBrowserEx->hWndListView)
                //    {

                //        // if a background color is specified
                //        if (pShellBrowserEx->TextBackgroundColorType==COLOR_TYPE_NONE)
                //            break;

                //        // troubles occurs only for detail view
                //        if (ListView_GetView(pShellBrowserEx->hWndListView)!=LV_VIEW_DETAILS)
                //            break;

                //        LPNMLVCUSTOMDRAW  lplvcd = (LPNMLVCUSTOMDRAW)pnmh;
                //        
                //        switch(lplvcd->nmcd.dwDrawStage)
                //        {
                //        case CDDS_PREPAINT:
                //            return CDRF_NOTIFYITEMDRAW;

                //        case CDDS_ITEMPREPAINT:
                //            ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
                //            lplvcd->clrTextBk=pShellBrowserEx->BackgroundColor;
                //            return CDRF_DODEFAULT;
                //        }
                //    }
                //}
                //break;

            case LVN_BEGINLABELEDITA:
            case LVN_BEGINLABELEDITW:
                {
                    CShellBrowserEx* pShellBrowserEx=(CShellBrowserEx*)dwRefData;
                    pShellBrowserEx->bIsEditing=TRUE;
                }
                break;
            case LVN_ENDLABELEDITA:
            case LVN_ENDLABELEDITW:
                {
                    CShellBrowserEx* pShellBrowserEx=(CShellBrowserEx*)dwRefData;
                    pShellBrowserEx->bIsEditing=FALSE;
                }
                break;
            case NM_CLICK:
                {
                    CShellBrowserEx* pShellBrowserEx=(CShellBrowserEx*)dwRefData;
                    if (pShellBrowserEx->bOneClickActivate)
                    {
                        LPNMITEMACTIVATE lpnmitem = (LPNMITEMACTIVATE) lParam;
                        if (lpnmitem->iItem!=-1)
                        {
                            if (pShellBrowserEx->OnDefaultCommand(pShellBrowserEx->pShellView)!=S_OK)
                            {
                                // assume only one item is selected
                                LONG_PTR Style=GetWindowLongPtr(pShellBrowserEx->hWndListView,GWL_STYLE);
                                SetWindowLongPtr(pShellBrowserEx->hWndListView,GWL_STYLE,Style | LVS_SINGLESEL);

                                // assume clicked item is the selected one
                                ListView_SetItemState (pShellBrowserEx->hWndListView,lpnmitem->iItem,LVIS_SELECTED,0x000F);

                                // get pidl associated to selected item
                                CIDA *pData = pShellBrowserEx->BeginGetItems(TRUE);
                                if (pData)
                                {
                                    // get first selected item
                                    LPCITEMIDLIST pidl = pShellBrowserEx->GetPIDLItem(pData,0);

                                    if (pidl)
                                    {
                                        // warning pidl is relative to parent pidl --> get full idl
                                        LPITEMIDLIST pToExecuteIdl;
                                        pToExecuteIdl=::ILCombine(pShellBrowserEx->FolderPidl,pidl);

                                        if (pToExecuteIdl)
                                        {
                                            LPITEMIDLIST pTargetIdl;
                                            TCHAR lpParameters[MAX_PATH];
                                            *lpParameters=0;
                                            TCHAR lpDirectory[MAX_PATH];
                                            *lpDirectory=0;
                                            int nShow = SW_NORMAL;

                                            if (pShellBrowserEx->IsLink(pToExecuteIdl,&pTargetIdl))
                                            {
                                                if (pTargetIdl)
                                                {
                                                    HRESULT hResult;
                                                    // Get the link object.
                                                    IShellLink* pLink=NULL;
                                                    hResult = pShellBrowserEx->pShellFolder->GetUIObjectOf(pShellBrowserEx->hWndParent,1,(LPCITEMIDLIST*)&pToExecuteIdl,IID_IShellLink,0,(void**)&pLink);
                                                    if (SUCCEEDED(hResult) && pLink)
                                                    {
                                                        pLink->GetArguments(lpParameters,MAX_PATH);
                                                        pLink->GetWorkingDirectory(lpDirectory,MAX_PATH);
                                                        pLink->GetShowCmd(&nShow);
                                                        pLink->Release();
                                                    }

                                                    ::ILFree(pToExecuteIdl);
                                                    pToExecuteIdl=pTargetIdl;
                                                }

                                            }
                                            else
                                            {
                                                ::SHGetPathFromIDList(pShellBrowserEx->FolderPidl,lpDirectory);
                                            }

                                            SHELLEXECUTEINFO ShellInfo;
                                            memset(&ShellInfo, 0, sizeof(ShellInfo));
                                            ShellInfo.cbSize = sizeof(SHELLEXECUTEINFO);
                                            ShellInfo.lpVerb =  NULL; // do not use _T("open") : it can fail on some cases
                                            ShellInfo.nShow = nShow; 
                                            ShellInfo.lpParameters=lpParameters;
                                            ShellInfo.lpDirectory=lpDirectory;
                                            ShellInfo.lpIDList = (LPVOID)pToExecuteIdl;
                                            ShellInfo.fMask = SEE_MASK_IDLIST;
                                            ShellInfo.hwnd = pShellBrowserEx->hWndShellView;

                                            // open selected item
                                            // if (!::ShellExecuteEx(&ShellInfo))
                                            if (!::CShellExecuteAndBringToTop::ShellExecuteAndBringToTop(&ShellInfo))
                                            {
                                                CAPIError::ShowLastError();
                                            }
                                            else if ((int)ShellInfo.hInstApp<33)
                                            {
                                                TCHAR sz[MAX_PATH];
                                                _tcscpy(sz,_T("Error Opening File"));
                                                ::MessageBox(NULL,sz,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                                            }
                                            pShellBrowserEx->EndGetItems();

                                            ::ILFree(pToExecuteIdl);
                                        }
                                    }
                                }

                                // restore style (multi selection)
                                SetWindowLongPtr(pShellBrowserEx->hWndListView,GWL_STYLE,Style);
                            }
                        }
                    }
                }
                break;
//#ifdef _DEBUG
//            default:
//                {
//                    if (LVN_FIRST >= pnmh->code) // LVN_XX is defined as LVN_FIRST - Value
//                    {
//                        TCHAR sz[MAX_PATH];
//                        _stprintf(sz,_T("CShellBrowserEx::SubClassProcShellView LVN_FIRST-%u\r\n"),LVN_FIRST - pnmh->code);
//                        OutputDebugString(sz);
//                    }
//                }
//                break;
//#endif
            }
        }
        break;
//#ifdef _DEBUG
//    default:
//        if (uMsg>=LVN_FIRST)
//        {
//            TCHAR sz[512];
//            _stprintf(sz,_T("CShellBrowserEx::SubClassProcShellView LVN_FIRST+%u\r\n"),uMsg-LVN_FIRST);
//            OutputDebugString(sz);
//        }
//        if (uMsg>=WM_USER)
//        {
//            TCHAR sz[512];
//            _stprintf(sz,_T("CShellBrowserEx::SubClassProcShellView WM_USER+%u\r\n"),uMsg-WM_USER);
//            OutputDebugString(sz);
//        }
//        else
//        {
//            TCHAR sz[512];
//            _stprintf(sz,_T("CShellBrowserEx::SubClassProcShellView Message=0x%X wParam=0x%X lParam=0x%X\r\n"),uMsg,wParam,lParam);
//            OutputDebugString(sz);
//        }
//        break;
//#endif
    }
    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
}

//-----------------------------------------------------------------------------
// Name: SubClassProcListView
// Object: listview subclassing
// Parameters :
//     in  : 
//     out : 
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK CShellBrowserEx::SubClassProcListView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData)
{
    UNREFERENCED_PARAMETER(uIdSubclass);
    CShellBrowserEx* pShellBrowserEx = (CShellBrowserEx*)dwRefData;
    switch (uMsg)
    {
        case WM_COMMAND:
            if (pShellBrowserEx->bTrashArrangeMessages && CShellBrowser::IsSortingMessage(wParam))
            {
                // discard arrange message to avoid dead locks

                // trash message
                return 0;
            }
            break;

        // shell browser sends an LVM_SETBKIMAGE message on view change, overwriting our last LVM_SETBKIMAGE
        // just trash message
        case LVM_SETBKIMAGEA:
        case LVM_SETBKIMAGEW:
            {
                if (pShellBrowserEx->bSettingBackGroundImageInternally)
                {
                    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
                }
            }
            return TRUE;

        // shell browser sends an LVM_SETTEXTCOLOR message on view change, overwriting our last LVM_SETTEXTCOLOR
        // just trash message
        case LVM_SETTEXTCOLOR:
        case LVM_SETTEXTBKCOLOR:
        case LVM_SETBKCOLOR:
            if (pShellBrowserEx->bApplyingTextStyle)
            {
                return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
            }
            else
                return TRUE;

        case LVM_SETSELECTEDCOLUMN:// sadly not send on seven :(
            // if a background color / image has been set, avoid selected column background color
            if (pShellBrowserEx->BackgroundType!=BACKGROUND_TYPE_NONE)
                return TRUE;
            break;

        case WM_XBUTTONDOWN: // handle mouse media buttons
            {
                WORD fwKeys = GET_KEYSTATE_WPARAM (wParam); 
                // WORD fwButton = GET_XBUTTON_WPARAM (wParam);
                // int xPos = GET_X_LPARAM(lParam); 
                // int yPos = GET_Y_LPARAM(lParam);
                switch (fwKeys)
                {
                case MK_XBUTTON1: // backward
                    pShellBrowserEx->BrowseBack();
                    return TRUE;

                case MK_XBUTTON2: // forward
                    pShellBrowserEx->BrowseForward();
                    return TRUE;
                }
                break;
            }
//#ifdef _DEBUG
//        default:
//            if (uMsg>=LVM_FIRST)
//            {
//                TCHAR sz[512];
//                _stprintf(sz,_T("CShellBrowserEx::SubClassProcListView LVM_FIRST+%u\r\n"),uMsg-LVM_FIRST);
//                OutputDebugString(sz);
//            }
//
//            if (uMsg>=WM_USER)
//            {
//                TCHAR sz[512];
//                _stprintf(sz,_T("CShellBrowserEx::SubClassProcListView WM_USER+%u\r\n"),uMsg-WM_USER);
//                OutputDebugString(sz);
//            }
//            else
//            {
//                TCHAR sz[512];
//                _stprintf(sz,_T("CShellBrowserEx::SubClassProcListView Message=0x%X wParam=0x%X lParam=0x%X\r\n"),uMsg,wParam,lParam);
//                OutputDebugString(sz);
//            }
//            break;
//#endif
    }
    return ::DefSubclassProc(hWnd,uMsg,wParam,lParam);
}

//-----------------------------------------------------------------------------
// Name: ApplyTextStyle
// Object: apply defined text style
// Parameters :
//     in  : 
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CShellBrowserEx::ApplyTextStyle()
{
    this->bApplyingTextStyle = TRUE;
    // set text color
    switch(this->TextColorType)
    {
    case COLOR_TYPE_SPECIFIED:
        ListView_SetTextColor(this->hWndListView,this->TextColor);// send LVM_SETTEXTCOLOR
        break;
    case COLOR_TYPE_NONE:
    default:
        break;
    }

    // set text background color
    switch(this->TextBackgroundColorType)
    {
    case COLOR_TYPE_SPECIFIED:
        ListView_SetTextBkColor(this->hWndListView,this->TextBackgroundColor);
        break;
    case COLOR_TYPE_NONE:
    default:
        break;
    }
    this->bApplyingTextStyle = FALSE;
}

//-----------------------------------------------------------------------------
// Name: ApplyBackground
// Object: apply defined background
// Parameters :
//     in  : 
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CShellBrowserEx::ApplyBackground()
{
    // set background
    switch(this->BackgroundType)
    {
    case BACKGROUND_TYPE_COLOR:
        // avoid selected column (selected column is paint over background)
        ListView_SetSelectedColumn(this->hWndListView,-1);

        // set background color
        ListView_SetBkColor(this->hWndListView,this->BackgroundColor);

    	break;
    case BACKGROUND_TYPE_GRADIENT:
    case BACKGROUND_TYPE_PICTURE:
        {
            if (this->pBackgroundBitmap)
            {
                // avoid selected column (selected column is paint over background)
                ListView_SetSelectedColumn(this->hWndListView,-1);

                // set background image
                HBITMAP hBmp;
                this->pBackgroundBitmap->GetHBITMAP(ListView_GetBkColor(this->hWndListView),&hBmp);

                // avoid to trash our background change message
                this->bSettingBackGroundImageInternally=TRUE;

                LVBKIMAGE LvBkImage={0};
                LvBkImage.hbm=hBmp;
                LvBkImage.ulFlags= LVBKIF_TYPE_WATERMARK;
                // FIXME : Control Panel>System>Advanced>Performance>Settings>Visual Effect / "Use background image for each folder type"
                // must be enabled, else window messages like LVM_SETBKIMAGE are trashed
                // so for gradiant and picture background, this option must be checked
                ListView_SetBkImage(this->hWndListView,&LvBkImage);

                this->bSettingBackGroundImageInternally=FALSE;
            }
#ifdef _DEBUG
            else
            {
                if (IsDebuggerPresent())
                    DebugBreak();
            }
#endif
        }
        break;
    case BACKGROUND_TYPE_NONE:
    default:
        break;
    }

}