/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manage a shell browser with extended features
//-----------------------------------------------------------------------------

#pragma once
#include "ShellBrowser.h"
#include "../../String/AnsiUnicodeConvert.h"
#include "../../File/StdFileOperations.h"
#include "../../File/FilesFilters.h"
#include "../../APIError/APIError.h"

#include <commctrl.h>
#pragma comment (lib,"comctl32.lib")

#include <gdiplus.h>
#pragma comment (lib,"gdiplus.lib")

#include <uxtheme.h>
#pragma comment(lib,"UxTheme.lib")

class CShellBrowserEx : public CShellBrowser
{
public:
    enum tagBackgroundType
    {
        BACKGROUND_TYPE_NONE,
        BACKGROUND_TYPE_COLOR,
        BACKGROUND_TYPE_GRADIENT,
        BACKGROUND_TYPE_PICTURE,
    };

    enum tagColorType
    {
        COLOR_TYPE_NONE,
        COLOR_TYPE_SPECIFIED
    };

    enum tagBackgroundImagePosition
    {
        BACKGROUND_IMAGE_POSITION_STRETCHED=0,
        BACKGROUND_IMAGE_POSITION_CENTER,
        BACKGROUND_IMAGE_POSITION_UP_LEFT,
        BACKGROUND_IMAGE_POSITION_UP_CENTER,
        BACKGROUND_IMAGE_POSITION_UP_RIGHT,
        BACKGROUND_IMAGE_POSITION_LEFT,
        BACKGROUND_IMAGE_POSITION_RIGHT,
        BACKGROUND_IMAGE_POSITION_BOTTOM_LEFT,
        BACKGROUND_IMAGE_POSITION_BOTTOM_CENTER,
        BACKGROUND_IMAGE_POSITION_BOTTOM_RIGHT
    };

    CShellBrowserEx(HWND hWndParent);
    CShellBrowserEx(HWND hWndParent,FOLDERSETTINGS* pfs);
    CShellBrowserEx(HWND hWndParent,FOLDERSETTINGS* pfs,RECT* pBrowserRect);
    CShellBrowserEx(HWND hWndParent,FOLDERSETTINGS* pfs,tagShellBrowserView ShellBrowserView);
    CShellBrowserEx(HWND hWndParent,FOLDERSETTINGS* pfs,tagShellBrowserView ShellBrowserView,RECT* pBrowserRect);
    ~CShellBrowserEx(void);

    BOOL SetBackgroundType(tagBackgroundType BackgroundType);
    BOOL SetTextColor(COLORREF TextColor);
    BOOL SetTextBackgroundColor(COLORREF TextBackgroundColor);
    BOOL SetBackgroundColor(COLORREF BackgroundColor);
    BOOL SetBackgroundGradient(COLORREF BackgroundGradiantColor1,COLORREF BackgroundGradiantColor2,Gdiplus::LinearGradientMode BackgroundGradiantType);
    BOOL SetBackgroundPicture(TCHAR* ImagePath,tagBackgroundImagePosition BackgroundImagePosition);

    //enum InterpolationMode
    //{
    //    InterpolationModeInvalid          = QualityModeInvalid,
    //    InterpolationModeDefault          = QualityModeDefault,
    //    InterpolationModeLowQuality       = QualityModeLow,
    //    InterpolationModeHighQuality      = QualityModeHigh,
    //    InterpolationModeBilinear,
    //    InterpolationModeBicubic,
    //    InterpolationModeNearestNeighbor,
    //    InterpolationModeHighQualityBilinear,
    //    InterpolationModeHighQualityBicubic
    //};
    Gdiplus::InterpolationMode BackgroundPictureInterpolationMode;

    BOOL bShowSubFolders;
    BOOL bShowDrives;
    BOOL bOneClickActivate;
    BOOL bTrashArrangeMessages;

    BOOL SetFiltersInclusion(TCHAR* pszFiltersInclusion);
    BOOL SetFiltersExclusion(TCHAR* pszFiltersExclusion);

    typedef void (*pfListViewCreationCallBack)(HWND hWndListView,LPVOID UserParam);
    BOOL SetListViewCreationCallBack(pfListViewCreationCallBack CallBack,PVOID UserParam);

    typedef void (*pfViewChangedCallBack)(HWND hWndListView,tagShellBrowserView NewView,LPVOID UserParam);
    BOOL SetViewChangedCallBack(pfViewChangedCallBack CallBack,PVOID UserParam);

    // ICommDlgBrowser::IncludeObject specific implementation
    STDMETHODIMP IncludeObject(THIS_ struct IShellView *ppshv,LPCITEMIDLIST pidl);

    BOOL IsDrive(LPCITEMIDLIST pidl);
    BOOL IsEditing();
    virtual HRESULT UpdateRect(RECT* pRect);
    virtual HRESULT Refresh();

protected:
    PVOID ListViewCreationCallBackUserParam;
    pfListViewCreationCallBack ListViewCreationCallBack;

    PVOID ViewChangedCallBackUserParam;
    pfViewChangedCallBack ViewChangedCallBack;

    BOOL bGdiPlusSuccessfullyStarted;

    tagBackgroundType BackgroundType;
    tagColorType TextColorType;
    tagColorType TextBackgroundColorType;
    COLORREF TextColor;
    COLORREF TextBackgroundColor;
    COLORREF BackgroundColor;
    TCHAR BackgroundImagePath[MAX_PATH];
    tagBackgroundImagePosition BackgroundImagePosition;
    COLORREF BackgroundGradiantColor1;
    COLORREF BackgroundGradiantColor2;
    Gdiplus::LinearGradientMode BackgroundGradiantType;
    BOOL bSettingBackGroundImageInternally;
    BOOL bApplyingTextStyle;


    BOOL bIsEditing;
    Gdiplus::Bitmap* pBackgroundBitmap;

    CFilesFilters* pFilesFiltersInclusion;
    CFilesFilters* pFilesFiltersExclusion;

    HCURSOR hHandCursor;
    ULONG_PTR gdiplusToken;
    void CommonConstructor();
    virtual void InternalAfterListViewCreationCallBackVirtual();
    void InternalAfterListViewCreationCallBack();
    virtual void InternalBeforeBrowseCallBackVirtual(LPCITEMIDLIST pidlToBeBrowsed);
    void InternalBeforeBrowseCallBack(LPCITEMIDLIST pidlToBeBrowsed);
    virtual void InternalBeforeViewChangedCallBackVirtual(tagShellBrowserView NewShellBrowserView);
    void InternalBeforeViewChangedCallBack(tagShellBrowserView NewShellBrowserView);
    virtual void InternalAfterViewChangedCallBackVirtual(tagShellBrowserView NewShellBrowserView);
    void InternalAfterViewChangedCallBack(tagShellBrowserView NewShellBrowserView);
    HRESULT InternalIncludeObject(LPCITEMIDLIST pidl);
    HRESULT InternalUpdateRect(RECT* pRect,BOOL bForceRedraw);
    void ApplyBackground();
    void ApplyTextStyle();
    void SetListviewHook();
    void AdjustListViewStyle();
    BOOL IsViewChangeCommandMessage(WPARAM wParam,OUT SIZE_T* pViewIndex);
    static LRESULT CALLBACK SubClassProcShellView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData);
    static LRESULT CALLBACK SubClassProcListView(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,UINT_PTR uIdSubclass,DWORD_PTR dwRefData);
};
