/*
Copyright (C) 2004-2016 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2016 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper for tooltip control
//-----------------------------------------------------------------------------

#include "tooltip.h"

//-----------------------------------------------------------------------------
// Name: CToolTip
// Object: Constructor.
// Parameters :
//     in  : HWND ControlHandle : target control of the tooltip
//           TCHAR* pszText : text shown in tooltip
//           BOOL BalloonTooltip : TRUE to set balloon style
//     out :
//     return : 
//-----------------------------------------------------------------------------
CToolTip::CToolTip(HINSTANCE hInstance,HWND ControlHandle,TCHAR* pszText,BOOL BalloonTooltip,BOOL bCenterTipOnControl)
{
    this->CommonConstructor(hInstance,ControlHandle,pszText,BalloonTooltip,bCenterTipOnControl,FALSE,FALSE);
}

CToolTip::CToolTip(HINSTANCE hInstance,HWND ControlHandle,TCHAR* pszText,TCHAR* Title,HICON hIcon,BOOL BalloonTooltip,BOOL bCenterTipOnControl)
{
    this->CommonConstructor(hInstance,ControlHandle,pszText,BalloonTooltip,bCenterTipOnControl,FALSE,FALSE);
    this->SetTitleWithIconHandle(Title,hIcon);
}
CToolTip::CToolTip(HINSTANCE hInstance,HWND ControlHandle,TCHAR* pszText,BOOL BalloonTooltip,BOOL bCenterTipOnControl,BOOL CloseButton,BOOL ParseLinks)
{
    this->CommonConstructor(hInstance,ControlHandle,pszText,BalloonTooltip,bCenterTipOnControl,CloseButton,ParseLinks);
}

// CommonConstructor : link should be inside text like TEXT("Some Text <A>http://www.xyz.com</A> some other text\r\nsome other text")
void CToolTip::CommonConstructor(HINSTANCE hInstance,HWND hWndParent,TCHAR* pszText,BOOL BalloonTooltip,BOOL bCenterTipOnControl,BOOL CloseButton,BOOL ParseLinks)
{
    // struct specifying control classes to register
    INITCOMMONCONTROLSEX iccex; 
    DWORD dwStyle;

    this->hInstance = hInstance;
    this->hWndParent = hWndParent;

#ifdef _DEBUG 
    if (ParseLinks && (pszText == LPSTR_TEXTCALLBACK) )
    {
        ::OutputDebugString(TEXT("CToolTip::CommonConstructor Warning LPSTR_TEXTCALLBACK is incompatible with link parsing at least on Win 7 64bit\r\n"));
        ::OutputDebugString(TEXT("CToolTip::CommonConstructor Warning Tooltips are affected when parent change and can stop working (no more TTN_GETDISPINFO through WM_NOTIFY message for LPSTR_TEXTCALLBACK) In that case ToolTip must be destroyed and created again\r\n"));
    }
#endif

    /* INITIALIZE COMMON CONTROLS */
    iccex.dwICC = ICC_WIN95_CLASSES;
    iccex.dwSize = sizeof(INITCOMMONCONTROLSEX);
    InitCommonControlsEx(&iccex);

    // set style
    dwStyle=WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP;
    if (BalloonTooltip)
        dwStyle|=TTS_BALLOON;
    if (CloseButton)
        dwStyle|=TTS_CLOSE;

    /* CREATE A TOOLTIP WINDOW */
    this->hwndToolTip = CreateWindowEx(WS_EX_TOPMOST,
                                        TOOLTIPS_CLASS,
                                        NULL,
                                        dwStyle,
                                        CW_USEDEFAULT,
                                        CW_USEDEFAULT,
                                        CW_USEDEFAULT,
                                        CW_USEDEFAULT,
                                        hWndParent,
                                        (HMENU)NULL/*ControlId*/,// not child window --> no control id
                                        hInstance,
                                        NULL
                                        );

    if (!this->hwndToolTip)
        return;

    // Do the standard ToolTip coding. 
    TOOLINFO ti={0};

    ti.cbSize = sizeof(ti);
    ti.uFlags = TTF_TRANSPARENT | TTF_SUBCLASS | TTF_IDISHWND;
    if (bCenterTipOnControl)
        ti.uFlags |= TTF_CENTERTIP;

    // TTF_PARSELINKS link should be inside text like "text <a>http://Link</a> text..."
    if (ParseLinks)
        ti.uFlags |= TTF_PARSELINKS;
    

    ti.hwnd = hWndParent;
    ti.uId = (UINT_PTR)hWndParent;
    ti.hinst = hInstance;
    ti.lpszText = pszText;

    // GetClientRect(hWndParent, &ti.rect); not required due to TTF_IDISHWND
    SendMessage(this->hwndToolTip, TTM_ADDTOOL, 0, (LPARAM) &ti );

    // multi lines tooltip : if you let it to -1 (auto size), tooltip is adjust on first line size
    // so if next lines are larger then the first one, you will miss some words
    // By putting larger size tooltip will be auto adjusted to larger line
    // TTM_SETMAXTIPWIDTH : width of control, text length can be larger. In that case it will be put on multiple lines
    SendMessage(this->hwndToolTip, TTM_SETMAXTIPWIDTH, 0, (LPARAM) 400 );
}

LRESULT CToolTip::SetMaxWidth(int Width)
{
    // multi lines tooltip : if you let it to -1 (auto size), tooltip is adjust on first line size
    // so if next lines are larger then the first one, you will miss some words
    // By putting larger size tooltip will be auto adjusted to larger line
    // TTM_SETMAXTIPWIDTH : width of control, text length can be larger. In that case it will be put on multiple lines
    return SendMessage(this->hwndToolTip, TTM_SETMAXTIPWIDTH, 0, (LPARAM) Width );
}

//-----------------------------------------------------------------------------
// Name: ~CToolTip
// Object: Destructor.
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
CToolTip::~CToolTip()
{
    if (this->hwndToolTip)
    {
        ::DestroyWindow(this->hwndToolTip);
        this->hwndToolTip = NULL;
    }
}

//-----------------------------------------------------------------------------
// Name: GetControlHandle
// Object: return handle of Tool tip
// Parameters :
//     in  : 
//     out :
//     return : handle of Tool tip
//-----------------------------------------------------------------------------
HWND CToolTip::GetControlHandle()
{
    return this->hwndToolTip;
}

LRESULT CToolTip::SetText(TCHAR* pszText)
{
    TOOLINFO ti = {0};
    ti.cbSize = sizeof(ti);
    ti.uFlags = TTF_IDISHWND;
    ti.hwnd = this->hWndParent;
    ti.uId = (UINT_PTR)this->hWndParent;
    ti.hinst = this->hInstance;
    ti.lpszText = pszText;
    return SendMessage((HWND) this->hwndToolTip,(UINT)TTM_UPDATETIPTEXT, (WPARAM)0, (LPARAM)&ti);
}

//-----------------------------------------------------------------------------
// Name: SetTitle
// Object: Set Title and icon for the tooltip
// Parameters :
//     in  : 
//     out :
//     return : result of SendMessage
//-----------------------------------------------------------------------------
LRESULT CToolTip::SetTitle(TCHAR* Title,TOOLTIP_ICON Icon)
{
    return SendMessage((HWND) this->hwndToolTip,(UINT)TTM_SETTITLE, (WPARAM)Icon, (LPARAM)Title);
}
LRESULT CToolTip::SetTitleWithIconHandle(TCHAR* Title,HICON Icon)
{
    return SendMessage((HWND) this->hwndToolTip,(UINT)TTM_SETTITLE, (WPARAM)Icon, (LPARAM)Title);
}

//-----------------------------------------------------------------------------
// Name: SetTextColor
// Object: Set Text Color for the tooltip
// Parameters :
//     in  : COLORREF Color : new color
//     out :
//     return : result of SendMessage
//-----------------------------------------------------------------------------
LRESULT CToolTip::SetTextColor(COLORREF Color)
{
    return SendMessage(this->hwndToolTip,TTM_SETTIPTEXTCOLOR,(WPARAM)Color,0);
}
//-----------------------------------------------------------------------------
// Name: SetBackGroundColor
// Object: Set SetBackGroundColor for the tooltip
// Parameters :
//     in  : COLORREF Color : new color
//     out :
//     return : result of SendMessage
//-----------------------------------------------------------------------------
LRESULT CToolTip::SetBackGroundColor(COLORREF Color)
{
    return SendMessage(this->hwndToolTip,TTM_SETTIPBKCOLOR,(WPARAM)Color,0);
}

LRESULT CToolTip::Activate(BOOL bActivate)
{
    return SendMessage(this->hwndToolTip,TTM_ACTIVATE,(WPARAM)bActivate,0);
}

void CToolTip::HideToolTip()
{
    SendMessage(this->hwndToolTip,TTM_POP,0,0);
}
void CToolTip::ShowToolTip()
{
    SendMessage(this->hwndToolTip,TTM_POPUP,0,0);
}