/*
Copyright (C) 2004-2016 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2016 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class for a TransparentToolbar control
//-----------------------------------------------------------------------------

#include "TransparentToolbar.h"

#define RGB_TO_BGR(x) ( ( (x & 0xFF) << 16 ) | (x & 0xFF00) | ( (x & 0xFF0000) >> 16 ) | (x & 0xFF000000) )
#define BGR_TO_RGB RGB_TO_BGR

//-----------------------------------------------------------------------------
// Name: CTransparentToolbar
// Object: constructor
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
CTransparentToolbar::CTransparentToolbar(HINSTANCE hInstance,HWND hParent,ITransparentToolbarEvents* pEventsHandler,BOOL bInitiallyVisible,BOOL bVertical,
                                        tagBackgroundType BackgroundType,COLORREF BackGroundColor,COLORREF BackGroundGradiantColor1,COLORREF BackGroundGradiantColor2,TCHAR* BackgroundImage,
                                        COLORREF TransparentColor,BOOL bUseTransparentColor,
                                        BYTE OpacityPercent,
                                        tagBarPositionMode BarPositionMode,
                                        LONG X,LONG Y,
                                        LONG Width,LONG Height,
                                        int IconWidth,int IconHeight,
                                        BUTTON_DESCRIPTION* ArrayButtons,
                                        SIZE_T ArrayButtonsSize,
                                        BUTTON_DESCRIPTION* pMoveButton,BOOL bShowMoveButton,
                                        BOOL bAllowWindowMoveFromBackGround,
                                        RECT* pMargins,BOOL bTopMost
                                        )
{
    this->pEventsHandler = pEventsHandler;
    this->TransparentColor=TransparentColor;
    this->bUseTransparentColor = bUseTransparentColor;
    this->bVertical = bVertical;
    this->Margins = *pMargins;
    this->bTopMost=bTopMost;
    this->bAllowWindowMoveFromBackGround = bAllowWindowMoveFromBackGround;
    this->ptCurrent.x=0;
    this->ptCurrent.y=0;

    this->pToolTip = NULL;
    this->CurrentToolTipText = NULL;
    this->BackgroundType = BackgroundType;
    this->BackGroundColor=BackGroundColor;
    this->BackGroundGradiantColor1 = BackGroundGradiantColor1;
    this->BackGroundGradiantColor2 = BackGroundGradiantColor2;
    this->OpacityPercent=OpacityPercent;
    this->BarPositionMode = BarPositionMode;

    Gdiplus::GdiplusStartupInput gdiplusStartupInput;
    this->gdiplusToken=NULL;
    this->bGdiPlusStarted=(Gdiplus::GdiplusStartup(&this->gdiplusToken, &gdiplusStartupInput, NULL)==Gdiplus::Ok);

    if (BackgroundImage && this->bGdiPlusStarted)
    {
        WCHAR wcsBackgroundImage[MAX_PATH];
#if (defined(UNICODE)||defined(_UNICODE))
        wcsncpy(wcsBackgroundImage,BackgroundImage,MAX_PATH);
#else
        CAnsiUnicodeConvert::AnsiToUnicode(BackgroundImage,wcsBackgroundImage,MAX_PATH);
#endif
        this->objImage = new Gdiplus::Image(wcsBackgroundImage,TRUE);
        if (this->objImage->GetLastStatus()!=Gdiplus::Ok)
        {
            delete this->objImage;
            this->objImage = NULL;
        }
   }
   else
       this->objImage=NULL;

    // use transparent color for background brush
    this->BackGroundBrush=::CreateSolidBrush(this->BackGroundColor);

    this->bLeftButtonDown=FALSE;
    this->bLeftButtonDownOnBackGround=FALSE;
    this->bCapture=FALSE;
    this->bMouseOverMoveIcon=FALSE;

    this->hInstance=hInstance;
    this->hWndParent=hParent;

    this->IconWidth=IconWidth;
    this->IconHeight=IconHeight;

    this->NextDialogControlId=0;
    this->NbButtonsLeft=0;
    this->NbButtonsRightOrBottom=0;
    this->NbButtonsCenterOrMiddle=0;
    this->LastOverButtonIndex=(DWORD)-1;

    this->bMoveButtonCreated = bShowMoveButton;
    this->bShowMoveButton = TRUE;
    // copy move button description
    if (pMoveButton)
        memcpy(&this->MoveButton.Description,pMoveButton,sizeof(BUTTON_DESCRIPTION));
    else
        memset(&this->MoveButton.Description,0,sizeof(BUTTON_DESCRIPTION));

    this->MoveButton.CurrentIconType = IconType_NORMAL;
    if (this->bMoveButtonCreated)
    {
        switch(this->MoveButton.Description.Position)
        {
        case ButtonPosition_LEFT_OR_TOP:
            this->NbButtonsLeft++;
            break;
        case ButtonPosition_CENTER_OR_MIDDLE:
            this->NbButtonsCenterOrMiddle++;
            break;
        case ButtonPosition_RIGHT_OR_BOTTOM:
            this->NbButtonsRightOrBottom++;
            break;
        }
    }
    else
        memset(&this->MoveButton.Description,0,sizeof(BUTTON_DESCRIPTION));


    // copy other buttons description
    this->ArrayButtonsDescriptionSize=ArrayButtonsSize;
    this->ArrayButtonsDescription=new BUTTON_DESCRIPTION_EX[this->ArrayButtonsDescriptionSize];
    this->IconsSpacersLeftOrTop =0;
    this->IconsSpacersCenterOrMiddle =0;
    this->IconsSpacersRightOrBottom =0;


    for (SIZE_T Cnt=0;Cnt<this->ArrayButtonsDescriptionSize;Cnt++)
    {
        this->ArrayButtonsDescription[Cnt].CurrentIconType = IconType_NORMAL;
        memcpy(&this->ArrayButtonsDescription[Cnt].Description,&ArrayButtons[Cnt],sizeof(BUTTON_DESCRIPTION));

        switch(this->ArrayButtonsDescription[Cnt].Description.Position)
        {
        case ButtonPosition_LEFT_OR_TOP:
            this->NbButtonsLeft++;
            this->IconsSpacersLeftOrTop+=this->ArrayButtonsDescription[Cnt].Description.SpacerBeforeIconInPixels;
            break;
        case ButtonPosition_CENTER_OR_MIDDLE:
            this->NbButtonsCenterOrMiddle++;
            this->IconsSpacersCenterOrMiddle+=this->ArrayButtonsDescription[Cnt].Description.SpacerBeforeIconInPixels;
            break;
        case ButtonPosition_RIGHT_OR_BOTTOM:
            this->NbButtonsRightOrBottom++;
            this->IconsSpacersRightOrBottom+=this->ArrayButtonsDescription[Cnt].Description.SpacerBeforeIconInPixels;
            break;
        }
    }

    this->InitialPositionX=X;
    this->InitialPositionY=Y;
    this->CurrentPositionX=X;
    this->CurrentPositionY=Y;

    DWORD NbExtraButtons;
    NbExtraButtons = 0;
    if (this->bMoveButtonCreated)
        NbExtraButtons++;
    if (bVertical)
    {
        this->MinWidth = IconWidth+Margins.left+Margins.right;
        this->MinHeight=(LONG)((this->ArrayButtonsDescriptionSize+NbExtraButtons)*IconHeight);
        this->MinHeight+=this->IconsSpacersLeftOrTop+this->IconsSpacersCenterOrMiddle+this->IconsSpacersRightOrBottom;
        this->MinHeight+=Margins.top+Margins.bottom;
    }
    else
    {
        this->MinHeight = IconHeight+Margins.top+Margins.bottom;
        this->MinWidth=(LONG)((this->ArrayButtonsDescriptionSize+NbExtraButtons)*IconWidth);
        this->MinWidth+=this->IconsSpacersLeftOrTop+this->IconsSpacersCenterOrMiddle+this->IconsSpacersRightOrBottom;
        this->MinWidth+=Margins.left+Margins.right;
    }

    if (Height<this->MinHeight)
        Height = this->MinHeight;
    if (Width<this->MinWidth)
        Width = this->MinWidth;
    this->Width=Width;
    this->Height=Height;

    this->hMoveCursor = ::LoadCursor (NULL, IDC_SIZEALL);

    this->CreateControl();

    if (bInitiallyVisible)
        this->Show(TRUE);
}


CTransparentToolbar::~CTransparentToolbar(void)
{
    if (this->pToolTip)
    {
        delete this->pToolTip;
        this->pToolTip = NULL;
    }

	if (this->hWnd)
    {
		::DestroyWindow(this->hWnd);
        this->hWnd = NULL;
    }

    this->ArrayButtonsDescriptionSize=0;
    if (this->ArrayButtonsDescription)
    {
        delete this->ArrayButtonsDescription;
        this->ArrayButtonsDescription = NULL;
    }

    ::DeleteObject(this->BackGroundBrush);
    this->BackGroundBrush = NULL;

    if (this->objImage)
    {
        delete this->objImage;
        this->objImage=NULL;
    }

    if (this->bGdiPlusStarted)
    {
        Gdiplus::GdiplusShutdown(this->gdiplusToken);
        this->bGdiPlusStarted=FALSE;
    }
}

//-----------------------------------------------------------------------------
// Name: GetControlHandle
// Object: return the handle to the control
// Parameters :
//     in  : 
//     out :
//     return : HWND handle to the control
//-----------------------------------------------------------------------------
HWND CTransparentToolbar::GetControlHandle()
{
    return this->hWnd;
}

//-----------------------------------------------------------------------------
// Name: SetWidth
// Object: set control width and reposition buttons according to new size
// Parameters :
//     in  : 
//     out :
//     return : FALSE if new width is not enough to contain icons
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::SetWidth(LONG Width)
{
    if (this->Width==Width)
        return TRUE;

    BOOL bRet=TRUE;
    this->Width=Width;

    // check for sufficient size
    if (this->Width<this->MinWidth)
    {
        this->Width=this->MinWidth;
        bRet = FALSE;
    }

    // update window size
    RECT Rect;
    ::GetWindowRect(this->hWnd,&Rect);
    ::SetWindowPos(this->hWnd,NULL,0,0,this->Width,Rect.bottom-Rect.top,SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);

    // update icon position
    this->UpdateIconsPosition();

    ::RedrawWindow( this->hWnd,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME | RDW_ALLCHILDREN | RDW_INTERNALPAINT);

    return bRet;
}

//-----------------------------------------------------------------------------
// Name: SetWidth
// Object: set control height and reposition buttons according to new size
// Parameters :
//     in  : 
//     out :
//     return : FALSE if new Height is not enough to contain icons
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::SetHeight(LONG Height)
{
    if (this->Height==Height)
        return TRUE;

    BOOL bRet=TRUE;
    this->Height=Height;

    // check for sufficient size
    if (this->Height<this->MinHeight)
    {
        this->Height=this->MinHeight;
        bRet = FALSE;
    }

    // update window size
    RECT Rect;
    ::GetWindowRect(this->hWnd,&Rect);
    ::SetWindowPos(this->hWnd,0,0,0,Rect.right-Rect.left,this->Height,SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);

    // update icon position
    this->UpdateIconsPosition();

    ::RedrawWindow( this->hWnd,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME | RDW_ALLCHILDREN | RDW_INTERNALPAINT);

    return bRet;
}

//-----------------------------------------------------------------------------
// Name: SetWidth
// Object: set control height and reposition buttons according to new size
// Parameters :
//     in  : 
//     out :
//     return : FALSE if new size is not enough to contain icons
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::SetPosition(int x,int y,int Width,int Height)
{
    BOOL bRet=TRUE;

    // check for sufficient size
    if (Height<this->MinHeight)
    {
        Height=this->MinHeight;
        bRet = FALSE;
    }
    // check for sufficient size
    if (Width<this->MinWidth)
    {
        Width=this->MinWidth;
        bRet = FALSE;
    }

    this->Height=Height;
    this->Width=Width;

    // update window size
    ::SetWindowPos(this->hWnd,NULL,x,y,this->Width,this->Height,SWP_NOACTIVATE | SWP_NOZORDER );

    // update icon position
    this->UpdateIconsPosition();

    ::RedrawWindow( this->hWnd,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME | RDW_ALLCHILDREN | RDW_INTERNALPAINT);

    return bRet;
}

//-----------------------------------------------------------------------------
// Name: SetMargins
// Object: set control margins and reposition buttons according to new margins
// Parameters :
//     in  : 
//     out :
//     return : FALSE if Height or width are not enough to contain icons
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::SetMargins(RECT* pMargins)
{
    BOOL bRet=TRUE;
    this->Margins = *pMargins;

    DWORD NbExtraButtons;
    NbExtraButtons = 0;
    if (this->bMoveButtonCreated)
        NbExtraButtons++;

    // check for sufficient size
    if (bVertical)
    {
        this->MinWidth = this->IconWidth+this->Margins.left+this->Margins.right;
        this->MinHeight=(LONG)((this->ArrayButtonsDescriptionSize+NbExtraButtons)*this->IconHeight);
        this->MinHeight+=this->IconsSpacersLeftOrTop+this->IconsSpacersCenterOrMiddle+this->IconsSpacersRightOrBottom;
        this->MinHeight+=this->Margins.top+this->Margins.bottom;
    }
    else
    {
        this->MinHeight = this->IconHeight+this->Margins.top+this->Margins.bottom;
        this->MinWidth=(LONG)((this->ArrayButtonsDescriptionSize+NbExtraButtons)*this->IconWidth);
        this->MinWidth+=this->IconsSpacersLeftOrTop+this->IconsSpacersCenterOrMiddle+this->IconsSpacersRightOrBottom;
        this->MinWidth+=this->Margins.left+this->Margins.right;
    }

    if ( (this->Height<this->MinHeight) || (this->Width<this->MinWidth) )
    {
        this->Height=this->MinHeight;
        this->Width=this->MinWidth;
        bRet = FALSE;

        // update window size
        ::SetWindowPos(this->hWnd,0,0,0,this->Width,this->Height,SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);
    }

    // update icon position
    this->UpdateIconsPosition();

    ::RedrawWindow( this->hWnd,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME | RDW_ALLCHILDREN | RDW_INTERNALPAINT);

    return bRet;
}


//-----------------------------------------------------------------------------
// Name: CreateControl
// Object: create and show the control
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
BOOL  CTransparentToolbar::CreateControl()
{
    WNDCLASSEX	wc;
    // register background window class if needed
    if (!::GetClassInfoEx (this->hInstance, CTransparentToolbar_TransparentToolbar_CLASSEX, &wc))
    {
        ZeroMemory (&wc, sizeof(wc));
        wc.cbSize	     = sizeof (wc);
        wc.cbWndExtra    = 0;
        wc.hInstance     = this->hInstance;
        wc.hCursor	     = LoadCursor (NULL, IDC_ARROW);
        wc.hbrBackground = NULL;
        wc.lpfnWndProc   = CTransparentToolbar::WindowProcStatic;
        wc.lpszClassName = CTransparentToolbar_TransparentToolbar_CLASSEX;
        wc.style         = CS_DBLCLKS | // allow double clicking
                           CS_HREDRAW | CS_VREDRAW; // redraw the entire contents of the client area whenever the window changes size
        ::RegisterClassEx (&wc);
    }

    this->CurrentPositionX=this->InitialPositionX;
    this->CurrentPositionY=this->InitialPositionY;

    DWORD dwStyle = WS_POPUP;
    DWORD dwExStyle = WS_EX_TOOLWINDOW | WS_EX_LAYERED;
    if (this->bTopMost)
        dwExStyle|=WS_EX_TOPMOST;

    if (this->BarPositionMode == BarPositionMode_CHILD)
    {
        dwStyle = WS_CHILD;
        dwExStyle = 0;
    }

    // create background window
    this->hWnd = 
        ::CreateWindowEx(
                        dwExStyle,
                        CTransparentToolbar_TransparentToolbar_CLASSEX, // Window class name
                        NULL,    // Window name
                        dwStyle,// Window style
                        // Position & size
                        this->InitialPositionX,
                        this->InitialPositionY,
                        this->Width,
                        this->Height,
                        this->hWndParent,	// Parent window handle
                        (HMENU)this->NextDialogControlId++,// Application defined ID must be unique for all child windows with the same parent window
                        this->hInstance,	// Application instance
                        this				// Parameter
                        );

    if (!this->hWnd)
        return FALSE;

    // put CTransparentToolbar object in window parameter
    ::SetWindowLongPtr(this->hWnd,GWLP_USERDATA,(LONG_PTR)this);

    DWORD dwFlags=0;
    if (this->OpacityPercent!=100)
        dwFlags|=LWA_ALPHA;
    if (this->bUseTransparentColor)
        dwFlags|=LWA_COLORKEY;

    // set transparent color with opacity to 80%
    // ::SetLayeredWindowAttributes(this->hWnd, this->TransparentColor, (255 * 80) / 100, LWA_COLORKEY | LWA_ALPHA);
    ::SetLayeredWindowAttributes(this->hWnd, this->TransparentColor, (255 * this->OpacityPercent) / 100, dwFlags);

    // update icons positions
    this->UpdateIconsPosition();

    this->pToolTip = new CToolTip(this->hInstance,this->hWnd,LPSTR_TEXTCALLBACK,FALSE,FALSE);

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: UpdateIconsPosition
// Object: update icons positions according to control width
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CTransparentToolbar::UpdateIconsPosition()
{
    LONG NextLeftOrTopPos;
    LONG NextCenterOrMiddlePos;
    LONG NextRightOrBottomPos;
    LONG xPos;
    LONG yPos;

    NextLeftOrTopPos=0;
    if (this->bVertical)
    {
        NextRightOrBottomPos=this->Height-this->Margins.bottom-this->NbButtonsRightOrBottom*this->IconHeight-this->IconsSpacersRightOrBottom;
        NextCenterOrMiddlePos=(this->Height-this->NbButtonsCenterOrMiddle*this->IconHeight-this->Margins.top-this->Margins.bottom+this->IconsSpacersCenterOrMiddle)/2;
    }
    else
    {
        NextRightOrBottomPos=this->Width-this->Margins.right-this->NbButtonsRightOrBottom*this->IconWidth-this->IconsSpacersRightOrBottom;
        NextCenterOrMiddlePos=(this->Width-this->NbButtonsCenterOrMiddle*this->IconWidth-this->Margins.left-this->Margins.right+this->IconsSpacersCenterOrMiddle)/2;
    }

    xPos = this->Margins.left;
    yPos = this->Margins.right;

    // update command icons
    for (SIZE_T Cnt=0;Cnt<this->ArrayButtonsDescriptionSize;Cnt++)
    {
        if (this->ArrayButtonsDescription[Cnt].Description.Position==ButtonPosition_LEFT_OR_TOP)
        {
            NextLeftOrTopPos+=this->ArrayButtonsDescription[Cnt].Description.SpacerBeforeIconInPixels;
            if (bVertical)
            {
                yPos=NextLeftOrTopPos;
                NextLeftOrTopPos+=this->IconHeight;
            }
            else
            {
                xPos=NextLeftOrTopPos;
                NextLeftOrTopPos+=this->IconWidth;
            }
            this->ArrayButtonsDescription[Cnt].Position.x = xPos;
            this->ArrayButtonsDescription[Cnt].Position.y = yPos;
        }
    }
    if (this->bMoveButtonCreated)
    {
        if (this->MoveButton.Description.Position==ButtonPosition_LEFT_OR_TOP)
        {
            NextLeftOrTopPos+=this->MoveButton.Description.SpacerBeforeIconInPixels;
            if (bVertical)
            {
                yPos=NextLeftOrTopPos;
                NextLeftOrTopPos+=this->IconHeight;
            }
            else
            {
                xPos=NextLeftOrTopPos;
                NextLeftOrTopPos+=this->IconWidth;
            }

            this->MoveButton.Position.x = xPos;
            this->MoveButton.Position.y = yPos;
        }
    }

    // avoid left items to be hidden by centered items
    if (NextCenterOrMiddlePos<NextLeftOrTopPos)
        NextCenterOrMiddlePos=NextLeftOrTopPos;

    for (SIZE_T Cnt=0;Cnt<this->ArrayButtonsDescriptionSize;Cnt++)
    {
        if (this->ArrayButtonsDescription[Cnt].Description.Position==ButtonPosition_CENTER_OR_MIDDLE)
        {
            NextCenterOrMiddlePos+=this->ArrayButtonsDescription[Cnt].Description.SpacerBeforeIconInPixels;
            if (bVertical)
            {
                yPos=NextCenterOrMiddlePos;
                NextCenterOrMiddlePos+=this->IconHeight;
            }
            else
            {
                xPos=NextCenterOrMiddlePos;
                NextCenterOrMiddlePos+=this->IconWidth;
            }

            this->ArrayButtonsDescription[Cnt].Position.x = xPos;
            this->ArrayButtonsDescription[Cnt].Position.y = yPos;
        }
    }
    if (this->bMoveButtonCreated)
    {
        if (this->MoveButton.Description.Position==ButtonPosition_CENTER_OR_MIDDLE)
        {
            NextCenterOrMiddlePos+=this->MoveButton.Description.SpacerBeforeIconInPixels;
            if (bVertical)
            {
                yPos=NextCenterOrMiddlePos;
                NextCenterOrMiddlePos+=this->IconHeight;
            }
            else
            {
                xPos=NextCenterOrMiddlePos;
                NextCenterOrMiddlePos+=this->IconWidth;
            }
            
            this->MoveButton.Position.x = xPos;
            this->MoveButton.Position.y = yPos;
        }
    }

    // avoid centered items to be hide by right items
    if (NextRightOrBottomPos<NextCenterOrMiddlePos)
        NextRightOrBottomPos=NextCenterOrMiddlePos;

    for (SIZE_T Cnt=0;Cnt<this->ArrayButtonsDescriptionSize;Cnt++)
    {
        if (this->ArrayButtonsDescription[Cnt].Description.Position==ButtonPosition_RIGHT_OR_BOTTOM)
        {
            NextRightOrBottomPos+=this->ArrayButtonsDescription[Cnt].Description.SpacerBeforeIconInPixels;
            if (bVertical)
            {
                yPos=NextRightOrBottomPos;
                NextRightOrBottomPos+=this->IconHeight;
            }
            else
            {
                xPos=NextRightOrBottomPos;
                NextRightOrBottomPos+=this->IconWidth;
            }
            this->ArrayButtonsDescription[Cnt].Position.x = xPos;
            this->ArrayButtonsDescription[Cnt].Position.y = yPos;
        }
    }
    if (this->bMoveButtonCreated)
    {
        if (this->MoveButton.Description.Position==ButtonPosition_RIGHT_OR_BOTTOM)
        {
            NextRightOrBottomPos+=this->MoveButton.Description.SpacerBeforeIconInPixels;
            if (bVertical)
            {
                yPos=NextRightOrBottomPos;
                NextRightOrBottomPos+=this->IconHeight;
            }
            else
            {
                xPos=NextRightOrBottomPos;
                NextRightOrBottomPos+=this->IconWidth;
            }
            this->MoveButton.Position.x = xPos;
            this->MoveButton.Position.y = yPos;
        }
    }
}

//-----------------------------------------------------------------------------
// Name: GetMinWidth
// Object: return minimum required width for storing all icons
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LONG CTransparentToolbar::GetMinWidth()
{
    return this->MinWidth;
}

//-----------------------------------------------------------------------------
// Name: GetMinHeight
// Object: return minimum required height for storing all icons
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LONG CTransparentToolbar::GetMinHeight()
{
    return this->MinHeight;
}

//-----------------------------------------------------------------------------
// Name: ShowMove
// Object: hide / show move button (Warning: buttons layout is not changed)
// Parameters :
//     in  : BOOL bShow
//     out :
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::ShowMove(BOOL bShow)
{
    if (!this->bMoveButtonCreated)
        return FALSE;

    if (this->bShowMoveButton != bShow)
    {
        this->bShowMoveButton = bShow;
        RECT MoveRect;
        this->GetButtonRectEx(&this->MoveButton,&MoveRect);
        return ::InvalidateRect(this->hWnd,&MoveRect,TRUE);
    }
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: ChangeIcons
// Object: change icons for a command button
// Parameters :
//     in  : DWORD ButtonId
//           HICON hIcon,HICON hIconHot,HICON hIconDown,HICON hIconDisabled : icons handles
//     out :
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::ChangeIcons(DWORD ButtonId,HICON hIcon,HICON hIconHot,HICON hIconDown,HICON hIconDisabled)
{
    for (SIZE_T Cnt=0;Cnt<this->ArrayButtonsDescriptionSize;Cnt++)
    {
        if (this->ArrayButtonsDescription[Cnt].Description.ButtonId == ButtonId)
        {
            this->ArrayButtonsDescription[Cnt].Description.hIcon=hIcon;
            this->ArrayButtonsDescription[Cnt].Description.hIconHot=hIconHot;
            this->ArrayButtonsDescription[Cnt].Description.hIconDown=hIconDown;
            this->ArrayButtonsDescription[Cnt].Description.hIconDisabled=hIconDisabled;

            if ( this->ArrayButtonsDescription[Cnt].Description.Enabled )
            {
                if (Cnt ==(SIZE_T) this->LastOverButtonIndex )
                {
                    if (this->bLeftButtonDown)
                        // this->SetIconImgType(&this->ArrayButtonsDescription[Cnt],IconType_DOWN);
                        this->ArrayButtonsDescription[Cnt].CurrentIconType = IconType_DOWN;
                    else
                        // this->SetIconImgType(&this->ArrayButtonsDescription[Cnt],IconType_HOT);
                        this->ArrayButtonsDescription[Cnt].CurrentIconType = IconType_HOT;
                }
                else
                {
                    // set normal icon
                    // this->SetIconImgType(&this->ArrayButtonsDescription[Cnt],IconType_NORMAL);
                    this->ArrayButtonsDescription[Cnt].CurrentIconType = IconType_NORMAL;
                }
            }
            else
            {
                // this->SetIconImgType(&this->ArrayButtonsDescription[Cnt],IconType_DISABLED);
                this->ArrayButtonsDescription[Cnt].CurrentIconType = IconType_DISABLED;
            }

            RECT ButtonRect;
            ButtonRect.left = this->ArrayButtonsDescription[Cnt].Position.x;
            ButtonRect.top = this->ArrayButtonsDescription[Cnt].Position.y;
            ButtonRect.right =  ButtonRect.left + this->IconWidth;
            ButtonRect.bottom = ButtonRect.top + this->IconHeight;
            ::InvalidateRect(this->hWnd,&ButtonRect,TRUE);
            return TRUE;
        }
    }
    return FALSE;
}

//-----------------------------------------------------------------------------
// Name: EnableButton
// Object: Enable / Disable a command button
// Parameters :
//     in  : DWORD ButtonId
//           BOOL bEnable
//     out :
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::EnableButton(DWORD ButtonId,BOOL bEnable)
{
    for (SIZE_T Cnt=0;Cnt<this->ArrayButtonsDescriptionSize;Cnt++)
    {
        if (this->ArrayButtonsDescription[Cnt].Description.ButtonId == ButtonId)
        {
            this->ArrayButtonsDescription[Cnt].Description.Enabled=bEnable;

            if (bEnable)
            {
                if ((SIZE_T)this->LastOverButtonIndex == Cnt)
                {
                    if (this->bLeftButtonDown)
                        // this->SetIconImgType(&this->ArrayButtonsDescription[Cnt],IconType_DOWN);
                        this->ArrayButtonsDescription[Cnt].CurrentIconType = IconType_DOWN;
                    else
                        // this->SetIconImgType(&this->ArrayButtonsDescription[Cnt],IconType_HOT);
                        this->ArrayButtonsDescription[Cnt].CurrentIconType = IconType_HOT;
                }
                else
                    // this->SetIconImgType(&this->ArrayButtonsDescription[Cnt],IconType_NORMAL);
                    this->ArrayButtonsDescription[Cnt].CurrentIconType = IconType_NORMAL;
            }
            else
                // this->SetIconImgType(&this->ArrayButtonsDescription[Cnt],IconType_DISABLED);
                this->ArrayButtonsDescription[Cnt].CurrentIconType = IconType_DISABLED;
                
            RECT ButtonRect;
            ButtonRect.left = this->ArrayButtonsDescription[Cnt].Position.x;
            ButtonRect.top = this->ArrayButtonsDescription[Cnt].Position.y;
            ButtonRect.right =  ButtonRect.left + this->IconWidth;
            ButtonRect.bottom = ButtonRect.top + this->IconHeight;
            ::InvalidateRect(this->hWnd,&ButtonRect,TRUE);
            return TRUE;
        }
    }
    return FALSE;
}

//-----------------------------------------------------------------------------
// Name: GetButtonUserData
// Object: get user data associated to button
// Parameters :
//     in  : IN DWORD ButtonId
//     out : OUT PVOID* pUserData
//     return : 
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::GetButtonUserData(IN DWORD ButtonId,OUT PVOID* pUserData)
{
    for (SIZE_T Cnt=0;Cnt<this->ArrayButtonsDescriptionSize;Cnt++)
    {
        if (this->ArrayButtonsDescription[Cnt].Description.ButtonId == ButtonId)
        {
            *pUserData = this->ArrayButtonsDescription[ButtonId].Description.UserData;
            return TRUE;
        }
    }
    return FALSE;
}

//-----------------------------------------------------------------------------
// Name: GetButtonRect
// Object: get rectangle used by button
// Parameters :
//     in  : IN DWORD ButtonId
//     out : OUT RECT* pButtonRect
//     return : 
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::GetButtonRect(IN DWORD ButtonId,OUT RECT* pButtonRect)
{
    for (SIZE_T Cnt=0;Cnt<this->ArrayButtonsDescriptionSize;Cnt++)
    {
        if (this->ArrayButtonsDescription[Cnt].Description.ButtonId == ButtonId)
        {
            pButtonRect->left = this->ArrayButtonsDescription[Cnt].Position.x;
            pButtonRect->top = this->ArrayButtonsDescription[Cnt].Position.y;
            pButtonRect->right = pButtonRect->left + this->IconWidth;
            pButtonRect->bottom = pButtonRect->top + this->IconHeight;
            // ::GetWindowRect(this->ArrayButtonsDescription[Cnt].hWndButton,pButtonRect);
            return TRUE;
        }
    }
    return FALSE;
}

BOOL CTransparentToolbar::GetButtonRectEx(IN BUTTON_DESCRIPTION_EX* pButtonDescription,OUT RECT* pButtonRect)
{
    pButtonRect->left = pButtonDescription->Position.x;
    pButtonRect->top = pButtonDescription->Position.y;
    pButtonRect->right = pButtonRect->left + this->IconWidth;
    pButtonRect->bottom = pButtonRect->top + this->IconHeight;
    return TRUE;
}

LRESULT CTransparentToolbar::DoPaint(HDC hdc)
{
    // for layered window, WM_PAINT will never been called, so current function won't be called
    UNREFERENCED_PARAMETER (hdc);

    RECT rect;
    // Get window coordinates, and normalize.
    ::GetWindowRect(this->hWnd, &rect);
    rect.right = rect.right - rect.left;  // Get width.
    rect.bottom = rect.bottom - rect.top; // Get height.
    rect.left = rect.top = 0;
    // solid brush
    // ::DrawBackGround(hdc, &rect, this->BackGroundBrush);
    this->DrawBackGround(hdc,&rect);

    HICON hIcon;
    BUTTON_DESCRIPTION_EX* pButtonsDescription;
    BUTTON_DESCRIPTION_EX* pButtonsDescriptionEnd;
    pButtonsDescriptionEnd = this->ArrayButtonsDescription+this->ArrayButtonsDescriptionSize;
    for (pButtonsDescription = this->ArrayButtonsDescription;
        pButtonsDescription<pButtonsDescriptionEnd;
        pButtonsDescription++)
    {
        hIcon = this->GetIconImg(pButtonsDescription);
        // until rect is computed for icon size we don't need to clip it
        // CDrawClippedIcon::DrawClippedIcon(hdc,&Rect,hIcon,CDockingTitleBar_ICON_SIZE);
        ::DrawIconEx(hdc,pButtonsDescription->Position.x,pButtonsDescription->Position.y,hIcon,this->IconWidth,this->IconHeight,0,NULL,DI_NORMAL);
    }

    if (this->bMoveButtonCreated && this->bShowMoveButton)
    {
        hIcon = this->GetIconImg(&this->MoveButton);
        ::DrawIconEx(hdc,this->MoveButton.Position.x,this->MoveButton.Position.y,hIcon,this->IconWidth,this->IconHeight,0,NULL,DI_NORMAL);
    }

    return 1;
}

void CTransparentToolbar::DrawBackGround(HDC hdc,RECT* pRect)
{
    // gradiant brush
    switch(this->BackgroundType)
    {
    ////////////////////
    // gradient
    ////////////////////
    // use gdi values 
    //LinearGradientModeHorizontal = 0,
    //LinearGradientModeVertical = 1,
    //LinearGradientModeForwardDiagonal = 2,
    //LinearGradientModeBackwardDiagonal = 3
    case BACKGROUND_LINEAR_GRADIENT_MODE_HORIZONTAL:
    case BACKGROUND_LINEAR_GRADIENT_MODE_VERTICAL:
    case BACKGROUND_LINEAR_GRADIENT_MODE_FORWARD_DIAGONAL:
    case BACKGROUND_LINEAR_GRADIENT_MODE_BACKWARD_DIAGONAL:
        this->SetBackgroundGradient(hdc,pRect,this->BackGroundGradiantColor1,this->BackGroundGradiantColor2,(Gdiplus::LinearGradientMode)this->BackgroundType);
        break;

    ////////////////////
    // single color
    ////////////////////
    case BACKGROUND_SOLID_BRUSH:
        // draw back ground
        ::FillRect(hdc, pRect, this->BackGroundBrush);
        break;

    ////////////////////
    // image
    ////////////////////
    case BACKGROUND_IMAGE_POSITION_STRETCHED:
    case BACKGROUND_IMAGE_POSITION_CENTER:
    case BACKGROUND_IMAGE_POSITION_UP_LEFT:
    case BACKGROUND_IMAGE_POSITION_UP_CENTER:
    case BACKGROUND_IMAGE_POSITION_UP_RIGHT:
    case BACKGROUND_IMAGE_POSITION_LEFT:
    case BACKGROUND_IMAGE_POSITION_RIGHT:
    case BACKGROUND_IMAGE_POSITION_BOTTOM_LEFT:
    case BACKGROUND_IMAGE_POSITION_BOTTOM_CENTER:
    case BACKGROUND_IMAGE_POSITION_BOTTOM_RIGHT:
        // draw image
        this->SetBackgroundPicture(hdc,pRect);
        break;
    }
}
//-----------------------------------------------------------------------------
// Name: SetBackgroundGradient
// Object: set background gradient
// Parameters :
//     in  : COLORREF BackgroundGradiantColor1
//           COLORREF BackgroundGradiantColor2
//           Gdiplus::LinearGradientMode BackgroundGradiantType
//                      typedef enum {
//                          LinearGradientModeHorizontal = 0,
//                          LinearGradientModeVertical = 1,
//                          LinearGradientModeForwardDiagonal = 2,
//                          LinearGradientModeBackwardDiagonal = 3
//                      } Gdiplus::LinearGradientMode;
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::SetBackgroundGradient(HDC hdc,RECT* pRect,COLORREF BackgroundGradiantColor1,COLORREF BackgroundGradiantColor2,Gdiplus::LinearGradientMode BackgroundGradiantType)
{
    if (!this->bGdiPlusStarted)
        return FALSE;

    /////////////////////////
    // create graphic associated to destination bitmap
    ///////////////////////// 
    Gdiplus::Graphics* objGraphic = Gdiplus::Graphics::FromHDC(hdc);
    if (!objGraphic)
        return FALSE;
    if (objGraphic->GetLastStatus()!=Gdiplus::Ok)
    {
        delete objGraphic;
        return FALSE;
    }

    Gdiplus::Rect GdiplusBrowserRect(pRect->left,pRect->top,pRect->right-pRect->left,pRect->bottom-pRect->top);
    Gdiplus::Color Color1;
    Gdiplus::Color Color2;
    Color1.SetFromCOLORREF(BackgroundGradiantColor1 | 0xff000000); // "| 0xff000000" --> convert RGB to RGBA
    Color2.SetFromCOLORREF(BackgroundGradiantColor2 | 0xff000000); // "| 0xff000000" --> convert RGB to RGBA
    // create gradient
    Gdiplus::LinearGradientBrush objLinearGradientBrush(GdiplusBrowserRect,
                                                        Color1,
                                                        Color2,
                                                        BackgroundGradiantType
                                                        );

    // draw gradient to destination bitmap
    if (objGraphic->FillRectangle(&objLinearGradientBrush,GdiplusBrowserRect)!=Gdiplus::Ok)
    {
        delete objGraphic;
        return FALSE;
    }

    delete objGraphic;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SetBackgroundPicture
// Object: set background picture
// Parameters :
//     in  : 
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::SetBackgroundPicture(HDC hdc,RECT* pRect)
{
    if (!this->bGdiPlusStarted)
        return FALSE;
    if (!this->objImage)
        return FALSE;

    LONG ImageWidth=this->objImage->GetWidth();
    LONG ImageHeight=this->objImage->GetHeight();

    /////////////////////////
    // create destination bitmap
    ///////////////////////// 
    LONG ControlWith=pRect->right-pRect->left;
    LONG ControlHeight=pRect->bottom-pRect->top;

    /////////////////////////
    // create graphic associated to destination bitmap
    ///////////////////////// 
    Gdiplus::Graphics* objGraphic= Gdiplus::Graphics::FromHDC(hdc);
    if (objGraphic->GetLastStatus()!=Gdiplus::Ok)
        return FALSE;

    Gdiplus::Status RetValue;

    LONG xLeft=0;
    LONG xCenter=(ControlWith-ImageWidth)/2;
    LONG xRight=ControlWith-ImageWidth;
    LONG yUp=0;
    LONG yCenter=(ControlHeight-ImageHeight)/2;
    LONG yDown=ControlHeight-ImageHeight;
    LONG x;
    LONG y;
    LONG DstImageWidth=ImageWidth;
    LONG DstImageHeight=ImageHeight;

    // draw resized image in destination bitmap
    switch(this->BackgroundType)
    {
    case BACKGROUND_IMAGE_POSITION_UP_LEFT:
        // get image position
        x = xLeft;
        y = yUp;
        break;
    case BACKGROUND_IMAGE_POSITION_UP_CENTER:
        // get image position
        x = xCenter;
        y = yUp;
        break;
    case BACKGROUND_IMAGE_POSITION_UP_RIGHT:
        // get image position
        x = xRight;
        y = yUp;
        break;

    case BACKGROUND_IMAGE_POSITION_LEFT:
        // get image position
        x = xLeft;
        y = yCenter;
        break;
    case BACKGROUND_IMAGE_POSITION_CENTER:
        // get image position
        x = xCenter;
        y = yCenter;
        break;
    case BACKGROUND_IMAGE_POSITION_RIGHT:
        // get image position
        x = xRight;
        y = yCenter;
        break;

    case BACKGROUND_IMAGE_POSITION_BOTTOM_LEFT:
        // get image position
        x = xLeft;
        y = yDown;
        break;
    case BACKGROUND_IMAGE_POSITION_BOTTOM_CENTER:
        // get image position
        x = xCenter;
        y = yDown;
        break;
    case BACKGROUND_IMAGE_POSITION_BOTTOM_RIGHT:
        // get image position
        x = xRight;
        y = yDown;
        break;

    case BACKGROUND_IMAGE_POSITION_STRETCHED:
    default:
        // set InterpolationMode (InterpolationModeNearestNeighbor is better for transparency color
        objGraphic->SetInterpolationMode(Gdiplus::InterpolationModeNearestNeighbor);//InterpolationModeHighQualityBicubic

        // get image position
        x = xLeft;
        y = yUp;
        DstImageWidth=ControlWith;
        DstImageWidth+=8;// +=8 : FIXME gdi sucks --> ms got some work to do
        DstImageHeight=ControlHeight;
        break;
    }


    // first fill rect with background brush in case image doesn't cover full space
    // gdi uses abgr and gdi+ argb
    Gdiplus::Color BackColor((Gdiplus::ARGB)BGR_TO_RGB(this->BackGroundColor) | 0xff000000); // | 0xff000000 to make opaque color
    Gdiplus::SolidBrush BackBrush(BackColor);
    

    // create transparent attributes
    Gdiplus::ImageAttributes objImageAttributes;

    if (this->bUseTransparentColor)
    {
        Gdiplus::Color TransColor((Gdiplus::ARGB)BGR_TO_RGB(this->TransparentColor) | 0xff000000); // | 0xff000000 to make opaque color
        objImageAttributes.SetColorKey(
                                        Gdiplus::Color(0,TransColor.GetRed(), TransColor.GetGreen(), TransColor.GetBlue()),
                                        Gdiplus::Color(255,TransColor.GetRed(), TransColor.GetGreen(), TransColor.GetBlue()),
                                        Gdiplus::ColorAdjustTypeDefault);
    }

    // draw image with transparency
    objGraphic->FillRectangle(&BackBrush,0,0,ControlWith,ControlHeight);
    RetValue=objGraphic->DrawImage(objImage, Gdiplus::Rect(x,y,DstImageWidth,DstImageHeight),0,0,ImageWidth,ImageHeight,Gdiplus::UnitPixel,&objImageAttributes,NULL,NULL);
    delete objGraphic;

    return (RetValue==Gdiplus::Ok);
}

//-----------------------------------------------------------------------------
// Name: OnMouseMove
// Object: called when the control moves (WM_MOVE)
// Parameters :
//     in  : WM_MOVE params
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CTransparentToolbar::OnMove(WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(wParam);

    LONG NewPositionX=GET_X_LPARAM(lParam);   // horizontal position
    LONG NewPositionY=GET_Y_LPARAM(lParam);   // vertical position

    if(!this->IsInMoveOperation())
    {
        LONG DeltaX=NewPositionX-this->CurrentPositionX;
        LONG DeltaY=NewPositionY-this->CurrentPositionY;

        if ( (DeltaX!=0) || (DeltaY!=0) ) // can occur at control creation
        {
            // call move callback if any
            if (this->pEventsHandler)
                this->pEventsHandler->OnTransparentToolbarMove(this,DeltaX,DeltaY);
        }
    }
    this->CurrentPositionX = NewPositionX;
    this->CurrentPositionY = NewPositionY;

    return 0;
}

//-----------------------------------------------------------------------------
// Name: OnClose
// Object: called when window is asked to close
//          Notice : just use "return 0;" to avoid application closing (DefWindowProc calls DestroyWindow)
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CTransparentToolbar::OnClose(WPARAM wParam,LPARAM lParam)
{
    if (this->pEventsHandler)
        return this->pEventsHandler->OnTransparentToolbarClose(this,wParam,lParam);
    else
        return ::DefWindowProc(this->hWnd, WM_CLOSE, wParam, lParam);
}

//-----------------------------------------------------------------------------
// Name: OnKeyDown
// Object: called when mouse moves over the control (WM_KEYDOWN)
//          Notice : An application should return zero if it processes this message
// Parameters :
//     in  : WM_KEYDOWN params
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CTransparentToolbar::OnKeyDown(WPARAM wParam,LPARAM lParam)
{
    if (this->pEventsHandler)
        return this->pEventsHandler->OnTransparentToolbarKeyDown(this,wParam,lParam);
    else
        return 1;
}

//-----------------------------------------------------------------------------
// Name: OnMouseMove
// Object: called when mouse moves over the control (WM_MOUSEMOVE)
// Parameters :
//     in  : WM_MOUSEMOVE params
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CTransparentToolbar::OnMouseMove(WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);

    if (this->bLeftButtonDownOnBackGround)
    {
        POINT pt;
        ::GetCursorPos(&pt);
        if ( (this->ptCurrent.x == pt.x) && (this->ptCurrent.y == pt.y) )
            // windows is happy to send WM_MOUSEMOVE on a left click :(
            return 0;
        else
            this->StartCapture();
    }

    if (this->bCapture)
    {
        RECT Rect;
        POINT ptNew;
        LONG DeltaX;
        LONG DeltaY;

        ::GetCursorPos(&ptNew);
        // if position has changed
        if ( (ptNew.x!=this->ptCurrent.x)
            || (ptNew.y!=this->ptCurrent.y) )
        {
            // get current window rect
            ::GetWindowRect(this->hWnd,&Rect);

            // compute required displacement
            POINT p;
            DeltaX=ptNew.x-this->ptCurrent.x;
            DeltaY=ptNew.y-this->ptCurrent.y;
            p.x=Rect.left+DeltaX;
            p.y=Rect.top+DeltaY;

            this->ptCurrent=ptNew;

            if (this->BarPositionMode == BarPositionMode_POPUP)
            {
                // ::MoveWindow(this->hWnd,p.x,p.y,Rect.right-Rect.left,Rect.bottom-Rect.top,TRUE);
                ::SetWindowPos(this->hWnd,0,p.x,p.y,0,0,SWP_NOZORDER | SWP_NOSIZE);
            }

            // call move callback if any
            if (this->pEventsHandler)
                this->pEventsHandler->OnTransparentToolbarMove(this,DeltaX,DeltaY);
        }
    }
    else
    {
        // update cursor position
        ::GetCursorPos(&this->ptCurrent);

        // is mouse over move icon
        if (this->IsMouseOverMoveIcon())
        {
            // restore previous over button icon
            if (this->LastOverButtonIndex!=(DWORD)-1)
            {
                this->pToolTip->Activate(FALSE);

                if ( this->ArrayButtonsDescription[this->LastOverButtonIndex].Description.Enabled )
                    this->SetIconImgType(&this->ArrayButtonsDescription[this->LastOverButtonIndex],IconType_NORMAL);
                this->LastOverButtonIndex=(DWORD)-1;
            }

            // if button left is down
            if (wParam==MK_LBUTTON)
            {
                // if button left was not down previously
                if (!this->bLeftButtonDown)
                {
                    this->SetIconImgType(&this->MoveButton,IconType_DOWN);
                    this->bLeftButtonDown=TRUE;
                }
            }

            // if mouse was already on move icon
            if (this->bMouseOverMoveIcon)
                // do nothing
                return 0;

            // mouse enter the icon

            // change bMouseOverMoveIcon flag state
            this->bMouseOverMoveIcon=TRUE;

            this->CurrentToolTipText = this->MoveButton.Description.ToolTipText;
            this->pToolTip->Activate(*this->CurrentToolTipText!=0);

            if (!this->bLeftButtonDown)
                // change Icon
                this->SetIconImgType(&this->MoveButton,IconType_HOT);
           
        }
        else // mouse is not over move icon
        {
            // if mouse was over move icon
            if (this->bMouseOverMoveIcon)
            {
                // we have to change icon
                if (this->bMoveButtonCreated)
                    this->SetIconImgType(&this->MoveButton,IconType_NORMAL);
                // update bMouseOverMoveIcon flag state
                this->bMouseOverMoveIcon=FALSE;
                this->bLeftButtonDown=FALSE;
            }
        }

        // if mouse is over move icon we don't have to check if it's over an other button
        if (this->bMouseOverMoveIcon)
            // so go out
            return 0;

        // get required new icon type according to mouse left button state
        // and check if left button state has changed
        DWORD ButtonIndex;
        BOOL LeftButtonStateChanges=FALSE;
        IconType NewIconType;
        if (wParam==MK_LBUTTON)
        {
            NewIconType=IconType_DOWN;
            if (!this->bLeftButtonDown)
            {
                this->bLeftButtonDown=TRUE;
                LeftButtonStateChanges=TRUE;
            }
        }
        else
        {
            NewIconType=IconType_HOT;
            if (this->bLeftButtonDown)
            {
                this->bLeftButtonDown=FALSE;
                LeftButtonStateChanges=TRUE;
            }
        }

        // if mouse is over a command icon
        if (this->IsMouseOverIcon(&ButtonIndex))
        {
            // if left button state has changed or if mouse is over an other icon
            if ( (this->LastOverButtonIndex!=ButtonIndex) || LeftButtonStateChanges )
            {
                this->pToolTip->Activate(FALSE);

                // restore previous selected icon state (if not current icon)
                if ( (this->LastOverButtonIndex!=(DWORD)-1) && (this->LastOverButtonIndex!=ButtonIndex) )
                {
                    if ( this->ArrayButtonsDescription[this->LastOverButtonIndex].Description.Enabled )
                        this->SetIconImgType(&this->ArrayButtonsDescription[this->LastOverButtonIndex],IconType_NORMAL);
                }

                // update current selected icon state
                this->LastOverButtonIndex=ButtonIndex;
                if ( this->ArrayButtonsDescription[ButtonIndex].Description.Enabled )
                    this->SetIconImgType(&this->ArrayButtonsDescription[ButtonIndex],NewIconType);

                this->CurrentToolTipText = this->ArrayButtonsDescription[ButtonIndex].Description.ToolTipText;
                // activate tooltip only if string is specified
                this->pToolTip->Activate(*this->CurrentToolTipText!=0);
            }
        }
        else // mouse is no more over a command icon
        {
            // restore previous selected icon state
            if (this->LastOverButtonIndex!=(DWORD)-1)
            {
                if ( this->ArrayButtonsDescription[this->LastOverButtonIndex].Description.Enabled )
                    this->SetIconImgType(&this->ArrayButtonsDescription[this->LastOverButtonIndex],IconType_NORMAL);
                this->LastOverButtonIndex=(DWORD)-1;
            }

            this->CurrentToolTipText = NULL;
            this->pToolTip->Activate(FALSE);
        }
    }

    return 0;
}

LRESULT CTransparentToolbar::OnNotify(WPARAM wParam,LPARAM lParam)
{
    UNREFERENCED_PARAMETER(wParam);
    switch(((LPNMHDR)lParam)->code)
    {
        case TTN_GETDISPINFO:
        {
            LPNMTTDISPINFO lpnmtdi = (LPNMTTDISPINFO) lParam;
            lpnmtdi->hinst = NULL;
            lpnmtdi->uFlags=0;
            if (this->CurrentToolTipText)
                lpnmtdi->lpszText=this->CurrentToolTipText;
            else
                lpnmtdi->lpszText=_T("");
        }
        break;
    }
    return 0;
}


//-----------------------------------------------------------------------------
// Name: OnMouseLButtonDown
// Object: called on WM_LBUTTONDBLCLK
// Parameters :
//     in  : WM_LBUTTONDBLCLK params
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CTransparentToolbar::OnMouseLButtonDoubleClick(WPARAM wParam, LPARAM lParam)
{
    // WM_LBUTTONDOWN, WM_LBUTTONUP, WM_LBUTTONDBLCLK, and WM_LBUTTONUP
    UNREFERENCED_PARAMETER(lParam);
    UNREFERENCED_PARAMETER(wParam);
    // is mouse over icon
    DWORD ButtonIndex;
    // if mouse is over button, jump back to standard WM_LBUTTONDOWN case
    if (this->IsMouseOverIcon(&ButtonIndex) || this->IsMouseOverMoveIcon())
    {
        this->OnMouseLButtonDown(NULL,NULL);
        return 0;
    }

    if (this->pEventsHandler)
        this->pEventsHandler->OnTransparentToolbarBackgroundLeftDoubleClick(this);
    return 0;
}
void CTransparentToolbar::StartCapture()
{
    ::SetCursor(this->hMoveCursor);
    ::SetCapture( this->hWnd );
    this->bCapture=TRUE;
}

//-----------------------------------------------------------------------------
// Name: OnMouseLButtonDown
// Object: called on WM_LBUTTONDOWN
// Parameters :
//     in  : WM_LBUTTONDOWN params
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CTransparentToolbar::OnMouseLButtonDown(WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);
    UNREFERENCED_PARAMETER(wParam);

    this->pToolTip->Activate(FALSE);

    // is mouse over icon
    DWORD ButtonIndex;
    if (this->IsMouseOverIcon(&ButtonIndex))
    {
        if ( this->ArrayButtonsDescription[ButtonIndex].Description.Enabled )
            this->SetIconImgType(&this->ArrayButtonsDescription[ButtonIndex],IconType_DOWN);
        this->bLeftButtonDown=TRUE;
        return 0;
    }

    // is mouse over Move icon
    if (this->IsMouseOverMoveIcon())
    {
        this->SetIconImgType(&this->MoveButton,IconType_DOWN);

        this->bLeftButtonDown=TRUE;
        this->StartCapture();
    }

    // else mouse is over background
    if (this->bAllowWindowMoveFromBackGround)
    {
        this->bLeftButtonDownOnBackGround = TRUE;
        // store cursor position to detect "false" WM_MOUSEMOVE on left click
        // (windows is happy to send WM_MOUSEMOVE on a left click)
        ::GetCursorPos(&this->ptCurrent);
    }

    return 0;
}

//-----------------------------------------------------------------------------
// Name: OnMouseLButtonUp
// Object: called on WM_LBUTTONUP and WM_KILLFOCUS
// Parameters :
//     in  : WM_LBUTTONUP or WM_KILLFOCUS params (not used)
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CTransparentToolbar::OnMouseLButtonUp(WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);
    UNREFERENCED_PARAMETER(wParam);
    // reset button down state
    this->bLeftButtonDown=FALSE;
    this->bLeftButtonDownOnBackGround=FALSE;

    if (this->bCapture)
    {
        ::ReleaseCapture();
        this->bCapture=FALSE;

        if (this->IsMouseOverMoveIcon())
            this->SetIconImgType(&this->MoveButton,IconType_HOT);
        // else mouse is over background --> nothing to do

        if (this->pEventsHandler)
            this->pEventsHandler->OnTransparentToolbarMoveEnd(this);

        return 0;
    }
    // else

    DWORD ButtonIndex;
    if (!this->IsMouseOverIcon(&ButtonIndex))
        return 0;

    if (!this->ArrayButtonsDescription[ButtonIndex].Description.Enabled )
        return 0;

    this->SetIconImgType(&this->ArrayButtonsDescription[ButtonIndex],IconType_HOT);

    // call button click callback if any
    if (this->pEventsHandler)
        this->pEventsHandler->OnTransparentToolbarButtonClick(this,this->ArrayButtonsDescription[ButtonIndex].Description.ButtonId);

    return 0;
}

//-----------------------------------------------------------------------------
// Name: OnMouseRButtonUp
// Object: called on WM_RBUTTONUP
// Parameters :
//     in  : WM_RBUTTONUP params (not used)
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CTransparentToolbar::OnMouseRButtonUp(WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);
    UNREFERENCED_PARAMETER(wParam);
    POINT CursorPos;
    ::GetCursorPos(&CursorPos);

    DWORD ButtonIndex;
    if (this->IsMouseOverIcon(&ButtonIndex))
    {
        // if button is disabled, let background right click handle the event
        if ( this->ArrayButtonsDescription[ButtonIndex].Description.Enabled )
        {
            if (this->pEventsHandler)
                this->pEventsHandler->OnTransparentToolbarButtonRightClick(this,ButtonIndex);
            return 0; // avoid background event to occur
        }
    }
    RECT Rect={0};
    ::GetWindowRect(this->hWnd,&Rect);
    if (::PtInRect(&Rect,CursorPos))
    {
        if (this->pEventsHandler)
            this->pEventsHandler->OnTransparentToolbarBackgroundRightClick(this,&CursorPos);
    }

    return 0;
}

//-----------------------------------------------------------------------------
// Name: OnMouseLeave
// Object: called on WM_MOUSELEAVE
// Parameters :
//     in  : WM_MOUSELEAVE params (not used)
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CTransparentToolbar::OnMouseLeave(WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);
    UNREFERENCED_PARAMETER(wParam);

    // mouse can't be over icon so update flag and restore normal icon
    this->bMouseOverMoveIcon=FALSE;

    // reset button down state
    this->bLeftButtonDown=FALSE;
    if (this->bMoveButtonCreated)
        this->SetIconImgType(&this->MoveButton,IconType_NORMAL);

    if (this->LastOverButtonIndex!=(DWORD)-1)
    {
        if ( this->ArrayButtonsDescription[this->LastOverButtonIndex].Description.Enabled )
            this->SetIconImgType(&this->ArrayButtonsDescription[this->LastOverButtonIndex],IconType_NORMAL);
        this->LastOverButtonIndex=(DWORD)-1;
    }

    return 0;
}

LRESULT CTransparentToolbar::OnMouseMediaButtonDown(WPARAM wParam,LPARAM lParam)
{
#pragma message("WARNING MISSING ALL OTHER MEDIA BUTTONS, SEE OnKeyDown_ShellBrowserContainerBase !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
    // an application should return TRUE from this message if it processes it
    if (this->pEventsHandler)
        return this->pEventsHandler->OnTransparentToolbarMouseMediaButtonDown(this,wParam,lParam);
    else
        return FALSE;
}

//-----------------------------------------------------------------------------
// Name: IsMouseOverIcon
// Object: check if mouse is over icon
// Parameters :
//     in  :
//     out : int* pButtonIndex
//     return : 
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::IsMouseOverIcon(DWORD* pButtonIndex)
{
    *pButtonIndex=(DWORD)-1;
    POINT pt;
    RECT Rect;
    ::GetCursorPos(&pt);
    ::ScreenToClient(this->hWnd,&pt);
    
    for (SIZE_T Cnt=0;Cnt<this->ArrayButtonsDescriptionSize;Cnt++)
    {
        this->GetButtonRectEx(&this->ArrayButtonsDescription[Cnt],&Rect);
        if (::PtInRect(&Rect,pt))
        {
            *pButtonIndex=(DWORD)Cnt;
            return TRUE;
        }
    }
    return FALSE;
}

//-----------------------------------------------------------------------------
// Name: IsMouseOverMoveIcon
// Object: check if mouse is over TransparentToolbar
// Parameters :
//     in  :
//     out :
//     return : 
//-----------------------------------------------------------------------------
BOOL CTransparentToolbar::IsMouseOverMoveIcon()
{
    if (!this->bMoveButtonCreated)
        return FALSE;

    POINT pt;
    RECT Rect;
    ::GetCursorPos(&pt);
    ::ScreenToClient(this->hWnd,&pt);
    this->GetButtonRectEx(&this->MoveButton,&Rect);

    return ::PtInRect(&Rect,pt);
}

//-----------------------------------------------------------------------------
// Name: SetIconImgType
// Object: set icon image depending on Type
// Parameters :
//     in  : BUTTON_DESCRIPTION_EX* pButtonDescription : itm on which to apply changes
//           IconType Type : IconType_NORMAL, IconType_HOT or IconType_DOWN
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CTransparentToolbar::SetIconImgType(BUTTON_DESCRIPTION_EX* pButtonDescription,IconType Type)
{
    pButtonDescription->CurrentIconType = Type;
    RECT Rect;
    this->GetButtonRectEx(pButtonDescription,&Rect);
    ::InvalidateRect(this->hWnd,&Rect,TRUE);
}

HICON CTransparentToolbar::GetIconImg(BUTTON_DESCRIPTION_EX* pButtonDescription)
{
    switch (pButtonDescription->CurrentIconType)
    {
    default:
    case IconType_NORMAL:
        return pButtonDescription->Description.hIcon;
    case IconType_HOT:
        return pButtonDescription->Description.hIconHot;
    case IconType_DOWN:
        return pButtonDescription->Description.hIconDown;
    case IconType_DISABLED:
        return pButtonDescription->Description.hIconDisabled;
    }
}
