/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class for a TransparentToolbar control
//-----------------------------------------------------------------------------


#pragma once

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0501 // for xp os
#endif
#include <windows.h>
#include <commctrl.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)
#include <gdiplus.h>
#pragma comment (lib,"gdiplus.lib")

#include "../Window/WindowBase.h"
#include "../ToolTip/ToolTip.h"

#define CTransparentToolbar_TransparentToolbar_CLASSEX _T("CTransparentToolbarClassName")
#define CTransparentToolbar_ToolTipTextLen 128

class CTransparentToolbar;

class ITransparentToolbarEvents
{
public:
    virtual void OnTransparentToolbarMove(CTransparentToolbar* pSender,LONG DeltaX,LONG DeltaY)=0;
    virtual void OnTransparentToolbarMoveEnd(CTransparentToolbar* pSender)=0;
    virtual void OnTransparentToolbarButtonClick(CTransparentToolbar* pSender,DWORD ButtonId)=0;
    virtual void OnTransparentToolbarButtonRightClick(CTransparentToolbar* pSender,DWORD ButtonId)=0;
    virtual void OnTransparentToolbarBackgroundLeftDoubleClick(CTransparentToolbar* pSender)=0;
    virtual void OnTransparentToolbarBackgroundRightClick(CTransparentToolbar* pSender,POINT* pCursorPos)=0;
    // OnTransparentToolbarKeyDown: An application should return zero if it processes this message
    virtual LRESULT OnTransparentToolbarKeyDown(CTransparentToolbar* pSender,WPARAM wParam, LPARAM lParam)=0;
    virtual LRESULT OnTransparentToolbarMouseMediaButtonDown(CTransparentToolbar* pSender,WPARAM wParam, LPARAM lParam)=0;
    // OnClose : just use return 0; to avoid window closing (DefWindowProc calls DestroyWindow)
    virtual LRESULT OnTransparentToolbarClose(CTransparentToolbar* pSender,WPARAM wParam,LPARAM lParam)=0;
};

class CTransparentToolbarEventsBase: public ITransparentToolbarEvents
{
public:
    virtual void OnTransparentToolbarMove(CTransparentToolbar* pSender,LONG DeltaX,LONG DeltaY)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(DeltaX);
        UNREFERENCED_PARAMETER(DeltaY);
    }
    virtual void OnTransparentToolbarMoveEnd(CTransparentToolbar* pSender)
    {
        UNREFERENCED_PARAMETER(pSender);
    }
    virtual void OnTransparentToolbarButtonClick(CTransparentToolbar* pSender,DWORD ButtonId)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(ButtonId);
    }
    virtual void OnTransparentToolbarButtonRightClick(CTransparentToolbar* pSender,DWORD ButtonId)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(ButtonId);
    }
    virtual void OnTransparentToolbarBackgroundLeftDoubleClick(CTransparentToolbar* pSender)
    {
        UNREFERENCED_PARAMETER(pSender);
    }
    virtual void OnTransparentToolbarBackgroundRightClick(CTransparentToolbar* pSender,POINT* pCursorPos)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(pCursorPos);
    }

    // OnTransparentToolbarKeyDown: An application should return zero if it processes this message
    virtual LRESULT OnTransparentToolbarKeyDown(CTransparentToolbar* pSender,WPARAM wParam, LPARAM lParam)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(wParam);
        UNREFERENCED_PARAMETER(lParam);
        return 1;
    }

    virtual LRESULT OnTransparentToolbarMouseMediaButtonDown(CTransparentToolbar* pSender,WPARAM wParam, LPARAM lParam)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(wParam);
        UNREFERENCED_PARAMETER(lParam);

        // WORD fwKeys = GET_KEYSTATE_WPARAM (wParam); 
        // WORD fwButton = GET_XBUTTON_WPARAM (wParam);
        // int xPos = GET_X_LPARAM(lParam); 
        // int yPos = GET_Y_LPARAM(lParam);
        //switch (fwKeys)
        //{
        //case MK_XBUTTON1: // backward
        //    return TRUE;

        //case MK_XBUTTON2: // forward
        //    return TRUE;
        //}

        // an application should return TRUE from this message if it processes it
        return FALSE;
    }

    // OnClose : just use return 0; to avoid window closing (DefWindowProc calls DestroyWindow)
    virtual LRESULT OnTransparentToolbarClose(CTransparentToolbar* pSender,WPARAM wParam,LPARAM lParam)
    {
        UNREFERENCED_PARAMETER(pSender);
        // OnClose : just use return 0; to avoid application closing (DefWindowProc calls DestroyWindow)
        return ::DefWindowProc(((CWindowBase*)pSender)->GetControlHandle(), WM_CLOSE, wParam, lParam);
    }
};

class CTransparentToolbar:public CWindowBase
{
public:
    typedef enum tagBarPositionMode
    {
        BarPositionMode_POPUP=0, // tool bar external to parent layout
        BarPositionMode_CHILD,   // tool bar internal to parent layout
    };

    typedef enum tagButtonPosition
    {
        ButtonPosition_LEFT_OR_TOP=0,
        ButtonPosition_CENTER_OR_MIDDLE,
        ButtonPosition_RIGHT_OR_BOTTOM
    };

    
    typedef struct tagButtonDescription
    {
        HICON hIcon;
        HICON hIconHot;
        HICON hIconDown;
        HICON hIconDisabled;
        tagButtonPosition Position;
        BOOL Enabled;
// #pragma message ("LineBreakBeforeIcon to be implemented") : commented to avoid multiple messages in each cpp including .h
        // BOOL  LineBreakBeforeIcon;
        DWORD SpacerBeforeIconInPixels;
        DWORD ButtonId;
        PVOID UserData;
        TCHAR ToolTipText[CTransparentToolbar_ToolTipTextLen];
    }BUTTON_DESCRIPTION,*PBUTTON_DESCRIPTION;

    typedef enum tagBackgroundType
    {   
        // gradiant brush : BackgroundColor and BackgroundColor2 fields used
        // tips gdi values are used
        //    LinearGradientModeHorizontal = 0,
        //    LinearGradientModeVertical = 1,
        //    LinearGradientModeForwardDiagonal = 2,
        //    LinearGradientModeBackwardDiagonal = 3
        BACKGROUND_LINEAR_GRADIENT_MODE_HORIZONTAL = 0,
        BACKGROUND_LINEAR_GRADIENT_MODE_VERTICAL = 1,
        BACKGROUND_LINEAR_GRADIENT_MODE_FORWARD_DIAGONAL = 2,
        BACKGROUND_LINEAR_GRADIENT_MODE_BACKWARD_DIAGONAL = 3,

        // solid brush : only BackgroundColor field used
        BACKGROUND_SOLID_BRUSH,

        // image brush : only BackgroundImage field used
        BACKGROUND_IMAGE_POSITION_STRETCHED,
        BACKGROUND_IMAGE_POSITION_CENTER,
        BACKGROUND_IMAGE_POSITION_UP_LEFT,
        BACKGROUND_IMAGE_POSITION_UP_CENTER,
        BACKGROUND_IMAGE_POSITION_UP_RIGHT,
        BACKGROUND_IMAGE_POSITION_LEFT,
        BACKGROUND_IMAGE_POSITION_RIGHT,
        BACKGROUND_IMAGE_POSITION_BOTTOM_LEFT,
        BACKGROUND_IMAGE_POSITION_BOTTOM_CENTER,
        BACKGROUND_IMAGE_POSITION_BOTTOM_RIGHT
    };

private:

    typedef enum IconType{IconType_NORMAL,IconType_HOT,IconType_DOWN,IconType_DISABLED};

    typedef struct tagButtonDescriptionEx
    {
        BUTTON_DESCRIPTION Description;
        POINT Position;
        IconType CurrentIconType;
    }BUTTON_DESCRIPTION_EX,*PBUTTON_DESCRIPTION_EX;

    HINSTANCE hInstance;
    HWND hWndParent;

    ITransparentToolbarEvents* pEventsHandler;

    BOOL bMouseOverMoveIcon;
    BOOL bCapture;
    BOOL bLeftButtonDown;
    BOOL bLeftButtonDownOnBackGround;
    BOOL bAllowWindowMoveFromBackGround;

    POINT ptCurrent;
    HCURSOR hMoveCursor;
    COLORREF TransparentColor;
    BOOL bUseTransparentColor;
    tagBackgroundType BackgroundType;
    COLORREF BackGroundColor;
    COLORREF BackGroundGradiantColor1;
    COLORREF BackGroundGradiantColor2;
    Gdiplus::Image* objImage;
    ULONG_PTR gdiplusToken;
    BOOL bGdiPlusStarted;
    BYTE OpacityPercent;
    tagBarPositionMode BarPositionMode;

    HBRUSH BackGroundBrush;

    LONG InitialPositionX;
    LONG InitialPositionY;
    LONG CurrentPositionX;
    LONG CurrentPositionY;
    LONG Width;
    LONG Height;
    int IconWidth;
    int IconHeight;
    UINT NextDialogControlId;
    LONG IconsSpacersLeftOrTop;
    LONG IconsSpacersCenterOrMiddle;
    LONG IconsSpacersRightOrBottom;
    LONG MinWidth;
    LONG MinHeight;
    RECT Margins;
    BOOL bTopMost;
    BOOL bVertical;

    DWORD LastOverButtonIndex;
    LONG NbButtonsLeft;
    LONG NbButtonsRightOrBottom;
    LONG NbButtonsCenterOrMiddle;

    BUTTON_DESCRIPTION_EX MoveButton;
    BOOL bMoveButtonCreated;
    BOOL bShowMoveButton;
    BUTTON_DESCRIPTION_EX* ArrayButtonsDescription;
    SIZE_T ArrayButtonsDescriptionSize;
    CToolTip* pToolTip;
    TCHAR* CurrentToolTipText;

    BOOL CreateControl();

    BOOL IsMouseOverIcon(DWORD* pButtonIndex);
    BOOL IsMouseOverMoveIcon();
    BOOL IsInMoveOperation(){return this->bCapture;}

    void StartCapture();

    virtual LRESULT OnMove(WPARAM wParam, LPARAM lParam);

    void SetIconImgType(BUTTON_DESCRIPTION_EX* pButtonDescription,IconType Type);
    HICON GetIconImg(BUTTON_DESCRIPTION_EX* pButtonDescription);
    void UpdateIconsPosition();
    void DrawBackGround(HDC hdc,RECT* pRect);
    BOOL SetBackgroundGradient(HDC hdc,RECT* pRect,COLORREF BackgroundGradiantColor1,COLORREF BackgroundGradiantColor2,Gdiplus::LinearGradientMode BackgroundGradiantType);
    BOOL SetBackgroundPicture(HDC hdc,RECT* pRect);
    virtual LRESULT DoPaint(HDC dc);
    virtual LRESULT OnMouseLButtonDoubleClick(WPARAM wParam, LPARAM lParam);
    virtual LRESULT OnMouseLeave(WPARAM wParam,LPARAM lParam);
    virtual LRESULT OnMouseLButtonDown(WPARAM wParam,LPARAM lParam);
    virtual LRESULT OnMouseLButtonUp(WPARAM wParam,LPARAM lParam);
    virtual LRESULT OnMouseRButtonUp(WPARAM wParam,LPARAM lParam);
    virtual LRESULT OnMouseMove(WPARAM wParam,LPARAM lParam);
    virtual LRESULT OnNotify(WPARAM wParam,LPARAM lParam);
    // OnClose : just use return 0; to avoid application closing (DefWindowProc calls DestroyWindow)
    virtual LRESULT OnClose(WPARAM wParam,LPARAM lParam);
    // OnKeyDown: An application should return zero if it processes this message
    virtual LRESULT OnKeyDown(WPARAM wParam,LPARAM lParam);
    virtual LRESULT OnMouseMediaButtonDown(WPARAM wParam,LPARAM lParam);
    BOOL GetButtonRectEx(IN BUTTON_DESCRIPTION_EX* pButtonDescription,OUT RECT* pRect);
public:

    CTransparentToolbar(HINSTANCE hInstance,HWND hParent,ITransparentToolbarEvents* pEventsHandler,BOOL bInitiallyVisible,BOOL bVertical,
                        tagBackgroundType BackgroundType,COLORREF BackGroundColor,COLORREF BackGroundGradiantColor1,COLORREF BackGroundGradiantColor2,TCHAR* BackgroundImage,
                        COLORREF TransparentColor,BOOL bUseTransparentColor,
                        BYTE OpacityPercent,
                        tagBarPositionMode BarPositionMode,
                        LONG X,LONG Y,
                        LONG Width,LONG Height,
                        int IconWidth,int IconHeight,
                        BUTTON_DESCRIPTION* ArrayButtons,
                        SIZE_T ArrayButtonsSize,
                        BUTTON_DESCRIPTION* pMoveButton,BOOL bShowMoveButton,
                        BOOL bAllowWindowMoveFromBackGround,
                        RECT* pMargins,BOOL bTopMost
                        );
    ~CTransparentToolbar(void);

    BOOL SetWidth(LONG Width);
    LONG GetMinWidth();
    BOOL SetHeight(LONG Height);
    LONG GetMinHeight();
    BOOL SetMargins(RECT* pMargins);
    BOOL SetPosition(int x,int y,int cx,int cy);
    BOOL IsVertical(){return this->bVertical;}
    tagBarPositionMode GetBarPositionMode(){return this->BarPositionMode;}

    HWND GetControlHandle();
    BOOL ShowMove(BOOL bShow);
    BOOL EnableButton(DWORD ButtonId,BOOL bEnable);
    BOOL ChangeIcons(DWORD ButtonId,HICON hIcon,HICON hIconHot,HICON hIconDown,HICON hIconDisabled);
    BOOL GetButtonUserData(IN DWORD ButtonId,OUT PVOID* pUserData);
    BOOL GetButtonRect(IN DWORD ButtonId,OUT RECT* pButtonRect);
};

