/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper for tree view control
//-----------------------------------------------------------------------------
#pragma once

#include "../../File/TextFile.h"
#include "../../File/StdFileOperations.h"
#include "../../String/WildCharCompare.h"
#include "../../Clipboard/Clipboard.h"
#include "../Menu/PopUpMenu.h"
#include "../../String/AnsiUnicodeConvert.h"

#include <windowsx.h>
#include <commctrl.h>

#define CTreeview_REPLACED_CHAR_ARRAY_SIZE 3
#define CTreeview_REPLACED_CHAR_MAX_INCREASE 5
#define CTreeview_TXT_FILE_SPLITTER _T("\t")

#define CTreeview_MAX_PATH  4096

class CTreeview;
class ITreeviewEvents
{
public:
    virtual void OnTreeviewPopUpMenuItemClick(CTreeview* pSender,UINT MenuID)=0;
    virtual void OnTreeviewCheckedStateChanged(CTreeview* pSender,HTREEITEM hItem)=0;
    virtual void OnTreeviewSelectedItemChanged(CTreeview* pSender,LPNMTREEVIEW pChangeInfos)=0;
    virtual void OnTreeviewDoubleClick(CTreeview* pSender,HTREEITEM hItem)=0;
    
};

class CTreeviewEventsBase:public ITreeviewEvents
{
public:
    virtual void OnTreeviewPopUpMenuItemClick(CTreeview* pSender,UINT MenuID)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(MenuID);
    }
    virtual void OnTreeviewCheckedStateChanged(CTreeview* pSender,HTREEITEM hItem)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(hItem);
    }
    virtual void OnTreeviewSelectedItemChanged(CTreeview* pSender,LPNMTREEVIEW pChangeInfos)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(pChangeInfos);
    }
    virtual void OnTreeviewDoubleClick(CTreeview* pSender,HTREEITEM hItem)
    {
        UNREFERENCED_PARAMETER(pSender);
        UNREFERENCED_PARAMETER(hItem);
    }
};


class CTreeview
{
public:
    typedef enum tagCheckState
    {
        CHECK_STATE_CHECKED=2,
        CHECK_STATE_UNCHECKED=1,
        CHECK_STATE_NO_CHECK_BOX_IMAGE=0
    };
    
    typedef enum tagSearchStringType
    {
        SearchStringType_WILD_COMPARE=0,    // CWildCharCompare::WildICmp default for ascending compatibility
        SearchStringType_REGULAR_EXPRESSION,// regexp
        SearchStringType_BASIC              // _tcsstr
    };

    CPopUpMenu* pPopUpMenu;

    BOOL Clear();
    BOOL Remove(HTREEITEM hItem);
    BOOL Expand(HTREEITEM hItem,BOOL FullBranch);
    BOOL Expand(HTREEITEM hItem);
    BOOL Expand();
    BOOL Collapse(HTREEITEM hItem,BOOL FullBranch);
    BOOL Collapse(HTREEITEM hItem);
    BOOL Collapse();

    HTREEITEM GetRoot();
    HTREEITEM GetParent(HTREEITEM hItem);
    HTREEITEM GetChild(HTREEITEM hItem);
    HTREEITEM GetPrevious(HTREEITEM hItem);
    HTREEITEM GetNext(HTREEITEM hItem);
    HTREEITEM Insert(HTREEITEM hParent,HTREEITEM hInsertAfter,TCHAR* Text,LPARAM UserParam);
    HTREEITEM Insert(HTREEITEM hParent,HTREEITEM hInsertAfter,TCHAR* Text);
    HTREEITEM Insert(HTREEITEM hParent,TCHAR* Text,LPARAM UserParam);
    HTREEITEM Insert(HTREEITEM hParent,TCHAR* Text);
    // InsertSorted : insert item alphabetically sorted
    HTREEITEM InsertSorted(HTREEITEM hParent,TCHAR* Text,LPARAM UserParam=NULL);
    HWND GetControlHandle();
    CTreeview(HWND hWndTreeView,BOOL HasCheckBoxes=FALSE,BOOL HasDefaultMenuItems=TRUE);
    virtual ~CTreeview(void);

    BOOL SetEventsHandler(ITreeviewEvents* pTreeviewEvents);

    void EnableCustomDraw(BOOL bEnable);
    void EnablePopUpMenu(BOOL bEnable);

    BOOL Save(BOOL bExpandedOnly,BOOL bSelectedOnly);
    BOOL Save(BOOL bExpandedOnly,BOOL bSelectedOnly,TCHAR* DefaultExtension);
    BOOL Save(TCHAR* pszFileName,BOOL bExpandedOnly,BOOL bSelectedOnly);
    DWORD GetItemTextLen(HTREEITEM hTreeItem);
    BOOL GetItemText(HTREEITEM hTreeItem,TCHAR* pszText,DWORD TextMaxSize);
    DWORD GetItemDepth(HTREEITEM hItem);
    DWORD GetChildCount(HTREEITEM hItem); // get first level children count
    HTREEITEM GetSelectedItem();
    BOOL SetSelectedItem(HTREEITEM hItem);
    BOOL SetBoldState(HTREEITEM hItem,BOOL bBold);
    BOOL IsSelected(HTREEITEM hItem);
    BOOL IsChecked(HTREEITEM hItem);
    BOOL IsParentChecked(HTREEITEM hItem);
    BOOL IsChildChecked(HTREEITEM hItem);
    BOOL GetCheckedState(HTREEITEM hItem,tagCheckState* pCheckState);
    BOOL SetCheckedState(HTREEITEM hItem,tagCheckState CheckState);
    BOOL RemoveCheckForItem(HTREEITEM hItem);

    BOOL OnNotify(WPARAM wParam, LPARAM lParam);
    BOOL SetItemUserData(HTREEITEM hItem,LPVOID UserData);
    BOOL GetItemUserData(HTREEITEM hItem,OUT LPVOID* pUserData);

    typedef BOOL (*pfParseCallBack)(HTREEITEM hItem,LPVOID UserParam); // return TRUE to continue parsing, FALSE to stop it
    BOOL Parse(pfParseCallBack ParseCallBack,LPVOID UserParam);
    BOOL Parse(HTREEITEM hItem,pfParseCallBack ParseCallBack,LPVOID UserParam);
    BOOL Parse(HTREEITEM hItem,DWORD MaxDepthFromItem,pfParseCallBack ParseCallBack,LPVOID UserParam);
    BOOL Parse(HTREEITEM hItem,DWORD MaxDepthFromItem,pfParseCallBack ParseCallBack,LPVOID UserParam,BOOL ReverseOrder);
    BOOL ParseSpecifiedDepth(DWORD SpecifiedDepthFromRoot,pfParseCallBack ParseCallBack,LPVOID UserParam);
    BOOL ParseSpecifiedDepth(HTREEITEM hItem,DWORD SpecifiedDepthFromRoot,pfParseCallBack ParseCallBack,LPVOID UserParam);
    BOOL ParseSpecifiedDepth(HTREEITEM hItem,DWORD SpecifiedDepthFromRoot,pfParseCallBack ParseCallBack,LPVOID UserParam,BOOL ReverseOrder);

    BOOL Find(TCHAR* Str,HTREEITEM* phFirstMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL Find(TCHAR* Str,DWORD MaxDepthFromRoot,HTREEITEM* phFirstMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL FindNext(TCHAR* Str,HTREEITEM hItem,HTREEITEM* phNextMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL FindNext(TCHAR* Str,HTREEITEM hItem,DWORD MaxDepthFromItem,BOOL AllowParentSearch,HTREEITEM* phNextMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL FindPrevious(TCHAR* Str,HTREEITEM hItem,HTREEITEM* phPreviousMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL FindPrevious(TCHAR* Str,HTREEITEM hItem,DWORD MaxDepthFromItem,BOOL AllowParentSearch,HTREEITEM* phPreviousMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL FindAtSpecifiedDepth(TCHAR* Str,DWORD DepthFromRoot,HTREEITEM* phFirstMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL FindNextAtSpecifiedDepth(TCHAR* Str,HTREEITEM hItem,DWORD DepthFromRoot,HTREEITEM* phNextMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL FindNextAtSpecifiedDepth(TCHAR* Str,HTREEITEM hItem,DWORD DepthFromRoot,BOOL AllowParentSearch,HTREEITEM* phNextMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL FindPreviousAtSpecifiedDepth(TCHAR* Str,HTREEITEM hItem,DWORD DepthFromRoot,HTREEITEM* phPreviousMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL FindPreviousAtSpecifiedDepth(TCHAR* Str,HTREEITEM hItem,DWORD DepthFromRoot,BOOL AllowParentSearch,HTREEITEM* phPreviousMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL FindInChildren(TCHAR* Str,HTREEITEM hItem,DWORD MaxDepthFromItem,HTREEITEM* phFirstMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);

    BOOL SetItemText(HTREEITEM hItem,TCHAR* Text);
    BOOL SetItemIconIndex(HTREEITEM hItem,int IconIndex,int SelectedIconIndex);
    BOOL SetItemIconIndex(HTREEITEM hItem,int IconIndex);
    BOOL GetItemIconIndex(HTREEITEM hItem,int* pIconIndex,int* pSelectedIconIndex);
    BOOL GetItemIconIndex(HTREEITEM hItem,int* pIconIndex);
    void SetDefaultMenuIcons(HINSTANCE hInstance,int IdIconCopySelected,int IdIconSaveSelected,int IdIconSaveExpanded,int IdIconSaveAll,int Width,int Height);
    void SetDefaultMenuIcons(HICON IconCopySelected,HICON IconSaveSelected,HICON IconSaveExpanded,HICON IconSaveAll);
    int AddIcon(HMODULE hModule,int IdIcon,int Width,int Height);
    int AddIcon(HMODULE hModule,int IdIcon);
    int AddIcon(HICON hIcon);

    HIMAGELIST SetImageList(HIMAGELIST hImgList,BOOL NewImageListDestructionManagedByTreeView);
    FORCEINLINE HIMAGELIST GetImageList(){return this->hImgListNormal;}

    BOOL RemoveIcon(int IconIndex);
    BOOL RemoveAllIcons();

    BOOL MoveItemTop(HTREEITEM hTreeItem);
    BOOL MoveItemUp(HTREEITEM hTreeItem);
    BOOL MoveItemDown(HTREEITEM hTreeItem);
    BOOL MoveItemBottom(HTREEITEM hTreeItem);
protected:
    HWND hWndControl;
    HWND hWndParent;
    HIMAGELIST hImgListNormal;
    BOOL ImageListDestructionManagedByTreeView;

    enum tagSavingType
    {
        SAVING_TYPE_XML,
        SAVING_TYPE_HTML,
        SAVING_TYPE_TXT
    };
    tagSavingType SavingType;
    HANDLE hFile;

    ITreeviewEvents* pTreeviewEvents;

    BOOL CustomDrawEnabled;
    BOOL PopUpMenuEnabled;
    UINT MenuIdCopySelected;
    UINT MenuIdSaveExpanded;
    UINT MenuIdSaveSelected;
    UINT MenuIdSaveAll;

    void CommonConstructor(BOOL HasDefaultMenuItems);

    TCHAR pszFileName[CTreeview_MAX_PATH];
    DWORD GetTreeViewItemDepth(HTREEITEM hTreeItem);
    void ShowError(TCHAR* psz);
    BOOL WriteTreeViewItemHeader(DWORD ItemDepth);
    BOOL WriteTreeViewItemFooter();
    BOOL WriteTreeViewItemContent(HTREEITEM hTreeItem);
    BOOL SaveTreeViewItemContent(HTREEITEM hTreeItem,DWORD ItemDepth,BOOL bExpandedOnly);
    BOOL OpenFile(TCHAR* pszFileName);
    BOOL CloseFile(BOOL bSuccess);
    void SwitchCheckedState(HTREEITEM hItem);
    void StrReplace(TCHAR* pszInputString,TCHAR* pszOutputString,TCHAR* pszOldStr,TCHAR* pszNewStr);
    void XMLConvert(TCHAR* pszInputString,TCHAR* pszOutputString,BOOL bToXML);
    typedef struct tagFindStruct
    {
        CTreeview* pTreeView;
        TCHAR* SearchedStringOrRegularExpr;
        BOOL ItemFound;
        HTREEITEM hMatchingItem;
        tagSearchStringType SearchStringType;
    }FIND_STRUCT,*PFIND_STRUCT;
    static BOOL FindCallBackStatic(HTREEITEM hItem,LPVOID UserParam);

    typedef struct tagIsChildCheckedCallBackStruct
    {
        CTreeview* pTreeView;
        BOOL ChildChecked;
    }IS_CHILD_CHECKED_CALLBACK_STRUCT,*PIS_CHILD_CHECKED_CALLBACK_STRUCT;
    static BOOL IsChildCheckedCallBack(HTREEITEM hItem,LPVOID UserParam);

    BOOL Parse(HTREEITEM hItem,DWORD MaxDepthFromItem,DWORD SpecifiedDepthFromRoot,pfParseCallBack ParseCallBack,LPVOID UserParam,BOOL ReverseOrder);
    BOOL FindCallBack(HTREEITEM hItem,FIND_STRUCT* pFindStruct);
    BOOL FindNext(TCHAR* Str,HTREEITEM hItem,DWORD MaxDepthFromItem,DWORD SpecifiedDepthFromRoot,BOOL AllowParentSearch,HTREEITEM* phNextMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL FindPrevious(TCHAR* Str,HTREEITEM hItem,DWORD MaxDepthFromItem,DWORD SpecifiedDepthFromRoot,BOOL AllowParentSearch,HTREEITEM* phPreviousMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);
    BOOL Find(TCHAR* Str,HTREEITEM hFromItem,DWORD MaxDepthFromItem,DWORD SpecifiedDepthFromRoot,BOOL ReverseOrder,BOOL AllowParentSearch,HTREEITEM* phNextMatchingItem,IN tagSearchStringType SearchStringType=SearchStringType_WILD_COMPARE);

    virtual LRESULT OnCustomDraw(WPARAM wParam,LPARAM lParam);
    virtual COLORREF GetBackgroundColor(HTREEITEM hTreeItem);
    virtual COLORREF GetForegroundColor(HTREEITEM hTreeItem);

    LRESULT OnCustomDrawBase(WPARAM wParam,LPARAM lParam);
    COLORREF GetBackgroundColorBase(HTREEITEM hTreeItem);
    COLORREF GetForegroundColorBase(HTREEITEM hTreeItem);

    typedef enum tagMoveItemOperation
    {
        MoveItemOperation_TOP,
        MoveItemOperation_UP,
        MoveItemOperation_DOWN,
        MoveItemOperation_BOTTOM
    };

    class CTreeviewSortingObject
    {
    public:
        HTREEITEM hItem;
        LPVOID OriginalParam;
        // SIZE_T Index;
        CTreeviewSortingObject(HTREEITEM hItem,
                               LPVOID OriginalParam// ,
                               // SIZE_T Index
                               )
        {
            this->hItem = hItem;
            this->OriginalParam = OriginalParam;
            // this->Index = Index;
        }
    };
    BOOL MoveItem(HTREEITEM hTreeItem,tagMoveItemOperation MoveOperation);
    static int CALLBACK CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
};
