/*
Copyright (C) 2004-2016 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2016 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "WindowBase.h"

CWindowBase::CWindowBase()
{
    this->hWnd = NULL;
    this->bTrackMouseEventSet = FALSE;
    this->bEnabledoublebuffering = TRUE;
    this->bUseParentWindowContentAsBackground = FALSE;
}
CWindowBase::~CWindowBase()
{
    if (this->hWnd)
    {
        ::DestroyWindow(this->hWnd);
        this->hWnd = NULL;
    }
}

LRESULT CWindowBase::DoPaint(HDC hdc)
{
    RECT Rect;
    this->GetClientRect(&Rect);
    ::FillRect(hdc,&Rect,::GetSysColorBrush(COLOR_3DFACE));
    return 0;
}

BOOL CWindowBase::ScreenToClientEx(IN HWND hWndRef,IN OUT LPRECT pRect)
{
    /*
    POINT p;
    p.x=lpRect->left;
    p.y=lpRect->top;
    ScreenToClient(hDlg,&p);
    lpRect->left=p.x;
    lpRect->top=p.y;
    p.x=lpRect->right;
    p.y=lpRect->bottom;
    ScreenToClient(hDlg,&p);
    lpRect->right=p.x;
    lpRect->bottom=p.y;
    */
    if (!::ScreenToClient (hWndRef, (LPPOINT)pRect))
        return FALSE;
    if(!::ScreenToClient (hWndRef, ((LPPOINT)pRect)+1))
        return FALSE;
    return TRUE;
}

LRESULT CALLBACK CWindowBase::WindowProcStatic(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
    CWindowBase* pWindow;
    switch (uMsg)
    {
    case WM_NCCREATE:
        pWindow = (CWindowBase*)(((CREATESTRUCT*)lParam)->lpCreateParams);
        break;
    case WM_CREATE:
        pWindow = (CWindowBase*)(((CREATESTRUCT*)lParam)->lpCreateParams);
        // put current object in control window user data (can't be set in WM_NCCREATE)
        pWindow->hWnd = hWnd;
        ::SetWindowLongPtr(hWnd,GWLP_USERDATA,(LONG_PTR)pWindow);
        break;
    default:
        pWindow = (CWindowBase*)::GetWindowLongPtr(hWnd,GWLP_USERDATA);
        break;
    }

    if (pWindow)
        return pWindow->WindowProc(hWnd,uMsg,wParam,lParam);
    else
        return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CWindowBase::OnMouseWheel(WPARAM wParam,LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);

    int zDelta = GET_WHEEL_DELTA_WPARAM(wParam);
    for( int i = 0; i < abs(zDelta / WHEEL_DELTA * 2); i++ ) 
        ::SendMessage(this->hWnd,WM_VSCROLL, zDelta > 0 ? SB_LINEUP : SB_LINEDOWN,0);

    return 0;
}
LRESULT CWindowBase::OnMouseHWheel(WPARAM wParam,LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);

    int zDelta = GET_WHEEL_DELTA_WPARAM(wParam);
    for( int i = 0; i < abs(zDelta / WHEEL_DELTA * 2); i++ ) 
        ::SendMessage(this->hWnd,WM_HSCROLL, zDelta > 0 ? SB_LINERIGHT : SB_LINELEFT,0);
    return 0;
}

// WndProc example
LRESULT CALLBACK CWindowBase::WindowProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
    switch(uMsg)
    {
    MSG_HANDLER(WM_NOTIFY, this->OnNotify)
    MSG_HANDLER(WM_MOUSEMOVE, this->OnMouseMoveInternal)
    MSG_HANDLER(WM_ERASEBKGND, this->OnEraseBackground)
    MSG_HANDLER(WM_PAINT, this->OnPaint)
    MSG_HANDLER(WM_SIZING, this->OnSizing)
    MSG_HANDLER(WM_SIZE, this->OnSize)
    MSG_HANDLER(WM_NCHITTEST, this->OnNCHitTest)
    MSG_HANDLER(WM_PRINTCLIENT, this->OnPrintClient)
    MSG_HANDLER(WM_MOUSELEAVE, this->OnMouseLeaveInternal)
    MSG_HANDLER(WM_LBUTTONUP, this->OnMouseLButtonUp)
    MSG_HANDLER(WM_LBUTTONDOWN, this->OnMouseLButtonDown)
    MSG_HANDLER(WM_RBUTTONUP, this->OnMouseRButtonUp)
    MSG_HANDLER(WM_RBUTTONDOWN, this->OnMouseRButtonDown)
    MSG_HANDLER(WM_MOUSEACTIVATE, this->OnMouseActivate)
    MSG_HANDLER(WM_COMMAND, this->OnCommand)
    MSG_HANDLER(WM_KILLFOCUS, this->OnKillFocus)
    MSG_HANDLER(WM_SETFOCUS, this->OnSetFocus)
    MSG_HANDLER(WM_MOVE, this->OnMove)
    MSG_HANDLER(WM_LBUTTONDBLCLK, this->OnMouseLButtonDoubleClick)
    MSG_HANDLER(WM_CLOSE, this->OnClose)
    MSG_HANDLER(WM_CREATE, this->OnCreate)
    MSG_HANDLER(WM_DESTROY, this->OnDestroy)
    MSG_HANDLER(WM_KEYDOWN, this->OnKeyDown)
    MSG_HANDLER(WM_KEYUP, this->OnKeyUp)
    MSG_HANDLER(WM_GETDLGCODE, this->OnGetDlgCode)
    MSG_HANDLER(WM_CAPTURECHANGED, this->OnMouseCaptureChanged)
    MSG_HANDLER(WM_ACTIVATE, this->OnActivate)
    MSG_HANDLER(WM_VSCROLL, this->OnVScroll)
    MSG_HANDLER(WM_HSCROLL, this->OnHScroll)
    MSG_HANDLER(WM_MOUSEWHEEL, this->OnMouseWheel)
    MSG_HANDLER(WM_MOUSEHWHEEL, this->OnMouseHWheel)
    MSG_HANDLER(WM_XBUTTONDOWN, this->OnMouseMediaButtonDown)
    case WM_NULL:return 0;
    default:
        // return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
        return this->OnMessage(hWnd,uMsg,wParam,lParam);
    }
}

// No need to manage message, the DefWindowProc do the job for us and stop sending WM_PAINT messages
BOOL CWindowBase::EnableRedraw(BOOL bEnable)
{
    LRESULT RetValue;
    if (bEnable)
    {
        RetValue = ::SendMessage(this->hWnd, WM_SETREDRAW, TRUE, 0);
        // MSDN :
        // Note  RedrawWindow with the specified flags is used instead of InvalidateRect because the former is necessary for some controls 
        // that have nonclient area on their own, or have window styles that cause them to be given a nonclient area 
        // (such as WS_THICKFRAME, WS_BORDER, or WS_EX_CLIENTEDGE). If the control does not have a nonclient area, 
        // RedrawWindow with these flags will do only as much invalidation as InvalidateRect would.
        if (this->hWnd)
            ::RedrawWindow(this->hWnd, NULL, NULL, RDW_ERASE | RDW_FRAME | RDW_INVALIDATE | RDW_ALLCHILDREN);
    }
    else
    {
        // MSDN : 
        // If the application sends the WM_SETREDRAW message to a hidden window, the window becomes visible 
        // (that is, the operating system adds the WS_VISIBLE style to the window)
        RetValue = ::SendMessage(this->hWnd, WM_SETREDRAW, FALSE, 0);
    }
    // MSDN : An application returns zero if it processes this message
    return (RetValue == 0);
}

LRESULT CWindowBase::OnPaint(WPARAM wParam,LPARAM lParam)
{
    UNREFERENCED_PARAMETER(wParam);
    UNREFERENCED_PARAMETER(lParam);
    PAINTSTRUCT PaintStruct;
    HDC hdc;
    LRESULT RetValue;

    ::BeginPaint(this->hWnd,&PaintStruct);
    hdc = PaintStruct.hdc;
    if (this->bEnabledoublebuffering)
    {
        RECT Rect;
        ::GetClientRect(this->hWnd,&Rect);
        this->MemoryDeviceContext.Update(PaintStruct.hdc,&Rect);
        hdc = this->MemoryDeviceContext.GetDc();
        if (this->bUseParentWindowContentAsBackground)
        {
            // not possible due to WS_CLIPCHILDREN | WS_CLIPSIBLINGS
            this->MemoryDeviceContext.FillFromDc(PaintStruct.hdc);
        }
    }

    // do paint operation into 
    //      1) the memory device context if double buffering is enabled
    //   or 2) directly in window device context if double buffering is disabled
    RetValue = this->DoPaint(hdc);

    if (this->bEnabledoublebuffering)
    {
        // copy data from memory device context to window hdc
        this->MemoryDeviceContext.DoCopy(PaintStruct.hdc);
        this->MemoryDeviceContext.ReleaseGdiObjects();
    }
    ::EndPaint(this->hWnd,&PaintStruct);
    return RetValue;
}
LRESULT CWindowBase::OnPrintClient(WPARAM wParam,LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);
    LRESULT RetValue;
    HDC hdc;
    hdc = (HDC)wParam;
    RetValue = this->DoPaint(hdc);
    return RetValue;
}

LRESULT CWindowBase::OnMouseMoveInternal(WPARAM wParam,LPARAM lParam)
{
    // MSDN : The application must call TrackMouseEvent when the mouse reenters its window if it requires further tracking of mouse hover behavior.
    if (!this->bTrackMouseEventSet)
    {
        // we have to watch hWndTransparentToolbar Mouse leave event
        TRACKMOUSEEVENT TrackEvent={0};
        TrackEvent.cbSize=sizeof(TRACKMOUSEEVENT);
        TrackEvent.dwFlags=TME_LEAVE;
        TrackEvent.hwndTrack=this->hWnd;
        ::TrackMouseEvent(&TrackEvent);
        this->bTrackMouseEventSet=TRUE;
    }

    return this->OnMouseMove(wParam,lParam);
}
LRESULT CWindowBase::OnMouseLeaveInternal(WPARAM wParam,LPARAM lParam)
{
    // MSDN : The application must call TrackMouseEvent when the mouse reenters its window if it requires further tracking of mouse hover behavior.
    this->bTrackMouseEventSet = FALSE;

    return this->OnMouseLeave(wParam,lParam);
}

DWORD CWindowBase::GetHeight()
{
    RECT Rect;
    if (this->GetWindowRect(&Rect))
    {
        return Rect.bottom-Rect.top;
    }
    //else
    return 0;
}
DWORD CWindowBase::GetWidth()
{
    RECT Rect;
    if (this->GetWindowRect(&Rect))
    {
        return Rect.right-Rect.left;
    }
    //else
    return 0;
}

BOOL CWindowBase::IsMouseOverCurrentWindow()
{
    POINT pt;
    RECT Rect;
    ::GetCursorPos(&pt);
    ::GetWindowRect(this->hWnd,&Rect);
    return ::PtInRect(&Rect,pt);
}
