/*
Copyright (C) 2004-2016 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2016 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: base class Window controls
//-----------------------------------------------------------------------------



#pragma once

#include <windows.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)

#include "../MemoryDeviceContext/MemoryDeviceContext.h"

#ifndef GET_X_LPARAM
#define GET_X_LPARAM(lp)                        ((int)(short)LOWORD(lp))
#endif
#ifndef GET_Y_LPARAM
#define GET_Y_LPARAM(lp)                        ((int)(short)HIWORD(lp))
#endif

#define MSG_HANDLER(Msg,Func) case Msg: return Func(wParam,lParam);

#ifndef DeleteAndCleanIfNotNull
    // usage : DeleteAndCleanIfNotNull(this->pObject)
    #define DeleteAndCleanIfNotNull(x) if (x){delete x; x=NULL;}
#endif

#ifndef FreeAndCleanIfNotNull
// usage : DeleteAndCleanIfNotNull(this->pObject)
#define FreeAndCleanIfNotNull(x) if (x){free(x); x=NULL;}
#endif

#ifndef ReleaseAndCleanIfNotNull
    // usage : ReleaseAndCleanIfNotNull(this->pObject)
    #define ReleaseAndCleanIfNotNull(x) if (x){x->Release(); x=NULL;}
#endif

#ifndef DeleteAndCleanArrayIfNotNull
    // usage : DeleteAndCleanArrayIfNotNull(Array)
    #define DeleteAndCleanArrayIfNotNull(Array) if (Array){delete[] Array; Array=NULL;}
#endif

#ifndef CloseHandleAndCleanIfNotNull
    // usage : CloseHandleAndCleanIfNotNull(hHandle)
    #define CloseHandleAndCleanIfNotNull(x) if (x){::CloseHandle(x); x=NULL;}
#endif

#ifndef DestroyIconAndCleanIfNotNull
    // usage : DestroyIconAndCleanIfNotNull(hIcon)
    #define DestroyIconAndCleanIfNotNull(x) if (x){::DestroyIcon(x); x=NULL;}
#endif

#ifndef DestroyWindowAndCleanIfNotNull
    // usage : DestroyIconAndCleanIfNotNull(hWnd)
    #define DestroyWindowAndCleanIfNotNull(x) if (x){::DestroyWindow(x); x=NULL;}
#endif

#ifndef DeleteFontAndCleanIfNotNull
    // usage : DeleteFontAndCleanIfNotNull(hFont)
    #define DeleteFontAndCleanIfNotNull(x) if (x){DeleteObject((HGDIOBJ)(HFONT)(x)); x=NULL;}
#endif

#ifndef _DefineStringLen
    // _DefineStringLen : remove string \0 from array size
    #define _DefineStringLen(Array) (_countof(Array)-1)
#endif

#ifndef _DefineStringLenWithEndingZero
    #define _DefineStringLenWithEndingZero(Array) (_countof(Array))
#endif

#ifndef BreakInDebugMode
    #ifdef _DEBUG
        #define BreakInDebugMode if ( ::IsDebuggerPresent() ) { ::DebugBreak();}
    #else 
        #define BreakInDebugMode 
    #endif
#endif

class CWindowBase
{
private:
    BOOL bTrackMouseEventSet;

    virtual LRESULT CALLBACK WindowProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);

    virtual LRESULT OnMouseMoveInternal(WPARAM wParam,LPARAM lParam);
    virtual LRESULT OnMouseLeaveInternal(WPARAM wParam,LPARAM lParam);
    virtual LRESULT OnPaint(WPARAM wParam,LPARAM lParam); // only DoPaint should be sub classed
    virtual LRESULT OnPrintClient(WPARAM wParam,LPARAM lParam);// only DoPaint should be sub classed

protected:
    HWND hWnd;
    CMemoryDeviceContext MemoryDeviceContext;
    BOOL bEnabledoublebuffering;// double buffering must be unactivated for layer windows (transparency)
    BOOL bUseParentWindowContentAsBackground;

    // protected only to allow to be used by control creation (wc.lpfnWndProc= used for ::RegisterClassEx(&wc))
    static LRESULT CALLBACK WindowProcStatic(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);

    // OnCreate should return zero to continue creation of the window. If OnCreate returns 1, the window is destroyed and the CreateWindowEx or CreateWindow function returns a NULL handle
    virtual LRESULT OnCreate(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return 0;}
    // OnClose : just use return 0; to avoid application closing (DefWindowProc calls DestroyWindow)
    virtual LRESULT OnClose(WPARAM wParam,LPARAM lParam){return ::DefWindowProc(this->hWnd, WM_CLOSE, wParam, lParam);}
    // OnDestroy : If an application processes this message, it should return zero
    virtual LRESULT OnDestroy(WPARAM wParam,LPARAM lParam){return ::DefWindowProc(this->hWnd, WM_DESTROY, wParam, lParam);}
    virtual LRESULT OnCommand(WPARAM wParam,LPARAM lParam)
    {
        UNREFERENCED_PARAMETER(wParam);
        UNREFERENCED_PARAMETER(lParam);
        //switch(LOWORD(wParam))
        //{
        //case IDOK:
        //    this->Close(IDOK);
        //    break;
        //case IDCANCEL:
        //    this->Close(IDCANCEL);
        //    break;
        //}
        return 0;
    }
    virtual LRESULT OnNotify(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return 0;}

    virtual LRESULT OnMouseLButtonDown(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return 0;}
    virtual LRESULT OnMouseLButtonUp(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return 0;}
    virtual LRESULT OnMouseRButtonDown(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return 0;}
    virtual LRESULT OnMouseRButtonUp(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return 0;}
    virtual LRESULT OnMouseMove(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return 0;}
    virtual LRESULT OnHScroll(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return 0;}
    virtual LRESULT OnVScroll(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return 0;}
    virtual LRESULT OnMouseWheel(WPARAM wParam,LPARAM lParam);
    virtual LRESULT OnMouseHWheel(WPARAM wParam,LPARAM lParam);
    virtual LRESULT OnMouseMediaButtonDown(WPARAM wParam,LPARAM lParam)
    {
        UNREFERENCED_PARAMETER(wParam);
        UNREFERENCED_PARAMETER(lParam);

        // WORD fwKeys = GET_KEYSTATE_WPARAM (wParam); 
        // WORD fwButton = GET_XBUTTON_WPARAM (wParam);
        // int xPos = GET_X_LPARAM(lParam); 
        // int yPos = GET_Y_LPARAM(lParam);
        //switch (fwKeys)
        //{
        //case MK_XBUTTON1: // backward
        //    return TRUE;

        //case MK_XBUTTON2: // forward
        //    return TRUE;
        //}

        // an application should return TRUE from this message if it processes it
        return FALSE;
    }

    //OnMouseCaptureChanged: An application should return zero if it processes this message
    //A window in some specific cases can receives this message even if it calls StartCapture itself: this can be checked with lParam (A handle to the window gaining the mouse capture)
    // (if it's the case it can result of bug like multiple call of IsDialogMessage) so take care of lParam == this->hWnd for the OnMouseCaptureChanged !!!
    //A window receives this message even if it calls ReleaseCapture itself. 
    //An application should not attempt to set the mouse capture in response to this message
    virtual LRESULT OnMouseCaptureChanged(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return 0;}

    // OnKeyDown: An application should return zero if it processes this message
    virtual LRESULT OnKeyDown(WPARAM wParam,LPARAM lParam)
    {
        UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);
        return 1;
    }
    // OnKeyUp: An application should return zero if it processes this message
    virtual LRESULT OnKeyUp(WPARAM wParam,LPARAM lParam)
    {
        UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);
        return 1;
    }
    // If an application processes this message, it should return zero
    virtual LRESULT OnMove(WPARAM wParam,LPARAM lParam){return ::DefWindowProc(this->hWnd, WM_MOVE, wParam, lParam);}
    // add CS_DBLCLKS to WNDCLASSEX.style for RegisterClassEx call to allow double clicking
    virtual LRESULT OnMouseLButtonDoubleClick(WPARAM wParam, LPARAM lParam){return ::DefWindowProc(this->hWnd, WM_LBUTTONDBLCLK, wParam, lParam);}
    virtual LRESULT OnMouseLeave(WPARAM wParam,LPARAM lParam){return ::DefWindowProc(this->hWnd, WM_MOUSELEAVE, wParam, lParam);}
    // WM_MOUSEACTIVATE : The parent window receives this message only if the child window passes it to the DefWindowProc function
    virtual LRESULT OnMouseActivate(WPARAM wParam,LPARAM lParam){return ::DefWindowProc(this->hWnd, WM_MOUSEACTIVATE, wParam, lParam);}
    virtual LRESULT OnSize(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return 0;}
    // OnSizing : An application should return 1 if it processes this message
    virtual LRESULT OnSizing(WPARAM wParam,LPARAM lParam){UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);return 0;}
    // OnEraseBackground : return 1 : don't erase background (application should return nonzero if it erases the background; otherwise, it should return zero)
    virtual LRESULT OnEraseBackground(WPARAM wParam,LPARAM lParam){return ::DefWindowProc(this->hWnd, WM_ERASEBKGND, wParam, lParam);}
    // OnKillFocus: An application should return zero if it processes this message
    virtual LRESULT OnKillFocus(WPARAM wParam,LPARAM lParam){return ::DefWindowProc(this->hWnd, WM_KILLFOCUS, wParam, lParam);}
    // OnSetFocus: An application should return zero if it processes this message
    virtual LRESULT OnSetFocus(WPARAM wParam,LPARAM lParam){return ::DefWindowProc(this->hWnd, WM_SETFOCUS, wParam, lParam);}
    // manage other messages
    virtual LRESULT OnMessage(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam){return ::DefWindowProc(hWnd, uMsg, wParam, lParam);}
    // Notice Only DoPaint should be used by derived classes
    virtual LRESULT DoPaint(HDC hdc);

    virtual LRESULT OnActivate(WPARAM wParam,LPARAM lParam){return ::DefWindowProc(this->hWnd, WM_ACTIVATE, wParam, lParam);}

    virtual LRESULT OnNCHitTest(WPARAM wParam,LPARAM lParam){return ::DefWindowProc(this->hWnd,WM_NCHITTEST, wParam, lParam);}
    

    virtual LRESULT OnGetDlgCode(WPARAM wParam,LPARAM lParam)
    {
        UNREFERENCED_PARAMETER(wParam);UNREFERENCED_PARAMETER(lParam);
        return DLGC_WANTALLKEYS;
    }
public:
    CWindowBase();
    virtual ~CWindowBase();

    BOOL IsMouseOverCurrentWindow();
    DWORD GetHeight();
    DWORD GetWidth();
    FORCEINLINE BOOL GetWindowRect(OUT RECT* pRect){return ::GetWindowRect(this->hWnd,pRect);}
    BOOL GetClientRect(OUT RECT* pRect)
    {
        return ::GetClientRect(this->hWnd,pRect);
        // if (!::GetWindowRect(this->hWnd,pRect))
        //    return FALSE;
        // ::OffsetRect(pRect,-pRect->left,-pRect->top);
        // return TRUE;
    }
    BOOL ScreenToClientEx(IN HWND hWndRef,IN OUT LPRECT pRect);
    FORCEINLINE BOOL ScreenToClient(IN OUT LPRECT pRect){return this->ScreenToClientEx(this->hWnd,pRect);}
    FORCEINLINE HWND GetControlHandle(){return this->hWnd;}
    FORCEINLINE HWND GetDlgItem(IN int nIDDlgItem){return ::GetDlgItem(this->hWnd,nIDDlgItem);}
    FORCEINLINE int GetControlId(IN HWND hWindow){return ::GetDlgCtrlID(hWindow);}
    
    virtual BOOL Show(BOOL bShow){return ::ShowWindow(this->hWnd,(bShow)?SW_SHOWNORMAL:SW_HIDE);}
    FORCEINLINE BOOL Show(){return this->Show(TRUE);}
    FORCEINLINE BOOL Hide(){return this->Show(FALSE);}
    FORCEINLINE BOOL HideItem(IN int nIDDlgItem){return ::ShowWindow(this->GetDlgItem(nIDDlgItem),SW_HIDE);}
    FORCEINLINE BOOL ShowItem(IN int nIDDlgItem){return ::ShowWindow(this->GetDlgItem(nIDDlgItem),SW_SHOWNORMAL);}
    FORCEINLINE BOOL ShowItem(IN int nIDDlgItem,IN int CmdShow){return ::ShowWindow(this->GetDlgItem(nIDDlgItem),CmdShow);}

    FORCEINLINE BOOL IsVisible(){return ::IsWindowVisible(this->hWnd);}
    FORCEINLINE BOOL HasVisibleStyle(){return ( (::GetWindowLongPtr(this->hWnd,GWL_STYLE) & WS_VISIBLE) != 0);}

    FORCEINLINE BOOL SetPosition(LONG x,LONG y){return ::SetWindowPos(this->hWnd,0,x,y,0,0,SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE);}
    FORCEINLINE BOOL SetPositionFromHwnd(HWND hWnd,LONG x,LONG y){return ::SetWindowPos(hWnd,0,x,y,0,0,SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE);}
    FORCEINLINE BOOL SetPositionFromId(int Id,LONG x,LONG y){return ::SetWindowPos(this->GetDlgItem(Id),0,x,y,0,0,SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE);}

    FORCEINLINE BOOL SetRect(RECT* pRect){return ::SetWindowPos(this->hWnd,NULL,pRect->left,pRect->top,pRect->right-pRect->left,pRect->bottom-pRect->top,SWP_NOACTIVATE | SWP_NOZORDER);}
    FORCEINLINE BOOL SetRectFromHwnd(HWND hWnd,RECT* pRect){return ::SetWindowPos(hWnd,NULL,pRect->left,pRect->top,pRect->right-pRect->left,pRect->bottom-pRect->top,SWP_NOACTIVATE | SWP_NOZORDER);}
    FORCEINLINE BOOL SetRectFromId(int Id,RECT* pRect){return ::SetWindowPos(this->GetDlgItem(Id),NULL,pRect->left,pRect->top,pRect->right-pRect->left,pRect->bottom-pRect->top,SWP_NOACTIVATE | SWP_NOZORDER);}
    FORCEINLINE BOOL SetSize(LONG Width,LONG Height){return ::SetWindowPos(this->hWnd,NULL,0,0,Width,Height,SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);}
    FORCEINLINE BOOL SetSizeFromHwnd(HWND hWnd,LONG Width,LONG Height){return ::SetWindowPos(hWnd,NULL,0,0,Width,Height,SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);}
    FORCEINLINE BOOL SetSizeFromId(int Id,LONG Width,LONG Height){return ::SetWindowPos(this->GetDlgItem(Id),NULL,0,0,Width,Height,SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);}


    BOOL EnableRedraw(BOOL bEnable);
    FORCEINLINE BOOL Redraw()
    {
        if (this->hWnd)
            return ::RedrawWindow( this->hWnd,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME | RDW_ALLCHILDREN); 
        return FALSE;
    }
    FORCEINLINE BOOL Invalidate(BOOL bEraseBackground=FALSE,RECT* pRect=NULL)
    {
        if (this->hWnd)
            return ::InvalidateRect(this->hWnd,pRect,bEraseBackground);
        return FALSE;
    }

    FORCEINLINE HICON LoadIcon(HINSTANCE hInstance,int IconId,int Width,int Height){return (HICON)::LoadImage(hInstance,MAKEINTRESOURCE(IconId),IMAGE_ICON,Width,Height,LR_DEFAULTCOLOR|LR_SHARED);}
};
