/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "IniFileEx.h"

// #pragma message (__FILE__ " Information : don't forget #include \"IniFileEx.cpp\"\r\n")
#ifndef _countof
#define _countof(array) (sizeof(array)/sizeof(array[0]))
#endif

template<class T> CIniFileEx<T>::CIniFileEx(TCHAR* FileName):CIniFile(FileName)
{

}
template<class T> CIniFileEx<T>::~CIniFileEx()
{

}

template<class T> BOOL CIniFileEx<T>::Set(TCHAR* SectionName, std::vector<T*>* pArrayItems)
{
    BOOL bRet=TRUE;
    BOOL bCurrentRet;
    TCHAR OldSection[MAX_PATH];
    TCHAR SubSectionName[2*MAX_PATH];
    int ItemCount=(int)pArrayItems->size();
    if (!this->CIniFile::Set(SectionName,(DWORD)ItemCount))
        return FALSE;

    // store current section name
    _tcscpy(OldSection,this->CurrentSection);

    for (int Cnt=0;Cnt<ItemCount;Cnt++)
    {
        _sntprintf(SubSectionName,_countof(SubSectionName),_T("%s_%u"),SectionName,(UINT)Cnt);

        // set current section name
        this->SetCurrentSectionName(SubSectionName);

        // bRet= bRet && ((IIniFileArrayItem*)(*pArrayItems)[Cnt])->Save((CIniFile*)this); bug and optimization function not called as soon as bRet is FALSE
        bCurrentRet = ((IIniFileArrayItem*)(*pArrayItems)[Cnt])->Save((CIniFile*)this);
        bRet= bRet && bCurrentRet;
    }

    // restore previous section
    _tcscpy(this->CurrentSection,OldSection);

    return bRet;
}

template<class T> BOOL CIniFileEx<T>::Get(TCHAR* SectionName,OUT std::vector<T*>* pArrayItems)
{
    TCHAR OldSection[MAX_PATH];
    TCHAR SubSectionName[2*MAX_PATH];
    BOOL bRet=TRUE;
    DWORD ItemCount=0;
    T* pObject;
    IIniFileArrayItem* pIniFileArrayItem;

    // assume pArrayItems is empty
    T* pT;
    std::vector<T*>::iterator Iterator;
    for (Iterator = pArrayItems->begin();Iterator!=pArrayItems->end();++Iterator)
    {
        pT = *Iterator;
        delete pT;
    }
    pArrayItems->clear();
   
    // store current section name
    _tcscpy(OldSection,this->CurrentSection);
    
    // get number of items
    this->CIniFile::Get(SectionName,0,&ItemCount);

    // for each item
    for (DWORD Cnt=0;Cnt<ItemCount;Cnt++)
    {
        // forge section name
        _sntprintf(SubSectionName,_countof(SubSectionName),_T("%s_%u"),SectionName,Cnt);

        // set current section name
        this->SetCurrentSectionName(SubSectionName);

        // create new object
        pObject=new T();

        // get IIniFileArrayItem implementation interface for object
        pIniFileArrayItem=(IIniFileArrayItem*)pObject;

        // call object Load method
        if (pIniFileArrayItem->Load((CIniFile*)this))
        {
            // on success add item to vector
            pArrayItems->push_back(pObject);
        }
        else
        {
            // on error delete object
            bRet=FALSE;
            delete pObject;
        }
    }

    // restore previous section
    _tcscpy(this->CurrentSection,OldSection);

    return bRet;
}

template<class T> BOOL CIniFileEx<T>::Set(TCHAR* SectionName, T* pItem)
{
    BOOL bRet;
    TCHAR OldSection[MAX_PATH];

    // store current section name
    _tcscpy(OldSection,this->CurrentSection);

    // set current section name
    this->SetCurrentSectionName(SectionName);

    bRet = ((IIniFileArrayItem*)pItem)->Save((CIniFile*)this);

    // restore previous section
    _tcscpy(this->CurrentSection,OldSection);

    return bRet;
}
template<class T> T* CIniFileEx<T>::Get(TCHAR* SectionName)
{
    TCHAR OldSection[MAX_PATH];

    // store current section name
    _tcscpy(OldSection,this->CurrentSection);

    // set current section name
    this->SetCurrentSectionName(SectionName);

    T* pItem=new T();
    if (!((IIniFileArrayItem*)pItem)->Load((CIniFile*)this))
    {
        delete pItem;
        pItem=NULL;
    }

    // restore previous section
    _tcscpy(this->CurrentSection,OldSection);

    return pItem;
}