/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#pragma once

#include <windows.h>
#include <vector>
#include "IniFile.h"

class IIniFileArrayItem
{
public:
    virtual BOOL Save(CIniFile* pIniFile)=0;
    virtual BOOL Load(CIniFile* pIniFile)=0; // warning by returning FALSE, 
                                               // object will be considered as badly initialized ,
                                               // and will be deleted (not inserted in output vector)
};
/*
// implementation example

#include "IniFileEx.h"
#include "IniFileEx.cpp"

class COptions:public IIniFileArrayItem
{
public:
    COptions(); // constructor called by CIniFileEx
    ~COptions();

    int Property1;
    TCHAR Property2[MAX_PATH];

    BOOL Save(CIniFile* pIniFile);
    BOOL Load(CIniFile* pIniFile);
};

COptions::COptions()
{

}

COptions::~COptions()
{

}

BOOL COptions::Save(CIniFile* pIniFile)
{
    BOOL bRet=TRUE;
    BOOL bCurrentRet;
    bCurrentRet = pIniFile->Set(_T("Property1"),this->Property1);
    bRet=bRet && bCurrentRet;
    bCurrentRet = pIniFile->Set(_T("Property2"),(TCHAR*)this->Property2);
    bRet=bRet && bCurrentRet;
    return bRet;
}

BOOL COptions::Load(CIniFile* pIniFile)
{
    BOOL bRet=TRUE;
    BOOL bCurrentRet;
    bCurrentRet = pIniFile->Get(_T("Property1"),0,&this->Property1);
    bRet=bRet && bCurrentRet;
    bCurrentRet = pIniFile->Get(_T("Property2"),_T(""),(TCHAR*)this->Property2,MAX_PATH);
    bRet=bRet && bCurrentRet;
    return bRet;
}

int main()
{
    // create a CIniFileEx object for COptions class
    // .ini file path is "c:\MySettingFile.ini"
    CIniFileEx<COptions> IniFileEx(_T("c:\\MySettingFile.ini"));

    ///////////////////////////////////////////////////////
    // saving
    ///////////////////////////////////////////////////////

    // create an option object
    COptions ObjOptions;

    // set object values
    ObjOptions.Property1=123456;
    _tcscpy(ObjOptions.Property2,_T("Property2_String"));

    // save options
    IniFileEx.Set(_T("Options"),&ObjOptions);

    ///////////////////////////////////////////////////////
    // loading
    ///////////////////////////////////////////////////////

    // load options
    COptions* pObjOptions=IniFileEx.Get(_T("Options"));
    if (!pObjOptions)
    return -1;

    // use options

    delete pObjOptions;
}

*/

template <class T> class CIniFileEx:public CIniFile // T must be derived from IIniFileArrayItem
{
public:
    CIniFileEx(TCHAR* FileName);
    ~CIniFileEx();
    BOOL Set(TCHAR* SectionName, T* pItem);
    T* Get(TCHAR* SectionName);

    BOOL Set(TCHAR* SectionName, std::vector<T*>* pArrayItems);
    BOOL Get(TCHAR* SectionName,OUT std::vector<T*>* pArrayItems);
};