/*
Copyright (C) 2004-2023 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ShellExecuteAndBringToTop.h"

BOOL CShellExecuteAndBringToTop::SetWindowToTopFromhProcess(HWND hwnd, LPARAM lParam)
{
    tagCallBackParam* pParam= (tagCallBackParam*)lParam;
    DWORD ProcessId = 0;
    
    ::GetWindowThreadProcessId(hwnd, &ProcessId);
    if (ProcessId == pParam->ProcessId)
    {
        if (::IsIconic(hwnd))
        {
            ::ShowWindow(hwnd,SW_RESTORE);
        }
        else if (::IsWindowVisible(hwnd) == FALSE)
        {
            // do not show window that are hidden
            // this can be the normal way of doing of some application 
            // to get there main window hidden, and in this case, 
            // it is unlikely to force to show this hidden window
            return TRUE; // continue enumerating windows
        }

        // way to set top
        ::SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE|SWP_SHOWWINDOW);
        ::SetWindowPos(hwnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE|SWP_SHOWWINDOW);
        
        ::BringWindowToTop(hwnd);
        ::SetForegroundWindow(hwnd);
        pParam->bSuccess = TRUE;
        return FALSE; // stop enumerating windows
    }
    return TRUE; // continue enumerating windows
}
    
BOOL CShellExecuteAndBringToTop::ShellExecuteAndBringToTop(SHELLEXECUTEINFO* pExecuteInfo)
{
    BOOL RetValue;
    BOOL bClosehProcess;
    if (pExecuteInfo->fMask & SEE_MASK_NOCLOSEPROCESS)
    {
        bClosehProcess = FALSE;
    }
    else
    {
        pExecuteInfo->fMask |=SEE_MASK_NOCLOSEPROCESS;
        bClosehProcess = TRUE;
    }

    pExecuteInfo->fMask |=SEE_MASK_NOASYNC  // SEE_MASK_FLAG_DDEWAIT == SEE_MASK_NOASYNC
                           | SEE_MASK_DOENVSUBST // do environment substitution if required
                           | SEE_MASK_CONNECTNETDRV; // reconnect net drive if required
    pExecuteInfo->nShow = SW_SHOWNORMAL;
    RetValue = ::ShellExecuteEx(pExecuteInfo);
    if (RetValue 
        && pExecuteInfo->hProcess // m$ is happy to return NULL hProcess info in multiple conditions (see MSDN : SHELLEXECUTEINFO)
        )
    {
        tagCallBackParam Param={0};
        Param.ProcessId = ::GetProcessId(pExecuteInfo->hProcess);
        if (bClosehProcess)
        {
            ::CloseHandle(pExecuteInfo->hProcess);
            pExecuteInfo->hProcess = 0;
        }
        if (Param.ProcessId)
        {
            DWORD ForeGroundWindowProcessId;
            HWND hWndForeGround;

            for (DWORD Cnt=0;(Cnt<4) && (!Param.bSuccess);Cnt++) // do retry during 2 seconds until application is bring to top (500ms is sometimes not enough during high HDD usage)
            {
                // Sleep to wait app window to be created and appear before another try
                ::Sleep(500);

                // check if foreground window belongs to created process ID
                hWndForeGround = ::GetForegroundWindow();
                ForeGroundWindowProcessId = 0;
                ::GetWindowThreadProcessId(hWndForeGround, &ForeGroundWindowProcessId);
                if (ForeGroundWindowProcessId == Param.ProcessId)
                    break;

                // else
                ::EnumWindows(CShellExecuteAndBringToTop::SetWindowToTopFromhProcess, (LPARAM) &Param);
            }
        }
    }
    return RetValue;
}

HINSTANCE CShellExecuteAndBringToTop::ShellExecute(HWND hwnd,LPCTSTR lpOperation,LPCTSTR lpFile,LPCTSTR lpParameters,LPCTSTR lpDirectory,INT nShowCmd)
{
    #ifndef SEE_MASK_DEFAULT
        #define SEE_MASK_DEFAULT           0x00000000
    #endif
    SHELLEXECUTEINFO ExecuteInfo={0};
    ExecuteInfo.cbSize = sizeof(ExecuteInfo);
    ExecuteInfo.hwnd = hwnd;
    ExecuteInfo.lpVerb = lpOperation;
    ExecuteInfo.lpFile = lpFile;
    ExecuteInfo.lpParameters = lpParameters;
    ExecuteInfo.lpDirectory = lpDirectory;
    ExecuteInfo.nShow = nShowCmd;
    ExecuteInfo.fMask = SEE_MASK_DEFAULT;
    CShellExecuteAndBringToTop::ShellExecuteAndBringToTop(&ExecuteInfo);
    return ExecuteInfo.hInstApp;
}

