/*
Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2015 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ShellExecuteAndBringToTopEx.h"
#include "../File/StdFileOperations.h"

#include <shlobj.h>
#include <shellapi.h>
#pragma comment(lib,"shell32.lib")

#include <comdef.h>
#ifdef _NATIVE_WCHAR_T_DEFINED
    #ifdef _DEBUG
        #pragma comment(lib, "comsuppwd.lib")
    #else
        #pragma comment(lib, "comsuppw.lib")
    #endif
#else
    #ifdef _DEBUG
        #pragma comment(lib, "comsuppd.lib")
    #else
        #pragma comment(lib, "comsupp.lib")
    #endif
#endif
void CShellExecuteAndBringToTopEx::ShowComError(HRESULT hr,HWND hwnd)
{
    ::_com_error err(hr);
    LPCTSTR errMsg = err.ErrorMessage();
    ::MessageBox(hwnd,errMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
}

BOOL CShellExecuteAndBringToTopEx::ShellExecute(TCHAR* Path,HWND hwnd)
{
    // if file is a link
    if ( CStdFileOperations::DoesExtensionMatch(Path,_T("lnk")) )
    {
        return CShellExecuteAndBringToTopEx::ShellExecuteLink(Path,hwnd);
    }

    // get item directory to launch it in it's directory
    TCHAR Directory[MAX_PATH];
    CStdFileOperations::GetFileDirectory(Path,Directory,MAX_PATH);

    // open selected item
    //if (((int)::ShellExecute(NULL,NULL, // do not use _T("open") : it can fail on some cases
    //                         pFolderPopUpMenuItemInfos->GetPath(),NULL,Directory,SW_SHOWNORMAL))<33)
    if (((int)CShellExecuteAndBringToTop::ShellExecute(NULL,NULL, // do not use _T("open") : it can fail on some cases
                                                       Path,NULL,Directory,SW_SHOWNORMAL))<33)
    {
        TCHAR sz[2*MAX_PATH];
        _tcscpy(sz,_T("Error opening "));
        _tcscat(sz,Path);
        ::MessageBox(hwnd,sz,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    return TRUE;
}


//-----------------------------------------------------------------------------
// Name: ShellExecuteLink
// Object: launch a link
// Parameters :
//     in  : TCHAR* Path : link path
//     out :
//     return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CShellExecuteAndBringToTopEx::ShellExecuteLink(TCHAR* Path,HWND hwnd)
{
    BOOL bRet = FALSE;
    HRESULT hResult;
    IShellLink *pShellLink = NULL;
    IPersistFile *pPersistFile = NULL;
    WCHAR* pwLinkName;
    PIDLIST_ABSOLUTE pidl = NULL;
    SHELLEXECUTEINFO ShInfo={0};

    // WIN32_FIND_DATA ShellLinkWin32FindData;

    // create IShellLink object
    hResult = ::CoCreateInstance(  CLSID_ShellLink,
                                    NULL,
                                    CLSCTX_INPROC_SERVER,
                                    IID_IShellLink,
                                    (LPVOID*)&pShellLink);
    if ( (hResult!=S_OK) || (pShellLink==NULL) )
        goto CleanUp;

    // get IPersistFile interface
    hResult = pShellLink->QueryInterface(IID_IPersistFile,(LPVOID*)&pPersistFile);
    if ( (hResult!=S_OK) || (pPersistFile==NULL) )
        goto CleanUp;

#if (defined(UNICODE)||defined(_UNICODE))
    pwLinkName=Path;
#else
    CAnsiUnicodeConvert::AnsiToUnicode(Path,&pwLinkName);
#endif

    // load shortcut infos
    hResult = pPersistFile->Load(pwLinkName,STGM_READ);
#if ( (!defined(UNICODE)) && (!defined(_UNICODE)))
    free(pwLinkName);
#endif
    if ( hResult!=S_OK )
    {
        CShellExecuteAndBringToTopEx::ShowComError(hResult,hwnd);
        goto CleanUp;
    }

    // get shortcut target path
    //TCHAR TargetPath[MAX_PATH];
    //*TargetPath=0;
    TCHAR lpParameters[MAX_PATH];
    *lpParameters=0;
    TCHAR lpDirectory[MAX_PATH];
    *lpDirectory=0;
    int nShow = SW_NORMAL;

    // don't work for new links...
    //hResult = pShellLink->GetPath(  TargetPath,
    //                                MAX_PATH,
    //                                &ShellLinkWin32FindData,
    //                                SLGP_UNCPRIORITY 
    //                              );
    // going to the new m$ way
    DWORD Flags;
    Flags = SLR_ANY_MATCH
            | SLR_INVOKE_MSI 
            | SLR_NO_UI 
            | SLR_UPDATE;
    hResult = pShellLink->Resolve(NULL,Flags);
// try to continue even in case of Resolve failure
//    if ( hResult!=S_OK )
//    {
//#ifdef _DEBUG // avoid noisy messages
//        CShellExecuteAndBringToTopEx::ShowComError(hResult);
//#endif
//        goto CleanUp;
//    }
    hResult = pShellLink->GetIDList(&pidl);
    if ( ( hResult!=S_OK ) || (pidl==NULL) )
    {
        CShellExecuteAndBringToTopEx::ShowComError(hResult,hwnd);
        goto CleanUp;
    }

    //if (!::SHGetPathFromIDList(pidl,TargetPath))//  The address of a buffer to receive the file system path. This buffer must be at least MAX_PATH characters in size.
    //    goto CleanUp;

    pShellLink->GetArguments(lpParameters,MAX_PATH);
    pShellLink->GetWorkingDirectory(lpDirectory,MAX_PATH);
    pShellLink->GetShowCmd(&nShow);

    ShInfo.cbSize = sizeof(ShInfo);
    ShInfo.fMask = SEE_MASK_IDLIST;
    ShInfo.lpDirectory = lpDirectory;
    ShInfo.lpIDList = pidl;
    ShInfo.lpParameters = lpParameters;
    ShInfo.nShow = nShow;
    ShInfo.lpVerb = NULL;// do not use _T("open") : it can fail on some cases
    ShInfo.hwnd = hwnd;
    // open selected item
    // if (!::ShellExecuteEx(&ShInfo))
    if (!::CShellExecuteAndBringToTop::ShellExecuteAndBringToTop(&ShInfo))
    //if ((int)::ShellExecute(NULL,
    //                        NULL,// do not use _T("open") : it can fail on some cases
    //                        TargetPath,lpParameters,lpDirectory,nShow)<33)
    {
        //TCHAR sz[2*MAX_PATH];
        //_tcscpy(sz,_T("Error opening "));
        //_tcscat(sz,TargetPath);
        // ::MessageBox(hwnd,sz,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        ::MessageBox(hwnd,_T("Error launching shortcut"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
    }

    bRet = TRUE;

CleanUp:
    if (pidl)
        ::CoTaskMemFree(pidl);

    if (pShellLink)
        pShellLink->Release();

    if (pPersistFile)
        pPersistFile->Release();

    return bRet;
}

